package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.configreport.ConfigReport;
import org.eso.ohs.phase2.apps.ot.configreport.ConfigReportRecord;
import org.eso.ohs.phase2.apps.ot.gui.ConfigReportBrowser;

/**
 * Action class to generate and display a config report on the selected
 * BusinessObjects.
 * 
 * @author Nick Kornweibel
 */

public class ConfigReportAction extends ActionSuperclass {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(ConfigReportAction.class);

    protected ObjectContainer folderView_;

    protected boolean folder_;

    /**
     * indicates whether the end of the iteration is reached or if we are still
     * processing the body of set of data upon which the iteration is being
     * performed
     */
    private boolean finished_ = false;

    /**
     * Constructs a ConfigReportAction.
     */
    public ConfigReportAction(ObjectContainer folderView, boolean folder,
            String name) {
        super((JFrame)folderView.getTopLevelAncestor(), name);
        folderView_ = folderView;
        folder_ = folder;
    }

    public ConfigReportAction(ObjectContainer folderView, 
            JComponent comp, boolean folder, String name) {
        super(comp, name);
        folderView_ = folderView;
        folder_ = folder; 
    }

    /**
     * Event handler for the ConfigReportAction action.
     * 
     * @param ev
     *            user-initiated ActionEvent to quit.
     */

    public void actionPerformedImpl(ActionEvent evt) {
        Long[] idLongs = null;

        ObjectManager mMgr = ObjectManager.getObjectManager();

        if (!folder_) {
            idLongs = folderView_.getAllSelected();
        } else {
            idLongs = folderView_.getAll();
        }

        idLongs = (Long[]) ObjUtils.uniqueOccurrences(idLongs);

        if (idLongs == null || idLongs.length == 0) {
            JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
                    new Object[]
                    { "There are no selected Business Objects ",
                            "Which object should be reported?", },
                    "Nothing selected", JOptionPane.ERROR_MESSAGE);
            return; // nothing selected?
        }
        new IncrementalActionAutomaton(new IncrementalReport(idLongs),
                (JFrame) folderView_.getTopLevelAncestor(),
                "Generating report ...");
    }

    public class IncrementalReport implements IncrementalAction {

        private ObjectManager objMgr = ObjectManager.getObjectManager();

        private Long[] idLongs_;

        private ConfigReport rpt_;

        // The instrument of the OBs selected for reporting - there can
        // only be one instrument, otherwise it is an error.
        private String instrument_;

        // Track if an error was encountered during report processing
        // Report processing halts and no report is shown upon error.
        private boolean errorEncountered_ = false;

        public IncrementalReport(Long[] ids) {
            idLongs_ = ids;
            rpt_ = new ConfigReport();
        }

        /**
         * @param int
         *            counter - will be passed to the iteration method
         * @return number of steps remaining
         */
        public int getNumStepsRemaining(int counter) {
            int rc = 0;
            if (!finished_) {
                rc = idLongs_.length - counter;
            }
            return rc;
        }

        /**
         * Set up for report processing.
         */
        public void startOfAction() {
            finished_ = false;
        }

        /**
         * disply the report, if no error was encountered.
         */
        public void endOfAction() {
            if (!errorEncountered_) {
                showFinalMsg();
            }
        }

        /**
         * @param int
         *            counter - will be passed to the iteration method
         */
        public void performStep(int counter) {
            if (!finished_) {
                doReport(idLongs_[counter]);
            }
            if (errorEncountered_ || counter + 1 == idLongs_.length) {
                finished_ = true;
            }
        }

        /**
         * doReport does the report for the BO identified with the parameter
         * "id"
         * 
         * @param Long
         *            id
         */
        public void doReport(Long id) {
            try {
                Media device = Config.getCfg().getMediaFromId(id.longValue());
                Class objType = Config.getCfg().getClassFromId(id.longValue());
                BusinessObject bo = (BusinessObject) objMgr.getBusObj(device,
                        id.longValue(), objType);
                if (bo instanceof CalibrationBlock) {
                    CalibrationBlock blk = (CalibrationBlock) bo;
                    String instr = blk.getOd().getInstrument();
                    if (instrument_ == null) {
                        instrument_ = instr;
                        // make sure there is a report defined for
                        // this instrument.
                        if (!rpt_.isReportSupported(instrument_)) {
                            JOptionPane.showMessageDialog(folderView_
                                    .getTopLevelAncestor(), new Object[]
                            {
                                    "There is no Configuration Report"
                                            + " defined for instrument "
                                            + instrument_,
                                    "No reporting is possible.", },
                                    "Config Report Error",
                                    JOptionPane.ERROR_MESSAGE);
                            errorEncountered_ = true;
                        }
                    } else if (!instrument_.equals(instr)) {
                        // make sure we are not reporting on blocks from
                        // multiple instruments.
                        JOptionPane.showMessageDialog(folderView_
                                .getTopLevelAncestor(), new Object[]
                        {
                                "The selected OBs cover a multiple of "
                                        + "instruments.",
                                "Config Reports are only possible"
                                        + " for OBs from a single instrument.",
                                "Reporting not possible.", },
                                "Config Report Error",
                                JOptionPane.ERROR_MESSAGE);
                        errorEncountered_ = true;
                    }
                    if (!errorEncountered_) {
                        rpt_.addOB(blk);
                    }
                }
            } catch (ObjectNotFoundException e) {
                stdlog_.error(e);
                ErrorMessages.announceNoObject(null, e);
                return;
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(null, e);
                return;
            }
        }

        /**
         * showFinalMsg shows a dialog box with information
         */
        public void showFinalMsg() {
            String report = rpt_.generateReport(true);
            Vector reportSet = rpt_.getReportSet();

            ConfigReportBrowser crb = new ConfigReportBrowser(
                    "Configuration Report");
            crb.setReport(report.toString());

            for (int r = 0; r < reportSet.size(); r++) {
                ConfigReportRecord rep = (ConfigReportRecord) reportSet
                        .elementAt(r);
                for (int i = 0; i < rep.getOBIds().size(); i++) {
                    ConfigReportBrowser.addToOBToConfigReportTable(
                            ConfigReport.intFromString( 
                            		rep.getOBIds().elementAt(i)),
                            		rep.getOBTypes().elementAt(i),  
                            		rep.getOBStatus().elementAt(i),  
                            		rep.getOBProgramNames().elementAt(i),
                            		rep.getOBNames().elementAt(i),
                            		rep.getRAs().elementAt(i), r + 1);
                }
            }

            JFrame frm = new JFrame("Configuration Report");
            JPanel mainReportPanel = new JPanel(new BorderLayout());
            mainReportPanel.add(crb.getView(), BorderLayout.CENTER);
            JTextArea reportPanel = new JTextArea(report.toString(), 30, 25);
            mainReportPanel.add(new JScrollPane(reportPanel),
                    BorderLayout.NORTH);
            frm.getContentPane().add(mainReportPanel);
            frm.pack();
            frm.setSize(800, 850);
            frm.setVisible(true);
            crb.startQuery();
        }
    }
}
