package org.eso.ohs.phase2.apps.p2pp;

import java.util.Enumeration;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SystemStatusEvent;
import org.eso.ohs.core.utilities.SystemStatusListener;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectEvent;
import org.eso.ohs.persistence.ObjectListener;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Factory class for Folder objects.
 * It's main purpose is to listen for new objects installed into
 * the Objectmanager and add the corresponding Folder as a listener
 * for changes in the object, so that the summary information can
 * be kept uptodate.
 *
 * <p>
 * $Id: FolderFactory.java,v 1.7 2006/03/21 14:56:15 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public final class FolderFactory {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger(FolderFactory.class);


    /** A listener for updates to the Persistence's registry of objects */
    private static Listener listener_ = new Listener();

    /** private constructor: no instantiation -- all methods are static. */
    private FolderFactory() {
    }

    /**
     * Returns a reference to the FolderFactory ObjectListener.
     * This is added by the main program as a listener to
     * Persistence changes and keeps Folder objects uptodate
     * by adding and removing them as change listeners for the
     * relevant objects.
     * 
     * @return a reference to the FolderFactory ObjectListener.
     */
    public static ObjectListener getObjectListener() {
	return listener_;
    }

    /**
     * Returns a reference to the Folder object for the medium
     * and directory specified.  If a copy does not exist in
     * memory or on the persistent medium, then it is generated.
     * 
     * @return a reference to appropriate Folder object.
     * 
     * @exception ObjectIOException
     *			if there was an I/O error reading the object.
     */
    public static Folder getFolder(Media device, DirectoryNode dir)
				throws ObjectIOException {
	Persistence objMgr = Persistence.getInstance();
	Folder folder = null;

	try {
	    folder = (Folder)objMgr.getBusObj(device, dir.getId(),
						Folder.class);
	    if (folder.getMedia() != device || folder.getNode() != dir) {
		folder.setMediaNode(device, dir);
	    }
	}
	catch (ObjectNotFoundException e) {
	    folder = new Folder();
	    folder.setId(dir.getId());
	    folder.setMediaNode(device, dir);
	    objMgr.register(device, dir, folder);
	    folder.reconstructSummaryInformation();
	}

	if (Media.DBASE.equals(device) && dir.getParent() == null) {
	    folder.setName("ESO Database");
	}

	dir.setObject(folder);
	return folder;
    }

    /**
     * Iterates over the DirectoryNodes for a given medium and
     * forces a read of the corresponding folder and any ObservingRun.
     * Called from the main program.
     * 
     * <p> Seems a bit icky, maybe needs some redesign.
     * 
     * @param device	medium whose folders must be read.
     * 
     * @exception ObjectIOException
     *			if there was an I/O error reading the object.
     */
    public static void loadBaseObjects(Media device)
			    throws ObjectIOException {
	Persistence objMgr = Persistence.getInstance();
	DirectoryNode root = objMgr.getRoot(device);
	Enumeration enum0 = root.nodes();
	DirectoryNode node;
	Folder folder;

	while (enum0.hasMoreElements()) {
	    node = (DirectoryNode)enum0.nextElement();

		    // Force Folder to be read into memory.
	    folder = getFolder(device, node);

		    // Force ObservingRun to be read into memory.
	    ObservingRun obsRun = folder.getObsRun();
	}
    }


    /** Listener for updates to the Persistence's registry of objects */
    private static class Listener implements ObjectListener,
						SystemStatusListener {

	public Listener() {
	    P2PP.addSystemStatusListener(this);
	}

	private boolean affectsSummaryInfo(ObjectEvent ev) {
	    return ev.getStorableObject() instanceof BusinessObject
		    && Media.LOCAL.equals(ev.getMedia())
		    && Config.getCfg().isUserLevel(ev.getObjectClass());
	}

	/**
	 * Event handler: an object was added to the registry.
	 * 
	 * @param ev	event describing what object was added.
	 */
	public void objectAdded(ObjectEvent ev) {
	    StorableObject sobj = ev.getStorableObject();

	    if ( ! affectsSummaryInfo(ev) ) {
		return;
	    }

	    try {
		DirectoryNode dir = ev.getDirectoryNode();
		Folder folder = getFolder(ev.getMedia(), dir);

		sobj.addPropertyChangeListener(folder);
		if (ev.getType() == ObjectEvent.OBJECT_CREATED) {
		    folder.addSummaryEntry((BusinessObject)sobj, null);
		}
	    }
	    catch (ObjectNotFoundException e) {
		// XXX should do something more sensible with this
		stdlog_.error(e);
	    }
	    catch (ObjectIOException e) {
		// XXX should do something more sensible with this
		stdlog_.error(e);
	    }
	}

	/**
	 * Event handler: an object was moved between two folders.
	 * 
	 * @param ev	event describing what object was removed.
	 */
	public void objectMoved(ObjectEvent ev) {
	    StorableObject sobj = ev.getStorableObject();
	    DirectoryNode fromDir = ev.getOldDirectoryNode();
	    DirectoryNode toDir = ev.getDirectoryNode();

	    if ( ! affectsSummaryInfo(ev) ) {
		return;
	    }

	    try {
		Folder oldFolder = getFolder(ev.getMedia(), fromDir);
		Folder newFolder = getFolder(ev.getMedia(), toDir);

		if (sobj != null) sobj.removePropertyChangeListener(oldFolder);
		Summary summ = oldFolder.removeSummaryEntry(
		    ev.getObjectId(),
		    ev.getObjectClass()
		);

		if (sobj != null) sobj.addPropertyChangeListener(newFolder);
		newFolder.insertSummary(summ);
	    }
	    catch (ObjectIOException e) {
		// XXX should do something more sensible with this
		stdlog_.error(e);
	    }
	}

	/**
	 * Event handler: an object was removed from the registry.
	 * 
	 * @param ev	event describing what object was removed.
	 */
	public void objectRemoved(ObjectEvent ev) {
	    StorableObject sobj = ev.getStorableObject();

	    if ( ! affectsSummaryInfo(ev) ) {
		return;
	    }

	    try {
		DirectoryNode dir = ev.getDirectoryNode();
		Folder folder = getFolder(ev.getMedia(), dir);

		if (sobj != null) sobj.removePropertyChangeListener(folder);
		folder.removeSummaryEntry(
		    ev.getObjectId(),
		    ev.getObjectClass()
		);
	    }
	    catch (ObjectIOException e) {
		// XXX should do something more sensible with this
		stdlog_.error(e);
	    }
	}


	public void objectRemovedFromRegistry(ObjectEvent ev) {

	}




	public void systemStatusChange(SystemStatusEvent ev) {
	    Persistence mMgr = Persistence.getInstance();

	    if (ev.getType() == SystemStatusEvent.RELOGIN) {

		mMgr.addObjectListener(FolderFactory.getObjectListener());
		try {
		    FolderFactory.loadBaseObjects(Media.LOCAL);
		}
		catch (ObjectIOException e) {
		    stdlog_.error(e);
		}

	    }

	}

    }

}

