package org.eso.ohs.phase2.apps.p2pp;

import java.awt.event.ActionListener;
import java.util.Hashtable;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryEvent;
import org.eso.ohs.persistence.DirectoryListener;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
/**
 * Action class to create/maintain a hierarchical menu reflecting
 * the directory layout.  The instance of this class is added as
 * an action listener to each of the menu items.
 * 
 * <p>
 * $Id: HierarchicalMenu.java,v 1.9 2006/04/27 13:25:00 pnunes Exp $
 */

abstract public class HierarchicalMenu implements ActionListener {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(HierarchicalMenu.class);

    /** parent menu item of the one we want to update */
    private JMenu menuParent_;

    /** medium on which the business object is to be created */
    private Media device_;

    /* Text of top-level menu item */
    private String name_;

    private HierarchyView hView_;

    /** Menu item for root directory */
    private JMenu menuRoot_;

    /** Mapping from menuItem to DirectoryNode, for actionPerformed() */
    private Hashtable<JMenuItem, DirectoryNode> menuToNode;

    /** Cached reference to Persistence */
    private Persistence objMgr = Persistence.getInstance();

    /**
     * Constructs a move BusinessObject menu handler.
     * 
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     * @param name	  text for top-level menu item in the hierarchy
     */
    public HierarchicalMenu(
        Media device,
        JMenu menuParent,
        HierarchyView hView,
        String name) {
        device_ = device;
        name_ = name;
        hView_ = hView;
        menuParent_ = menuParent;

        menuToNode = new Hashtable<JMenuItem, DirectoryNode>();

        objMgr.addDirectoryListener(new Listener());
    }

    /**
     * Utility method to actually update the menu hierarchy to follow
     * the directory hierarchy.
     */
    public void updateMenu() throws ObjectIOException {
        if (menuRoot_ == null) {
            return; // nothing has been setup yet.
        }

        for (int j = menuParent_.getItemCount() - 1; j >= 0; j--) {
            if (menuParent_.getItem(j) == menuRoot_) {
                menuParent_.remove(j);
                menuParent_.insert(createFolderMenu(), j);
                return;
            }
        }
    }

    /**
     * Returns the DirectoryNode corresponding to a given menuItem.
     * 
     * @param	obj	menu item to be converted
     * @return		the corresponding directory node, or null if not found
     */
    protected DirectoryNode getNodeFromMenuItem(Object obj) {
        return menuToNode.get(obj);
    }

    /**
     * Returns the media upon which the directory hierarchy is stored.
     * 
     * @return		media type, eg Media.LOCAL
     */
    public Media getMedia() {
        return device_;
    }

    /**
     * Returns a hierarchy of menuItems that describe the folder hierarchy.
     * 
     * @return		menu hierarchy
     * @exception ObjectIOException
     *                  if there was an I/O error reading the object.
     */
    public JMenuItem createFolderMenu() throws ObjectIOException {
        DirectoryNode root = objMgr.getRoot(device_);

        menuToNode.clear();

        menuRoot_ = (JMenu) createFolderSubmenu(root, true);
        menuRoot_.setText(name_);

        return menuRoot_;
    }

    public void setEnabled(boolean enabled) {
        menuRoot_.setEnabled(enabled);
    }

    /**
     * Recursive utility method to create a menu hierarchy corresponding to
     * a folder hierarchy.
     * 
     * @param	node		root node for the directory hierarchy
     * @param	allowAsLeaf	can the given folder be displayed on it's own?
     * @param	showParent	should the parent node be shown in the submenu?
     * @return			menu hierarchy
     */
    private JMenuItem createFolderSubmenu(
        DirectoryNode node,
        boolean topLevel) {
        JMenuItem mi;

        if (!topLevel && node.isEmpty()) {
            mi = new JMenuItem(node.getDescription());
            mi.addActionListener(this);
            menuToNode.put(mi, node);
            return mi;
        }

        DirectoryNode[] dirs =
            HierarchyView.sortDirNodes(node.getDirectories());

        JMenu mnu = new JMenu(node.getDescription());

        if (!topLevel) {
            mi = new JMenuItem(node.getDescription());
            mi.addActionListener(this);
            menuToNode.put(mi, node);
            mnu.add(mi);
            mnu.addSeparator();
        }

        int period = hView_.getDisplayedPeriod();

        if (topLevel && period != HierarchyView.ALL_PERIODS) {
            for (int j = 0; j < dirs.length; j++) {
                try {
                    Folder folder = FolderFactory.getFolder(device_, dirs[j]);
                    ObservingRun or = folder.findObsRun();

                    if (or.getPeriod() == period) {
                        mnu.add(createFolderSubmenu(dirs[j], false));
                    }
                } catch (ObjectIOException e) {
                    stdlog_.error(e);
                }
            }
        } else {
            for (int j = 0; j < dirs.length; j++) {
                mnu.add(createFolderSubmenu(dirs[j], false));
            }
        }

        return mnu;
    }

    /**
     * Check that the two given directories have the same instrument,
     * so that an object can be moved between them.  If they are different
     * an error message is displayed to the user.
     * 
     * @param	oldParent	directory currently containing object.
     * @param	newParent	directory that the object should be moved to.
     * @return			menu hierarchy
     */
    protected boolean checkSameInstrument(
        DirectoryNode oldParent,
        DirectoryNode newParent) {
        ObservingRun oldObsRun = ((Folder) oldParent.getObject()).findObsRun();
        ObservingRun newObsRun = ((Folder) newParent.getObject()).findObsRun();

        String oldInstrument = oldObsRun.getInstCode();
        String newInstrument = newObsRun.getInstCode();

        if (oldInstrument.equals(newInstrument)) {
            return true;
        }

        JOptionPane.showMessageDialog(
            menuParent_.getTopLevelAncestor(),
            new Object[] {
                "The destination observing run ",
                "uses a different instrument.",
                " ",
                "Move failed." },
            "Move failed",
            JOptionPane.ERROR_MESSAGE);

        return false;
    }

    /**
     * Check that the two given directories have the same observing run
     * 
     * @param   oldParent   directory currently containing object.
     * @param   newParent   directory that the object should be moved.
     * @return  boolean 
     */
    protected boolean checkSameRun(
        DirectoryNode oldParent,
        DirectoryNode newParent) {
        ObservingRun oldObsRun = ((Folder) oldParent.getObject()).findObsRun();
        ObservingRun newObsRun = ((Folder) newParent.getObject()).findObsRun();
        if(oldObsRun.getId() != newObsRun.getId())
            return false;
        else
            return true;
    }

    /**
     * Listener to update the menu hierarchy each time the directory
     * structure changes.
     */
    private class Listener implements DirectoryListener {

        /**
         * Handler for directory events: called when a directory is added
         * 
         * @param ev	event giving more info about the new directory.
         */
        public void directoryAdded(DirectoryEvent ev) {
            try {
                updateMenu();
            } catch (ObjectIOException e) {
                stdlog_.error(e);
            }
        }

        /**
         * Handler for directory events: called when a directory is moved
         * 
         * @param ev	event giving more info about the new directory.
         */
        public void directoryMoved(DirectoryEvent ev) {
            try {
                updateMenu();
            } catch (ObjectIOException e) {
                stdlog_.error(e);
            }
        }

        /**
         * Handler for directory events: called when a directory is changed
         * 
         * @param ev	event giving more info about the new directory.
         */
        public void directoryChanged(DirectoryEvent ev) {
            try {
                updateMenu();
            } catch (ObjectIOException e) {
                stdlog_.error(e);
            }
        }

        /**
         * Handler for directory events: called when a directory is removed
         * 
         * @param ev	event giving more info about the new directory.
         */
        public void directoryRemoved(DirectoryEvent ev) {
            try {
                updateMenu();
            } catch (ObjectIOException e) {
                stdlog_.error(e);
            }
        }

    }
}
