package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.util.Stack;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
/**
 * Action class to delete a business object in the chosen directory.
 * 
 * <p>
 * $Id: DeleteBOAction.java,v 1.11 2006/04/27 13:25:00 pnunes Exp $
 * <p>
 * @author Bob Kemp
 */
public class DeleteBOAction extends ActionSuperclass {

    private static Logger stdlog_ =
        Logger.getLogger(DeleteBOAction.class);

    /** Threshold for naming individual objects in messages to user */
    private final static int DELETION_THRESHOLD = 5;

    /** medium on which the business object is to be deleted */
    Media device_;

    /** view of the folder in which the OB is to be deleted */
    ObjectContainer folderView_;

    /**
     * Constructs a delete BusinessObject action.
     * 
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     */
    public DeleteBOAction(Media device, ObjectContainer folderView) {
        super((JComponent) folderView, "Delete");
        device_ = device;
        folderView_ = folderView;
    }

    /**
     * Event handler for the delete BusinessObject action.
     * 
     * @param ev       user-initiated ActionEvent to delete an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Long[] idLongs = folderView_.getAllSelected();
        Folder folder = ((FolderGridView) folderView_).getFolder();

        if (idLongs.length == 0) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "There are no selected objects of this type.",
                    "Which objects should be deleted?",
                    },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return; // nothing selected?
        }

        StringBuffer sb = new StringBuffer("Are you sure you want to delete ");

        if (idLongs.length > DELETION_THRESHOLD) {
            sb.append("all ");
            sb.append(idLongs.length);
            // "remaining" because we're ignoring
            // those that still have references.
            sb.append(" objects?");
        } else {
            Summary summ;

            sb.append("\n");
            for (int j = 0; j < idLongs.length; j++) {
                if (idLongs[j] != null) {
                    summ = folder.findSummary(idLongs[j].longValue());
                    sb.append("    ");
                    sb.append(summ.getName());
                    sb.append("\n");
                }
            }
        }

        int option =
            JOptionPane.showConfirmDialog(
                folderView_.getTopLevelAncestor(),
                sb.toString(),
                "Delete",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null);

        switch (option) {
            case JOptionPane.CLOSED_OPTION :
                return;
            case JOptionPane.CANCEL_OPTION :
                return;
        }

        new P2PPIncrementalActionAutomaton(
            new IncrementalDeletion(device_, idLongs),
            (MenuGroups) folderView_.getTopLevelAncestor(),
            "Deleting...");

    }

    /**
     * Utility class to delete a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalDeletion implements IncrementalAction {
        private Persistence objMgr = Persistence.getInstance();

        private Long[] idLongs_;
        private Media device_;

        public IncrementalDeletion(Media device, Long[] idLongs) {
            device_ = device;
            idLongs_ = idLongs;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        public void startOfAction() {
        }

        public void endOfAction() {
        }

        public void performStep(int counter) {
            if (idLongs_[counter] != null) {
                // Main body of the "loop" + fall thru
                doDeletion(idLongs_[counter].longValue());
            }
        }

        public void doDeletion(long id) {
            try {
                // XXX Call getBusObj() to ensure object is in memory.
                // XXX Workaround for bug in deleteBusObj, et al.
                BusinessObject bo =
                    (BusinessObject) objMgr.getBusObj(
                        device_,
                        id,
                        Config.getCfg().getClassFromId(id));
                Stack<BusinessObject> stk = new Stack<BusinessObject>();
                // Part 1: set the id for each BO.
                // This is the part that can generate exceptions.     	   
                stk.push(bo);
                while (!stk.isEmpty()) {
                    BusinessObject busObj = stk.pop();
                    BusinessObject[] children = busObj.getBusObjChildren();
                    for (int j = 0; j < children.length; j++) {
                        stk.push(children[j]);
                    }
                    objMgr.deleteBusObj(
                        device_,
                        busObj.getId(),
                        busObj.getClass());
                }
                if(bo instanceof ObservationBlock) {
                    ObservationBlock ob = (ObservationBlock)bo;
                    //FindingChartSession.decFcCount(ob);
                }

            } catch (ObjectNotFoundException e) {
                stdlog_.error(e);
                announceNoObject(e);
            } catch (ObjectIOException e) {
                announceIOError(e);
            }
        }
    }

}
