# File   : orangAPIsetup.tcl
# Purpose: Initialize the TCL scripting environment for ORANG
# Author : Paulo Nunes
# Date   : Fri Aug 26   2005
#
# define the procedures that are available to the TCL script developers
# based on scriptAPIsetup.tcl wrote by Nick Kornweibel

## $Id: orangAPIsetup.tcl,v 1.19 2006/03/22 04:30:43 amchavan Exp $

# include java
package require java


# init:
# Initialization procedure. Must be called at the beginning to set up the 
# environment: build the scripting box, initialize the data structures, etc. 
proc init {} {
    global scriptingBox
    set scriptingBox [java::call org.eso.ohs.phase2.orang.OrangScriptingBox getInstance]
 
}


proc getOBCount {}  {
	global scriptingBox
    return [$scriptingBox getNumberOfObservationBlocks]

}

# getOB:
# return the OB at the specified index (or null if the index is out of
# range). First index is 0.
# returned objects should be checked using evmValid

proc getOB {index} {
    global scriptingBox
    return [$scriptingBox getObSchedInfoAt $index]
}

proc getOrangModel {} {
	global scriptingBox
	return [$scriptingBox getOrangModel]
}


# 
# objValid:
# Used to determine if an object returned by procedures is valid or not
# This should be used to check all object returned by evm procedures.

proc objValid in {
    if { [java::isnull $in] } {
        return 0
    } else {
        return 1
    }
}

# setOBRankScore:
#
# Sets the OB ranking score

proc setOBRankScore { index score description } {
	global scriptingBox
    $scriptingBox setOBScore $index $score $description
}

proc getOBRankScore { index } {
	global scriptingBox
	return [$scriptingBox getOBScore $index]
}


# The priority given by the user to an OB; lower values indicate
# higher priority.
# Priority is relative to other OBs of the same run.
proc getUserPriority ob {
    return [$ob getUserPriority]
}

# The airmass of the OB, it's a PCF (not used for the ranking)
proc getAirmass ob {
    return [$ob getAirmass]
}

# Declination of the OB, in degrees
proc getDec ob {
    return [$ob getDec]
}

# Differential declination of the OB, in milliarc seconds
proc getDiffDec ob {
    return [$ob getDiffDec]
}

# Differential right ascension of the OB, in milliarc seconds
proc getDiffRA ob {
    return [$ob getDiffRA]
}

# The Fractional Lunar Illumination constraint of the OB,
# a number between 0 and 1
proc getFLI ob {
    return [$ob getFractionalLunarIllumination]
}

# The Moon Angular Distance constraint of the OB, in degrees,
# a number between 0 and 180
proc getMoonAngularDistance ob {
    return [$ob getMoonAngularDistance]
}

# Proper motion in declination of the OB
proc getPropDec ob {
    return [$ob getPropDec]
}

# Proper motion in right ascension of the OB
proc getPropRA ob {
    return [$ob getPropRA]
}

# Right ascension of the OB, in degrees
proc getRa ob {
    return [$ob getRa]
}

# The Seeing constraint of the OB,
# a number between 0.2 and 2.0 (image size in arcseconds)
proc getSeeing ob {
    return [$ob getSeeing]
}

# The Sky Transparency constraint of the OB, one of 
# 1PHO (Photometric), 2CLE (Clear), 3THN (Variable, thin cirrus) or 4THK (Variable, thick cirrus)
proc getSkyTransparency ob {
    return [$ob getSkyTransparency]
}

# The Strehl Ratio constraint of the OB, 
# a number between 0.0 and 100.0
proc getStrehlRatio ob {
    return [$ob getStrehlRatio]
}

# The execution time of the OB, in seconds
proc getExecutionTime ob {
    return [$ob getExecutionTime]
}

# The observing Period of the OB, e.g. 78 
proc getPeriod ob {
    return [$ob getPeriod]
}

# The run ID of the OB, e.g. 78.A-0123(B) 
proc getProgId ob {
    return [$ob getProgId]
}

# Either V (visitor) or S (service) 
proc getObsMode ob {
    return [$ob getObsMode]
}

# The telescope the OB should be executed on, e.g. UT2
proc getTelescope ob {
    return [$ob getTelescope]
}

# The instrument the OB should be executed on, e.g. UVES
proc getInstrument ob {
    return [$ob getInstrument]
}

# The rank class of the OB, one of A, B or C
proc getRankClass ob {
    return [$ob getRankClass]
}

# NOT USED, it is a schedule column
proc getRank ob {
    return [$ob getRank]
}

# 1 if the OB belongs to a time critical run, 0 otherwise 
proc getTimeCritical ob {

	java::try {	
    	set critical [$ob getTimeCritical]    	
	} catch {NullPointerException err} {	
    	java::throw [java::new org.eso.ohs.phase2.orang.OrangScriptException "Run properties not instantiated"]
	}
	
	return $critical
}


# 1 if the OB belongs to a Chilean run, 0 otherwise 
proc getChilean ob {
	return [$ob getChilean]
}

# 1 if the OB belongs to a pre-imaging run, 0 otherwise 
proc getPreImaging ob {
	return [$ob getPreImaging]
}

# 1 if the OB belongs to a run that was already started; that is, at least
# another OB of that same run was executed.
# 0 otherwise 
proc getRunStarted ob {
	return [$ob isRunStarted]
}

# 1 if the OB belongs to a carryover run, 0 otherwise 
proc getCarryOver ob {
	return [$ob getCarryOver]
}

# 1 if the OB belongs to a run of a large program, 0 otherwise 
proc getLarge ob {
	return [$ob getLarge]
}

# 1 if the second half of the time interval defined in the OB was reached, 0 otherwise 
proc getSecondHalfTI ob {
	return [$ob getSecondHalfTimeInterval]
}

# The  LST for the start time defined in the ORANG GUI, in hours
proc getLST {} {
	set orangModel [getOrangModel]
	return [$orangModel getLST]
}

# The hour angle of an OB (informally: LST - RA)
proc getHourAngle ob {

	# convert the RA from degrees to hours
	set ra [getRa $ob]
	set ra [expr $ra / 15]

    # LST is expressed in hours
	set lst [getLST]
	
	set hourAngle [format "%+10.2f" [expr $lst - $ra]]
	return $hourAngle
} 

# The current FLI, 0 to 1
proc getCurrentFLI {} {
	set orangModel [getOrangModel]	
	return [$orangModel getFLI]
}


# The current seeing, as input in the ORANG "Settings" panel
proc getCurrentSeeing {} {	
	set orangModel [getOrangModel]	
	return [$orangModel getSeeing]
}

# The current wind direction, in degrees, as input in the ORANG "Settings" panel
proc getCurrentWindDirection {} {
	set orangModel [getOrangModel]
	return [$orangModel getWind]
}

# The current sky transparency value, as input in the ORANG "Settings" panel;
# one of 1PHO (Photometric), 1CLE (Clear), 3THN (Variable, thin cirrus) or 4THK (Variable, thick cirrus)
proc getCurrentSkyTransparency {} {
	set orangModel [getOrangModel]
	return [$orangModel getSkyTransp]
}

# To be overridden by individual ranking scripts
proc rank ob {}

# Ranks all the OBs in observable queue
proc rankall {} {

	set obcount [getOBCount]
	# puts "TCL ranking: No OBs: $obcount"
		
    set i 0    
    while { $i < $obcount } {        
		set ob [getOB $i]		
		set valid [objValid $ob]		
		set score ""
		set description ""
		
		if ($valid) {			
			set result [rank $ob]
			
			set score [lindex $result 0]
			set description [lindex $result 1]			
	        setOBRankScore $i $score $description
			
		}  else {		
			puts "OB #$i not valid"
		}	
			
        incr i 1 
	}	
}



# end of file scriptAPIsetup.tcl
