#!/bin/sh
#
# $Id: icalib.sh,v 1.7 2009/05/25 08:09:29 cguirao Exp $
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2003 European Southern Observatory
# LICENSE
#
# NAME:      icalib.sh
# TYPE:      Bourne Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    CG/KH - ESO/DMD/SEG
# DESCRIPTION: Update the calibDB with the content of the tarball file 
#            delivered by DFO/QC and containing master calibration frames
#            in a flat and unique directory structure
# USAGE:    icalib.sh master-calib-XXXX.tar.gz
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

# MAIN
# main program starts here
currentDir=`pwd`
depot="/data/calibDB/tmp"
cmd=`basename $0`


# first argument is the input tar file (compressed or not)
DELIVERY=$1
LOG_FLAG=$2
#LOG_PREFIX=`echo $DELIVERY | awk -F. '{print $1}'` 
LOG_PREFIX=`echo $DELIVERY | sed -e 's/\(^.*\).tar.gz$/\1/' -e 's/\(^.*\).tgz$/\1/' -e  's/\(^.*\).tar$/\1/'`
LOG_FILE=icalib-${LOG_PREFIX}.`date +"%F-%T"`.log

# second argument is optional and meant to log the output of icalib
if [ "$LOG_FLAG" != "NOLOG" ]; then
  echo "$cmd: logging in /tmp/$LOG_FILE"
  exec $cmd $DELIVERY NOLOG 2>&1 | tee /tmp/$LOG_FILE
  exit 0
fi

echo "-------------------------------------------------------------------"
echo " ${cmd}: shell script to update the calibDB with the content of"
echo " the tarball file provided by DFO/QC with new master calibration"
echo " frames."
echo ""
echo " BEWARE !!!"
echo "    CalibDB updates provided by DFO/QC must ALWAYS be performed"
echo "    with ${cmd}, NOT with IPIP !"
echo "    Similarly, new pipeline releases provided by developers or SEG"
echo "    must ALWAYS be performed with ipip, NOT with ${cmd} !"
echo "-------------------------------------------------------------------"
echo ""


#
# Check you are 'quality' user
#
if [ `whoami` != "quality" ]; then
  echo "*** ERROR ***: Only user \"quality\" is allowed to run this script. Must exit." 
  exit
fi


#
# Check first argument is a file
#
if [ ! -f $DELIVERY ] ; then
   echo "*** ERROR *** Argument $DELIVERY is not found. Must exit."
   echo "    Usage: $cmd XXXX.tar or $cmd XXXX.tar.gz or $cmd XXXX.tgz" 
   exit
fi

#
# Check the first argument extension is tar, tar.gz or tgz
#
extension=`echo "${DELIVERY}" | awk -F"${LOG_PREFIX}" '{print $2}'`
if [ "${extension}" != ".tar" -a  "${extension}" != ".tar.gz" -a "${extension}" != ".tgz" ]
then
   echo "*** ERROR *** Argument $DELIVERY is not a tar file. Must exit."
   echo "Usage: $cmd XXXX.tar or $cmd XXXX.tar.gz or $cmd XXXX.tgz" 
   exit
fi

#
# Check the tar or tar.gz file does not contains subdirectories
#
if [ "${extension}" = ".tar" ]
then
   command="tar tvf ${DELIVERY}"
elif [ "${extension}" = ".tar.gz" ]
then
   command="tar tvzf ${DELIVERY}"
elif [ "${extension}" = ".tgz" ]
then
   command="tar tvzf ${DELIVERY}"
fi
$command | awk '{print $6}' | while read file 
do  
  subdir=`dirname $file` 
  if [ "$subdir" != "." ]; then
    echo "*** ERROR *** The file ${DELIVERY} is not flat!!" 
    echo "*** ERROR *** It contains, at least, subdirectory: $subdir"
    exit 1
  fi
done

# For a strange reason the previus "exit 1" does not stop the script, so
# check it again 
if [ $? -ne 0 ]; then 
  echo "*** Bye"
  exit 1
fi


#
# Run the check script checkCalibDBtar.sh to make sure the tarball file
# follows the new flat directory structure
# CG: Removed the checkCalibDBtar.sh as the DELIVERY files produced by DFO
#     are not the same as those for IPIP
#
#checkCalibDBtar.sh ${DELIVERY}
#if [ $? -ne 0 ]; then
#  exit
#fi

#
# Check the current directory is the depot one. If not, raise an ERROR and exits.
#
if [ ${currentDir} != "${depot}" ]
then
   echo "*** ERROR *** The current directory is not the one where to run this script, and to place the input tar file."
   echo "    ERROR *** It must be: ${depot}. If the latter directory is missing, please, create it as 'quality'."
   exit
fi

# 
# Check the depot directory has no previous FITS files.
# If yes, raise an ERROR and exits.
#
find ${depot} -type f -name "*.*fits" | wc -l | grep -q 0 
if [ $? -ne 0 ]; then 
   echo "*** ERROR *** $cmd: FITS or TFITS files detected under ${depot}"
   echo "*** ERROR *** Please check these files, remove them or move them away then re-run ${cmd}."
   find /tcal/ -type f -name "*.*fits"
   exit
fi

#
# Check the /tcal dir exists
#
if [ ! -d /tcal ] ; then
   echo "$cmd: Directory /tcal does not exist or is not readable. Can not continue."
   exit
fi

#
# Check the /tcal dir is empty of files
#
find /tcal/ -type f  | wc -l | grep -q 0 
if [ $? -ne 0 ]; then 
   echo "*** ERROR *** Files detected in /tcal: "
   find /tcal/ -type f  
   echo "*** ERROR *** /tcal is not empty. Please check, move them away and re-start ${cmd}."
   exit
fi

#
# Check msqld is running
#
msql_process=`ps -ef | grep msql2d| grep -v grep`
if [ -z "$msql_process" ]; then
  echo "*** ERROR *** msql2d daemon not running."
  echo "    ERROR *** launch msql2d daemon, and try again."
  exit
fi

#
# Check the database calibDB exists
#
echo "\q" | msql calibDB >/dev/null
if [ $? -ne 0 ]; then 
  echo "*** ERROR *** database calibDB may not exist."
  echo "    ERROR ***  Create the calibDB with script createCalibDB.sh"
  exit
fi

#
# Check there is no DO (pilDataOrganizer or DataOrganiser)
#
DOPID=`ps -ef | grep DataOrganiser | grep -v grep | awk '{print $2}'`
if [ -n "$DOPID" ]; then
  echo "*** ERROR *** There is a DataOrganiser with PID=$DOPID still running"
  echo "    ERROR *** Stop the DataOrganiser as \"pipeline\" with command:"
  echo "    ERROR *** \"doControl stop\" and run again icalb"
  exit
fi
DOPID=`ps -ef | grep pilDataOrganizer | grep -v grep | awk '{print $2}'`
if [ -n "$DOPID" ]; then
  echo "*** ERROR *** There is a pilDataOrganizer with PID=$DOPID still running"
  echo "    ERROR *** Stop the pilDataOrganizer as \"pipeline\" with  command:"
  echo "    ERROR *** \"stopDO\" and run again ${cmd}"
  exit
fi
DOPID=`ps -ef | grep pilScheduler | grep -v grep | awk '{print $2}'`
if [ -n "$DOPID" ]; then
  echo "*** ERROR *** There is a pilScheduler with PID=$DOPID still running"
  echo "    ERROR *** Stop the pilScheduler as \"pipeline\" with  command:"
  echo "    ERROR *** \"stopRBS\" and run again ${cmd}"
  exit
fi

echo "Preliminary checks: OK"
echo ""
#
# Compute the list of valid instruments from the calibDB
#
rm -f /tmp/calibdb_inst_list.txt
echo "select * from instrument \g" | msql calibDB | gawk -F\| '{print $2}' | sed -e 's/^.\(.....\).*/\1/' -e 's/UVES-//' -e 's/ins_i//' | sort -u  | tr '[A-Z]' '[a-z]' > /tmp/calibdb_inst_list.txt


#
# Untar the delivery into the depot directory
#
echo "Untar the input tar file under ${depot}:"
if [ "${extension}" = ".tar" ]
then
   tar xvf $DELIVERY
elif [ "${extension}" = ".tar.gz" ]
then
   gunzip -c $DELIVERY | tar xvf -
elif [ "${extension}" = ".tgz" ]
then
   tar xzvf $DELIVERY
fi

#
# For every file, check the INSTRUME, If INSTRUME is in the list of available
# instruments as provided by the MSQL database, move the file to /tcal/<inst>.
# Otherwise, raise a warning, and skip the file.
# 
rm -rf /tmp/files_in_tcal.txt ; touch /tmp/files_in_tcal.txt
rm -rf /tmp/cksum_tcal.txt ; touch /tmp/cksum_tcal.txt

echo ""
echo "Check that every FITS or TFITS file has a valid INSTRUME keyword value ..."
ls *.*fits >/dev/null
if [ $? -ne 0 ]; then
  echo "*** ERROR *** No FITS or TFITS files found in this directory $PWD"
  exit 1
fi

for myFile in `(ls *.*fits)`
do 
  echo -n "   $myFile: "
  inst=`dfits $myFile | fitsort -d "INSTRUME" | awk '{print $2}' | awk '{print $1}'` 

  # convert some instrument values into a 5 char lowercase string (as required by the calibDB)
  if [ "${inst}" = "CRIRES" ]; then inst="crire";fi
  if [ "${inst}" = "GIRAFFE" ]; then inst="giraf";fi
  if [ "${inst}" = "NAOS+CONICA" ]; then inst="naco";fi
  if [ "${inst}" = "SINFONI" ]; then inst="sinfo";fi
  if [ "${inst}" = "VIRCAM" ]; then inst="vcam";fi
 
  inst=`echo $inst | tr '[A-Z]' '[a-z]'`

  egrep -q -w $inst /tmp/calibdb_inst_list.txt
  if [ $? -ne 0 ]; then 
     echo "*** ERROR *** File $myFile contains no valid INSTRUME value. File skipped."
     echo "It must be one of the following:"
     cat /tmp/calibdb_inst_list.txt
     exit 1
  else
     cksum $myFile >> /tmp/cksum_tcal.txt
     mkdir -p /tcal/${inst}/cal
     mv $myFile /tcal/${inst}/cal/
     echo "/tcal/${inst}/cal/$myFile" >> /tmp/files_in_tcal.txt
     echo "valid INSTRUME value found. File moved to the /tcal/${inst}/cal/ directory."
  fi
done

#
# Now, actually run the update of the calibDB
#
echo ""
echo "Now, actually run the calibDB update ..."
##########################
cd /tcal
echo "chmod -R +w ."
chmod -R +w .
echo "badnames_in_calibDB.sh"
badnames_in_calibDB.sh | wc -l | grep -q 0
if [ $? -ne 0 ]; then 
  echo "*** ERROR *** Badnames detected by the badnames_in_calibDB.sh script"
  echo "    ERROR *** Contact DMO/DFO/QC to possibly remove them from $DELIVERY, and try again with an updated input tar file." 
  badnames_in_calibDB.sh
  exit 1
fi
echo "updateCalibDb -a"
updateCalibDb -a
if [ $? -ne 0 ]; then 
  echo "*** ERROR *** updateCalibDb failed with error code: $?"
  echo "    ERROR *** please, report to DMO/DFO/QC and to SDD/SED/SEG."
  exit 1
fi
##########################

echo "Update of CalibDB finished."
echo ""
#
# Verifying
#
echo "Verifying the calibDB updates ..."



#
# Check the /tcal dir is empty of files after updatdCalibDb
#
echo -n "Check that directory /tcal is empty: "
find /tcal/ -type f  | wc -l | grep -q 0 
if [ $? -ne 0 ]; then 
   echo "*** ERROR *** There are still FITS or TFITS files located under /tcal while it should not "
   find /tcal/ -type f  
   echo "Please check and possibly remove them."
   exit 1
fi
echo "OK"

#
# Compute the cksums of the files newly ingested into /cal/
#
echo -n "Compare cksums of files newly ingested with the ones from the original input tar file:"
rm -f /tmp/cksum_cal.txt

for longFile in `(cat /tmp/files_in_tcal.txt)`
do
   inst=`echo $longFile | awk -F"/" '{print $3}'`
   file=`echo $longFile | awk -F"/" '{print $5}'`
   cd /cal/$inst/cal/
   cksum $file >> /tmp/cksum_cal.txt
done

diff /tmp/cksum_tcal.txt /tmp/cksum_cal.txt
if [ $? -ne 0 ]; then
   echo "*** ERROR *** cksums are different between /cal and original files placed in /tcal"
   exit 1
fi
echo " OK, all cksums identical."



#
# Verifying the calibDB
#
echo "Check that every newly ingested file has an entry in the MSQL calibDB:"
for longFile in `(cat /tmp/files_in_tcal.txt)`
do 
   instUp=`echo $longFile | gawk -F '/' '{print $3}' | tr '[a-z]' '[A-Z]'`
   file=`echo $longFile | gawk -F '/' '{print $5}'`
  if [ -n "$instUp" ]; then 
    echo -n "   $file in calibDB:" 
    echo "select * from component where comp_id like '${instUp}%'\g" | msql  calibDB | grep -q $file
    if [ $? -ne 0 ]; then
      echo "*** ERROR *** file $file not in the calibDB"
      exit 1
    else
      echo " OK"
    fi
  fi
done

#
# The END
#
echo ""
if [ -f /tmp/$LOG_FILE ]
then
   echo "Complete log file available at /tmp/$LOG_FILE"
fi
echo "THE END"
exit 0





