#!/bin/sh
#
# ---------------------------------------------------------------------------
# HEADER
# ---------------------------------------------------------------------------
#
# GENERAL DESCRIPTION
# -------------------
#    Bourne Shell script to archive files on a remote host.
#    Each time a file is checked-out, a lock is set: only one user can update
#    the file.
#    When a file has been archived, it can be automatically "webcp", i.e.
#    published and accessible from the DFS Group website:
#    http://www.eso.org/projects/dfs/team/
#    The automatic publishing depends on the file category. For some
#    categories, publishing is done manually by DFS Group QA.
#
# CREATED BY:  K. HAGGOUCHI
# CURRENT VERSION: see "CurrentVersion" variable in the Main
# CREATION DATE: 2000/12/14
#
# CHANGES
# -------
# WHO   WHAT                                              WHEN          WHERE
# KH    automatic update of "Last Update" field via sed   2001/01/19    1_0
# KH    add chmod g+w and a+r before cp to remoteHost     2001/01/26    1_1
# KH    complete new release to access the DFS            2001/06/27    1_2
#       shared repository    
# KH    mpeltzer instead of mnelson, read doc directly    2001/12/17    2_1
#       accessible via automatic emails
# KH    add 15000 as PSC for the DFS/VLTI interface       2002/02/04    2_2
# KH    port to opus3: /export/diskri/dataflow ...        2002/06/10    2_3
# KH    add HTML and HTM as format for schedule files     2002/10/07    2_4
# KH    replace conica by naco                            2002/11/29    2_4_1
# KH    send email to SEG when new Release Notes          2002/12/10    2_4_2
# KH    add NGAMS to the ngas project                     2002/12/11    2_4_3
# KH    add VISIR to the projects                         2003/01/17    2_4_4
# KH    pb yjung with "which" commmand for mailx and      2003/01/23    2_4_5
#       gzip: put in comment
# KH    simplify the 'error' & 'beware' functions (were   2003/01/30    2_4_6
#       returning execute permission denied errors)
# KH    add SINFONI to the projects                       2003/06/03    2_4_7
# KH    add XML to -page option                           2003/06/16    2_4_8
# KH    dmdsec email address (instead of mpeltzer)        2003/09/08    2_4_9
# KH    update list of document types  (EPS)              2004/01/29    2_4_10
# KH    update list of document types  (PPT)              2004/01/29    2_4_11
# KH    add OCA and REI to the projects                   2005/02/09    2_4_12
# KH    add GIRAFFE to the projects, fix bug in VISIR     2005/03/02    2_4_13
# --------------------------------------------------------------------------
# PSEUDO CODE
# --------------------------------------------------------------------------
#
# INITIAL SETTINGS
#
# MAIN
# Read and verify arguments: 'CheckType', 'FileCategory' and 'RemoteFile' 
# (file on RemoteHost machine).
#
# According to the FileCategory, and if List is requested, perform it,
# then exit.
#
# set lockfile name: 'LockFile'=RemoteFile.LOCK
#
# performing check-in / check-out:
# case check-out
#    if LockFile  exists on RemoteHost
#    then
#       echo "File locked: impossible to check it out"
#       exit 
#    else
#       if RemoteFile exists on RemoteHost
#       then
#          cp RemoteFile on local directory
#          rename it as LockFile on RemoteHost
#       else
#          echo "File not found on RemoteHost: impossible to check it out"
#          exit 
#       fi
#    fi
# case check-in  
#    if LockFile exists on RemoteHost
#    then
#       if RemoteFile is locked by the current user
#       then
#          mv  RemoteFile on RemoteHost           
#          remove LockFile on RemoteHost
#          echo "file successfully archived"
#       else
#          echo "File locked by another user: wait for unlock, and check it
#          out."
#       fi
#    else
#       if RemoteFile already exists on RemoteHost
#       then
#          echo "Archive not performed, since user has first to check-out
#                the file."
#       else
#          mv  RemoteFile on RemoteHost
#          echo "First archive of the file."
#          send an automatic email to DFS QA (and possibly to SDD secretary).
#       fi
#    fi
# case *
#    echo "Error: CheckType not defined"
#    exit
# esac
#


#
# ---------------------------------------------------------------------------
# INTERNAL FUNCTIONS
# ---------------------------------------------------------------------------
#


#error()
#{
#  standard error message
#   necho '\n***ERROR  ! ***' $
#* > /dev/tty
#   false
#}
error()
{
#  standard error message
   necho "\n***ERROR  ! ***"
}



#beaware()
#{
#  standard be aware message
#   echo '\n***WATCH OUT  ! ***' $* > /dev/tty
#   false
#}
beaware()
{
#  standard be aware message
   necho "\n***WATCH OUT  ! ***"
}

necho()
{
#
#  A safe way to perform echo without sending a new-line
#
   if [ "$SystemOS" = "Linux" ]
   then
      echo -e $NNN "$*" $CCC > /dev/tty
   else
      echo $NNN "$*" $CCC > /dev/tty
   fi
}




# --------------------------------------------------
#  A  safe way to perform a copy into the repository
# --------------------------------------------------
#
# assume that SourceFile and TargetFile are already set

safecp()
{
chmod g+w ${SourceFile}
chmod a+r ${SourceFile}
chgrp "dmd_df" ${SourceFile}

TargetFileBis=${TargetFile}".bis"

cp ${SourceFile} ${TargetFileBis} 2>/dev/null

TestListBis=`(ls -d ${TargetFileBis}) 2>/dev/null`

if [ "${TestListBis}" = "${TargetFileBis}" ]
then
    FileDiff=`diff ${SourceFile} ${TargetFileBis}` 
    if [ "${FileDiff}" = "" ]
    then
#      remove previous TargetFile version     
       rm  -f ${TargetFile} 2>/dev/null
#      rename the Target
       mv $TargetFile.bis  $TargetFile
       chmod g+w ${TargetFile} 2>/dev/null
       chmod a+r ${TargetFile} 2>/dev/null
       chgrp "dmd_df" ${TargetFile} 2>/dev/null
    else
       error "copy of $SourceFile started but failed."
#      rm -f ${TargetFileBis}
       exit
     fi
else
     error
     necho "   You don't have permission to copy a file in the DFS repository."
     necho "   Please, contact your system administrator for support."
     necho ""
     exit
fi
}




# ---------------------------------------------------
# verification of file name against DFS Group naming
# rules for web pages or Release Notes ONLY.
# ---------------------------------------------------
#

checkProjectName()
{
#set -x
ProjectName="misc"
if [ "${OrigRemoteFile}" != "" ]
then
   UpperListPatern=`(echo ${OrigRemoteFile} | tr '[a-z]' '[A-Z]')`;
   PaternToSearch=`(echo ${UpperListPatern} | awk -F- '{print $1}' -)`
   #echo "PaternToSearch  here:  xxx${PaternToSearch}xxx"
   case "$PaternToSearch" in
#   ADVANCED) ProjectName="advanced" ;;
   AMBER) ProjectName="amber" ;;
   ASTO) ProjectName="asto" ;;
   BET) ProjectName="bet" ;;
   CALIBDB) ProjectName="calibdb" ;;
   COMMON) ProjectName="common" ;;
   CPL) ProjectName="cpl" ;;
   CRIRES) ProjectName="crires" ;;
   DBB) ProjectName="ohs" ;;
   DFS) ProjectName="dfs" ;;
#   ETC) ProjectName="etc" ;;
   EVM) ProjectName="ohs" ;;
#   FIBER) ProjectName="fiber" ;;
   FLAMES) ProjectName="flames" ;;
   FORS) ProjectName="fors" ;;
   GASGANO) ProjectName="gasgano" ;;
   GENERAL) ProjectName="general" ;;
   GIRAFFE|GIRAF) ProjectName="giraffe" ;;
   ISAAC) ProjectName="isaac" ;;
   LTS) ProjectName="ohs" ;;
   MIDAS) ProjectName="midas" ;;
   MIDI) ProjectName="midi" ;; 
   NACO) ProjectName="naco" ;;
   NGAS|NGAMS) ProjectName="ngas" ;;
   OCA) ProjectName="oca" ;;
   OLAS) ProjectName="olas" ;;
   OMEGACAM) ProjectName="omegacam" ;;
   OT) ProjectName="ohs" ;;
   P2PP) ProjectName="ohs" ;;
   PHRS) ProjectName="ohs" ;;
   PANELTOOL) ProjectName="ohs" ;;
#   PIPELIB) ProjectName="pipelib" ;;
#   PLDFS) ProjectName="pl-dfs" ;;
   PRIMA) ProjectName="prima" ;;
   QC) ProjectName="qc" ;;
   RBS) ProjectName="rbs" ;;
   REI) ProjectName="rei" ;;
   SAF|FTU) ProjectName="saf" ;;
   SEG) ProjectName="seg" ;;
   SINFONI|SINFO) ProjectName="sinfoni" ;;
   SOFI) ProjectName="sofi" ;;
   STS) ProjectName="ohs" ;;
   UVES) ProjectName="uves" ;;
   VIMOS) ProjectName="vimos" ;;
   VINCI) ProjectName="vinci" ;;
   VIRCAM) ProjectName="vircam" ;;
   VISIR) ProjectName="visir" ;;
   VLT) ProjectName="vlt" ;;
   VLTI) ProjectName="vlti" ;;
#   VST) ProjectName="vst" ;;
   INFRAS) ProjectName="infras" ;;
   INTERF) ProjectName="interf" ;;
   OHS) ProjectName="ohs" ;;
   DO) ProjectName="do" ;;
   *) ProjectName="misc" ;;
   esac
fi
set +x
}

# ---------------------------------------------------
# verify the file name against DFS Group naming rules
# for docread and docsrc ONLY.
# ---------------------------------------------------
#

checkFileName()
{

# Role is to check all file naming rules given at:
# http://www.eso.org/projects/dfs/team/ESO-doc-proc.html#name


# but first, it's not possible to enter a filename which is not in PWD
# --------------------------------------------------------------------

DocName=`(echo ${OrigRemoteFile} | grep -i "\/") 2>/dev/null`;
if [ "$DocName" = "${OrigRemoteFile}" ]
then
   error
   necho "   Impossible to archive, get or check-out a file which is not in"
   necho "   your current directory: $PWD"
   necho "   Please, first go to the directory where the file is located, or"
   necho "   move it into the current directory."
   necho ""
   exit 1
fi



#
# Check the Project code
# ----------------------
ProjectCode=`(echo ${RemoteFile} | awk -F- '{print $1}' -)`
case "$ProjectCode" in
VLT|VLTI|ALMA|GEN)
#    echo " "
#   necho "verifying filename: PROJECT code is valid."
   ;;
*)   error;
   necho "   Invalid PROJECT code.";
   necho "   Please, use one of the following values:  VLT  VLTI  ALMA  GEN";
   necho "   Must be in UPPERCASE and followed by a dash (-) character.";
   necho ""
   exit
   ;;
esac


#
# Check the Type Code
# -------------------
TypeCode=`(echo ${RemoteFile} | awk -F- '{print $2}' -)`
case "$TypeCode" in
ICD|INS|MAN|MEM|MIN|PLA|PRO|SOW|SCH|SPE|TRE|VER)
#    echo " "
#   necho "verifying filename: Document TYPE code is valid."
   ;;
*)   error;
   necho "   Invalid document TYPE code. Please, ONLY use values given at:";
   necho "   http://www.eso.org/projects/dfs/team/type-doc-list.txt";
   necho "   Must be UPPERCASE and separated by dash (-) characters.";
   necho ""
   exit
   ;;
esac

#
# Check the Originator code
# -------------------------
OriginatorCode=`(echo ${RemoteFile} | awk -F- '{print $3}' -)`
case "$OriginatorCode" in
ESO)
#   echo " "
#   necho "verifying filename: ORIGINATOR code is valid."
   ;;
*)   error;
    necho "   Invalid ORIGINATOR code. Allowed value(s): ESO";
#   necho "   Invalid ORIGINATOR code. Please, ONLY use value(s) given at:";
#   necho "   http://www.eso.org/projects/dfs/team/psc-def-list.txt";
#   necho "   (e.g.  ESO-...).";
   necho "   Must be separated by dash (-) characters.";
   necho ""
   exit
   ;;
esac


#
# Check the PSC
# -------------
FilePSC=`(echo ${RemoteFile} | awk -F- '{print $4}' -)`
case "$FilePSC" in
19000|19100|19200|19300|19400|19500|19600|17240|14600|13700|15000)
#   necho "verifying filename: PSC code is valid."
   HeaderRepFile="${OrigPath}doc/rep/templates/header-"${FilePSC}"-rep.htm"
   ;;
*)   error;
   necho "   Invalid PSC code. Please, ONLY use values given at:";
   necho "   http://www.eso.org/projects/dfs/team/psc-def-list.txt";
   necho "   Must be UPPERCASE and separated by dash (-) characters.";
   necho ""
   exit
   ;;
esac


#
# Check the chrono number
# -----------------------
ChronoNumber=`(echo ${RemoteFile} | awk -F- '{print $5}' -)`
IssueExist=`(echo ${ChronoNumber} | grep '\.')`
if [ "${IssueExist}" = "${ChronoNumber}" ]
then
   error
   echo "   Issue number is missing while it is mandatory."
   echo "   Please, specifiy an issue number (-Vi) after the chrono number."
   echo ""
   exit
fi
IssueExist=`(echo ${ChronoNumber} | grep '\_')`
if [ "${IssueExist}" = "${ChronoNumber}" ]
then
   error
   echo "   Issue number is missing while it is mandatory."
   echo "   Please, specifiy an issue number (-Vi) after the chrono number."
   echo ""
   exit
fi

#
# filter out non-numerical values.
TestChronoNumber=`(echo ${ChronoNumber} | grep '[^0-9]')`
if [ "${TestChronoNumber}" != "" ]
then
   error
   echo "   The chrono number must contain numerical characters, ONLY."
   echo ""
   exit
fi

# add 0 to be sure to have a numerical value
ChronoValue=`expr $ChronoNumber + 0`

if [ "${ChronoValue}" -lt 0 -o "${ChronoValue}" -gt 99999 ]
then
   error
   echo "   The chrono number must be a positive integer value, not greater"
   echo ""
   echo "   than 99999."
   echo ""
   exit
fi 


#
# Check the Issue number
# ----------------------
DocIssueFormat=`(echo ${RemoteFile} | awk -F- '{print $6}' -)`
IssueNumber=`(echo ${DocIssueFormat} | awk -F. '{print $1}' -)`

#
# filter out case where version number contains a dot (Vx.y)
ExtendedNb=`(echo ${DocIssueFormat} | awk -F. '{print $2}' -)`
TestExtendedNb=`(echo ${ExtendedNb} | grep '[0-9]')`
if [ "${TestExtendedNb}" != "" ]
then
   error
   echo "   The Version number must be a UNIQUE positive integer value (Vi"
   echo "   and not Vi_j or Vi.j)."
   echo "   It must NOT contain any non-numerical character (such as"
   echo "   underscore, dot, dash, ...)."
   echo ""
   exit
fi

#
# filter out case where version number contains a dash (Vx-y)
ExtendedNb=`(echo ${RemoteFile} | awk -F- '{print $7}' -)`
TestExtendedNb=`(echo ${ExtendedNb} | grep '[0-9]')`
if [ "${TestExtendedNb}" != "" ]
then
   error
   echo "   The Version number must be a UNIQUE positive integer value, not"
   echo "   greater than 999."
   echo "   It must NOT contain any non-numerical character (such as"
   echo "   underscore, dot, dash, ...)."
   echo ""
   exit
fi

#
# filter out case where version number doesn't start with V (or v)
IssuePatern=`(echo ${IssueNumber} | grep -i "V") 2>/dev/null`;
if [ "$IssuePatern" != "${IssueNumber}" ]
then
   error
   echo "   Incorrect release number: must start by 'V', then be a unique"
   echo "   numerical value, i.e. WITHOUT any intermediate character (such as"
   echo "\underscore, dot, dash, ...)."
   echo ""
   exit
fi

#
# filter out case where version number contains non-numerical characters (A-Z-_.;,).
IssueDigits=`(echo ${IssueNumber} | awk -FV '{print $2}' -)`
TestIssueDigits=`(echo ${IssueDigits} | grep '[^0-9]')`
if [ "${TestIssueDigits}" != "" ]
then
   error
   echo "   The version number must contain numerical characters, ONLY."
   echo "   It must NOT contain any non-numerical character (such as"
   echo "   underscore, dot, dash, ...)."
   echo "   It must be a unique postivive integer, not greater than 999."
   echo ""
   exit
fi

#
# Here, as it's a unique numerical number, filter out values out of range.
IssueValue=`expr $IssueDigits + 0`
if [ "${IssueValue}" -lt 0 -o "${IssueValue}" -gt 9999 ]
then
   error
   echo "   The version number must be a positive integer, not greater than"
   echo "   9999."
   echo "   It must NOT contain any non-numerical character (such as"
   echo "   underscore, dot, dash, ...)."
   exit
fi 


}




# -------------------------------------------------
# untar and look for HTML pages (for -docread only)
# -------------------------------------------------
#
checkTarFile()
{

WhereAmI=`pwd`
WhereToGo=${RemotePath}"tar/"${NameButExt}"/"

# remove any previous files under this directory
WhatToRemove=${WhereToGo}"*"
rm -rf ${WhatToRemove}

# to set the directory with appropriate permissions
# first remove it, and then re-create it.
rmdir ${WhereToGo}
mkdir ${WhereToGo}
chmod g+w ${WhereToGo}
chmod a+r ${WhereToGo}
chgrp "dmd_df" ${WhereToGo}

# copy the Target File where I need to uncompress/untar/verify it.
cp ${TargetFile} ${WhereToGo}
TempTargetFile=`(echo ${TargetFile} | awk -F/ '{print $NF}' -)`
TempTargetFile=${WhereToGo}${TempTargetFile}
chmod g+w ${TempTargetFile}
chmod a+r ${TempTargetFile}
chgrp "dmd_df" ${TempTargetFile}

# Uncompress, if necessary.
if [ "${DocExtension}" = "TGZ" ]
then
   TempTargetFile=${WhereToGo}${NameButExt}".TGZ"
   gunzip ${TempTargetFile}
#  File has been uncompressed
   TempTargetFile=${WhereToGo}${NameButExt}".tar"
fi

if [ "${DocExtension}" = "TAR" -a "${DocExtension2}" = "GZ" ]
then
   TempTargetFile=${WhereToGo}${NameButExt}".TAR.GZ"
   gunzip ${TempTargetFile}
#  File has been uncompressed
   TempTargetFile=${WhereToGo}${NameButExt}".TAR"
fi

# As we have now an uncompressed TAR file, we need 
# to check whether it contains HTML pages, and if so,
# untar and publish the whole HTML pages.
# --------------------------------------------------- 
IsLatex=`(gtar -tf ${TempTargetFile} | grep 'index.html$' )`
if [ "IsLatex" != "" ]
then
   IndexNumber=`(gtar -tf ${TempTargetFile} | grep -c 'index.html$' )`
   if [ "${IndexNumber}" = "1" ]   
   then
      FileOnWeb="tar/"${NameButExt}"/"${IsLatex}
      gtar -xof ${TempTargetFile} -C ${WhereToGo}   
#     force adequate permissions, because may have changed
#     after the gtar -xof
      chmod g+w ${WhereToGo}
      chmod a+r ${WhereToGo}
      chgrp "dmd_df" ${WhereToGo}
      cd ${WhereToGo}
      for i in `(gtar -tf ${TempTargetFile}) 2>/dev/null` 
      do
         chmod g+w $i
         chmod a+r $i
         chgrp "dmd_df" $i          
      done
      cd ${WhereAmI}
   else
      beaware
      necho "   The Tar file contains 2 or more 'index.html' files."
      necho "   Don't know which one to choose. Publishing failed."
   fi
fi

# in any case, this TempTargetFile must be removed.
rm -rf ${TempTargetFile}


}





# ------------------------------------------------------
# update the DFS repository web page (for -docread only)
# ------------------------------------------------------
#
updateDFSrepository()
{

# For any new version of a readable document, automatically
# update the DFS Group repository.
# ---------------------------------------------------------

CurrentDate=`(date | awk '{print $1 " " $2 " " $3 " , " $NF}')`
# this is tricky: $NF because year is either $6 or $7 (depends on the host !)

ESODocRef0=`(expr ${RemoteFile} : '\(.*-V\)') 2>/dev/null`
ESODocRef=`(basename ${ESODocRef0} "-V")`

# RemoteFileOnWeb can be RemoteFile itself or a directory (Latex)
RemoteFileOnWeb="../"${FilePSC}"/"${FileOnWeb}

# same as RemoteFileOnWeb but via http
HttpFileOnWeb="Direct access to the document:  http://www.eso.org/projects/dfs/dfs-shared/doc/"${FilePSC}"/"${FileOnWeb}


ItemDocLine1="  <tr>    <td width=\"30%\" valign=\"top\">${LocalUser}<br>"
ItemDocLine2="  ${ESODocRef}</td>"
ItemDocLine3="    <td width=\"50%\" valign=\"top\">"
ItemDocLine4="    <a href=\"${RemoteFileOnWeb}\">${DocumentTitle}</a></td>"
ItemDocLine5="    <td width=\"10%\" valign=\"top\">${CurrentDate}<br>"
ItemDocLine6="    ${DocIssueFormat}</td>  </tr>" 
ItemDocLine=${ItemDocLine1}${ItemDocLine2}${ItemDocLine3}${ItemDocLine4}${ItemDocLine5}${ItemDocLine6}
LocalDocItem="LocalDocItem"
touch  ${LocalDocItem}
echo "$ItemDocLine" > $LocalDocItem


MyDocItem="${OrigPath}doc/rep/DocItems/Item-${RemoteFile}.htm"
#chmod g+w $LocalDocItem
#chmod a+r $LocalDocItem
#chgrp "dmd_df" ${LocalDocItem}

SourceFile=${LocalDocItem}
TargetFile=${MyDocItem}
safecp
#cp $LocalDocItem $MyDocItem

#chmod g+w $MyDocItem 2>/dev/null
#chmod a+r $MyDocItem 2>/dev/null
#chgrp "dmd_df" ${MyDocItem} 2>/dev/null

rm -f $LocalDocItem

#
# as my new DocItem is copied into the repository, I need now to retrieve ALL of them, i.e ones
# related to ALL possible types having same PSC I just modified.

#
# First, I need to define a unique temporary file
ConcatDate=`(date | awk '{print $2$3$4$NF}' -)`
RepTempFile=${OrigPath}"doc/rep/DocItems/Rep"${ConcatDate}${LocalUser}

FooterTypeDoc=$OrigPath"doc/rep/templates/footer-type-doc.htm"

# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# ICD
PathICDDocItems=${OrigPath}"doc/rep/DocItems/Item-*ICD*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-ICD-type.htm"
cat $HeaderTypeDoc > $RepTempFile
for i in `(ls -r $PathICDDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile


# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# INS
PathINSDocItems=${OrigPath}"doc/rep/DocItems/Item-*INS*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-INS-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathINSDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile


# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# MAN
PathMANDocItems=${OrigPath}"doc/rep/DocItems/Item-*MAN*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-MAN-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathMANDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile



# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# MEM
PathMEMDocItems=${OrigPath}"doc/rep/DocItems/Item-*MEM*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-MEM-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathMEMDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile



# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# MIN
PathMINDocItems=${OrigPath}"doc/rep/DocItems/Item-*MIN*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-MIN-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathMINDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile



# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# PLA
PathPLADocItems=${OrigPath}"doc/rep/DocItems/Item-*PLA*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-PLA-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathPLADocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile



# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# PRO
PathPRODocItems=${OrigPath}"doc/rep/DocItems/Item-*PRO*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-PRO-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathPRODocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile


# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# REP
PathREPDocItems=${OrigPath}"doc/rep/DocItems/Item-*REP*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-REP-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathREPDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile



# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# SPE
PathSPEDocItems=${OrigPath}"doc/rep/DocItems/Item-*SPE*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-SPE-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathSPEDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile

# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# TRE
PathTREDocItems=${OrigPath}"doc/rep/DocItems/Item-*TRE*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-TRE-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathTREDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile



# DocItems files sorted in reverse alphabetic order, i.e. latest one first
# VER
PathVERDocItems=${OrigPath}"doc/rep/DocItems/Item-*VER*"${FilePSC}"*.htm"
HeaderTypeDoc="${OrigPath}doc/rep/templates/header-VER-type.htm"
cat $HeaderTypeDoc >> $RepTempFile
for i in `(ls -r $PathVERDocItems) 2>/dev/null` 
do
   cat $i >> $RepTempFile
done
cat $FooterTypeDoc >> $RepTempFile


# I can then create/update the corresponding DFS repository list
OldDfsRep=$OrigPath"doc/rep/dfs-doc-repository-"${FilePSC}".htm"
NewDfsRep="NewDfsRep"
FooterRep=$OrigPath"doc/rep/templates/footer-rep.htm"

cat $HeaderRepFile $RepTempFile $FooterRep > $NewDfsRep

#chmod g+w ${NewDfsRep}
#chmod a+r ${NewDfsRep}
#chgrp "dmd_df" ${NewDfsRep}

SourceFile=${NewDfsRep}
TargetFile=${OldDfsRep}
safecp
#cp $NewDfsRep $OldDfsRep

#chmod g+w ${OldDfsRep} 2>/dev/null
#chmod a+r ${OldDfsRep} 2>/dev/null
#chgrp "dmd_df" ${OldDfsRep} 2>/dev/null

rm -f $NewDfsRep
rm -f $RepTempFile
}





HelpHeader()
{
#
# Header displayed when Help is required
#
touch $HelpHeaderFile

echo "NAME    webcheck --- archive DFS Group document files" > $HelpHeaderFile
echo "DESCRIPTION" >> $HelpHeaderFile
echo "   Command line tool to archive, get a copy or check-out a document"  >> $HelpHeaderFile
echo "   file on a dedicated DFS Group repository."  >> $HelpHeaderFile

echo "   It can be used only if three following conditions apply:" >> $HelpHeaderFile
echo "   1) run with uid belonging to the 'dmd_df' group (gid=202)" >> $HelpHeaderFile
echo "   2) if 'dmd_df' is not the default group, being able to chgrp to it." >> $HelpHeaderFile
echo "   3) have read/write access to the DFS shared disk: /home/dataflow/bin/QAtools" >> $HelpHeaderFile
echo "      or  /export/diskri/dataflow/bin/QAtools" >> $HelpHeaderFile
echo "      (this is the disk where the SNIFF repository is located)" >> $HelpHeaderFile
echo "   Please, ask your system administrator for support." >> $HelpHeaderFile

echo "   Webcheck is to run at least on following hosts: usg0, arcus1, arcus2, dfs1 and dfs2 (Solaris);" >> $HelpHeaderFile
echo "   seg1, dfs4 and arcdev1 (HP-UX)." >> $HelpHeaderFile

echo "   Each time a file is checked-out by a given DFS Group member," >> $HelpHeaderFile
echo "   a lock is set, disabling all other DFS Group members to" >> $HelpHeaderFile
echo "   check it out again and thus modify it." >> $HelpHeaderFile

echo "   Following file categories can be archived in the repository:" >> $HelpHeaderFile
echo "      * stable/complete DFS Group document issues" >> $HelpHeaderFile
echo "      * DFS Group To-Do lists" >> $HelpHeaderFile
echo "      * DFS Group schedules" >> $HelpHeaderFile
echo "      * DFS Group Release Notes (to be delivered for any new software release)" >> $HelpHeaderFile
echo "      * web pages which can contribute to the DFS Group website" >> $HelpHeaderFile

echo "   All archived files are accessible from the DFS Group website." >> $HelpHeaderFile
echo "   Only exception is about sources of documents (i.e. documents archived" >> $HelpHeaderFile
echo "   via the -docsrc argument), which can only be retrieved via webcheck." >> $HelpHeaderFile
echo "   Depending on the file category and the fact the file is archived for the" >> $HelpHeaderFile
echo "   'first' time, publishing on the DFS Group website is automatic or will" >> $HelpHeaderFile
echo "   require a manual intervention from the DFS QA manager." >> $HelpHeaderFile


echo "   Only 'stable and complete versions' of documents written by DFS Group" >> $HelpHeaderFile
echo "   members must be archived via webcheck. Webcheck is not meant" >> $HelpHeaderFile
echo "   to support working/living versions or document drafts." >> $HelpHeaderFile


echo "   The file name is verified and must then be compliant with DFS Group" >> $HelpHeaderFile
echo "   rules related to the file category." >> $HelpHeaderFile

}


HelpUsage()
{
#
# Usage for webcheck, when Help is required
#
touch $HelpUsageFile

echo "USAGE    webcheck  -in   -xxx   filename   Archive a file." > HelpUsageFile
echo "   webcheck  -get  -xxx   filename   Get a copy of an archived file" >> HelpUsageFile
echo "                  (without setting a lock)." >> HelpUsageFile
echo "   webcheck  -out  -xxx   filename   Check-out an archived file" >> HelpUsageFile
echo "                    (and set a lock)." >> HelpUsageFile
echo "   webcheck  -l    -xxx   [searchpatern]   List the DFS repository content." >> HelpUsageFile
echo "where   -xxx specifies ONE (and only one) of following file categories:" >> HelpUsageFile
echo "      -todo       To-Do list file." >> HelpUsageFile
echo "      -rel        Release Note file." >> HelpUsageFile

echo "      -sched      schedule file." >> HelpUsageFile
echo "      -page      web page." >> HelpUsageFile
echo "      -docsrc      SOURCES of a document to be configuration" >> HelpUsageFile
echo "            controlled (WORD or Latex format, preferably)." >> HelpUsageFile
echo "      -docread   READABLE copy of a document to be configuration" >> HelpUsageFile
echo "            controlled." >> HelpUsageFile

echo "ARGUMENT SYNONYMS     -l       --> -list" >> HelpUsageFile
echo "     -in      --> -i" >> HelpUsageFile
echo "     -out      --> -o" >> HelpUsageFile
echo "     -get      --> -g" >> HelpUsageFile
echo "     -todo      --> -t" >> HelpUsageFile
echo "     -rel      --> -r  or  -n (like release or note)" >> HelpUsageFile
echo "     -sched   --> -s" >> HelpUsageFile
echo "     -page      --> -p  or  -w (like page or web)" >> HelpUsageFile
echo "     -docread   --> -R  or  -read (like readable)" >> HelpUsageFile
echo "     -docsrc   --> -S  or  -src (like sources)" >> HelpUsageFile

}


HelpOperands()
{
#
# Operands for webcheck, when Help is required
#
touch $HelpOperandsFile

echo "   filename is mandatory with -in, -get and -out arguments." > HelpOperandsFile

echo "   Any file archived with a new specific name (i.e. name not already" >> HelpOperandsFile
echo "   existing in the DFS repository) is considered as 'archived for" >> HelpOperandsFile
echo "   the first time'." >> HelpOperandsFile
echo "   A file archived for the first time is not automatically published on" >> HelpOperandsFile
echo "   the DFS Group web, but the DFS Group QA manager (and, for -docsrc," >> HelpOperandsFile
echo "   the SDD secretary, too) automatically receives a related notification" >> HelpOperandsFile
echo "   and may then decide to publish it." >> HelpOperandsFile
echo "   Exception is for any new READABLE document archived for the first time" >> HelpOperandsFile
echo "   (-docread): such document is automatically published and accessible at:" >> HelpOperandsFile
echo "   http://www.eso.org/projects/dfs/team/DFS-system.html" >> HelpOperandsFile

echo "   Any file already archived and published on the web can be" >> HelpOperandsFile
echo "   checked-out, modified locally and re-archived: the updated" >> HelpOperandsFile
echo "   version will then automatically be updated on the web." >> HelpOperandsFile

echo "FILE NAMING RULES" >> HelpOperandsFile
echo "   Every file must be named with rules depending on its category." >> HelpOperandsFile
echo "   Deviations may lead the file to be rejected by webcheck." >> HelpOperandsFile

echo "-todo filename:" >> HelpOperandsFile
echo "   must start with one of the DFS Group projects standard abbreviations" >> HelpOperandsFile
echo "   (see: http://www.eso.org/projects/dfs/team/abbreviations.html)" >> HelpOperandsFile
echo "   followed by a dash (-) character," >> HelpOperandsFile
echo "   than ends with '-to-do-open.txt' or '-to-do-close.txt'" >> HelpOperandsFile

echo "-sched filename:" >> HelpOperandsFile
echo "   must start one of the DFS Group standard abbreviations" >> HelpOperandsFile
echo "   (see: http://www.eso.org/projects/dfs/team/abbreviations.html)," >> HelpOperandsFile
echo "   followed by the '-sched-' string, rest of filename being free." >> HelpOperandsFile
echo "   Authorized file extensions are: GIF, JPG and TXT" >> HelpOperandsFile

echo "-page filename:" >> HelpOperandsFile
echo "   must start with one of the DFS Group standard abbreviations" >> HelpOperandsFile
echo "   (see: http://www.eso.org/projects/dfs/team/abbreviations.html)" >> HelpOperandsFile
echo "   followed by a dash (-) character, rest of filename being free." >> HelpOperandsFile
echo "   If not, the file is archived anyway, but in a misc directory. " >> HelpOperandsFile
echo "   Authorized file extensions are: PDF, HTML, HTM, XML, PS, EPS, GIF, JPG, TXT, PPS or PPT" >> HelpOperandsFile

echo "-rel filename:" >> HelpOperandsFile
echo "   must start with one of the DFS Group standard abbreviations" >> HelpOperandsFile
echo "   (see: http://www.eso.org/projects/dfs/team/abbreviations.html)" >> HelpOperandsFile
echo "   followed by a dash (-) character." >> HelpOperandsFile
echo "   If not, the file is archived anyway, but in a misc directory." >> HelpOperandsFile
echo "   Filename must also contain the release number '-i', '-i_j' or '-i_j_k'" >> HelpOperandsFile
echo "   (where i, j and k are digits from 0 to 9)." >> HelpOperandsFile
echo "   Other additional informative strings are allowed." >> HelpOperandsFile
echo "   Authorized file extension is: TXT (even mandatory)." >> HelpOperandsFile

echo "-docsrc filename:" >> HelpOperandsFile
echo "   must strictly be compliant with DFS Group naming rules for documents" >> HelpOperandsFile
echo "   (see: http://www.eso.org/projects/dfs/team/ESO-doc-proc.html#name)" >> HelpOperandsFile
echo "   for instance 'PROJ-TYPE-ESO-PSC-chrono-Vi.ext'" >> HelpOperandsFile
echo "   Authorized file extensions are: DOC, TAR and TGZ." >> HelpOperandsFile

echo "-docread filename:" >> HelpOperandsFile
echo "   must strictly be compliant with DFS Group naming rules for documents" >> HelpOperandsFile
echo "   (see: http://www.eso.org/projects/dfs/team/ESO-doc-proc.html#name)" >> HelpOperandsFile
echo "   for instance 'PROJ-TYPE-ESO-PSC-chrono-Vi.ext'" >> HelpOperandsFile
echo "   Authorized file extensions are: HTML, PS, PDF, TAR and TGZ." >> HelpOperandsFile
echo "   If extension is TAR or TGZ, webcheck untars the file and look for an" >> HelpOperandsFile
echo "   'index.html' file to automatically publish the document on the web." >> HelpOperandsFile

echo "   Please notice: once archived, a file is renamed as 'filename-archived' (on your local host)." >> HelpOperandsFile


echo "OPERANDS   filename must 'stricly' be consistent with naming" >> HelpOperandsFile
echo "   conventions defined for DFS Group documents (see" >> HelpOperandsFile
echo "   http://www.eso.org/projects/dfs/team/ESO-doc-proc.html#name)" >> HelpOperandsFile
echo "   Tar files are allowed for docread and docsrc documents only:" >> HelpOperandsFile
echo "   otherwise, webcheck handles one file at a time." >> HelpOperandsFile
echo "   Creating tar archive via the 'gtar' command is recommended." >> HelpOperandsFile
echo "   GNU compressed files are allowed whatever the file category is." >> HelpOperandsFile

echo "OPTIONS    [searchpatern] is allowed when using the -l argument only," >> HelpOperandsFile
echo "   to limit number of entries which can be returned when listing" >> HelpOperandsFile
echo "   the DFS repository content." >> HelpOperandsFile
echo "   This patern must follow the regular expression rules (man regexp" >> HelpOperandsFile
echo "   for details)." >> HelpOperandsFile
echo "   [searchpatern] can be used or not for any file category, but it is" >> HelpOperandsFile
echo "   mandatory when listing the web pages repository content (using -page" >> HelpOperandsFile
echo "   argument) or listing Release Notes (-rel)." >> HelpOperandsFile


echo "EXAMPLES" >> HelpOperandsFile
echo "   webcheck -l -todo      -->   lists all To-Do lists already" >> HelpOperandsFile
echo "               archived in the repository." >> HelpOperandsFile
echo "   webcheck -l -docread MAN-ESO-19000" >> HelpOperandsFile
echo "            -->   returns all existing readable documents" >> HelpOperandsFile
echo "               whose name contains the 'MAN-ESO-19000'" >> HelpOperandsFile
echo "               patern" >> HelpOperandsFile
echo "   webcheck -in -sched UVES-sched.gif   -->   archive the 'UVES-sched.gif' file" >> HelpOperandsFile
echo "   webcheck -get -docread VLT-MAN-ESO-19000-1781-V24.pdf" >> HelpOperandsFile
echo "            -->   get a copy of the readable file named" >> HelpOperandsFile
echo "               VLT-MAN-ESO-19000-1781-V24.pdf" >> HelpOperandsFile


echo "TIPS TO AUTOMATICALLY OPEN A READABLE DOCUMENT FROM NETSCAPE" >> HelpOperandsFile
echo "   In order to automatically open a PostScript file from Netscape," >> HelpOperandsFile
echo "   please, configure your web browser as following:" >> HelpOperandsFile
echo "   * select the 'Edit>Preferences...' menu item." >> HelpOperandsFile
echo "   * select the 'Navigator>Applications' category." >> HelpOperandsFile
echo "   * in the 'Description' box, verify a 'Postscript Document' entry" >> HelpOperandsFile
echo "     already exists. If so, select it and click on 'Edit'." >> HelpOperandsFile
echo "   * If not, click on 'New'." >> HelpOperandsFile
echo "   * Enter 'application/postscript' as MIMEType." >> HelpOperandsFile
echo "   * Enter 'ai,eps,ps' as Suffixes." >> HelpOperandsFile
echo "   * Select 'Handled By>Application'." >> HelpOperandsFile
echo "   * Enter your path to the ghostview executable: should be similar to" >> HelpOperandsFile
echo "     /usr/server/X11/bin/ghostview %s" >> HelpOperandsFile
echo "   * Verify such path via the 'which' command." >> HelpOperandsFile

echo "   For PDF, same sequence except:" >> HelpOperandsFile
echo "   * Enter 'application/pdf' as MIMEType" >> HelpOperandsFile
echo "   * Enter 'pdf' as Suffixes" >> HelpOperandsFile
echo "   * Select 'Handled By>Application'" >> HelpOperandsFile
echo "   * Enter your path to the acroread executable: should be similar to" >> HelpOperandsFile
echo "     /usr/server/bin/acroread %s" >> HelpOperandsFile




}


HelpUsageOnly()
{
#
# Display Usage only for webcheck, when an error occurs
#
HelpUsage
more $HelpUsageFile
rm -rf $HelpUsageFile
}


#
# ---------------------------------------------------------------------------
# MAIN
# ---------------------------------------------------------------------------

#
# --------------------------------------------------------------------------
# INITIAL SETTINGS
# --------------------------------------------------------------------------
#


# CURRENT VERSION:
CurrentVersion=2_5

# this is to the OS
SystemOS=`uname -s`

LocalUser=`whoami`

#PATH=/sbin:/usr/sbin:/usr/bin:/bin:/usr/5bin:/usr/ucb:/usr/server/gnu/bin
#export PATH

ListFile="no"
ListPatern=""

FileCategory="none"
FileCategoryNumber=0

CheckType="none"
CheckTypeNumber=0

FilePSC="19000"


HelpHeaderFile="HelpHeaderFile"
HelpUsageFile="HelpUsageFile"
HelpOperandsFile="HelpOperandsFile"
HelpFile="HelpFile"

HeaderRepFile=""

RemoteFile=""

OrigPath="/home/dataflow/dfs-shared/"
RemotePath=${OrigPath}


# -----------------------------------------------------------------------------------------
# First, verify which OS is used (and, even if no more useful here, set the remote command)
# -----------------------------------------------------------------------------------------

if [ "${SystemOS}" = "SunOS" ]; 
then
   RemoteCmd=rsh
elif [ "${SystemOS}" = "HP-UX" ]; 
then
   RemoteCmd=remsh
elif [ "${SystemOS}" = "Linux" ]; 
then
   RemoteCmd=rsh
#   echo "Am here,   ${RemoteCmd}"
else
   error
   echo "System OS ${SystemOS} is not supported."
   exit 1
fi


# -- ---------------------------------------------------
# second, display version or help, if requested and exit
# ------------------------------------------------------

if [ "$1" = "-v" -o "$1" = "-version" ]
then
   necho "webcheck version: $CurrentVersion"
   exit
elif [ "$1" = "-h" -o "$1" = "-help" -o "$1" = "" ]
then
   HelpHeader
   HelpUsage
   HelpOperands
   cat $HelpHeaderFile $HelpUsageFile $HelpOperandsFile > $HelpFile
   more $HelpFile
   rm -rf $HelpHeaderFile $HelpUsageFile $HelpOperandsFile $HelpFile
   exit
else
   necho "Initial verifications ..."
fi



# ----------------------------
# verify group ID on LocalHost
# ----------------------------

ConcatDate=`(date | awk '{print $2$3$4$NF}' -)`
GroupDummyFile="Group"${ConcatDate}${LocalUser}
touch ${GroupDummyFile}

GroupTest=`(ls -l ${GroupDummyFile} | awk '{print $4}' -)`

if [ "${GroupTest}" != "dmd_df" ]
then
   chgrp "dmd_df" ${GroupDummyFile} 2>/dev/null
   GroupTest2=`(ls -l ${GroupDummyFile} | awk '{print $4}' -)`
   if [ "${GroupTest2}" != "dmd_df" ]
   then
      error
      necho "   You are not member of group 'dmd_df' or you are not allowed"
      necho "   to assign this value."
      necho "   Please, contact your system administrator for support."
      necho ""
      rm -f ${GroupDummyFile}
      exit
   fi
else
   necho "Good ! groupID (dmd_df) is as expected"
fi
rm -f ${GroupDummyFile}





# -------------------------------------------------------------------------------
# verify the mail command (not always the same on machines where the tool runs !)
# -------------------------------------------------------------------------------
TestMailx=`which mailx  2>/dev/null`
if [ "$TestMailx" = "" ]
then
   # if which returns blank, then it means mailx was not found. Try to find mail instead.
   TestMail=`which mail  2>/dev/null`
   if [ "$TestMail" = "" ]
   then
      # if which returns blank, then it means mail was not found, report a warning.
      necho "--- Warning --- mail and mailx command NOT found. Automatic email notification not enabled"
   else
      necho "Good ! find 'mail' command to be used for email notification"
      MailCmd="mail"   
   fi
else
   # if which does not return blank, then it means mailx was found
   MailCmd="mailx"
   necho "Good ! find 'mailx' command to be used for email notification"
fi


# -------------------------------------------------------------------------------
# verify the gzip command (not always the same on machines where the tool runs !)
# -------------------------------------------------------------------------------

TestGzip=`which gzip  2>/dev/null`
#GzipNotFound=`(which gzip | grep -i "not found")  2>/dev/null`

if [ "$TestGzip" = "" ]
then
   echo "   No gzip command available. Please, verify your PATH variable or"
   echo "   contact your system administrator for support."
   exit
else
   necho "Good ! find `which gzip` command to be used to compress files before archiving"
fi



# --------------------------------
# verify access to the shared disk
# --------------------------------
#echo "RemotePath: ${RemotePath}" 
TestAccess=`(ls -d ${RemotePath}) 2>/dev/null`
if [ "$TestAccess" != "$RemotePath" ]
then
#   OrigPath="/home/dfs1rp3/dataflow/dfs-shared/"
#   RemotePath=$OrigPath
#   TestAccess=`(ls -d $RemotePath) 2>/dev/null`
#   if [ "$TestAccess" != "$RemotePath" ]
#   then
      OrigPath="/export/diskri/dataflow/dfs-shared/"
      RemotePath=$OrigPath
      TestAccess=`(ls -d $RemotePath) 2>/dev/null`
      if [ "$TestAccess" != "$RemotePath" ]
      then
         error
         necho "   Sorry, you don't have access to the DFS repository !"
         necho "   (i.e. to the opus3:/export/diskri/ directory)."
         necho "   First, check that your machine belongs to one of following"
         necho "   machine sets: h_eso, h_dmd, h_pcs or h_linuxtrusted, via"
	 necho "   he ypmatch command. (e.g. ypmatch h_dmd netgroup)."
	 necho "   Second, you have to configure your local machine to mount"
         necho "   this disk."
         necho "   Please, contact your system administrator for support:"
         necho "   it's usually Helpdesk (or, if your machine is maintained"
	 necho "   by SEG, cguirao@eso.org)."  
         exit 1
      else
	 necho "read/write access to DFS repository is fine."
      fi
#   fi
else
   necho "Good ! access to the DFS repository is fine"
fi


# -------------------------------------------------------------
# verify that you can write and remove files on the shared disk
# -------------------------------------------------------------

ConcatDate=`(date | awk '{print $2$3$4$NF}' -)`
AccessDummyFile="Write"${ConcatDate}${LocalUser}

touch $AccessDummyFile
#chmod g+w ${AccessDummyFile}
#chmod a+r ${AccessDummyFile}
#chgrp "dmd_df" ${AccessDummyFile}

SourceFile=${AccessDummyFile}
TargetFile=${RemotePath}"tmp/"${AccessDummyFile}
safecp
# cp $AccessDummyFile ${RemotePath}tmp/.
#chmod g+w ${RemotePath}tmp/${AccessDummyFile} 2>/dev/null
#chmod a+r ${RemotePath}tmp/${AccessDummyFile} 2>/dev/null
#chgrp "dmd_df" ${RemotePath}tmp/${AccessDummyFile} 2>/dev/null

TestAccess=`(ls ${RemotePath}tmp/${AccessDummyFile}) 2>/dev/null`
if [ "$TestAccess" != "${RemotePath}tmp/${AccessDummyFile}" ]
then
   error
   necho "   No write access to the DFS shared disk !"
   necho "   (i.e. to the opus3:/home/dataflow/ disk)."
   necho "   Please, contact your system administrator for support:"
   necho "   it's usually Helpdesk (or, if your machine is maintained"
   necho "   by SEG, cguirao@eso.org)."  
   exit
else
   chmod g+w ${RemotePath}tmp/${AccessDummyFile}
   rm -f ${RemotePath}tmp/${AccessDummyFile}
   TestAccess=`(ls ${RemotePath}tmp/$AccessDummyFile) 2>/dev/null`
   if [ "$TestAccess" = "${RemotePath}tmp/$AccessDummyFile" ]
   then
      error
      necho "   No right to remove files from the DFS shared disk !"
      necho "   (i.e. on the opus3:/home/dataflow/ disk)."
      necho "   Please, contact your system administrator for support:"
      necho "   it's usually Helpdesk (or, if your machine is maintained"
      necho "   by SEG, cguirao@eso.org)."  
      exit
   fi
fi
rm -f ${AccessDummyFile}*




# -------------------------------------------
# Read arguments and set the final RemotePath
# ------------------------------------------- 
while true; do
case $1 in
   -list|-l)      ListFile=yes
	    ;;
   -in|-i)  CheckType=in
	    CheckTypeNumber=`expr $CheckTypeNumber + 1`
            ;;
   -get|-g) CheckType=get
	    CheckTypeNumber=`expr $CheckTypeNumber + 1`
            ;;
   -out|-o) CheckType=out
	    CheckTypeNumber=`expr $CheckTypeNumber + 1`
            ;;
   -todo|-t)   FileCategory=todo;
	    FileCategoryNumber=`expr $FileCategoryNumber + 1`
            RemotePath=$OrigPath"action-items/";
            if [ "$CheckType" = "in" -o "$CheckType" = "get" -o "$CheckType" = "out" ]
	    then
               OrigRemoteFile=$2;
	       RemoteFile=`(echo ${OrigRemoteFile} | tr '[a-z]' '[A-Z]')`;
	       DocExtension=`(echo ${RemoteFile} | awk -F. '{print $2}' -)`
	       DocExtension2=`(echo ${RemoteFile} | awk -F. '{print $3}' -)`
	    fi
            if [ "$ListFile" = "yes" ]
	    then
	       ListPatern=$2;
	    fi
            ;;
   -sched|-s)  FileCategory=sched;
	    FileCategoryNumber=`expr ${FileCategoryNumber} + 1`
            RemotePath=$OrigPath"schedules/";
            if [ "${CheckType}" = "in" -o "${CheckType}" = "get" -o "${CheckType}" = "out" ]
	    then
      	       OrigRemoteFile=$2;
	       RemoteFile=`(echo ${OrigRemoteFile} | tr '[a-z]' '[A-Z]')`;
	       NameButExt=`(echo ${RemoteFile} | awk -F. '{print $1}' -)`;
	       DocExtension=`(echo ${RemoteFile} | awk -F. '{print $2}' -)`;
	       DocExtension2=`(echo ${RemoteFile} | awk -F. '{print $3}' -)`;
	    fi
            if [ "${ListFile}" = "yes" ]
	    then
	       ListPatern=$2;
	    fi
            ;;
   -page|-p|-w|-web)   FileCategory=page;
	    FileCategoryNumber=`expr ${FileCategoryNumber} + 1`
  	    OrigRemoteFile=$2;
            if [ "${CheckType}" = "in" -o "${CheckType}" = "get" -o "${CheckType}" = "out" ]
	    then
	       RemoteFile=`(echo ${OrigRemoteFile} | tr '[a-z]' '[A-Z]')`
	       NameButExt=`(echo ${RemoteFile} | awk -F. '{print $1}' -)`
	       DocExtension=`(echo ${RemoteFile} | awk -F. '{print $2}' -)`
	       DocExtension2=`(echo ${RemoteFile} | awk -F. '{print $3}' -)`
            elif [ "${ListFile}" = "yes" ]
	    then
	       ListPatern=$2;
	    fi
	    #echo "RemotePath before: ${RemotePath}"
            checkProjectName
	    #echo "RemotePath after: ${RemotePath}"
            RemotePath=$OrigPath"web/"${ProjectName}"/"
             ;;
   -rel|-r|-n)    FileCategory=rel;
	    FileCategoryNumber=`expr $FileCategoryNumber + 1`
	    OrigRemoteFile=$2;
            if [ "$CheckType" = "in" -o "$CheckType" = "get" -o "$CheckType" = "out" ]
	    then
	       RemoteFile=`(echo ${OrigRemoteFile} | tr '[a-z]' '[A-Z]')`;
	       NameButExt=`(echo ${RemoteFile} | awk -F. '{print $1}' -)`;
	       DocExtension=`(echo ${RemoteFile} | awk -F. '{print $2}' -)`
	       DocExtension2=`(echo ${RemoteFile} | awk -F. '{print $3}' -)`
            elif [ "$ListFile" = "yes" ]
	    then
	       ListPatern=$2;
	    fi
	    checkProjectName;
            RemotePath=$OrigPath"releases/"${ProjectName}"/";
            ;;
   -docread|-R|-read)    FileCategory=docread;
	    FileCategoryNumber=`expr $FileCategoryNumber + 1`
            if [ "$CheckType" = "in" -o "$CheckType" = "get" -o "$CheckType" = "out" ]
	    then
	       OrigRemoteFile=$2
	       RemoteFile=`(echo ${OrigRemoteFile} | tr '[a-z]' '[A-Z]')`;
	       NameButExt=`(echo ${RemoteFile} | awk -F. '{print $1}' -)`;
	       DocExtension=`(echo ${RemoteFile} | awk -F. '{print $2}' -)`
	       DocExtension2=`(echo ${RemoteFile} | awk -F. '{print $3}' -)`
	       checkFileName;
	    fi
	    RemotePath=${OrigPath}"doc/"${FilePSC}"/";
            if [ "$ListFile" = "yes" ]
	    then
	       ListPatern=$2;
	    fi
#            echo "ListPatern: $ListPatern"
            ;;
   -docsrc|-S|-src) FileCategory=docsrc;
	    FileCategoryNumber=`expr $FileCategoryNumber + 1`

            if [ "$CheckType" = "in" -o "$CheckType" = "get" -o "$CheckType" = "out" ]
	    then
	       OrigRemoteFile=$2;
	       RemoteFile=`(echo ${OrigRemoteFile} | tr '[a-z]' '[A-Z]')`;
	       NameButExt=`(echo ${RemoteFile} | awk -F. '{print $1}' -)`;
	       DocExtension=`(echo ${RemoteFile} | awk -F. '{print $2}' -)`
	       DocExtension2=`(echo ${RemoteFile} | awk -F. '{print $3}' -)`
	       checkFileName;
	    fi
	    RemotePath=${OrigPath}"other/"${FilePSC}"/";
            if [ "$ListFile" = "yes" ]
	    then
	       ListPatern=$2;
	    fi
            ;;
   -*)      error
            necho "   Unknown argument value"
            necho "   For help, please enter: webcheck -h";
            exit
            ;;
   *)       break ;;
   esac
shift
done

if [ "${ListPatern}" = "" ]
then
   ListPatern="r"
fi

# --------------------------
# List the DFS repositories
# --------------------------

if [ "${ListFile}" = "yes" ]
then
#  -in, -get and -out are forbidden with -l
#  ----------------------------------------
   if [ "${CheckTypeNumber}" != "0" ]
   then
      error
      necho "   You can NOT archive, get or check-out if -l argument is"
      necho "   already entered"
      necho "   You must choose one action, and only one: list, archive,"
      necho "   get a copy or check-out a document."
      necho "   For help, please enter: webcheck -h"
      exit
   fi

#  One and only one FileCategory with -l
#  -------------------------------------
   if [ "${FileCategoryNumber}" != "1" ]
   then
      error
      necho "   ONE file category is mandatory when specifying -l argument."
      necho "   For help, please enter: webcheck -h"
      exit
   fi


# OK, listing DFS repository content is now allowed.
# --------------------------------------------------
   case "${FileCategory}" in
   todo)    necho "                ----------------------------------"
	    echo "                     DFS To-Do lists repository";
	    echo "                 ----------------------------------"
	    echo ""
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
            ;;
   sched)   necho "                ----------------------------------"
	    echo "                       DFS schedules repository";
	    echo "                 ----------------------------------"
	    echo ""
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
            ;;
   page)    necho "                ---------------------------------"
	    echo "                      DFS web pages repository"
	    echo "                 ---------------------------------"
	    echo ""
#	    specific case here: web pages are sorted according to
#           the official DFS project abbreviations given at:
#           http://www.eso.org/projects/dfs/team/abbreviations.html
#           if no patern is specified, no result are returned, since it
#           may retrieve too many files.
	    if [ "${ListPatern}" = "r" ]
	    then
               necho "   As no search patern has been specified, the"
	       necho "   page/misc directory is listed."
	       necho "   Please, use official DFS group abbreviations given at:"
	       necho "   http://www.eso.org/projects/dfs/team/abbreviations.html"
	       necho "   as search patern."
	    fi

#           force ListPatern to be lowercase, since web page sub-directories
#           are named with lowercase.
	    LowerListPatern=`(echo ${ListPatern} | tr '[A-Z]' '[a-z]')`;
#	    echo "RemotePath: ${RemotePath}   LowerListPatern: ${LowerListPatern}"
	    ls -lo ${RemotePath} | grep -i "${LowerListPatern}" 2>/dev/null
            ;;
   rel)     necho "                ----------------------------------"
	    echo "                    DFS Release Notes repository";
	    echo "                 ----------------------------------"
	    echo ""
#	    specific case here: Release Notes are sorted according to
#           the official DFS project abbreviations given at:
#           http://www.eso.org/projects/dfs/team/abbreviations.html
#           if no patern is specified, no result are returned, since it
#           may retrieve too many files.
	    if [ "${ListPatern}" = "r" ]
	    then
               necho "   As no search patern has been specified, the"
	       necho "   ReleaseNotes/misc directory is listed."
	       necho "   Please, use official DFS group abbreviations given at:"
	       necho "   http://www.eso.org/projects/dfs/team/abbreviations.html"
	       necho "   as search patern."
	    fi

#           force ListPatern to be lowercase, since web page sub-directories
#           are named with lowercase.
	    LowerListPatern=`(echo ${ListPatern} | tr '[A-Z]' '[a-z]')`;
	    ls -lo ${RemotePath} | grep -i "${LowerListPatern}" 2>/dev/null

            ;;
   docread)     necho "      --------------------------------------------"
	    echo "         DFS READ-ONLY documentation repository"
	    echo "      --------------------------------------------"
	    echo ""
	    RemotePath=$OrigPath"doc/19000/"
            necho "PSC 19000 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/19100/"
            necho "PSC 19100 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/19200/"
            necho "PSC 19200 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/19300/"
            necho "PSC 19300 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/19400/"
            necho "PSC 19400 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/19500/"
            necho "PSC 19500 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/19600/"
            necho "PSC 19600 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/15000/"
            necho "PSC 15000 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/13700/"
            necho "PSC 13700 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/14600/"
            necho "PSC 14600 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
	    necho " "
	    RemotePath=$OrigPath"doc/17240/"
            necho "PSC 17240 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}" | grep -v "drwx"
            necho " "
            ;;
   docsrc)     necho "      --------------------------------------------"
	    echo "          DFS documentation repository (SOURCES)"
	    echo "      --------------------------------------------"
	    RemotePath=$OrigPath"other/19000/"
            necho "PSC 19000 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/19100/"
            necho "PSC 19100 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/19200/"
            necho "PSC 19200 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/19300/"
            necho "PSC 19300 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/19400/"
            necho "PSC 19400 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/19500/"
            necho "PSC 19500 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/19600/"
            necho "PSC 19600 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/15000/"
            necho "PSC 15000 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/13700/"
            necho "PSC 13700 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/14600/"
            necho "PSC 14600 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
	    necho " "
	    RemotePath=$OrigPath"other/17240/"
            necho "PSC 17240 ===>"
	    ls -lo ${RemotePath} | grep -i "${ListPatern}"
            necho " "
            ;;
   none)    error
	    necho "   Unknown file category";
            necho "   For help, please enter: webcheck -h";
            exit
            ;;
   *)       ;;
   esac

#   necho "filename.LOCK file means that filename is currently locked."
#   necho "File owner indicates who performed the check-out : he has first to"
#   necho "archive the file to enable other DFS Group members to check it out"
#   necho "and modify it.\n"
   exit
fi



# ----------------------------------------------------
# Some more verifications before check-in / check-out
# ----------------------------------------------------

# the CheckType must be in, get or out
# ------------------------------------

if [ "$CheckType" != "in"  -a  "$CheckType" != "get" -a  "$CheckType" != "out" ]
then
   error
   necho "   If you don't want to list, it must be: -in, -get or -out."
   necho "   For help, please enter: webcheck -h"
   exit
fi


# it can not be both check-in, get copy and check-out
# ---------------------------------------------------

if [ "${CheckTypeNumber}" != "1" ]
then
   error
   necho "   You can NOT archive, get a copy AND check-out at the same time."
   necho "   For help, please enter: webcheck -h"
   exit 1
fi

            
# a specific filename must be entered
# ------------------------------------

   if [ "${OrigRemoteFile}" = "" -o "${FileCategoryNumber}" != "1" ]
   then
      error
      necho "   File category or filename is missing !"
      necho "   For help, please enter: webcheck -h"
      exit 1
   fi


# it's not possible to specify a directory in RemotePath
# ------------------------------------------------------
if [ "$CheckType" = "get" -o "$CheckType" = "out" ]
then
   DirName=`(ls -l ${RemotePath} | grep ${OrigRemoteFile} | grep "^d") 2>/dev/null`;

   if [ "$DirName" != "" ]
   then
      error
      necho "   ${OrigRemoteFile} is a directory."
      necho "   Only unique file can be specified."
      necho "   If you need to handle a set of files, please, use a tar file."
      exit
   fi
fi

# it's not possible to enter a filename which ends by .LOCK
# ----------------------------------------------------------

DocName=`(echo ${OrigRemoteFile} | grep -i "\.LOCK") 2>/dev/null`;
if [ "$DocName" = "${OrigRemoteFile}" ]
then
   error
   necho "   Impossible to archive or check-out a .LOCK file."
   necho "   Please, verify your filename !"
   necho "   For help, please enter: webcheck -h"
   exit 1
fi


# it's not possible to enter a filename which ends by -archived
# -------------------------------------------------------------

DocName=`(echo ${OrigRemoteFile} | awk -F- '{print $NF}' -) 2>/dev/null`;
if [ "$DocName" = "archived" ]
then
   error
   necho "   Impossible to archive or check-out a file already archived."
   necho "   Please, remove the -archived extension from your filename !"
   necho "   For help, please enter: webcheck -h"
   exit 1
fi



# --------------------------------
# verifications for check-in only.
# --------------------------------

if [ "$CheckType" = "in" ]
then
#  once the filename is valid, then verify it does exist in the current
#  directory.
   LocalFile=`(ls -d ${OrigRemoteFile}) 2>/dev/null` 
   if [ "$LocalFile" != "${OrigRemoteFile}" ]
   then
      error
      necho "   Archiving file ${OrigRemoteFile} failed."
      necho "   This file does NOT exist in your current directory."
      necho "   Please, verify filename, or get a copy, or check it out first."
      necho "   For help, please enter: webcheck -h"
      exit 1
   fi


# it's not possible to specify a directory locally
# ------------------------------------------------
   DirName=`(ls -l $PWD | grep ${OrigRemoteFile} | grep "^d") 2>/dev/null`;

   if [ "$DirName" != "" ]
   then
      error
      necho "   ${OrigRemoteFile} is a directory."
      necho "   Only unique file can be specified."
      necho "   If you need to handle a set of files, please, use a tar file."
      exit
   fi


# Now, verify authorized file formats for DFS To-Do lists
# -------------------------------------------------------

if [ "$FileCategory" = "todo" ]
then
   DocName=`(echo ${OrigRemoteFile} | grep -i "\-to") 2>/dev/null`;
   if [ "$DocName" != "${OrigRemoteFile}" ]
   then
      DocName=`(echo ${OrigRemoteFile} | grep -i "\-to") \
               2>/dev/null`;
      necho "DocName: ${DocName}: --- OrigRemoteFile:${OrigRemoteFile}:" 
      if [ "${DocName}" != "${OrigRemoteFile}" ]
      then
  	 error
         necho "   Filename must be xxx-to-do-open.txt or"
         necho "   xxx-to-do-close.txt when using the -todo argument !"
	 necho "   xxx must be one of abbreviations given at:"
         necho "   http://www.eso.org/projects/dfs/team/abbreviations.html"
	 necho "   File type must be TXT (text) or HTML only."
         necho "Please, apply correct file naming for To-Do lists."
         exit 1
      fi
   fi
   if [ "${DocExtension}" != "TXT" -a "${DocExtension}" != "HTML" -a "${DocExtension}" != "HTM" ]
   then
      error
      necho "   ${DocExtension} is not a valid extension."
      necho "   -todo argument applies on TXT or HTML files ONLY !"
      exit
   else
      if [ "$CheckType" = "in" ]
      then
	 DocName=`(file ${OrigRemoteFile} | grep -i "text") 2>/dev/null`;
	 if [ "$DocName" = "" ]
	 then
	    error
	    necho "   This is not a text file !"
	    necho "   You can archive ONLY text files for -todo."
	    exit
	 fi
      fi
#      necho "verifying filename: extension/format is valid."
   fi
fi


# verify authorized file formats for DFS doc sources
# ---------------------------------------------------
if [ "$FileCategory" = "docsrc" ]
then
   if [ "${DocExtension}" != "DOC" -a "${DocExtension}" != "TAR"\
     -a "${DocExtension}" != "TGZ" ]
   then
      error
      necho "   ${DocExtension} is not a valid extension."
      necho "   -docsrc argument applies on file having a .doc extension"
      necho "   (e.g. WORD), or on TAR archive, ONLY !"
      exit
#   else
#      necho "verifying filename: extension/format is valid."
   fi
fi




# verify authorized file formats for DFS Read-Only doc files
# -----------------------------------------------------------
if [ "$FileCategory" = "docread" ]
then
   if [ "${DocExtension}" != "HTM" -a "${DocExtension}" != "HTML"\
     -a "${DocExtension}" != "PS" -a "${DocExtension}" != "EPS"\
     -a "${DocExtension}" != "TAR" -a "${DocExtension}" != "TGZ"\
     -a "${DocExtension}" != "PDF" ]
   then
      error
      necho "   ${DocExtension} is not a valid extension."
      necho "   -docread argument applies on HTML, Postscript or PDF files ONLY !"
      exit
#   else
#      necho "verifying filename: extension/format is valid."
   fi
fi


# verify authorized file formats for DFS schedules
# -------------------------------------------------
if [ "$FileCategory" = "sched" ]
then
   DocName=`(echo ${OrigRemoteFile} | grep -i "\-sched") 2>/dev/null`;
   if [ "$DocName" != "${OrigRemoteFile}" ]
   then
      error
      necho "   Filename must be xxx-sched.* when using -sched argument !"
      necho "   xxx must be one of abbreviations given at:"
      necho "   http://www.eso.org/projects/dfs/team/abbreviations.html"
      necho "   File format must be GIF, JPG or TXT (text), only."
      necho "Please, apply correct file naming for schedules."
      exit
   fi

   if [ "${DocExtension}" != "GIF" -a "${DocExtension}" != "JPG"\
     -a "${DocExtension}" != "TXT" -a "${DocExtension}" != "HTML" -a "${DocExtension}" != "HTM" ]  
   then
      error
      necho "   ${DocExtension} is not a valid extension."
      necho "   -sched argument applies on HTML, HTM, GIF, JPG or TXT files ONLY !"
      exit
#   else
#      necho "verifying filename: extension/format is valid."
   fi
fi


# verify authorized file formats for DFS web pages
# -------------------------------------------------

if [ "$FileCategory" = "page" ]
then
   if [ "${DocExtension}" != "HTM" -a "${DocExtension}" != "HTML"\
     -a "${DocExtension}" != "PS" -a "${DocExtension}" != "EPS"\
     -a "${DocExtension}" != "PDF" -a "${DocExtension}" != "GIF"\
     -a "${DocExtension}" != "JPG" -a "${DocExtension}" != "XML"\
     -a "${DocExtension}" != "TXT" -a "${DocExtension}" != "PPT"\
     -a "${DocExtension}" != "PPS" ]
   then
      error
      necho "   ${DocExtension} is not a valid extension."
      necho "   Please, use HTML, XML, PS, PDF, GIF, JPG, TXT, PPS or PPT extensions for -page ONLY !"
      exit
#   else
#      necho "verifying filename: extension/format is valid."
   fi
fi

# verify authorized file formats for DFS Release Notes
# ----------------------------------------------------

if [ "$FileCategory" = "rel" ]
then
   if [ "${DocExtension}" != "TXT" ]
   then
      error
      necho "   ${DocExtension} is not a valid extension."
      necho "   Please, use TXT extension for -rel ONLY !"
      exit
#   else
#      necho "verifying filename: extension/format is valid."
   fi
fi


# 
# verify tar file content
# -----------------------
if [ "${DocExtension}" = "TAR" -a "${DocExtension2}" = "" ]
then
   necho "Compress TAR file before archiving ..."
   gzip ${OrigRemoteFile}
   OrigRemoteFile=${OrigRemoteFile}".gz"
   RemoteFile=${RemoteFile}".GZ"
   DocExtension2="GZ"
fi

#
# End of verifications for check-in only.
fi


# Great ! the filename is correct and consistent with CheckType
# --------------------------------------------------------------
 
#necho "\nCheck ${CheckType} requested for file ${OrigRemoteFile}"



if [ "${FileCategory}" = "docread" -o  "${FileCategory}" = "docsrc" ]
then
#  in that case, force filename to be UPPERCASE

#  set lock file name: LockFile=OrigRemoteFile.LOCK
   LockFile=${RemoteFile}".LOCK"
#  verify if the file exists on RemoteHost
   FileExist=`(ls -d ${RemotePath}${RemoteFile}) 2>/dev/null`
   FileToCheck=${RemotePath}${RemoteFile}
else
   LockFile=${OrigRemoteFile}".LOCK"
   FileExist=`(ls -d ${RemotePath}${OrigRemoteFile}) 2>/dev/null`
   FileToCheck=${RemotePath}${OrigRemoteFile}
fi


# verify if a lock exits on RemoteHost 
LockExist=`(ls -d ${RemotePath}${LockFile}) 2>/dev/null` 




# ------------------------------------------------------
# We are now ready to perform the check-in or check-out
# ------------------------------------------------------



case "$CheckType" in
get)   if [ "$FileExist" = "${RemotePath}${OrigRemoteFile}" ]
       then
#         the File exists on RemoteHost: get copy is allowed.
          necho "Accessing the DFS repository ..."
          cp ${RemotePath}${OrigRemoteFile} .

          necho "File ${OrigRemoteFile} copied from DFS repository !"
       else
#         Impossible to get a copy: file is not in the repository.
	  error
          necho "   File ${OrigRemoteFile} does not exist in DFS repository."
          necho "   Please, list repository content and check exact file name."
          exit 1
       fi
       ;;
out)   if [ "$LockExist" = "${RemotePath}${LockFile}" ]
        then
#          There is a lock set by someone else: impossible to check-out.
	   LockUserDate=`ls -ld ${RemotePath}${LockFile} \
                      | awk '{print $3 " since " $6 " " $7 " " $8}'`
	   error
	   necho "   File ${OrigRemoteFile} locked by ${LockUserDate}."
	   necho "   Sorry, impossible to check it out !"
           exit 1
        else
	   if [ "$FileExist" = "${RemotePath}${OrigRemoteFile}" ]
           then
#             No lock and the File exists on RemoteHost: check-out is allowed.
              necho "Accessing the DFS repository ..."
#              cp ${RemotePath}${OrigRemoteFile} ${RemotePath}${LockFile}
              touch ${RemotePath}${LockFile}

              cp ${RemotePath}${OrigRemoteFile} .

#             Be aware ! do NOT remove RemoteFile on RemoteHost otherwise it
#             will not be accessible anymore from our web browser.
#             rm -f ${RemotePath}${OrigRemoteFile}

              necho "File ${OrigRemoteFile} checked-out from DFS repository !"

           else
#             Impossible to check-out a file which is not in the repository.
	      error
              necho "   File ${OrigRemoteFile} does not exist"
              necho "   in DFS repository."
	      necho "   Please, verify the filename or first archive the file"
	      necho "   into the repository."
              exit 1
           fi
        fi
	;;
in)     necho "Accessing the DFS repository ..."
        if [ "$LockExist" = "${RemotePath}${LockFile}" ]
        then
           LockUser=`ls -ld ${RemotePath}${LockFile} \
                        | awk '{print $3}'`
           if [ "$LocalUser" = "$LockUser" ]
           then
#             A lock exists but has been set by the current user: check-in
#             is authorized.
              if [ "${DocExtension}" = "TXT" -a "${DocExtension2}" = "" ]
              then
                 ConcatDate=`(date | awk '{print $2$3$4$NF}' -)`
                 Sed1TempFile="Sed1"${ConcatDate}${LocalUser}
  	         sed "s/^Last update: .*/Last update: `date` \
                 by: $LocalUser/1" $OrigRemoteFile > $Sed1TempFile
                 mv $Sed1TempFile $OrigRemoteFile
              fi
#             File must be writable by the group.
#              chmod g+w ${OrigRemoteFile}
#	      chmod a+r ${OrigRemoteFile}
#	      chgrp "dmd_df" ${OrigRemoteFile}

#             here, I check-in !
	      SourceFile="${OrigRemoteFile}"

              if [ "${FileCategory}" = "docread" -o  "${FileCategory}" = "docsrc" ]
              then
#                in that case, force target filename to be UPPERCASE
		 TargetFile=${RemotePath}${RemoteFile}
	      else
	         TargetFile="${RemotePath}${OrigRemoteFile}"
	      fi

	      safecp
#              chmod g+w ${TargetFile} 2>/dev/null
#	      chmod a+r ${TargetFile} 2>/dev/null
#	      chgrp "dmd_df" ${TargetFile} 2>/dev/null


              rm -f ${RemotePath}${LockFile}
              necho "File ${OrigRemoteFile} successfully archived."

              if [ "$FileCategory" = "docread" ]
              then
	         echo "Please, enter the document title: "
                 read DocumentTitle
                 echo "Document title: ${DocumentTitle}" > ${TitleDummyFile}

		 FileOnWeb=${RemoteFile}
		 if [ "${DocExtension}" = "TGZ" -o "${DocExtension}" = "TAR" ]
		 then
                    necho "As it's a tar archive, still needs some time to be published ..."
                    checkTarFile
		 fi
                 updateDFSrepository

	      fi

# pb: this mailCmd does not work !! why ???
#             email to khaggouc because to be always informed.
#              $MailCmd -s \
# "${OrigRemoteFile} has been UPDATED by ${LocalUser}. Type: ${FileCategory}." \
#              khaggouc@eso.org < $TitleDummyFile

#             rename the file we just archived on local host.
              mv ${OrigRemoteFile} ${OrigRemoteFile}"-archived"
              if [ "${FileCategory}" != "docsrc" ]
	      then
                 necho "This document is now automatically published on the DFS Group website."
	      fi
          else
              error
   	      necho "   File ${OrigRemoteFile} locked by ${LockUser}."
              necho "   It means you did not update the reference version"
	      necho "   (i.e. did not check out from remote host FIRST) !"
              necho "   So, please, wait for unlock, check out the reference version"
	      necho "   modify it locally, and check it in again."
              exit 1
           fi
        else
	   if [ "$FileExist" = "${FileToCheck}" ]
           then
              error
   	      necho "   File ${OrigRemoteFile} already exists in DFS repository."
              necho "   Sorry, you can not archive twice the same file."
              necho "   You have either to rename it giving a new version number, or"
	      necho "   or check-out FIRST the reference issue, and modify it locally."
              exit 1
           else
              if [ "${DocExtension}" = "TXT" -a "${DocExtension2}" = "" ]
              then
#                First, I need to define a unique temporary file
                 ConcatDate=`(date | awk '{print $2$3$4$NF}' -)`
                 Sed2TempFile="Sed2"${ConcatDate}${LocalUser}
  	         sed "s/^Last update: .*/Last update: `date` \
                 by: $LocalUser/1" $OrigRemoteFile > $Sed2TempFile
                 mv $Sed2TempFile $OrigRemoteFile
              fi

#             ################################################
#             This is the first time the file is checked-in.
#             ################################################


#              chmod g+w ${OrigRemoteFile}
#	      chmod a+r ${OrigRemoteFile}
#	      chgrp "dmd_df" ${OrigRemoteFile}

#             Here, I check-in !
	      SourceFile="${OrigRemoteFile}"

              if [ "${FileCategory}" = "docread" -o  "${FileCategory}" = "docsrc" ]
              then
#                in that case, force target filename to be UPPERCASE
		 TargetFile=${RemotePath}${RemoteFile}
	      else
	         TargetFile="${RemotePath}${OrigRemoteFile}"
	      fi

	      safecp
              necho "File ${OrigRemoteFile} successfully archived in the DFS repository."

#              chmod g+w ${TargetFile} 2>/dev/null
#	      chmod a+r ${TargetFile} 2>/dev/null
#	      chgrp "dmd_df" ${TargetFile} 2>/dev/null

#             create a Dummy file to  email the document title
	      ConcatDate=`(date | awk '{print $2$3$4$NF}' -)`
	      TitleDummyFile="Title"${ConcatDate}${LocalUser}
	      MailBody="Document mentioned in email subject has been archived for the first time."
              echo ${MailBody} > ${TitleDummyFile}

              if [ "$FileCategory" = "docread" ]
              then
	         echo "Please, enter the document title: "
                 read DocumentTitle
                 echo "Document title: ${DocumentTitle}" > ${TitleDummyFile}

		 FileOnWeb=${RemoteFile}
		 if [ "${DocExtension}" = "TGZ" -o "${DocExtension}" = "TAR" ]
		 then
                    necho "As it's a tar archive, still needs some time to be published ..."

#                   First, must create a specific directory
		    mkdir ${RemotePath}"tar/"${NameButExt}
                    checkTarFile
		 fi

                 updateDFSrepository

                 echo "${HttpFileOnWeb}" >> ${TitleDummyFile}


# send email to the SDD secretary
                  $MailCmd -s \
 "New!  WEB READABLE ${RemoteFile} file has been archived by ${LocalUser}." \
                  mpeltzer@eso.org < ${TitleDummyFile}

# send email to the submitter
                  $MailCmd -s \
 "New!  WEB READABLE ${RemoteFile} file has been archived by ${LocalUser}." \
                  ${LocalUser}@eso.org < ${TitleDummyFile}

                 necho "This document is now automatically published on the DFS Group website."

		 beaware
                 necho "Please, don't forget to archive the corresponding sources."
		 
	      else
                 necho "If it's a web readable document file, a To-Do list, or a DFS component schedule,"
                 necho "and the corresponding file name is stricly consistent with related naming"
	         necho "rules, it is automatically published on the DFS group website."
	         necho "If not, the QA manager is informed and will publish it soon, if appropriate."
              fi

               if [ "$FileCategory" = "page" ]
               then
HttpFileOnWeb="Direct access to the document:  http://www.eso.org/projects/dfs/dfs-shared/web/"${ProjectName}"/"${OrigRemoteFile}
                 echo "${HttpFileOnWeb}" >> ${TitleDummyFile}
	       fi
              $MailCmd -s "New! ${OrigRemoteFile} file has been archived by ${LocalUser}. Type: ${FileCategory}." khaggouc@eso.org < ${TitleDummyFile}

               if [ "$FileCategory" = "rel" ]
               then
HttpFileOnWeb=" Direct access to the document:  http://www.eso.org/projects/dfs/dfs-shared/releases/"${ProjectName}"/"${OrigRemoteFile}
                 echo "${HttpFileOnWeb}" >> ${TitleDummyFile}
#                 send email to whole SEG
                  $MailCmd -s \
 "New! Release Notes: ${RemoteFile} document have been archived by ${LocalUser}." \
                  seg@eso.org < ${TitleDummyFile}

#                send email to the submitter
                  $MailCmd -s \
 "New! Release Notes: ${RemoteFile} document have been archived by ${LocalUser}." \
                  ${LocalUser}@eso.org < ${TitleDummyFile}
	       fi

               if [ "$FileCategory" = "docsrc" ]
               then
#                 send email to the SDD secretary
                  $MailCmd -s \
 "New!  SOURCES of ${RemoteFile} document have been archived by ${LocalUser}." \
                  mpeltzer@eso.org < ${TitleDummyFile}

#                 send email to the submitter
                  $MailCmd -s \
 "New!  SOURCES of ${RemoteFile} document have been archived by ${LocalUser}." \
                  ${LocalUser}@eso.org < ${TitleDummyFile}
               fi

              rm -f $TitleDummyFile
              mv ${OrigRemoteFile} ${OrigRemoteFile}"-archived"
           fi
        fi
	;;
*)      error
        necho "   Check type not defined. Operation aborted."
	necho "   For help, please enter: webcheck -h"
        exit 1
	;;
esac


#
# end of file
#
