// -*-c++-*-
#ifndef _ArcAmbirep_h_
#define _ArcAmbirep_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcAmbiRep.h,v 1.4 2005/09/15 17:34:32 nferguso Exp $
 *
 * ArcAmbiRep.h - class definitions for class ArcAmbiRep, a class 
 * representing an application that read from the
 * Ambient repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * M Albrecht           08 Jan 1999  Fixed tables to point to ambient DB
 * Neil Ferguson        20 Oct 2004  Converted to DfsStdLib/SybConn
 */

#include <ArcDbGen.h>
#include <ArcTime.h>
#include <ArcStatistics.h>
#include <ArcSeeing.h>
#include <ArcMeteo.h>

#ifdef CORRLIB
#include <ArcMoonVal.h>
#endif



/**
 * This class and its methods are meant to be used for read from the
 * ambient repository database.
 * An instance consists of references to the database.
 * This class includes a set of member functions for retrieving the 
 * meteorological data.
 */
class ArcAmbiRep : public ArcDbGen {

public:

    /// enum type defining the meteo sites 
    enum MeteoSite {
	/// PARANAL
	PARANAL = 0,
	/// LASILLA
	LASILLA
    };	

    /// enum type defining the level where the measures have been taken 
    enum MeasureLevel {
	/// Underground 
	GROUND = 1,
	/// 10 Meters
	METER_10 = 2,
 	/// 30 Meters
	METER_30 = 3
   };	

private:

//* Read the values returned by the SELECT statement and
//* Compute the following statistics: min, max, mean, standard deviation, RMS 
    int readStatistics(ArcStatistics& statisticValues, 
                       DfsCString& select, 
                       double scale);
    
// Get the table meteo related to the site
inline const DfsCString getMeteoTab(const MeteoSite site)
{
    if(site == PARANAL)
      return("ambient..meteo_paranal");
    else
      return("ambient..meteo_lasilla");
}

// Get the table sky related to the site
inline const DfsCString getSkyTab(const MeteoSite site)
{
    if(site == PARANAL)
      return("ambient..sky_paranal");
    else
      return("ambient..sky");
}

  // Get the table seeing related to the site
inline const DfsCString getSeeingTab(const MeteoSite site)
{
    if(site == PARANAL)
      return("ambient..seeing_paranal");
    else
      return("ambient..seeing_lasilla");
}
  
public:
    
/**@name Constructors
*/
//@{
    // Default constructor: it supplies a connection to the Ambient repository.
    // The default values are read from a configuration file
    ArcAmbiRep(DfsCString dbAlias="AMBIREP");

    /// Constructor that requires username, password, server and database name (a default is supplied).
    // In case an empty server name is given, the value of $DSQUERY variable is taken
    ArcAmbiRep(DfsCString dbName, DfsCString dbPasswd, 
               DfsCString dbServer="", DfsCString dbName="ambient");

    /// Constructor that requires a valid SybaseConnection object
    ArcAmbiRep(SybaseConnection& database);

//@}


/**@name  Methods for data retrieval
*/
//@{

/**
****************************************************************************
 * This method retrieves from the ambient database the values of SEEING 
 * parameter for a given period. 
 * The following statistics are computed:
  - average   (returned by method ArcStatistics::getMean())
  - standard deviation (returned by method ArcStatistics::getStdDev())
  - RMS       (returned by method ArcStatistics::getRMS())
  - min value (returned by method ArcStatistics::getMin())
  - max value (returned by method ArcStatistics::getMax())

@memo
 Method for extracting statistics on the SEEING from ambient database.
@return
 0 if OK, 1 if an error occurred, -2 if no data were found for the given period
@param
statisticValues (OUTPUT) class containing the computed values,
fromDate - (INPUT) start of period (MJD format)
toDate   - (INPUT) end of period (MJD format)
site   - (INPUT) physical site (LASILLA or PARANAL)
@see
ArcStatistics for the list of methods, enum MeteoSite for the list of sites
*/

   int getSeeingStat(ArcStatistics& statisticValues,
                     const MJD_DATE& fromDate, const MJD_DATE& toDate, 
                     const MeteoSite site=PARANAL);

//@}

/**
****************************************************************************
 * This method insert in the ambient database the values of SEEING 
 * parameters extracted from a PAF file. 
 * Values inserted are:
   FWHM
   RA (milliarcsec)
   DEC (milliarcsec)
   AIRMASS
@memo
  This method inserts in the ambient database the values of SEEING parameters extracted from a PAF file.
@return
 0 if OK, 1 if an error occurred
@param
seeing (INPUT) class containing the values to be inserted in the db,
dbSession - (INPUT) valid connection to the database
site   - (INPUT) physical site (LASILLA or PARANAL)
@see
ArcSeeing for the list of methods, SybaseConnection class, enum MeteoSite for the list of sites
*/

int insertSeeing(const ArcSeeing& seeing,
                 SybaseConnection& dbSession, 
                 const MeteoSite site=PARANAL);
                            
/**
****************************************************************************
 * This method insert in the ambient database the values of Meteo 
 * parameters extracted from a PAF file. 
 * Values inserted are:
   Temperature at ground level and 10/30 meters
   Dew point temperature at 10/30 meters
   Relative humidity at 10/30 meters
   Pressure
   Wind speed and direction at 10/30 meters
@memo
  This method inserts in the ambient database the values of Meteo parameters 
  extracted from a PAF file.
@return
 0 if OK, 1 if an error occurred
@param
meteo (INPUT) class containing the values to be inserted in the db,
dbSession - (INPUT) valid connection to the database
site   - (INPUT) physical site (LASILLA or PARANAL)
@see
ArcMeteo for the list of methods, SybaseConnection class, enum MeteoSite for the list of sites
*/

int insertMeteo(const ArcMeteo& meteo,
                SybaseConnection& dbSession, 
                const MeteoSite site=PARANAL);
                            
}; // end of definition of class ArcAmbiRep

#endif /* _ArcAmbirep_h_ */

