// -*-c++-*-
#ifndef _ArcAstoFile_h_
#define _ArcAstoFile_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcAstoFile.h,v 1.4 2005/11/22 16:49:24 cguirao Exp $
 *
 * ArcAstoFile.h - class definitions for class ArcAstoFile, a class 
 * representing an application that read from the
 * table allfiles of asto database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  25 May 1998  Created
 * Neil Ferguson        28 Oct 2004  Converted to DfsStdLib/SybConn
 */

#include "DfsCString.h"
#include "DfsTPtrSortedVector.h"
#include <sybconnection.h>

/**
 * This class is used by the ArcProdRep class for storing the data retrieved
 * from the database table allfiles ; 
 * this class contains only methods 
 * for accessing the class's members, none of them access the database fields.
 */
class ArcAstoFile {

public:

    /// enum type that support different types of media
    enum MediaType {
	/// Magnetic Disk = 0
	MD = 0,
	/// CD-ROM = 1
	CD = 1
    } ;	
    
private:
    
    // File ID (column "file_id")
    DfsCString fileId_;
    
    // Url
    DfsCString fileUrl_;
    
    // Mount point
    DfsCString mountPoint_;
    
    // relative pathname (it must be added to the mount point)
    DfsCString fileName_;
    
    // label that identifies the magnetic disk or the CD-ROM
    DfsCString diskName_;
    
    // size of the compressed file
    int compSize_;
    
    // size of the uncompressed file
    int uncompSize_;
    
    // media type 
    enum MediaType mediaType_;
    
    // host address
    DfsCString hostAddress_;
    
public:

/**@name Constructors
 */
//@{
/**
****************************************************************************
* Default constructor.
*/
    
    ArcAstoFile();
    
/**
****************************************************************************
* Constructor.
*/
    ArcAstoFile(enum MediaType mediaType);
    
//@}

    // Copy Constructor required by DfsTValSortedVector<T>
    ArcAstoFile(const ArcAstoFile & t) ;

    // Operator's overloading required by DfsTValSortedVector<T>
    ArcAstoFile& operator= (const ArcAstoFile& t);
    
    DfsBoolean operator== (const ArcAstoFile& t) const;
   
    DfsBoolean operator< (const ArcAstoFile& t) const;
    
/**@name Members access (Read mode)
 */
//@{

    /// Get the value stored in the column file_id
    // Get the value of private member fileId_ 
    inline const DfsCString getDpId() {return fileId_;}
    
    /// Get the url extracted from the columns mount_point+file_name
    // Get the value of private member fileUrl_ 
    inline const DfsCString getFileUrl() {return fileUrl_;}

    /// Get the mount point stored in the column mount_point
    // Get the value of private member mountPoint_ 
    inline const DfsCString getMountPoint() {return mountPoint_;}

    /// Get the relative pathname stored in the column file_name
    // Get the value of private member fileName_ 
    inline const DfsCString getFileName() {return fileName_;}

    /// Get the magnetic or CD name stored in the column diskname
    // Get the value of private member diskName_ 
    inline const DfsCString getDiskName() {return diskName_;}

    /// Get the compressed file size stored in the column fsize
    // Get the value of private member compSize_ 
    inline const int getCompSize() {return compSize_;}

    /// Get the uncompressed file size stored in the column uncomp_fsize
    // Get the value of private member uncompSize_ 
    inline const int getUncompSize() {return uncompSize_;}

    /// Get the type of media where the file is stored (MD or CD)
    // Get the value of private member mediaType_ 
    inline const enum MediaType getMediaType() {return mediaType_;}

    /// Get the host address where the file is stored (column server_id)
    // Get the value of private member hostAddress_ 
    inline const DfsCString getHostAddress() {return hostAddress_;}

//@}

/**@name Members access (Write mode)
 */
//@{

    /// Set the value of private member fileUrl_ 
    inline void fileUrl(DfsCString url) {fileUrl_ = url;}
        
    /// @brief Set private member values based on the current row of the 
    /// given SybaseResultSet. 
    inline void storeSQLQueryResult(SybaseResultSet& results); 

//@}

}; // end of definition of class ArcProduct

/*
 * Default constructor
 */
inline ArcAstoFile::ArcAstoFile() 
    : fileId_(""),
      fileUrl_(""),
      mountPoint_(""),
      fileName_(""),
      diskName_(""),
      compSize_(0),
      uncompSize_(0),
      mediaType_(MD),
      hostAddress_("")
{;}

/*
 * Constructor
 */
inline ArcAstoFile::ArcAstoFile(enum MediaType mediaType) 
    : fileId_(""),
      fileUrl_(""),
      mountPoint_(""),
      fileName_(""),
      diskName_(""),
      compSize_(0),
      uncompSize_(0),
      mediaType_(mediaType),
      hostAddress_("")
{;}

/*
 * Copy Constructor
 */
inline ArcAstoFile::ArcAstoFile(const ArcAstoFile& t) 
    : fileId_(t.fileId_),
      fileUrl_(t.fileUrl_),
      mountPoint_(t.mountPoint_),
      fileName_(t.fileName_),
      diskName_(t.diskName_),
      compSize_(t.compSize_),
      uncompSize_(t.uncompSize_),
      mediaType_(t.mediaType_),
      hostAddress_(t.hostAddress_)
{;}

inline ArcAstoFile& ArcAstoFile::operator= (const ArcAstoFile& t)
{
    fileId_      = t.fileId_;
    fileUrl_     = t.fileUrl_;
    mountPoint_  = t.mountPoint_;
    fileName_    = t.fileName_;
    diskName_    = t.diskName_;
    compSize_    = t.compSize_;
    uncompSize_  = t.uncompSize_;
    mediaType_   = t.mediaType_;
    hostAddress_ = t.hostAddress_;
          
    return *this;
}

inline DfsBoolean ArcAstoFile::operator== (const ArcAstoFile& t) const
{
    return fileId_ == t.fileId_;
}

/**  this definition is required by the template class DfsTValOrderedVector<T>
     in order to be able to sort the vector
**/
inline DfsBoolean ArcAstoFile::operator< (const ArcAstoFile& t) const
{
    return mediaType_  < t.mediaType_ ||
	(mediaType_ == t.mediaType_ && diskName_ < t.diskName_);
}

// Set private member values based on the current row of the given 
// SybaseResultSet. 
inline void ArcAstoFile::storeSQLQueryResult(SybaseResultSet& results)
{
    // Initialise all affected members before reading new data. 
    fileId_ = fileName_ = mountPoint_ = diskName_ = "";
    compSize_ = uncompSize_ = 0; 

    // Column count and current column name. 
    int numColumns = results.getColumnCount(); 
    DfsCString columnName; 

    // Read new data. 
    for(int i = 1; i <= numColumns; ++i)
    {
        columnName = results.getColumnName(i); 
        if(columnName == "file_id")
        {
            fileId_ = results.getStringValue(i);
        }
        else if(columnName == "fname")
        {
            fileName_ = results.getStringValue(i);
        }
        else if(columnName == "fsize")
        {
            compSize_ = results.getLongValue(i);
        }
        else if(columnName == "uncomp_fsize")
        {
            uncompSize_ = results.getLongValue(i);
        }
        else if(columnName == "mount_point")
        {
            mountPoint_ = results.getStringValue(i);
        }
        else if((mediaType_ == CD) && (columnName == "volume"))
        {
            diskName_ = results.getStringValue(i);
        }
        else if((mediaType_ != CD) && (columnName == "diskname"))
        {
            diskName_ = results.getStringValue(i);
        }
    }
}

/**
****************************************************************************
* It is used to create an instance of DfsTPtrSortedVector
* that is a single linked list of ArcAstoFile instances
* An instance of ArcSortFileList can be accessed like an array, 
* by using the operator[]. 
* The call to the entries() function will return the number
* of ArcProduct instances actually in the list.
* NOTE: Since instances of ArcAstoFile in the DfsTPtrSortedVector instance
* were created on the heap, they must be destroyed manually.
* The clear() member function of DfsTPtrSortedVector
* takes care of that task.
@memo
It is used to create an instance of DfsTPtrSortedVector
that is a list of ArcAstoFile instances
@see
DfsTPtrSortedVector template class
*/

typedef DfsTPtrSortedVector<ArcAstoFile> ArcSortFileList;

#endif /* _ArcAstoFile_h_ */
