// -*-c++-*-
#ifndef _ArcMoonval_h_
#define _ArcMoonval_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcMoonVal.h,v 1.1 2007/10/17 08:12:10 cmoins Exp $
 *
 * ArcMoonVal.h - class definitions for class ArcMoonVal, a class 
 * for computing moon related parameters 
 *
 * who                  when       what
 * -------------------  --------   ----------------------------------------
 * Elisabetta Angeloni  26 Jan 98  Created
 */
 
#include <ostream> 

#include <ArcTime.h>
#include <ArcSkyCalc.h>

const double LAT_NTT = -29.2584; // deg
const double LON_NTT =  70.7345; // deg

const double LAT_UT1 = -24.6259; // deg
const double LON_UT1 =  70.4032; // deg

const double LAT_UT2 = -24.6254; // deg
const double LON_UT2 =  70.4030; // deg

const double LAT_UT3 = -24.6251; // deg
const double LON_UT3 =  70.4027; // deg

const double LAT_UT4 = -24.6253; // deg
const double LON_UT4 =  70.4022; // deg

/// enum type defining the telescope 
enum Telescope {
    /// NTT
    NTT,
    /// UT1
    UT1,
    /// UT2
    UT2,
    /// UT3
    UT3,
    /// UT4
    UT4
};
 

/// Simple class containing the results of moon computations
class ArcMoonVal{

private:
   
    double  phase_;
    double  altitude_;
    double  azimuth_;
    double  distanceToTarget_;
    double  fli_;
 
public:

// Default constructor: do nothing
    ArcMoonVal();
  
/**@name Constructors
 */
//@{

/**
****************************************************************************
* Main Constructor: it computes the following moon parameters: 
- moon altitude          (returned by method ArcMoonVal::getMoonAlt())
- moon azimuth           (returned by method ArcMoonVal::getMoonAz())
- distance to the target (returned by method ArcMoonVal::getMoonDistance())
- FLI                    (returned by method ArcMoonVal::getFLI())
- moon phase             (returned by method ArcMoonVal::getMoonPhase())
This service does not require any connections to databases. 

@memo
Compute the following moon parameters: altitude, azimuth, distance to the target,
FLI and phase. 
@return
0 if OK, -1 if an error occurred
@param
objRa     - (INPUT) RA of the target object (degrees)
objDec    - (INPUT) DEC of the target object (degrees)
mjdStart  - (INPUT) start of period (MJD format)
telescope - (INPUT) observing telescope (NTT, UT1, UT2, UT3, UT4)
@see
enum Telescope for the list of telescopes
*/
    ArcMoonVal(const double& objRa, const double& objDec, 
	       const MJD_DATE& mjdStart,
	       const Telescope telescope=UT1);

//@}

    /// Get the value of Moon Altitude
    inline double getMoonAlt() {return altitude_;}
    
    /// Get the value of Moon Azimuth 
    inline double getMoonAz() {return azimuth_;}

    /// Get the value of Moon Distance to the Target
    inline double getMoonDistance() {return distanceToTarget_;}
    
    /// Get the value of Moon FLI
    inline double getFLI() {return fli_;}

    /// Get the value of Moon Phase
    inline double getMoonPhase() {return phase_;}

    // Set the value of Moon Phase
    inline void moonPhase(double d) {phase_ = d;}

    // Set the value of Moon Altitude 
    inline void moonAlt(double d) {altitude_ = d;}

    // Set the value of Moon Azimuth
    inline void moonAz(double d) {azimuth_ = d;}

    // Set the value of Moon Distance to the Target
    inline void moonDistance(double d) {distanceToTarget_ = d;}
        
    // Set the value of Moon FLI
    inline void moonFLI(double d) {fli_ = d;}
    
    std::ostream& print(std::ostream& o) const;

    };

inline std::ostream& ArcMoonVal::print(std::ostream& aStream) const
{
	using std::endl; 
	
    aStream << "Results Moon values "                       << endl
            << "Phase: "               << phase_            << endl
            << "Altitude: "            << altitude_         << endl
            << "Azimuth: "             << azimuth_          << endl
            << "FLI: "                 << fli_              << endl
            << "Distance to Target: "  << distanceToTarget_ << endl << endl;
    return aStream;
}

//* Service private to the ORO library used for computing moon values
//* The interface of this service are subject to changes
void computeMoonParam(const double& starRa, const double& starDec, 
                      const MJD_DATE& mjdStart, 
                      const double& longitude, const double& latitude,
                      double* lstValPtr, double* moonfliPtr,
                      double* moondistPtr, double* moonaltPtr, 
                      double* moonazPtr, double* moonphasePtr,
                      double* alphasunPtr, double* objdistPtr);

//* Service private to the ORO library used for getting the telescope coordinates
int getTelescopeCoord(double& latitude, double& longitude, 
                      const Telescope telescope);

#endif /* _ArcMoonval_h_ */
