// -*-c++-*-
#ifndef _ArcObrep_h_
#define _ArcObrep_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcObRep.h,v 1.3 2005/09/16 18:24:53 nferguso Exp $
 *
 * ArcObRep.h - class definitions for class ArcObRep, a class 
 * representing an application that read/write from/to the
 * OB repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        22 Nov 2004  Converted to DfsStdLib/SybConn
 * Neil Ferguson        16 Sep 2005  Removed obsolete methods. 
 */

#include <ArcDbGen.h>
#include <ArcObsBlock.h>
#include <ArcTplParam.h>
#include <ArcTemplate.h>
#include <ArcObLog.h>

const char OB_STATUS_EXECUTED   = 'X';
const char OB_STATUS_REDUCED    = 'R';
const char OB_STATUS_VERIFIED   = 'V';
const char OB_STATUS_CERTIFIED  = 'C';
const char OB_STATUS_ALARMED    = 'A';

/**
 * This class and its methods are meant to be used for reading from
 * the OB repository database.
 * An instance consists of references to the database and the tables
 * where the observation blocks data resides.
 * This class includes a set of member functions for retrieving the 
 * observation blocks data.
 */
class ArcObRep : public ArcDbGen {
    
private:

    //
    DfsCString obs_blocksTab_;
    //
    DfsCString constr_setsTab_;
    //
    DfsCString logTab_;
    
    //
    int checkOBStatus(const DfsCString& currOBStatus, 
                      const DfsCString& newOBStatus,
                      DfsCString& logString);
    // 
    int decodeOBStatus(const DfsCString& currOBStatus, 
                       DfsCString& logString);
    
public:
    
/**@name Constructors
 */
//@{
    // Default constructor: it supplies a connection to the OB repository.
    // The default values are read from a configuration file
    ArcObRep(DfsCString dbAlias="OBREP");
    
    /// Constructor that requires username, password, server and database name (a default is supplied).
    // In case an empty server name is given, the value of $DSQUERY variable is taken
    ArcObRep(DfsCString dbName, DfsCString dbPasswd, 
             DfsCString dbServer="", DfsCString dbName="obrep");
    
//@}
    
/**@name  Methods for data retrieval
 * These methods  retrieves data from the tables obs_blocks and
 * constraint_sets of OB Rep database using an outer join
 * All of them execute the following SELECT statement:
 SELECT
 obs_blocks.id,
 obs_blocks.run_id,
 obs_blocks.ts_id,
 obs_blocks.item_name,
 obs_blocks.obsComments,
 obs_blocks.userComments,
 obs_blocks.obsMode,
 obs_blocks.status,
 obs_blocks.calibrationReq,
 obs_blocks.reductionRule,
 obs_blocks.type,
 obs_blocks.cs_id,
 obs_blocks.last_mod_date,
 constr_sets.seeing,
 constr_sets.sky_transparency,
 constr_sets.airmass,
 constr_sets.fli,
 constr_sets.moon_distance
 FROM
 obs_blocks, constr_sets
 WHERE
 obs_blocks.cs_id = constr_sets.id (*)
 * In addition, each function executes a specific where clause
 */
//@{
    
/**
****************************************************************************
* Retrieve from the database an observation block given its id

@memo
Retrieve from the database an observation block given its id
@return
0 if OK, 1 if an error occurred
@param
obsBlock (OUTPUT) - retrieved block,
obId - (INPUT) OB id, 
@see
ArcObsBlock class
*/
    
    int getOB(ArcObsBlock& obsBlock, const int& obId);

/**
****************************************************************************
* Generic method for retrieving from the database a list of OBs 
* given a generic where clause. 
* If no where clause is specified, all the OBs from the table
* obs_blocks are retrieved.
* In order to build the where clause, the methods that return the column's names
* should be used.

@memo
Generic method for retrieving from the database a list of OBs 
given a generic where clause. 
@return
0 if OK, 1 if an error occurred
The number of retrieved items is given by OBList.entries()
NOTE: it user's responsability to free the list using OBList.clear()
@param
OBList (OUTPUT) list containing the retrieved OBs,
whereClause - (INPUT) generic where clause
@see
ArcOBList user-defined type, DfsCString class
*/
    int getOB(ArcOBList& OBList, 
              const DfsCString& whereClause = DfsCString("0 = 0")); 

    int getParamList(ArcTplParamList& paramList, const int& OBId, int tplNo);
    
//@}
    
// * These methods get the column's name of the tables obs_blocks and
// * constraint_sets of OB Rep database.
// * These methods must be used for building a generic where clause
    
    // Observation Block ID (column "obs_blocks.id")
    inline const DfsCString obsIdColumn() { return(obs_blocksTab_ + ".ob_id"); }
    
    // programme ID (column "obs_blocks.run_id")
    inline const DfsCString progIdColumn() { return(obs_blocksTab_ + ".run_id"); }
    
    // Template Signature ID (column "obs_blocks.ts_id")
    //inline const DfsCString templateSigColumn() { return(obs_blocksTab_ + ".ts_id"); }
    
    // User-assigned name of the OB (column "obs_blocks.item_name")
    inline const DfsCString obsNameColumn() { return(obs_blocksTab_ + ".item_name"); }
    
    // Information related to OB execution in service mode (column "obs_blocks.obsComments")
    //inline const DfsCString obsCommentsColumn() { return(obs_blocksTab_ + ".obsComments"); }
    
    // OB-related information (column "obs_blocks.userComments")
    inline const DfsCString userCommentsColumn() { return(obs_blocksTab_ + ".userComments"); }
    
    // Observing mode (column "obs_blocks.obsMode")
    //inline const DfsCString obsModeColumn() { return(obs_blocksTab_ + ".obsMode"); }
    
    // Status of the Observation Block (column "obs_blocks.status")
    inline const DfsCString obsStatusColumn() { return(obs_blocksTab_ + ".status"); }
    
    // (column "obs_blocks.calibrationReq")
    inline const DfsCString calibrationReqColumn() { return(obs_blocksTab_ + ".calibrationReq"); }
    
    // (column "obs_blocks.reductionRule")
    //inline const DfsCString reductionRuleColumn() { return(obs_blocksTab_ + ".reductionRule"); }
    
    // Observation Type  (column "obs_blocks.type")
    inline const DfsCString obsTypeColumn() { return(obs_blocksTab_ + ".type"); }
    
    // Constraint Set ID (column "obs_blocks.cs_id")
    inline const DfsCString OBconstraintSetIdColumn() { return(obs_blocksTab_ + ".cs_id"); }
    
    // Last modification date (column "obs_blocks.last_mod_date")
    inline const DfsCString modifDateColumn() { return(obs_blocksTab_ + ".last_mod_date"); }
    
    // Last modification date (column "constraint_sets.id")
    inline const DfsCString constraintSetIdColumn() { return(constr_setsTab_ + ".cs_id"); }
    
    // Seeing or Image quality (column "constraint_sets.seeing")
    inline const DfsCString seeingColumn() { return(constr_setsTab_ + ".seeing"); }
    
    // Sky Transparency (column "constraint_sets.sky_transparency")
    inline const DfsCString skyTransparencyColumn() { return(constr_setsTab_ + ".sky_transparency"); }
    
    // Seeing or Image quality (column "constraint_sets.airmass")
    inline const DfsCString airMassColumn() { return(constr_setsTab_ + ".airmass"); }
    
    // Max Fractional Lunar Illumination (column "constraint_sets.fli")
    inline const DfsCString fliColumn() { return(constr_setsTab_ + ".fli"); }
    
    // Min allowed angular distance (column "constraint_sets.moon_distance")
    inline const DfsCString moonDistanceColumn() { return(constr_setsTab_ + ".moon_distance"); }
    
}; // end of definition of class ArcObRep

#endif /* _ArcObrep_h_ */
