// -*-c++-*-
#ifndef _ArcProduct_h_
#define _ArcProduct_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcProduct.h,v 1.4 2005/10/17 13:22:28 nferguso Exp $
 *
 * ArcProduct.h - class definitions for class ArcProduct, a class 
 * representing an application that read from the
 * tables data_products and dp_others of obsevations database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  25 May 1998  Created
 * Neil Ferguson        22 Nov 2004  Converted to DfsStdLib/SybConn
 */

#include "DfsCString.h"
#include "DfsTValSlist.h"
#include <ArcTime.h>


/**
 * This class is used by the ArcProdRep class for storing the data retrieved
 * from the database table data_products ; 
 * this class contains only methods 
 * for accessing the class's members, none of them access the database fields.
 */
class ArcProduct {

private:

    // Data Product ID (column "dp_id")
    DfsCString dpId_;

    // Exposure start date (column "mjd_obs")
    MJD_DATE mjdobs_;

    // RA  (milliarcsec) (column "ra")
    int ra_;

    // DEC (milliarcsec) (column "dec")
    int dec_;

    // Exptime (millisec) (column "exptime")
    double exptime_;

    // Instrument ID (column "ins_id")
    DfsCString insId_;

    // Programme ID (column "object")
    DfsCString Object_;

    // Programme ID (column "prog_id")
    DfsCString progId_;

    // Product type (column "dp_type")
    DfsCString dpType_;

    // Product category (column "dp_cat")
    DfsCString dpCatg_;

    // Product technique (column "dp_tech")
    DfsCString dpTech_;

    DfsBoolean operator== (const ArcProduct& t) const;
    
public:

/**@name Constructors
 */
//@{
/**
****************************************************************************
* Default constructor.
*/
 
    ArcProduct();

//@}

    // Used by the ArcProdRep class
    DfsCString selectItemList(const DfsCString& data_products);

    // Set private member values based on the current row of the 
    // given SybaseResultSet. 
    void storeSQLQueryResult(SybaseResultSet& results); 

/**@name Members access (Read mode)
 */
//@{

    /// Get the value stored in the column data_products.dp_id
    // Get the value of private member dpId_ 
    inline const DfsCString getDpId() {return dpId_;}

    /// Get the value stored in the column data_products.mjd_obs
    // Get the value of private member mjdobs_
    inline const MJD_DATE getMjdObs() {return mjdobs_;}

    /// Get the value stored in the column data_products.ra (milliarcsec)
    inline const int getRa() {return ra_;}

    /// Get the value stored in the column data_products.dec (milliarcsec)
    inline const int getDec() {return dec_;}

    /// Get the value stored in the column data_products.exptime
    inline const double getExptime() {return exptime_;}

    /// Get the value stored in the column data_products.ins_id
    // Get the value of private member insId_ 
    inline const DfsCString getInsId() {return insId_;}

    /// Get the value stored in the column data_products.object
    inline const DfsCString getObject() {return Object_;}

    /// Get the value stored in the column data_products.prog_id
    inline const DfsCString getProgId() {return progId_;}

    /// Get the value stored in the column data_products.dp_type
    inline const DfsCString getDPType() {return dpType_;}

    /// Get the value stored in the column data_products.dp_cat
    inline const DfsCString getDPCatg() {return dpCatg_;}

    /// Get the value stored in the column data_products.dp_tech
    inline const DfsCString getDPTech() {return dpTech_;}

//@}

}; // end of definition of class ArcProduct


/*
 * Constructor
 */

inline ArcProduct::ArcProduct() 
    : dpId_(""), 
      mjdobs_(0.0),
      ra_(0),
      dec_(0),
      exptime_(0.0),
      insId_(""),
      Object_(""), 
      progId_(""), 
      dpType_(""), 
      dpCatg_(""), 
      dpTech_("") 
{;}

inline DfsBoolean ArcProduct::operator== (const ArcProduct& t) const
{
    return dpId_ == t.dpId_;
}

// Reader matching the select list-item below
//
// Set private member values based on the current row of the given 
// SybaseResultSet. 
inline void ArcProduct::storeSQLQueryResult(SybaseResultSet& results)
{
    // Read new data. 
    int i = 0; 
    
    dpId_ = results.getStringValue(++i);
    mjdobs_ = results.getDoubleValue(++i);
    ra_ = results.getLongValue(++i);
    dec_ = results.getLongValue(++i);
    exptime_ = results.getDoubleValue(++i);
    insId_ = results.getStringValue(++i);
    Object_ = results.getStringValue(++i);
    progId_ = results.getStringValue(++i);
    dpType_ = results.getStringValue(++i);
    dpCatg_ = results.getStringValue(++i);
    dpTech_ = results.getStringValue(++i);
}

/*
 * Build the select item list from data_products table
 */

inline DfsCString ArcProduct::selectItemList(const DfsCString& data_products)
{
    // Build the following selection list:
    // 
    //     dp_id, mjd_obs, ra, dec, exptime, ins_id, object, prog_id,
    //     dp_type, dp_cat, dp_tech 
    // 
    // qualified with the given data_products table name. 
    
    DfsCString items(
        data_products + ".dp_id, " 
        + data_products + ".mjd_obs, "
        + data_products + ".ra, "
        + data_products + ".dec, "
        + data_products + ".exptime, "
        + data_products + ".ins_id, "
        + data_products + ".object, "
        + data_products + ".prog_id, "
        + data_products + ".dp_type, "
        + data_products + ".dp_cat, "
        + data_products + ".dp_tech");
        
    return items;
}

/**
****************************************************************************
* It is used to create an instance of DfsTValSlist 
* that is a single linked list of ArcProduct instances
* An instance of ArcProductList can be accessed like an array, 
* by using the operator[]. 
* The call to the entries() function will return the number
* of ArcProduct instances actually in the list.
* NOTE: Since instances of ArcProduct in the DfsTValSlist instance
* were created on the heap, they must be destroyed manually.
* The clear() member function of DfsTValSlist
* takes care of that task.
@memo
It is used to create an instance of DfsTValSlist 
that is a list of ArcProduct instances
@see
DfsTValSlist template class
*/

typedef DfsTValSlist<ArcProduct> ArcProdList;

#endif /* _ArcProduct_h_ */
