// -*-c++-*-
#ifndef _ArcQCParam_h_
#define _ArcQCParam_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcQCParam.h,v 1.3 2005/08/18 17:03:27 nferguso Exp $
 *
 * ArcQCParam.h - class definitions for class ArcQCParam, a class 
 * representing an application that read/write from/to the
 * table qcr_param 
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        01 Nov 2004  Converted to DfsStdLib/SybConn
 */

#if defined (__linux__)
#include <sys/time.h>
#include <unistd.h>
#endif

#include "DfsCString.h"
#include "DfsNumConvert.h"
#include "DfsTValSlist.h"
#include "TimeStamp.h"

// Conventional value used for initialize the integre value with NULLs
const int NULL_INT_QC_PARAM = -32768;

/// enum type that support different types of QC parameter 
enum QCParamType {
    /// SEEING
    SEEING,
    /// AIR_MASS
    AIR_MASS,
    /// Maximum Fractional Lunar Illumination (FLI) or Moon Phase
    FLI, 
    /// Moon Distance
    MOON_DISTANCE,
    /// Sky Transparency
    SKY_TRANSPARENCY
} ;	

/**
 * This class is used by the ArcQCReport class for storing the data retrieved or
 * to be recorded in the database table qcr_param ; 
 * this class contains only methods 
 * for accessing the class's members, none of them access the database fields.
 */
class ArcQCParam {

public:

    // primary key
    DfsCString dpId_;
 
    // Last modification date (column "updated")
    TimeStamp updateDate_;

    // SEEING parameters
    // measured value
    int seeing_;

    // target value
    int seeingTarget_;

    // min value
    int seeingMin_;

    // max value
    int seeingMax_;

    // status
    DfsCString seeingStatus_;

    // comment
    DfsCString seeingComment_;
    
    // SKY TRANSPARENCY parameters
    // measured value
    DfsCString skyTransparency_;

    // target value
    DfsCString skyTransparencyTarget_;

    // min value
    DfsCString skyTransparencyMin_;

    // max value
    DfsCString skyTransparencyMax_;

    // status
    DfsCString skyTransparencyStatus_;

    // comment
    DfsCString skyTransparencyComment_;
    
    // AIR MASS parameters
    // measured value
    int airMass_;

    // target value
    int airMassTarget_;

    // min value
    int airMassMin_;

    // max value
    int airMassMax_;

    // status
    DfsCString airMassStatus_;

    // comment
    DfsCString airMassComment_;  
      
    // FLI parameters
    // measured value
    int fli_;

    // target value
    int fliTarget_;

    // min value
    int fliMin_;

    // max value
    int fliMax_;

    // status
    DfsCString fliStatus_;

    // comment
    DfsCString fliComment_;
      
    // MOON DISTANCE parameters
    // measured value
    int moonDistance_;

    // target value
    int moonDistanceTarget_;

    // min value
    int moonDistanceMin_;

    // max value
    int moonDistanceMax_;

    // status
    DfsCString moonDistanceStatus_;

    // comment
    DfsCString moonDistanceComment_;
      
    DfsBoolean operator== (const ArcQCParam& t) const;

public:

/**@name Constructors
 */
//@{
/**
****************************************************************************
* Default constructor.
The parameters have the default value NULL_INT_QC_PARAM

*/
    ArcQCParam();

//@}

    /// Detect whether at least there is one not null QC parameter
    DfsBoolean isNull() const; 

    /// @brief Returns a string in the format "(column_names) VALUES 
    /// (column_values)" holding this ArcQCParam object's contents, 
    /// ready for use as part of an SQL INSERT command. 
    DfsCString getSQLInsertData() const; 
/**
****************************************************************************
* This method prints out the content of the ArcQCParam class

@memo
Print out the content of the ArcQCParam class
@param
output stream (default is standard output)
*/

    std::ostream&  print(std::ostream& o) const;
    

}; // end of definition of class ArcQCParam

inline ArcQCParam::ArcQCParam() 
    : dpId_(""), 
      seeing_(NULL_INT_QC_PARAM),
      seeingTarget_(NULL_INT_QC_PARAM), 
      seeingMin_(NULL_INT_QC_PARAM),
      seeingMax_(NULL_INT_QC_PARAM),
      seeingStatus_(""),
      seeingComment_(""),
      skyTransparency_(""),
      skyTransparencyTarget_(""), 
      skyTransparencyMin_(""),
      skyTransparencyMax_(""),
      skyTransparencyStatus_(""),
      skyTransparencyComment_(""),
      airMass_(NULL_INT_QC_PARAM),
      airMassTarget_(NULL_INT_QC_PARAM), 
      airMassMin_(NULL_INT_QC_PARAM),
      airMassMax_(NULL_INT_QC_PARAM),
      airMassStatus_(""),
      airMassComment_(""),
      fli_(NULL_INT_QC_PARAM),
      fliTarget_(NULL_INT_QC_PARAM), 
      fliMin_(NULL_INT_QC_PARAM),
      fliMax_(NULL_INT_QC_PARAM),
      fliStatus_(""),
      fliComment_(""),
      moonDistance_(NULL_INT_QC_PARAM),
      moonDistanceTarget_(NULL_INT_QC_PARAM), 
      moonDistanceMin_(NULL_INT_QC_PARAM),
      moonDistanceMax_(NULL_INT_QC_PARAM),
      moonDistanceStatus_(""),
      moonDistanceComment_("")
{
    // This block does not need to do anything - TimeStamps initialise 
    // to the current UT time automatically. 
}

inline DfsBoolean ArcQCParam::operator== (const ArcQCParam& t) const
{
    return dpId_ == t.dpId_;
}

// Returns a string in the format "(column_names) VALUES (column_values)" 
// holding this ArcQCParam object's contents, ready for use as part of an 
// SQL INSERT command. 
inline DfsCString ArcQCParam::getSQLInsertData() const 
{
    // DP ID (string type)
    DfsCString columns("(qcr_dp_id");
    DfsCString values(") VALUES (" + SybUtils::quoteForSQL(dpId_.asSTLString())); 

    // LAST UPDATE DATE (TimeStamp type)
    columns += ", updated";
    values  += ", \'" + updateDate_.asString("%x ") 
               + updateDate_.getTime() + "\'";

    // SEEING (integer type)
    if(seeing_ != NULL_INT_QC_PARAM)
    {
        columns += ", seeing";
        values  += ", " + DfsNumConvert::asString(seeing_);
    }

    if(seeingTarget_ != NULL_INT_QC_PARAM)
    {
        columns += ", seeing_tgt";
        values  += ", " + DfsNumConvert::asString(seeingTarget_);
    }

    if(seeingMin_ != NULL_INT_QC_PARAM)
    {
        columns += ", seeing_min";
        values  += ", " + DfsNumConvert::asString(seeingMin_);
    }

    if(seeingMax_ != NULL_INT_QC_PARAM)
    {
        columns += ", seeing_max";
        values  += ", " + DfsNumConvert::asString(seeingMax_);
    }

    // SEEING (string type)
    columns += ", seeing_st";
    values  += ", " + SybUtils::quoteForSQL(seeingStatus_.asSTLString());

    columns += ", seeing_com";
    values  += ", " + SybUtils::quoteForSQL(seeingComment_.asSTLString());

    // SKY TRANSPARENCY (string type)
    columns += ", sky_transparency";
    values  += ", " + SybUtils::quoteForSQL(skyTransparency_.asSTLString());

    columns += ", sky_transparency_tgt";
    values  += ", " + SybUtils::quoteForSQL(skyTransparencyTarget_.asSTLString());

    columns += ", sky_transparency_min";
    values  += ", " + SybUtils::quoteForSQL(skyTransparencyMin_.asSTLString());

    columns += ", sky_transparency_max";
    values  += ", " + SybUtils::quoteForSQL(skyTransparencyMax_.asSTLString());

    columns += ", sky_transparency_st";
    values  += ", " + SybUtils::quoteForSQL(skyTransparencyStatus_.asSTLString());

    columns += ", sky_transparency_com";
    values  += ", " + SybUtils::quoteForSQL(skyTransparencyComment_.asSTLString());

    // AIR MASS (integer type)      
    if(airMass_ != NULL_INT_QC_PARAM)
    {
        columns += ", air_mass";
        values  += ", " + DfsNumConvert::asString(airMass_);
    }

    if(airMassTarget_ != NULL_INT_QC_PARAM)
    {
        columns += ", air_mass_tgt";
        values  += ", " + DfsNumConvert::asString(airMassTarget_);
    }

    if(airMassMin_ != NULL_INT_QC_PARAM)
    {
        columns += ", air_mass_min";
        values  += ", " + DfsNumConvert::asString(airMassMin_);
    }

    if(airMassMax_ != NULL_INT_QC_PARAM)
    {
        columns += ", air_mass_max";
        values  += ", " + DfsNumConvert::asString(airMassMax_);
    }

    // AIR MASS (string type)      
    columns += ", air_mass_st";
    values  += ", " + SybUtils::quoteForSQL(airMassStatus_.asSTLString());

    columns += ", air_mass_com";
    values  += ", " + SybUtils::quoteForSQL(airMassComment_.asSTLString());

    // FRACTIONAL LUNAR ILLUMINATION (integer type)      
    if(fli_ != NULL_INT_QC_PARAM)
    {
        columns += ", fli";
        values  += ", " + DfsNumConvert::asString(fli_);
    }

    if(fliTarget_ != NULL_INT_QC_PARAM)
    {
        columns += ", fli_tgt";
        values  += ", " + DfsNumConvert::asString(fliTarget_);
    }

    if(fliMin_ != NULL_INT_QC_PARAM)
    {
        columns += ", fli_min";
        values  += ", " + DfsNumConvert::asString(fliMin_);
    }

    if(fliMax_ != NULL_INT_QC_PARAM)
    {
        columns += ", fli_max";
        values  += ", " + DfsNumConvert::asString(fliMax_);
    }

    // FRACTIONAL LUNAR ILLUMINATION (string type)      
    columns += ", fli_st";
    values  += ", " + SybUtils::quoteForSQL(fliStatus_.asSTLString());

    columns += ", fli_com";
    values  += ", " + SybUtils::quoteForSQL(fliComment_.asSTLString());

    // MOON DISTANCE (integer type)      
    if(moonDistance_ != NULL_INT_QC_PARAM)
    {
        columns += ", moon_distance";
        values  += ", " + DfsNumConvert::asString(moonDistance_);
    }

    if(moonDistanceTarget_ != NULL_INT_QC_PARAM)
    {
        columns += ", moon_distance_tgt";
        values  += ", " + DfsNumConvert::asString(moonDistanceTarget_);
    }

    if(moonDistanceMin_ != NULL_INT_QC_PARAM)
    {
        columns += ", moon_distance_min";
        values  += ", " + DfsNumConvert::asString(moonDistanceMin_);
    }

    if(moonDistanceMax_ != NULL_INT_QC_PARAM)
    {
        columns += ", moon_distance_max";
        values  += ", " + DfsNumConvert::asString(moonDistanceMax_);
    }

    // MOON DISTANCE (string type)      
    columns += ", moon_distance_st";
    values  += ", " + SybUtils::quoteForSQL(moonDistanceStatus_.asSTLString());

    columns += ", moon_distance_com";
    values  += ", " + SybUtils::quoteForSQL(moonDistanceComment_.asSTLString());

    return(columns + values + ")"); 
}

inline DfsBoolean ArcQCParam::isNull() const 
{
    return ( seeing_                          == NULL_INT_QC_PARAM
	     && seeingTarget_                    == NULL_INT_QC_PARAM 
	     && seeingMin_                       == NULL_INT_QC_PARAM
	     && seeingMax_                       == NULL_INT_QC_PARAM
	     && seeingStatus_.length()           == 0
	     && seeingComment_.length()          == 0
	     && skyTransparency_.length()        == 0
	     && skyTransparencyTarget_.length()  == 0 
	     && skyTransparencyMin_.length()     == 0
	     && skyTransparencyMax_.length()     == 0
	     && skyTransparencyStatus_.length()  == 0
	     && skyTransparencyComment_.length() == 0
	     && airMass_                         == NULL_INT_QC_PARAM
	     && airMassTarget_                   == NULL_INT_QC_PARAM
	     && airMassMin_                      == NULL_INT_QC_PARAM
	     && airMassMax_                      == NULL_INT_QC_PARAM
	     && airMassStatus_.length()          == 0
	     && airMassComment_.length()         == 0
	     && fli_                             == NULL_INT_QC_PARAM
	     && fliTarget_                       == NULL_INT_QC_PARAM 
	     && fliMin_                          == NULL_INT_QC_PARAM
	     && fliMax_                          == NULL_INT_QC_PARAM
	     && fliStatus_.length()              == 0
	     && fliComment_.length()             == 0
	     && moonDistance_                    == NULL_INT_QC_PARAM
	     && moonDistanceTarget_              == NULL_INT_QC_PARAM 
	     && moonDistanceMin_                 == NULL_INT_QC_PARAM
	     && moonDistanceMax_                 == NULL_INT_QC_PARAM
	     && moonDistanceStatus_.length()     == 0
	     && moonDistanceComment_.length()    == 0 );
       
}

inline std::ostream& ArcQCParam::print(std::ostream& aStream) const
{
    using std::endl;

    aStream << "QC Parameters of frame "  << dpId_  << endl
            << "Last Modification date: " 
                   << updateDate_.asString("%x %X") << endl
            << "SEEING: "         << seeing_        << endl
            << "Seeing target: "  << seeingTarget_  << endl
            << "Seeing min: "     << seeingMin_     << endl
            << "Seeing max: "     << seeingMax_     << endl
            << "Seeing status: "  << seeingStatus_  << endl
            << "Seeing comment: " << seeingComment_ << endl 
            << "SKY TRANSPARENCY: "         << skyTransparency_        << endl
            << "Sky transparency target: "  << skyTransparencyTarget_  << endl
            << "Sky transparency min: "     << skyTransparencyMin_     << endl
            << "Sky transparency max: "     << skyTransparencyMax_     << endl
            << "Sky transparency status: "  << skyTransparencyStatus_  << endl
            << "Sky transparency comment: " << skyTransparencyComment_ << endl
            << "AIR MASS: "         << airMass_        << endl
            << "Air mass target: "  << airMassTarget_  << endl
            << "Air mass min: "     << airMassMin_     << endl
            << "Air mass max: "     << airMassMax_     << endl
            << "Air mass status: "  << airMassStatus_  << endl
            << "Air mass comment: " << airMassComment_ << endl 
            << "FLI: "         << fli_        << endl
            << "Fli target: "  << fliTarget_  << endl
            << "Fli min: "     << fliMin_     << endl
            << "Fli max: "     << fliMax_     << endl
            << "Fli status: "  << fliStatus_  << endl
            << "Fli comment: " << fliComment_ << endl 
            << "MOON DISTANCE: "         << moonDistance_        << endl
            << "Moon distance target: "  << moonDistanceTarget_  << endl
            << "Moon distance min: "     << moonDistanceMin_     << endl
            << "Moon distance max: "     << moonDistanceMax_     << endl
            << "Moon distance status: "  << moonDistanceStatus_  << endl
            << "Moon distance comment: " << moonDistanceComment_ << endl << endl;

    return aStream;
}

/**
****************************************************************************
* It is used to create an instance of DfsTValSlist
* that is a single linked list of ArcQCParam instances
* An instance of ArcQCParamList can be accessed like an array,
* by using the operator[].
* The call to the entries() function will return the number
* of ArcQCParam instances actually in the list.
* NOTE: Since instances of ArcQCParam in the DfsTValSlist instance
* were created on the heap, they must be destroyed manually.
* The clear() member function of DfsTValSlist
* takes care of that task.
@memo
It is used to create an instance of DfsTValSlist
that is a list of ArcQCParam instances
@see
DfsTValSlist template class
*/

typedef DfsTValSlist<ArcQCParam> ArcQCParamList;

#endif /* _ArcQCParam_h_ */
