// -*-c++-*-
#ifndef _ArcStatistics_h_
#define _ArcStatistics_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcStatistics.h,v 1.1 2004/09/06 13:09:39 cguirao Exp $
 *
 * ArcStatistics.h - class definitions for class ArcStatistics, a class 
 * representing an application that compute mean standard deviation and RMS
 *
 * who                  when       what
 * -------------------  --------   ----------------------------------------
 * Elisabetta Angeloni  26 Jan 98  Created
 */

#include <math.h>

/// Simple class containing the results of statistical computations
class ArcStatistics{

private:
   
    double  mean_;
    double  stddev_;
    double  rms_;
    double  min_;
    double  max_;
  
public:
  
    /// Get the Min value 
    inline double getMin() {return min_;}

    /// Get the Max value
    inline double getMax() {return max_;}

    /// Get the Mean value 
    inline double getMean() {return mean_;}

    /// Get the Standard Deviation value 
    inline double getStdDev() {return stddev_;}
    
    /// Get the RMS value 
    inline double getRMS() {return rms_;}
    
    // Set the Min value 
    inline void min(double d) {min_ = d;}
    
    // Set the Max value 
    inline void max(double d) {max_ = d;}
    
    // Set the Mean value 
    inline void mean(double d) {mean_ = d;}
    
    // Set the Standard Deviation value 
    inline void stdDev(double d) {stddev_ = d;}
    
    // Set the RMS value 
    inline void RMS(double d) {rms_ = d;}
    
    ostream&  print (ostream& o) const;

/**@name Constructors
 */
//@{
    // Default constructor: it init the internal fields to 0 values
    ArcStatistics();
//@}

    void calcMeanMaxMin (int * dataPtr, int dimVector, double scale = 1.0);
    void calcAll (int * dataPtr, int dimVector, double scale = 1.0);

    }; // end of definition of class ArcStatistics


inline ostream& ArcStatistics::print( ostream& aStream ) const
{
    aStream << "Results Statistics "              << endl
            << "Min: "                 << min_    << endl
            << "Max: "                 << max_    << endl
            << "Mean: "                << mean_   << endl
            << "Standard Deviation: "  << stddev_ << endl
            << "RMS: "                 << rms_    << endl << endl;
    return aStream;
}

inline ArcStatistics::ArcStatistics()
    : mean_(0.0), 
      stddev_(0.0),
      rms_(0.0),
      min_(0.0),
      max_(0.0)
{ ; }

/**
****************************************************************************
* This method computes the following statistics:
- average   (returned by method ArcStatistics::getMean())
- min value (returned by method ArcStatistics::getMin())
- max value (returned by method ArcStatistics::getMax())
NOTE: the input array must not contain null or not significant values

@memo
* Compute the following statistics: min, max, mean.
@param
dataPtr (INPUT) array of integers with the values on which the statistics must be computed
dimVector - (INPUT) dimension of the above array valuesPtr
scale     - (INPUT) factor by which the integer value must be multiplied
*/
inline void ArcStatistics::calcMeanMaxMin (int * dataPtr, int dimVector, double scale)
{
    if (dataPtr == NULL || dimVector == 0)
	mean_ = 0.;

    mean_ = dataPtr[0];
    min_  = dataPtr[0];
    max_  = dataPtr[0];

    for (int  i = 1; i < dimVector; i++)
    {
	mean_ += dataPtr[i];
	if(min_ > dataPtr[i])  min_ = dataPtr[i]; 
	if(max_ < dataPtr[i])  max_ = dataPtr[i]; 
    }
            
    mean_ *= scale/(double)dimVector;
    min_  *= scale;
    max_  *= scale;      
}

/**
****************************************************************************
* This method computes the following statistics:
- average   (returned by method ArcStatistics::getMean())
- standard deviation (returned by method ArcStatistics::getStdDev())
- RMS       (returned by method ArcStatistics::getRMS())
- min value (returned by method ArcStatistics::getMin())
- max value (returned by method ArcStatistics::getMax())
NOTE: the input array must not contain null or not significant values

@memo
* Compute the following statistics: min, max, mean, standard deviation, RMS 
@param
dataPtr (INPUT) array of integers with the values on which the statistics must be computed
dimVector - (INPUT) dimension of the above array valuesPtr
scale     - (INPUT) factor by which the integer value must be multiplied
*/
inline void ArcStatistics::calcAll(int * dataPtr, int dimVector, double scale)
{
    if (dataPtr == NULL || dimVector == 0)
	stddev_ = 0.;
      
    calcMeanMaxMin(dataPtr, dimVector, scale);
      
    double valmm;
    for (int  i = 0; i < dimVector; i++)
    {
	valmm = (double) dataPtr[i] * scale - mean_;
	stddev_ += valmm * valmm;
    }

    if(dimVector > 2)
	stddev_ = stddev_ / (double) (dimVector-1);
      
    rms_ = sqrt(stddev_);

}

#endif /* _ArcStatistics_h_ */
