// -*-c++-*-
#ifndef _ArcTime_h_
#define _ArcTime_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcTime.h,v 1.1 2007/10/17 08:12:10 cmoins Exp $
 *
 * ArcTime.h - general C routines for managing date translation to Mjd
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        20 Oct 2004  Converted to DfsStdLib/SybConn
 */

#if defined (__linux__)
#include <sys/time.h>
#include <unistd.h>
#endif

#include <errno.h>
#include <stdlib.h>
#include "DfsCString.h"

// Typedef for MJD dates
// MJD is the number of days sice 17th November 1858
typedef double MJD_DATE;

// Constant: number of days between 01/01/1901 and mjd0
const double MJD_1901 = 15385.;

// Constant: number of days between 01/01/1970 and mjd0
const double MJD_1970 = 40587.;

// Constant: number of days between 01/01/1980 and mjd0
const double MJD_1980 = 44239.;

// Constant: number of days between 01/01/1970 and 01/01/1980
const double DAYS_1970_1980 = 3652.;

// Constant: number of days between 01/01/1970 and 01/01/1980
const unsigned long SECS_1970_1980 = 315532800;

// Translates the number of seconds given by Unix time into MJD value 
// Unix returns the number of seconds since 1st January 1970 00:00:00 UTC 
inline const MJD_DATE UnixTimeToMjd(const unsigned long secs) { return (double(secs)/86400. + MJD_1970); }

// Returns the current time in mjd format
inline const MJD_DATE getNowMjd()
{
    // Init the date-time fields with the current UT time
    struct timeval tv;
    struct timezone tz;
    gettimeofday(&tv, &tz);

    // add the milliseconds
    double seconds = double(tv.tv_sec) + double(tv.tv_usec)/1000000.;

    // get Mjd value
    return (seconds/86400. + MJD_1970);
}

// Returns the current time in Unix time format
inline const unsigned long getNowUnixTime()
{
    // Init the date-time fields with the current UT time
    struct timeval tv;
    struct timezone tz;
    gettimeofday(&tv, &tz);

    return tv.tv_sec;
}

// Translates MJD value into Unix time (number of seconds since 1st January 1970 00:00:00 UTC) 
inline const unsigned long MjdToUnixTime(MJD_DATE mjd) { return (unsigned long)(((mjd - MJD_1970)*86400.)+0.5); }

// Translates MJD value into Unix time since 1980 (number of seconds since 1st January 1980 00:00:00 UTC) 
inline const long MjdToUnixTime80(MJD_DATE mjd) { return (long)(((mjd - MJD_1980)*86400.)+0.5); }

// Translates a string in the form YYYY-MM-DDThh:mm:ss 
// into Unix time since 1970 
inline const unsigned long stringToUnixTime(const DfsCString& dateString)
{
    struct tm tm;
    
    char buf[20];
    if(dateString.length() > 10) {
        // the date format is YYYY-MM-DDThh:mm:ss
    	strncpy(buf, dateString.data(), 19);
	buf[19]='\0';
    }
    else
        // the date format is YYYY-MM-DD
    	sprintf(buf, "%sT00:00:00", dateString.data());
    
    char *p = strchr(buf,'T');
    if (p)
	*p = ' ';
    
    if(strptime(buf, "%Y-%m-%d %T", &tm) == NULL)
	return 0;
    
    // convert tm to Unix time (since 1970)
    errno = 0;
    unsigned long unixTime = mktime(&tm);
    if(errno == 0 )
	return unixTime;
    else
	return 0;
}

// Translates a string in the form YYYY-MM-DDThh:mm:ss or YYYY-MM-DD
// into Unix time since 1980 
inline const long stringToUnixTime80(const DfsCString& dateString)
{
    unsigned long unixTime = stringToUnixTime(dateString);
    long unixTime80 = 0;
    if(unixTime != 0)
	unixTime80 = unixTime - SECS_1970_1980;
   
    return unixTime80;
}

// Translates a string in the form YYYY-MM-DDThh:mm:ss or YYYY-MM-DD 
// into MJD value 
inline const MJD_DATE stringToMjd(const DfsCString& dateString) 
{ 
    unsigned long unixTime = stringToUnixTime(dateString);
    if(unixTime != 0)
	return (UnixTimeToMjd(unixTime));
    else
	return 0.0;
}

// Convertes an integer with a coordinate in milliarcsecs 
// into a string in Degree format 
inline const DfsCString masToDegree(const int& mas) 
{ 
    int d,m;
    double s;
    char buff[30];
    
    d = abs(mas)/3600000;
    m = (abs(mas) - d*3600000)/60000;
    s = (abs(mas) - d*3600000 - m*60000)/1000.;
    
    if(mas < 0) d*=-1;
    
    sprintf(buff, "%02d:%02d:%2.1f", d, m, s);
    
    return(DfsCString(buff));   
}

#endif /* _ArcTime_h_ */
