// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observator
// LICENSE:   ESO internal use only
//
// PROJECT:   VLT Data Flow Syste
// AUTHOR:    Christophe Moins- ESO/SDD/DF
// SUBSYSTEM: arcsys
//
// $Revision: 1.4 $
// $Log: CalibObsBlock.h,v $
// Revision 1.4  2008/09/11 08:31:07  vforchi
// fixed compilation problems with gcc 4.2.4
//
// Revision 1.3  2007/05/04 09:54:29  cmoins
// removal of storeData, updateObsBlockDB and getMaxTime implementation
//
// Revision 1.2  2007/04/26 12:53:33  cmoins
// adding of a StorageService template parameter
//
// Revision 1.1  2007/04/25 15:52:56  cmoins
// creation of a class implementing a calibration observation block
//
// 
//------------------------------------------------------------------------------

#ifndef CALIBOBSBLOCK_H
#define CALIBOBSBLOCK_H

#include <string>
#include <DfsLog.h>
#include <boost/tokenizer.hpp>
#include "ObsBlock.h"
#include "RawDataProduct.h"

using namespace std;

/// Calibration observation block template class.
/*
 * A calibration observation block contains only calibration files. It will use 
 * an ObservationService and a StorageService template parameter to remain 
 * independent of the data retrieval layer and of the storage service layer.
 */
template <class ObservationService, class StorageService>
class CalibObsBlock : public ObsBlock<StorageService>
{
public :

#if defined UNIT_TEST
  friend class TestCalibObsBlock;
  friend class TestObsProgPackage;
#endif
 
  /// \brief Constructor
  /**
   * \param blockId block identifier
   * \param connAlias data repository connection parameter
   */
  CalibObsBlock(int blockId,
		string connAlias):
    ObsBlock<StorageService>(blockId,connAlias)
  {}

  /// \brief Destructor
  ~CalibObsBlock()
  {}

  /// \brief fetches a list of files associated to the block.
  /**
   * \param startTime start time of the files retrieval
   * \param startTimeAdjustment correction to apply on start time for 
   *                            calibration files retrieval
   * \param endTime end time of the files retrieval
   * \param endTimeAdjustment   correction to apply on end time for 
   *                            calibration files retrieval
   * \param rlistOfUsedInstruments list of instruments that should be 
   *                               excluded to retrieve calibration files
   * \return 0 if successful, -1 otherwise 
   */
  virtual int fetchData(double  startTime,
			double  startTimeAdjustment,
			double  endTime,
			double  endTimeAdjustment,
			string& rlistOfUsedInstruments);
  
  /// \brief instantiates a copy of the object
  /// returns the address of the copy
  virtual ObsBlock<StorageService>* clone() const;

};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Fetches a list of files associated to the block
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
CalibObsBlock<ObservationService,StorageService>::
fetchData(double startTime,
	  double  startTimeAdjustment,
	  double endTime,
	  double  endTimeAdjustment,
	  string& rlistOfUsedInstrument) 
{
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;

  boost::char_separator<char> sep(",");
  int retStatus = 0;
  string connAlias = ObsBlock<StorageService>::getAlias();
  ObservationService observationService(connAlias);  

  DFS_TRACE("CalibObsBlock<ObservationService,StorageService>::fetchData");
  int blockId = ObsBlock<StorageService>::getBlockId();
  RawDataProduct::ResultSet dataFileSet = ObsBlock<StorageService>::getDataFileSet();

  if (observationService.findObsBlockCalibFiles(blockId,
						startTime,
						endTime,
						rlistOfUsedInstrument,
						dataFileSet) > 0)
    {
      DFS_INFO("Files retrieval succeeded for %d",blockId);
      ObsBlock<StorageService>::setDataFileSet(dataFileSet);
    }
  else
    {
      DFS_ERROR("Files retrieval failed for %d",blockId);
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// instantiates a copy of the object
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
ObsBlock<StorageService>*
CalibObsBlock<ObservationService,StorageService>::clone() const
{
  return new CalibObsBlock<ObservationService,StorageService>(*this);
}

#endif /* CALIBOBSBLOCK_H */
