// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Stefano Zampieri - ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.6 $
// $Log: CondorJob.h,v $
// Revision 1.6  2008/09/29 09:43:32  vforchi
// modified abScheduler to manage more complex ABs and avoid using condor for complex tasks,
// because it seems to be quite slow
//
// Revision 1.5  2006/07/03 13:51:14  szampier
// checking boost:filesystem exceptions. improved documentation.
// changed submitAbToCondor to read configuration parameters from
// input rather than from environment variables
//
// Revision 1.4  2006/06/22 15:57:36  szampier
// uniquely timestamped condor job directories to prevent clashes
//
// Revision 1.3  2006/06/21 13:08:29  szampier
// removed userName parameter in submitAbToCondor
//
// Revision 1.2  2006/06/20 17:19:11  szampier
// set default userName = vista in submitAbToCondor
//
// Revision 1.1  2006/06/20 16:14:31  szampier
// first check-in
//
// 
// Programmatic access to Condor
// 
//------------------------------------------------------------------------------

#ifndef CONDOR_CLIENT_H
#define CONDOR_CLIENT_H

#include <string>
#include <list>
#include <map>

#include <iostream>
#include <fstream>
#include <sstream>
#include <cassert>
#include <iomanip>

#include <DfsLog.h>

// forward declarations
class CondorJob;
class CondorDag;

/// Programmatic access to Condor Jobs
/**
 * The CondorJob class can be used to create condor jobs and submit
 * them to Condor
 *
 * <p>
 * For more information about Condor please visit the condor web pages:
 * http://www.cs.wisc.edu/condor/
 */
class CondorJob
{
 public:
  
  /// \brief Constructor
  /// \param f The job filename. If not defined, it will be constructed from the Job ID
  CondorJob(const std::string& f);
  
  /// \brief Get job ID
  const std::string& getId() const { return id_; }
  
  /// \brief Get job filename
  const std::string& getFileName() const { return fileName_; }
  
  /// \brief Set path to condor_submit command
  /// \param c The path to condor_submit
  void setSubmitCommand(const std::string& c) { submitCommand_ = c; }
  
  /// \brief Set universe
  /// \param u The universe
  void setUniverse(const std::string& u) { universe_ = u; }
  
  /// \brief Set program to be executed
  /// \param e The program to be executed
  void setExecutable(const std::string& e) { executable_ = e; }
  
  /// \brief Set filename where the program standard output is written
  /// \param o The output filename
  void setOutput(const std::string& o) { output_ = o; }
  
  /// \brief Set filename where the program standard error is written
  /// \param e The error filename
  void setError(const std::string& e) { error_ = e; }
   
  /// \brief Set filename used by Condor to log entries about the job execution
  /// \param l The log filename
  void setLog(const std::string& l) { log_ = l; }
  
  /// \brief Set command line arguments to be passed to the program
  /// \param a The command line arguments
  void setArguments(const std::string& a) { arguments_ = a; }
  
  /// \brief Set file transfer flag
  /// \param s The file transfer flag
  void setShouldTransferFiles(const std::string& s) { shouldTransferFiles_ = s; }
  
  /// \brief Set initial dir
  /// \param i The initial dir
  void setInitialDir(const std::string& i) { initialDir_ = i; }
  
  /// \brief Set notification flag
  /// \param n The notification flag
  void setNotification(const std::string& n) { notification_ = n; }
  
  /// \brief Save job to disk
  /// \return 0 in case of success, -1 in case of failure
  int save();
  
  /// \brief Submit job to Condor
  /// \return 0 in case of success, -1 in case of failure
  int submit();
  
 private:
  /// The job unique identifier
  std::string id_;
  /// The job filename
  std::string fileName_;
  /// The path to condor_submit command
  std::string submitCommand_;
  /// The Condor universe - defaults to "vanilla"
  std::string universe_;
  /// The program to be executed
  std::string executable_;
  /// File where the program standard output is written
  std::string output_;
  /// File where the program standard error is written
  std::string error_;
  /// File used by Condor to log entries about the job execution
  std::string log_;
  /// Command line arguments to be passed to the program
  std::string arguments_;
  /// Controls the Condor file transfer mechanism - defaults to "NO"
  std::string shouldTransferFiles_;
  /// The initial working directory - defaults to current working directory
  std::string initialDir_;
  /// Controls the Condor notification mechanism - defaults to "NEVER"
  std::string notification_;
};


/// Less-than comparator for CondorJob objects.
/**
 * This class is needed to store CondorJob objects in an STL map
 * \relates CondorJob
 */
struct CondorJobComparator
{
  bool operator()(const CondorJob& leftOperand,
		  const CondorJob& rightOperand) const
  {
    // TODO: think about a better sorting criterium for CondorJobs
    return (leftOperand.getId() < rightOperand.getId());
  }
};


/// Programmatic access to Condor DAGs (Directed Acyclic Graph)
/**
 * A directed acyclic graph (DAG) can be used to represent a set of programs
 * where the input, output, or execution of one or more programs is dependent
 * on one or more other programs.
*/
class CondorDag
{
 public:
  /// Equivalent of std::list<CondorJob>
  typedef std::list<CondorJob> job_list_t;
  /// Equivalent of std::list<CondorJob>::iterator
  typedef std::list<CondorJob>::iterator job_list_iterator_t;
  typedef std::map<CondorJob, job_list_t, CondorJobComparator> dag_t;
  typedef std::map<CondorJob, job_list_t, CondorJobComparator>::iterator
    dag_iterator_t;
  
  /// \brief Constructor
  /// \param f The DAG filename
  CondorDag(const std::string& f);
  
  /// \brief Set current working directory
  /// \param i The current working directory
  void setInitialDir(const std::string& i) { initialDir_ = i; }
  
  /// \brief Set path to condor_submit_dag command
  /// \param c The path to condor_submit_dag
  void setSubmitCommand(const std::string& c) { submitCommand_ = c; }
  
  /// \brief Add a condor job to the DAG
  /// \param job The CondorJob object to be added to the DAG
  /// \param parents The list of CondorJob objects this job depends on
  void addJob(const CondorJob& job, const job_list_t& parents);
  
  /// \brief Save DAG to disk
  /// \return 0 in case of success, -1 in case of failure
  int save();
  
  /// \brief Submit DAG to Condor
  /// \return 0 in case of success, -1 in case of failure
  int submit();
  
 private:
  /// The directed acyclic graph
  dag_t dag_;
  /// The DAG filename
  std::string fileName_;
  /// The current working directory
  std::string initialDir_;
  /// The path to condor_submit_dag command
  std::string submitCommand_;
};


/// The CondorSubmit namespace contains utility functions to
/// submit Association Blocks to Condor
namespace CondorSubmit
{
  /// \brief Check that the given path is valid and exists
  /// \param fileName The filename to check
  /// \return 0 if the given path is valid, -1 otherwise
  int checkPath(const std::string& fileName);
  
  /// \brief Submit an Association Block to Condor
  /// \param abFilename The Association Block filename
  /// \param numExt The number of extensions
  /// \param userName The user name
  /// \param condorBaseDir The directory where condor jobs are created
  /// \param processAbCmd The process AB command
  /// \param joinAbCmd The join AB command
  /// \param condorSubmitCmd The condor submit command
  /// \param condorSubmitDagCmd The condor submit dag command
  /// \return 0 on success, -1 on failure
  int submitAbToCondor(const std::string& abFilename,
		       int numExt,
		       const std::string& userName,
		       const std::string& condorBaseDir,
		       const std::string& processAbCmd,
		       const std::string& joinAbCmd,
		       const std::string& condorSubmitCmd,
		       const std::string& condorSubmitDagCmd,
		       const std::string& workingDir);
}


#endif
