// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Stefano Zampieri - ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.2 $
// $Log: ConnectionPool.h,v $
// Revision 1.2  2006/07/03 13:44:10  szampier
// Reaper is now a subclass of ACE_Task, for better thread handling.
// declared (but not defined) copy c'tor and assignment operator as private methods.
//
// Revision 1.1  2006/05/30 12:33:35  szampier
// first check-in
//
// 
// ConnectionPool - maintains pools of database connections
//
// Note: this class is a C++ port of org.eso.ohs.core.utilities.ConnectionPool.java
//------------------------------------------------------------------------------

#ifndef CONNECTION_POOL_H
#define CONNECTION_POOL_H

#include <ctime>
#include <vector>

#include <ace/Task.h>
#include <ace/Thread_Manager.h>
#include <ace/Thread_Mutex.h>
#include <ace/Guard_T.h>

#include <sybconnection.h>
#include <DfsLog.h>

#define MAX_POOL_SIZE 20

/// Maintains pools of database connections
/**
 *
 * ConnectionPool maintains pools of database connections, including
 * enforcing an upper limit and closing connections that have not recently
 * been used.  If there are no connections left in the pool, new ones are
 * allocated upto the given limit of MAX_CONNECTIONS.  If all permitted
 * connections have been created and are currently allocated, the client
 * must wait for one to be released.
 * 
 * <p>
 * An inner class, Reaper, checks every so often for idle connections.
 * Those that have not been used in the last MAX_IDLE_LIFETIME milliseconds
 * are removed from the pool and closed.
 * 
 * <p>
 * Note: this class is a C++ port of org.eso.ohs.core.utilities.ConnectionPool.java
 * 
 */

class ConnectionPool
{
 private:
  
  /// Maintains connection information
  class Connection
    {
    public:
      /// Destructor
      ~Connection();
      /// The database connection
      SybaseConnection* sybconn;
      /// This flag is set to false when this connection is used by a process.
      bool available;
      /// The server name
      std::string server;
      /// The user name
      std::string user;
      /// The password
      std::string password;
      /// Last time this connection was released.
      time_t lastReleaseTime;
      /// Creation time
      time_t creationTime;
    };
  
  
  /// Dead connection reaper
  /**
   * The reaper class that scans the pool 
   * and closes old (un-needed) connnections.
   */
  class Reaper : public ACE_Task<ACE_MT_SYNCH>
  {
  public:
    /// \brief Constructor.
    /// \param pool the connection pool
    /// \param sleepTime
    /// \param maxIdleLifeTime
    /// \param maxLifeTime
    Reaper(ConnectionPool* pool,
	   time_t sleepTime = 5,
	   time_t maxIdleLifeTime = 60,
	   time_t maxLifeTime = 600);
    
    /// Check for unused connections
    virtual int svc(void);
    /// Set sleep time
    void sleepTime(time_t t) { sleepTime_ = t; }
    /// Set max idle life time
    void maxIdleLifeTime(time_t t) { maxIdleLifeTime_ = t; }
    /// Set max life time
    void maxLifeTime(time_t t) { maxLifeTime_ = t; }
    
  private:
    /// The connection pool
    ConnectionPool* connectionPool_;
    /// Seconds between checks of idle/old connections
    time_t sleepTime_;
    /// Seconds a connection can remain idle before being closed    
    time_t maxIdleLifeTime_;
    /// Seconds unallocated connection lasts before being closed
    time_t maxLifeTime_;
  };
  
public:
  /// Equivalent to ACE_Thread_Mutex
  typedef ACE_Thread_Mutex lock_t;
  
  /// Equivalent to std::vector<Connection>
  typedef std::vector<Connection> pool_t;
  
  /// Equivalent to std::vector<Connection>::iterator
  typedef std::vector<Connection>::iterator iterator_t;
  
  /// Get unique instance (synchronised)
  /**
   * The access to the unique instance is synchronised
   * using the double-checked locking optimisation
   */
  static ConnectionPool* instance();
  
  /// Destructor
  ~ConnectionPool();
  
  /// Shutdown the reaper
  void shutdownReaper()
  {
    DFS_TRACE("ConnectionPool::shutdownReaper");
    DFS_DEBUG("Terminating reaper thread");
    ACE_Thread_Manager::instance()->cancel_task(&reaper_);
    reaper_.wait();
  }
  
  /// \brief Set reaper sleep time
  /// \param the sleep time
  void setReaperSleepTime(time_t t) { reaper_.sleepTime(t); }
  
  /// \brief Set connection max idle life time
  /// \param the max idle life time
  void setMaxIdleLifeTime(time_t t) { reaper_.maxIdleLifeTime(t); }
  
  /// \brief Set connection max life time
  /// \param the max life time
  void setMaxLifeTime(time_t t) { reaper_.maxLifeTime(t); }
  
  /// \brief Set max pool size (synchronised)
  /// \param the max pool size
  void setMaxPoolSize(int size);
  
  /// \brief Get an existing or new DB connection. Return NULL if a
  /// \brief connection could not be found (synchronised)
  /// \param server the DB server name
  /// \param user the DB user name
  /// \param password the DB password
  /// \return a DB connection or NULL if a connection could not be found
  SybaseConnection* acquireConnection(const std::string& server,
				      const std::string& user,
				      const std::string& password);
  
  /// \brief Release the given connection back into the pool (synchronised)
  /// \param conn the DB connection
  /// \return 0 in case of success, -1 in case of error
  int releaseConnection(SybaseConnection* conn);
  
 private:
  /// Constructor
  /**
   * Creates a reaper object to remove unused connections
   */
  ConnectionPool();
  
  /// Copy constructor
  /**
   * This method is not defined and should never be called
   */
  ConnectionPool(const ConnectionPool&);
  
  /// Assignment operator
  /**
   * This method is not defined and should never be called
   */
  const ConnectionPool& operator= (const ConnectionPool&);
  
  /// \brief Create a new DB connection. Return NULL if a
  /// connection could not be created
  /// \return a new DB connection or NULL if a connection could not be created
  /// \param server the DB server name
  /// \param user the DB user name
  /// \param password the DB password  
  SybaseConnection* allocateNewConnection(const std::string& server,
					  const std::string& user,
					  const std::string& password);
  
  /// \brief Remove connection from the pool
  /// \param pos Iterator pointing to connection to be removed
  /// \return  An iterator pointing to the next element (or end())
  iterator_t removeConnection(iterator_t pos);
  
  /// The unique instance of this class
  static ConnectionPool* instance_;

  /// The lock used for synchronisation
  static lock_t lock_;

  /// The connection pool
  pool_t pool_;

  /// The max pool size
  static unsigned int maxPoolSize_;

  /// The reaper object
  Reaper reaper_;
};


#endif
