// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2003, 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWCString replacement
//
// $Revision: 1.13 $
// $Log: DfsCString.h,v $
// Revision 1.13  2008/07/24 16:26:23  nferguso
// Added ISO 14882 compliant forward declarations, as required by GCC since
// version 4.1.0.
//
// Revision 1.12  2005/07/22 18:51:20  nferguso
// Ported to DfsStdLib / SybConn.
//
// Revision 1.11  2004/09/27 14:34:06  nferguso
// Documentation tweak.
//
// Revision 1.10  2004/09/24 15:03:41  nferguso
// Added global toUpper() function.
//
// Revision 1.9  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.8  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.7  2004/05/13 09:08:01  nferguso
// Documentation updates.
//
// Revision 1.6  2004/03/30 11:42:59  nferguso
// Added new memory handling code in fitMemory() to avoid unnecessarily
// reallocating and copying string memory in std::string::reserve() calls
// using older versions of GNU LibStdC++. Preprocessor directives are
// provided so that fitMemory() can be compiled for the best match to
// the LibStdC++ version being used.
//
// Corrected DfsCString comparators to return DfsBoolean rather than bool.
//
// Revision 1.5  2004/03/26 15:07:39  nferguso
// Copyright years updated.
//
// Revision 1.4  2004/02/20 10:07:37  nferguso
// - checkIndex() is now a utility function in its own namespace.
// - capacity() added to allow stricter testing
// - more documentation for fitMemory()
//
// Revision 1.3  2004/01/14 14:00:16  nferguso
// All RogueWave-style string functionality required by the DFS is now fully
// implemented and unit tested.
//
// DfsCString memory management can use std::string::reserve() calls, or a
// less efficient copy-and-swap method. This version defaults to the copy-
// and-swap form, for LibStdC++ < 3.4 (see DfsCString::fitMemory() source).
//
// Revision 1.2  2003/12/19 15:03:02  nferguso
//
// String-handling classes can be built and used, although some things remain
// to be done. Most significantly:
//
// - methods working with iostreams are not fully compliant with RogueWave
// - STL string reserve() memory management is incomplete
//
// No non-string RogueWave classes have been replaced yet.
//
// Revision 1.1.1.1  2003/11/28 14:21:16  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_CSTRING_H
#define DFS_CSTRING_H

// C++ STL includes
#include <iosfwd>

// Includes from this library
#include <DfsConstants.h>
#include <DfsCSubString.h>
#include <DfsUtils.h>

// Forward declaration. 
class DfsCSubString; 

/// The DfsCString class is a partial replacement of the RWCString class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 *
 * The RWCString class contains an implementation of copy-on-write, where 
 * copied strings continue to point to the original data until an actual 
 * change is required. In contrast, the DfsCString class allows the STL 
 * string it contains to handle efficiency issues and memory allocation 
 * policy, whenever possible. 
 * 
 * GNU LibStdC++ 3.4.0 provides copy-on-write as standard within its STL 
 * strings. Memory allocation is a more complex issue, however - please 
 * see the fitMemory() documentation for details. 
 *
 * <B>WARNING:</B> This class is not intended for use with strings which 
 * contain multibyte characters. 
 * 
 * As DfsCString methods should correspond exactly to the original RogueWave 
 * API, this documentation is sparse. Consult the RogueWave documentation 
 * for more details. 
 */ 
class DfsCString
{
    /// @brief This class is only instantiated by other DFS string classes, 
    /// in order to manipulate substrings. It is equivalent to RWCSubString. 
    friend class DfsCSubString;

    friend std::ostream& operator<<(std::ostream& stream, 
                                    const DfsCString& sourceString);
    friend std::istream& operator>>(std::istream& stream, 
                                    DfsCString& destString);

public:
    /// For specifying case-sensitivity in DfsCString method calls. 
    typedef enum {
        exact,          ///< Case-sensitive. 
        ignoreCase      ///< Case-insensitive. 
    } CaseCheck;

    /// @brief For specifying character-stripping requirements in DfsCString 
    /// method calls. 
    typedef enum {
        leading,        ///< Strip leading characters. 
        trailing,       ///< Strip trailing characters. 
        both            ///< Strip characters from both ends. 
    } StripType; 

    /// Creates a null DfsCString with length zero. 
    DfsCString();

    /// @brief Creates a new DfsCString from the null-terminated character 
    /// string @a sourceString. 
    DfsCString(const char* sourceString);

    /// @brief Creates a new DfsCString from the first @a length bytes of 
    /// @a sourceString, including any embedded null bytes. 
    DfsCString(const char* sourceString, size_t length);

    /// @brief Creates a null DfsCString with length zero, but with @a capacity
    /// bytes of preallocated storage. 
    DfsCString(DfsSize_T capacity);

    /// Creates a new DfsCString from the std::string @a sourceString. 
    DfsCString(const std::string& sourceString);

    /// Copy constructor. 
    DfsCString(const DfsCString& sourceString);

    /// Creates a new DfsCString from the DfsCSubString @a sourceSubString. 
    DfsCString(const DfsCSubString& sourceSubString);

    /// Creates a new DfsCString containing the single character @a c. 
    DfsCString(char c);

    /// Virtual destructor for safety of inheritance, eg. by DfcParseString. 
    virtual ~DfsCString();

    /// Type conversion operator to const char*. 
    operator const char*() const;

    /// @brief Replaces the contents of this DfsCString with a copy of the 
    /// @a sourceString's contents. 
    DfsCString& operator=(const char* sourceString);

    /// @brief Replaces the contents of this DfsCString with a copy of the 
    /// @a sourceString's contents. 
    DfsCString& operator=(const std::string& sourceString);

    /// @brief Replaces the contents of this DfsCString with a copy of the 
    /// @a sourceString's contents. 
    DfsCString& operator=(const DfsCString& sourceString);

    /// Appends the @a sourceString to this DfsCString.
    DfsCString& operator+=(const char* sourceString);

    /// Appends the @a sourceString to this DfsCString.
    DfsCString& operator+=(const DfsCString& sourceString);

    /// Returns the character at the given @a index. 
    /** This operator always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    char operator[](size_t index) const throw(DfsBoundsErr);

    /// Returns the character at the given @a index. 
    /** This operator always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     *
     * <B>WARNING:</B> As with RWCStrings, DfsCStrings may reallocate their 
     * string storage at arbitrary times. This will invalidate all character 
     * references returned by operator[]. Users are advised not to store these 
     * references.
     */
    char& operator[](size_t index) throw(DfsBoundsErr);

    /// @brief Returns a substring of this string, containing @a length 
    /// characters and starting at @a startIndex. 
    DfsCSubString operator()(size_t startIndex, size_t length);

    /// @brief Returns a substring of this string, containing @a length 
    /// characters and starting at @a startIndex. 
    /** NOTE: The RogueWave implementation of this operator does <I>not</I> 
     * return a constant substring, although their documentation says that 
     * it does. The DfsCString implementation complies with the RogueWave 
     * documentation, and returns a constant substring. 
     */ 
    const DfsCSubString operator()(size_t startIndex, size_t length) const;

    /// Appends the @a sourceString to this DfsCString.
    DfsCString& append(const char* sourceString);

    /// Appends the @a sourceString to this DfsCString.
    DfsCString& append(const DfsCString& sourceString);

    /// Returns this DfsCString's contents as an STL string. 
    const std::string& asSTLString() const; 

    /// @brief Returns the number of characters this string can accommodate 
    /// without reallocating. 
    size_t capacity() const;

    /// Compares this DfsCString with the @a compareString. 
    /** Comparisons are made lexicographically; localisation is according 
     * to the current global C locale. 
     * 
     * @param compareString The string to compare with. 
     * @param compareCase Case sensitivity: 
     *                    - #exact
     *                    - #ignoreCase
     * @return 
     *         - < 0 if this DfsCString is less than compareString. 
     *         -   0 if this DfsCString is equal to compareString. 
     *         - > 0 if this DfsCString is greater than compareString. 
     */
    int compareTo(const char* compareString, 
                  CaseCheck compareCase = DfsCString::exact) const;

    /// Compares this DfsCString with the @a compareString. 
    /** @see compareTo() */
    int compareTo(const DfsCString& compareString, 
                  CaseCheck compareCase = DfsCString::exact) const;

    /// Returns true if this DfsCString contains the @a pattern. 
    DfsBoolean contains(const char* pattern, 
                        CaseCheck compareCase = DfsCString::exact) const; 

    /// Returns true if this DfsCString contains the @a pattern. 
    DfsBoolean contains(const DfsCString& pattern, 
                        CaseCheck compareCase = DfsCString::exact) const;

    /// Returns the contents of this DfsCString as a constant C string. 
    const char* data() const;

    /// Searches for the given @a character within this DfsCString. 
    /** If the character is not found, or an embedded null byte is found before
     * the character, then this method returns DFS_NPOS. 
     *
     * <B>WARNING</B> 
     * 
     * The RogueWave documentation for this method is not self-consistent, and 
     * their implementation is different again. The method does not return the 
     * RW_NPOS value when called to search for a null byte. If a null byte is 
     * found, it returns the index, as normal. However, if no embedded null 
     * bytes exist, it returns length(), which is out of range for an index 
     * value. 
     * 
     * The DFS version behaves the same way, except for when no embedded null 
     * bytes exist. It then returns DFS_NPOS instead of length(). 
     */
    size_t first(char character) const;

    /// Searches for any one of the given characters within this DfsCString. 
    /** If none of the characters are found, or an embedded null byte is found 
     * before any of the characters could be found, then this method returns 
     * DFS_NPOS. 
     */ 
    size_t first(const char* charList) const;

    /// Searches for the @a pattern within this DfsCString. 
    /** Localisation is according to the current global C locale. 
     *
     * @param pattern The pattern to search for. 
     * @param startIndex An optional starting index for the search. 
     * @param compareCase Case sensitivity: 
     *                    - #exact
     *                    - #ignoreCase
     * @return The index of the first match, or DFS_NPOS if no match was found.
     */
    size_t index(const char* pattern, 
                 size_t startIndex = 0, 
                 CaseCheck compareCase = DfsCString::exact) const;

    /// Searches for the @a pattern within this DfsCString. 
    /** @see index() */
    size_t index(const DfsCString& pattern, 
                 size_t startIndex = 0, 
                 CaseCheck compareCase = DfsCString::exact) const;

    /// Inserts the @a sourceString into this DfsCString at the given @a index.
    DfsCString& insert(size_t index, const DfsCString& sourceString);

    /// Returns true if this DfsCString has zero length (ie. is null). 
    DfsBoolean isNull() const;

    /// @brief Searches for the last occurrence of the given @a character 
    /// within this DfsCString. 
    /** If the character is not found, or an embedded null byte appears between
     * it and the end of this DfsCString, then this method returns DFS_NPOS. 
     *
     * <B>WARNING</B> 
     * 
     * The RogueWave documentation for this method is not self-consistent, and 
     * their implementation is different again. In particular, if the RogueWave
     * version is asked to search for a null byte, it returns the index of the 
     * <I>first</I> embedded null byte. If none are found, it returns length(), 
     * which is out of range for an index value. 
     * 
     * When asked to find a character in a string which contains embedded null 
     * bytes, the RogueWave version will return the index of the last instance 
     * of the character that appears before the first embedded null byte, or 
     * RW_NPOS if the character does not appear in that part of the string. 
     *
     * The DFS version of this method emulates the RogueWave one when no null 
     * bytes are involved. However, if asked to search for a null byte, the 
     * DFS implementation returns the index of the <I>last</I> embedded null 
     * byte, or DFS_NPOS if none exist. 
     * 
     * When asked to find a character in a string which contains embedded null 
     * bytes, the DFS implementation returns the index of the last instance 
     * of the character that appears after the last embedded null byte, or 
     * DFS_NPOS if the character does not appear in that part of the string. 
     */
    size_t last(char character) const;

    /// Returns the length of this DfsCString in bytes. 
    size_t length() const;

    /// Reads characters from the @a stream into this DfsCString until EOF.
    /** The previous contents of the DfsCString are discarded. */
    std::istream& readFile(std::istream& stream); 

    /// @brief Reads characters from the @a stream into this DfsCString until 
    /// a newline is encountered, or until EOF. 
    /** Newlines are removed from the stream and discarded. The previous 
     * contents of the DfsCString are also discarded. If @a skipWhiteSpace is 
     * true, leading whitespace will be skipped before storing any characters. 
     */
    std::istream& readLine(std::istream& stream, 
                           DfsBoolean skipWhiteSpace = true); 

    /// @brief Reads characters from the @a stream into this DfsCString until 
    /// a null byte is encountered, or until EOF. 
    /** Null bytes are removed from the stream and discarded. The previous 
     * contents of the DfsCString are also discarded. 
     */
    std::istream& readString(std::istream& stream); 

    /// @brief Removes all characters from @a startIndex to the end of this 
    /// DfsCString. 
    DfsCString& remove(size_t startIndex);

    /// @brief Removes @a removeChars characters from this DfsCString, 
    /// starting at @a startIndex. 
    DfsCString& remove(size_t startIndex, size_t removeChars);

    /// @brief Replaces @a replaceChars characters in this DfsCString, 
    /// starting at @a startIndex, with a copy of @a sourceString. 
    DfsCString& replace(size_t startIndex, 
                        size_t replaceChars, 
                        const char* sourceString);

    /// @brief Changes the length of this DfsCString to @a newSize characters,
    /// adding spaces or truncating as required. 
    void resize(size_t newSize);

    /// @brief Strips the given @a character off this DfsCString according 
    /// to the given @a stripping requirement, returning the result as a 
    /// DfsCSubString. 
    /** As with RWCStrings, this method will return the null substring when 
     * called on an empty string, or a string containing nothing but the 
     * character to be stripped. 
     */ 
    const DfsCSubString strip(StripType stripping = DfsCString::trailing, 
                              char character = ' ') const;

    /// @brief Converts all upper-case characters in this DfsCString to their 
    /// lower-case equivalents. 
    /** Localisation is according to the current global C locale. */ 
    void toLower();

    /// @brief Converts all lower-case characters in this DfsCString to their 
    /// upper-case equivalents. 
    /** Localisation is according to the current global C locale. */ 
    void toUpper();

protected:
    /// @brief This STL string stores the contents of a DfsCString, and 
    /// provides much of its functionality. 
    std::string stlString;

    /// Stores the STL string length at the last call of fitMemory(). 
    size_t lastLength;

    /// Resizes allocated memory to fit the current string contents. 
    /** <B>RATIONALE</B>
     *
     * By default, RogueWave strings always constrain their memory usage 
     * to the smallest multiple of 16 bytes needed to hold their current 
     * string contents. 
     * 
     * Although STL strings automatically allocate more memory if required, 
     * they do not release it to the system when it is no longer needed. As 
     * a result, surplus memory must be released using an explicit call to 
     * std::string::reserve(). 
     * 
     * The fitMemory() method contains common code to check for surplus memory 
     * and call std::string::reserve() when necessary. It is called by all 
     * DfsCString methods that can change the string's memory requirements. 
     *
     * <B>COMPILATION</B>
     *
     * When compiling DfsStdLib, examine the definition of fitMemory() and 
     * ensure that the HAVE_SHRINK_TO_FIT_STRINGS macro is correctly defined 
     * for the standard C++ library being used. This macro takes account of 
     * the fact that not all standard C++ libraries support the shrink-to-fit 
     * std::string::reserve() call. 
     * 
     * If the HAVE_SHRINK_TO_FIT_STRINGS macro can be defined, check whether 
     * the HAVE_FAST_STRING_RESERVE can also be defined. This second macro 
     * accounts for the fact that some standard C++ libraries unnecessarily 
     * reallocate memory and copy its contents within std::string::reserve(). 
     *
     * <B>PERFORMANCE</B>
     *
     * The DfsCString code affected by these two macros has only been tested 
     * with GNU LibStdC++ v3.3.x, plus CVS development snapshots of v3.4 and 
     * v3.5. 
     *
     * <B>HAVE_SHRINK_TO_FIT_STRINGS</B>
     *
     * DfsCStrings defining HAVE_SHRINK_TO_FIT_STRINGS with LibStdC++ v3.4 
     * (or newer) will experience an allocation performance gain of about 3% 
     * over the slower copy-and-swap strategy that would otherwise be used. 
     * 
     * Whether they define HAVE_SHRINK_TO_FIT_STRINGS or not, DfsCStrings will 
     * always waste some memory, as follows: 
     *
     *  - per string of up to 115 characters - no wastage. 
     *  - per string of up to 4067 characters - 127 bytes. 
     *  - per string of over 4067 characters - 4095 bytes. 
     *
     * This is due to the performance-oriented allocation strategy used by 
     * LibStdC++. 
     * 
     * <B>WARNING:</B> DfsCStrings defining HAVE_SHRINK_TO_FIT_STRINGS with 
     * LibStdC++ v3.3 (or older) cannot reclaim surplus memory. The resulting 
     * strings can waste up to twice the memory required by their peak length 
     * since creation. 
     * 
     * <B>HAVE_FAST_STRING_RESERVE</B>
     * 
     * DfsCStrings that define HAVE_FAST_STRING_RESERVE with an appropriate 
     * version of LibStdC++ will experience an allocation performance gain 
     * of about 5% by omitting redundant (LibStdC++ implementation specific) 
     * calculations within fitMemory(). 
     *
     * <B>WARNING:</B> DfsCStrings will display severely degraded performance 
     * if HAVE_FAST_STRING_RESERVE is defined: 
     * 
     *  - with an inappropriate version of LibStdC++. 
     *  - when HAVE_SHRINK_TO_FIT_STRINGS is not defined. 
     * 
     * @sa DfsCString::lastLength 
     */
    void fitMemory(); 

private:
    
};

/// Equality comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator==(const DfsCString& leftOperand,
                             const char*       rightOperand);

/// Equality comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator==(const char*       leftOperand,
                             const DfsCString& rightOperand);

/// Equality comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator==(const DfsCString& leftOperand,
                             const DfsCString& rightOperand);

/// Inequality comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator!=(const DfsCString& leftOperand,
                             const char*       rightOperand);

/// Inequality comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator!=(const char*       leftOperand,
                             const DfsCString& rightOperand);

/// Inequality comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator!=(const DfsCString& leftOperand,
                             const DfsCString& rightOperand);

/// Less-than comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator<(const DfsCString& leftOperand,
                            const DfsCString& rightOperand);

/// Greater-than comparator for DfsCString objects. 
/** @relates DfsCString */
extern DfsBoolean operator>(const DfsCString& leftOperand,
                            const DfsCString& rightOperand);

/// @brief Performs string concatenation on the operands, returning the 
///        result as a DfsCString.
/** @relates DfsCString */
extern DfsCString operator+(const DfsCString& leftOperand, 
                            const DfsCString& rightOperand);

/// @brief Performs string concatenation on the operands, returning the 
///        result as a DfsCString.
/** @relates DfsCString */
extern DfsCString operator+(const char*       leftOperand, 
                            const DfsCString& rightOperand);

/// @brief Performs string concatenation on the operands, returning the 
///        result as a DfsCString.
/** @relates DfsCString */
extern DfsCString operator+(const DfsCString& leftOperand, 
                            const char*       rightOperand);

/// STL ostream output operator for DfsCString objects. 
/** Stream operators are made friends in order to allow delegation to the 
 * existing stream operators for STL strings. 
 * @relates DfsCString 
 */
extern std::ostream& operator<<(std::ostream& stream, 
                                const DfsCString& sourceString); 

/// STL istream input operator for DfsCString objects. 
/** Stream operators are made friends in order to allow delegation to the 
 * existing stream operators for STL strings. 
 * @relates DfsCString 
 */
extern std::istream& operator>>(std::istream& stream, 
                                DfsCString& destString); 

/// @brief Returns a copy of @a sourceString with all upper-case characters 
///        converted to their lower-case equivalents. 
/** @relates DfsCString */
extern DfsCString toLower(const DfsCString& sourceString); 

/// @brief Returns a copy of @a sourceString with all lower-case characters 
///        converted to their upper-case equivalents. 
/** @relates DfsCString */
extern DfsCString toUpper(const DfsCString& sourceString); 

#endif  // DFS_CSTRING_H
