// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWFile replacement
//
// $Revision: 1.6 $
// $Log: DfsFile.h,v $
// Revision 1.6  2004/09/01 13:32:06  nferguso
// Avoid missing return warnings on HP.
//
// Revision 1.5  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.4  2004/07/29 14:26:21  nferguso
// The (disabled) copy constructor and copy-assignment operator now cast
// their parameters to void. This stops the compiler from giving unwanted
// warnings about unused parameters.
//
// Revision 1.3  2004/04/05 10:43:10  nferguso
// Added DfsFile::Flush() method.
//
// Revision 1.2  2004/03/26 15:07:39  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/03/22 18:04:17  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_FILE_H
#define DFS_FILE_H

// C includes
#include <cassert>
#include <cstdio>

// UNIX includes
#include <unistd.h>

// Includes from this library
#include <DfsConstants.h>

/// The DfsFile class is a partial replacement of the RWFile class. 
/** Only methods required by existing DFS code have been reimplemented, as 
 * new DFS code should be implemented using STL capabilities. 
 * 
 * This class is essentially a thin wrapper about the corresponding C stdio 
 * function calls, except for the Exists() method, which requires the POSIX 
 * @c access() function from @c unistd.h. 
 *
 * NOTE: The SeekToEnd() method will only work correctly if the @c fseek() C 
 * call provides meaningful results in @c SEEK_END mode. This is not required 
 * by ISO C, but is supported in most versions of UNIX, including those used 
 * by the DFS. 
 *
 * As DfsFile methods should correspond exactly to the original RogueWave 
 * API, this documentation is sparse. Consult the RogueWave documentation 
 * for more details. 
 */
class DfsFile
{
public:
    /// Creates a DfsFile to access file @a filename with the mode @a mode. 
    /** The mode is specified in the same way as for the standard C library 
     * function @c fopen(). 
     */ 
    DfsFile(const char* filename, const char* mode = 0);

    /// Carries out any pending operations and closes the file. 
    ~DfsFile();

    /// @brief Returns the current file pointer position, in bytes from the 
    /// start of the file. 
    Dfsoffset CurOffset();

    /// Carries out any pending operations & flushes the file's I/O buffers. 
    DfsBoolean Flush();

    /// @brief Returns the @c FILE* that this DfsFile instance is using to 
    /// access its file. 
    FILE* GetStream();

    /// @brief Returns @c true if this DfsFile instance refers to a valid 
    /// file, @c false otherwise. 
    /** <B>WARNING:</B> Calling a method other than isValid() on an invalid 
     * DfsFile will probably cause an immediate crash. 
     */      
    DfsBoolean isValid() const;

    /// Reads one character from the file, storing it in @a character. 
    /** Returns @c true if the read was successful, @c false otherwise. */
    DfsBoolean Read(char& character);

    /// @brief Moves the file pointer to a position @a offset bytes from the 
    /// start of the file. 
    /** Returns @c true if the move was successful, @c false otherwise. */
    DfsBoolean SeekTo(Dfsoffset offset);

    /// Moves the file pointer to the end of the file. 
    /** Returns @c true if the move was successful, @c false otherwise.
     *
     * NOTE: This method will only work correctly if the @c fseek() C call 
     * provides meaningful results in @c SEEK_END mode. This is not required 
     * by ISO C, but is supported in most versions of UNIX, including those 
     * used by the DFS. 
     *
     * The only alternative is to repeatedly read data from the file until 
     * reading fails at EOF, which is significantly slower. 
     */
    DfsBoolean SeekToEnd();

    /// Writes the null-terminated character string @a string to the file. 
    /** Returns @c true if the write was successful, @c false otherwise. */
    DfsBoolean Write(const char* string);

    /// @brief Returns @c true if @a filename can be accessed in the specified 
    /// @a mode, @c false otherwise. 
    /** Modes are as follows: 
     * - @c F_OK The file exists (also implied by [XWR]_OK). 
     * - @c X_OK The file is executable. 
     * - @c W_OK The file is writable. 
     * - @c R_OK The file is readable. 
     * Modes may be bitwise-ORed together in any combination. 
     */ 
    static DfsBoolean Exists(const char* filename, int mode = F_OK);

protected:
    
private:
    /// This C @c FILE refers to the file operated on by this DfsFile instance.
    FILE* file;

    /// @brief @c True if the file specified at construction of this DfsFile 
    /// instance was successfully opened/created, @c false otherwise. 
    bool validFile; 
    
    /// Denotes the last type of access operation performed on the file. 
    enum {NONE,         ///< The last access did not read or write. 
          READ,         ///< The last access was a read. 
          WRITE,        ///< The last access was a write. 
          UNDEF         ///< The file state is currently undefined. 
    } lastAccess;

    /// @brief The copy constructor is not provided by RogueWave and should 
    /// never be used. 
    DfsFile(const DfsFile& source)
    {
        static_cast<void>(source);   // Avoid unused parameter warnings. 
        assert(0);
    }

    /// @brief The assignment operator is not provided by RogueWave and should 
    /// never be used. 
    DfsFile& operator=(const DfsFile& source)
    {
        static_cast<void>(source);   // Avoid unused parameter warnings. 
        assert(0);
        return(*this);               // Avoid missing return warnings. 
    }
};

#endif  // DFS_FILE_H
