// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Stefano Zampieri - ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.1 $
// $Log: DfsLogConfig.h,v $
// Revision 1.1  2006/08/02 15:40:17  szampier
// first check in
//
// 
// Dfs Logging Configuration - encapsulation of ACE Logging Strategy.
// 
//------------------------------------------------------------------------------

#include <string>

#include "DfsLog.h"

/// Dfs Logging Configuration - encapsulation of ACE Logging Strategy.
/**
 * This class provides convenience methods for
 * configuring the logging system. In particular
 * it allows to set the initial behaviour, the output
 * target(s) and the logging priorities.
 * 
 * One of the most powerful features of the logging strategy
 * is the ability to rotate the application's log files when
 * they reach a specified size.
 *
 * NOTE: in order to activate the log rotation, it is essential to:
 * - set the log file sampling interval using setInterval()
 * - set the maximum log file size using setMaxFileSize()
 * - set the maximum number of log files using setMaxNumFiles()
 * - run the reactor event loop using the static method runReactorEventLoop()
 * - at program exit, terminate the reactor event loop using endReactorEventLoop()
 *
 * For more information about the ACE Logging Strategy please refer
 * to the ACE documentation. See for example the ACE Programmer's Guide
 * Chapter 3.8 Runtime Configuration with the ACE Logging Strategy
 */

class DfsLogConfig
{
 public:
  /// Default constructor
  DfsLogConfig();
  
  /// \brief Set flags used to control logging
  /// \param f A bitwise-or of options flags used to set the initial
  ///          behavior and logging sink(s). Valid values are:
  ///          OSTREAM, STDERR, LOGGER, VERBOSE, SILENT, VERBOSE_LITE
  void setCtrlFlags(const std::string& f) { ctrlFlags_ = f; }
  
  /// Set the interval, in seconds, at which the log file is sampled
  void setInterval(int i) { interval_ = i; }
  
  /// Set maximum log file size in Kbytes
  void setMaxFileSize(int s) { maxFileSize_ = s; }
  
  /// Set program name for %n format specifier
  void setProgramName(const std::string& p) { programName_ = p; }
  
  /// Set maximum number of log files to create
  void setMaxNumFiles(int n) { maxNumFiles_ = n; }
  
  /// \brief Set log priority mask
  /// \param p A bitwise-or of priority flags used to set the log priority.
  ///          Valid values are:
  ///           DEBUG, INFO, WARNING, NOTICE, ERROR, CRITICAL, ALERT, EMERGENCY
  ///          ~DEBUG,~INFO,~WARNING,~NOTICE,~ERROR,~CRITICAL,~ALERT,~EMERGENCY
  void setPriority(const std::string& p) { priority_ = p; }
  
  /// Set log file name used when OSTREAM is specified as output target
  void setFileName(const std::string& f) { fileName_ = f; }
  
  /// \brief Apply the configuration options
  /// \retval  0 On success.
  /// \retval -1 On failure.
  int configure();
  
  /// \brief Spawns a new thread of control to run the reactor event loop
  /// \retval  0 On success.
  /// \retval -1 On failure.
  static int runReactorEventLoop();
  
  /// \brief Terminates the reactor event loop
  /// \retval  0 On success.
  /// \retval -1 On failure.
  static int endReactorEventLoop();
  
 private:
  std::string ctrlFlags_;
  int interval_;
  int maxFileSize_;
  std::string programName_;
  int maxNumFiles_;
  std::string priority_;
  std::string fileName_;
  
  /// Runs the reactor event loop in a separate thread of control
  static void * runReactor(void *);
};
