// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Stefano Zampieri - ESO/DMD/DFS
// SUBSYSTEM: n/a
//
// $Revision: 1.3 $
// $Log: DfsProperties.h,v $
// Revision 1.3  2006/08/28 17:22:10  nferguso
// Documentation/comment updates.
//
// Revision 1.2  2006/06/20 15:39:27  nferguso
// Added methods for retrieving other types of configuration parameter -
// WARNING: methods for unsigned int and ACE_Time_Value DO NOT WORK.
//
// Revision 1.1  2006/05/23 08:27:00  szampier
// first check-in
//
// 
// 
// DFS Properties - access to configuration files
// 
//------------------------------------------------------------------------------

#ifndef DFS_PROPERTIES_H
#define DFS_PROPERTIES_H

#include <string>

#include "ace/Configuration.h"
#include "ace/Configuration_Import_Export.h"
#include "ace/Time_Value.h"

#include "DfsLog.h"

/// This class provides convenient access to ACE style configuration files. 
/** The DfsProperties class is implemented according to the @e Singleton 
 * design pattern. It is used as follows: 
 *
 * - Call DfsProperties::instance() to get the singleton instance. 
 * - Call status() to ensure that the instance is correctly initialised. 
 * - Call load() to load the properties from a configuration file. 
 * - Call open_section() to open a configuration section within the newly 
 *   loaded data.
 * - Call get() to get a specific property value from that section. 
 */ 
class DfsProperties
{
public:
  
    /// Get the DfsProperties (singleton) instance. 
    static DfsProperties* instance();
  
    /// Virtual destructor. 
    virtual ~DfsProperties() { };
  
    /// Returns the initialisation status of the DfsProperties instance. 
    /** @retval 0 the instance is correctly initialised and may be used. 
     * @retval -1 the instance did not initialise correctly and must not 
     *            be used.
     */
    int status() { return status_; }
  
    /// Load all properties from the given file into memory. 
    /** @param filename the configuration file name. 
     *
     * @retval  0 on success. 
     * @retval -1 on failure. 
     */ 
    int load(const std::string& filename);
  
    /// Open the given configuration file @a section. 
    /** A configuration file section consists of the section name enclosed 
     * in square brackets, such as: 
     * 
     * @c [ABScheduler] 
     *
     * @param section the section name. 
     *
     * @retval  0 on success. 
     * @retval -1 on failure. 
     */ 
    int open_section(const std::string& section);
  
    // @{
    /// @brief Get the value of the property named @a key, convert it to 
    /// the required data type and store it in @a val. 
    /** @retval 0 on success. 
     * @retval -1 on failure. 
     */ 
    int get(const std::string& key, std::string& val); 
    int get(const std::string& key, u_int& val); 
    int get(const std::string& key, ACE_Time_Value& val);
    int get(const std::string& key, bool& val);
    // @} 
  
    // @{
    /// @brief Open the specified @a section in the configuration and get 
    /// the value of the property named @a key. Convert the value to the 
    /// required data type and store it in @a val. 
    /** @retval 0 on success. 
     * @retval -1 on failure. 
     */ 
    int get(const std::string& section, 
            const std::string& key, 
            std::string& val);
    int get(const std::string& section, 
            const std::string& key, 
            u_int& val); 
    int get(const std::string& section, 
            const std::string& key, 
            ACE_Time_Value& val); 
    int get(const std::string& section, 
            const std::string& key, 
            bool& val); 
    // @} 
  
protected:
    /// Private constructor. 
    DfsProperties(); 

    /// DfsProperties objects may not be copied. 
    DfsProperties(const DfsProperties& source)
    {
        DFS_TRACE(ACE_TEXT("DfsProperties::DfsProperties")
                  ACE_TEXT("(const DfsProperties&)")); 

        DFS_FATAL(ACE_TEXT("DfsProperties::DfsProperties")
                  ACE_TEXT("(const DfsProperties&)") 
                  ACE_TEXT(" must not be called.\n")); 

        ACE_ASSERT(0);  // This method should never be called. 
    }
    
    /// DfsProperties objects may not be copied. 
    DfsProperties& operator=(const DfsProperties& source)
    {
        DFS_TRACE(ACE_TEXT("DfsProperties::operator=()")); 

        DFS_FATAL(ACE_TEXT("DfsProperties::operator=()") 
                  ACE_TEXT(" must not be called.\n")); 

        ACE_ASSERT(0);  // This method should never be called. 
        return(*this);  // Avoid missing return warnings. 
    }

    /// The singleton instance. 
    static DfsProperties* instance_;
  
private:
    /// The initialisation status of the singleton instance. 
    int status_;

    /// Stores the currently-loaded configuration data. 
    ACE_Configuration_Heap config_;

    /// Imports configuration data from files. 
    ACE_Ini_ImpExp* config_importer_;

    /// @brief A reference to a configuration section within the currently 
    /// loaded configuration data. 
    ACE_Configuration_Section_Key section_;
};

#endif
