// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWTPtrDlist replacement
//
// $Revision: 1.5 $
// $Log: DfsTPtrDlist.h,v $
// Revision 1.5  2004/08/23 08:53:01  nferguso
// Extracted common list functionality from DfsTPtrDlist, DfsTPtrSlist,
// DfsTValDlist, and DfsTValSlist into a new DfsListBase base class,
// which is inherited by the four list classes.
//
// Revision 1.4  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.3  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.2  2004/03/26 15:07:39  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/03/11 13:21:53  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_TPTR_DLIST_H
#define DFS_TPTR_DLIST_H

// Includes from this library.
#include <DfsListBase.h>
#include <DfsTPtrDlistIterator.h>

// Required to resolve adverse include file ordering. 
template<typename ELEMENT> class DfsTPtrDlistIterator;

/// @brief The DfsTPtrDlist class is a partial replacement of the 
/// RWTPtrDlist class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 * 
 * NB: Indexed element access has linear complexity for lists. Although 
 * DfsTPtrDlist provides sequential access optimisation, vector indices 
 * are still faster. Non-sequential access should be avoided altogether. 
 * 
 * As DfsTPtrDlist methods should correspond exactly to the original 
 * RogueWave API, this documentation is sparse. Consult the RogueWave 
 * documentation for more details. 
 */ 
template<typename ELEMENT>
class DfsTPtrDlist : public DfsListBase<void*>
{
    /// DfsTPtrDlistIterators need access to the private data members. 
    friend class DfsTPtrDlistIterator<ELEMENT>;

public:
    /// Creates an empty DfsTPtrDlist. 
    DfsTPtrDlist<ELEMENT>() : 
        DfsListBase<void*>()
    {
        // This constructor does nothing beyond base class initialisation. 
    }

    /// Copy constructor. 
    DfsTPtrDlist<ELEMENT>(const DfsTPtrDlist<ELEMENT>& source) : 
        DfsListBase<void*>(source)
    {
        // This constructor does nothing beyond base class initialisation. 
    }
    
    // The default destructor suffices for this class. 

    /// @brief Replaces the contents of this DfsTPtrDlist with a copy 
    /// of the @a source list's contents. 
    DfsTPtrDlist<ELEMENT>& operator=(const DfsTPtrDlist<ELEMENT>& source)
    {
        // Delegate assignment to the base list class. 
        this->DfsListBase<void*>::operator=(source); 
        return(*this); 
    }

    /// Returns the element at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    ELEMENT*& operator[](size_t index) throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, listLength);

        // Move the last indexed element iterator to the new index, and 
        // return a reference to the pointer stored at that location. 
        return(*reinterpret_cast<ELEMENT**>(&(*moveLastElementPos(index))));
    }
    
    /// Returns the element at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    ELEMENT* const& operator[](size_t index) const throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, listLength);

        // Move the last indexed element iterator to the new index, and 
        // return a reference to the pointer stored at that location. 
        return(*reinterpret_cast<ELEMENT**>(&(*moveLastElementPos(index))));
    }

    /// Inserts the given @a element at the end of this DfsTPtrDlist. 
    void append(ELEMENT* element)
    {
        stlList.push_back(static_cast<void*>(element));
        ++listLength; 
    }
    
    /// @brief Clears all stored pointers, destroying the objects they point 
    /// to by calling the @c delete operator. 
    /** <B>WARNING:</B> This method will <I>not</I> call @c delete[] for an 
     * array ELEMENT type. Array types must be deleted by the user. 
     */ 
    void clearAndDestroy()
    {
        // Loop through all stored pointers, deleting their objects. 
        std::list<void*>::iterator deletePos = stlList.begin();
        std::list<void*>::iterator endPoint = stlList.end();

        while(deletePos != endPoint)
        {
            delete static_cast<ELEMENT*>(*deletePos);
            ++deletePos;
        }

        stlList.clear();                        // Clear the pointers. 
        lastElementPos = stlList.begin();       // Reset the last indexed
        lastElementIndex = 0;                   // element position. 
        lastElementValid = false;
        listLength = 0;                         // List length is zero. 
    }
    
protected:
    
private:

};

#endif  // DFS_TPTR_DLIST_H
