// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWTPtrOrderedVector replacement
//
// $Revision: 1.6 $
// $Log: DfsTPtrOrderedVector.h,v $
// Revision 1.6  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.5  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.4  2004/03/26 15:07:40  nferguso
// Copyright years updated.
//
// Revision 1.3  2004/02/27 16:21:29  nferguso
// - Adjusted a Doxygen comment.
// - Removed unnecessary constness in nested PtrEquals class.
//
// Revision 1.2  2004/02/24 10:18:51  nferguso
// Minor comment correction.
//
// Revision 1.1  2004/02/23 13:45:43  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_TPTR_ORDERED_VECTOR_H
#define DFS_TPTR_ORDERED_VECTOR_H

// Standard C++ includes
#include <algorithm>
#include <functional>
#include <vector>

// Includes from this library.
#include <DfsUtils.h>

/// @brief The DfsTPtrOrderedVector class is a partial replacement of the 
/// RWTPtrOrderedVector class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 * 
 * As DfsTPtrOrderedVector methods should correspond exactly to the original 
 * RogueWave API, this documentation is sparse. Consult the RogueWave 
 * documentation for more details. 
 */ 
template<typename ELEMENT>
class DfsTPtrOrderedVector
{
public:
    /// Creates an empty DfsTPtrOrderedVector. 
    DfsTPtrOrderedVector<ELEMENT>() 
    {
        // This constructor does not need to do anything. 
    }
    
    /// Copy constructor. 
    DfsTPtrOrderedVector<ELEMENT>(const DfsTPtrOrderedVector<ELEMENT>& source) :
        stlVector(source.stlVector)
    {
        // This constructor does nothing beyond member initialisation. 
    }

    // The default destructor suffices for this class. 

    /// @brief Replaces the contents of this DfsTPtrOrderedVector with a copy 
    /// of the @a source vector's contents. 
    DfsTPtrOrderedVector<ELEMENT>& operator=(const DfsTPtrOrderedVector<ELEMENT>& source)
    {
        if(&source != this)
        {
            stlVector = source.stlVector;
        }

        return(*this);
    }

    /// Returns the pointer at the given @a index. 
    /** This operator always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    ELEMENT*& operator[](size_t index) throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());
        
        return(*reinterpret_cast<ELEMENT**>(&stlVector[index]));
    }

    /// Returns the pointer at the given @a index. 
    /** This operator always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    ELEMENT* const& operator[](size_t index) const throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());

        return(*reinterpret_cast<ELEMENT* const*>(&stlVector[index]));
    }

    /// Appends the given @a element to the end of this DfsTPtrOrderedVector. 
    void append(ELEMENT* element)
    {
        // Append the element. 
        stlVector.push_back(static_cast<void*>(element));
    }

    /// Returns the pointer at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    ELEMENT* const& at(size_t index) const throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());

        return(*reinterpret_cast<ELEMENT* const*>(&stlVector[index]));
    }    

    /// @brief Clears all stored pointers, without affecting the objects they 
    /// point to. 
    void clear()
    {
        stlVector.clear();
    }

    /// @brief Returns @c true if this DfsTPtrOrderedVector contains a 
    /// pointer to an object that compares equal to the object pointed 
    /// to by @a element, @c false otherwise. 
    /** NB: This method performs a linear search through the vector. */
    bool contains(const ELEMENT* element) const
    {
        std::vector<void*>::const_iterator foundPos(
            find_if(stlVector.begin(), stlVector.end(), PtrEquals(element))); 

        return(foundPos != stlVector.end());
    }

    /// Returns the number of entries in this DfsTPtrOrderedVector. 
    size_t entries() const
    {
        return(stlVector.size());
    }

    /// Inserts the given @a element at the end of this DfsTPtrOrderedVector. 
    /** @return Always returns @c true. */
    bool insert(ELEMENT* element)
    {
        // Insert the element. 
        stlVector.push_back(static_cast<void*>(element));

        return(true);
    }

    /// @brief Returns @c true if this DfsTPtrOrderedVector is empty, @c false 
    /// otherwise. 
    bool isEmpty() const
    {
        return(stlVector.empty());
    }

    /// Returns the number of entries in this DfsTPtrOrderedVector. 
    size_t length() const
    {
        return(stlVector.size());
    }

private:
    /// @brief This class permits equality comparisons on objects for which 
    /// only pointers are available. 
    class PtrEquals : std::unary_function<const void*, bool> 
    {
    public:
        /// @brief Creates a new PtrEquals instance to make comparisons with 
        /// the object referenced by @a compare. 
        PtrEquals(const ELEMENT* compare) :
            element(compare)
        {
            // This constructor does nothing beyond member initialisation. 
        }

        /// @brief This operator returns the result of an equality comparison 
        /// on the object pointed to by @a param and the object provided at 
        /// construction time. 
        bool operator()(const void* param)
        {
            return(*static_cast<const ELEMENT*>(param) == *element); 
        }

    private:
        /// A pointer to the object to compare with. 
        const ELEMENT* element;
    };

    /// @brief This vector stores the contents of a DfsTPtrOrderedVector, and 
    /// provides part of its functionality. 
    std::vector<void*> stlVector;
};

#endif  // DFS_TPTR_ORDERED_VECTOR_H 
