// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWTValDlist replacement
//
// $Revision: 1.6 $
// $Log: DfsTValDlist.h,v $
// Revision 1.6  2004/09/01 13:50:05  nferguso
// DfsListBase is inherited as a dependent template instantiation - base
// class member accesses must be qualified with 'this'.
//
// Revision 1.5  2004/08/23 08:53:01  nferguso
// Extracted common list functionality from DfsTPtrDlist, DfsTPtrSlist,
// DfsTValDlist, and DfsTValSlist into a new DfsListBase base class,
// which is inherited by the four list classes.
//
// Revision 1.4  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.3  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.2  2004/03/26 15:07:40  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/03/15 12:31:20  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_TVAL_DLIST_H
#define DFS_TVAL_DLIST_H

// Includes from this library.
#include <DfsListBase.h>

/// @brief The DfsTValDlist class is a partial replacement of the 
/// RWTValDlist class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 * 
 * NB: Indexed element access has linear complexity for lists. Although 
 * DfsTValDlist provides sequential access optimisation, vector indices
 * are still faster. Non-sequential access should be avoided altogether. 
 * 
 * As DfsTValDlist methods should correspond exactly to the original 
 * RogueWave API, this documentation is sparse. Consult the RogueWave 
 * documentation for more details. 
 */ 
template<typename ELEMENT>
class DfsTValDlist : public DfsListBase<ELEMENT>
{
public:
    /// Creates an empty DfsTValDlist. 
    DfsTValDlist<ELEMENT>() : 
        DfsListBase<ELEMENT>()
    {
        // This constructor does nothing beyond base class initialisation. 
    }

    /// Copy constructor. 
    DfsTValDlist<ELEMENT>(const DfsTValDlist<ELEMENT>& source) : 
        DfsListBase<ELEMENT>(source)
    {
        // This constructor does nothing beyond base class initialisation. 
    }
    
    // The default destructor suffices for this class. 

    /// @brief Replaces the contents of this DfsTValDlist with a copy 
    /// of the @a source list's contents. 
    DfsTValDlist<ELEMENT>& operator=(const DfsTValDlist<ELEMENT>& source)
    {
        // Delegate assignment to the base list class. 
        this->DfsListBase<ELEMENT>::operator=(source); 
        return(*this); 
    }

    /// Returns the element at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    const ELEMENT& operator[](size_t index) const throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, this->listLength);

        // Move the last indexed element iterator to the new index, and 
        // return a reference to the element stored at that location. 
        return(*(this->moveLastElementPos(index)));
    }

    /// Inserts the given @a element at the end of this DfsTValDlist. 
    void append(const ELEMENT& element)
    {
        (this->stlList).push_back(element);
        ++(this->listLength); 
    }

protected:
    
private:
    
};

#endif  // DFS_TVAL_DLIST_H
