// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWTValOrderedVector replacement
//
// $Revision: 1.4 $
// $Log: DfsTValOrderedVector.h,v $
// Revision 1.4  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.3  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.2  2004/03/26 15:07:40  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/02/24 10:21:08  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_TVAL_ORDERED_VECTOR_H
#define DFS_TVAL_ORDERED_VECTOR_H

// Standard C++ includes
#include <vector>

// Includes from this library.
#include <DfsUtils.h>

/// @brief The DfsTValOrderedVector class is a partial replacement of the 
/// RWTValOrderedVector class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 * 
 * As DfsTValOrderedVector methods should correspond exactly to the original 
 * RogueWave API, this documentation is sparse. Consult the RogueWave 
 * documentation for more details. 
 */ 
template<typename ELEMENT>
class DfsTValOrderedVector
{
public:
    /// Creates an empty DfsTValOrderedVector. 
    DfsTValOrderedVector<ELEMENT>() 
    {
        // This constructor does not need to do anything. 
    }
    
    /// Copy constructor. 
    DfsTValOrderedVector<ELEMENT>(const DfsTValOrderedVector<ELEMENT>& source) : 
        stlVector(source.stlVector)
    {
        // This constructor does nothing beyond member initialisation. 
    }

    // The default destructor suffices for this class. 

    /// @brief Replaces the contents of this DfsTValOrderedVector with a copy 
    /// of the @a source vector's contents. 
    DfsTValOrderedVector<ELEMENT>& 
    operator=(const DfsTValOrderedVector<ELEMENT>& source)
    {
        if(&source != this)
        {
            stlVector = source.stlVector;
        }

        return(*this);
    }

    /// Appends the given @a element to the end of this DfsTValOrderedVector. 
    void append(const ELEMENT& element)
    {
        // Append the element. 
        stlVector.push_back(element);
    }

    /// Returns the element at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    ELEMENT& at(size_t index) throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());

        return(stlVector[index]);
    }

    /// Returns the element at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    const ELEMENT& at(size_t index) const throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());

        return(stlVector[index]);
    }

    /// Returns the number of entries in this DfsTValOrderedVector. 
    size_t length() const
    {
        return(stlVector.size());
    }

private:
    /// @brief This vector stores the contents of a DfsTValOrderedVector, and 
    /// provides part of its functionality. 
    std::vector<ELEMENT> stlVector;
};

#endif  // DFS_TVAL_ORDERED_VECTOR_H 
