// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWTValSlistIterator replacement
//
// $Revision: 1.6 $
// $Log: DfsTValSlistIterator.h,v $
// Revision 1.6  2004/09/24 15:06:46  nferguso
// Added operator+=(size_t).
//
// Revision 1.5  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.4  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.3  2004/04/05 12:19:32  nferguso
// Rationalised include hierarchy.
//
// Revision 1.2  2004/03/26 15:07:40  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/03/14 18:39:16  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_TVAL_SLIST_ITERATOR_H
#define DFS_TVAL_SLIST_ITERATOR_H

// Standard C includes. 
#include <cassert>

// Includes from this library.
#include <DfsConstants.h>
#include <DfsTValSlist.h>

// Required to resolve adverse include file ordering. 
template<typename ELEMENT> class DfsTValSlist;

/// @brief The DfsTValSlistIterator class is a partial replacement of the 
/// RWTValSlistIterator class. 
/** Only methods required by existing DFS code have been reimplemented, as 
 * new DFS code should be implemented using STL capabilities. 
 * 
 * As DfsTValSlistIterator methods should correspond exactly to the original 
 * RogueWave API, this documentation is sparse. Consult the RogueWave 
 * documentation for more details. 
 */ 
template<typename ELEMENT>
class DfsTValSlistIterator
{
public:
    /// Creates a new DfsTValSlistIterator for the given @a parent list. 
    DfsTValSlistIterator<ELEMENT>(DfsTValSlist<ELEMENT>& parent) : 
        parentList(&parent), 
        currentPos(parent.stlList.begin()),
        preBegin(true)
    {
        // This constructor does nothing beyond member initialisation. 
    }
    
    // The default destructor suffices for this class. 

    /// Advances this DfsTValSlistIterator to the next element. 
    /** Returns @c true if the iterator is referencing a defined value after 
     * it has been advanced, or @c false otherwise. 
     */
    DfsBoolean operator()()
    {
        if(preBegin)
        {
            // Conceptual advance to the first element. 
            preBegin = false; 
        }
        else if(currentPos != parentList->stlList.end())
        {
            // At least one position follows the current one - advance to 
            // the next position. 
            ++currentPos;
        }
        
        if(currentPos == parentList->stlList.end())
        {
            // The iterator has advanced beyond the last element. 
            return(false);
        }
        else
        {
            // The iterator is referencing an element. 
            return(true);
        }
    }

    /// Advances this DfsTValSlistIterator to the next element. 
    /** Returns @c true if the advance left the iterator referencing a 
     * defined value, or @c false otherwise. 
     *
     * NB: This is not strictly what the RogueWave documentation describes, 
     * but it matches their implementation. 
     */
    DfsBoolean operator++()
    {
        if(preBegin)
        {
            // Conceptual advance to the first element. 
            preBegin = false; 
        }
        else if(currentPos != parentList->stlList.end())
        {
            // At least one position follows the current one - advance to 
            // the next position. 
            ++currentPos;
        }

        return(currentPos != parentList->stlList.end());
    }

    /// @brief Acts as if operator++() had been called @a steps times, 
    /// returning the result of the last call. 
    DfsBoolean operator+=(size_t steps)
    {
        while((steps-- > 0) && this->operator++()) {}

        return(currentPos != parentList->stlList.end()); 
    }
    
    /// @brief Inserts the given @a element immediately after the element 
    /// currently referenced by this DfsTValSlistIterator. 
    /** NOTES: 
     *
     * - If the iterator is (conceptually) set to the undefined position just 
     * before the first element, the inserted element will become the first 
     * element in the list. 
     * - If the iterator references a defined value, the inserted element will 
     * be placed immediately after that value in the list. 
     * - <B>WARNING:</B> If the iterator is set to the undefined position just 
     * after the last element, the behaviour of this method is undefined. 
     * 
     * Although these points are not documented by RogueWave, this behaviour 
     * matches their implementation. 
     */
    void insertAfterPoint(const ELEMENT& element)
    {
        if(preBegin)
        {
            // Insert the element at the start of the list. 
            currentPos = parentList->stlList.insert(currentPos, element); 
        }
        else
        {
            // Insert the element immediately after the current position. 
            parentList->stlList.insert(
                ++(typename std::list<ELEMENT>::iterator(currentPos)),
                element);
        }

        // The parent list is now one element longer. 
        parentList->listLength += 1;

        // Reset the last indexed element position in the parent list. 
        parentList->lastElementPos = parentList->stlList.begin(); 
        parentList->lastElementIndex = 0; 
        parentList->lastElementValid = true;
    }

    /// @brief Returns the DfsTValSlist element currently referenced by this 
    /// DfsTValSlistIterator. 
    ELEMENT key()
    {
        return(*currentPos);
    }

    /// Resets this DfsTValSlistIterator to the state it had on creation. 
    void reset()
    {
        currentPos = parentList->stlList.begin(); 
        preBegin = true; 
    }
    
protected:
    
private:
    /// The DfsTValSlist that this DfsTValSlistIterator refers to. 
    DfsTValSlist<ELEMENT>* parentList;

    /// @brief This iterator refers to the parent DfsTValSlist's encapsulated 
    /// STL list; it already provides most of the required functionality. 
    typename std::list<ELEMENT>::iterator currentPos; 

    /// @brief True if this DfsTValSlistIterator is (conceptually) set to the 
    /// undefined position just before the first element. 
    bool preBegin; 

    /// @brief The copy constructor is not provided by RogueWave and should 
    /// never be used. 
    DfsTValSlistIterator<ELEMENT>(const DfsTValSlistIterator<ELEMENT>& source) 
    {
        assert(0);
    }

    /// @brief The assignment operator is not provided by RogueWave and should 
    /// never be used. 
    DfsTValSlistIterator<ELEMENT>& operator=(
        const DfsTValSlistIterator<ELEMENT>& source) 
    {
        assert(0);
    }
};

#endif  // DFS_TVAL_SLIST_ITERATOR_H
