// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWTValSortedVector replacement
//
// $Revision: 1.5 $
// $Log: DfsTValSortedVector.h,v $
// Revision 1.5  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.4  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.3  2004/03/26 15:07:40  nferguso
// Copyright years updated.
//
// Revision 1.2  2004/02/24 10:19:36  nferguso
// Minor comment correction.
//
// Revision 1.1  2004/02/20 17:47:01  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_TVAL_SORTED_VECTOR_H
#define DFS_TVAL_SORTED_VECTOR_H

// Standard C++ includes
#include <algorithm>
#include <functional>
#include <vector>

// Includes from this library.
#include <DfsUtils.h>

/// @brief The DfsTValSortedVector class is a partial replacement of the 
/// RWTValSortedVector class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 * 
 * DfsTValSortedVector is implemented using an underlying STL vector of 
 * pointers to void, which has two advantages: 
 *
 * - less copying overhead in the underlying vector during sorted insert()
 *   calls, and during removeFirst() calls. 
 * - only std::vector<void*> need be instantiated for any ELEMENT type. 
 *
 * The disadvantage is that DfsTValSortedVector must explicitly copy inserted 
 * elements, and delete removed ones. However, these operations are still kept 
 * hidden from the user, which is the main requirement. 
 *
 * NB: The RogueWave version of this template used by DFS does not accept a 
 * SORT parameter. Instead, it always sorts using less-than ordering, with 
 * the smallest item first. 
 *
 * As DfsTValSortedVector methods should correspond exactly to the original 
 * RogueWave API, this documentation is sparse. Consult the RogueWave 
 * documentation for more details. 
 */ 
template<typename ELEMENT, typename SORT = std::less<ELEMENT> >
class DfsTValSortedVector
{
public:
    /// Creates an empty DfsTValSortedVector. 
    DfsTValSortedVector<ELEMENT, SORT>() 
    {
        // This constructor does not need to do anything. 
    }
    
    /// Copy constructor. 
    DfsTValSortedVector<ELEMENT, SORT>(
        const DfsTValSortedVector<ELEMENT, SORT>& source)
    {
        // Copy and store all items from the source vector. 
        size_t numItems = source.length();

        for(size_t i = 0; i < numItems; ++i)
        {
            stlVector.push_back(
                static_cast<void*>(
                    new ELEMENT(*static_cast<ELEMENT*>(source.stlVector[i]))));
        }
    }

    /// @brief The contents of the underlying STL vector must be explicitly 
    /// deleted, hence this destructor. 
    ~DfsTValSortedVector<ELEMENT, SORT>() 
    {
        // Delete all items in this DfsTValSortedVector. 
        size_t numItems = length();

        for(size_t i = 0; i < numItems; ++i)
        {
            delete static_cast<ELEMENT*>(stlVector[i]);
        }
    }

    /// @brief Replaces the contents of this DfsTValSortedVector with a copy 
    /// of the @a source vector's contents. 
    DfsTValSortedVector<ELEMENT, SORT>& 
    operator=(const DfsTValSortedVector<ELEMENT, SORT>& source)
    {
        if(&source != this)
        {
            // Delete all items in this DfsTValSortedVector. 
            size_t numItems = length();

            for(size_t i = 0; i < numItems; ++i)
            {
                delete static_cast<ELEMENT*>(stlVector[i]);
            }

            stlVector.clear();          // Remove old pointers. 

            // Copy and store all items from the source vector. 
            numItems = source.length();

            for(size_t i = 0; i < numItems; ++i)
            {
                stlVector.push_back(
                    static_cast<void*>(
                        new ELEMENT(
                            *static_cast<ELEMENT*>(source.stlVector[i]))));
            }            
        }

        return(*this);
    }

    /// Returns the element at the given @a index. 
    /** This operator always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    const ELEMENT& operator[](size_t index) const throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());

        return(*static_cast<ELEMENT*>(stlVector[index]));
    }

    /// @brief Inserts the given @a element at the appropriate point determined
    /// by the sorting criterion. 
    /** @return Always returns @c true. */
    bool insert(const ELEMENT& element)
    {
        // Find the point where this element should be inserted. 
        std::vector<void*>::iterator 
            insertPoint(lower_bound(stlVector.begin(), 
                                    stlVector.end(), 
                                    static_cast<const void*>(&element),
                                    PtrSort()));

        // Insert the element. 
        stlVector.insert(insertPoint, 
                         static_cast<void*>(new ELEMENT(element)));

        return(true);
    }

    /// @brief Returns @c true if this DfsTValSortedVector is empty, @c false 
    /// otherwise. 
    bool isEmpty() const
    {
        return(stlVector.empty());
    }

    /// Returns the number of entries in this DfsTValSortedVector. 
    size_t length() const
    {
        return(stlVector.size());
    }

    /// Removes and returns the first element. 
    /** This method throws a DfsBoundsErr exception if the DfsTValSortedVector 
     * is empty. 
     */
    ELEMENT removeFirst() throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(0, stlVector.size());
        
        // Store the first element before... 
        ELEMENT element(*static_cast<ELEMENT*>(stlVector[0]));
        
        // ...deleting it and removing the old pointer from the vector. 
        delete static_cast<ELEMENT*>(stlVector[0]);
        stlVector.erase(stlVector.begin());

        return(element);
    }

private:
    /// @brief This class adapts the object sorting criterion SORT, so that it 
    /// can also be used to sort pointers to those objects. 
    class PtrSort : std::binary_function<const void*, const void*, bool> 
    {
    public:
        /// @brief This operator returns the result of applying the object 
        /// sorting criterion to the objects pointed to by its parameters. 
        bool operator()(const void* param1, const void* param2)
        {
            return(ObjSort(*static_cast<const ELEMENT*>(param1), 
                           *static_cast<const ELEMENT*>(param2)));
        }

    private:
        /// An instance of the object sorting criterion. 
        SORT ObjSort;
    };

    /// @brief This vector stores the contents of a DfsTValSortedVector, and 
    /// provides part of its functionality. 
    std::vector<void*> stlVector;
};

#endif  // DFS_TVAL_SORTED_VECTOR_H 
