// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWTValVector replacement
//
// $Revision: 1.4 $
// $Log: DfsTValVector.h,v $
// Revision 1.4  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.3  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.2  2004/03/26 15:07:40  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/02/24 14:25:14  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_TVAL_VECTOR_H
#define DFS_TVAL_VECTOR_H

// Standard C++ includes
#include <vector>

// Includes from this library.
#include <DfsUtils.h>

/// @brief The DfsTValVector class is a partial replacement of the 
/// RWTValVector class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 * 
 * As DfsTValVector methods should correspond exactly to the original 
 * RogueWave API, this documentation is sparse. Consult the RogueWave 
 * documentation for more details. 
 */ 
template<typename ELEMENT>
class DfsTValVector
{
public:
    /// Creates an empty DfsTValVector. 
    DfsTValVector<ELEMENT>() 
    {
        // This constructor does not need to do anything. 
    }

    /// Creates a DfsTValVector with the given number of @a elements. 
    DfsTValVector<ELEMENT>(size_t elements) :
        stlVector(elements)
    {
        // This constructor does nothing beyond member initialisation. 
    }
    
    /// Copy constructor. 
    DfsTValVector<ELEMENT>(const DfsTValVector<ELEMENT>& source) : 
        stlVector(source.stlVector)
    {
        // This constructor does nothing beyond member initialisation. 
    }

    // The default destructor suffices for this class. 

    /// @brief Replaces the contents of this DfsTValVector with a copy 
    /// of the @a source vector's contents. 
    DfsTValVector<ELEMENT>& 
    operator=(const DfsTValVector<ELEMENT>& source)
    {
        if(&source != this)
        {
            stlVector = source.stlVector;
        }

        return(*this);
    }

    /// Returns the element at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    const ELEMENT& operator[](size_t index) const throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());

        return(stlVector[index]);
    }

    /// Returns the element at the given @a index. 
    /** This method always performs bounds checking on the index, and throws 
     * a DfsBoundsErr exception for index values which are out of range. 
     */
    ELEMENT& operator[](size_t index) throw(DfsBoundsErr)
    {
        // Delegate range check on index. 
        DfsStd::checkIndex(index, stlVector.size());

        return(stlVector[index]);
    }

    /// Returns the number of entries in this DfsTValVector. 
    size_t length() const
    {
        return(stlVector.size());
    }

private:
    /// @brief This vector stores the contents of a DfsTValVector, and 
    /// provides part of its functionality. 
    std::vector<ELEMENT> stlVector;
};

#endif  // DFS_TVAL_VECTOR_H 
