// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2003 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWxmsg replacement
//
// $Revision: 1.4 $
// $Log: DfsXmsg.h,v $
// Revision 1.4  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.3  2004/07/29 14:59:02  nferguso
// DfsXmsg now inherits from std::exception.
//
// Revision 1.2  2003/12/19 15:07:32  nferguso
//
// String-handling classes can be built and used, although some things remain
// to be done. Most significantly:
//
// - methods working with iostreams are not fully compliant with RogueWave
// - STL string reserve() memory management is incomplete
//
// No non-string RogueWave classes have been replaced yet.
//
// Revision 1.1.1.1  2003/11/28 14:21:16  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_XMSG_H
#define DFS_XMSG_H

// C++ STL includes. 
#include <string>
#include <exception>

/// The DfsXmsg class is a partial replacement of the RWxmsg class. 
/** Only methods required by existing DFS code have been reimplemented, 
 * as new DFS code should be implemented using STL capabilities. 
 *
 * The original RWxmsg class was based on <I>xmsg</I>, which was proposed 
 * as a base class for exceptions in the draft ISO C++ standard. However, 
 * <I>xmsg</I> was never adopted. Instead, the final standard specified the 
 * <I>exception</I> class, with a different interface. DfsXmsg provides both 
 * interfaces, so it can be used either as a RogueWave exception, or as an 
 * STL one. This allows CPPUnit-based tests to provide more information in 
 * the event of an error. 
 *
 * NOTE: The RogueWave RWxmsg has a default assignment operator, which should 
 * have been disabled, as it is unsafe. The DfsXmsg implementation allows the 
 * default assignment operator to work safely and correctly. 
 *
 * As DfsXmsg methods should correspond exactly to the original RogueWave 
 * API, this documentation is sparse. Consult the RogueWave documentation 
 * for more details. 
 */ 
class DfsXmsg : public std::exception
{
public:
    /// Constructs a new DfsXmsg exception with the given @a reason string. 
    DfsXmsg(const char* reason) :
        stlString(reason)
    {
        // This constructor does nothing beyond member initialisation. 
    }

    /// Virtual destructor for safety of inheritance, eg. by DfsBoundsErr. 
    virtual ~DfsXmsg() throw()
    {
        // This destructor does not need to do anything. 
    }

    /// @brief Provides access to the reason string held by this exception 
    /// through the RogueWave interface. 
    /** @return A string describing the reason for the exception. */
    virtual const char* why() const
    {
        return(stlString.c_str());
    }

    /// @brief Provides access to the reason string held by this exception 
    /// through the STL interface. 
    /** @return A string describing the reason for the exception. */
    virtual const char* what() const throw()
    {
        return(stlString.c_str());
    }

protected:
    /// This STL string stores the reason string for the exception. 
    std::string stlString;

private:
    
};

#endif // DFS_XMSG_H
