// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.9 $
// $Log: FileIO.h,v $
// Revision 1.9  2006/10/12 13:49:12  jknudstr
// -
//
// Revision 1.8  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef FILEIO_H
#define FILEIO_H


// C includes
#include <sys/types.h>
#include <sys/stat.h>

// C++ includes
#include <exception>

// STL includes
#include <string>
#include <vector>
#include <sstream>

// Local includes.
#include <ArcSysEx.h>


#define FILEIO_SEP '/'
#define FILEIO_DEF_MODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH)


using namespace std;


/// Class to handle basic file manipulation.
class FileIO
{
  public:
    /// Constructor method initializing the object from a filename.
    FileIO(const string& fileName = "",
	   const bool    read = true,
	   const bool    write = true);

    /// Contructor method initializing the object from a list of path
    /// components, used internally to build up the associated filename.
    FileIO(const vector<string>& pathList,
	   const bool            read = true,
	   const bool            write = true);
    
    /// Destructor method.
    ~FileIO();

    /// Reset the object making it ready to handle a new, associated file.
    FileIO& clear(void);

    /// Add a path to the internal filename.
    FileIO& addPath(const string& path);

    /// Set the value of the internal filename.
    FileIO& setFileName(const string& fileName);
    
    /// Build up a filename according to the path components in the vector
    /// and set the value of the internal filename.
    FileIO& buildFileName(const vector<string>&  pathList);

    /// Return true if the path of the associated filename exists, otherwise
    /// false is returned.
    bool    pathExists(void);

    /// Return true if the associated file exists, otherwise fals is returned.
    bool    exists(void);

    /// Open the file according to the read/write flags. If the file does
    /// not exist, it is created.
    FileIO& open(const string&  fileName,
		 const bool     read,
		 const bool     write);

    /// Close the connection to the associated file.
    FileIO& close(void);

    /// Get the complete name of the associated file/path.
    string& getCompleteName(void);

    /// Get the path of the associated file.
    string& getPath(void);

    /// Get the basename (name without path) of the associated file.
    string& getBaseName(void);

    /// Load the contents of the associated file into the referenced buffer.
    FileIO& load(stringstream&  buf);

    /// Check the existance of the path of the associated filename. If it 
    /// does not exist, it is created.
    FileIO& checkCreateDir(const string& path = "");

    /// Write the contents of the referenced buffer into the associated file.
    /// If the file does not exist, it is created.
    FileIO& write(const string&  buf);

    /// Write the contents of the referenced buffer into the associated file.
    /// If the file does not exist, it is created.
    FileIO& write(const char*  buf);

    /// Rename the associated file to the target filename specified.
    FileIO& rename(const string&  newPath); 

    /// Get the extension of the associated internal filename.
    string& getExtension(void);

    /// Check and set if necessary the extension of the associated file.
    FileIO& setExtension(const string&  ext);

    /// Execute a match based on the associated filename (match pattern).
    /// Matching entries are added in the referenced vector.
    FileIO& match(vector<string>&  fileList,
		  const int        maxMatches = 1000);

    /// Return true if the path associated with the object is a file.
    bool isFile(void);

    /// Remove the associated file.
    FileIO& remove(const bool force = true);

  protected:

  private:
    mode_t       mode_;    

    FileIO&      getExtension(string&  ext);

    string       completeName_;
    string       ext_;
    string       path_;
    string       baseName_;

    int          fd_;
    bool         read_;
    bool         write_;

    stringstream buf_;
};

#endif // FILEIO_H

// EOF

