// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observator
// LICENSE:   ESO internal use only
//
// PROJECT:   VLT Data Flow Syste
// AUTHOR:    Christophe Moins- ESO/SDD/DF
// SUBSYSTEM: arcsys
//
// $Revision: 1.4 $
// $Log: GenObsBlock.h,v $
// Revision 1.4  2008/09/11 08:31:07  vforchi
// fixed compilation problems with gcc 4.2.4
//
// Revision 1.3  2007/05/04 11:56:40  cmoins
// removal of storeData, updateProgBlock and getMaxTime implementation
//
// Revision 1.2  2007/04/26 12:56:17  cmoins
// adding of a StorageService temaplte parameter
//
// Revision 1.1  2007/04/25 15:57:17  cmoins
// implementation of a general observation block
//
// 
//------------------------------------------------------------------------------

#ifndef GENOBSBLOCK_H
#define GENOBSBLOCK_H

#include <string>
#include <sstream>
#include <DfsLog.h>
#include <iomanip>
#include <boost/tokenizer.hpp>
#include "ObsBlock.h"
#include "RawDataProduct.h"

using namespace std;

/// General observation block template class.
/*
 * A general observation block contains science and calibration files. It will
 * use an ObservationService and a Storage template parameter to remain 
 * independent of the data retrieval layer and of the storage service layer.
 */
template <class ObservationService,class StorageService>
class GenObsBlock : public ObsBlock<StorageService>
{
public :

#if defined UNIT_TEST
  friend class TestGenObsBlock;
  friend class TestObsProgPackage;
#endif


  /// \brief Constructor
  /**
   * \param blockId block identifier
   * \param connAlias data repository connection parameter
   */
  GenObsBlock(int blockId,
	   string connAlias):
    ObsBlock<StorageService>(blockId,connAlias)
  {}

  /// \brief Destructor
  ~GenObsBlock()
  {}

  /// \brief fetches a list of files associated to the block.
  /**
   * \param startTime start time of the files retrieval
   * \param startTimeAdjustment correction to apply on start time for 
   *                            calibration files retrieval
   * \param endTime end time of the files retrieval
   * \param endTimeAdjustment   correction to apply on end time for 
   *                            calibration files retrieval
   * \param rlistOfUsedInstruments list of instruments that should be 
   *                               excluded to retrieve calibration files
   * \return 0 if successful, -1 otherwise 
   */
  virtual int fetchData(double  startTime,
			double  startTimeAdjustment,
			double  endTime,
			double  endTimeAdjustment,
			string& rlistOfUsedInstruments);

  /// \brief instantiates a copy of the object
  /// returns the address of the copy
  virtual ObsBlock<StorageService>* clone() const;
 
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Fetches a list of files associated to a general observation the  block
//------------------------------------------------------------------------------
template <class ObservationService, class StorageService>
int
GenObsBlock<ObservationService,StorageService>::
fetchData(double startTime,
	  double  startTimeAdjustment,
	  double endTime,
	  double  endTimeAdjustment,
	  string& rlistOfUsedInstrument) 
{
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;

  boost::char_separator<char> sep(",");
  int retStatus = 0;
  int blockId = ObsBlock<StorageService>::getBlockId();
  string connAlias = ObsBlock<StorageService>::getAlias();
  ObservationService observationService(connAlias);  

  DFS_TRACE("GenObsBlock<ObservationService>::fetchData");

  int nbCalibFiles=0;
  RawDataProduct::ResultSet dataFileSet = ObsBlock<StorageService>::getDataFileSet();

  int nbScienceFiles = 
    observationService.findObsBlockScienceFiles(blockId,
						startTime,
						endTime,
						dataFileSet);
  if (nbScienceFiles >= 0)
    {
      double minTime=0;
      double maxTime=0;
      string obsBlockUsedInstrument;
      for (RawDataProduct::pRawDataProduct  pdataFile = dataFileSet.begin();
	   pdataFile != dataFileSet.end();pdataFile++)
	{
	  // looking for min and max times for time refining afterwards
	  if (pdataFile == dataFileSet.begin())
	    {
	      minTime = pdataFile->mjdObs_;
	      maxTime = pdataFile->mjdObs_;
	    }
	  else if (pdataFile->mjdObs_ < minTime)
	    minTime = pdataFile->mjdObs_;
	  else if (pdataFile->mjdObs_ > maxTime)
	    maxTime = pdataFile->mjdObs_;

	  // Getting the list of used instruments
	  if (obsBlockUsedInstrument.find(pdataFile->insId_) == string::npos)
	    {
	      if (!obsBlockUsedInstrument.empty())
		obsBlockUsedInstrument += ",";
	      obsBlockUsedInstrument += pdataFile->insId_;
	    } 
	}
      // Proceeds to time refining 
      if (!obsBlockUsedInstrument.empty())
	{
	  // some sciences files have been retrieved for this block.
	  // with the list of the instruments they are associated to,
	  // calibration files are retrieved using the list of files
	  // they are associated to.
	  // Proceeds to time refining
	  // if the time adjustment is below one millisecond, 
	  // no adjustment is performed and the same time as the calibration
	  // files is used. 
	  if (fabs(startTimeAdjustment) > DPPackerDef::ONE_MS)
	    startTime = minTime + startTimeAdjustment;
	  if (fabs(endTimeAdjustment) > DPPackerDef::ONE_MS)
	    endTime = maxTime + endTimeAdjustment;
	  nbCalibFiles = 
	    observationService.findCalibFilesByInst(obsBlockUsedInstrument,
						    startTime,
						    endTime,
						    rlistOfUsedInstrument,
						    dataFileSet);
	  // update of the list of instruments for which 
	  // calibration files have been retrieved
	  tokenizer tok(obsBlockUsedInstrument,sep);
          const int MS_ACCURACY = 15; // 15 digits to get ms accuracy
	  stringstream startTimeStr;
	  stringstream endTimeStr;
	  startTimeStr << setprecision(MS_ACCURACY) << startTime;
	  endTimeStr << setprecision(MS_ACCURACY) << endTime;
	  for(tokenizer::iterator beg=tok.begin(); beg!=tok.end();++beg)
	    {
	      rlistOfUsedInstrument += 
		*beg + " " + startTimeStr.str() + " " + endTimeStr.str();
	      if (beg != tok.end())
		{
		  rlistOfUsedInstrument+= ",";
		}
	    }  
	}
      else
	{ 
	  // No science files was found, only calibration files associated
	  // to this obs id are retrieved.
	  startTime = startTime + startTimeAdjustment;
	  endTime = endTime + endTimeAdjustment;
	  nbCalibFiles = 
	    observationService.findObsBlockCalibFiles(blockId,
						      startTime,
						      endTime,
						      rlistOfUsedInstrument,
						      dataFileSet);
	}
	
    }
  if ((nbScienceFiles > 0 || nbCalibFiles > 0))
    {
      DFS_INFO("Files retrieval succeeded for %d",blockId);
      ObsBlock<StorageService>::setDataFileSet(dataFileSet);
    }
  else
    {
      DFS_ERROR("Files retrieval failed for %d",blockId);
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// instantiates a copy of the object
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
ObsBlock<StorageService>*
GenObsBlock<ObservationService,StorageService>::clone() const
{
  return new GenObsBlock<ObservationService,StorageService>(*this);
}
#endif /* GENOBSBLOCK_H */
