// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.1 $
// $Log: KeyDAO.h,v $
// Revision 1.1  2007/04/25 15:00:17  cmoins
// creation of a table key DAO class
//
// 
//------------------------------------------------------------------------------

#ifndef KEY_DAO_H
#define KEY_DAO_H

#include <string>
using namespace std;

/// Data Access Object associated to media request identifiers
/**
 * This class allows to get a unique identifier. It relies on a table
 * that holds a key value which is incremented each time a request 
 *  for a new identifier is done.
 */
template <class ConnectionHandle> class KeyDAO
{
public:
 
  /// \brief Constructor
  /// \param keyTable identifier table
  KeyDAO(string keyTable);

  /// \brief Destructor
  ~KeyDAO(){}
  
  /// \brief gets a media request identifier from the database
  /**
   *  \param rconnectionHandle data repository connection handle
   *  \return an identifier if successful, -1 otherwise
   */
  int getMediaId(const ConnectionHandle& rconnectionHandle) const;

private:
  string keyTable_; /// table storing identifiers
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
//------------------------------------------------------------------------------
template <class ConnectionHandle>
KeyDAO<ConnectionHandle>::
KeyDAO(string keyTable):
  keyTable_(keyTable)
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gets a media request identifier from the keys table
// This operation is inserted into a transaction to guarantee that a unique 
// identifier will be generated.
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
KeyDAO<ConnectionHandle>::
getMediaId(const ConnectionHandle& rconnectionHandle) const
{
  int mediaId = -1;

  DFS_TRACE("KeyDAO<ConnectionHandle>::getMediaId");

 // beginning of the transaction
  rconnectionHandle.beginTransaction();

  string sqlStatement = "update  " + keyTable_ + " "
   +  "set   keyValue = keyValue + 1 "
   + "where keyName = 'mediaId'";
  typename ConnectionHandle::ResultSet res = 
    rconnectionHandle.execute(sqlStatement);
  if (res.commandSucceeded())
    {
      sqlStatement = "select keyValue from "
        +  keyTable_ + " "
        +  "where keyName = \"mediaId\"";
      typename ConnectionHandle::ResultSet res = 
	rconnectionHandle.execute(sqlStatement);
      if (res.commandSucceeded() && res.hasResultSet())
        {
	  const int MEDIA_ID_INDEX = 1;
	  if  (res.next())
	    {
	      mediaId = res.getLongValue(MEDIA_ID_INDEX);
	    }
	}
    }   
  if (mediaId != -1)
     rconnectionHandle.commitTransaction();
  else
    {
      rconnectionHandle.rollbackTransaction();
      DFS_ERROR("Could not get an identifier from the database");
    }
  return mediaId;
}
#endif // KEY_DAO_H
// -*-++-*
