// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.5 $
// $Log: MagDiskFileDAO.h,v $
// Revision 1.5  2008/02/01 14:59:45  mpadovan
// The new remove() method now takes also the host name parameter.
//
// Revision 1.4  2008/01/29 16:07:43  mpadovan
// Added the method remove().
//
// Revision 1.3  2007/07/25 07:27:21  cmoins
// doxygen comments correction
//
// Revision 1.2  2007/07/04 12:37:09  cmoins
// an error message is sent if no location is found
//
// Revision 1.1  2007/05/02 09:35:23  cmoins
// adding of a DAO class to MagDiskFile objects
//
// 
//------------------------------------------------------------------------------

#ifndef MAG_DISK_FILE_DAO_H
#define MAG_DISK_FILE_DAO_H

#include <string>
#include <DfsLog.h>
#include "MagDiskFile.h"

/// Data Access Object associated to MagDiskFiles objects.
/**
 *  This class allows to retrieve files on magnetic disks associated
 *  to file identifiers 
 *  It will rely on two tables to fetch information :
 * - one storing data about files stored on the magnetic disk
 * - one storing data about the magnetic disk
 *
 * It has a ConnectionHandle template parameter that represents 
 * the type of connection handle to the data repository so that it remains
 * indepedent of any connection system.
 */
template <class ConnectionHandle> class MagDiskFileDAO
{
public:

  /// \brief Constructor
  /**
   * \param mdFilesTable table storing data about files on magnetic disk
   * \param mdsTable table storing data about magnetic disks
   */
  MagDiskFileDAO(string mdFilesTable,
		 string mdsTable);  

  /// \brief Finds the location of a file
  /// \param rdiskFile file whose location shall be retrieved
  /// \param rconnectionHandle database connection handle
  /// \return 0 if the operation was successful, -1 otherwise
  int findFileLocation(MagDiskFile& rdiskFile,
		       const ConnectionHandle& rconnectionHandle) const;

  /// \brief Removes the record of the file with the given id and host name.
  /// \param fileId id of the file to remove
  /// \param hostName name of the host where the file is located.
  /// \param rconnectionHandle database connection handle
  /// \return the number of records removed, -1 in case of error
  int remove(const std::string& fileId,
             const std::string& hostName,
             const ConnectionHandle& rconnectionHandle) const;

private:

  static const string MDF_FILE_ID_FIELD;
  static const string MDF_DISK_NAME_FIELD;
  static const string MDF_FILE_NAME_FIELD;
  static const string MDS_DISK_NAME_FIELD;
  static const string MDS_MOUNT_POINT_FIELD;
  static const string MDF_SIZE_FIELD;
  static const string MDF_UNCOMP_SIZE_FIELD;
  static const string MDF_STATUS_FIELD;
  static const string MDS_MOUNTED_FIELD;
      
  /// table storing information about files on disks
  string mdFilesTable_;        
  /// table storing information about disks 
  string mdsTable_;    

};

// static constant creation
template <class ConnectionHandle> 
const string MagDiskFileDAO<ConnectionHandle>::
MDF_FILE_ID_FIELD   = "f.file_id";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDF_DISK_NAME_FIELD = "f.diskname";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDF_FILE_NAME_FIELD     = "f.fname";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDS_DISK_NAME_FIELD     = "m.diskname";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDS_MOUNT_POINT_FIELD   = "m.mount_point";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDF_SIZE_FIELD          = "f.fsize";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDF_UNCOMP_SIZE_FIELD   = "f.uncomp_fsize";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDF_STATUS_FIELD        = "f.status";

template <class ConnectionHandle>
const string MagDiskFileDAO<ConnectionHandle>::
MDS_MOUNTED_FIELD       = "m.mounted";


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
//------------------------------------------------------------------------------
template <class ConnectionHandle>
MagDiskFileDAO<ConnectionHandle>::
MagDiskFileDAO(string mdFilesTable,
	       string mdsTable):
  mdFilesTable_(mdFilesTable),
  mdsTable_(mdsTable)
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Finds the location of a set of files
// Return 0 in case of success, -1 otherwise
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
MagDiskFileDAO<ConnectionHandle>::
findFileLocation(MagDiskFile& rdiskFile,
		 const ConnectionHandle& rconnectionHandle) const
{
  const string CURRENT_STATUS = "C"; // status where files shall be 

  bool hasResultSet;
  int retStatus = 0;
  string select;
  string from;
  string where;
  string sqlStatement;
  typename ConnectionHandle::ResultSet res ;
  typename ConnectionHandle::ListOfFieldIndex lofi;
       
  DFS_TRACE("MagDiskFileDAO<ConnectionHandle>::findFileLocation");

  select = "select " + MDF_FILE_NAME_FIELD
    + "," + MDF_SIZE_FIELD 
    + "," + MDF_UNCOMP_SIZE_FIELD
    + "," + MDS_DISK_NAME_FIELD 
    + "," + MDS_MOUNT_POINT_FIELD;
  from = " from " + mdsTable_ + " m, " + mdFilesTable_ + " f";
  where = " where " + MDF_FILE_ID_FIELD + "='" + rdiskFile.fileId_ + "'"
    + " AND " + MDF_STATUS_FIELD + "= \'" + CURRENT_STATUS  +"\'"
    + " AND " + MDS_MOUNTED_FIELD + "= \'Y\'"
    + " AND " + MDF_DISK_NAME_FIELD + " = " + MDS_DISK_NAME_FIELD;
  sqlStatement = select + " " + from + " " + where;
  if ((rconnectionHandle.executeSelect(sqlStatement,
				       res,
				       lofi) != -1))
    {
      retStatus = -1;
      hasResultSet = res.hasResultSet();
      if (hasResultSet)
	{
	  while( res.next())
	    {
	      // At least one location was retrieved
	      retStatus = 0;
	      rdiskFile.fname_ = 
		rconnectionHandle.getStringValue(res,
						 lofi[MDF_FILE_NAME_FIELD]);
	      rdiskFile.fsize_ =
		rconnectionHandle.getLongValue(res,
					       lofi[MDF_SIZE_FIELD]);
	      rdiskFile.uncompFSize_ =
		rconnectionHandle.getLongValue(res,
					       lofi[MDF_UNCOMP_SIZE_FIELD]);
	      rdiskFile.magDisk_.diskName_ =
		rconnectionHandle.getStringValue(res,
						 lofi[MDS_DISK_NAME_FIELD]);
	      rdiskFile.magDisk_.mountPoint_ =
		rconnectionHandle.getStringValue(res,
						 lofi[MDS_MOUNT_POINT_FIELD]);
	    }
	}
    }
  else
    retStatus = -1;
  if (retStatus)
    {
      DFS_ERROR("Could not retrieve any file location");
    }
  return retStatus;
}

template <class ConnectionHandle>
int MagDiskFileDAO<ConnectionHandle>::remove(
  const std::string& fileId,
  const std::string& hostName,
  const ConnectionHandle& rconnectionHandle) const
{
  DFS_TRACE("MagDiskFileDAO<ConnectionHandle>::remove");
  
  const std::string query = "delete from asto..mdfiles where " \
                            "file_id='" + fileId + "' and " +
                            "diskname='" + hostName + "'";

  typename ConnectionHandle::ResultSet res = rconnectionHandle.execute(query);

  int rowCount = -1;

  if( res.commandSucceeded() )
  {
    rowCount = res.getRowCount();
  }
  
  return rowCount;
}


#endif // MAG_DISK_FILE_DAO_H
// -*-++-*-
