// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.7 $
// $Log: MediaDAO.h,v $
// Revision 1.7  2007/09/17 13:35:52  cmoins
// source_id set to empty string when a package is created
//
// Revision 1.6  2007/07/24 13:56:50  cmoins
// request type becomes a user one
//
// Revision 1.5  2007/07/04 12:21:24  cmoins
// insert returns the number of inserted rows
//
// Revision 1.4  2007/07/04 11:44:52  cmoins
// no row retrieved is not considered as an error situation any more
//
// Revision 1.3  2007/05/29 07:16:52  cmoins
// user media requests become archive ones to be handled by astoControl
//
// Revision 1.2  2007/05/15 09:57:51  cmoins
// replacement of info logging messages by debug ones
//
// Revision 1.1  2007/04/25 15:09:34  cmoins
// cretion of a DAO class associated to the Media class
//
// 
// 
//------------------------------------------------------------------------------

#ifndef MEDIA_DAO_H
#define MEDIA_DAO_H

#include <string>
#include <sstream>
#include <DfsLog.h>
#include "Media.h"

using namespace std;

/// Data Access Object associated to Media objects.
/**
 * This class allows to insert or retrieve media creation request related 
 * informations into the asto database. It will rely on a table storing 
 * data about media requests.
 */
template <class ConnectionHandle> class MediaDAO
{
public:
 
  /// \brief Constructor that sets the media request table
  /// \param mediaTable media request table
  MediaDAO(string mediaTable);

  /// \brief Destructor
  MediaDAO(){}
  
  /// \brief insert a media creation request  inside the database.
  /**
   * \param rmedia media request to be inserted
   * \param rconnectionHandle data repository connection handle
   * \return the number of inserted rows if successful, -1 otherwise
   */
  int insert(const Media& rmedia,
	     const ConnectionHandle& rconnectionHandle) const;

  /// \brief retrieves the data of a media from its request name
  /**
   * \param mediaName name of the media request
   * \param rmedia  media data
   * \param rconnectionHandle data repository connection handle
   * \return the number of retrieved media if successful, -1 otherwise
   */
  int findMedia(string mediaName,
		Media& rmedia,
		const ConnectionHandle& rconnectionHandle) const;
 
private:

  /// Fields used in the query
  static const string MEDIA_NAME_FIELD;
  static const string MEDIA_ID_FIELD;
  static const string SOURCE_ID_FIELD;
  static const string DATE_REQUESTED_FIELD;
  static const string MEDIA_REQUEST_TYPE_FIELD;
  static const string MEDIA_TYPE_FIELD;
  static const string NUM_COPIES_FIELD;
  static const string UNIT_NAME_FORMAT_FIELD;
  static const string STATUS_FIELD;
  static const string HOST_FIELD;

  string mediaTable_; // table storing data about media requests
};

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::MEDIA_NAME_FIELD = "media_request_name";

template <class ConnectionHandle> const string 
MediaDAO<ConnectionHandle>::MEDIA_ID_FIELD = "media_request_id";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::SOURCE_ID_FIELD = "source_id";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::DATE_REQUESTED_FIELD = "date_requested";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::MEDIA_REQUEST_TYPE_FIELD = "media_request_type";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::MEDIA_TYPE_FIELD = "media_type";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::NUM_COPIES_FIELD = "num_copies";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::UNIT_NAME_FORMAT_FIELD = "unit_name_format";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::STATUS_FIELD = "status";

template <class ConnectionHandle> const string
MediaDAO<ConnectionHandle>::HOST_FIELD = "host";

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
//------------------------------------------------------------------------------
template <class ConnectionHandle>
MediaDAO<ConnectionHandle>::
MediaDAO(string mediaTable):
  mediaTable_(mediaTable)
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// insert informations related to a media request creation inside the database
// Return 0 if the operation was successful, -1 otherwise
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
MediaDAO<ConnectionHandle>::
insert(const Media& rmedia,
       const ConnectionHandle& rconnectionHandle) const
{
  const int UNIT_NAME_NB_DIGIT = 3;
  const string IN_PROGRESS_STATUS = "I";
  const string USER_REQUEST_TYPE = "USER";
  const string NO_SOURCE_ID = "";

  int retStatus = 0;

  DFS_TRACE("MediaDAO<ConnectionHandle>::insert");

  string insertInto = "insert into " + mediaTable_ + " ";
  string columnList = "(" + MEDIA_ID_FIELD + "," + MEDIA_NAME_FIELD
    + "," + SOURCE_ID_FIELD + "," + DATE_REQUESTED_FIELD 
    + "," + MEDIA_REQUEST_TYPE_FIELD + "," + MEDIA_TYPE_FIELD  
    + "," + NUM_COPIES_FIELD + "," + UNIT_NAME_FORMAT_FIELD 
    + "," + STATUS_FIELD + "," + HOST_FIELD + ")";

  //  the name of a storage unit  will be made of the package name
  // plus a number of three digits
  stringstream unitNameFormat;
  unitNameFormat << rmedia.mediaRequestName_
		 << "%." << UNIT_NAME_NB_DIGIT << "d";
  stringstream insertValues;  
  insertValues << "values(" << rmedia.mediaRequestId_ 
	       << ",'" << rmedia.mediaRequestName_ 
	       << "','" << NO_SOURCE_ID << "'"
	       <<  ",getdate(),'" << USER_REQUEST_TYPE << "'" 
               << ",'" << rmedia.mediaType_ << "'"
	       << "," << rmedia.numCopies_  
	       << ",'" << unitNameFormat.str() << "'"
	       << ",'" << IN_PROGRESS_STATUS << "','" << rmedia.host_ << "')";
  string sqlStatement = insertInto + columnList + insertValues.str();
  typename ConnectionHandle::ResultSet res = 
    rconnectionHandle.execute(sqlStatement);

  if (res.commandSucceeded())
    {
      retStatus = res.getRowCount();
      //mediaRequest data insertion in the database was successful 
      DFS_DEBUG("Media Request creation successful for %s",
	         rmedia.mediaRequestName_.c_str());
    }
  else
    {
      DFS_ERROR("Media Request creation failed for %s",
		rmedia.mediaRequestName_.c_str());
      retStatus = -1;
    }
  return retStatus;
} 


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// retrieves the data of a media from its request name
// if several identifier are associated to a media request name, only the data
// associated to the highest one are selected
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
MediaDAO<ConnectionHandle>::  
findMedia(string mediaName,
	  Media& rmedia,
	  const ConnectionHandle& rconnectionHandle) const
{
  int retStatus = 0;
  int curMediaId;
  int mediaIdMax = -1;
  string dateRequested;
  string mediaRequestType;
  string mediaType;
  int numCopies=0;
  string unitNameFormat;
  string status;
  string host;

  typename ConnectionHandle::ResultSet res;
  typename ConnectionHandle::ListOfFieldIndex listOfFieldIndex;

  // SQL statement building
  string select = "select  " + MEDIA_NAME_FIELD + "," + MEDIA_ID_FIELD 
    + "," + DATE_REQUESTED_FIELD + "," + MEDIA_REQUEST_TYPE_FIELD
    + "," + MEDIA_TYPE_FIELD + "," + NUM_COPIES_FIELD 
    + "," + UNIT_NAME_FORMAT_FIELD + "," + STATUS_FIELD + "," + HOST_FIELD + " ";
 
  string from   =  "from " + mediaTable_ + " ";
  string where  =  "where " + MEDIA_NAME_FIELD + "='" + mediaName + "'";
  string sqlStatement  = select + from + where;

  if (rconnectionHandle.executeSelect(sqlStatement,
				      res,
				      listOfFieldIndex) != -1)
    {
      bool hasResultSet = res.hasResultSet();
     
      if (hasResultSet)
	{
	  while(res.next())
	    {
	      curMediaId =  
		rconnectionHandle.getLongValue(res,
					       listOfFieldIndex[MEDIA_ID_FIELD]);
	      if (curMediaId > mediaIdMax)
		{
		  mediaIdMax = curMediaId;
		  dateRequested = 
		    rconnectionHandle.getStringValue(res,
						     listOfFieldIndex[DATE_REQUESTED_FIELD]);
		  mediaRequestType = 
		    rconnectionHandle.getStringValue(res,
						     listOfFieldIndex[MEDIA_REQUEST_TYPE_FIELD]);
		  mediaType = 
		    rconnectionHandle.getStringValue(res,
						     listOfFieldIndex[MEDIA_TYPE_FIELD]);
		  numCopies = 
		    rconnectionHandle.getLongValue(res,
						   listOfFieldIndex[NUM_COPIES_FIELD]);
		  unitNameFormat =
		    rconnectionHandle.getStringValue(res,
						     listOfFieldIndex[UNIT_NAME_FORMAT_FIELD]);
		  status = 
		    rconnectionHandle.getStringValue(res,
						     listOfFieldIndex[STATUS_FIELD]);
		  host = 
		    rconnectionHandle.getStringValue(res,
						     listOfFieldIndex[HOST_FIELD]);
		}
            }
	}
    
      if (mediaIdMax == -1)
	{
	  DFS_DEBUG("Could not retrieve any media data for %s",
		    mediaName.c_str());
	  retStatus = 0;
	}
      else
	{
	  rmedia.mediaRequestName_ = mediaName;
	  rmedia.mediaRequestId_ = mediaIdMax;
	  rmedia.dateRequested_ = dateRequested;
	  rmedia.mediaRequestType_ = mediaRequestType;
	  rmedia.mediaType_ = mediaType;
	  rmedia.numCopies_ = numCopies;
	  rmedia.unitNameFormat_ = unitNameFormat;
	  rmedia.status_ = status[0];
	  rmedia.host_ = host;
	  int nbMedia = 1;
	  retStatus = nbMedia;
	}
    }
  else
    {
      DFS_DEBUG("Error when executing %s",sqlStatement.c_str());
      retStatus = -1;
    }
  return retStatus;
}

#endif // MEDIA_DAO_H
// -*-++-*
