// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.1 $
// $Log: MediaTypeDAO.h,v $
// Revision 1.1  2007/07/05 09:18:36  cmoins
// DFS04059 : creation of a media type DAO to handle the media type
//
// 
//------------------------------------------------------------------------------

#ifndef MEDIA_TYPE_DAO_H
#define MEDIA_TYPE_DAO_H

#include <string>
#include <iostream>
#include <sstream>
#include <DfsLog.h>
#include "MediaType.h"

using namespace std;

/// Data Access Object associated to MediaType objects.
/**
 * This class allows to insert or retrieve media type related informations 
 * into the asto database. It will rely on a table storing informations
 * about the different media type that can be used to store data.
 */
template <class ConnectionHandle> class MediaTypeDAO
{
public:
 
  /// \brief Constructor that sets the media type table
  /// \param mediaTypeTable stores data about the different types of media
  MediaTypeDAO(string mediaTypeTable);

  /// \brief Destructor
  ~MediaTypeDAO(){}

  /// \brief retrieves the whole set of available media types.
  /**
   * \param rmediaTypeSet set of available media types
   * \param rconnectionHandle data repository connection handle
   * \return the number of retrieved media types if successful, -1 otherwise 
   */
  int findAllMediaType(MediaType::ResultSet& rmediaTypeSet,
		       const ConnectionHandle& rconnectionHandle) const;

private:

  /// Set of fields used in the query
  static const string MEDIA_TYPE_FIELD;
  static const string CAPACITY_FIELD;
  static const string THRESHOLD_FIELD;
  static  const string SPLIT_SIZE_FIELD; 
  static  const string BLOCK_SIZE_FIELD;
  static const string FILE_OVERHEAD_FIELD;
  static const string DIRECTORY_OVERHEAD_FIELD;
  static const string DESCRIPTION_FIELD;

  /// table storing data on available media types
  string mediaTypeTable_;
};

// constant creation
template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::MEDIA_TYPE_FIELD =  "media_type";

template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::CAPACITY_FIELD = "capacity";

template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::THRESHOLD_FIELD = "threshold";

template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::SPLIT_SIZE_FIELD = "splitSize";

template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::BLOCK_SIZE_FIELD = "blockSize";

template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::FILE_OVERHEAD_FIELD = "fileOverhead";

template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::DIRECTORY_OVERHEAD_FIELD = "directoryOverHead";

template <class ConnectionHandle> const string 
MediaTypeDAO<ConnectionHandle>::DESCRIPTION_FIELD = "description";


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
//------------------------------------------------------------------------------
template <class ConnectionHandle>
MediaTypeDAO<ConnectionHandle>::MediaTypeDAO(string mediaTypeTable):
  mediaTypeTable_(mediaTypeTable)
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// retrieves the whole set of available media types
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
MediaTypeDAO<ConnectionHandle>::  
findAllMediaType(MediaType::ResultSet& rmediaTypeSet,
		 const ConnectionHandle& rconnectionHandle) const
{
  int retStatus = 0;
  int nbMediaType = 0;
  string currentMediaType;
  typename ConnectionHandle::ResultSet res;
  typename ConnectionHandle::ListOfFieldIndex listOfFieldIndex;

  DFS_TRACE("MediaTypeDAO<ConnectionHandle>::findAllMediaType");

  // SQL statement building
  string select = "select  " + MEDIA_TYPE_FIELD + " ";
  string from   =  "from " + mediaTypeTable_ + " ";
 
  string sqlStatement = select + from ;
  if (rconnectionHandle.executeSelect(sqlStatement,
				      res,
				      listOfFieldIndex) != -1)
    {
      bool hasResultSet = res.hasResultSet();
      if (hasResultSet)
	{
	  while( res.next())
	    {
	      currentMediaType =  
                 rconnectionHandle.getStringValue(res,
						  listOfFieldIndex[MEDIA_TYPE_FIELD]);
	      rmediaTypeSet.push_back(MediaType(currentMediaType));
	      nbMediaType++;
	    }
	}
      retStatus = nbMediaType;
    }
  else 
    {
      DFS_ERROR("Error executing SQL command %s", sqlStatement.c_str());
      retStatus = -1;
    }
  return retStatus;
}
#endif // MEDIA_TYPE_DAO
// -*-++-*
