// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.6 $
// $Log: NgasFileService.h,v $
// Revision 1.6  2008/01/28 16:33:51  mpadovan
// First check-in.
//
// Revision 1.5  2006/12/19 10:39:23  cmoins
// Removal of the database name as parameter
//
// Revision 1.4  2006/12/15 15:23:08  cmoins
// DAO & Service design modification
//
// Revision 1.3  2006/12/14 13:58:17  cmoins
// cvs header adding
//
//
// Generic Data access service layer to NgasFile objects
// 
//------------------------------------------------------------------------------

#ifndef NGAS_FILE_SERVICE_H
#define NGAS_FILE_SERVICE_H

#include <string>
#include <DfsLog.h>
#include "NgasFile.h"

/// Generic Data access service layer to NgasFile objects
/**
 * This class can be used to access NgasFile objects stored
 * in any data repository
 */
template <class NgasFileDAO, class DataSourceConnection> 
class NgasFileService 
{
public:
 

  /// \brief Constructor
  /// \param dbServer the DB server name
  /// \param dbUser the DB user name
  /// \param dbPassword the DB password
  /// \param tableName the name of the ngas file table
  NgasFileService(const std::string& dbServer,
	          const std::string& dbUser,
		  const std::string& dbPasswd,
		  const std::string& tableName);
  
  /// \brief Insert the given NgasFile object
  /// \param rngasFile the replication request
  /// \return the number of inserted objects (0 or 1), or -1 in case of error
  int insert(const NgasFile& rngasFile);

  /// \Insert data in the ngas files table by fetching rows associated to 
  /// \rsrcdiskId, rfileId and fileVersion, setting for each of them the disk_id
  /// \field to rdestdiskId and the creation date to the current one and 
  /// \inserting the result into ngas_files.
  /// \param rsrcdiskId  : the source disk
  /// \param rfileId     : file id of the files to be retrieved
  /// \param fileVersion : file version of the file to be retrieved
  /// \param rdestdiskId : the destination disk
  /// \return the number of inserted objects (0 or 1), or -1 in case of error
  int insert(const std::string& rsrcdiskId,
             const std::string& rfileId,
             int fileVersion,
             const std::string& rdestdiskId);

  
  /// \brief Update the NgasFile object with given fileId
  /// \param fileId the file ID
  /// \param status the new status
  /// \return the number of updated objects (0 or 1), or -1 in case of error
  int update(const std::string& rfileId,
	     NgasFile::NgasFileStatus status);
  
  /// \brief Update the NgasFile object with given fileId,fileVersion,diskId
  /// \param fileId the file ID
  /// \param fileVersion the file version
  /// \param diskId the disk ID
  /// \param status the new status
  /// \return the number of updated objects (0 or 1), or -1 in case of error
  int update(const std::string& rfileId,
	     int fileVersion,
	     const std::string& rdiskId,
	     NgasFile::NgasFileStatus status);
  
  /// \brief Remove the NgasFile object with given fileId
  /// \param fileId the file ID
  /// \param fileVersion the file version
  /// \param diskId the disk ID
  /// \return the number of removed objects (0 or 1), or -1 in case of error
  int remove(const std::string& rfileId,
	     int fileVersion = 0,
	     const std::string& rdiskId = "");

  /// \brief Retrieve from the DB the NgasFile object with given fileId
  /// \param[in] fileId the file ID
  /// \param[out] rngasFileSet the selected NgasFile objects
  /// \return the number of retrieved objects, or -1 in case of error
  int find(const std::string& rfileId, 
           NgasFile::ResultSet& rngasFileSet );
  
  /// \brief Retrieve from the DB the NgasFile objects with given status
  /// \param[in] status the request status
  /// \param[out] rs the selected replication request objects
  /// \return the number of retrieved objects, or -1 in case of error
  int find(NgasFile::NgasFileStatus status, 
           NgasFile::ResultSet& rngasFileSet);


  /// \brief Retrieve from the DB the NgasFile objects with a given 
  /// file id, and version number.
  /// \param rfileId the file ID
  /// \param fileVersion the file version
  /// \param rngasFileSet set of retrieved files
  /// \return the number of retrieved objects, or -1 in case of error
  int find(const std::string& rfileId,
           int fileVersion/*,
           NgasFile::ResultSet& rngasFileSet*/); // mrc

 
  /// \brief Retrieve from the DB the NgasFile objects with a given 
  /// file id, disk id and version number.
  /// \param rfileId the file ID
  /// \param fileVersion the file version
  /// \param rdiskId the disk ID
  /// \param rngasFileSet set of retrieved files
  /// \return the number of retrieved objects, or -1 in case of error
  int find(const std::string& rfileId,
	   int fileVersion,
	   const std::string& rdiskId,
           NgasFile::ResultSet& rngasFileSet);
 
  /// \brief Retrieve all ReplicationRequest objects from the DB
  /// \param[out] rngasFileSet the selected ngas file objects
  /// \return the number of retrieved objects, or -1 in case of error
  int loadAll(NgasFile::ResultSet& rngasFileSet);

private :
   std::string dbServer_;
   std::string dbUser_;
   std::string dbPasswd_;
   NgasFileDAO ngasFileDAO_;  

};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
NgasFileService<NgasFileDAO, DataSourceConnection>::NgasFileService(
                               const std::string& dbServer,
			       const std::string& dbUser,
			       const std::string& dbPasswd,
			       const std::string& tableName) :
  dbServer_(dbServer),
  dbUser_(dbUser),
  dbPasswd_(dbPasswd),
  ngasFileDAO_(NgasFileDAO(tableName))
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Inserts a new NgasFile object into the data repository
// Return the number of inserted objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection>  
int
NgasFileService<NgasFileDAO, DataSourceConnection>::insert(
                                                      const NgasFile& rngasFile)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::insert");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::insert.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.insert(rngasFile,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Insert data in the ngas files table by fetching rows associated to 
// rsrcdiskId, rfileId and fileVersion, setting for each of them the disk_id
// field to rdestdiskId and the creation date to the current one and 
// inserting the result into ngas_files.
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection>  
int
NgasFileService<NgasFileDAO, DataSourceConnection>::insert(
                           const std::string& rsrcdiskId,
                           const std::string& rfileId,
                           int fileVersion,
                           const std::string& rdestdiskId)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::insert");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::insert.\
                Could not acquire connection");
  }
  else
  {
     rowCount =  ngasFileDAO_.insert(rsrcdiskId,
                                     rfileId,
                                     fileVersion,
                                     rdestdiskId,
                                     dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Update the NgasFile object with given fileId
// Return the number of updated objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int
NgasFileService<NgasFileDAO, DataSourceConnection>::update(
                       const std::string& rfileId,
		       NgasFile::NgasFileStatus status)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::update");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::update.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.update(rfileId,
                                    status,
                                    dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Update the NgasFile object with given fileId,fileVersion,diskId
// Return the number of updated objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int
NgasFileService<NgasFileDAO, DataSourceConnection>::update(const std::string& rfileId,
		       int fileVersion,
		       const std::string& rdiskId,
		       NgasFile::NgasFileStatus status)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::update");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::update.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.update(rfileId,
                                    fileVersion,
                                    rdiskId,
                                    status,
                                    dsConnection);
     dsConnection.release();
  }
  return rowCount;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Remove the NgasFile object with given fileId
// Return the number of removed objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int
NgasFileService<NgasFileDAO, DataSourceConnection>::remove(
                       const std::string& rfileId,
		       int fileVersion,
		       const std::string& rdiskId)
{
  int rowCount = -1; 

  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::remove");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::remove.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.remove(rfileId,
                                    fileVersion,
                                    rdiskId,
                                    dsConnection);
     dsConnection.release();
  }
  return rowCount;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Retrieve from the DB the NgasFile object with given fileId
// Return the number of retrieved objects, or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int
NgasFileService<NgasFileDAO, DataSourceConnection>::find(
                     const std::string& rfileId,
	             NgasFile::ResultSet& rngasFileSet)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::find");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::find.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.find(rfileId,
                                  rngasFileSet,
                                  dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Retrieve from the DB the NgasFile objects with given status
// Return the number of retrieved objects, or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int
NgasFileService<NgasFileDAO, DataSourceConnection>::find(
                     NgasFile::NgasFileStatus status,
	             NgasFile::ResultSet& rngasFileSet)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::find");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::find.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.find(status,
                                  rngasFileSet,
                                  dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// \brief Retrieve from the DB the NgasFile objects with a given 
//  file id, and version number.
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int 
NgasFileService<NgasFileDAO, DataSourceConnection>::find(
    const std::string& rfileId,
    int fileVersion/*,
    NgasFile::ResultSet& rngasFileSet*/)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::find");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::find.\
                Could not acquire connection");
  }
  else   // mrc
  {
     rowCount = ngasFileDAO_.find(rfileId,
                                  fileVersion,
                                  /*rngasFileSet,*/
                                  dsConnection);
     dsConnection.release();
  }
  return rowCount;

}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// \brief Retrieve from the DB the NgasFile objects with a given 
//  file id, disk id and version number.
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int 
NgasFileService<NgasFileDAO, DataSourceConnection>::find(const std::string& rfileId,
  	             int fileVersion,
	             const std::string& rdiskId,
                     NgasFile::ResultSet& rngasFileSet)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::find");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::find.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.find(rfileId,
                                  fileVersion,
                                  rdiskId,
                                  rngasFileSet,
                                  dsConnection);
     dsConnection.release();
  }
  return rowCount;

}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Retrieve all NgasFile objects from the DB
// Return the number of retrieved objects, or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class DataSourceConnection> 
int
NgasFileService<NgasFileDAO, DataSourceConnection>::loadAll(
                                              NgasFile::ResultSet& rngasFileSet)
{
  int rowCount = -1; 
  
  DFS_TRACE("NgasFileService<NgasFileDAO, class DataSourceConnection>::loadAll");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("NgasFileService<NgasFileDAO, class DataSourceConnection>::loadAll.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.loadAll(rngasFileSet,
                                     dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

#endif //NGAS_FILE_SERVICE_H
