// -*-c++-*-
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observator
// LICENSE:   ESO internal use only
//
// PROJECT:   VLT Data Flow Syste
// AUTHOR:    Christophe Moins- ESO/SDD/DF
// SUBSYSTEM: arcsys
//
// $Revision: 1.11 $
// $Log: ObsProg.h,v $
// Revision 1.11  2009/06/15 12:30:10  vforchi
// Removed daemon mode, added reduced data handling
//
// Revision 1.10  2008/01/27 18:53:58  cmoins
// removal of ( and ) characters in program id when building the directory structure
//
// Revision 1.9  2008/01/02 12:53:24  cmoins
// set replaced by vector to define ResultSet
//
// Revision 1.8  2007/07/10 11:16:46  cmoins
// removal of useless comments
//
// Revision 1.7  2007/07/10 08:56:44  cmoins
// removal of getMaxTime + removal of the previously used instruments list
//
// Revision 1.6  2007/07/04 11:59:31  cmoins
// DFS04052 : more meaningful messages when file database ingestion or storage is not possible
//  CVS: ----------------------------------------------------------------------
//
// Revision 1.5  2007/05/24 11:38:57  cmoins
// previously retrieved files are taken into account when looking for package files
//
// Revision 1.4  2007/05/15 15:34:03  cmoins
// removal of progId_ to build media files
//
// Revision 1.3  2007/05/15 10:14:40  cmoins
// modification of logging messages
//
// Revision 1.2  2007/05/04 12:20:50  cmoins
// adding of storeData, updateProgDB and getMaxTime implementation
//
// Revision 1.1  2007/04/25 15:57:41  cmoins
//
//
//------------------------------------------------------------------------------
#ifndef OBSPROG_H
#define OBSPROG_H

#include <string>
#include <vector>
#include "FileCopy.h"
#include "DPPackerDef.h"
#include "Media.h"
#include "MediaFile.h"

using namespace std;

/// Observation prog base class
/**
 * All the program classes in charge of retrieving files associated to a 
 *certain type of program, to store them in a staging area and to update a 
 * data repository will inherit from it. It can not be instantiate.
 * It takes a StorageService template parameter that deals with the 
 * storage of a package content in a data repository.
 */
template <class StorageService>
class ObsProg
{
public :
  
  /// set of observation program
  typedef vector<ObsProg<StorageService>*> ResultSet;
  typedef typename ResultSet::iterator pObsProg;
 
  /// \brief Constructor
  /// \param progId program identifier
  /// \param connAlias data repository connection parameter
  ObsProg(string progId,
	  string connAlias):
    progId_(progId),
    connAlias_(connAlias)
  {}
 
  /// \brief Destructor
  virtual ~ObsProg()
  {}

  /// \brief fetches a list of files associated to the program
  /**
   * \param startTime start time of the files retrieval
   * \param startTimeAdjustment correction to apply on start time for 
   *                            calibration files retrieval
   * \param endTime end time of the files retrieval
   * \param endTimeAdjustment   correction to apply on end time for 
   *                            calibration files retrieval
   * \param previousFileSet     files already retrieved and to be excluded
   *                            from the current retrieval
   * \return 0 if successful, -1 otherwise 
   */
  virtual int fetchData(double  startTime,
			double  startTimeAdjustment,
			double  endTime,
			double  endTimeAdjustment,
			const MediaFile::ResultSet& rpreviousFileSet,
			vector<string> reducedDataLocations) = 0;

  /// \brief stores the content of the program inside a staging area.
  /**
   * \param destHost host where the files shall be stored
   * \param stagingArea directory where the files will be stored.
   * \return 0 if successful, -1 otherwise
   */
  virtual int storeData(string destHost,
			string stagingArea);

  ///\brief updates the data repository with the files in the staging area
  /**
   * rmedia media request files will be associated to
   * \return 0 if successful, -1 otherwise
   */
  virtual int updateProgDB(Media& rmedia);


  /// \brief instantiates a copy of the object
  /// returns the address of the copy
  virtual ObsProg* clone() const = 0;

protected :

  // Set of operations that give access to the data member to the subclasses

  /// \brief set the database connection alias
  void setAlias(const string& connAlias)
  {
    connAlias_ = connAlias;
  }

  /// \brief get the database connection alias
  /// \return the connection alias
  string getAlias()
  {
    return connAlias_;
  }

  /// \brief set the data file set data member
  void setDataFileSet(const RawDataProduct::ResultSet& dataFileSet)
  {
    dataFileSet_ = dataFileSet;
  }
  /// \brief get the data file set data member
  /// \return the data file set data member
  RawDataProduct::ResultSet getDataFileSet()
  {
    return dataFileSet_;
  }

  /// \brief get the program identifier
  string getProgId()
  {
    return progId_;
  }

private :
  /// program identifier
  string progId_;
  /// database connection parameters alias
  string connAlias_; 
  /// set of files of the program
  RawDataProduct::ResultSet dataFileSet_;


};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Stores the content of the program inside a staging area.
//------------------------------------------------------------------------------
template <class StorageService>
int
ObsProg<StorageService>::storeData(string destHost,
				   string stagingArea)
{
  int retStatus = 0;
  int nbCopiedFiles = 0;
  string progId; // prog id without ( and ) characters

  DFS_TRACE("GenObsProg<ObservationService>::storeData");
  // ( and ) characters are removed so that program directories
  // may be handled by any program
  for (unsigned int index = 0; index < progId_.length(); index++){
    if (progId_[index] != '(' &&
      progId_[index] != ')'){
      progId += progId_[index];
    }
  }

  string mainStorageDirectory = stagingArea + "/" + progId;
  string storageDirectory;

  for (RawDataProduct::pRawDataProduct pdataFile = dataFileSet_.begin();
       pdataFile != dataFileSet_.end();pdataFile++)
    {	 
      TimeStamp nightProductTS(pdataFile->mjdObs_); 
      string nightProduct = nightProductTS.getNight();
      storageDirectory = mainStorageDirectory + "/" + nightProduct;
      if (!DPPackerCopy::copyFile(destHost,storageDirectory,pdataFile))
	{
	 
	  // complete file name is updated to the copy one
	  const_cast<string&>(pdataFile->magDiskFile_.fname_) = 
	    progId + "/" + nightProduct + "/" + pdataFile->dpCat_ 
	    + "/" + pdataFile->magDiskFile_.fname_;
	  nbCopiedFiles++;
	}
    }
  if ((unsigned int)nbCopiedFiles != dataFileSet_.size())
    {
      DFS_ERROR("Some files have not been copied");
      retStatus = -1;
    }
  else if (dataFileSet_.empty())
    {
      DFS_INFO("No file to be stored for program %s",progId_.c_str());
    }
  return retStatus;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// updates the data repository with the files in the staging area
//------------------------------------------------------------------------------
template <class StorageService>
int
ObsProg<StorageService>::updateProgDB(Media& rmedia) 
{
  int retStatus = 0;
  StorageService storageService(connAlias_);
  MediaFile::ResultSet mediaFileSet;
  string fileName;

  DFS_TRACE("GenObsBlock<ObservationService>::updateProgDB");

  if (!dataFileSet_.empty())
    {
      for (RawDataProduct::pRawDataProduct pdataFile = dataFileSet_.begin();
	   pdataFile != dataFileSet_.end();pdataFile++)
	{
	  if (pdataFile->magDiskFile_.status_ == DPPackerDef::FILE_RETRIEVED)
	    {
	      fileName =  pdataFile->magDiskFile_.fname_;
	      mediaFileSet.push_back(MediaFile(pdataFile->dpId_,
					       fileName,
					       pdataFile->magDiskFile_.fsize_,
					       pdataFile->magDiskFile_.uncompFSize_,
					       DPPackerDef::NO_COMPRESSION_FLAG,
					       pdataFile->magDiskFile_.status_));
	    }

	}
      if (!storageService.insertMediaFiles(rmedia,
					   mediaFileSet))
	{
	  DFS_INFO("Media Files insertion succeeded for %s",progId_.c_str());
	  dataFileSet_.clear();
	}
      else
	{
	  // the insertion did not succeed for the file,
	  // it is removed from the set of files 
	  DFS_ERROR("Media Files insertion failed for %s",progId_.c_str());
	  retStatus = -1;
	}
    }
  else
    {
      DFS_INFO("No file to be ingested in database for program %s",
	       progId_.c_str());
    }
  return retStatus;
}


#endif /* OBSPROG_H */
