/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.5 $
// $Log: Qc0Frame.h,v $
// Revision 1.5  2008/09/11 08:31:07  vforchi
// fixed compilation problems with gcc 4.2.4
//
// Revision 1.4  2007/10/18 08:18:06  cmoins
// storage of converted files in /tmp instead of current directory
//
// Revision 1.3  2007/10/18 08:02:11  cmoins
// use of conversion factor for measured values
//
// Revision 1.2  2007/10/17 08:42:04  cmoins
// replacement of getFrameFileName() by getFileName()
// replacement of openFrame() by open()
//
// Revision 1.1  2007/10/17 08:11:15  cmoins
// Replacement of Qcm by Qc0
//
//
// 
//------------------------------------------------------------------------------

#ifndef QC0_FRAME_H
#define QC0_FRAME_H

#include <map>
#include <vector>
#include <string>
#include <sstream>
#include <DfsLog.h>
#include "FitsUtils.h"
#include "AmbientService.h"


/**
 *  Implementing of a default instrument frame
 */ 
struct ConstraintSet;

class Qc0Frame
{
public:

  /// structure implementing a parameter with its constraint and value
  struct ParamData 
  {
    ParamData():
      keyword_(""),
      checkedConstraint_(""),
      convFactor_(1.)
    {
      value_.str("");
      constraint_.str("");
    }
    ParamData(std::string keyword,
	      double convFactor):
      keyword_(keyword),
      checkedConstraint_(""),
      convFactor_(convFactor)
    {
      value_.str("");
      constraint_.str("");
    }

    //Copy constructor
    ParamData(const ParamData& rparamData) {
      *this = rparamData;
    }

    // assignment operator
    ParamData& operator = (const ParamData& rparamData) {
      value_             << rparamData.value_.str();
      constraint_        << rparamData.constraint_.str();
      keyword_           =  rparamData.keyword_;
      checkedConstraint_ =  rparamData.checkedConstraint_;
      convFactor_        =  rparamData.convFactor_;
      return *this;
    }

    /// value of the parameter
    std::stringstream value_;
    /// constraint associated to the parameter
    std::stringstream constraint_;
    /// Possibly, the keyword of the parameter in a frame header
    std::string keyword_;
    /// checked constraint tag
    std::string checkedConstraint_;
    /// conversion factor
    double convFactor_;
  };

  /// \brief Default Constructor
  Qc0Frame()
  {
    pframeContent_  = 0;
    pconstraintSet_ = 0;
  }

  /// \brief Constructor
  /**
   * \param frameFileName   name of the frame file 
   * \param ambientAlias    ambient database connection alias
   * \param obrepAlias      obrep database connection alias
   * \param observationSite site of observation
   */
  Qc0Frame(std::string frameFileName,
	   std::string ambientAlias,
	   std::string obrepAlias,
	   std::string arSystemAlias,
	   AmbientService::ObservationSite observationSite);

  /// \brief Destructor
  ~Qc0Frame();

  /// \brief copy constructor
  /**
   * \param rframe frame to be copied
   * returns a copy of the frame 
   */
  Qc0Frame(const Qc0Frame& rframe);

  /// \brief assignment operator
  /**
   * \param rframe frame to be assigned
   * returns the result of the assignment
   */
  Qc0Frame& operator = (const Qc0Frame& rframe) ;

  /// \brief less than operator, for the need of associative containers
  bool operator < (const Qc0Frame& rframe) const {
    return (frameFileName_ < rframe.frameFileName_);
  }

  /// \brief performs a configuration of the frame
  /// returns 0 if successful, -1 otherwise
  int configure(bool onlyMeasValue);

  /// \brief gets the category of the frame
  /// return the category if successful, an empty string otherwise.
  std::string getCategory() const ;

  /**
   * Gets the air mass measured value, constraint and checked constraint flag
   * \param rairMass air mass parameter
   * return 0 if successful, -1 otherwise
   */
  int getAirMass(ParamData& rairMass) const;
 
  /**
   * Gets the seeing  measured value, constraint and checked constraint flag
   * \param rseeing  seeing parameter
   * \param mjdStart mjd start
   * \param expTime  exposure time
   * return 0 if successful, -1 otherwise
   */
  int getSeeing(ParamData& rseeing) const;

  /**
   * Gets the moon distance measured value, constraint and checked 
   * constraint flag.
   * \param rmoonDist moon distance parameter
   * return 0 if successful, -1 otherwise
   */
  int getMoonDist(ParamData& rmoonDist) const;

  /**
   * Gets the moon illumination measured value, constraint and  checked 
   * constraint flag.
   * \param rmoonFLI  moon illumination parameter
   * return 0 if successful, -1 otherwise
   */
  int getMoonFLI(ParamData& rmoonFLI) const;

  /**
   * gets the strehl measured value, constraint and checked constraint 
   * flag.
   * \param rstrehl strehl parameter
   * return 0 if successful, -1 otherwise
   */
  int getStrehl(ParamData& rstrehl) const;

  /**
   * gets the sky transparency measured value, constraint and checked 
   * constraint flag.
   * \param rskyTransparency sky transparency parameter
   * return 0 if successful, -1 otherwise
   */
  int getSkyTransparency(ParamData& rskyTransparency) const;

  /// \brief gives the name of the frame file without the path
  /// \returns the name of the frame file
  std::string getFileName() const;

  /// \brief gets the observation block id of the frame
  /// \returns the observation block id
  int getObsId() const { return obsId_;}

  /// \brief gets the last error message
  /// \returns the last error message
  std::string getErrorMessage() { return errorMessage_; }

private :

  static const std::string CONVERTED_EXTENSION;
  static const std::string HDR_EXTENSION;
  static const std::string CONVERTED_FILE_DIR;
  /// string associated to parameters data
  static const std::string NOT_APPLICABLE;
  static const std::string CONDITION_FULFILLED;
  static const std::string CONDITION_NOT_FULFILLED;
  static const std::string NO_CHECKED_CONDITION_TAG;

  /// \brief gets the mjd observation of the frame
  /// return the mjd obs if successful, -1 otherwise
  double getMjdObs() const;

  /// \brief gets the exposure time of the frame
  /// return the exposure time in seconds if successful, -1 otherwise
  double getExpTime() const;

  /// \brief opens the frame file
  /// returns 0 if successful, -1 otherwise
  int open();

  /// \brief performs some ressources cleanup
  /// returns 0 if successful, -1 otherwise
  int cleanup();

  /// Name of the fits file  
  std::string frameFileName_;
  /// pointer to the frame content
  ArcSysFits::FitsFile* pframeContent_;
  /// ambient database connection alias
  std::string ambientAlias_;
  /// obrep database connection alias
  std::string obrepAlias_;
  /// ARSystem database connection alias
  std::string arSystemAlias_;
  /// observation site
  AmbientService::ObservationSite observationSite_;
  /// Set of constraints of the frame
  ConstraintSet* pconstraintSet_;
  /// id of the observation block the frame belongs to
  int obsId_;
  /// start time of the observation
  double mjdStart_;
  /// exposure time
  double expTime_;
  /// Last error message
  std::string errorMessage_;
};
#endif // QC0_FRAME_H
