/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.1 $
// $Log: Qc0FrameAnalyser.h,v $
// Revision 1.1  2007/10/17 08:11:15  cmoins
// Replacement of Qcm by Qc0
//
//
// 
//------------------------------------------------------------------------------
/**
 * class that provides a report on quality control parameters on a set of 
 * frames stored in a directory.
 */
#ifndef QC0_FRAME_ANALYSER_H
#define QC0_FRAME_ANALYSER_H

#include <string>
#include <ostream>
#include <vector>
#include "Qc0FrameAnalyserConfig.h"
#include "Qc0Frame.h"

class Qc0FrameAnalyser
{
public:

  /// \brief Default Constructor
  /**
   * \param argc number of arguments of the command line
   * \param argv command line
   */
  Qc0FrameAnalyser(int argc, const char* argv[]);

  /// \brief Destructor
  ~Qc0FrameAnalyser(){}

  /// \brief performs the configuration of the frames analyser
  /** 
   * returns 0 if successful, 1 if no other operation shall be performed
   * -1 otherwise
   */
  int configure();
  
  /**
   * gives a report about the quality controler parameters of the frame
   * stored in the selected directory 
   * returns 0 if successful, -1 otherwise
   */
  int getReport() const;

private :

  /// Set of parameter names that may be read from a frame
  static const std::string SEEING;
  static const std::string SKY_TRANSPARENCY;
  static const std::string AIRMASS;
  static const std::string MOON_FLI;
  static const std::string MOON_DISTANCE;
  static const std::string STREHL;
  static const std::string STREHL1;
  static const std::string STREHL2;
  static const std::string STREHL3;
  static const std::string BASELINE_ID;

  /// Type definition
  /// Type associating an id to a parameter
  typedef std::map<std::string,Qc0Frame::ParamData> ParamDataList;

  /// Type associating a frame name to a set of parameter
  typedef std::map<Qc0Frame,ParamDataList> FrameParamList;

  /// types associating a get method of Qc0Frame to a parameter
  typedef  int (Qc0Frame::* pFrameGet)(Qc0Frame::ParamData&) const;
  typedef std::map<std::string,pFrameGet> pFrameGetMap;

  /**
   * gives the ambient parameters and constraints of the frame
   * stored in the directory
   * \param rframeParamList List of frame parameters
   * \param rbadFrameList   List of frames for which errors showed up
   */
  int getFrameParameters(FrameParamList& rframeParamList,
			 std::vector<Qc0Frame>& rbadFrameList) const;

  /**
   * prints on the standard output or a file the ambient parameters values,
   * constraints and validity test of a set of frames
   * \param rframeParamList List of frame parameters
   * \param rbadFrameList   List of frames for which errors showed up
   */
  int printReport(FrameParamList& rframeParamList,
		  std::vector<Qc0Frame>& rbadFrameList) const;

  /// configuration of the frame analyser
  Qc0FrameAnalyserConfig frameAnalyserConfig_;
  /// name of the file containing the report
  std::string reportFile_;
  /// directory where files to be analysed will be fetched
  std::string polledDir_;
  /// List of the parameters that shall be handled
  ParamDataList paramList_;
  /// parameter/ frame operation map
  pFrameGetMap frameGetMap_;

};
#endif // QC0_FRAME_ANALYSER_H
