// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.9 $
// $Log: RawDataProductDAO.h,v $
// Revision 1.9  2008/01/14 10:11:59  mpadovan
// Intermediate checkin.
//
// Revision 1.7  2007/07/25 07:28:17  cmoins
// doxygen comments correction
//
// Revision 1.6  2007/07/10 09:04:08  cmoins
// the list of previously used instruements is not used any more
// when retrieving calibration files
//
// Revision 1.5  2007/05/15 09:59:34  cmoins
// replacement of info logging  messages by debug ones
//
// Revision 1.4  2007/05/02 09:39:39  cmoins
// removal of files location on magnetic disk retrieval
//
// Revision 1.3  2007/04/26 15:19:48  cmoins
// modification of doxygen comments
//
// Revision 1.2  2007/04/26 12:52:12  cmoins
// doxygen comments modification
//
// Revision 1.1  2007/04/25 15:13:10  cmoins
// creation of a DAO class associated to the RawDataProduct class
//
// 
//------------------------------------------------------------------------------

#ifndef RAW_DATA_PRODUCT_DAO_H
#define RAW_DATA_PRODUCT_DAO_H

#include <iostream>
#include <string>
#include <sstream>
#include <set>
#include <DfsLog.h>
#include <boost/tokenizer.hpp>
#include <iomanip>
#include "RawDataProduct.h"

using namespace std;

/// Data Access Object associated to RawDataProducts objects.
/**
 *  This class allows to retrieve raw products files in an observation table.
 *  It will also rely on two other tables to fetch informations
 * of files on magnetic disk :
 * - one storing data about files stored on the magnetic disk
 * - one storing data about the magnetic disk
 *
 * Note : when giving a list of instruments already handled,
 * the format shall be the following :
 * Inst1 start1 end1,Inst2 start2 end2,..
 * It takes a ConnectionHandle template parameter that represents 
 * the type of connection handle to the data repository so that it remains
 * indepedent of any connection system.
 */
template <class ConnectionHandle> class RawDataProductDAO
{
public:
 
  /// Possible category of files 
  enum Category {SCIENCE=0,CALIB=1,REDUCED=2}; 
  /// to indicate that no instrument is used
  static const string NO_INST;   

  /// \brief Constructor
  /**
   * \param obsDataProductTable table storing observation program files data
   */
  RawDataProductDAO(string obsDataProductTable);

  /// \brief finds the file associated to a given id.
  /**
   * \param fileId id of the file to be retrieved
   * \param rdataFileSet set of files associated to the identifier
   * \param rconnectionHandle connection handle to the data repository
   * \return the number of files if successful, -1 otherwise
   */
  int findByFileId(string fileId,
		   RawDataProduct::ResultSet& rdataFileSet,
		   const ConnectionHandle& rconnectionHandle) const;

  /**
   *  Finds files from a given program for a given time range
   * according to their category (science, reduced or calibration)
   * \param progId identifier of the observation program
   * \param catg category of files to be retrieved
   * \param startTime start time of the retrieval
   * \param endTime end time of the retrieval
   * \param rdataFileSet set of science files
   * \param rconnectionHandle connection handle to the data repository
   * \return the number of files if the operation was successful, -1 otherwise
   */
  int findByCatgAndProgId(string progId,
			  Category catg,
			  double startTime,
			  double endTime,
			  RawDataProduct::ResultSet& rdataFileSet,
			  const ConnectionHandle& rconnectionHandle) const;

  /**
   * Finds files from a given observation block for a given time range
   * according to their category (science, reduced or calibration)
   * \param obsId identifier of the observation block
   * \param catg category of files to be retrieved
   * \param startTime start time of the retrieval
   * \param endTime end time of the retrieval
   * \param rdataFileSet set of science files
   * \param rconnectionHandle  connection handle to the data repository
   * \return the number of files if the operation was successful, -1 otherwise
   */
  int findByCatgAndObsId(int obsId,
			 Category catg,
			 double startTime,
			 double endTime,
			 RawDataProduct::ResultSet& rdataFileSet,
			 const ConnectionHandle& rconnectionHandle) const;

  /**
   * Finds files asssociated to a set of instruments for a given time
   * range and according to their category (science, reduced or calibration)
   * \param instList List of instruments for which calibration files
   *                 shall be retrieved.
   * \param catg category of files to be retrieved
   * \param startTime start time of the retrieval
   * \param endTime end time of the retrieval
   * \param rdataFileSet set of  observation program files 
   * \param rconnectionHandle  connection handle to the data repository
   * \return the number of files if the operation was successful, -1 otherwise
   */
  int findByCatgAndInst(string instList,
			Category catg,
			double startTime,
			double endTime,
			RawDataProduct::ResultSet& rdataFileSet,
			const ConnectionHandle& rconnectionHandle) const;

  /**
   *  Finds files from a given program for a given time range
   * associated to a set of instruments and according to their category 
   * (science, reduced or calibration).
   * \param progId identifier of the observation program
   * \param instList List of instruments for which calibration files
   *                 shall be retrieved.
   * \param catg category of files to be retrieved
   * \param startTime start time of the retrieval
   * \param endTime end time of the retrieval
   * \param rdataFileSet set of  observation program files 
   * \param rconnectionHandle  connection handle to the data repository
   * \return the number of files if the operation was successful, -1 otherwise
   */
  int findByCatgAndProgIdAndInst(string progId,
				 string instList,
				 Category catg,
				 double startTime,
				 double endTime,
				 RawDataProduct::ResultSet& rdataFileSet,
				 const ConnectionHandle& rconnectionHandle) 
    const;

  /**
   * Finds files from a given observation block for a given time range
   * associated to a set of instruments and according to their category 
   * (science, reduced or calibration).
   * \param obsId identifier of the observation block
   * \param instList List of instruments for which calibration files
   *                 shall be retrieved.
   * \param catg category of files to be retrieved
   * \param startTime start time of the retrieval
   * \param endTime end time of the retrieval
   * \param rdataFileSet set of  observation program files 
   * \param rconnectionHandle connection handle to the data repository
   * \return the number of files if the operation was successful, -1 otherwise
   */
  int findByCatgAndObsIdAndInst(int obsId,
				string instList,
				Category catg,
				double startTime,
				double endTime,
				RawDataProduct::ResultSet& rdataFileSet,
				const ConnectionHandle& rconnectionHandle) 
    const;

  /**
   * Removes the object with the given file id.
   * \param rfileId id of the file to be removed
   * \param rconnectionHandle connection handle to the data repository
   * \return the number of files removed if successful, -1 otherwise
   */
   int remove( string fileId,
               const ConnectionHandle& rconnectionHandle ) const;

private:

  // static constant declaration   
  /// if no identifier is used
  static const string NO_ID;          
  /// category id to string map
  static const string CATEGORY_MAP[];    
  /// prog id field in the table
  static const string PROG_ID_FIELD;     
  /// file id field in the table
  static const string ODP_FILE_ID_FIELD; 
  /// obs block field in the table
  static const string OBSBLOCK_ID_FIELD; 
  /// category field in the table
  static const string CATEGORY_FIELD;    
  /// observation time field in the table
  static const string DATE_FIELD;         
  /// instrument field in the table
  static const string INSTRUMENT_FIELD;  
 
  /** 
   * Set of instruments. Variables of this type shall have this format :
   * Inst1 start1 end1,Inst2 start2 end
   */
  typedef set<string> InstrumentIdSet;   
  typedef InstrumentIdSet::iterator pInstrumentId;

  /**
   * Finds files from a given program or observation block
   * for a given time range associated to a set of instruments and 
   * according to their category (science, reduced or calibration) 
   * \param progOrObsBlockfieldName name of the field in the table for 
   *                                a block or a program.
   * \param progOrObsBlockId identifier of a block or of a program
   * \param instList List of instruments for which calibration files
   *                 shall be retrieved.
   * \param catg category of files to be retrieved
   * \param startTime start time of the retrieval
   * \param endTime end time of the retrieval
   * \param rdataFileSet       set of observation program files 
   * \param rconnectionHandle  connection handle to the data repository
   * \return the number of files if the operation was successful, -1 otherwise
   */
  int findByCatgAndInstAndId(string progOrObsBlockfieldName,
			     string progOrObsBlockId,
			     string instList,
			     Category catg,
			     double startTime,
			     double endTime,
			     RawDataProduct::ResultSet& rdataFileSet,
			     const ConnectionHandle& rconnectionHandle) const;

  /// \brief Executes a sql statement and creates a set of files
  /**
   * \param sqlStatement statement to be executed
   * \param rdataFileSet set of files issued by the execution of the statement
   * \param rconnectionHandle connection handle to the data repository
   * \return the number of files if the operation was successful, -1 otherwise
   */
  int performFilesRetrieval(string sqlStatement,
			    RawDataProduct::ResultSet& rdataFileSet,
			    const ConnectionHandle& rconnectionHandle) const;

  string obsDataProductTable_; /// table storing the observation products
};

// static constant creation
template <class ConnectionHandle> 
const string RawDataProductDAO<ConnectionHandle>::
NO_INST = "NO_INST";

template <class ConnectionHandle> 
const string RawDataProductDAO<ConnectionHandle>::
CATEGORY_MAP[] = {"<>\'CALIB\'","=\'CALIB\'"};

template <class ConnectionHandle> 
const string RawDataProductDAO<ConnectionHandle>::
NO_ID = "NO_ID";

template <class ConnectionHandle> 
const string RawDataProductDAO<ConnectionHandle>::
PROG_ID_FIELD = "prog_id";

template <class ConnectionHandle>
const string RawDataProductDAO<ConnectionHandle>::
ODP_FILE_ID_FIELD = "dp_id";

template <class ConnectionHandle>
const string RawDataProductDAO<ConnectionHandle>::
OBSBLOCK_ID_FIELD = "ob_id";

template <class ConnectionHandle>
const string RawDataProductDAO<ConnectionHandle>::
CATEGORY_FIELD = "dp_cat";

template <class ConnectionHandle>
const string RawDataProductDAO<ConnectionHandle>::
DATE_FIELD = "mjd_obs";

template <class ConnectionHandle>
const string RawDataProductDAO<ConnectionHandle>::
INSTRUMENT_FIELD = "ins_id";

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
//------------------------------------------------------------------------------
template <class ConnectionHandle>
RawDataProductDAO<ConnectionHandle>::
RawDataProductDAO(string obsDataProductTable):
  obsDataProductTable_(obsDataProductTable)
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// finds the files associated to a given id.
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
findByFileId(string fileId,
	     RawDataProduct::ResultSet& rdataFileSet,
	     const ConnectionHandle& rconnectionHandle) const
{
  int retStatus = 0;

  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::findByFileId");

  string select = "select  " + PROG_ID_FIELD + "," + ODP_FILE_ID_FIELD + ","
    + INSTRUMENT_FIELD  + "," + DATE_FIELD + "," + OBSBLOCK_ID_FIELD + ","
    + CATEGORY_FIELD + " ";
  string from   =  "from " + obsDataProductTable_ + " ";
  string where  =  "where " +
    ODP_FILE_ID_FIELD + "='" + fileId + "'";
  string sqlStatement = select + from + where;
  int nbFiles =
    performFilesRetrieval(sqlStatement,rdataFileSet,rconnectionHandle);
  if (nbFiles < 0 )
    {
      DFS_ERROR("Did not retrieve files associated to %s", fileId.c_str());
      retStatus = -1;
    }
  else 
    retStatus = nbFiles;
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Finds files from a given program for a given time range according to their 
// category (science, reduced or calibration)
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
findByCatgAndProgId(string progId,
		    Category catg,
		    double startTime,
		    double endTime,
		    RawDataProduct::ResultSet& rdataFileSet,
		    const ConnectionHandle& rconnectionHandle) const
{
  int retStatus = 0;

  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::findByCatgAndProgId");

  progId = "'" + progId + "'";
  retStatus = findByCatgAndInstAndId(PROG_ID_FIELD,
				     progId,
				     NO_INST,
				     catg,
				     startTime,
				     endTime,
				     rdataFileSet,
				     rconnectionHandle);
  if (retStatus > 0)
    {
      DFS_DEBUG("Files retrieval succeeded for prog %s",progId.c_str());
    }
  else if (!retStatus)
    {
      DFS_DEBUG("No File retrieved for prog %s",progId.c_str());
    }
  else
    {
      DFS_ERROR("Files retrieval failed for prog %s",progId.c_str());
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Finds files from a given block for a given time range according to their 
// category (science,reduced or calibration)
// Return the number of selected objects, or -1 in case of error
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
findByCatgAndObsId(int obsId,
		   Category catg,
		   double startTime,
		   double endTime,
		   RawDataProduct::ResultSet& rdataFileSet,
		   const ConnectionHandle& rconnectionHandle) const
{
  int retStatus = 0;
  stringstream obsIdStr;
  obsIdStr << obsId;
 
  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::findByCatgAndObsId");

  retStatus = findByCatgAndInstAndId(OBSBLOCK_ID_FIELD,
				     obsIdStr.str(),
				     NO_INST,
				     catg,
				     startTime,
				     endTime,
				     rdataFileSet,
				     rconnectionHandle);
  if (retStatus > 0)
    {
      DFS_DEBUG("Files retrieval succeeded for block %d",obsId);
    }
  else if (!retStatus)
    {
      DFS_DEBUG("No File retrieved for block %d",obsId);
    }
  else
    {
      DFS_ERROR("Files retrieval failed for block %d",obsId);
      retStatus = -1;
    }
  
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Finds files asssociated to a set of instruments for a given time range and
// according to their category (science, reduced or calibration) 
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
findByCatgAndInst(string instList,
		  Category catg,
		  double startTime,
		  double endTime,
		  RawDataProduct::ResultSet& rdataFileSet,
		  const ConnectionHandle& rconnectionHandle) const
{
  int retStatus = 0;
 
  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::findByCatgAndInst");

  if (instList != NO_INST)
    retStatus = findByCatgAndInstAndId(NO_ID,
				       NO_ID,
				       instList,
				       catg,
				       startTime,
				       endTime,
				       rdataFileSet,
				       rconnectionHandle);
  else
    retStatus = -1;
  if (retStatus > 0)
    {
      DFS_DEBUG("Files retrieval succeeded for %s",instList.c_str());
    }
  else if (!retStatus)
    {
      DFS_DEBUG("No File retrieved for  %s",instList.c_str());
    }
  else
    {
      DFS_ERROR("Files retrieval failed for %s",instList.c_str());
      retStatus = -1;
    }

  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Finds files from a given program for a given time range
// associated to a set of instruments and according to their category 
// (science, reduced or calibration)
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
findByCatgAndProgIdAndInst(string progId,
			   string instList,
			   Category catg,
			   double startTime,
			   double endTime,
			   RawDataProduct::ResultSet& rdataFileSet,
			   const ConnectionHandle& rconnectionHandle) const
{
  int retStatus=0;

  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::findByCatgAndProgIdAndInst");

  progId = "'" + progId + "'"; 
  retStatus = findByCatgAndInstAndId(PROG_ID_FIELD,
				     progId,
				     instList,
				     catg,
				     startTime,
				     endTime,
				     rdataFileSet,
				     rconnectionHandle);
  if (retStatus > 0)
    {
      DFS_DEBUG("Files retrieval succeeded for prog %s",progId.c_str());
    }
  else if (!retStatus)
    {
      DFS_DEBUG("No File retrieved for prog %s",progId.c_str());
    }
  else
    {
      DFS_ERROR("Files retrieval failed for prog %s",progId.c_str());
      retStatus = -1;
    }
  return retStatus;   
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Finds files from a given observation block for a given time range
// associated to a set of instruments and according to their category 
// (science, reduced or calibration) 
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
findByCatgAndObsIdAndInst(int obsId,
			  string instList,
			  Category catg,
			  double startTime,
			  double endTime,
			  RawDataProduct::ResultSet& rdataFileSet,
			  const ConnectionHandle& rconnectionHandle) const
{
  int retStatus=0;
  stringstream obsIdStr;
  obsIdStr << obsId;

  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::findByCatgAndObsIdAndInst");

  retStatus = findByCatgAndInstAndId(OBSBLOCK_ID_FIELD,
				     obsIdStr.str(),
				     instList,
				     catg,
				     startTime,
				     endTime,
				     rdataFileSet,
				     rconnectionHandle);
  if (retStatus > 0)
    {
      DFS_DEBUG("Files retrieval succeeded for block %d",obsId);
    }
  else if (!retStatus)
    {
      DFS_DEBUG("No File retrieved for block %d",obsId);
    }
  else
    {
      DFS_ERROR("Files retrieval failed for block %d",obsId);
      retStatus = -1;
    }
  return retStatus;   
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Finds files from an observation program or block for a given time range
// asssociated to a set of instruments and according to their category 
// (science, reduced or calibration) 
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
findByCatgAndInstAndId(string progOrObsBlockfieldName,
		       string progOrObsBlockId,
		       string instList,
		       Category catg,
		       double startTime,
		       double endTime,
		       RawDataProduct::ResultSet& rdataFileSet,
		       const ConnectionHandle& rconnectionHandle) const
{
  int retStatus = 0;
  string logMessage;

  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::findByCatgAndInstAndId");

  // to parse a string whose separator is a ","
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
  boost::char_separator<char> sep(",");

  stringstream startTimeStr;
  stringstream endTimeStr;   

  const int MS_ACCURACY = 15; // number of digits of times to have ms accuracy
  startTimeStr << setprecision(MS_ACCURACY) << startTime;
  endTimeStr << setprecision(MS_ACCURACY) << endTime;

  // Calibration files retrieval
  // SQL statement building
  string select = "select  " + PROG_ID_FIELD + "," + ODP_FILE_ID_FIELD + ","
    + INSTRUMENT_FIELD  + "," + DATE_FIELD + "," + OBSBLOCK_ID_FIELD + ","
    + CATEGORY_FIELD + " ";
  string from   =  "from " + obsDataProductTable_ + " ";
  string where  =  "where " +
    DATE_FIELD + " between " + startTimeStr.str() + " AND " + endTimeStr.str()
    + " AND " + CATEGORY_FIELD + CATEGORY_MAP[catg];

  if (progOrObsBlockId != NO_ID)
    {
      where += " AND " + progOrObsBlockfieldName + "=" + progOrObsBlockId;
      logMessage = progOrObsBlockId;
    }  
  if (instList != NO_INST)
    {
      logMessage += instList;
      // some instruments are available. The associated files on the given
      // time range will be retrieved
      where += " AND (" + INSTRUMENT_FIELD + " IN (";
      tokenizer tok(instList,sep);
      for(tokenizer::iterator beg=tok.begin(); beg!=tok.end();++beg)
	{
	  if (beg != tok.begin())
	    where += ",";
	  where += "'" + *beg + "'";
	}  
      where += "))";
    } 

  string sqlStatement = select + " " + from + " " + where;
  int nbFiles = performFilesRetrieval(sqlStatement,rdataFileSet,rconnectionHandle);
  if (nbFiles >= 0)
    {     
      DFS_DEBUG("Files retrieval succeeded for %s",progOrObsBlockId.c_str());
      retStatus = nbFiles;
    }
  else
    {
      DFS_ERROR("Files retrieval failed for %s",progOrObsBlockId.c_str());
      retStatus = -1;
    }
  return retStatus;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Executes a sql statement and creates a set of files
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
performFilesRetrieval(string sqlStatement,
		      RawDataProduct::ResultSet& rdataFileSet,
		      const ConnectionHandle& rconnectionHandle) const
{

  int retStatus = 0;
  int nbFiles = 0; 

  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::performFilesRetrieval");

  typename ConnectionHandle::ListOfFieldIndex listOfFieldIndex;
  typename ConnectionHandle::ResultSet res ;
  if ((rconnectionHandle.executeSelect(sqlStatement,
				       res,
				       listOfFieldIndex) != -1))
    {  
      const string CALIB_CAT   = "CALIB"; 
      const string SCIENCE_CAT = "SCIENCE"; 
      string fileProgId;
      string fileId;
      string instrumentId;
      bool hasResultSet = res.hasResultSet();
      while(hasResultSet && res.next())
	{
	  // File Creation
	  fileId = 
	    rconnectionHandle.getStringValue(res,
					     listOfFieldIndex[ODP_FILE_ID_FIELD]);
	  RawDataProduct newDataFile(fileId);
	    
	  newDataFile.insId_ = 
	    rconnectionHandle.getStringValue(res,
					     listOfFieldIndex[INSTRUMENT_FIELD]);
	  newDataFile.mjdObs_ =
	    rconnectionHandle.getDoubleValue(res,
					     listOfFieldIndex[DATE_FIELD]);
	  newDataFile.obId_ =
	    rconnectionHandle.getLongValue(res,
					   listOfFieldIndex[OBSBLOCK_ID_FIELD]);
	  newDataFile.progId_ = 
	    rconnectionHandle.getStringValue(res,
					     listOfFieldIndex[PROG_ID_FIELD]);
          string category = 
	    rconnectionHandle.getStringValue(res,
					     listOfFieldIndex[CATEGORY_FIELD]);
          if (category == CALIB_CAT)
             newDataFile.dpCat_ = CALIB_CAT;
          else
             newDataFile.dpCat_ = SCIENCE_CAT;

	  //File insertion
	  rdataFileSet.insert(newDataFile);
	  nbFiles++;
	}
      retStatus = nbFiles;
    }
  else 
    {
      DFS_ERROR("Error executing SQL command %s", sqlStatement.c_str());
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// removes the file(s) with the given id.
//------------------------------------------------------------------------------
template <class ConnectionHandle>
int
RawDataProductDAO<ConnectionHandle>::
remove(string fileId,
       const ConnectionHandle& rconnectionHandle) const
{
  int rowCount = -1;

  DFS_TRACE("RawDataProductDAO<ConnectionHandle>::remove");

  std::string query = "delete from " + obsDataProductTable_ +
                      " where " + ODP_FILE_ID_FIELD + "='" + fileId + "'";

  typename ConnectionHandle::ResultSet res = rconnectionHandle.execute(query);

  if( res.commandSucceeded() )
  {
    rowCount = res.getRowCount();
  }
  else
  {
    DFS_ERROR("Command failed: could not delete file with id '%s'.",
      fileId.c_str());
  }

  DFS_DEBUG("Command succeeded: deleted %d file(s) with id '%s'.",
      rowCount, fileId.c_str());

  return rowCount;
}

#endif // RAW_DATA_PRODUCT_DAO_H
// -*-++-*-
