// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.36 $
// $Log: RepAgent.h,v $
// Revision 1.36  2007/12/03 16:24:17  mpadovan
// Added a new friend: TestNgas2LocalCopyTask.
//
// Revision 1.35  2007/11/29 16:51:23  mpadovan
// RepAgent is now also a friend of TestRepQMonTask.
//
// Revision 1.34  2007/11/22 13:01:15  cmoins
// adding of friend classes TestRepGenTask and TestTlIngStatusTask
//
// Revision 1.33  2007/09/27 13:29:06  jknudstr
// tickets: DFS03134, DFS03929, DFS03931, DFS03960, DFS04125, DFS04167,
// DFS04318.
//
// Revision 1.32  2007/05/10 15:34:24  jknudstr
// Testing ArcRepSA2PA ...
//
// Revision 1.31  2007/05/03 15:32:07  jknudstr
// Intermediate check-in while developing the SA->PA Archive Replicator.
//
// Revision 1.30  2007/04/11 09:56:31  jknudstr
// Implemented uncompressing only for Ngas2LocalCopy Task.
//
// Revision 1.29  2007/01/11 15:35:10  jknudstr
// -
//
// Revision 1.28  2006/12/20 15:20:43  jknudstr
// -
//
// Revision 1.27  2006/12/13 16:31:09  cmoins
// Taking in account of SybReplicationRequestDAO renaming in ReplicationRequestDAOSyb
// (the sequel)
//
// Revision 1.26  2006/12/13 16:27:56  cmoins
// Taking in account of SybReplicationRequestDAO renaming in ReplicationRequestDAOSyb
//
// Revision 1.25  2006/10/12 13:49:12  jknudstr
// -
//
// Revision 1.24  2006/08/11 14:59:47  jknudstr
// -
//
// Revision 1.23  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef REPAGENT_H
#define REPAGENT_H


// STL includes
#include <string>

// External packages includes
#include "ace/Mutex.h"
#include <boost/program_options.hpp>

// Local includes
#include <ArcSysEx.h>
#include <DfsProperties.h>
#include <ProTrieve.h>
#include <ReplicationRequest.h>
#include <ReplicationAgentServiceSyb.h>
#include <NgamsClient.h>
#include <RepReqMsg.h>


namespace po = boost::program_options;


/** DB interface class for handling Replication Requests.
 *
 * The class is used to access the Replication Queue Table in the DB.
 * It is used to retrieve information about pending entries, update and
 * remove entries from the DB Replication Queue.
 */
class DbCon: public ReplicationAgentServiceSyb
{
  public:
    /// Constructor preparing for connecting to the specified RDBMS.
    /**
     * Constructor method setting up the connection parameters.
     *
     * See ReplicationAgentServiceSyb.
     */   
    DbCon(const string&    dbServer,
	  const string&    dbUser,
	  const string&    dbPassword,
	  const string&    ngasFilesTableName,
	  const string&    repReqTableName):
	ReplicationAgentServiceSyb(dbServer, dbUser, dbPassword, 
				   ngasFilesTableName, repReqTableName)
    {
	DFS_TRACE("DbCon::DbCon()");
    }

    /// Destructor method.
    ~DbCon()
    {
	DFS_TRACE("DbCon::~DbCon()");
    }
};


/** Provides business logic for the Replication Agent tasks.
 *
 * The purpose of the RepAgent Class, is to provide different services to the
 * internal tasks in each instance of the Replication Agent.
 * 
 * <p>
 * Normally it should only be instantiated once and subsequently shared among
 * the various tasks (threads).
 * 
 * <p>
 * The class provides basic services such as interfacing to the pro_trieve
 * archive control SW, the associated NGAS systems, handling of the 
 * configuration parameters and more.
 */
class RepAgent
{
  public:

#if defined UNIT_TEST
  friend class TestRepGenTask;
  friend class TestTlIngStatusTask;
  friend class TestRepQMonTask;
  friend class TestNgas2LocalCopyTask;
#endif
    typedef enum {
	failure = -1,
	success = 0
    } Status;

    /// Constructor method.
    RepAgent(const string&  applicationName);

    /// Destructor method.
    ~RepAgent(void);

    /// Return reference to application name member.
    const string&  applicationName(void) {return appName_;}

    /// Return reference to Client ID.
    const string&  clientId(void) {return clientId_;}

    /// Return reference to the associated boost options object.
    po::variables_map&  options(void);

    /// Return reference to the associated DfsProperties object.
    DfsProperties& properties(void);

    /// Return reference to the associated DB Connection object.
    DbCon& dbCon(void);

    /// Initialize the object.
    /**
     * Initialize the object according to the associated configuration file.
     *
     * \return Reference to the object itself.
     */
    RepAgent&   initialize(void);

    /// Initialize a ProTrieve instance from the internal configuration. 
    RepAgent&   initProTrieve(ProTrieve& proTrieveObj);
    
    /// Initialize an NgamsClient instance from the internal configuration. 
    RepAgent&   initNgasClient(const string&  type,
			       NgamsClient&   ngasClient,
			       const bool     uncompress);

    /// Return reference to the pro_trieve Archive Requests Object Map.
    ProTrieveArchReqsMap& proTriArchReqMap(void);

    /// Generate a unique new integer index.
    /// 
    /**
     * Generate a unique new integer index. The index is only unique for this
     * session of the server. This is used internally for generating unique
     * file/directory names, usually combined with a timestamp.
     *
     * \return New index.
     */
    unsigned long getNewIndex(void);

    /// Clean up NGAS staging area, used when retrieving files.
    /**
     * Remove left-over files from the NGAS staging area.
     *
     * \param stagingArea Base directory of the staging area.
     * \param tmpDir Sub-directory in the staging area.
     *
     * \return Void.
     */
    void cleanUpNgasStg(const string&  stagingArea,
			const string&  tmpDir);

    /// Retrieve a file from the given NGAS system.
    /**
     * Retrieve a file from the given NGAS system associated with the client
     * object.
     *
     * \param repReq Replicatation Request object associated with the file to 
     * be retrieved.
     * \param client Instance of NgamsClient Class used to contact NGAS.
     * \param targetName Name of directory or complete path where the 
     * requested file should end up.
     * \param tmpDir sub-directory within the staging area (stgArea) under
     * which the file should be stored temporarily while receiving it.
     * \param finalTargetFile Final name of the file requested.
     *
     * \return NgamStatus object with information about the execution of the 
     * request.
     */
    NgamsStatus retrieveFromNgas(RepReqMsg&        repReq,
				 NgamsClient&      client,
				 const string&     stgDir,
				 const string&     stgFile,
				 const string&     tmpDir,
				 string&           finalTargetFile);

    /// Simulate interaction with pro_trieve?
    bool simulProTrieve(void) {return simulProTrieve_;}

    /// Simulate interaction with NGAS?
    bool simulNgas(void) {return simulNgas_;}

    // Constants:

    // Miscelleneous constants.
    /// General timeout applied when needed.
    static const int    GEN_TIMEOUT;
    /// Timeout to apply after an exception ocurrs, before resuming service.
    static const int    EXCEPTION_DELAY;
    /// Tag used to indicate a staging file in the DB Replication Queue.
    static const string STG_FILE;
    /// Main thread sleep time.
    static const int    MAIN_THREAD_WAIT;
    /// Minimum space that should be available on staging areas (in MB).
    static const float  MIN_STG_SPACE;
    /// DB NGAS Files Table name.
    static const string NGAS_FILES_TABLE;

    // Code names for replication Agent Instances.
    /// Name of the PA to SA Replication Agent.
    static const string REP_AGENT_PA_SA;
    /// Name of the SA to PA Replication Agent.
    static const string REP_AGENT_SA_PA;
    /// Name of the MU to PA Replication Agent.
    static const string REP_AGENT_MU_PA;
    /// Name of the MU to FC Replication Agent.
    static const string REP_AGENT_MU_FC;

    // Code names for the various tasks.
    /// Name of the NGAS To Local Copy Task.
    static const string NGAS_2_LOCAL_COPY_TASK;
    /// Name of the NGAS To NGAS Clone Task.
    static const string NGAS_2_NGAS_CLONE_TASK;
    /// Name of the NGAS To TL Ingestion Task.
    static const string NGAS_2_TL_ING_TASK;
    /// Name of the Report Generation Task.
    static const string REP_GEN_TASK;
    /// Name of the Replication Queue Monitoring Task.
    static const string REP_Q_MON_TASK;
    /// Name of the TL To NGAS Restore Task.
    static const string NGAS_RESTORE_TASK;
    /// Name of the TL File Availability Check Task.
    static const string TL_FILE_AVAIL_CHK_TASK;
    /// Name of the TL Ingestion Status Task.
    static const string TL_ING_STATUS_TASK;
    /// Name of the TL Request Task.
    static const string TL_REQ_TASK;

    // Command line options.
    /// Name of command line option to specify the configuration file.
    static const string  OPT_CFG_FILE;
    static const string  OPT_HELP;
    static const string  OPT_VERSION;

    // Configuration file properties.

    /// Tag for General Section.
    static const string CFG_GEN_SECTION;
    /// Client ID parameter name.
    static const string CFG_GEN_CLI_ID;

    /// Tag for Logging Section.
    static const string CFG_LOG_SECTION;
    /// Log filename.
    static const string CFG_LOG_FILE;
    /// Log control flags (see ACE documentation).
    static const string CFG_LOG_CTRL;
    /// Log priority (see ACE documentation).
    static const string CFG_LOG_PRIORITY;

    /// Tag for the Reporting Section.
    static const string CFG_REPORT_SECTION;
    /// Directory where copies of the reports should be dumped.
    static const string CFG_REPORT_DIR;
    /// List of email recipents to receive the reports.
    static const string CFG_REPORT_RECIPIENTS;
    /// Report Generation Time parameter name.
    static const string CFG_REPORT_TIME;

    /// Tag for DB Section.
    static const string CFG_DB_SECTION;
    /// DB server parameter name.
    static const string CFG_DB_SRV;
    /// DB user parameter name.
    static const string CFG_DB_USER;
    /// DB user password parameter name.
    static const string CFG_DB_PSWD;
    /// DB name parameter name.
    static const string CFG_DB_NAME;
    /// DB Replication Queue Table name.
    static const string CFG_REP_Q_TABLE;
    /// DB NGAS Files Table.
    static const string CFG_NGAS_FILES_TABLE;

    /// Tag for pro_trieve section in the configuration.
    static const string CFG_PROTRIEVE_SECTION;
    /// Pro_trieve client ID parameter name.
    static const string CFG_PROTRIEVE_CLI_ID;
    /// Pro_trieve system ID parameter name.
    static const string CFG_PROTRIEVE_SYS_ID;
    /// Pro_trieve basic staging directory parameter name.
    static const string CFG_PROTRIEVE_BASE_DIR;
    /// Pro_trieve context parameter name.
    static const string CFG_PROTRIEVE_CONTEXT;
    /// Pro_trieve SA volume ID parameter name.
    static const string CFG_PROTRIEVE_SA_VOL_ID;
    /// Simulate interaction with pro_trieve.
    static const string CFG_PROTRIEVE_SIMUL;

    /// Tag for NGAS section in the configuration.
    static const string CFG_NGAS_SECTION;
    /// List of source NGAS servers parameter name.
    static const string CFG_NGAS_SRC_SRV_LIST;
    /// List of destination NGAS servers parameter name.
    static const string CFG_NGAS_DST_SRV_LIST;    

    /// Tag for Tasks section in the configuration.
    static const string CFG_TASKS_SECTION;
    /// Rep. Queue Monitoring poll time parameter name.
    static const string CFG_TASKS_REP_Q_POLL_TIME;
    /// Number of NGAS to TL Ingestion Threads parameter name.
    static const string CFG_TASKS_NO_NGAS_2_TL_TASKS;
    /// TL Ingestion Status Task status polling time.
    static const string CFG_TASKS_TL_ING_STAT_POLL_TIME;
    /// Number of NGAS to Local File Threads parameter name.
    static const string CFG_TASKS_NO_NGAS_2_LOCAL_TASKS;
    /// NGAS to Local Target Directory parameter name.
    static const string CFG_TASKS_NGAS_2_LOCAL_TARGET;
    /// NGAS to Local Task: Prepend File Version to target filename.
    static const string CFG_TASKS_NGAS_2_LOCAL_UNIQUE_NAME;
    /// NGAS Restore Task: Number of instances.
    static const string CFG_TASKS_NO_NGAS_RESTORE_TASKS;
    /// TL File Availability Check Task: Poll time.
    static const string CFG_TASKS_TL_FILE_AVAIL_CHK_POLL_TIME;


  private:
    string                   appName_;

    string                   clientId_;

    po::variables_map        options_;

    DfsProperties*           props_;

    DbCon*                   dbCon_;

    ProTrieveArchReqsMap     proTrieveArchReqs_;

    int                      simulProTrieve_;
    int                      simulNgas_;

    unsigned long            index_;
    ACE_Mutex                indexMutex_; 

    ACE_Mutex                ngasMutex_; 
    bool                     cleanedUpNgasStg_;
};


#endif // REPAGENT_H

// EOF
