// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.11 $
// $Log: ReplicationAgentService.h,v $
// Revision 1.11  2007/02/16 14:37:52  jknudstr
// Improved error log for railed call to setReplicationSuccessful().
//
// Revision 1.10  2007/02/05 16:43:29  cmoins
// Creation of dppacker persistent layer access
//
// Revision 1.9  2007/01/08 13:11:27  cmoins
// update of setReplicationSuccessful + adding of insertNgasFile, removeNgasFile
// findNgasFile
//
// Revision 1.8  2006/12/20 12:01:44  jknudstr
// -
//
// Revision 1.7  2006/12/19 09:41:06  cmoins
// removal of the database name as a parameter
//
// Revision 1.6  2006/12/19 08:01:20  cmoins
// adding of the updateReplicationRequest(rfileid,fileVersion,rdiskid,status,rmessage)
//
// Revision 1.5  2006/12/19 07:36:14  jknudstr
// -
//
// Revision 1.4  2006/12/18 14:43:54  cmoins
// Management of the database connection in the service layer
//
// Revision 1.3  2006/12/15 15:35:01  cmoins
// DAO & Service design modification
//
// Revision 1.2  2006/12/14 13:59:48  cmoins
// cvs header adding
//
//
// Generic Data access service layer to NgasFile and ReplicationRequest objects
// 
//------------------------------------------------------------------------------

#ifndef REPLICATION_AGENT_SERVICE_H
#define REPLICATION_AGENT_SERVICE_H

#include <string>
#include <DfsLog.h>
#include "NgasFile.h"
#include "ReplicationRequest.h"

/// Generic Data access service layer to NgasFile and ReplicationRequest objects
/**
 * The goal of this class is to perform coordinated actions on NgasFile 
 * and ReplicationRequest objects
 */
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
class ReplicationAgentService 
{
public:
 
  /// \brief Constructor
  /// \param dbServer the DB server name
  /// \param dbUser the DB user name
  /// \param dbPassword the DB password
  /// \param ngasFiletableName the name of the ngas file table
  /// \param rrTableName the name of the replication request table
  ReplicationAgentService(const std::string& dbServer,
	          const std::string& dbUser,
		  const std::string& dbPasswd,
		  const std::string& ngasFileTableName,
                  const std::string& rrTableName);
 
  /// \brief Insert the given ReplicationRequest object
  /// \param rreplicationRequest the replication request
  /// \return the number of inserted objects (0 or 1), or -1 in case of error
  int insertReplicationRequest(const ReplicationRequest& rreplicationRequest); 

  /// \brief updates the ReplicationRequest object with given fileId
  /// \param fileId the file ID
  /// \param status the new status
  /// \param message the new message
  /// \return the number of updated objects (0 or 1), or -1 in case of error
  int updateReplicationRequest(const std::string& rfileId,
                               ReplicationRequest::RequestStatus status,
                               const std::string& rmessage);

  /// \brief Update the ReplicationRequest object with given fileId,fileVersion,diskId
  /// \param fileId the file ID
  /// \param fileVersion the file version
  /// \param diskId the disk ID
  /// \param status the new status
  /// \param message the new message
  /// \return the number of updated objects (0 or 1), or -1 in case of error
  int updateReplicationRequest(const std::string& rfileId,
           	               int fileVersion,
	                       const std::string& rdiskId,
	                       ReplicationRequest::RequestStatus status,
	                       const std::string& rmessage);


   
  int setReplicationSuccessful(const std::string& rfileId,
			       int   fileVersion,
			       const std::string& rsrcdiskId,
			       const std::string& rdestdiskId);


  /// \brief Remove the ReplicationRequest object with given fileId
  /// \param fileId the file ID
  /// \param fileVersion the file version
  /// \param diskId the disk ID
  /// \return the number of removed objects (0 or 1), or -1 in case of error 
  int removeReplicationRequest(const std::string& rfileId,
                               int   fileVersion,
                               const std::string& rdiskId); 

  /// \brief Retrieve from the DB the ReplicationRequest object with given fileId
  /// \param[in] fileId the file ID
  /// \param[out] rs the selected replication request objects
  /// \return the number of retrieved objects, or -1 in case of error
  int findReplicationRequest(const std::string& rfileId, 
                             ReplicationRequest::ResultSet& rrs); 

  /// \brief Retrieve from the DB the ReplicationRequest objects with given status
  /// \param[in] status the request status
  /// \param[out] rs the selected replication request objects
  /// \return the number of retrieved objects, or -1 in case of error
  int findReplicationRequest(typename ReplicationRequest::RequestStatus status, 
                             ReplicationRequest::ResultSet& rrs); 

  /// \brief Retrieve all ReplicationRequest objects from the DB
  /// \param[out] rs the selected replication request objects
  /// \return the number of retrieved objects, or -1 in case of error
  int loadAllReplicationRequest(ReplicationRequest::ResultSet& rrs);

  /// \brief Insert the given NgasFile object
  /// \param rngasFile the ngas file
  /// \return the number of inserted objects (0 or 1), or -1 in case of error
  int insertNgasFile(const NgasFile& rngasFile); 

  /// \brief removes the given NgasFile object
  /// \param rngasFile the ngas file
  /// \return the number of inserted objects (0 or 1), or -1 in case of error
  int removeNgasFile(const std::string& rfileId,
                     int   fileVersion,
                     const std::string& rdiskId);

  /// \brief Retrieve from the DB the NgasFile object with given fileId
  /// \param[in] fileId the file ID
  /// \param[out] nfs the selected ngas files objects
  /// \return the number of retrieved objects, or -1 in case of error
  int findNgasFile(const std::string& rfileId, 
                   NgasFile::ResultSet& nfs); 

private : 
   std::string dbServer_;
   std::string dbUser_;
   std::string dbPasswd_;
   NgasFileDAO ngasFileDAO_;  
   ReplicationRequestDAO rrDAO_;
};


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::ReplicationAgentService(
                               const std::string& dbServer,
			       const std::string& dbUser,
			       const std::string& dbPasswd,
		               const std::string& ngasFileTableName,
                               const std::string& rrTableName):
  dbServer_(dbServer),
  dbUser_(dbUser),
  dbPasswd_(dbPasswd),
  ngasFileDAO_(NgasFileDAO(ngasFileTableName)),
  rrDAO_(ReplicationRequestDAO(rrTableName))
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Inserts the given ReplicationRequest object
// Return the number of inserted objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::insertReplicationRequest(
                                 const ReplicationRequest& rreplicationRequest)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::insertReplicationRequest");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::insertReplicationRequest.\
                Could not acquire connection");
  }
  else
  {
     rowCount = rrDAO_.insert(rreplicationRequest,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// updates the ReplicationRequest object with given fileId
// Return the number of inserted objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::updateReplicationRequest(
                                 const std::string& rfileId,
                                 ReplicationRequest::RequestStatus status,
                                 const std::string& rmessage)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::updateReplicationRequest");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::updateReplicationRequest.\
                Could not acquire connection");
  }
  else
  {
     rowCount = rrDAO_.update(rfileId,status,rmessage,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// updates the ReplicationRequest object with given fileId,fileVersion,diskId.
// Return the number of inserted objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::updateReplicationRequest(
                               const std::string& rfileId,
           	               int fileVersion,
	                       const std::string& rdiskId,
	                       ReplicationRequest::RequestStatus status,
	                       const std::string& rmessage)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::updateReplicationRequest");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::updateReplicationRequest.\
                Could not acquire connection");
  }
  else
  {
     rowCount = rrDAO_.update(rfileId,
                              fileVersion,
                              rdiskId,
                              status,
                              rmessage,
                              dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// updates the replication request table and inserts a new file in the ngas file 
// table when a replication was successful.
// Return the number of inserted objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int 
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
			DataSourceConnection>::
setReplicationSuccessful(const std::string& rfileId,
			 int   fileVersion,
			 const std::string& rsrcdiskId,
			 const std::string& rdestdiskId)
{
  int rowCount1 = -1, rowCount2 = -1, stat = -1; 

  DFS_TRACE("ReplicationAgentService::setReplicationSuccessful");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::setReplicationSuccessful.\
                Could not acquire connection");
  }
  else
  {
     dsConnection.beginTransaction();
     rowCount1 = rrDAO_.update(rfileId,
                               fileVersion,
                               rsrcdiskId,
			       ReplicationRequest::Replicated,
                               "",
			       dsConnection);
     rowCount2 = ngasFileDAO_.insert(rsrcdiskId, 
                                     rfileId,
                                     fileVersion,
				     rdestdiskId, 
                                     dsConnection);
     if (rowCount1 == 1 && rowCount2 ==1 )
	 {
	 dsConnection.commitTransaction();
	 stat = 0;
	 }
     else
	 {
	 dsConnection.rollbackTransaction();
	 DFS_ERROR("ReplicationAgentService::setReplicationSuccessful. "
		   "Error updating replication request to successful. "
		   "Replication Request Queue Update: %d. "
		   "NGAS Files Insert: %d", rowCount1, rowCount2);
	 }
     dsConnection.release();
  }
  return stat;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Removes the ReplicationRequest object with given fileId
// Return the number of removed objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::removeReplicationRequest(
                                 const std::string& rfileId,
                                 int   fileVersion,
                                 const std::string& rdiskId)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::removeReplicationRequest");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::removeReplicationRequest.\
                Could not acquire connection");
  }
  else
  {
     rowCount = rrDAO_.remove(rfileId,fileVersion,rdiskId,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Retrieves from the DB the ReplicationRequest object with given fileId
// Return the number of retrieved objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::findReplicationRequest(
                                 const std::string& rfileId, 
                                 ReplicationRequest::ResultSet& rrs)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::findReplicationRequest");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::findReplicationRequest.\
                Could not acquire connection");
  }
  else
  {
     rowCount = rrDAO_.find(rfileId,rrs,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Retrieve from the DB the ReplicationRequest objects with given status
// Return the number of retrieved objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::findReplicationRequest(
                             typename ReplicationRequest::RequestStatus status, 
                             ReplicationRequest::ResultSet& rrs)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::findReplicationRequest");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::findReplicationRequest.\
                Could not acquire connection");
  }
  else
  {
     rowCount = rrDAO_.find(status,rrs,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Retrieve all ReplicationRequest objects from the DB
// return the number of retrieved objects, or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::loadAllReplicationRequest(
                             ReplicationRequest::ResultSet& rrs)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::loadAllReplicationRequest");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::loadAllReplicationRequest.\
                Could not acquire connection");
  }
  else
  {
     rowCount = rrDAO_.loadAll(rrs,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Inserts the given NgasFile object
// Return the number of inserted objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::insertNgasFile(
                                 const NgasFile& rngasFile)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::insertNgasFile");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::insertNgasFile.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.insert(rngasFile,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Removes the NgasFile object with given fileId
// Return the number of removed objects (0 or 1), or -1 in case of error
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::removeNgasFile(
                                 const std::string& rfileId,
                                 int   fileVersion,
                                 const std::string& rdiskId)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::removeNgasFile");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::removeNgasFile.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.remove(rfileId,fileVersion,rdiskId,dsConnection);
     dsConnection.release();
  }
  return rowCount;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// brief Retrieve from the DB the NgasFile object with given fileId
//------------------------------------------------------------------------------
template <class NgasFileDAO, class ReplicationRequestDAO, 
          class DataSourceConnection>
int
ReplicationAgentService<NgasFileDAO, ReplicationRequestDAO,
                        DataSourceConnection>::findNgasFile(
                                                     const std::string& rfileId, 
                                                     NgasFile::ResultSet& nfs)
{
  int rowCount = -1; 

  DFS_TRACE("ReplicationAgentService::findNgasFile");

  DataSourceConnection dsConnection(dbServer_,dbUser_,dbPasswd_);
  int connectionStatus = dsConnection.acquire(); 
  if (connectionStatus)
  {
     DFS_ERROR("ReplicationAgentService::findNgasFile.\
                Could not acquire connection");
  }
  else
  {
     rowCount = ngasFileDAO_.find(rfileId,nfs,dsConnection);
     dsConnection.release();
  }
  return rowCount;

}
#endif //REPLICATION_AGENT_SERVICE_H
