// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Stefano Zampieri - ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.7 $
// $Log: ReplicationRequest.h,v $
// Revision 1.7  2007/01/08 13:12:56  cmoins
// Replication Request iterator type creation
//
// Revision 1.6  2006/12/15 15:25:06  cmoins
// DAO & Service design modification
//
// Revision 1.5  2006/07/03 13:07:35  szampier
// removed source and destination information for replication requests.
// added const qualifier to methods.
//
// Revision 1.4  2006/05/31 15:54:10  jknudstr
// -
//
// Revision 1.3  2006/05/30 14:50:48  jknudstr
// Added ReplicationRequest::genKey(), added support for status Error.
//
// Revision 1.2  2006/05/17 16:32:54  szampier
// improved documentation
//
// Revision 1.1  2006/05/05 16:04:22  szampier
// first check-in
//
// 
// 
// Replication Request business object
// 
//------------------------------------------------------------------------------


#ifndef REPLICATION_REQUEST_H
#define REPLICATION_REQUEST_H

#include <string>
#include <sstream>
#include <ctime>

/// Replication Request business object
/**
 * This class represents a request for replicating a data object
 * in the context of the ESO archive systems
 */
#include <vector>
class ReplicationRequest
{
 public:
  /// For specifying the status of a ReplicationRequest
  typedef enum {
    Scheduled = 0,
    Pending = 1,
    Requested = 2,
    Ready = 3,
    Replicating = 4,
    Replicated = 5,
    Reported = 6,
    Error = -1
  } RequestStatus;
 
  typedef std::vector<ReplicationRequest> ResultSet; 
  typedef ResultSet::iterator pReplicationRequest; 

  /// \brief Constructor
  /// \param fileId the file ID
  ReplicationRequest(const std::string& fileId = "");
  
  /// \brief Get fileId
  /// \return the file ID
  const std::string& fileId() const { return fileId_; }
  
  /// \brief Set fileId
  /// \param id the file ID
  void fileId(const std::string& id) { fileId_ = id; }
  
  /// \brief Get diskId
  /// \return the disk ID
  const std::string& diskId() const { return diskId_; }
  
  /// \brief Set diskId
  /// \param id the disk ID
  void diskId(const std::string& id) { diskId_ = id; }
  
  /// \brief Get submitterId
  /// \return the submitter ID
  const std::string& submitterId() const { return submitterId_; }

  /// \brief Set submitterId
  /// \param id the submitter ID
  void submitterId(const std::string& id) { submitterId_ = id; }
  
  /// \brief Get statusString
  /// \return the status string
  const std::string& statusString() const { return statusString_; }  
  
  /// \brief Set statusString
  /// \param s the status string
  void statusString(const std::string& s) { statusString_ = s; }
  
  /// \brief Get message
  /// \return the message
  const std::string& message() const { return message_; }
  
  /// \brief Set message
  /// \param m the message
  void message(const std::string& m) { message_ = m; }
  
  /// \brief Get fileVersion
  /// \return the file version
  const int fileVersion() const { return fileVersion_; }
  
  /// \brief Set fileVersion
  /// \param v the file version
  void fileVersion(int v) { fileVersion_ = v; }

  /// \brief Get priority
  /// \return the priority
  const int priority() const { return priority_; }
  
  /// \brief Set priority
  /// \param p the priority
  void priority(int p) { priority_ = p; }
  
  /// \brief Get statusId
  /// \return the status ID
  const RequestStatus statusId() const { return statusId_; }

  /// \brief Set statusId
  /// \param s the status ID
  void statusId(RequestStatus s) { statusId_ = s; }
  
  /// \brief Set statusId
  /// \param s the status ID
  void statusId(int s);
  
  /// \brief Get submissionTime
  /// \return the submission time as timeval record
  const struct timeval& submissionTime() const { return submissionTime_; }
  
  /// \brief Get submissionTime
  /// \return the submission time as double
  double submissionTimeAsDouble() const {
    return (submissionTime_.tv_sec + submissionTime_.tv_usec/1000000.);
  }

  /// \brief Set submissionTime
  /// \param t the submission time
  void submissionTime(const struct timeval& t) { submissionTime_ = t; }

  /// \brief Get lastUpdateTime
  /// \return the last update time as timeval record
  const struct timeval& lastUpdateTime() const { return lastUpdateTime_; }
  
  /// \brief Get lastUpdateTime
  /// \return the last update time as double
  double lastUpdateTimeAsDouble() const {
    return (lastUpdateTime_.tv_sec + lastUpdateTime_.tv_usec/1000000.);
  }
  
  /// \brief Set lastUpdateTime
  /// \param t the last update time
  void lastUpdateTime(const struct timeval& t) { lastUpdateTime_ = t; }
  
  /// \brief Get string representation of the ReplicationRequest
  /// \return the string representation of this
  std::string toString() const;
  
  /// \brief Convert RequestStatus to string
  /// \param rs the request status
  static std::string statusAsString(RequestStatus rs);
  
  /// \brief Generate (unique) key for this request.
  /// \return Key for this object.
  std::string genKey(const std::string& sep = "_") const;
  
private:
  std::string fileId_;
  int fileVersion_;
  std::string diskId_;
  std::string submitterId_;
  struct timeval submissionTime_;
  int priority_;
  RequestStatus statusId_;
  std::string statusString_;
  struct timeval lastUpdateTime_;
  std::string message_;
};


#endif
