// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Stefano Zampieri - ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.5 $
// $Log: ReplicationRequestDAOSyb.h,v $
// Revision 1.5  2006/12/19 09:39:04  cmoins
// removal of the database name as a  parameter
//
// Revision 1.4  2006/12/18 14:45:02  cmoins
// Management of the database connection at the service layer
//
// Revision 1.3  2006/12/15 15:26:39  cmoins
// DAO & Service design modification
//
// Revision 1.2  2006/12/14 14:01:20  cmoins
// cvs header adding
//
// 
// Sybase Data Access Object for ReplicationRequest objects
// 
//------------------------------------------------------------------------------

#ifndef REPLICATION_REQUEST_DAO_SYB_H
#define REPLICATION_REQUEST_DAO_SYB_H

#include <iostream>
#include <sstream>

#include <DfsLog.h>
#include <ConnectionPool.h>
#include <SybUtils.h>
#include "ReplicationRequest.h"

class ServiceConnectionSyb;

/// Sybase Data Access Object for ReplicationRequest objects
/**
 * This class can be used to access ReplicationRequest objects stored
 * in a Sybase table.
 */
class ReplicationRequestDAOSyb
{
public:
 
  /// \brief Constructor
  /// \param tableName the name of the replication queue
  ReplicationRequestDAOSyb(const std::string& tableName);
  
  /// \brief Insert the given ReplicationRequest object
  /// \param rr the replication request
  /// \param rconnectionHandle database connection handle
  /// \return the number of inserted objects (0 or 1), or -1 in case of error
  int insert(const ReplicationRequest& rr,
             const ServiceConnectionSyb& rconnectionHandle);
  
  /// \brief Update the ReplicationRequest object with given fileId
  /// \param fileId the file ID
  /// \param status the new status
  /// \param message the new message
  /// \param rconnectionHandle database connection handle
  /// \return the number of updated objects (0 or 1), or -1 in case of error
  int update(const std::string& fileId,
	     ReplicationRequest::RequestStatus status,
	     const std::string& message, 
             const ServiceConnectionSyb& rconnectionHandle);
  
  /// \brief Update the ReplicationRequest object with given fileId,fileVersion,diskId
  /// \param fileId the file ID
  /// \param fileVersion the file version
  /// \param diskId the disk ID
  /// \param status the new status
  /// \param message the new message
  /// \param rconnectionHandle database connection handle
  /// \return the number of updated objects (0 or 1), or -1 in case of error
  int update(const std::string& fileId,
	     int fileVersion,
	     const std::string& diskId,
	     ReplicationRequest::RequestStatus status,
	     const std::string& message,
             const ServiceConnectionSyb& rconnectionHandle);
  
  /// \brief Remove the ReplicationRequest object with given fileId
  /// \param fileId the file ID
  /// \param fileVersion the file version
  /// \param diskId the disk ID
  /// \param rconnectionHandle database connection handle
  /// \return the number of removed objects (0 or 1), or -1 in case of error
  int remove(const std::string& fileId,
	     int fileVersion,
	     const std::string& diskId,
             const ServiceConnectionSyb& rconnectionHandle);
  
  /// \brief Retrieve from the DB the ReplicationRequest objects with given status
  /// \param[in] status the request status
  /// \param[out] rs the selected replication request objects
  /// \param rconnectionHandle database connection handle
  /// \return the number of retrieved objects, or -1 in case of error
  int find(ReplicationRequest::RequestStatus status, 
           ReplicationRequest::ResultSet& rs,
           const ServiceConnectionSyb& rconnectionHandle);
  
  /// \brief Retrieve from the DB the ReplicationRequest object with given fileId
  /// \param[in] fileId the file ID
  /// \param[out] rs the selected replication request objects
  /// \param rconnectionHandle database connection handle
  /// \return the number of retrieved objects, or -1 in case of error
  int find(const std::string& fileId, 
           ReplicationRequest::ResultSet& rs,
           const ServiceConnectionSyb& rconnectionHandle);
  
  /// \brief Retrieve all ReplicationRequest objects from the DB
  /// \param[out] rs the selected replication request objects
  /// \param rconnectionHandle database connection handle
  /// \return the number of retrieved objects, or -1 in case of error
  int loadAll(ReplicationRequest::ResultSet& rs,
              const ServiceConnectionSyb& rconnectionHandle);
  
  /// the DB field name for fileId
  static const std::string fileIdField;

  /// the DB field name for fileVersion
  static const std::string fileVersionField;

  /// the DB field name for diskId
  static const std::string diskIdField;

  /// the DB field name for submitterId
  static const std::string submitterIdField;

  /// the DB field name for submissionTime
  static const std::string submissionTimeField;

  /// the DB field name for priority
  static const std::string priorityField;

  /// the DB field name for statusId
  static const std::string statusIdField;

  /// the DB field name for lastUpdateTime
  static const std::string lastUpdateTimeField;

  /// the DB field name for message
  static const std::string messageField;

  /// the DB field name for statusString
  static const std::string statusStringField;
  
protected:
  /// \brief Select ReplicationRequest objects from the DB
  /// \param[out] rs the selected replication request objects
  /// \param[in] whereClause the optional where clause
  /// \return the number of selected objects, or -1 in case of error
  int select_(const ServiceConnectionSyb& rconnectionHandle,
              ReplicationRequest::ResultSet& rs, 
              const std::string& whereClause = "");
  
  /// \brief Return a comma separated list of all fields in the replication queue table
  std::string allFields();
  
  /// \brief Convert a SybaseResultSet object to a ReplicationRequest object
  /// \param[in] res the ResultSet
  /// \param[out] rr the ReplicationRequest
  /// \return 0 if the conversion was succesful, -1 otherwise
  int convertResultSetToReplicationRequest(SybaseResultSet& res,
					   ReplicationRequest& rr);

  /// \Executes the query given in parameter
  /// \Return the number of objects issued by the query execution
  /// \or -1 in case of error.
  /// \param query query to be executed
  /// \param rconnectionHandle database connection handle
  int executeQuery(std::string query,
                   const ServiceConnectionSyb& rconnectionHandle);

  
private:
  std::string tableName_;
};

#endif // REPLICATION_REQUEST_DAO_SYB_H
