// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.4 $
// $Log: ServiceConnectionStub.h,v $
// Revision 1.4  2007/04/25 15:17:11  cmoins
// replacement of get...Field by get...Value
//
// Revision 1.3  2007/03/05 10:03:39  cmoins
// adding of executeSelect,getStringField and getLongField
//
// Revision 1.2  2007/02/08 16:24:30  cmoins
// execute returns SybaseResultSet instead of int
//
// Revision 1.1  2006/12/19 14:04:07  cmoins
// Adding of stub classes for offline tests
//
//
//  Stub Connection class
// 
//------------------------------------------------------------------------------

#ifndef SERVICE_CONNECTION_STUB_H
#define SERVICE_CONNECTION_STUB_H

#include <string>
#include <map>
#include <ConnectionPool.h>

/// database connection stub layer for sybase service classes
/**
 * The goal of this class is to implement fake databases related operations
 * to allow service layer testing independently from the database.
 */
class ServiceConnectionStub 
{
public:

  typedef map<string,int> ListOfFieldIndex;

  enum FieldType { INT_FIELD,LONG_FIELD,STRING_FIELD};
  
  /// \brief Constructor
  /// \param dbServer the DB server name
  /// \param dbUser the DB user name
  /// \param dbPassword the DB passwd
  ServiceConnectionStub(const std::string& dbServer,
	               const std::string& dbUser,
		       const std::string& dbPasswd);
  
  /// \brief Destructor
  ~ServiceConnectionStub();
  
  /// \brief Gives the access to the database
  /// \returns 0 if successful
  int acquire();

  /// \brief releases the access to the database
  /// \return 0 if successful
  int release();

  /// \brief begins a transaction
  /// \return 0 if successful
  int beginTransaction();

  /// \brief commit a transaction
  /// \return 0 if successful
  int commitTransaction();

  /// \rollbacks a transaction
  /// \return 0 if successful
  int rollbackTransaction();

  /// \performs the execution of a query
  /// \return the result set of the query
  SybaseResultSet execute(std::string query) const;

  /// \performs the execution of a query
  /// \param query query to be executed
  /// \param rrowSet result set of rows
  /// \return 0 if successful
  int executeSelect(std::string query,
                    SybaseResultSet& rres,                                                          ListOfFieldIndex& rlistOfFieldIndex) const;

  /// \Gets the value of a string type field associated to a given index
  /// \param res result set of a query where values are fetched
  /// \param fieldIndex index of the field
  /// \return the value of the field
  string getStringValue(SybaseResultSet& rres,
                        int fieldIndex) const;

 /// \Gets the value of a double type field associated to the given index
  /// \param res result set of a query where values are fetched
  /// \param fieldIndex index of the field
  /// \return the value of the field
  double getDoubleValue(SybaseResultSet& rres,
                        int fieldIndex) const;

  /// \Gets the value of a long type field associated to the given index
  /// \param res result set of a query where values are fetched
  /// \param fieldIndex index of the field
  /// \return the value of the field
  long getLongValue(SybaseResultSet& rres,
                    int fieldIndex) const;
  
private:

  std::string dbServer_;
  std::string dbUser_;
  std::string dbPasswd_;

  int conn_;
  int transactionNumber_;
};

// Inline methods implementation
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a string type field
// returns the value of the field
//------------------------------------------------------------------------------
inline 
string ServiceConnectionStub::getStringValue(SybaseResultSet& rres,
                                             int fieldIndex) const
{
  string fieldValue;
  if (rres.isNull(fieldIndex))
    {
      fieldValue = "";
    }
  else
    {
      fieldValue = rres.getStringValue(fieldIndex);
    }

  return fieldValue;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a double type field
// returns the value of the field
//------------------------------------------------------------------------------
inline 
double ServiceConnectionStub::getDoubleValue(SybaseResultSet& rres,
                                             int fieldIndex) const
{
  double fieldValue;
  if (rres.isNull(fieldIndex))
    {
      fieldValue = 0.;
    }
  else
    {
      fieldValue = rres.getDoubleValue(fieldIndex);
    }
  return fieldValue;
   
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a long type field
// returns the value of the field
//------------------------------------------------------------------------------
inline 
long ServiceConnectionStub::getLongValue(SybaseResultSet& rres,
                                         int fieldIndex) const
{
  long fieldValue;
  if (rres.isNull(fieldIndex))
    {
      fieldValue = 0;
    }
  else
    {
      fieldValue = rres.getLongValue(fieldIndex);
    }
  return fieldValue;
   
}
#endif // SERVICE_CONNECTION_STUB_H
