// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.13 $
// $Log: ServiceConnectionSyb.h,v $
// Revision 1.13  2007/11/12 14:39:15  cmoins
// adding of getDateTimevalValue
//
// Revision 1.12  2007/10/31 16:45:44  cmoins
// adding of the database name parameter to the constructor
//
// Revision 1.11  2007/10/25 09:04:37  cmoins
// adding of the database name as a parameter
//
// Revision 1.10  2007/10/12 12:33:03  cmoins
// adding of server, database and user names accessors
//
// Revision 1.9  2007/07/25 07:28:29  cmoins
// doxygen comments correction
//
// Revision 1.8  2007/04/27 13:25:46  cmoins
// adding of getDateStringValue read dates
//
// Revision 1.7  2007/04/25 15:19:31  cmoins
// adding of readDBParameters
//
// Revision 1.6  2007/03/05 10:03:39  cmoins
// Cancellation of useless include
//
// Revision 1.5  2007/02/13 08:22:14  cmoins
// Transactions related operations become const
//
// Revision 1.4  2007/02/05 16:43:29  cmoins
// Creation of dppacker persistent layer access
//
// Revision 1.3  2006/12/15 15:27:54  cmoins
// DAO & Service design modification
//
// Revision 1.2  2006/12/14 14:03:16  cmoins
// cvs header adding
//
//
// Sybase database connection layer for sybase service classes
// 
//------------------------------------------------------------------------------

#ifndef SERVICE_CONNECTION_SYB_H
#define SERVICE_CONNECTION_SYB_H

#include <string>
#include <map>
#include <ctime>
#include <ConnectionPool.h>
#include <SybUtils.h>

/// Sybase database connection layer for sybase service classes
/**
 * The goal of this class is to handle the acquisition and the 
 * release of a connection to a database along with transactions.
 * It provides also methods to access values in the result set of a query.
 */
class ServiceConnectionSyb 
{
public:
  
  /// Type that associated to the field of a query an index
  typedef map<string,int> ListOfFieldIndex; 
  /// Set of results type
  typedef SybaseResultSet ResultSet;
  /// enum type of available fields
  enum FieldType { INT_FIELD,LONG_FIELD,STRING_FIELD};

  /// \brief Constructor
  /** 
   * \param rdbServer   DB server name
   * \param rdbUser     DB user name
   * \param rdbPasswd   DB password
   * \param rdbName     DB name
   */
  ServiceConnectionSyb(const std::string rdbServer,
	               const std::string rdbUser,
		       const std::string rdbPasswd,
		       const std::string rdbName="NONE");

  /// \brief Constructor
  /// \param dbAlias database alias to retrieve keywords parameter
  ServiceConnectionSyb(const std::string dbAlias);
  
  /// \brief Destructor
  ~ServiceConnectionSyb();

  /// \brief Gives the access to the database
  /// \returns 0 if successful
  int acquire();

  /// \brief Releases the access to the database
  /// \return 0 if successful
  int release();

  /// \brief Begins a transaction
  /// \return 0 if successful
  int beginTransaction() const;

  /// \brief Commit a transaction
  /// \return 0 if successful
  int commitTransaction() const;

  /// \brief Rollbacks a transaction
  /// \return 0 if successful
  int rollbackTransaction() const;

  /// \brief Performs the execution of a query
  /// \return the result set of the query
  ResultSet execute(std::string query) const;

  /// \brief Performs the execution of a query
  /**
   * Gives back a list of index for the fields specified in the query.
   * This list will be used afterwards to retrieve values in the 
   * result set.
   * \param query query to be executed
   * \param rres result set of rows
   * \param rlistOfFieldIndex list of field indexes in the result set
   * \return 0 if successful
   */
  int executeSelect(std::string query,
                    ResultSet& rres,
		    ListOfFieldIndex& rlistOfFieldIndex) const;

  /// gives the name of the database server
  std::string getServerName() { return dbServer_;}
  /// gives the name of the database
  std::string getDbName() { return dbName_;}
  /// gives the user name
  std::string getDbUser() { return dbUser_;}

  /// \brief Gets the string value associated to the field index in a result set
  /** \param rres result set of a query where values are fetched
   * \param fieldIndex index of the field
   * \return the value of the field
   */
  string getStringValue(ResultSet& rres,
                        int fieldIndex) const;


  /// \brief Gets the date string associated to the field index in a result set
  /** \param rres result set of a query where values are fetched
   * \param fieldIndex index of the field
   * \return the value of the field
   */
  string getDateStringValue(ResultSet& rres,
			    int fieldIndex) const;

  /// \brief Gets the timeval date associated to the field index in a result set
  /** 
   * \param rres result set of a query where values are fetched
   * \param fieldIndex index of the field
   * \return the value of the field
   */
  struct timeval getDateTimevalValue(ResultSet& rres,
			    int fieldIndex) const;

  /// \brief Gets the double value associated to the field index in a result set
  /** \param rres result set of a query where values are fetched
   *  \param fieldIndex index of the field
   *  \return the value of the field
   */
  double getDoubleValue(ResultSet& rres,
                        int fieldIndex) const;

  /// \brief Gets the long value associated to the field index in a result set
  /** \param rres result set of a query where values are fetched
   *  \param fieldIndex index of the field
   *  \return the value of the field
   */
  long getLongValue(ResultSet& rres,
                    int fieldIndex) const;

private:

  /// \brief Reads database connection parameters
  /** reads the dbrc file located in the home directory to retrieve
   *  the connection parameters. Shall be called if the object was instantiated
   *  with a database alias
   *  \return 0 if successful
   */
  int readDBParameters();

  /// name of the database server
  std::string dbServer_; 
  /// name of the database
  std::string dbName_;
  /// name of the database user
  std::string dbUser_;   
  /// password to connect to the database
  std::string dbPasswd_; 
   /// database alias to retrieve connection parameters
  std::string  dbAlias_; 
  /// number of the most recent pending transaction
  mutable int transactionNumber_;  
  /// sybase database connection handle
  SybaseConnection* conn_;   
};

// Inline methods implementation

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a string type field
//------------------------------------------------------------------------------
inline 
string ServiceConnectionSyb::getStringValue(ResultSet& rres,
                                            int fieldIndex) const
{
  string fieldValue;
  if (rres.isNull(fieldIndex))
    {
      fieldValue = "";
    }
  else
    {
      fieldValue = rres.getStringValue(fieldIndex);
    }

  return fieldValue;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a date type field expressed as time val
//------------------------------------------------------------------------------
inline 
struct timeval ServiceConnectionSyb::getDateTimevalValue(ResultSet& rres,
							 int fieldIndex) const
{
  timeval fieldValue;
  rres.getDateAsTimevalValue(fieldIndex,fieldValue);
  return fieldValue;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a date string type field
//------------------------------------------------------------------------------
inline 
string ServiceConnectionSyb::getDateStringValue(ResultSet& rres,
						int fieldIndex) const
{
  string fieldValue;
  fieldValue = rres.getDateStringValue(fieldIndex);
    

  return fieldValue;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a double type field
//------------------------------------------------------------------------------
inline 
double ServiceConnectionSyb::getDoubleValue(ResultSet& rres,
                                            int fieldIndex) const
{
  double fieldValue;
  if (rres.isNull(fieldIndex))
    {
      fieldValue = 0.;
    }
  else
    {
      fieldValue = rres.getDoubleValue(fieldIndex);
    }
  return fieldValue;
   
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gets the value of a long type field
//------------------------------------------------------------------------------
inline 
long ServiceConnectionSyb::getLongValue(ResultSet& rres,
                                        int fieldIndex) const
{
  long fieldValue;
  if (rres.isNull(fieldIndex))
    {
      fieldValue = 0;
    }
  else
    {
      fieldValue = rres.getLongValue(fieldIndex);
    }
  return fieldValue;
   
}

#endif // SERVICE_CONNECTION_SYB_H
