/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.16 $
// $Log: StorageService.h,v $
// Revision 1.16  2008/01/28 15:19:49  cmoins
// asto database becomes configurable
//
// Revision 1.15  2007/11/05 15:39:49  mpadovan
// Added new constructor overload that accepts ServerName, DBName, UserName,
// and password.
//
// Revision 1.14  2007/08/14 16:29:22  cmoins
// adding of a method to retrieve a media request content
//
// Revision 1.13  2007/08/01 16:13:43  cmoins
// adding of supend files and restore files functionalities
//
// Revision 1.12  2007/07/25 07:28:43  cmoins
// doxygen comments correction
//
// Revision 1.11  2007/07/12 08:15:46  cmoins
// in case a media request is already existing, checks that no action is
// associted to it
//
// Revision 1.10  2007/07/10 09:02:44  cmoins
// a debug message is displayed if no file is retrieved for a media request
//
// Revision 1.9  2007/07/05 13:52:27  cmoins
// DFS04061 : adding of a delete file method
//
// Revision 1.8  2007/07/05 09:19:10  cmoins
// DFS04059 : adding of findAllMediaType
//
// Revision 1.7  2007/07/04 12:22:43  cmoins
// taking in accound of MediaDAO::insert and MediaFileDAO::insert returning
// number of inserted rows
//
// Revision 1.6  2007/07/04 11:52:18  cmoins
// taking in account of mediaDAO.find return value change
//
// Revision 1.5  2007/05/24 11:35:58  cmoins
// media action handlement modification to allow different process to use the same media request id
//
// Revision 1.4  2007/05/15 10:05:39  cmoins
// adding of closeMediaInsert + replacement of info logging messages by debug ones
//
// Revision 1.3  2007/04/27 15:41:45  cmoins
// adding of media action tracking
//
// Revision 1.2  2007/04/26 15:20:17  cmoins
// modification of doxygen comments
//
// Revision 1.1  2007/04/25 15:20:01  cmoins
// creation of a service layer associated to the media request data repository
//
// 
// 
//------------------------------------------------------------------------------

#ifndef STORAGE_SERVICE_H
#define STORAGE_SERVICE_H

#include <string>
#include <DfsLog.h>
#include "Media.h"
#include "MediaFile.h"
#include "MediaAction.h"
#include "MediaType.h"
#include "dataAccessDef.h"

using namespace std;

/// Data Access Service layer for media requests information
/**
 * Data Access Service layer that deals with information about media requests 
 * and the files they are associated to.
 * It relies on two tables :
 * - one storing data about a media
 * - one storing informations about files stored on a media.
 *
 * There are four template parameters so that the layer remains independent of
 * any storage system or DAO class :
 * - ConnectionHandle : type of connection handle to the data repository
 * - MediaDAO : Media request DAO class
 * - MediaFileDAO : Media request files DAO class
 * - keyDAO : media request identifier DAO class
 */
template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO> 
class StorageService
{
public:

 
  /// \brief Constructor
  /// \param connAlias data repository connection keyword
  StorageService(string connAlias);
  
  /// \brief Constructor
  /**
   * \param dbServer DB server name
   * \param dbUser   DB user name
   * \param dbPasswd DB password
   * \param dbName   DB name
   */
  StorageService(	std::string dbServer,
		   			std::string dbUser,
		   			std::string dbPasswd,
		   			std::string dbName	);   

  /// \brief Destructor
  ~StorageService(){};

  
   /// \brief inserts a media in the data repository
   /**
   * \param rmedia media to be inserted 
   * \return 0 if the operation was successful, -1 otherwise
   */
  int insertMedia(Media& rmedia) const;

  /**
   * Performs all the operations that should be carried out
   * when the insertion of a media and its files is completed. 
   * \param rmedia media to be closed
   * \return 0 if the operation was successful, -1 otherwise
   */
  int closeMediaInsert(Media& rmedia) const;
  
  /**
   * Inserts informations about files associated to a media request.
   * This insertion is performed inside a transaction to keep 
   * tables consistency.
   * \param rmedia media whose files shall be inserted 
   * \param rmediaFileSet files  to be inserted
   * \return 0 if the operation was successful, -1 otherwise
   */
  int insertMediaFiles(const Media& rmedia,
		       MediaFile::ResultSet& rmediaFileSet) const ;

  /// \brief finds the content of a media request
  /**
   * \param mediaName name of the media request to retrieve
   * \param rmedia media data
   * \return the number of retrieved media if successful, -1 otherwise
   */
  int findMedia(string mediaName,
		Media& rmedia) const;

  /// \brief finds a list of files associated to a request along with its data
  /**
   * \param mediaName name of the media to retrieve
   * \param rmedia media data
   * \param rdataFileSet set of files associated to the media
   * \return the number of files if successful, -1 otherwise
   */
  int findMediaFiles(string mediaName,
		     Media& rmedia,
		     MediaFile::ResultSet& rdataFileSet) const;

  /// \brief removes a file from the data repository
  /**
   * \param mediaFileId name of the file to be deleted
   * \return the number of deleted files if successful, -1 otherwise
   */
  int deleteFile(string mediaName,
		 string mediaFileId) const;

  /// \brief suspends a file from a media
  /**
   * \param mediaFileId name of the file to be suspended
   * \return the number of suspended files if successful, -1 otherwise
   */
  int suspendFile(string mediaName,
		  string mediaFileId) const;

  /// \brief restores a file from a media
  /**
   * \param mediaFileId name of the file to be restored
   * \return the number of restored files if successful, -1 otherwise
   */
  int restoreFile(string mediaName,
		  string mediaFileId) const;

  /// \brief retrieves the whole set of available media types.
  /**
   * \param rmediaTypeSet set of available media types
   * \param rconnectionHandle data repository connection handle
   * \return the number of retrieved media types if successful, -1 otherwise 
   */
  int findAllMediaType(MediaType::ResultSet& rmediaTypeSet) const;

private :

  typedef MediaActionDAO<ConnectionHandle> MediaActionDAOCH;

  /// media table name
  static const string MEDIA_TABLE;       
  /// media files table name
  static const string MEDIA_FILES_TABLE; 
  /// media action table
  static const string MEDIA_ACTION_TABLE;
  /// media type table
  static const string MEDIA_TYPE_TABLE;
  /// identifier table name 
  static const string KEY_TABLE;         

  /// data repository connection keyword
  //string connAlias_;  
  /// Connection handle to the data repository
  mutable ConnectionHandle dsConnection_;
  /// Media request DAO class
  MediaDAO<ConnectionHandle> mediaDAO_; 
  /// Media files DAO class
  MediaFileDAO<ConnectionHandle> mediaFileDAO_; 
  /// Media action DAO class
   MediaActionDAOCH mediaActionDAO_;
  /// Media type DAO class
  MediaTypeDAO<ConnectionHandle> mediaTypeDAO_; 
  /// Media request id DAO class
  KeyDAO<ConnectionHandle> keyDAO_; 
};

// static constant creation
template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
 	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO> 
const string StorageService<ConnectionHandle, MediaDAO, MediaFileDAO,
			    MediaActionDAO,MediaTypeDAO,KeyDAO>::
MEDIA_TABLE = "media";

template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO> 
const string StorageService<ConnectionHandle, MediaDAO, MediaFileDAO,
			    MediaActionDAO,MediaTypeDAO,KeyDAO>::
MEDIA_FILES_TABLE = "media_files";

template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO> 
const string StorageService<ConnectionHandle, MediaDAO, MediaFileDAO,
			    MediaActionDAO,MediaTypeDAO,KeyDAO>::
MEDIA_ACTION_TABLE = "media_actions";

template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO> 
const string StorageService<ConnectionHandle, MediaDAO, MediaFileDAO,
			    MediaActionDAO,MediaTypeDAO,KeyDAO>::
MEDIA_TYPE_TABLE = "mediaTypes";

template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO> 
const string StorageService<ConnectionHandle, MediaDAO, MediaFileDAO,
			    MediaActionDAO,MediaTypeDAO,KeyDAO>::
KEY_TABLE = "keys";

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
// Note : The database is configurable
//------------------------------------------------------------------------------
template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
StorageService(string connAlias):
  dsConnection_(connAlias),
  mediaDAO_(dsConnection_.getDbName() + ".." + MEDIA_TABLE),
  mediaFileDAO_(dsConnection_.getDbName() + ".." + MEDIA_FILES_TABLE),
  mediaActionDAO_(dsConnection_.getDbName() + ".." + MEDIA_ACTION_TABLE),
  mediaTypeDAO_(dsConnection_.getDbName() + ".." + MEDIA_TYPE_TABLE),
  keyDAO_(dsConnection_.getDbName() + ".." + KEY_TABLE)
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor
// Note : The database is configurable
//------------------------------------------------------------------------------
template <class ConnectionHandle, 
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
StorageService(	std::string dbServer,
				std::string dbUser,
				std::string dbPasswd,
				std::string dbName	):
  dsConnection_(dbServer, dbUser, dbPasswd, dbName),
  mediaDAO_(dbUser + ".." + MEDIA_TABLE),
  mediaFileDAO_(dbUser + ".." + MEDIA_FILES_TABLE),
  mediaActionDAO_(dbUser + ".." + MEDIA_ACTION_TABLE),
  mediaTypeDAO_(dbUser + ".." + MEDIA_TYPE_TABLE),
  keyDAO_(dbUser + ".." + KEY_TABLE)
{
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Creates and identifier for a media and inserts it  in the database.
// Performs an entry in the database to keep a track of the current action 
// associated to the media.
// The identifier creation is performed inside a transaction so that an id is
// released if the insertion of the media failed
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
insertMedia(Media& rmedia) const 
{
  int retStatus = 0;

  DFS_TRACE("StorageService::insertMedia");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      MediaFile::ResultSet dataFileSet;
      int nbMedia = mediaDAO_.findMedia(rmedia.mediaRequestName_,
					rmedia,
					dsConnection_);
      if (nbMedia == 0)
	{
	  // If no retrieval has been performed yet for this media request,
	  // an identifier is created for this media which is inserted 
	  // in the database
          // beginning of the transaction
	  dsConnection_.beginTransaction();
	  rmedia.mediaRequestId_ = keyDAO_.getMediaId(dsConnection_);
	
	  if (rmedia.mediaRequestId_ != -1 &&
	      mediaDAO_.insert(rmedia,
			       dsConnection_)>=0)
	    {
	      rmedia.mediaAction_ = 
		MediaAction(rmedia.mediaRequestId_,
			    dataAccessDef::NO_INIT_STRING_FIELD,
			    MediaActionDAOCH::INGESTION_ACTION);
	      if (mediaActionDAO_.insert(rmedia.mediaAction_,
					 dsConnection_)>=0)
		{
		  dsConnection_.commitTransaction();
		  DFS_DEBUG("media creation and action insertion commited");
		}
	      else
		{
		  dsConnection_.rollbackTransaction();
		  DFS_ERROR("Rollback performed for media creation and"\
			    "action insertion");
		  retStatus = -1;
		}
	    }
	  else
	    {
	      dsConnection_.rollbackTransaction();
	      DFS_ERROR("Rollback performed for media creation and"\
			"action insertion");
	      retStatus = -1;
	    }
	}
      else if (nbMedia > 0)
	{
	  // a media request has already been inserted

	  MediaAction::ResultSet mediaActionSet;
	  if (!mediaActionDAO_.findMediaAction(rmedia.mediaRequestId_,
					       mediaActionSet,
					       dsConnection_))
	    {
	      // there is no pending action for this media request
	      // inserts a new action
	      rmedia.mediaAction_ = 
		MediaAction (rmedia.mediaRequestId_,
			     dataAccessDef::NO_INIT_STRING_FIELD,
			     MediaActionDAOCH::INGESTION_ACTION );
	      if (mediaActionDAO_.insert(rmedia.mediaAction_,
					 dsConnection_) < 0)
		retStatus = -1;
	    }
	  else
	    {
	      DFS_ERROR("There are pending actions for media request %d",
			rmedia.mediaRequestId_);
	      retStatus = -1;
	    }
	}
      else
	{
	  DFS_ERROR("Could not get any data related to %d",
		    rmedia.mediaRequestId_);
	  retStatus = -1;
	}
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::Insert.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Performs all the operations that should be carried out
// when the insertion of a media and its files is completed. 
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int 
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
closeMediaInsert(Media& rmedia) const
{
  int retStatus = 0;
  DFS_TRACE("StorageService::closeMediaInsert");
  //ConnectionHandle dsConnection(connAlias_);

  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      rmedia.mediaAction_ = 
	MediaAction (rmedia.mediaRequestId_,
		     dataAccessDef::NO_INIT_STRING_FIELD,
		     MediaActionDAOCH::INGESTION_ACTION );
      // removes from the data repository any action track linked to the request
      if (mediaActionDAO_.deleteMediaAction(rmedia.mediaAction_,
					    dsConnection_) < 0)
	retStatus = -1;
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::closeMediaInsert could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}
  

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Inserts informations about files associated to a media request and deletes
// the ingestion action in the database once all the files have been ingested
// This insertion is performed inside a transaction to keep tables consistency.
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
insertMediaFiles(const Media& rmedia,
		 MediaFile::ResultSet& rmediaFileSet) const
{
  int retStatus = 0;

  DFS_TRACE("StorageService::insertMediaFiles");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      // beginning of the transaction
      dsConnection_.beginTransaction();
    
      for (MediaFile::pMediaFile pcmediaFile = rmediaFileSet.begin();
	   pcmediaFile != rmediaFileSet.end();pcmediaFile++)
	{
	  pcmediaFile->mediaRequestId_ = rmedia.mediaRequestId_;
	  if (mediaFileDAO_.insert(*pcmediaFile,
				   dsConnection_)>= 0)
	    {
	      DFS_DEBUG("Insert succeeded for %s",
		        pcmediaFile->fileId_.c_str());
	    }
	  else
	    {
	      DFS_ERROR("Insert failed for %s",
			pcmediaFile->fileId_.c_str());
	    }
	}
      if (!retStatus)
	{
	  dsConnection_.commitTransaction();
	  DFS_DEBUG("Files Insert succeeded for %s",
		     rmedia.mediaRequestName_.c_str());
	}
    
      else
	{
	  dsConnection_.rollbackTransaction();
	  DFS_ERROR("Files Insert failed for %s",
		    rmedia.mediaRequestName_.c_str());
	  retStatus = -1;
	}
	
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::Insert.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// finds the content of a media request
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
findMedia(string mediaName,
	  Media& rmedia) const 
{
  int retStatus = 0;

  DFS_TRACE("StorageService::findMedia");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      retStatus = mediaDAO_.findMedia(mediaName,
				      rmedia,
				      dsConnection_);
      if (retStatus == 1)
	{
	  DFS_DEBUG("Media request %s successfully retrieved",
		    mediaName.c_str());
	}
      else if (retStatus == 0)
	{
	  DFS_DEBUG("no media request %s ",mediaName.c_str());
	}
      else if (retStatus > 1)
	{
	  DFS_ERROR("More than one media request associated to %s",
		    mediaName.c_str());
	  retStatus = -1;
	}
      else if (retStatus < 0)
	{
	  DFS_ERROR("An error occured when retrieving %s",mediaName.c_str());
	  retStatus = -1;
	}
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::findMediaFiles.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// finds a list of files associated to a media request
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
findMediaFiles(string mediaName,
	       Media& rmedia,
	       MediaFile::ResultSet& rdataFileSet) const
{
  int retStatus = 0;

  DFS_TRACE("StorageService::findMediaFiles");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      retStatus = mediaDAO_.findMedia(mediaName,
				      rmedia,
				      dsConnection_);
      if (retStatus > 0)
	{
	  // the media request has been retrieved
	  retStatus = mediaFileDAO_.findMediaFiles(rmedia.mediaRequestId_,
						   rdataFileSet,
						   dsConnection_);
	  if (retStatus >0)
	    {
	      DFS_DEBUG("Files of media request %s successfully retrieved",
		        mediaName.c_str());
	    }
	  else if (retStatus == 0)
	    {
	      DFS_DEBUG("No file retrieved for %s",mediaName.c_str());
	      retStatus = 0;
	    }
	  else if (retStatus < 0)
	    {
	      DFS_ERROR("An error occured when retrieving %s files",
			mediaName.c_str());
	      retStatus = -1;
	    }
	}
      else if (retStatus == 0)
	{
	  DFS_DEBUG("media request %s not retrieved",mediaName.c_str());
	}
      else if (retStatus < 0)
	{
	  DFS_ERROR("An error occured when retrieving %s",mediaName.c_str());
	}
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::findMediaFiles.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// removes a file from the data repository
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
deleteFile(string mediaName,
	   string mediaFileId) const
{
  int retStatus = 0;

  DFS_TRACE("StorageService::deleteFile");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      Media media;
      retStatus = mediaDAO_.findMedia(mediaName,
				      media,
				      dsConnection_);
      if (retStatus > 0)
	{
	  // the media request has been retrieved
	  retStatus = mediaFileDAO_.deleteFile(media.mediaRequestId_,
					       mediaFileId,
					       dsConnection_);
	  if (retStatus >0)
	    {
	      DFS_DEBUG("File with identifier %s of %s successfully deleted",
			mediaFileId.c_str(),mediaName.c_str());
	    }
	  else if (retStatus == 0)
	    {
	      DFS_DEBUG("File with identifier %s not deleted",
			mediaFileId.c_str());
	    }
	  else
	    {
	      DFS_ERROR("An error occured when deleting %s",
			mediaFileId.c_str());
	      retStatus = -1;
	    }
	}
      else if (retStatus == 0)
	{
	  DFS_DEBUG("media request %s not retrieved",mediaName.c_str());
	}
      else if (retStatus < 0)
	{
	  DFS_ERROR("An error occured when retrieving %s",mediaName.c_str());
	}
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::deleteFile.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// suspends a file from a media
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
suspendFile(string mediaName,
	   string mediaFileId) const
{
  int retStatus = 0;

  DFS_TRACE("StorageService::suspendFile");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      Media media;
      retStatus = mediaDAO_.findMedia(mediaName,
				      media,
				      dsConnection_);
      if (retStatus > 0)
	{
	  // the media request has been retrieved
	  retStatus = mediaFileDAO_.suspendFile(media.mediaRequestId_,
						mediaFileId,
						dsConnection_);
	  if (retStatus >0)
	    {
	      DFS_DEBUG("File with identifier %s of %s successfully suspended",
			mediaFileId.c_str(),mediaName.c_str());
	    }
	  else if (retStatus == 0)
	    {
	      DFS_DEBUG("File with identifier %s not suspended",
			mediaFileId.c_str());
	    }
	  else
	    {
	      DFS_ERROR("An error occured when suspending %s",
			mediaFileId.c_str());
	      retStatus = -1;
	    }
	}
      else if (retStatus == 0)
	{
	  DFS_DEBUG("media request %s not retrieved",mediaName.c_str());
	}
      else if (retStatus < 0)
	{
	  DFS_ERROR("An error occured when retrieving %s",mediaName.c_str());
	}
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::suspendFile.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// restores a file in a media
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
restoreFile(string mediaName,
	    string mediaFileId) const
{
  int retStatus = 0;

  DFS_TRACE("StorageService::restoreFile");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      Media media;
      retStatus = mediaDAO_.findMedia(mediaName,
				      media,
				      dsConnection_);
      if (retStatus > 0)
	{
	  // the media request has been retrieved
	  retStatus = mediaFileDAO_.restoreFile(media.mediaRequestId_,
						mediaFileId,
						dsConnection_);
	  if (retStatus >0)
	    {
	      DFS_DEBUG("File with identifier %s of %s successfully restored",
			mediaFileId.c_str(),mediaName.c_str());
	    }
	  else if (retStatus == 0)
	    {
	      DFS_DEBUG("File with identifier %s not restored",
			mediaFileId.c_str());
	    }
	  else
	    {
	      DFS_ERROR("An error occured when restoring %s",
			mediaFileId.c_str());
	      retStatus = -1;
	    }
	}
      else if (retStatus == 0)
	{
	  DFS_DEBUG("media request %s not retrieved",mediaName.c_str());
	}
      else if (retStatus < 0)
	{
	  DFS_ERROR("An error occured when retrieving %s",mediaName.c_str());
	}
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::restoreFile.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// retrieves the whole set of available media types.
//------------------------------------------------------------------------------
template <class ConnectionHandle,
          template <typename> class MediaDAO,
	  template <typename> class MediaFileDAO,
	  template <typename> class MediaActionDAO,
	  template <typename> class MediaTypeDAO,
	  template <typename> class KeyDAO>
int
StorageService<ConnectionHandle, MediaDAO,MediaFileDAO,MediaActionDAO,
	       MediaTypeDAO,KeyDAO>::
findAllMediaType(MediaType::ResultSet& rmediaTypeSet) const
{
  int retStatus = 0;

  DFS_TRACE("StorageService::findAllMediaType");

  //ConnectionHandle dsConnection(connAlias_);
  int connectionStatus = dsConnection_.acquire();
  if (!connectionStatus)
    {
      retStatus = mediaTypeDAO_.findAllMediaType(rmediaTypeSet,
						 dsConnection_);
      if (retStatus > 0)
	{
	  DFS_DEBUG("Media types successfully retrieved");
	}
      else if (retStatus == 0)
	{
	  DFS_DEBUG("No Media type retrieved");
	}
      else if (retStatus < 0)
	{
	  DFS_ERROR("An error occured when retrieving media types");
	}
      dsConnection_.release();
    }
  else
    {
      DFS_ERROR("StorageService::findAllMediaType.Could not acquire connection");
      retStatus = -1;
    }
  return retStatus;
}

#endif // STORAGE_SERVICE_H
