//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Michael Wiedmer - ESO/DMD/DPG>
//SUBSYSTEM: <Core>
//
//$Revision: 1.1 $
//$Log: cor_assert.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.3  1998/06/30 08:49:54  dataflow
//Removing $NAME
//
//Revision 1.2  1997/12/03 15:05:47  dfadmin
//update for NTT Jan 98
//
//Revision 1.8  1997/07/11 08:34:23  mwiedmer
//move code into CorDebug
//
//Revision 1.7  1997/04/11 13:32:06  mwiedmer
//use RWStrings
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.6  1996/10/17 12:57:27  mwiedmer
//remove compiler warnings
//
//Revision 1.5  1996/09/10 11:22:22  mwiedmer
//update debugging and debug printing
//mechanism
//
//Revision 1.4  1996/09/10 09:11:13  mwiedmer
//updated debugging mechanism
//
//Revision 1.3  1996/09/09 13:03:48  mwiedmer
//standardised
//
//Revision 1.2  1996/09/03 14:41:27  mwiedmer
//added RCS Name tag
//
//Revision 1.1  1996/09/02 11:04:05  dfadmin
//First release
//
//Revision 1.3  1996/09/02 09:45:36  mwiedmer
//first release
//
//Revision 1.2  1996/09/02 08:25:49  mwiedmer
//*** empty log message ***
//
//-----------------------------------------------------------------------------

#ifndef COR_ASSERT_H
#define COR_ASSERT_H

// includes from this library
#include <cor_debug.h>
// includes from other ESO libraries
// includes from other libraries
// system includes
#include <stdlib.h> // getenv()

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations


//=============================================================================
// Assertion utilities 
//
void corReportAssertFailure(const char*, int, const char*);

#define COR_ASSERT_REGARDLESS(ASSERTION,EXPLANATION) \
  if (ASSERTION) ; else \
    CorBase::reportAssertFailure(__FILE__, __LINE__, EXPLANATION)


//=============================================================================
// Assertion mechanism
#ifdef COR_USE_ASSERT


//@Man: COR_ASSERT_X()
//@Type: MACROS
//@See:  CorAssertHandler, COR_ASSERT
//@Memo: assertion macros
/**      This set of macros is provided to faciliate the use of assertions
         in a standard and uniform way. For a complete description look at
         the COR_ASSERT macro documentation.
*/

//@{

/// Assertion checking mechanism
#define COR_ASSERT(ASSERTION,EXPLANATION) \
  COR_ASSERT_REGARDLESS(ASSERTION,EXPLANATION)
/**
  {\footnotesize $Id: cor_assert.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <cor_assert.h> 

    COR_ASSERT(expression, explanation);
  \end{verbatim}

  Where \\
  'expression' is of type int (or better still, CorBool) \\
  'explanation' is printed out whenever 'expression' is FALSE


  {\large DESCRIPTION} \\
  A number of specific assertion categories are provided via other
  macros of form COR_ASSERT_<category>, some of which do not require
  'expression' because they assume it is already known to be FALSE.
  These should be used wherever possible since they provide more information
  to programmers and testers.

  The application can request that its own function handle an assertion
  failure - see cor_assert_handler.h

  *** Compile with option -DDEBUG if assertion checking is required

  {\large CAUTIONS} \\
  Warning!  By default a message is printed to standard error.  This is done
  using 'printf' instead of the 'stream' stuff in case 'main' is not written
  in C++ (in which case 'cerr' will not be initialized).  As a consequence,
  the IO may not be synchronized with stream IO.

  {\large EXAMPLES} \\
  \begin{verbatim}
    #include <cor_assert.h> 
    #include <CorBase.h> 

    CorClass MyClass : public CorBase {
    public:
      MyClass() : state(0) {}
      ~MyClass(){}
      void foo()
      {
        COR_ASSERT(state == 0, "Invalid state.");
      }
    protected:
      int state;
    };
  \end{verbatim}

  {\large SEE ALSO} \\
  \Ref{CorAssertHandler}
*/

#else

#define COR_ASSERT(ASSERT,EXPLANATION)

#endif  // DEBUG defined



/// Assertion always fails
#define COR_ASSERT_FAIL(EXPLANATION) \
  COR_ASSERT(0, EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through prohibited call
#define COR_ASSERT_PROHIBITED(EXPLANATION) \
  COR_ASSERT_FAIL("Call to prohibited function | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through base member
#define COR_ASSERT_MUST_OVERRIDE(EXPLANATION) \
  COR_ASSERT_FAIL("Member must be implemented in derived class | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through non-implemented function
#define COR_ASSERT_NOT_IMPLEMENTED(EXPLANATION) \
  COR_ASSERT_FAIL("Function not implemented | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through unknown/unhandled enumerator
#define COR_ASSERT_BAD_ENUMERATOR(EXPLANATION) \
  COR_ASSERT_FAIL("Unknown enumerator constant | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/


/// Assertion through invalid bool value
#define COR_ASSERT_BOOL(B,EXPLANATION) \
  COR_ASSERT(B==corTRUE || B==corFALSE, \
             "Corrupted boolean variable | " EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through incorrect state
#define COR_ASSERT_STATE(BOOL,EXPLANATION) \
  COR_ASSERT(BOOL, "Object in incorrect state for operation | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through invalid argument value
#define COR_ASSERT_ARGUMENT(BOOL,EXPLANATION) \
  COR_ASSERT(BOOL, "Argument out of range | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion though inconstistent class data
#define COR_ASSERT_CONSISTENT(BOOL,EXPLANATION) \
  COR_ASSERT(BOOL, "Class data is inconsistent | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through bad algorithm
#define COR_ASSERT_ALGORITHM(BOOL,EXPLANATION) \
  COR_ASSERT(BOOL, "Algorithm failure | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

/// Assertion through zero denominator
#define COR_ASSERT_DENOMINATOR(BOOL,EXPLANATION) \
  COR_ASSERT(BOOL, "Zero denominator in division | "EXPLANATION)
/**
  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
*/

//@}


#endif

// Set mode for emacs.
// Local Variables:
// mode:C++
// End:
