package org.eso.ohs.core.dbb.server;

// $Id: DbbSqlCompositeKey.java,v 1.3 2006/04/26 09:26:44 tcanavan Exp $

import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/** 
 * This class defines the a composite key
 * for defining a Join, possibly an left
 * outside join, containing one or more
 * fields.
 *
 * @author Nick Kornweibel for ESO
 * 
 **/
public class DbbSqlCompositeKey extends DbbSqlKey {

    private static Logger stdlog_ = 
        Logger.getLogger( DbbSqlCompositeKey.class );

    /** From (primary key) table column linked in the join */  
    Vector<String> pkColumns_ = new Vector<String>();

    /** To (foreign key) table column linked in the join */  
    Vector<String> fkColumns_ = new Vector<String>();
    
    /** does this key specify a left outer join */
    boolean leftOutsideJoin_ = false;

    //----------------------------------------
    // Constructors
    //----------------------------------------


    /**
     * Constructor called by the end application to build a preferred
     * key. 
     *
     * @param key_name	Key name (should be unique).
     * @param from_tbl	DbbSqlTable indicating the primary key table.
     * @param from_cols[]	String array of keys in from table.
     * @param to_tbl	DbbSqlTable indicating the foreign key table.
     * @param to_cols[]	String array of keys in the to table
     * @param leftOutsideJoin boolean: if true, this is a left outside 
     *        join, otherwise it is a direct join.
     * Assumes that the cols[] arrays are of equal length.
     */
    public DbbSqlCompositeKey(String key_name, 
		     DbbSqlTable from_tbl,
		     String from_cols[],
		     DbbSqlTable to_tbl,
		     String to_cols[],
		     boolean leftOutsideJoin) {

        super(key_name, from_tbl, from_cols[0], to_tbl, to_cols[0]);
    	
    	for (int i = 0; i < from_cols.length; i++) {
    	   pkColumns_.addElement(from_cols[i]);
           fkColumns_.addElement(to_cols[i]);
        }
    
    	/* Create a unique key name. */
    	name_ = generateKeyName();
    	
        leftOutsideJoin_ = leftOutsideJoin;
    }

    //----------------------------------------
    // Private methods
    //----------------------------------------

    /**
     * @return A unique key name, dynamically created from the tables and
     * the columns.
     * 
     * @param from_tbl	DbbSqlTable indicating the primary key table.
     * @param from_col	String indicating the primary key table column.
     * @param to_tbl	DbbSqlTable indicating the foreign key table.
     * @param to_col	String indicating the foreign key table column.
     */
    private String generateKeyName() {

    	StringBuffer key_name = new StringBuffer("compKey_");
        key_name.append(pkTable_.get());
        for (int i = 0; i < pkColumns_.size(); i++) {
            key_name.append("_"+pkColumns_.elementAt(i));
        }
        key_name.append(fkTable_.get());
        for (int i = 0; i < fkColumns_.size(); i++) {
            key_name.append("_"+fkColumns_.elementAt(i));
        }
    	
    	return key_name.toString();
    }
    
    /**  
     * Return a string representation of the sql join identified by
     * the object.
     *
     * @return Sql join.  
     */
    private String getJoinClause(String pkC, String fkC) {
        String op = leftOutsideJoin_ ? "*=" : "=";
        String ret = pkTable_ + "." + pkC + " "+op+" " +
            fkTable_ + "." + fkC;
        return ret;
    }

    
    //----------------------------------------
    // Public methods
    //----------------------------------------

    /** @return name_ */
    public String getName() { return name_; }

    /** @return from_tbl_ */
    public DbbSqlTable getPkTable() { return pkTable_; }


    /** @return to_tbl_ */
    public DbbSqlTable getFkTable() { return fkTable_; }

    /**
     * Redefine Object's method equals in order to make it a suitable
     * class of objects to be stored into stacks or vectors.<br>
     * Two DbbSqlKey are equals if they have the same name (the name of
     * a key should be unique).
     *
     * @param obj	Object
     * @return Boolean value: true if it's equal, false otherwise.  
     */
    public boolean equals(Object obj) {

    	DbbSqlKey key = (DbbSqlKey)obj;
    	String name = key.getName();
    
    	if(name_.equals(name))
    	    return true;
    
    	return false;
    }

    /**
     * @return the array of join clauses represented by this key
     */
    public String[] getJoinClauses() {
        String rc[] = new String[1];
        StringBuffer j = new StringBuffer();
        for (int i = 0; i < pkColumns_.size(); i++) {
            String andStr = (i == 0) ? "" : " AND ";
            j.append(andStr + getJoinClause(pkColumns_.elementAt(i),
                        fkColumns_.elementAt(i)));
        }
        rc[0] = j.toString();
        return rc;
    }  

    /** 
     * @return a string representation of the object.
     */
    public String toString() {
    	String ret = "Foreign key: " + name_ + "\n";
    	ret =  ret + "PK TABLE:    " + pkTable_ + "\n";
    	ret =  ret + "FK TABLE:    " + fkTable_ + "\n";
    	return ret;
    }
}
