package org.eso.ohs.core.dbb.server;

// $Id: DbbSqlJoinBuilder.java,v 1.4 2006/04/26 09:26:44 tcanavan Exp $

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/** 
 * This class allows to build up joins from the database metadata.
 * It implements the mechanism to navigate through the database table,
 * following the foreign keys, to build up a list of conditions
 * joining two given database tables.
 *
 * @author Carlo Boarotto, ESO 11-Sep-1999
 * 
 **/
public class DbbSqlJoinBuilder {
private static final String rcsid = "$Id: DbbSqlJoinBuilder.java,v 1.4 2006/04/26 09:26:44 tcanavan Exp $";

private static Logger stdlog_ = 
    Logger.getLogger( DbbSqlJoinBuilder.class );

    /**
     * Maximum number of iterations when navigating through the
     * database keys 
     */
    private static final int MAX_ITERATIONS = 100;

    /**
     * Current number of iterations when navigating through the
     * database keys 
     */ 
    private int iteration_ = 0;

    /**
     * Database session, to retrieve the metadata and the foreign
     * keys. 
     */
    private DbbSession session_;

    /**
     * List of databases already visited when retrieving the foreign
     * keys. 
     */
    private Vector<String> dbList_ = new Vector<String>();

    /**
     * Hash table containing the key information retrieved either from
     * the database metadata or from a flat file, containing as keyword
     * the foreign key unique name and as element the corresponding
     * DbbSqlKey object.
     */
    private Hashtable<String, DbbSqlKey> keyHashtable_ = new Hashtable<String, DbbSqlKey>();

    /** Hash table storing the table names and their foreign keys. */
    private Hashtable<DbbSqlTable, Vector<DbbSqlKey>> tableHashtable_ = new Hashtable<DbbSqlTable, Vector<DbbSqlKey>>();

    /** Starting table name. Stored to avoid nasty loopbacks. */
    private DbbSqlTable startTable_ = null;

    /**
     * Current working stack. It stores the keys; when the join has
     * been found, its clone is saved to the result vector fkStackVector_
     * and then all its elements are removed.  
     **/
    private DbbSqlJoinKeyStack<DbbSqlKey>fkStack_ = new DbbSqlJoinKeyStack<DbbSqlKey>();

    /**
     * The key's stacks (may be more than one) are stored in this
     * vector when defining a join.
     **/
    private Vector<DbbSqlJoinKeyStack> fkStackVector_ = new Vector<DbbSqlJoinKeyStack>();


    /** 
     * Boolean flag: if true get the keys from the database metadata,
     * otherwise try to cope with the keys provided by the end
     * application only.
     */
    private boolean getDbKeysFlag_ = false;


    /**
     * Internal class to return DbbSqlJoinBuilder processKey results.
     **/
    class IterateResult {
	public boolean found = false;
	public DbbSqlTable table = null;
    }


    //----------------------------------------
    // Constructor
    //----------------------------------------
	
    /**
     * Build a DbbSqlJoinBuilder setting its internal variables.
     * By default get the key information from the database metadata.
     *
     * @param db_session Database session.
     * @exception SQLException JDBC exception
     */
    public DbbSqlJoinBuilder(DbbSession db_session) throws SQLException {
	session_ = db_session;
	getDbKeysFlag_ = false;
    }


    /**
     * Build a DbbSqlJoinBuilder setting its internal variables.
     *
     * @param db_session	Database session.
     *
     * @param get_db_keys Boolean flag: if false skip the access to the
     *			database metadata to build the internal key
     *			hash tables and use the key information
     *			provided by the end application only.
     * @exception SQLException JDBC exception
     *
     */
    public DbbSqlJoinBuilder(DbbSession db_session,
			     boolean get_db_keys) 
	throws SQLException {
    
	session_ = db_session;
	getDbKeysFlag_ = get_db_keys;
    }


    //----------------------------------------
    // Private Methods
    //----------------------------------------

    /**
     * Retrieve the DbbKeys for all the table from the database and
     * store them into the foreign key hash table.  
     *
     * @param db_name Database name.
     * @exception SQLException JDBC exception
     */
    private void getDbbSqlKeys(String db_name) throws SQLException {

	/* If no database name is provided, take the default database from
	 * the session. 
	 */
	if(db_name != null || db_name.equals(""))
	    db_name = session_.getDefaultDatabase();

	/* Get the database metadata. */
	DatabaseMetaData dbmd = session_.getDatabaseMetadata(db_name);

	long start_time = System.currentTimeMillis();

	ResultSet rs_tables;
	ResultSet rs_keys;

	String[] types = {"TABLE"};

	/* Get all the tables from the database metadata */
	rs_tables = dbmd.getTables( null, null, null, types);

	/* Get the keys defined for each database (db_name) table. */
	while (rs_tables.next()) {

	    String table_cat = rs_tables.getString(1);
	    String table_own = rs_tables.getString(2);
	    String table_name = rs_tables.getString(3);

	    /* Create a table object with the data retrieved from the
	     * database. 
	     */
	    DbbSqlTable table = new DbbSqlTable(table_cat,
						table_own,
						table_name);

	    /* Get only the imported keys */
	    rs_keys = dbmd.getImportedKeys( null, null, table_name);



	    /* Create an entry in the table hash table for the current
	     * table. 
	     */
	    Vector<DbbSqlKey> vec = tableHashtable_.get(table);
	    if (vec == null) {
		vec = new Vector<DbbSqlKey>();
		tableHashtable_.put(table, vec);
	    }

	    /* Process the keys retrieved from the database for the
	     * current table. 
	     */
	    while (rs_keys.next()) {

		/* Generate a key object from the result set row. */
		DbbSqlKey new_key = new DbbSqlKey(rs_keys);

		/* NOTE: The key name should be unique! */
		String key_name = new_key.getName();
	  
		/* Check if the key has been already processed (if already
		 * processed, a key entry should already exist in the key
		 * hash table).
		 */
		if(!keyHashtable_.containsKey(key_name)) {

		    /* Add the key to the key hash table. */
		    keyHashtable_.put(key_name, new_key);

		    //if (stdlog_.isDebugEnabled())
			//stdlog_.debug("~~~ Key <" + key_name + "> added to keyHashtable_");

		    /* Since we got only the imported keys for the current
		     * table, we have to get the "Primary Key Table" from the
		     * result set row.  
		     */
		    DbbSqlTable other_table = new_key.getPkTable();
	    
		    /* Create an entry in the table hash table for the "Primary
		     * Key Table". 
		     */
		    Vector<DbbSqlKey> other_vec = tableHashtable_.get(other_table);
		    if (other_vec == null) {
			other_vec = new Vector<DbbSqlKey>();
			tableHashtable_.put(other_table, other_vec);
		    }
	    
		    /* Add the key to the current table entry in the table
		     * hash table. 
		     */
		    vec.addElement(new_key);
	    
		    //if (stdlog_.isDebugEnabled())
			//stdlog_.debug("~~~ Key <" + key_name + "> added to vec for " + table);

		    /* Add the key to the "Primary Key Table" entry in the
		     * table hash table.  
		     */
		    other_vec.addElement(new_key);
	    
		    //if (stdlog_.isDebugEnabled())
			//stdlog_.debug("~~~ Key <" + key_name + "> added to vec for " + other_table);
		}
	    }
	
	    rs_keys.close();
	}
      
	rs_tables.close();
	session_.releaseCurrentConnection();      

	///// It takes more or less ~2000 ms for opc63 ////
	if (stdlog_.isDebugEnabled()) {

	    
      
	    long end_time = System.currentTimeMillis();
	    long diff_time = end_time - start_time;
	    stdlog_.debug("Time: " + diff_time + " ms\n");
	}
    }


    /**
     * Check if the keys for a table database have been already stored
     * in memory.
     *
     * @param table Databese table object.
     * @exception SQLException JDBC exception
     */
    private void checkDatabaseKeys(DbbSqlTable table) throws SQLException {

	if(table == null)
	    return;

	/* Don't get the keys from the database metadata if getDbKeysFlag_
	 * is set to false.
	 */
	if(!getDbKeysFlag_) return;

	String db_name = table.getDatabase();

	/* Skip if the database has been already visited. */
	if(!dbList_.contains(db_name)) {
	    dbList_.addElement(db_name);
	    getDbbSqlKeys(db_name);
	}
    }


    //----------------------------------------
    // Protected Methods
    //----------------------------------------

    /**
     * Get the foreign keys for a given table.
     *
     * @param table	Table name.
     *
     * @return An array of DbbSqlKey objects.
     */ 
    protected DbbSqlKey[] getTableForeignKeys(DbbSqlTable table) {
      
	Vector vector = tableHashtable_.get(table);
	int len = vector.size();

	/* It is necessary to copy into an array (or at least to clone)
	 * the vector otherwise it takes too much time.
	 */
	DbbSqlKey[] ret_array = new DbbSqlKey[len];

	vector.copyInto(ret_array);

	return ret_array;
    }


    /** 
     * Check if a foreign key is linking two tables aready linked by a
     * different key on the stack. In this case return true and allow to
     * exit potential deadlocks.
     *
     * @param k	Foreign key object.
     *
     * @return Boolean value: true if the link has already been stored,
     *         false otherwise.
     *
     **/
    protected boolean isLinkingTheSameTables(DbbSqlKey k) {

	int len = fkStack_.size();
	DbbSqlKey current_key = null;

	DbbSqlTable from = k.getPkTable();
	DbbSqlTable to = k.getFkTable();

	/* Iterate all over the current stack */
	for(int i=0; i<len; i++) {

	    current_key = (DbbSqlKey)(fkStack_.elementAt(i));
	    DbbSqlTable current_from = current_key.getPkTable();
	    DbbSqlTable current_to = current_key.getFkTable();

	    /* Test the link comparing the tables... */
	    if((from.equals(current_from) && to.equals(current_to)) || 
	       (from.equals(current_to) && to.equals(current_from))) {

	
	
		return true;
	    }
	}

	return false;
    }
 

    /**
     * Check a foreign key. If it's valid to get the join, store the
     * stack (containing it) in the fkStackVector_.
     *
     * @param foreign_key		Foreign key to be processed 
     * @param from_table		Foreign key's starting table
     * @param to_table		Foreign key's ending table
     *
     * @return An IterateResult object with the following fields: found
     *	     (boolean flag to indicate that the target table has been
     *	     found) and table (String giving the name of the new
     *	     table to jump to).
     **/
    protected IterateResult processKey(DbbSqlKey foreign_key,
				       DbbSqlTable from_table,
				       DbbSqlTable to_table) {
    
	IterateResult res = new IterateResult();
	DbbSqlTable new_table = null;

	/* Check if the key is already stored in the current stack.
	 * In this case, return with a result (found = false, table = null)
	 */
	if(fkStack_.contains(foreign_key)) {

	    //if (stdlog_.isDebugEnabled())
		//stdlog_.debug("=== Key <" + foreign_key.getName() + "> already stored, skip.");
	    return res;
	}

	/* Check if the link has been already stored even if under a
	 * different key: it avoids nasty deadloks. 
	 */
	if(isLinkingTheSameTables(foreign_key)) {	   
	    return res;
	}

	/* Get the new table to analyze (excluding the from_table) */
	new_table = foreign_key.getPkTable();

	if(new_table.equals(from_table))
	    new_table = foreign_key.getFkTable();
    
	/* Check if the new table pointed by the key is equal to the very
	 * starting table. In this case, return with a result (found = false,
	 * table = null) to avoid nasty loopbacks.
	 */
	if(new_table.equals(startTable_)) {

	
	    return res;
	}

	/* At this point the key seems to be OK: add to the stack. */
	fkStack_.push(foreign_key);



	/* If the table pointed by the key is equal to the target table,
	 * save the current stack appending a clone to the found
	 * vector. Then reset the it, removing all the elements.
	 */
	if(new_table.equals(to_table)) {

	 

	    fkStackVector_.addElement((DbbSqlJoinKeyStack)fkStack_.clone());

	    /* Remove the current key from the current stack for the next
	     * iteration: the stack containing the final key has already
	     * been saved into fkStackVector_
	     */
	    fkStack_.pop();

	    /* Set the return structure with the proper values */
	    res.found = true;
	    res.table = null;
	} 
	else {

	 
      
	    /* In this brach the current table is not equal to the target
	     * table: set the return structure with the proper values 
	     */
	    res.found = false;
	    res.table = new_table;
	}

	return res;
    }


    /** 
     * Recoursive method to navigate through the database structure
     * stored in memory using the keyHashtable_ (following the foreign
     * keys) and to build the join structure (as vector of stacks,
     * fkStackVector_).
     *
     * @param from_table	Starting table name.
     * @param to_table	Target table name.
     *
     **/
    protected void join(DbbSqlTable from_table, DbbSqlTable to_table) {

	/* Stop the recoursion if there are more than MAX_ITERATIONS
         * iterations. 
	 */
	iteration_++;

	/////////////////// keep or trash ? /////////////////////
	if(iteration_ > MAX_ITERATIONS) {
	    fkStack_.removeAllElements();
	    return;
	}
	/////////////////// keep or trash ? /////////////////////


	IterateResult ir = null;
	boolean discharge_key = false;
	DbbSqlKey foreign_key;

	DbbSqlKey[] keys_array = getTableForeignKeys(from_table);
	int len = keys_array.length;

	for(int i=0; i<len; i++) {
      
	    foreign_key = keys_array[i];
      
	    ir = processKey(foreign_key,
			    from_table,
			    to_table);
      
	    /* If the key doesn't find the requested table but points to a
	     * valid table, jump to a new table and iterate again.
	     */
	    if(!ir.found && ir.table != null) {
	
	
	
		join(ir.table,to_table);
	    }
      
	    /* If the key doesn't point to a valid table, different from
	     * the starting one, and doesn't find the requested table,
	     * then set the boolean flag to remove it from the current
	     * stack.
	     */
	    if(!ir.found && ir.table == null)
		discharge_key = true;
	}
    
	/* Remove the last key on the stack if it has been marked "to be
	 * removed"
	 */
	if(discharge_key && !fkStack_.isEmpty()) {
      
	    DbbSqlKey removed = (fkStack_.pop());
      
	
	}
    }


    /**
     * Get a string containing the sql join statements. The join is
     * built up taking the shortest path between the two tables, going
     * through a third table.
     *
     * @param from_table	First table.
     * @param to_table	Second table.
     *
     * @return String containing the sql join statements.  
     * @exception SQLException JDBC exception
     */
    protected DbbSqlJoinKeyStack getJoinStack(DbbSqlTable from_table, 
					      DbbSqlTable to_table) 
	throws SQLException {

	/* Check if the database keys have been already stored in
	 * memory. 
	 */
	checkDatabaseKeys(from_table);
	checkDatabaseKeys(to_table);

	/* Set the internal variables before starting the recoursion */
	iteration_ = 0;
	fkStack_.removeAllElements();
	fkStackVector_.removeAllElements();
	startTable_ = from_table;

	/* Start the recoursion */
	join(from_table, to_table);

	/* Get the shortest path joining from_table and to_table from the
	 * vector containing the paths involving the cross_table 
	 */
	DbbSqlJoinKeyStack shortest = 
	    DbbSqlJoinKeyStack.getPreferredStack(from_table, to_table, 
						 fkStackVector_);

	return shortest;
    }


    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Add a DbbSqlKey object to the internal data structures in order
     * to evaluate join conditions defined by the end application (and
     * probably overriding the database foreign key definitions).
     * 
     * @param ney_key Newly defined foreign key to be added.
     */
    public void addDbbSqlKey(DbbSqlKey new_key) {

	String key_name = new_key.getName();
    
	/* If not contained, add to the hash table */
	if(!keyHashtable_.containsKey(key_name)) {
      
	    keyHashtable_.put(key_name, new_key);
      
	    //if (stdlog_.isDebugEnabled())
		//stdlog_.debug("~~~ Key <" + key_name + "> added to keyHashtable_");
      
	    /* Add the key to the table hashtable entries, under the two
	     * table keywords specified as pk_table and fk_table within the
	     * key itself.
	     */
	    DbbSqlTable table = new_key.getFkTable();
      
	    Vector<DbbSqlKey> vec = tableHashtable_.get(table);
      
	    if (vec == null) {
		vec = new Vector<DbbSqlKey>();
		tableHashtable_.put(table, vec);
	    }
      
	    DbbSqlTable other_table = new_key.getPkTable();
      
	    Vector<DbbSqlKey> other_vec = tableHashtable_.get(other_table);
      
	    if (other_vec == null) {
		other_vec = new Vector<DbbSqlKey>();
		tableHashtable_.put(other_table, other_vec);
	    }
      
	    vec.addElement(new_key);
      
	    //if (stdlog_.isDebugEnabled())
		//stdlog_.debug("~~~ Key <" + key_name + "> added to vec for " + table);
	    other_vec.addElement(new_key);
      
	    //if (stdlog_.isDebugEnabled())
		//stdlog_.debug("~~~ Key <" + key_name + "> added to vec for " + other_table);
	} else {

	    // Hope that it will substitute the key with the new one
	    keyHashtable_.put(key_name, new_key);
	}
    }


    /** 
     *
     * Build a join giving two tables, the primary and the secondary
     * table.
     * 
     * @param from_table	Primary table.   
     * @param to_table    Secondary table.
     *
     * @return A DbbSqlJoinInfo containing the join information.  
     * @exception SQLException JDBC exception
     */
    public DbbSqlJoinInfo getJoin(DbbSqlTable from_table, DbbSqlTable to_table)
	throws SQLException {

	DbbSqlJoinKeyStack join_path = getJoinStack(from_table, 
						    to_table);
      
	return join_path.getJoinInfo();   
    }


    //----------------------------------------
    //----------------------------------------
    // Debug methods
    //----------------------------------------
    //----------------------------------------

    /** Debug method, used to test the join information */
    public void testJoinInfo() {

	try {
	    DbbSqlTable from = null;
	    DbbSqlTable to = null;
	    DbbSqlTable cross = null;

	    DbbSqlTable t1 = new DbbSqlTable("opc63", "dbo", "obs_programmes");
	    DbbSqlTable t2 = new DbbSqlTable("opc63", "dbo", "eso_users");
	    DbbSqlTable t3 = new DbbSqlTable("opc63", "dbo", "obs_runs");
	    DbbSqlTable t4 = new DbbSqlTable("opc63", "dbo", 
					     "opc_telescopes"); 
	    DbbSqlTable t5 = new DbbSqlTable("opc63", "dbo", 
					     "user_affiliations");
	    DbbSqlTable c1 = new DbbSqlTable("opc63", "dbo", "proposed");
	    DbbSqlTable c2 = new DbbSqlTable("opc63", "dbo", "rated");
      
	    DbbSqlJoinInfo r1, r2, r3;

	    from = t1;
	    to = t5;
	    cross = c1;

	
	    r1 = getJoin(from, to);
	    stdlog_.debug("Result: \n" + r1 + "---\n");

	    from = t1;
	    to = t4;
	    cross = null;


	    r2 = getJoin(from, to);
	    stdlog_.debug("Result: \n" + r2 + "---\n");

	    Vector<DbbSqlJoinInfo> tmp = new Vector<DbbSqlJoinInfo>();
	    tmp.addElement(r1);
	    tmp.addElement(r2);

	    stdlog_.debug("Merging...");
	    r3 = DbbSqlJoinInfo.merge(tmp);
	    stdlog_.debug("Result: \n" + r3 + "---\n");
	}
	catch (Exception e) {
	    stdlog_.warn("",e);
	}
    }


    /**
     * Debug method, used to dump the join result as stack of foreign
     * keys 
     *
     * @param from_table	Starting table name.
     * @param to_table	Target table name.
     * 
     */
    public void dumpJoin(DbbSqlTable from_table, DbbSqlTable to_table) {

	
    
	long start_time = System.currentTimeMillis();

	/* Set the internal variables before starting the recoursion */
	iteration_ = 0;
	fkStack_.removeAllElements();
	fkStackVector_.removeAllElements();
	startTable_ = from_table;

	/* Start the recoursion */
	join(from_table, to_table);

	long end_time = System.currentTimeMillis();
	long diff_time = end_time - start_time;
    

    
	int v_len = fkStackVector_.size();

	for(int i=0; i<v_len; i++) {

	    
	    DbbSqlJoinKeyStack current = 
		(fkStackVector_.elementAt(i));

	    int s_len = current.size();

	    for(int k=0; k<s_len; k++) {
		DbbSqlKey key = current.getKeyAt(k);
		
	    }
	} 
    }


    /**
     *
     * Debug method for regression tests: loop over a list of tables.
     *
     **/
    public void testJoins() {

	String[] tables = {"obs_programmes", 
			   "eso_users",
			   "obs_runs", 
			   "sched_rep",
			   "sched_rep_tmp",
			   "opc_telescopes", 
			   "targets",
			   "figures",
			   "latex_source",
			   "user_affiliations"};

	// Print all the key evaluation steps
	// debug = true;

	DbbSqlTable t1 = null;
	DbbSqlTable t2 = null;
	int len = tables.length;

	try {    
	    for(int i=0; i<len; i++) {
      
		t1 = new DbbSqlTable(tables[i]);
      
		for(int k=i; k<len; k++) {

		    t2 = new DbbSqlTable(tables[k]);
	
		    if(t1.equals(t2))
			continue;

		    dumpJoin(t1, t2);
		}
	    }
	}
	catch (Exception e) {
	    stdlog_.warn("",e);
	}
    }


    /**
     *
     * Debug method for regression tests
     *
     **/
    public static void main(String[] args) {

	//DbbSession test = new DbbSession();

	//DbbSqlJoinBuilder jb = new DbbSqlJoinBuilder(test);
	//jb.debug = false;
	//jb.testJoins();
	//jb.testJoinInfo();
    }
}





