package org.eso.ohs.core.dbb.sql;

import org.apache.log4j.Logger;

/** 
 * This class is a SqlChunkGroup which generates an AND or OR expression
 * based on the value of two SqlChunk members, which represent the min and
 * max values of a cycling range.
 * 
 * It is assumed the values of the chunks may be parsed into floating point
 * numbers.
 *
 * if XminchunkVal < XmaxchunkVal then SQL is:
 *    ( fld <= XmaxchunkVal AND fld >= XminchunkVal)
 * else
 *    ( fld <= XmaxchunkVal OR fld >= XminchunkVal)
 *
 * @author Nick Kornweibel for ESO
 */

public class CyclingMaxMinChunk extends DbbSqlChunkGroup {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( CyclingMaxMinChunk.class );

    public CyclingMaxMinChunk(DbbSqlChunkGroupMember min,
            DbbSqlChunkGroupMember max) {
        addMember(min);
        addMember(max);
    }

    /**
     * See header for details on this expression.
     *
     * @return String containing the sql clause.
     */

    public String getExpressionClause() {
        String rc;
        DbbSqlChunk min = members_.elementAt(0);
        DbbSqlChunk max = members_.elementAt(1);
        double minVal = 0.0;
        double maxVal = 0.0;

        String minStr = null;
        String maxStr = null;

        // try and get a value for minStr
        if (min.isActive()) {
            minStr = min.getValue();
            if (minStr != null && minStr.length() > 0) {
                try {
                    minVal = Double.valueOf(minStr).doubleValue();
                } catch (NumberFormatException e) {
                    minStr = null;
                }
            } else {
                minStr = null;
            }
        }

        // try and get a value for maxStr
        if (max.isActive()) {
            maxStr = max.getValue();
            if (maxStr != null && maxStr.length() > 0) {
                try {
                    maxVal = Double.valueOf(maxStr).doubleValue();
                } catch (NumberFormatException e) {
                    maxStr = null;
                }
            } else {
                maxStr = null;
            }
        }

        String minStrExp = null;
        String maxStrExp = null;

        // create the expressions if we have good values.
        if (minStr != null) {
            DbbSqlOperator op = min.getOperator();
            minStrExp = "("+min.getColumnSpec()
                +" "+op.get()+" "+minStr+")";
        }

        if (maxStr != null) {
            DbbSqlOperator op = max.getOperator();
            maxStrExp = "("+max.getColumnSpec()
                +" "+op.get()+" "+maxStr+")";
        }

        // only build the compound expression if we have two
        // sub expressions - otherwise simply return the
        // sub expression we have, or "" if there is nothing to return.

        if (minStrExp != null && maxStrExp != null) {
            if (minVal <= maxVal) {
                rc = "("+minStrExp+" AND "+maxStrExp+")";
            } else {
                rc = "("+minStrExp+" OR "+maxStrExp+")";
            }
        } else if (minStrExp != null) {
            rc = minStrExp;
        } else if (maxStrExp != null) {
            rc = maxStrExp;
        } else {
            rc = "";
        }
        /*
        stdlog_.debug("CyclingMaxMinChunk: minStr: "+minStr
            +" :minStrExp: "+minStrExp+" :minVal: "+minVal);
        stdlog_.debug("CyclingMaxMinChunk: maxStr: "
            +maxStr+" :maxStrExp: "+maxStrExp+" :maxVal: "+maxVal);
        stdlog_.debug("CyclingMaxMinChunk: rc: "+rc);
        */
        return rc;
    }
}
