package org.eso.ohs.core.dbb.sql;

import java.io.Serializable;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbDataType;
import org.eso.ohs.core.utilities.TextUtils;

/**
 * This class defines a configurable SQL "chunk", that is, an object
 * which can be used to generate a fragment of an SQL SELECT statement.
 * <p>
 * $Id: DbbSqlChunk.java,v 1.10 2006/04/27 13:24:18 tcanavan Exp $
 * @version $Revision: 1.10 $
 * @author A M Chavan, ESO 26-Jul-99
 * @author Carlo Boarotto, ESO 01-Sep-1999
 */
public class DbbSqlChunk implements Serializable {

	private static Logger stdlog_ = Logger.getLogger(DbbSqlChunk.class);

	private boolean active_ = true;

	/** Constant defining the single quote character. */
	public static final char QUOTE = '\'';

	/** Constant defining the double quote character. */
	public static final char DOUBLEQUOTE = '\"';

	/**
	 * SQL data class for this chunk. Used in order to quote properly
	 * string values when building the query and to store the data
	 * type. If the data type class implements the interface
	 * org.eso.ohs.dbb.client.DbbDataType, then it can set the
	 * corresponding column on the view table to the proper width;
	 * otherwise the width of the view table column is set looking at
	 * the longest value to be displayed.
	 */
	protected Class datatype_ = null;

	/** Database primary table. */
	protected DbbSqlTable table_ = null;

	/** Database column name: primary table column. */
	protected String column_ = null;

	/** function to apply to column */
	protected String function_ = null;

	/** Current value: e.g. column value...*/
	protected String value_ = null;

	/**
	 * Label to be used for SELECT statement, like: <br>
	 * <code>SELECT UserID = eso_users.id...</code><br>
	 * where <code>UserID</code> is the label.
	 */
	protected String label_ = null;

	/**
	 * SQL expression to be used when building FROM clauses, like: <br>
	 * <code>substring( surname_dl, 1, 20 )</code>.<br>
	 * By default it is "", meaning that the database column name
	 * should be used.
	 *
	 * @see #column_
	 */
	// TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD
	// TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD
	protected String expression_ = "";

	/**
	 * Table alias to be used when building FROM clauses, like: <br>
	 * <code>FROM obrep..obs_blocks ob</code><br>
	 * where <code>ob</code> is the alias. <p>
	 * Currently unused.
	 */
	// TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD
	// TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD TBD
	protected String alias_ = "";

	/**
	 * Current SQL operator; by default it is the same as
	 * defaultOperator_. This operator is set whenever the user
	 * defines an operator in the DbbWidget.
	 *
	 * @see DbbSqlOperator
	 */
	protected DbbSqlOperator operator_ = null;

	/**
	 * Default SQL operator; by default it is the same as operator_
	 * @see DbbSqlOperator
	 */
	protected DbbSqlOperator defaultOperator_ = null;

	/**
	 * for checkboxes which are used for "is null" queries
	 */
	protected boolean useNull_ = false;

	private boolean isCaseSensitive;

	//--------------------------------------------------
	// Constructors
	//--------------------------------------------------
	
	/**
	 * Create an SQL chunk setting the table, column, column label, data type,
	 * SQL operator and function
	 * <br>
	 * @param table     Database table
	 * @param column    Database column name
	 * @param label     Column label
	 * @param function  function to apply to the column. It uses the pattern @column@ to substitute the column. e.g. sign(@column@)
	 */
	public DbbSqlChunk(
		DbbSqlTable table,
		String column,
		String label,
		String function) {
		this(table, column, label);
		function_ = function;
	}

	/**
	 * Create an SQL chunk setting the table, column, column label, data type,
	 * SQL operator and function
	 * <br>
	 * @param table     Database table
	 * @param column    Database column name
	 * @param label     Column label
	 * @param op        SQL default operator
	 * @param data_type Database column type (see DbbSqlChunk.datatype_)
	 * @param function  function to apply to the column. 
	 * It uses the pattern @column@ to substitute the column. e.g. sign(@column@)
	 */
	public DbbSqlChunk(
		DbbSqlTable table,
		String column,
		String label,
		DbbSqlOperator op,
		Class data_type,
		String function) {
		this(table, column, label, op, data_type);
		function_ = function;
	}

	/**
	 * Create an SQL chunk setting the table, the column, the column
	 * label, data type and the SQL operator.<br> This constructor
	 * should be used mostly for row selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 * @param label     Column label
	 * @param op        SQL default operator
	 * @param data_type Database column type (see DbbSqlChunk.datatype_)
	 */
	public DbbSqlChunk(
		DbbSqlTable table,
		String column,
		String label,
		DbbSqlOperator op,
		Class data_type) {

		/* Initialize internal data structures */
        if (table == null) {
            throw new IllegalArgumentException("table is null");
        }
		table_ = table;
		column_ = column;

		defaultOperator_ = op;
		operator_ = defaultOperator_;
		datatype_ = data_type;
		/*
		        stdlog_.debug("Calling chunk with table: " + table
		            + " column: " + column
		            + " label: " + label
		            + " operator: " + op
		            + " type: " + data_type);
		*/
		if (label == null)
			label_ = column;
		else
			label_ = label;

		if (data_type == null)
			datatype_ = String.class;
	}

	/**
	 * Create an SQL chunk setting the table, the column, data type and
	 * the SQL operator.<br> This constructor should be used mostly for
	 * row selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 * @param op        SQL default operator
	 * @param data_type Database column type (see DbbSqlChunk.datatype_)
	 */
	public DbbSqlChunk(
		DbbSqlTable table,
		String column,
		DbbSqlOperator op,
		Class data_type) {

		this(table, column, null, op, data_type);
		label_ = column;
	}

	/**
	 * Create an SQL chunk setting the table, the column and the
	 * database column type.<br> This constructor should be used mostly
	 * for column selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 * @param data_type Database column type (see DbbSqlChunk.datatype_)
	 */
	public DbbSqlChunk(DbbSqlTable table, String column, Class data_type) {
		this(table, column, null, null, data_type);
	}

	/**
	 * Create an SQL chunk setting the table, the column and the
	 * database column type.<br> This constructor should be used mostly
	 * for column selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 * @param label     Column label
	 * @param data_type Database column type (see DbbSqlChunk.datatype_). The
	 * widget in which the chunk is placed, passes to the sql chunk its value,
	 * the value.toString is then used by the datatype
	 */
	public DbbSqlChunk(
		DbbSqlTable table,
		String column,
		String label,
		Class data_type) {
		this(table, column, label, null, data_type);
	}

	/**
	 * Create an SQL chunk setting the table, the column and the SQL
	 * operator.<br> This constructor should be used mostly for row
	 * selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 * @param op        SQL default operator
	 */
	public DbbSqlChunk(DbbSqlTable table, String column, DbbSqlOperator op) {
		this(table, column, null, op, null);
	}

	/**
	 * Create an SQL chunk setting the table, the column and the SQL
	 * operator.<br> This constructor should be used mostly for row
	 * selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 * @param label     Column label
	 * @param op        SQL default operator
	 *
	 */
	public DbbSqlChunk(
		DbbSqlTable table,
		String column,
		String label,
		DbbSqlOperator op) {
		this(table, column, label, op, null);
	}

	/**
	 * Create an SQL chunk setting the table, the column and the
	 * label.<br> This constructor should be used mostly for column
	 * selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 * @param label     Column label
	 */
	public DbbSqlChunk(DbbSqlTable table, String column, String label) {
		this(table, column, label, null, null);
	}

	/**
	 * Create an SQL chunk setting the table and the column.<br> This
	 * constructor should be used mostly for column selection chunks.
	 *
	 * @param table     Database table
	 * @param column    Database column name
	 */
	public DbbSqlChunk(DbbSqlTable table, String column) {
		this(table, column, null, null, null);
	}

	//--------------------------------------------------
	// Public class methods
	//--------------------------------------------------

	/**
	 * method to set useNull_ flag
	 *
	 * @param b         value to set useNull_ to
	 */
	public void setUseNull(boolean b) {
		useNull_ = b;
	}

	/**
	 * getter for useNull_ flag
	 *
	 * @return useNull_ flag
	 */
	public boolean getUseNull() {
		return useNull_;
	}

	//--------------------------------------------------
	// Protected class methods
	//--------------------------------------------------

	/**
	 * Process the internal value to get the condition value and the
	 * possible operator (if any specified within the text).
	 */
	protected void processText() {

		int index = -1;
		String[] operators = DbbSqlOperator.getOperatorValues();

		String str_orig = value_.trim();
		String str = str_orig.toLowerCase();

		/* Reset the operator before starting */
		operator_ = null;

		for (int i = 0; i < operators.length; i++) {

			String op_str = operators[i];

			index = str.indexOf(op_str);

			/* The operator should be in the first position */
			if (index == 0) {

				/* Avoid: str = "likeness" -> op = like, text = ness. If
				 * "text" operator (i.e. len > 2), it shall be followed by a
				 * blank, otherwise skip it.
				 */
				int op_len = op_str.length();

				// TO BE REVIEWED - Known Bugs
				//
				// is operator doesn't work properly
				// ...ISAAC... becomes ...is 'AAC'...
				//
				if (DbbSqlOperator.isLiteral(op_str)
					&& str.charAt(op_len) != ' ')
					continue;

				try {
					operator_ = new DbbSqlOperator(op_str);

					value_ = str_orig.substring(op_len);

					/* Remove surrounding quotes from the varchar values before
					 * quoteSqlString (if given an operator the text will be
					 * surrouded by single or double quotes)
					 */
					if (datatype_ == String.class) {

						value_ = value_.trim();
						int value_len = value_.length();
						char first = value_.charAt(0);
						char last = value_.charAt(value_len - 1);
						if (first == QUOTE && last == QUOTE)
							value_ = value_.substring(1, value_len - 1);
						if (first == DOUBLEQUOTE && last == DOUBLEQUOTE)
							value_ = value_.substring(1, value_len - 1);
					}
				} catch (DbbInvalidSqlOperatorType e) {
					stdlog_.warn("", e);
				}

				break;
			}
		}

		value_ = value_.trim();

		/* Surround VARCHAR with double quotes */
		value_ = TextUtils.quoteSqlString(value_);

		if (datatype_ == String.class)
			value_ = QUOTE + value_ + QUOTE;
	}

	/**
	 * Convert the displayed format to the database format before
	 * setting the value that has to be passed to the query
	 */
	private void toDatabaseFormat() {

		/**
		stdlog_.info("CHUNK: label: " + getLabel() +
		    " column: " + getColumn() +
		    " class: " + datatype_);
		*/

		if (DbbDataType.class.isAssignableFrom(datatype_)) {

			//stdlog_.info("CONVERTING CHUNK: label: " + getLabel());

			try {
				DbbDataType new_inst = (DbbDataType) datatype_.newInstance();
				Object obj = new_inst.toDatabaseData(value_);
				value_ = obj.toString();
			} catch (Exception e) {

				/* Skip a conversion exception if the string value is
				 * a boolean because, only in this case, the exception
				 * is related to an attempt to convert a boolean value
				 * coming from the column selection chunks.
				 */
				if (!value_.equals("true") && !value_.equals("false")) {
					stdlog_.info(
						"ERR CHUNK: label: "
							+ getLabel()
							+ " column: "
							+ getColumn());
					stdlog_.warn("", e);
				}
			}
		}
	}

	//--------------------------------------------------
	// Public methods
	//--------------------------------------------------

	/** @return table_ attribute. */
	public DbbSqlTable getTable() {
		return table_;
	}

	/** @return column_ attribute. */
	public String getColumn() {
		return column_;
	}

	/** @return datatype_ attribute. */
	public Class<?> getDataType() {
		return datatype_;
	}

	/** @return label_ attribute. */
	public String getLabelWithoutQuotes() {
		return label_;
	}

	/** @return label_ attribute. */
	public String getLabel() {
		String label = QUOTE + label_ + QUOTE;
		return label;
	}

	/** @return function_ attribute. */
	// Added by AMC, 29-Jul-2003
	public String getFunction() {
		return function_;
	}

	/** Set the label_ attribute. */
	public void setLabel(String label) {
		label_ = label;
	}

	/**
	 * @return The column title that should be displayed in the table
	 */
	public String getColumnTitle() {
		if (label_ == null || label_.equals(""))
			return column_;
		return label_;
	}

	/**
	 * @return Full column specification in the form
	 *          <database>.<user>.<table>.<column>
	 */
	public String getColumnSpec() {
		StringBuffer col =
			new StringBuffer().append(table_.get()).append(".").append(column_);
		if (function_ != null) {
			String result = StringUtils.replace(function_,"@column@", col.toString());
			col = new StringBuffer(result);
		}
		return col.toString();
	}

	/** @return defaultOperator_ attribute. */
	public DbbSqlOperator getDefaultOperator() {
		return defaultOperator_;
	}

	/**
	 * @return An operator object equals to the operator_ attribute if
	 *         not null, otherwise to the defaultOperator_ attribute
	 */
	public DbbSqlOperator getOperator() {
		if (operator_ == null) {
			return defaultOperator_;
		} else {
			return operator_;
		}
	}

	/**
	 * Set the and process internally the value_ attribute.<br>
	 * Public in order to allow the aggregating DbbSqlWidget to set
	 * the chunk value
	 *
	 * @param value New value
	 */
	public void setValue(String value) {
		value_ = value;

		/* Don't process the text if the operator is equal to IN ot to NOT IN
		 */
		DbbSqlOperator curr_op = getOperator();

		if (curr_op != null) {
			if (getOperator().equals("in") || getOperator().equals("not in")) {
				return;
			}
		}

		processText();
		toDatabaseFormat();
	}

	/**
	 * Public in order to allow the aggregating DbbSqlWidget to set
	 * the chunk operator
	 *
	 * @param op New operator
	 */
	public void setOperator(DbbSqlOperator op) {
		operator_ = op;
		defaultOperator_ = op;
	}

	/** @return the value_ attribute. */
	public String getValue() {
		return value_;
	}

	/**
	 * Get a select clause from the chunk
	 *
	 * @return String containing the sql clause
	 */
	public String getSelectClause() {
		//    16May02 Add test for "not like"
		if (defaultOperator_ == null) {
			return getColumnSpec();
		}

		DbbSqlOperator op = getOperator();

		String ret = getColumnSpec();

		//Check if the user wanted a not, eg %-doglips%
		//the '-' must be the second character in the string
		if (op.get().equals("like") && value_.indexOf('-') == 2) {
			value_ = "'%" + value_.substring(value_.indexOf('-') + 1);
			ret = ret + " " + "not" + " " + op.get() + " " + value_;
		} else {
			ret = ret + " " + op.get() + " " + value_;
		}

		return ret;
	}

	/**
	 * @return true if this chunk is active (ie will be part of the
	 * resulting SQL statement
	 */

	public boolean isActive() {
		return active_;
	}

	/**
	 * Set this SQL chunk to be active or not. Non active chunks play
	 * no role in the resulting SQL statement and can be ignored.
	 */

	public void setActive(boolean active) {
		active_ = active;
	}

	/** @return A string giving the current instance internal data */
	public String toString() {
		DbbSqlOperator op = getOperator();
		String ret = "Column:   " + getColumnSpec();
		ret = ret + "\nLabel:    " + label_;
		if (op != null) {
			ret = ret + "\nOperator: " + op.get();
		}
		ret = ret + "\nValue:    " + value_ + "\n";
		return ret;
	}

	//----------------------------------------
	//----------------------------------------
	//  Debug & Unit Test
	//----------------------------------------
	//----------------------------------------
	public static void main(String[] args) {

		DbbSqlOperator op1 = null;
		DbbSqlTable tbl1 = null;
		DbbSqlTable tbl2 = null;
		DbbSqlTable tbl3 = null;
		DbbSqlChunk chunk = null;
		String val = null;

		try {
			op1 = new DbbSqlOperator("=");

			tbl1 = new DbbSqlTable("opc63", "dbo", "eso_users");
			tbl2 = new DbbSqlTable("obs_programmes");
			tbl3 = new DbbSqlTable("obrep", null, "obs_runs");

			chunk = new DbbSqlChunk(tbl1, "user_id", op1, Integer.class);

			val = "1685";
			chunk.setValue(val);
			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			chunk = new DbbSqlChunk(tbl2, "title", op1, String.class);

			val = "Likely a 1'.22'' declination";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = "ISAAC";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = "like 'paperino'";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = "pippo";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = "!= \"pluto\"";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			chunk = new DbbSqlChunk(tbl3, "programme_id", op1, Integer.class);

			val = "1685";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = " > 1685";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = " <= 1685";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = "!= 1685";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = "in (1671, 1685, 1686)";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			val = "not in (1721, 1750)";
			chunk.setValue(val);

			/**
			stdlog_.info("Value:   {" + val + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/

			chunk = new DbbSqlChunk(tbl3, "programme_id");

			/**
			stdlog_.info("Value:   {" + chunk.getValue() + "}\n" +
			       "Default:  " +
			       chunk.getDefaultOperator() + "\n" +
			       "Sql:      " + chunk.getSelectClause() + "\n");
			*/
		} catch (Exception e) {
			stdlog_.warn("", e);
		}
	}

	/**
	 * @return true if obj is a sqlChunk has the same label
	 */
	public boolean equals(Object obj) {
		boolean rc = false;
		DbbSqlChunk node = null;
		if (obj != null && obj instanceof DbbSqlChunk) {
			node = (DbbSqlChunk) obj;
			rc = getLabel().equals(node.getLabel());
		}
		return rc;
	}
	
	/**
	 * This method set the field isCaseSensitive to the given value and the field function_ to "lower(@column@)"
	 * If the field function_ has already a value, the method send a warning message and the force function_ to "lower(@column@)"
	 * It should be used only by DbbTextField and DbbTextFieldPattern
	 * @param isCaseSensitive
	 */
	public void setCaseSensitive(boolean isCaseSensitive){
		this.isCaseSensitive = isCaseSensitive;	
		if ((function_ != null) && (function_.length() > 0))
			stdlog_.warn("DbbSqlChunk function " + function_ + " is forced to lower(@column@)");
		if (!isCaseSensitive)
			function_ = "lower(@column@)";		
	}

	public boolean isCaseSensitive() {
		return isCaseSensitive;
	}
}
