package org.eso.ohs.core.dbb.sql;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * This class implements a data structure to store the data
 * retrieved from the database running a query.
 * <p>
 * $Id: DbbSqlQueryResult.java,v 1.4 2006/04/26 09:26:44 tcanavan Exp $
 * @version $Revision: 1.4 $
 * @author Carlo Boarotto, ESO 24-Sep-1999
 */
public class DbbSqlQueryResult implements Serializable {

    public static final String rcsid =
        "$Id: DbbSqlQueryResult.java,v 1.4 2006/04/26 09:26:44 tcanavan Exp $";

    private static Logger stdlog_ = Logger.getLogger(DbbSqlQueryResult.class);

    /** Number of rows. */
    private int rowCount_ = 0;

    /** Number of columns. */
    private int colCount_ = 0;

    /** Column names. */
    private String[] columnNames_;

    /** Column display sizes. */
    // Static because it shouldn't vary when processing different
    // parts of the same result set.
    // private static int[] columnSizes_;
    private int[] columnSizes_;

    /**
     * Column types mapped to Java object type (array of classes
     * corresponding to the columns).
     */
    private Class[] columnTypes_;

    /**
     * Array of strings containing the longest strings defined for each column
     */
    private String[] largeValues;

    /** Retrieved data. */
    protected Vector<Object> dataVector = null;


    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Build an DbbSqlQueryResult data structure.
     */
    public DbbSqlQueryResult() {
        dataVector = new Vector<Object>();
    }


    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /** @return The number of rows. */
    public int getRows() {
        return rowCount_;
    }

    /** @return The number of columns. */
    public int getColumns() {
        return colCount_;
    }

    /**
     * Set the number of columns
     * @param n Number of columns
     */
    public void setColumns(int n) {
        colCount_ = n;
        columnSizes_ = new int[n];
        largeValues = new String[n];

        for (int i = 0; i < n; i++) {
            columnSizes_[i] = 0;
        }
    }

    /**
     * Set the names of the columns
     * @param names Columns' names
     */
    public void setColumnNames(String[] names) {
        columnNames_ = names;
    }

    /**
     * Set the types of the columns
     * @param types Columns' types
     */
    public void setColumnTypes(Class[] types) {
        columnTypes_ = types;
    }

    /**
     * Update the column type with the classes specified in the
     * name_class_list. This method is called from the view to override
     * the default data type (got from the result set metadata) with
     * application specific types defined in the column selection
     * chunks.
     *
     * @param name_class_list List of column name - column class pairs.
     */
    public void updateColumnTypes(Vector<Serializable> name_class_list) {

        if (name_class_list == null)
            return;

        /* Scan the name_class_list vector starting from the last pair of
         * elements.
         */
        try {
            for (int i = name_class_list.size() - 2; i >= 0; i -= 2) {

                Class c = (Class)name_class_list.elementAt(i + 1);
                String name = (String)name_class_list.elementAt(i);

                /* Find the column having the same name and set its type. */
                if (columnTypes_ == null)
                    columnTypes_ = new Class[columnNames_.length];

                for (int k = 0; k < columnNames_.length; k++) {

                    if (name.equals(columnNames_[k])) {
                        columnTypes_[k] = c;
                        break;
                    }
                }
            }
        } catch (Exception e) {
            stdlog_.warn("", e);
            return;
        }

        // for (int j=0; j<columnTypes_.length; j++)
        //     stdlog_.debug("DbbSqlQueryResult:: column:" +
        //         j + " --- " + columnTypes_[j]);
    }

    /** @return The column names. */
    public String[] getColumnNames() {
        return columnNames_;
    }

    /** @return The column names. */
    public int[] getColumnSizes() {
        return columnSizes_;
    }

    /** @return The column classes. */
    public Class[] getColumnTypes() {
        return columnTypes_;
    }

    /** @return Vector of Object arrays containing the data. */
    public Vector<Object> getDataVector() {
        return dataVector;
    }

    /** @return The longest strings contained in each column
     *  @deprecated As largeValues is not set and therefore allways
     *              is null. getLongValues only was used in
     *              org.eso.ohs.dbb.client.DbbTable.addRows()
     *              which just retrieved the null value and did
     *              not use it.
     */
    public String[] getLongValues() {
        return largeValues;
    }

    /**
     * Read a row from a given result set and append it to the
     * internal data structure.
     *
     * @param rs Database result set
     */
    public void addRow(ResultSet rs) throws SQLException {

        Object[] row_array = new Object[colCount_];
        int col_index = 0;
        for (int i = 1; i <= colCount_; i++) {
            Object obj = rs.getObject(i);
            col_index = i - 1;
            row_array[col_index] = obj;

            ////// CB: it doesn't slow down the performances! ///////
            if (obj != null) {
                String current_str = obj.toString().trim();
                int current_len = current_str.length();

                if (current_len > columnSizes_[col_index]) {
                    columnSizes_[col_index] = current_len;
                    //largeValues[col_index] = current_str;
                    /*
                    if (stdlog_.isDebugEnabled()) {
                        stdlog_.debug("%%% Setting Column: <" +
                            col_index + "> Size: <" + current_len + ">");
                    }
                    */
                }
            }
            ////// CB: it doesn't slow down the performances! ///////
        }

        dataVector.addElement(row_array);
        rowCount_++;
    }
}

