package org.eso.ohs.core.docview.datatrans;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import org.apache.log4j.Logger;


/**
 * An abstract adaptor superclass to make a property of a Java object
 * accessible as a String through setValue()/getValue() methods of
 * this class.
 * 
 * <p>
 * Subclasses need only define two methods: toMetaLevelObject()
 * and fromMetaLevelObject() that convert between a String
 * representation of the value and the normal Java representation.
 * Note that Java primitive types must be wrapped in the corresponding
 * meta-level object, eg java.lang.Long for a long primitive value.
 * 
 * <p>
 * This class also includes a factory method to produce adaptors
 * for most of the Java primitive types.
 * 
 * 
 * <p>
 * For example, users will call it's
 * setValue/getValue methods.  Those calls will be translated into
 * calls to setX/getX on the object given to the constructor.
 * Values in the displayed_ array will first be mapped to the
 * corresponding value in displayed_, or vice versa.
 *
 * <p>
 * If the displayed/stored value is not present in the array,
 * the value is passed through unchanged.
 * 
 * <p>
 * $Id: ObjAdaptor.java,v 1.4.4.1 2006/10/09 13:19:58 gdonino Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.core.docview.gui.DocView
 * @see org.eso.ohs.core.docview.datatrans.AdaptorFactory
 * @see Object
 */
public abstract class ObjAdaptor implements ObjectAdaptor {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( ObjAdaptor.class );

    /** Utility constant for use with reflection and getter methods */
    private final static Object[] NO_ARGS = new Object[] {};

    /** getX() method for the property X of the given object. */
    protected Method getter_;

    /** setX() method for the property X of the given object. */
    protected Method setter_;

    /** Object whose property is to be updated. */
    protected Object obj_;

    /** Property of the object that is to be updated. */
    protected String property_;

    /** Is the propery read/write or readonly?  */
    protected boolean readWrite_ = true;

    /** Class representing the type of the property */
    protected Class metaClass_;


    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, eg from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * @param value	value of the property.
     */
    public abstract String fromMetaLevelObject(Object value);


    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, eg to a java.lang.Long
     * representing a long int.  Used with reflection to set
     * the new property value.
     *
     * @param value the new value of the property.
     */
    public abstract Object toMetaLevelObject(String value);


    /**
     * Construct an ObjectAdaptor for the given class.
     * 
     * @param metaClass		type of the object's property
     */
    public ObjAdaptor(Class metaClass) {
	metaClass_ = metaClass;
    }

    /**
     * Construct an ObjectAdaptor for the given class.
     * 
     * @param metaClass		type of the object's property
     * @param obj		Object with the property to be linked
     * @param prop		property to be linked
     */
    public ObjAdaptor(Class metaClass, Object obj, String prop) {
	this(metaClass);

	setObject(obj);
	setProperty(prop);
    }


    /**
     * Internal method to synchronize the state of the ObjectAdaptor
     * when one of the externally modifiable properties of this
     * object changes.
     */
    private void sync() {
	Method[] meths;

	if (obj_ == null || property_ == null) {
	    return;
	}

	meths = obj_.getClass().getMethods();

	findGetMethod(meths);

	if (readWrite_) {
	    findSetMethod(meths);
	}
	else {
	    setter_ = null;
	}
    }

    /**
     * Private utility method to update the getter_ property.
     * 
     * @param meths	array of methods for the Object to be linked
     * 
     * @exception IllegalArgumentException
     * 			thrown if the method is not found, not public, etc
     */
    private void findGetMethod(Method[] meths) {
	String getterName = "get" + property_;

	for (int j=0; j < meths.length; j++) {
		    // We're looking for:
		    //     public void setX(Type newVal);
		    //     public Type getX();

		    // If it's not a public method, skip it.
	    if ( ! Modifier.isPublic(meths[j].getModifiers())) {
		continue;
	    }

	    Class[] thrownExceptions = meths[j].getExceptionTypes();

		    // If it throws a checked exception, skip it.
	    if ( thrownExceptions.length > 0) {
		continue;
	    }

	    if (getterName.equals(meths[j].getName())) {

		// It's a getX method: check params + result

		Class[] parms = meths[j].getParameterTypes();

		if (meths[j].getReturnType().equals(metaClass_)
			&& parms.length == 0) {
		    getter_ = meths[j];
		}
	    }
	}

	if (getter_ == null) {
	    throw new IllegalArgumentException(
		"get method not found: " + property_
	    );
	}
    }

    /**
     * Private utility method to update the setter_ property.
     * 
     * @param meths	array of methods for the Object to be linked
     * 
     * @exception IllegalArgumentException
     * 			thrown if the method is not found, not public, etc
     */
    private void findSetMethod(Method[] meths) {
	String setterName = "set" + property_;

	for (int j=0; j < meths.length; j++) {
		    // We're looking for:
		    //     public void setX(Type newVal);
		    //     public Type getX();

		    // If it's not a public method, skip it.
	    if ( ! Modifier.isPublic(meths[j].getModifiers())) {
		continue;
	    }

	    Class[] thrownExceptions = meths[j].getExceptionTypes();

		    // If it throws a checked exception, skip it.
	    if ( thrownExceptions.length > 0) {
		continue;
	    }

	    if (setterName.equals(meths[j].getName())) {

		// It's a setX method: check params + result

		Class[] parms = meths[j].getParameterTypes();

		if (meths[j].getReturnType().equals(void.class)
			&& parms.length == 1
			&& parms[0].equals(metaClass_)) {
		    setter_ = meths[j];
		}
	    }
	}

	if (setter_ == null) {
	    throw new IllegalArgumentException(
		"set method not found: " + property_
	    );
	}
    }


    /**
     * Updates the nominated property of the Object.
     * 
     * @param newValue	value to be stored in the Object,
     * 			represented as a String
     */
    public void setValue(String newValue) {
        if (setter_ == null) {
            // object is read-only - can not setValue.
            return;
        }
	try {
	    setter_.invoke(obj_, new Object[] { toMetaLevelObject(newValue) });
	}
	catch (InvocationTargetException e) {
	    if (e.getTargetException() instanceof RuntimeException) {
		throw (RuntimeException)e.getTargetException();
	    }
	    else {
		stdlog_.error(e);
	    }
	}
	catch (IllegalAccessException e) {
	    stdlog_.error(e);
	}
    }


    /**
     * Retrieves the nominated property of the Object.
     * 
     * @return		value stored in the Object,
     * 			represented as a Java meta-level object,
     * 			eg Long instead of a long, etc.
     */
    public Object getRawValue() {
	Object result = null;

	try {
	    result = getter_.invoke(obj_, NO_ARGS);
	}
	catch (InvocationTargetException e) {
	    stdlog_.error(e);
	    throw (RuntimeException)e.getTargetException();
	}
	catch (IllegalAccessException e) {
	    stdlog_.error(e);
	}

	return result;
    }


    /**
     * Retrieves the nominated property of the Object.
     * 
     * @return	value stored in the Object, represented as a String
     */
    public String getValue() {
	return fromMetaLevelObject(getRawValue());
    }


  

    /**
     * Returns the current Object.
     * 
     * @return			current Object
     */
    public Object getObject() {
	return obj_;
    }

    /**
     * Sets the current Object.
     * 
     * @param obj	new Object
     */
    public void setObject(Object obj) {
	Object old = obj_;

	obj_ = obj;

		// Shortcut most common case.
		// Normally we're just changing the object, not the property
		// or class of object -- just the instance of it.
		// In this case, we do not need to redo the reflection, etc.
	if (old != null && obj != null && old.getClass() == obj.getClass()) {
	    return;
	}

	sync();
    }


    /**
     * Returns the current property name.
     * 
     * @return			property name
     */
    public String getProperty() {
	return property_;
    }

    /**
     * Sets the current property name.
     * 
     * @param name	property name
     */
    public void setProperty(String name) {
	if ( ! name.equals(property_)) {
	    property_ = name;
	    sync();
	}
    }

    /**
     * Returns true if the current property is modifiable
     * 
     * @return			property name
     */
    public boolean getReadWrite() {
	return readWrite_;
    }

    /**
     * Enables or disables modification of the value of the object's property.
     * 
     * @param readWrite		true if it can be updated, false otherwise
     */
    public void setReadWrite(boolean readWrite) {
	readWrite_ = readWrite;
	sync();
    }

    /**
     * Factory method to create an ObjectAdaptor for properties of
     * (most) Java primitive type, eg int, long, String.
     * 
     * @param cls	Class object for the given Object
     * @param property	Property of the given Object
     * 
     * @exception	NoSuchMethodException
     * 			thrown if there is no get method for this property,
     * 			ie no such property.
     */
    public static ObjectAdaptor getAdaptor(Class cls, String property)
					    throws NoSuchMethodException {
	Method getter = cls.getMethod("get"+property, new Class[] { });
	Class returnType = getter.getReturnType();
        ObjectAdaptor objAdap = null;

	if (returnType == String.class) {
	    objAdap = new StringAdaptor();
	}
	else if (returnType == int.class) {
	    objAdap = new IntAdaptor();
	}
	else if (returnType == long.class) {
	    objAdap = new LongAdaptor();
	}
	else if (returnType == float.class) {
	    objAdap = new FloatAdaptor();
	}
	else if (returnType == boolean.class) {
	    objAdap = new BooleanAdaptor();
	}
	else if (returnType == double.class) {
	    objAdap = new DoubleAdaptor();
	}

	return objAdap;
    }

}

