package org.eso.ohs.core.gui.widgets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;

import javax.swing.ButtonGroup;
import javax.swing.DefaultButtonModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

/*
 * Created by amchavan on Jul 14, 2005
 */

/**
 * A page for an {@link OHSWizard}: it's a subclass of JPanel
 * and as such it can be populated with any JComponent.
 * 
 * Some components classes, like JTextField, can also hook up with
 * the wizard by calling the appropriate form of the
 * <code>listenTo()</code> method (hooked up components are
 * "listened to"); for instance:
 * <pre>
   JComboBox instType = new JComboBox();
   instType.setName( "instType" );
   instType.addItem( "Basic" );
   instType.addItem( "Custom" );
   instType.addItem( "Full" );
   listenTo( instType );
 * </pre>
 * It is important that hooked up components have a non-null, unique name.
 * Widget names should be unique within the widget, not just the page.
 * 
 * Hooked up components partecipate in defining whether a wizard page 
 * is <em>ready</em> (that is, its <em>Next</em> button can be
 * enabled): see {@link #isReady()}. Also, such components contribute
 * to a wizard's key/value set (see {@link OHSWizard#get(String)}).
 * 
 * @see #isReady()
 * @version $Revision: 1.8 $
 * @author amchavan, Jul 14, 2005
 */

// $Id: OHSWizardPage.java,v 1.8 2006/04/26 09:26:44 tcanavan Exp $

public abstract class OHSWizardPage
    extends JPanel
    implements ActionListener, DocumentListener,ChangeListener ,ListSelectionListener {

     
    /** The wizard we belong to */
    protected OHSWizard wizard = null;
    
    /**
     * @param wizard The wizard we belong to.
     */
    public OHSWizardPage( OHSWizard wizard ) {
        super();
        this.wizard = wizard;
    }

    /**
     * Implements the ActionListener interface, for JRadioButtons,
     * JComboBox, etc. Stores the new value of the widget
     * in our internal key/value registry.
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed( ActionEvent e ) {
        checkVerified();
        
        String value = null;
        String key = null;
        Object source = e.getSource();
       
        if( source instanceof JComboBox ) {
            JComboBox combo = (JComboBox) source;
            key = getWidgetNames().get( source );
            if( key == null )
                throw new NullPointerException( "Internal error" );
            value = (String) combo.getSelectedItem();
          
        } 
        else if( source instanceof JRadioButton ) {
            JRadioButton rbutton = (JRadioButton) source;
            ButtonGroup group =
                ((DefaultButtonModel) rbutton.getModel()).getGroup();
            key = getWidgetNames().get( group );
            if( key == null )
                throw new NullPointerException( "Internal error" );
            value = e.getActionCommand();
        }
        else {
            throw new IllegalArgumentException( source.getClass().getName() +
                                                " is not supported." );
        }
        if (value != null) {
        wizard.put( key, value );
        } else {
            wizard.remove(key);
        }
    }

    private Hashtable<Object, String> getWidgetNames() {
        return wizard.getWidgetNames();
    }

    /**
     * Implements the DocumentListener interface, for JTextFields,
     * etc. Stores the new value of the widget
     * in our internal key/value registry.
     * 
     * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
     */
    public void changedUpdate( DocumentEvent e ) {
        processDocumentEvent( e );
    }
    
    /** 
     * Check if this page is verified; that is, if the
     * wizard can move on to the next page (or finish).
     */
    private void checkVerified() {
       if (wizard.getCurrentPage()!= null) {
        wizard.setVerified( this, this.isReady() );
       }
    }
    
    /**
     * Implements the DocumentListener interface, for JTextFields,
     * etc. Stores the new value of the widget
     * in our internal key/value registry.
     * 
     * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
     */
    public void insertUpdate( DocumentEvent e ) {
        processDocumentEvent( e );
    }

    /**
	 * OHSWizardPage is an abstract class. Concrete subclasses must implement
	 * this method, which enables the <em>Next</em> wizard button. <br>
	 * For instance, let's assume that the wizard page has a JTextField
	 * component (instDir) that should be non-empty, and a JComboBox component
	 * (instType) that should contain a valid (i.e. non-empty) selection. The
	 * <code>isReady()</code> method could be coded as:
	 * 
	 * <pre>
	 * protected boolean isReady() {
	 * 	return ((instDir.getText().length() != 0) &amp;&amp; (((String) instType
	 * 			.getSelectedItem())).length() != 0);
	 * }
	 * </pre>
	 * 
	 * @return <code>true</code> if the user has completed interacting with
	 *         the page, and the <em>Next</em> button can be enabled;
	 *         <code>false</code> otherwise. <br/> In general, a wizard page
	 *         is ready when all its widgets have valid content/selection.
	 */
    protected abstract boolean isReady();

    /**
	 * @return <code>true</code> if the input ButtonGroup has a valid
	 *         selection; <code>false</code> otherwise.
	 */
    protected boolean isReady( ButtonGroup buttongroup ) {
        return buttongroup.getSelection() != null;
    }
    
    /**
	 * @return <code>true</code> if the input Jlist has a valid
	 *         selection; <code>false</code> otherwise.
	 */
    protected boolean isReady( JList jlist ) {
        return (jlist.getSelectedIndices().length > 0);
    }
    

    /**
     * @return <code>true</code> if the current selection of
     *         the input JComboBox is not empty; 
     *         <code>false</code> otherwise.
     */
    protected boolean isReady( JComboBox combo ) {
        String selection = (String) combo.getSelectedItem();
        return selection.length() != 0;
    }

    /**
     * @return <code>true</code> if the input JTextField has some content; 
     *         <code>false</code> otherwise.
     */
    protected boolean isReady( JTextField textfield ) {
        return textfield.getText().length() != 0;
    }

    /**
     * Hook up the input JComboBox to the wizard; that is, 
     * listen (react) to changes in the current selection.
     * 
     * @param combo The combobox we listen to. Its <em>name</em>
     *              attribute should not be null and should be
     *              unique within the wizard. The current selection
     *              can be retrieved from the wizard by giving
     *              the combobox' <em>name</em> attribute as input
     *              to the wizard's {@link OHSWizard#get(String)}
     *              method.
     * 
     * @throws NullPointerException If the <em>name</em> attribute
     *                              of the input JComboBox is null.
     */
    protected void listenTo( JComboBox combo ) {
    	combo.addActionListener( this );
    	String name = combo.getName();
    	if( name == null )
    		throw new NullPointerException( "No name given to combobox." );
    	
    	// TODO Check that the name is unique within the wizard
    	getWidgetNames().put( combo, name );
    	
    	String value = (String) combo.getSelectedItem();
    	if (value != null) {
    		wizard.put( name, value );
    	}
    }
    
    
    /**
     * Hook up the input Jlist to the wizard; that is, 
     * listen (react) to changes in the current selection.
     * 
     * @param jList The jlist we listen to. Its <em>name</em>
     *              attribute should not be null and should be
     *              unique within the wizard. The current selection
     *              can be retrieved from the wizard by giving
     *              the jlist' <em>name</em> attribute as input
     *              to the wizard's {@link OHSWizard#get(String)}
     *              method.
     * 
     * @throws NullPointerException If the <em>name</em> attribute
     *                              of the input JComboBox is null.
     */
    protected void listenTo( JList jlist ) {
    	jlist.addListSelectionListener(this); //   addActionListener( this );
    	String name = jlist.getName();
    	if( name == null )
    		throw new NullPointerException( "No name given to JList." );
    	
    	// TODO Check that the name is unique within the wizard
    	getWidgetNames().put( jlist, name );
    	
    	
    	int numItems = jlist.getSelectedValues().length;
    	String [] values = new String[numItems];
    	int [] indices = jlist.getSelectedIndices();
    	for (int i = 0; i < numItems; i++) {
			values[i] = (String)jlist.getModel().getElementAt(indices[i]);
		}
    	
    	if (numItems > 0) {
    		wizard.put( name, values );
    	}
    }
    
    
    /**
     * Hook up the input JRadioButton to the wizard; that is, 
     * listen (react) to changes in its state. The radio button
     * should belong to an OHSButtonGroup instance.
     * 
     * @param combo The radio button we listen to. Its 
     *              <em>actionCommand</em>
     *              attribute should not be null and should be
     *              unique within the button group the radio button
     *              belongs to. <br/>
     *              The <em>actionCommand</em> of the currently 
     *              selected button in the button group 
     *              can be retrieved from the wizard by giving
     *              the group's <em>name</em> as input
     *              to the wizard's {@link OHSWizard#get(String)}
     *              method.
     *              
     * @throws RuntimeException If the input JRadioButton does not
     *                          belong to an OHSButtonGroup instance.
     * @throws NullPointerException If the <em>actionCommand</em> attribute
     *                              of the input JRadioButton is null.
     */
    protected void listenTo( JRadioButton button ) {
    	button.addActionListener( this );
    	
    	// Make sure we keep track of this radio button's group
    	//-----------------------------------------------------
    	ButtonGroup group = 
    		((DefaultButtonModel) button.getModel()).getGroup();
    	if( ! (group instanceof OHSButtonGroup))
    		throw new RuntimeException( "Button does not belong to an OHSButtonGroup" );
    	OHSButtonGroup ohsgroup = (OHSButtonGroup) group;
    	
    	if( getWidgetNames().contains( group )) 
    		return;
    	
    	String cmd = button.getActionCommand();
    	if( cmd == null )
    		throw new NullPointerException( "No actionCommand given to combobox." );
    	// TODO Check that the actionCommand is unique within the button group
    	
    	String key = ohsgroup.getName();
    	getWidgetNames().put( group, key );
    }
    
    public void listenTo(JSpinner spinner) { 
    	spinner.addChangeListener( this );
    	String name = spinner.getName();
    	if( name == null )
    		throw new NullPointerException( "No name given to combobox." );
    	
    	// TODO Check that the name is unique within the wizard
    	getWidgetNames().put( spinner, name );
    	
    	
    	String valueString = null;
    	valueString = getStringValue(spinner);
    	wizard.put( name, valueString ); 
    }
    
    /**
     * @param spinner
     * @return String
     */
    private String getStringValue(JSpinner spinner) {
    	String valueString;
    	Object value = spinner.getModel().getValue();
    	if (value  instanceof Number) {
    		valueString = ((Number)value).toString();
    	} else if (value  instanceof String) {
    		valueString = (String) value;
    	} else {
    		valueString = value.toString();
    	}
    	return valueString;
    }
    
    
    /**
     * Hook up the input JTextField to the wizard; that is, 
     * listen (react) to changes in its text 
     * (<code>Document</code>). Note that we
     * register ourselves as DocumentListener of the text field,
     * so that we can react to every keystroke.
     * 
     * @param combo The text field we listen to. Its <em>name</em>
     *              attribute should not be null and should be
     *              unique within the wizard. The current text
     *              can be retrieved from the wizard by giving
     *              the text field's <em>name</em> attribute as input
     *              to the wizard's {@link OHSWizard#get(String)}
     *              method.
     * 
     * @throws NullPointerException If the <em>name</em> attribute
     *                              of the input JTextField is null.
     */
    protected void listenTo( JTextField textfield ) {
        
        Document doc = textfield.getDocument();
        doc.addDocumentListener( this );
        String name = textfield.getName();
        if( name == null )
            throw new NullPointerException( "No name given to text field." );

        // TODO Check that the name is unique within the wizard
        getWidgetNames().put( doc, name );
    }

    /** Take care of changes in one of our text fields */
    private void processDocumentEvent( DocumentEvent e ) {
        checkVerified();
        Document doc = e.getDocument();
        String widget = getWidgetNames().get( doc );
        try {
            wizard.put(widget, doc.getText( 0, doc.getLength() ));
        }
        catch( BadLocationException e1 ) {
            // should never happen
            e1.printStackTrace();
        }
    }

    /**
     * Implements the DocumentListener interface, for JTextFields,
     * etc. Stores the new value of the widget
     * in our internal key/value registry.

     * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
     */
    public void removeUpdate( DocumentEvent e  ) {
        processDocumentEvent( e );
    }

    public void stateChanged(ChangeEvent e) {
       checkVerified();
        
        String value = null;
        String key = null;
        Object source = e.getSource();
       
        if( source instanceof JSpinner ) {
            JSpinner spinner = (JSpinner) source;
            key = getWidgetNames().get( source );
            if( key == null )
                throw new NullPointerException( "Internal error" );
            value =  getStringValue(spinner);     
        }  else
            throw new IllegalArgumentException( source.getClass().getName() +
                                                " is not supported." );
        wizard.put( key, value );
        
    }
    
    
	public void valueChanged(ListSelectionEvent e) {
	       checkVerified();
		String [] values = null;
        String key = null;
        Object source = e.getSource();
       
        if( source instanceof JList ) {
        	JList jlist = (JList) source;
            key = getWidgetNames().get( source );
            
            if( key == null )
                throw new NullPointerException( "Internal error" );
            
            Object [] selectedVals = jlist.getSelectedValues();
            values = new String[selectedVals.length];
            for (int i = 0; i < selectedVals.length; i++) {
            	values[i] = (String) selectedVals[i];
			}
        }  else
            throw new IllegalArgumentException( source.getClass().getName() +
                                                " is not supported." );
        wizard.put( key, values );
        
		
	}
    
      /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.getClass().getName();
    }

}
