package org.eso.ohs.core.utilities;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

/**
 * Implements a bidirectional key/value map; that is, a map where all
 * keys and -- optionally -- all values are constrained to be unique. 
 * 
 * @author Bob Kemp
 * @author amchavan, 12-Jan-2006: rewritten as a subclass of LinkedHashMap
 * 
 * @version $Revision: 1.7 $
 */

// $Id: AssocList.java,v 1.7 2006/04/27 10:37:42 tcanavan Exp $ 

public class AssocList<K,V> extends LinkedHashMap<K,V> {

    /** Flag: this association list enforces uniqueness of values */
    private boolean unique;
    
    /** Create a bidirectional association list */
    public AssocList() {
        this( true );
    }
    
    /**
     * Create an association list.
     * 
     * @param unique
     *            If <code>true</code>, the association list enforces
     *            uniqueness of key and values; if <code>false</code> only
     *            keys are forced to be unique.
     */
    public AssocList( boolean unique ) {
        super();
        this.unique = unique;
    }

    /**
     * Behaves exactly as as put() but returns this AssocList. (Left for
     * backwards compatibility.)
     * 
     * @see #put(Object, Object)
     * @throws IllegalArgumentException
     *             if either parameter is null or if <code>value</code> is not
     *             unique.
     */
    public AssocList<K, V> add(K newKey, V newValue)
        throws IllegalArgumentException {
        this.put( newKey, newValue );
        return this;
    }

    /**
     * Lookup a value by its numerical index; indices are defined by the
     * insertion order.
     * 
     * @param index The value associated to the specified index, or null if the
     *              map contains no mapping for this key.
     * 
     * @return Value corresponding to that index, if any, converted to a String;
     *         <code>null</code> otherwise
     *         
     * @see java.util.Map#get(java.lang.Object)
     */
    public String get( int index ) {
        Set entrySet = entrySet();
        int i = 0;
        for( Iterator iter = entrySet.iterator(); iter.hasNext(); ) {
            Map.Entry element = (Map.Entry) iter.next();
            if( i++ != index )
                continue;
            return element.getValue().toString();
        }
        return null;
    }

    /**
     * Reverse mapping: value -> key.
     * 
     * @param value Value whose key we want to find
     * 
     * @return Key of the input value converted to a String, if found;
     *         <code>null</code> otherwise
     * 
     * @throws RuntimeException If this association list does not enforce
     *                          unique values, and therefore a reverse
     *                          mapping is not possible.
     */
    public String getKey( Object value ) throws RuntimeException {
        if( ! unique )
            throw new RuntimeException(
                    "This association list does not enforce unique values." );
        
        Set entrySet = entrySet();
        for( Iterator iter = entrySet.iterator(); iter.hasNext(); ) {
            Map.Entry element = (Map.Entry) iter.next();
            if( value.equals( element.getValue() ))
                return element.getKey().toString();
        }
        return null;
    }

    /**
     * @return <code>true</code> if this association list enforces uniqueness
     *         of key and values; <code>false</code> if only keys are forced
     *         to be unique.
     */
    public boolean isUnique() {
        return unique;
    }

    /**
     * @return  The set of keys of this key/value mapping.
     */
    public String[] keys() {
        String[] keys = new String[ size() ];
        Set keySet = keySet();
        int i = 0;
        for( Iterator iter = keySet.iterator(); iter.hasNext(); ) {
            String key = (String) iter.next();
            keys[i++] = key;
        }
        return keys;
    }

    /**
     * @throws IllegalArgumentException
     *             If either parameter is null or if <code>value</code> is not
     *             unique.
     * 
     * @see java.util.Map#put(java.lang.Object, java.lang.Object)
     */
    public V put(K key,V value)
        throws IllegalArgumentException {
        
        if( key == null || value == null ) 
            throw new IllegalArgumentException( 
                    "Args key and value cannot be null" );
                
        if( unique && containsValue( value )) {
            throw new IllegalArgumentException( 
                    "Duplicated value \"" + value + "\"" );
        }
        
        return super.put( key, value );
    }

    /**
     * @return  The set of values of this key/value mapping.
     */
    public Object[] valuesAsObjects() {
        Collection values = values();
        Object[] ret = new Object[ size() ];
        int i = 0;
        for( Iterator iter = values.iterator(); iter.hasNext(); ) {
            Object value = iter.next();
            ret[i++] = value;
        }
        return ret;
    }

    /**
     * @return The set of values of this key/value mapping, each converted to a
     *         String (with toString()).
     */
    public String[] valuesAsStrings() {
        Collection values = values();
        String[] ret = new String[ size() ];
        int i = 0;
        for( Iterator iter = values.iterator(); iter.hasNext(); ) {
            Object value = iter.next();
            ret[i++] = value.toString();
        }
        return ret;
    }
}
