package org.eso.ohs.core.utilities;

import java.util.ArrayList;

/**
 * This class fires events to the various classes that 
 * have registed for these events.
 * This is the main class that will get called.
 * The concept here is that a gui class needs to be informed about events 
 * in the non-gui parts of the application. without coupling the two together.
 * SO 
 * 
 * GUI class registers as listener to the class that contains  a 
 * JobEventGeneratorList
 * 
 * The gui adds itself as a listener to that class. 
 * If an event happens that the gui is interested in the class that 
 * includes the JobEventGeneratorList fires the event to all the 
 * listeners. The Listeners can then displat the approriare error 
 * message. This leaves the GUI doing what it should do displaying error 
 * messages. 
 *   
 * @author tcanavan
 *
 */

public class JobEventGenerator implements JobEventGeneratorInterface {
    
    
    protected static String NEW_LINE = System.getProperty("line.separator");
    
    
    /**
     * A list of listeners
     */

    protected ArrayList<JobListener> mJobListenerList = null;
    
    /**
     * Add a JobListener.
     * 
     * @see org.eso.ohs.core.utilities.JobEventGeneratorInterface#addJobListener(JobListener)
     */

    public void addJobListener(JobListener jobListener) {
        getJobListeners().add(jobListener);
    }
    
    /**
     * Remove a Job Listener
     * @see org.eso.ohs.core.utilities.JobEventGeneratorInterface#removeJobListener(JobListener)
     */

    public void removeJobListener(JobListener jobListener) {
        if (jobListener != null) {
            getJobListeners().remove(jobListener);
        }
    }
    
    
    /**
     * Get the job listener
     * @return ArrayList
     */

    public ArrayList<JobListener> getJobListeners() {
        if (mJobListenerList == null) {
            mJobListenerList = new ArrayList<JobListener>();
        }
        return mJobListenerList;
    }
    
    /**
     * Tell the listeners the execution of the action there listening for 
     * has started
     */

    public void fireOnStarted() {
        for (int i = getJobListeners().size() - 1; i >= 0; i -= 1) {
            getJobListeners().get(i).onStarted();
        }
    }
    
    /**
     * Tell the listeners the execution of the action there listening for 
     * has finished
     */


    public void fireOnFinished() {
        for (int i = getJobListeners().size() - 1; i >= 0; i -= 1) {
            getJobListeners().get(i).onFinished();
        }
    }

    /**
     * Tell the listeners the execution of the action there listening for 
     * has cancelled
     */


    public void fireOnCanceled() {
        for (int i = getJobListeners().size() - 1; i >= 0; i -= 1) {
            getJobListeners().get(i).onCanceled();
        }
    }
    
    /**
     * Tell the listeners the execution of the action there listening for 
     * has generated a message
     * @param message
     */


    public void fireOnMessage(String message) {
        for (int i = getJobListeners().size() - 1; i >= 0; i -= 1) {
            getJobListeners().get(i).onMessage(message);
        }
    }
    
    /**
     * Tell the listeners  the execution of the action there listening for has
     * generated an exception
     * @param Exception 
     */


    public void fireOnException(Exception exception) {
        for (int i = getJobListeners().size() - 1; i >= 0; i -= 1) {
            getJobListeners().get(i).onException(exception);
        }
    }
    
    /**
     * Tell the listeners the execution of the action there listening for 
     * is in progress.
     * @param toProcess
     * @param processed
     */


    public void fireOnProgress(int toProcess, int processed) {
        for (int i = getJobListeners().size() - 1; i >= 0; i -= 1) {
            getJobListeners().get(i).onProgress(
                toProcess,
                processed);
        }
    }
    
    /**
     * Tell the listeners that the action has called an 
     * OutOfMemoryError
     * @param error
     */
    

    public void fireOnOutOfMemory(OutOfMemoryError error) {
        for (int i = getJobListeners().size() - 1; i >= 0; i -= 1) {
            getJobListeners().get(i).onOutOfMemory(error);
        }
    }

}
