package org.eso.ohs.core.utilities;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * A simple logger class.
 *
 * @author amchavan, 11-Oct-2001: extracted from LoggingStorageManager.java
 */
public class Logger {

    /**
     * Output stream to which all operations are logged.
     */
    private Writer writer_ = null;

    /**
     * Id of user for which we log operations.
     */
    private int userId_ = -1;

    /** Format for encoding/decoding text for an ISO format timestamp */
    private SimpleDateFormat isoTimeFormat_;

	

    /** @param wtr A buffered file writer. Logging output will be
     *             generated on this writer as well as on stdout. */
    public Logger( BufferedWriter wtr ) {

    	writer_ = wtr;
    
    	// XXX Beware of changing this!  Read the comment carefully first.
    	// DANGER!  First set timezone to ECT (European Central Timezone?)
    	//          Then create the isoTimeFormat_ object.
    	//          For some inexplicable reason, the order is important.
    	//		We need to set the default timezone to get the correct
    	//		time when printing new Date() in ISO format.
    	//		Without this, the time is about 2.5hrs fast.
    	TimeZone.setDefault(TimeZone.getTimeZone("ECT"));
    	isoTimeFormat_ = new SimpleDateFormat( Convert.ISO_DATE_FORMAT );
    }

    /**
     * @param logfile Pathname of a writeable file. Logging output
     *                will be appended to this file as well as 
     *                printed on stdout.
     */
    public Logger( String logfile ) throws IOException {
	   this( new FileWriter( logfile, true ));
    }


    /** @param w An unbuffered file writer. Logging output will be
     *             generated on this writer as well as on stdout. */
    public Logger( Writer w ) throws IOException {
	   this( new BufferedWriter( w ));
    }
    

    /** A logger created with this constructor will log to stdout only */
    public Logger() {
	   this( (BufferedWriter) null );
    }


    /**
     * Sets the id of the current user.
     * 
     * @param userId	the id of the end-user
     */
    public void setUserId(int userId) {
	   userId_ = userId;		    // Set user id for logging methods
    }

    /**
     * Close the writer associated with this logger, then dispose of
     * it.<BR>
     * After closing it, a Logger cannot be used anymore until a new
     * Writer is provided.
     *
     * @see #setWriter
     */
    public void close() {
    	if( writer_ != null ) {
    	    try                    { writer_.close(); }
    	    catch( IOException e ) { /* do nothing */ }
    	}
    	writer_ = null;
    }

    /** 
     * Set the Writer object for this server.
     * Any previous instance gets closed.
     * @param w The new Writer object.
     * @see #close
     */
    public void setWriter( Writer w ) {
    	close();
    	writer_ = w;
    }


    public void logStart(String methodName) {
	   logStart(methodName, "", "");
    }

    public void logStart(String methodName, Object arg1) {
	   logStart(methodName, arg1, "");
    }

    public void logStart(String methodName, Object arg1, Object arg2) {
	   log(methodName + " started " + arg1 + " " + arg2);
    }

    public void logComplete(String methodName) {
	   log(methodName + " completed");
    }

    public void logSuccess(String methodName) {
	   log(methodName + " succeeded");
    }

    public void logFailure(String methodName) {
	   log(methodName + " failed");
    }



    /**
     * Log an exception's name and message.<BR>
     * The stack trace gets printed on the standard output.
     */
    public void logException(String methodName, Exception e) {
	   log(methodName + " Exception: " + e.getClass().getName()
					+ ": " + e.getMessage());
	   e.printStackTrace();
    }


    public void log( String msg ) {
	   log( msg, true );
    }

    public void log(String msg, boolean withUserId ) {
    	String timestamp = isoTimeFormat_.format(new Date());
    	StringBuffer sb = new StringBuffer();
    
    	sb.append( '[' );
    	sb.append( timestamp );
    	sb.append( ']' );
    	sb.append( ' ' );
    	//  This is kludgy.
    	//   We should know better when to log the user ID
    	//   and when not.
    	if( withUserId && userId_ >= 0 ) {
    	    sb.append( userId_ );
    	    sb.append( ' ' );
    	}
    	sb.append( msg );
    
    	try {
    	    System.out.println( sb.toString() );
    	    sb.append( '\n' );
    	    // Note: the java.io.Writer class is thread-safe!
    	    //       BUT we must write log file entry in a single call.
    	    if( writer_ != null ) {
        		writer_.write( sb.toString() );
        		writer_.flush();
    	    }
    	}
    	catch (IOException e) {
    	    System.out.println( "Writing to log file failed" );
    	}
    }
}


