package org.eso.ohs.core.utilities;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StreamTokenizer;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * Parses "Paf" file using Java's built in StreamTokenizer. The constructor for
 * this class recieves a paramater which is a filename. A StreamTokenizer class
 * is then created. The comment character ia a # character. The Stream Tokens
 * are divided into pairs of strings and added to a list called the identifier
 * list. This list represents a 2 dimensional array of IDENTIFIERS and data.
 *
 * Finally data associated with the PAF file is initialized e.g. PAF.NAME
 *
 * $Id: PafFile.java,v 1.9.4.2 2007/07/20 11:57:39 pnunes Exp $
 */

public class PafFile {

	private final String[] mandatoryPafKeywords_ = { "PAF.HDR.START",
			"PAF.TYPE", "PAF.ID", "PAF.NAME", "PAF.DESC", "PAF.CRTE.NAME",
			"PAF.CRTE.DAYTIM", "PAF.LCHG.NAME", "PAF.LCHG.DAYTIM",
			"PAF.CHCK.NAME", "PAF.CHCK.DAYTIM", "PAF.CHCK.CHECKSUM",
			"PAF.HDR.END" };

	/** Standard log channel */
	private static Logger stdlog_ = Logger.getLogger(PafFile.class);

	/**
	 * Reading a stream
	 */

	private StreamTokenizer pafToken_;

	/**
	 * Name of the PAF file stored in PAF.NAME
	 */
	private String pafName_;

	/**
	 * Description of the PAF file stored in PAF.DESC
	 */
	private String pafDescription_;

	/**
	 * PAF Identifier stored in PAF.IDENTIFIER
	 */

	private String pafIdentifier_;

	/**
	 * PAF Type stored in PAF.TYPE
	 */
	private String pafType_;

	/**
	 * PAF Veersion stored in VERSION
	 */

	private String version_;

	private boolean importFile = false;

	/**
	 * This is a list of identifier and strings which are obtained from the PAF
	 * file
	 */
	private PafData[] identifierList_ = null;

	/**
	 * This reads a text file but keeps track of the line numbers
	 */

	private LineNumberReader fReader_;

	/**
	 * This is used to write out a line in a PafFile
	 */

	private PrintWriter out_;

	/**
	 * The following madatory PAF.lines are missing.
	 */

	private Vector<String> pafLinesMissing_ = new Vector<String>();

	/**
	 * This is an idList of name value pairs stored in a Vector It cannot be a
	 * hashtable as duplicate keywords can exist within a PAF file. For example
	 * TPL.PARAM in templateSignature
	 */

	private Vector<PafData> idList_ = new Vector<PafData>();

	/**
	 * Compile error msg string which contains data id it is impossible to parse
	 * the PAF file correctly
	 */

	private String compileErrorMsg_ = null;

	/**
	 * The short filename that the PAF file.
	 */

	private String shortFilename_;

	private String valueSeparator = " " ;

	private Hashtable<String, String> uniqueKeywords_ = new Hashtable<String, String>();

	/**
	 * Accept import boolean as in the import situation one does not want to
	 * trim blank lines out
	 *
	 * @param filename
	 *            PAF filename to be parsed e.g. *.isf *.tsf
	 *
	 */

	public PafFile(String filename, boolean wrtie, boolean importFile)
			throws IOException, FileNotFoundException {
		this.importFile = importFile;
		if (filename != null)
			initialise(filename);
	}

	/**
	 * Creates the StreamTokenixer object and initializes the identifierList
	 * variable which this class and all its sub classes will use to initialize
	 * the appropiate data sources.
	 *
	 * @param filename
	 *            PAF filename to be parsed e.g. *.isf *.tsf
	 *
	 */

	public PafFile(String filename) throws IOException, FileNotFoundException {
		if (filename != null)
			initialise(filename);
	}

	/**
	 * Creates the StreamTokenixer object and initializes the identifierList var
	 * iable which this class and all its sub classes will use to initialize the
	 * appro piate data sources.
	 *
	 * @param filename
	 *            PAF filename to be parsed e.g. *.isf *.tsf
	 *
	 */

	public PafFile(PafFile f) {
		identifierList_ = f.getIdentifierList();
		uniqueKeywords_ = f.getUniqueKeywordList();
		initPafHeaderValues();
		initPafLinesMissing();
	}

	public String toFileString() {
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < identifierList_.length; i++) {
			if (identifierList_[i].hasValue()) {
				buf.append(identifierList_[i].getIdentifier() + " " + "\""
						+ identifierList_[i].getValue() + "\"" + "\n");
			} else
				buf.append(identifierList_[i].getIdentifier() + " \"\"\n");
		}
		return buf.toString();
	}

	public void setKeywordValue(String id, String value) {
		for (int i = 0; i < identifierList_.length; i++) {
			if (identifierList_[i].getIdentifier().equals(id)) {
				identifierList_[i].setValue(value);
				return;
			}
		}

	}

	public Hashtable<String, String> getUniqueKeywordList() {
		return uniqueKeywords_;
	}

	public String getValueForKeyword(String id) {
		return uniqueKeywords_.get(id);
	}

	public boolean keyExists(String key) {
		if (uniqueKeywords_.containsKey(key))
			return true;
		return false;
	}

	/**
	 * Create a Paffile class which can be writtten to or read from
	 *
	 * @param filename
	 *            Name of PAF file to be parsed
	 * @param write
	 *            Whether Paf file can e written to or not
	 */

	public PafFile(String filename, boolean write) throws IOException {
		if (write) {
			File outFile = new File(filename);
			File dir = new File(outFile.getParent());
			if (!dir.canWrite()) {
				throw new IOException(
						"Error no rights to write this file in directory "
								+ dir.getName());
			}
			out_ = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
		} else
			initialise(filename);

	}

	public PafFile(Reader rdr) throws IOException {
		fReader_ = new LineNumberReader(rdr);
		init();
	}

	/**
	 * Initialis the memeber variables for the class
	 *
	 * @param filename
	 *            Name of PAF file to be parsed
	 */

	public void initialise(String filename) throws IOException,
			FileNotFoundException {
		File f = new File(filename);
		shortFilename_ = f.getName();
		stdlog_.debug("Init: " +filename);
		if (!f.exists())
			throw new FileNotFoundException("File does not exist: " + filename);
		if (!f.canRead()) {
			throw new IOException(
					"Permission denied - can't open file for read: " + filename);
		}
		fReader_ = new LineNumberReader(new FileReader(filename));
		init();
	}

	private void init() throws IOException {
		if(importFile) {
			valueSeparator = "\n";
		} else {
			valueSeparator = " ";
		}


		initIdentifierList();
		if (!compileError() && identifierList_ != null) {
			initPafHeaderValues();
			initPafLinesMissing();
		}
		fReader_.close();
	}

	/**
	 * Compiler error msg if there is a mismatch of quotes in the file
	 *
	 * @return compile error msg
	 */

	public String getCompileErrorMsg() {
		return compileErrorMsg_;
	}

	/**
	 * Paf file compiled correctly or not.
	 *
	 * @return a compile error has occured or not
	 */

	public boolean compileError() {
		if (compileErrorMsg_ != null && compileErrorMsg_.length() > 0)
			return true;
		return false;
	}

	/**
	 * All paf file mandatory keywords exist
	 *
	 * @return mandatory keywords exist.
	 */

	public boolean isPafFile() {
		if (identifierList_ == null)
			return false;
		for (int i = 0; i < mandatoryPafKeywords_.length; i++) {
			boolean found = false;
			for (int j = 0; j < identifierList_.length; j++) {
				if (identifierList_[j].getIdentifier().equals(
						mandatoryPafKeywords_[i])) {
					found = true;
					if (identifierList_[j].getIdentifier().equals("PAF.NAME")) {
						if (!identifierList_[j].hasValue()) {
							found = false;
							break;
						}
					}
					break;
				}
			}
			if (!found) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Paf Files short filename not including filenames parent.
	 *
	 * @return filename excluding patth
	 */

	public String getShortFilename() {
		return shortFilename_;
	}

	/**
	 * Write a line of text to the output stream
	 *
	 * @param s
	 *            line of text
	 */
	public void writeLine(String s) throws NullPointerException {
		out_.println(s);
	}

	/**
	 * Close the output stream
	 */

	public void closeWriter() throws NullPointerException {
		out_.close();
	}

	/**
	 * Sets the private members of the PAF class which are generic to all PAF
	 * file. PAF.NAME,PAF.DESC,PAF.ID
	 */

	private void initPafHeaderValues() {
		// if(identifierList_ == null)
		int size = identifierList_.length;
		for (int i = 0; i < size; i++) {
			PafData paf = identifierList_[i];
			String id = paf.getIdentifier();
			if (id.equals("PAF.NAME"))
				pafName_ = paf.getValue();
			if (id.equals("PAF.DESC"))
				pafDescription_ = paf.getValue();
			if (id.equals("PAF.ID"))
				pafIdentifier_ = paf.getValue();
			if (id.equals("PAF.TYPE"))
				pafType_ = paf.getValue();
			if (id.equals("PAF.VERSION"))
				version_ = paf.getValue();
		}
	}

	/**
	 * Build a list of missing paf file keywords that should be in the file
	 */
	private void initPafLinesMissing() {
		boolean found = false;
		for (int i = 0; i < mandatoryPafKeywords_.length; i++) {
			found = false;
			for (int j = 0; j < identifierList_.length; j++) {
				if (identifierList_[j].getIdentifier().equals(
						mandatoryPafKeywords_[i])) {
					found = true;
				}
			}
			if (!found)
				pafLinesMissing_.addElement(mandatoryPafKeywords_[i]);
		}
	}

	public Vector<PafData> getIdVector() {
		return idList_;
	}

	public String[] getPafLinesMissing() {
		String[] pafLinesMissing = new String[pafLinesMissing_.size()];
		pafLinesMissing_.copyInto(pafLinesMissing);
		return pafLinesMissing;
	}

	/**
	 * Read the value associated with a particular ID
	 *
	 * @param s
	 *            unique identifier.
	 */

	public String readLine(String s) {
		for (int i = 0; i < identifierList_.length; i++) {
			if (identifierList_[i].getIdentifier().equals(s))
				return identifierList_[i].getValue();
		}
		return null;
	}

	/**
	 * Return the value(s) associated with a given keyword
	 *
	 * @param s
	 *            unique identifier.
	 * @return String array of keyword values
	 */

	public String[] readKeywordValues(String s) {
		Vector<String> valueVec = new Vector<String>();
		for (int i = 0; i < identifierList_.length; i++) {
			if (identifierList_[i].getIdentifier().equals(s))
				valueVec.addElement(identifierList_[i].getValue());
		}
		String[] values = new String[valueVec.size()];
		valueVec.copyInto(values);
		return values;
	}

	/**
	 * Read the value associated with a particular ID
	 *
	 * @param s
	 *            unique identifier.
	 */

	public int getLineNumber(String s) {
		for (int i = 0; i < identifierList_.length; i++) {
			if (identifierList_[i].getIdentifier().equals(s))
				return identifierList_[i].getLineNumber();
		}
		return -1;
	}

	/**
	 * Returs the value for PAF.NAME in the PAF file.
	 *
	 * @return PAF.NAME keyword value
	 */

	public String getPafName() {
		return pafName_;
	}

	/**
	 * Returs the value for VERSION in the PAF file.
	 *
	 * @return VERSION keyword value
	 */

	public String getVersion() {
		return version_;
	}

	/**
	 * Returns the value for PAF.DESC in the paf file
	 *
	 * @return PAF.DESC keyword value
	 */

	public String getPafDescription() {
		return pafDescription_;
	}

	/**
	 * Retuern the value for PAF.ID in the PAF file
	 *
	 * @return PAF.ID keyword value
	 */

	public String getPafIdentifier() {
		return pafIdentifier_;
	}

	/**
	 * Returns the value for PAF.TYPE in the PAF file
	 *
	 * @return PAF.TYPE keyword value
	 */

	public String getPafType() {
		return pafType_;
	}

	/**
	 * Return a list of IDENTIFIER + DATA pairs
	 *
	 * @return IDENTIFIER + DATA pairs
	 */

	public PafData[] getIdentifierList() {
		return identifierList_;
	}

	/**
	 * Set pafName_ from PAF.NAME in PAF file.
	 *
	 * @param pafName
	 *            value from PAF.NAME
	 */

	public void setPafName(String pafName) {
		pafName_ = pafName;
	}

	/**
	 * Set VERSION in the PAF file.
	 *
	 * @param version
	 *            VERSION keyword value
	 */

	public void setVersion(String version) {
		version_ = version;
	}

	/**
	 * Set pafDescription from PAF.DESC.
	 *
	 * @param pafDescription
	 *            value from PAF.DESC
	 */

	public void setPafDescription(String pafDescription) {
		pafDescription_ = pafDescription;
	}

	/**
	 * Sets pafId_ from PAF.ID in .tsf file
	 *
	 * @param padIdentifier_
	 *            Value for PAF.ID
	 */

	public void setPafIdentifier(String pafIdentifier) {
		pafIdentifier_ = pafIdentifier;
	}

	/**
	 * Sets pafType_ from PAF.TYPE in the PAF file
	 *
	 * @param pafType_
	 *            Value from PAF.TYPE
	 */

	public void setPafType(String pafType) {
		pafType_ = pafType;
	}

	private boolean isComment(String line) {
		if (line.startsWith("#") || line.startsWith(";"))
			return true;
		return false;
	}

	private boolean processSingleLine(StringBuffer buf, String line,
			int beginQuote, int endQuote) {
		boolean found = false;
		if (endQuote > beginQuote) {
			String temp = line.substring(beginQuote, endQuote).trim();
			buf.append(TextUtils.revertEscapedString(temp));
			found = true;
		} else if (beginQuote == endQuote) {
			found = true;
		} else if (line.length() - 1 >= beginQuote) {
			buf.append(TextUtils.revertEscapedString(line.substring(beginQuote,
					line.length())));
		}
		return found;
	}


	/**
	 * An endtry has been discovered where a paf entry spans
	 * multiple lines.
	 * @param nextLine
	 * @param buf
	 * @param endQuote
	 * @return
	 */

	private boolean processMultiLine(String nextLine, StringBuffer buf,int endQuote) {
		boolean found = false;
		int colonTerminator = nextLine.indexOf(';');

		if (endQuote != -1 || colonTerminator != -1) {
			nextLine = removeComment(endQuote, nextLine, colonTerminator);
			if (endQuote == nextLine.length() - 1
					|| (endQuote == nextLine.length() - 2 && colonTerminator == nextLine
							.length() - 1)) {
				found = true;
				createLine(buf, endQuote, nextLine);
			} else if (endQuote == -1
					&& colonTerminator == nextLine.length() - 1) {
				found = true;
				createLine(buf, colonTerminator, nextLine);
			}
		} else {
			buf.append(valueSeparator);
			buf.append(TextUtils.revertEscapedString(nextLine));
		}
		return found;
	}

	/**
	 * Main parsing routine for the paffile.
	 * @throws IOException
	 */

	private void initIdentifierList() throws IOException {
		String id;
		String value;
		idList_ = new Vector<PafData>();
		String line;
		while ((line = fReader_.readLine()) != null) {
			line = line.trim();
			int lineNumber = fReader_.getLineNumber();
			StringTokenizer token = new StringTokenizer(line);
			if (!isComment(line) && token.countTokens() > 0) {
				id = token.nextToken();
				int beginQuote = line.indexOf('\"');
				value = "";
				if (beginQuote != -1) {
					beginQuote++;
					int endQuote = getEndQuote(line, beginQuote);
					StringBuffer buf = new StringBuffer();
					boolean singleProcess = processSingleLine(buf, line, beginQuote,
							endQuote);
					value = buf.toString();


					// no end of line terminator found so the data spans
					// multiple
					// lines
					if(!singleProcess) {
						boolean found = false;
						StringBuffer buf1 = new StringBuffer();
						while (!found && fReader_.ready()) {
							String nextLine = fReader_.readLine().trim();
							endQuote = getEndQuote(nextLine,0);
							//
							found = processMultiLine(nextLine,buf1,endQuote);
  					        if (!found && endQuote != -1
				                && endQuote != nextLine.length() - 1) {
							 	break;
							}
						}
						value = value + buf1.toString();
						if (!found) {
							setCompilerErrorMsg(id, lineNumber);
							return;
						}
					}
				} else if (id.indexOf(';') != -1) {
					id = id.substring(0, id.indexOf(';'));
				}
				setPafData(id, value, lineNumber);
			}
		}
		identifierList_ = new PafData[idList_.size()];
		idList_.copyInto(identifierList_);
	}

	private void setPafData(String id, String value, int lineNumber) {
		PafData pafData = new PafData();
		pafData.setIdentifier(id);
		pafData.setValue(value);
		pafData.setLineNumber(lineNumber);
		uniqueKeywords_.put(id, value);
		idList_.addElement(pafData);
	}

	private void setCompilerErrorMsg(String id, int lineNumber) {
		compileErrorMsg_ = "Id " + id
				+ "  has  no terminating quote at line number" + "("
				+ lineNumber + ")";
	}

	private void createLine(StringBuffer buf, int endQuote, String nextLine) {
		buf.append(valueSeparator);
		buf.append(TextUtils.revertEscapedString(nextLine.substring(0, endQuote)).trim());
	}

	private String removeComment(int endQuote, String nextLine,
			int colonTerminator) {
		int commentIndex = nextLine.lastIndexOf('#');
		if (commentIndex != -1 && commentIndex > endQuote
				&& commentIndex > colonTerminator) {
			nextLine = nextLine.substring(0, commentIndex);

		}
		return nextLine.trim();
	}

	/**
	 * @param line
	 * @param endQuote
	 * @param escapeNext
	 * @param j
	 * @return The index position in the line of the last quote.
	 */
	private int getEndQuote(String line, int beginQuote) {
		int j = beginQuote;
		boolean found = false;
		boolean escapeNext = false;
		while (j < line.length()) {
			if (escapeNext) {
				escapeNext = false;
				j++;
				continue;
			}
			char ch = line.charAt(j);
			if (ch == '\\') {
				escapeNext = true;
			} else if (ch == '\"') {
				found = true;
				break;
			}
			j++;
		}
		if (!found)
			return -1;
		return j;
	}

	/**
	 * Prints out a string representation of the class
	 *
	 * @return String representation of public data members.
	 */
	public String toString() {

		StringBuffer sb = new StringBuffer();
		sb.append("Paf Name=");
		sb.append(pafName_);
		sb.append("\n");
		sb.append("Paf Description=");
		sb.append(pafDescription_);
		sb.append("\n");
		sb.append("Paf Identifier=");
		sb.append(pafIdentifier_);
		sb.append("\n");
		sb.append("Paf Type=");
		sb.append(pafType_);
		sb.append("\n");
		sb.append("Paf Version=");
		sb.append(version_);
		sb.append("\n");
		return sb.toString();
	}

	public static void main(String args[]) {
		if (args.length < 1) {
			System.out.println("Missing paf file to test!");
			System.out.println("command <path to PAF file>");
			System.exit(1);
		}
		report(args[0]);

	}

	public static void report(String filePath) {
		PafFile f = null;
		try {
			f = new PafFile(filePath);
		} catch (Exception e) {
			System.out.println("Error reading paf file: " + e.toString());
		}
		System.out.println(f.reportToString());
	}

	public void report() {
		stdlog_.debug(reportToString());
	}

	public String reportToString() {
		StringBuffer rc = new StringBuffer();
		rc.append("Compile Error: " + compileError() + "\n");
		rc.append("Compile Msg  : " + getCompileErrorMsg() + "\n");
		rc.append("Is PAF File  : " + isPafFile() + "\n");
		rc.append("=================toString()=================" + "\n");
		rc.append(toString() + "\n");
		rc.append("=================Identifier List============" + "\n");
		PafData data[] = getIdentifierList();
		for (int i = 0; i < data.length; i++) {
			PafData d = data[i];
			rc.append("" + i + "\t<" + d.getIdentifier() + ">\t\t<"
					+ d.getValue() + ">" + "\n");
		}
		rc.append("============================================" + "\n");
		return rc.toString();
	}
}
