/*
 * Created by amchavan on Sep 22, 2004
 */

package org.eso.ohs.core.utilities.cvsproxy;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import org.apache.log4j.Logger;
import org.netbeans.lib.cvsclient.Client;
import org.netbeans.lib.cvsclient.admin.StandardAdminHandler;
import org.netbeans.lib.cvsclient.command.Builder;
import org.netbeans.lib.cvsclient.command.CommandAbortedException;
import org.netbeans.lib.cvsclient.command.CommandException;
import org.netbeans.lib.cvsclient.command.GlobalOptions;
import org.netbeans.lib.cvsclient.command.checkout.CheckoutCommand;
import org.netbeans.lib.cvsclient.command.commit.CommitCommand;
import org.netbeans.lib.cvsclient.command.diff.DiffCommand;
import org.netbeans.lib.cvsclient.command.diff.DiffInformation;
import org.netbeans.lib.cvsclient.command.export.ExportCommand;
import org.netbeans.lib.cvsclient.command.log.LogBuilder;
import org.netbeans.lib.cvsclient.command.log.LogCommand;
import org.netbeans.lib.cvsclient.command.log.LogInformation;
import org.netbeans.lib.cvsclient.command.tag.RtagCommand;
import org.netbeans.lib.cvsclient.command.tag.TagCommand;
import org.netbeans.lib.cvsclient.command.update.UpdateCommand;
import org.netbeans.lib.cvsclient.connection.AuthenticationException;
import org.netbeans.lib.cvsclient.connection.PServerConnection;
import org.netbeans.lib.cvsclient.event.CVSAdapter;
import org.netbeans.lib.cvsclient.event.FileInfoEvent;
//import org.netbeans.lib.cvsclient.command.add.AddCommand;
/**
 * A utility class wrapping many features of the CVS interface in a easy to use
 * module. The CVS interface itself is provided by org.netbeans.lib.cvsclient
 * (see http://javacvs.netbeans.org).
 * <P>
 * This class is meant to provide a central point of contact for all CVS-related
 * operations. One must first create an instance, then invoke CVS commands
 * from there. For instance:
<pre>
        String cvsroot = ":pserver:amchavan@cvssrv.hq.eso.org:/project1/CVS";
        String passwd = "Ay%hcy<y=";
        String module = "uss-unit-test";
        String dir = "/tmp";
        StringCvsConsole console = new StringCvsConsole();
        
        CvsProxy cvs = new CvsProxy( cvsroot, passwd, dir, console );
        boolean ret = cvs.checkout( module );
        if( ret == false )
            System.out.println( console.get() );   // some error!
</pre>
 * 
 * @see #CvsProxy(String, String, String, CvsConsole)
 * 
 * @version $Revision: 1.12 $
 * @author amchavan, 07-Jul-2004
 */

// $Id: CvsProxy.java,v 1.12 2006/08/10 14:35:08 gdonino Exp $
public class CvsProxy {

    /** The connection to the CVS server. */
    private PServerConnection connection;

    /**
     * Type of CVS connection; for instance <code>pserver</code>.<BR>
     * Note: only <code>pserver</code> is currently supported.
     */
    private String connectionType;

    /** Holds a copy of the CVSROOT string */
    private GlobalOptions globalOptions;

    /**
     * Host running the CVS server; for instance <code>cvssrv.hq.eso.org</code>
     */
    private String host;

    /**
     * Password of the user specified in the CVSROOT string, <strong>encrypted
     * </strong>.
     */
    private String password;

    /**
     * Pathname of the CVS repository; for instance <code>/project1/CVS</code>
     */
    private String repository;

    /** CVS user ID; for instance <code>amchavan</code> */
    private String user;

    /**
     * Pathname of a directory where a local working copy of the CVS-controlled
     * project will be stored.
     */
    private String workdir;

    /**
     * The console to which CVS can output its messages.
     */
    private CvsConsole console;
    
    private static Logger stdlog = Logger.getLogger(CvsProxy.class);
    
    /**
     * Constructor of the connection object.
     * 
     * @param cvsroot
     *            A string like
     *            <code>:pserver:amchavan@cvssrv.hq.eso.org:/project2/CVS</code>
     *            <BR>
     *            Note that only the <code>pserver</code> protocol is
     *            currently supported.
     * 
     * @param password
     *            Password of the user specified in the CVSROOT string,
     *            <strong>encrypted </strong>. To encrypt a password, use the
     *            <code>cvs login</code> command, then look into the
     *            <code>$HOME/.cvspass</code> file.
     * 
     * @param workdir
     *            Pathname of a directory where a local working copy of the
     *            CVS-controlled project will be stored; the directory will be
     *            created, if needed. <br>
     *            Pathname must be an absolute (canonical) pathname; for
     *            instance <code>D:\\tmp</code> (Windows) or <code>/tmp</code>
     *            (Unix).
     * 
     * @param console
     *            Where CVS messages will be reported. Different console types
     *            can be provided to this constructor: see class CvsConsole
     *            and its subclasses.
     * 
     * @see CvsConsole
     * @see StringCvsConsole
     * @see TerminalCvsConsole
     * @see JTextAreaCvsConsole
     * 
     * @throws CommandAbortedException
     * @throws AuthenticationException
     */
    public CvsProxy( String cvsroot, 
                     String password, 
                     String workdir, 
                     CvsConsole console )
            throws CommandAbortedException, AuthenticationException {

        parseCVSRoot( cvsroot );
        this.password = password;
        this.workdir  = workdir;
        this.console  = console;       
        
        globalOptions = new GlobalOptions();
        globalOptions.setCVSRoot( cvsroot );

        getClient( password, workdir );
    }
    
    /**
     * Constructor of the connection object.
     * 
     * @param cvsroot
     *            A string like
     *            <code>:pserver:amchavan@cvssrv.hq.eso.org:/project2/CVS</code>
     *            <BR>
     *            Note that only the <code>pserver</code> protocol is
     *            currently supported.
     * 
     * @param password
     *            Password of the user specified in the CVSROOT string,
     *            <strong>encrypted </strong>. To encrypt a password, use the
     *            <code>cvs login</code> command, then look into the
     *            <code>$HOME/.cvspass</code> file.
     * 
     * @param console
     *            Where CVS messages will be reported. Different console types
     *            can be provided to this constructor: see class CvsConsole
     *            and its subclasses.
     * 
     * @see CvsConsole
     * @see StringCvsConsole
     * @see TerminalCvsConsole
     * @see JTextAreaCvsConsole
     * 
     * @throws CommandAbortedException
     * @throws AuthenticationException
     */
   /* public CvsProxy( String cvsroot, 
            String password, 
            CvsConsole console )
    throws CommandAbortedException, AuthenticationException {

    	parseCVSRoot( cvsroot );
    	this.password = password;
    	this.workdir  = workdir;
    	this.console  = console;       

    	globalOptions = new GlobalOptions();
    	globalOptions.setCVSRoot( cvsroot );

    	getClient(password);
    }*/

    /**
     * Check out a project from the CVS server to the local directory.
     * 
     * @param project
     *            Name of the project to check out; for instance
     *            <code>ESOFORM</code>.
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandAbortedException
     * @throws CommandException
     * @throws AuthenticationException
     * @throws IOException
     */
    public boolean checkout( String project ) throws CommandAbortedException,
            CommandException, AuthenticationException, IOException {

        Client client = getClient( password, workdir );
        CheckoutCommand command = new CheckoutCommand( true, project );
        command.setBuilder( null );
        command.setPruneDirectories( true );
       // command.set
        

        boolean ret = client.executeCommand( command, globalOptions );

        closeConnection();
        return ret;
    }
    
    
    /**
     * Check out a project from the CVS server to the local directory.
     * 
     * @param project
     *            Name of the project to check out; for instance
     *            <code>ESOFORM</code>.
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandAbortedException
     * @throws CommandException
     * @throws AuthenticationException
     * @throws IOException
     */
    public boolean checkout( String project ,String revision) throws CommandAbortedException,
            CommandException, AuthenticationException, IOException {

        Client client = getClient( password, workdir );
        CheckoutCommand command = new CheckoutCommand( true, project );
        command.setBuilder( null );
        command.setCheckoutByRevision(revision);
        command.setPruneDirectories( true );
        
        boolean ret = client.executeCommand( command, globalOptions );

        closeConnection();
        return ret;
    }
    
    /**
     * Create a branch named branchTag at the level of the version tagged as whereTag
     * 
     * @param branchTag
     *            name of the branch we want create.
     *            
     * @param whereTag
     *            tag of the version we want to branch.
     *            
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandAbortedException
     * @throws CommandException
     * @throws AuthenticationException
     * @throws IOException
     */
    public boolean branch(String module, String branchTag) throws CommandAbortedException,
            CommandException, AuthenticationException, IOException {

        Client client = getClient( password );
        TagCommand command = new TagCommand();
        command.setBuilder( null );        
       // command.setOverrideExistingTag( true );
        command.setMakeBranchTag(true);
        command.setTag( branchTag );
      
        File files[] = new File[1];
        files[0] = new File( workdir + File.separator + module );
        command.setFiles( files );
       // stdlog.debug("Set command: " + result + " line: " + command.getCVSCommand());
        boolean ret = client.executeCommand( command, globalOptions );     

        closeConnection();
        return ret;
    }
    
    public boolean branch(String module, String branchTag, String whereTag ) throws CommandAbortedException,
    CommandException, AuthenticationException, IOException {

    	Client client = getClient( password );
    	RtagCommand command = new RtagCommand();
		       
		// command.setOverrideExistingTag( true );
		command.setMakeBranchTag(true);
		command.setTag( branchTag );
		String[] mod = {module};
		command.setModules(mod);
		
		boolean result = command.setCVSCommand('r', whereTag);
		/*File files[] = new File[1];
		files[0] = new File( workdir + File.separator + module );
		command.setFiles( files );*/
		stdlog.debug("Set command: " + result + " line: " + command.getCVSCommand());
		boolean ret = client.executeCommand( command, globalOptions );     
		
		closeConnection();
		return ret;
    }
    
    
    /**
     * Check out a project from the CVS server to the local directory without
     * including the CVS directory
     * 
     * @param project
     *            Name of the project to check out; for instance
     *            <code>ESOFORM</code>.
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandAbortedException
     * @throws CommandException
     * @throws AuthenticationException
     * @throws IOException
     */
    public boolean export( String project ,String revision, String directory) throws CommandAbortedException,
            CommandException, AuthenticationException, IOException {

    	
        Client client = getClient( password, workdir );
        ExportCommand command = new ExportCommand();
        command.setBuilder( null );
        command.addModule(project);
        command.setExportByRevision(revision);
        command.setPruneDirectories( true );
        command.setExportDirectory(directory);

       stdlog.debug("Export module " + project + " to the directory: " + directory);
        boolean ret = client.executeCommand( command, globalOptions );

        closeConnection();
        return ret;
    }
    
    
    
    /**
     * Commit any changes in the input module 
     * to the CVS repository.
     * 
     * @param module  Module to commit; must have been previously checked
     *                out of the repository with <code>checkout()</code>
     * 
     * @param message A log message; can be <code>null</code>.
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandException
     * @throws AuthenticationException
     * @throws CommandAbortedException
     * @throws IOException
     * 
     * @see #checkout(String)
     */
    public boolean commit( String module, String message ) 
        throws CommandAbortedException, 
               CommandException, 
               AuthenticationException, 
               IOException {

        File messageFile = null;
        
        Client client = getClient( password, workdir );
        CommitCommand command = new CommitCommand();
        command.setBuilder( null );
        
        // Write the log message to a file
        if( message != null ) {
            messageFile = File.createTempFile( "cvs-message", "txt" );
            FileWriter w = new FileWriter( messageFile );
            w.write( message );
            w.close();
            command.setLogMessageFromFile( messageFile.getAbsolutePath() );
        }
        
        
        File files[] = new File[1];
        files[0] = new File( workdir + File.separator + module );
        command.setFiles( files );
        
        boolean ret = client.executeCommand( command, globalOptions );

        closeConnection();
        if( messageFile != null )
            messageFile.delete();
        
        return ret;
    }
    
    /**
     * Commit any changes in the input module 
     * to the CVS repository.
     * 
     * @param module  Module to commit; must have been previously checked
     *                out of the repository with <code>checkout()</code>
     * 
     * @param message A log message; can be <code>null</code>.
     * 
     * @param revision commit on the branch tagged revision
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandException
     * @throws AuthenticationException
     * @throws CommandAbortedException
     * @throws IOException
     * 
     * @see #checkout(String)
     */
    public boolean commit( String module, String message, String revision ) 
        throws CommandAbortedException, 
               CommandException, 
               AuthenticationException, 
               IOException {

        File messageFile = null;
        
        Client client = getClient( password, workdir );
        CommitCommand command = new CommitCommand();
        command.setBuilder( null );
        
        // Write the log message to a file
        if( message != null ) {
            messageFile = File.createTempFile( "cvs-message", "txt" );
            FileWriter w = new FileWriter( messageFile );
            w.write( message );
            w.close();
            command.setLogMessageFromFile( messageFile.getAbsolutePath() );
        }
        
        
        File files[] = new File[1];
        files[0] = new File( workdir + File.separator + module );
        command.setFiles( files );
        
        boolean ret = client.executeCommand( command, globalOptions );

        closeConnection();
        if( messageFile != null )
            messageFile.delete();
        
        return ret;
    }

    /**
     * Update the input module. Prune option is not set.
     * 
     * @param module
     *            Module to update; must have been previously checked out of the
     *            repository with <code>checkout()</code>
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandException
     * @throws AuthenticationException
     * @throws CommandAbortedException
     * @throws IOException
     * 
     * @see #checkout(String)
     */
    public boolean update( String module) 
        throws CommandAbortedException, 
               CommandException, 
               AuthenticationException, 
               IOException {
        
        Client client = getClient( password, workdir );
        UpdateCommand command = new UpdateCommand();
        command.setBuilder( null );        
        command.setBuildDirectories(true);
        File files[] = new File[1];
        files[0] = new File( workdir + File.separator + module );
        command.setFiles( files );
        
        boolean ret = client.executeCommand( command, globalOptions );
    
        closeConnection();        
        return ret;
    }
    
    
    
    
    
    
    /**
     * Tag all sources in the input module with a given tag.
     * 
     * @param module
     *            Module to tag; must have been previously checked out of the
     *            repository with <code>checkout()</code>
     * 
     * @param tag
     *            The new tag; but be acceptable to CVS: no dots, dashes, etc.
     *            (see the documentation of the <code>cvs tag</code> command).
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandException
     * @throws AuthenticationException
     * @throws CommandAbortedException
     * @throws IOException
     * 
     * @see #checkout(String)
     */
    public boolean tag( String module, String tag ) 
        throws CommandAbortedException, 
               CommandException, 
               AuthenticationException, 
               IOException {
        
        Client client = getClient( password, workdir );
        TagCommand command = new TagCommand();
        command.setBuilder( null );        
        command.setOverrideExistingTag( true );
        command.setTag( tag );
        File files[] = new File[1];
        files[0] = new File( workdir + File.separator + module );
        command.setFiles( files );
 
        boolean ret = client.executeCommand( command, globalOptions );
    
        closeConnection();        
        return ret;
    }
    
    
    /**
     * Return all the tag names for the specified file.
     * 
     * 
     * @param module
     *            Module to tag; must have been previously checked out of the
     *            repository with <code>checkout()</code>
     * 
     * @param filename 
     * 			  File from which the log information are extracted.
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandException
     * @throws AuthenticationException
     * @throws CommandAbortedException
     * @throws IOException
     * 
     * @see #checkout(String)
     */
    public List getTags( String module, String filename ) 
        throws CommandAbortedException, 
               CommandException, 
               AuthenticationException, 
               IOException {
    	
    	List revisionList = null;
    	LogListener logListener = new LogListener();
        Client client = getClient( password, workdir );
        client.getEventManager().addCVSListener(logListener);
        
        LogCommand command = new LogCommand();
        LogBuilder builder = new LogBuilder(client.getEventManager(), command);
        command.setBuilder(builder);        
                
        File files[] = new File[1];
        files[0] = new File( workdir + File.separator + module + File.separator + filename);
        command.setFiles(files);
          
        boolean ret = client.executeCommand( command, globalOptions );   
        
        if (ret == true) {
        	revisionList = logListener.getRevisionList();
        } 
        
        closeConnection();    
        return revisionList;
    }
    
    
    
    
    
    /**
     * Return all the tag names for the specified file.
     * 
     * 
     * @param module
     *            Module to tag; must have been previously checked out of the
     *            repository with <code>checkout()</code>
     * 
     * @param filename 
     * 			  File from which the log information are extracted.
     * 
     * @return <code>true</code> if all went well, <code>false</code>
     *         otherwise.
     * 
     * @throws CommandException
     * @throws AuthenticationException
     * @throws CommandAbortedException
     * @throws IOException
     * 
     * @see #checkout(String)
     */
    public boolean isEsoformChanged( String module) 
        throws CommandAbortedException, 
               CommandException, 
               AuthenticationException, 
               IOException {
    	
    	boolean isChanged = false;
    	DiffListener diffListener = new DiffListener();
        Client client = getClient( password, workdir);
        client.getEventManager().addCVSListener(diffListener);
        
        DiffCommand command = new DiffCommand();
        Builder builder = command.createBuilder(client.getEventManager());
        command.setBuilder(builder);        
                
       
        boolean ret = client.executeCommand( command, globalOptions );   
        
        
        	isChanged = diffListener.isChanged();
        
        
        
        closeConnection();    
        return isChanged;
    }
    
    
    
    private void closeConnection() throws IOException {
        connection.close();
        connection = null;
    }

    /**
     * Opens and closes a connection to the CVS server. <BR>
     * For unit testing only.
     * 
     * @throws AuthenticationException
     * @throws CommandAbortedException
     * @throws IOException
     */
    void emptycommand() throws CommandAbortedException,
            AuthenticationException, IOException {

        Client client = getClient( password, workdir );
        closeConnection();
    }

    /**
     * @return A CVS client with a brand new connection and the given 
     *         console.
     */
    private Client getClient( String password, String localdir )
            throws CommandAbortedException, AuthenticationException {

        connection = new PServerConnection();
        connection.setUserName( getUser() );
        connection.setEncodedPassword( password );
        connection.setHostName( getHost() );
        connection.setRepository( getRepository() );
        connection.open();

        Client client = new Client( connection, new StandardAdminHandler() );
        client.setLocalPath( localdir );
        client.getEventManager().addCVSListener( console );
        
        return client;
    }
    
    /**
     * @return A CVS client with a brand new connection and the given 
     *         console.
     */
    private Client getClient( String password)
            throws CommandAbortedException, AuthenticationException {

        connection = new PServerConnection();
        connection.setUserName( getUser() );
        connection.setEncodedPassword( password );
        connection.setHostName( getHost() );
        connection.setRepository( getRepository() );
        connection.open();

        Client client = new Client( connection, new StandardAdminHandler() );
      //  client.setLocalPath( localdir );
        client.getEventManager().addCVSListener( console );
        
        return client;
    }

    /**
     * @return Returns the connectionType.
     */
    public String getConnectionType() {
        return connectionType;
    }

    /**
     * @return Returns the host.
     */
    public String getHost() {
        return host;
    }

    /**
     * @return Returns the repository.
     */
    public String getRepository() {
        return repository;
    }

    /**
     * @return Returns the user.
     */
    public String getUser() {
        return user;
    }

    /** Parse the input CVSROOT string and extract its components */
    private void parseCVSRoot( String root ) throws IllegalArgumentException {
        if( !root.startsWith( ":" ) )
            throw new IllegalArgumentException();

        int oldColonPosition = 0;
        int colonPosition = root.indexOf( ':', 1 );
        if( colonPosition == -1 )
            throw new IllegalArgumentException();
        connectionType = root.substring( oldColonPosition + 1, colonPosition );
        oldColonPosition = colonPosition;
        colonPosition = root.indexOf( '@', colonPosition + 1 );
        if( colonPosition == -1 )
            throw new IllegalArgumentException();
        user = root.substring( oldColonPosition + 1, colonPosition );
        oldColonPosition = colonPosition;
        colonPosition = root.indexOf( ':', colonPosition + 1 );
        if( colonPosition == -1 )
            throw new IllegalArgumentException();
        host = root.substring( oldColonPosition + 1, colonPosition );
        repository = root.substring( colonPosition + 1 );
        if( connectionType == null || user == null || host == null
                || repository == null )
            throw new IllegalArgumentException();
    }

    /**
     * Set the current working directory
     * @param working directory
     */
    public void setCurrentDir(String directory) {
    	this.workdir = directory;
    }
    
    /**
     * Returns the current working directory
     * @return working directory
     */
    public String getCurrentDir() {
    	return workdir;
    }
    
    
    //  Recursively delete a tree of directories
    private void remove( File root ) {
        if( root.isFile() )
            root.delete(); // delete individual files
        else {
            File files[] = root.listFiles();
            if( files != null )
                for( int i = 0; i < files.length; i++ )
                    remove( files[i] );

            root.delete(); // it's empty, can be deleted
        }
    }

    /**
     * Removes the local working directory and all its contents.
     */
    public void removeWorkdir() {
        File d = new File( workdir );
        remove( d );
    }
    
    private final class DiffListener extends CVSAdapter {

    	DiffInformation diffInfo = null;
    	DiffInformation.DiffChange diffChange_ = null;
    	public void fileInfoGenerated( FileInfoEvent event) {
    	      diffInfo = ((DiffInformation)event.getInfoContainer());
    	      if (diffInfo != null) {
    	      	diffChange_ = diffInfo.getFirstChange();
    	      }
    	}
    	
    	public boolean isChanged() {
    		if (diffChange_ != null) {
    			return true;
    		} else {
    			return false; 
    		}
    	}
    }
   
    
    
    private final class LogListener extends CVSAdapter {

    	List revList_ = null;  	
    	public void fileInfoGenerated( FileInfoEvent event) {
    	      revList_ = ((LogInformation)event.getInfoContainer()).getAllSymbolicNames();
    	}
    	
    	public List getRevisionList() {
    		return revList_;
    	}
    }
    
    
}
    