package org.eso.ohs.dfs;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.io.StringReader;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.AssocList;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;

/**
 * A business object for describing calibration requests.
 * 
 * @author Bob Kemp
 */

// $Id: CalibrationBlock.java,v 1.128 2006/04/27 10:37:43 tcanavan Exp $

public class CalibrationBlock extends BusinessObject {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(CalibrationBlock.class);

    private String instrumentComments;

    /**
     * States taken by a CalBlk or ObsBlk during the OB lifecycle. See Software
     * Requirements document for details.
     */

    public final static String[] QC_GRADE_VALUES = { "A", "B", "C", "D", "X" };

    public final static String PARTIALLY_DEFINED = "P";

    public final static String DEFINED = "D";

    public final static String ACCEPTED = "+";

    public final static String REJECTED = "-";

    public final static String QUEUED = "Q";

    public final static String INITIATED = "I";

    public final static String STARTED = "S";

    public final static String EXECUTED = "X";

    public final static String ABORTED = "A";

    public final static String CANCELED = "K";

    public final static String VERIFIED = "V";

    //public final static String CERTIFIED = "C";
    public final static String COMPLETED = "C";

    public final static String RELEASED = "R";

    public final static String ALARM = "!";

    public final static String MUST_REPEAT = "M";

    //  public final static String ARCHIVED = "%"; // item 1340
    public final static String PAUSED = "p";

    public final static int MIN_FLAMES_EXECTIME = 1200;

    /**
     * Calibration Type Code
     */

    public static final String CALBLK_TYPE = "C";

    private boolean execTimeUpToDate = true;

    /**
     * Observation Block Type Code
     */

    public static final String OBSBLK_TYPE = "O";

    private transient PropertyChangeEvent execTmeEvent = new PropertyChangeEvent(
            this, "ExecutionTime", null, null);

    /**
     * Mappings from the status field value to the displayed text
     */
    private static final AssocList<String,String> statusValues_ = 
        new AssocList<String,String>().add( "(P)artiallyDefined", PARTIALLY_DEFINED )
                       .add( "(D)efined", DEFINED )
                       .add( "(-)Rejected", REJECTED )
                       .add( "(+)Accepted", ACCEPTED )
                       .add( "(Q)ueued", QUEUED )
                       .add( "(I)nitiated", INITIATED )
                       .add( "(S)tarted", STARTED )
                       .add( "(p)aused", PAUSED )
                       .add( "(A)borted", ABORTED )
                       .add( "e(X)ecuted", EXECUTED )
                       .add( "(M)ustRepeat", MUST_REPEAT )
                       .add( "(V)erified", VERIFIED )
                       .add( "(!)Alarm", ALARM )
                       .add( "(C)ompleted", COMPLETED )
                       .add( "(R)eleased", RELEASED )
                       .add( "(K)anceled", CANCELED );

    //      .add("Archived", ARCHIVED) // item 1340
    //      .add("Certified", CERTIFIED)

    public static AssocList getStatusValues() {
        return statusValues_;
    }

    /**
     * Get the status code from its index from within the associated list
     * 
     * @param index
     * @return String
     */

    public static String getStatusCode(int index) {
        return statusValues_.get(index);
    }

    /**
     * Get the full description of status value from the code stored in the
     * database
     * 
     * @param statusCode
     * @return String
     */

    public static String getStatusCodeShortDescription(String statusCode) {
        return statusValues_.getKey(statusCode);
    }

    /**
     * Get the full description of status value from the code stored in the
     * database
     * 
     * @param statusCode
     * @return String
     */

    public static String getLongStatusValue(String statusCode) {
        return statusValues_.getKey(statusCode);
    }

    /**
     * Convert the "NewStatus" field of an ObsBlockStatus event generated by BOB
     * to a CB status value.
     * 
     * See Appx. G.2.4 of the OHS SRD and sec. 3.4.2 of the VCS-OHS ICD.
     * 
     * <br>
     * ChangeLog: A M Chavan, 19-Jan-2000, creation
     */

    public static final AssocList<String,String> bobStatusValues =
        // amchavan, 13-Jan-2006: adapted to new AssocList implementation
        new AssocList<String,String>( false ).add( "VERIFYFAIL", ABORTED)
                              .add("STARTED", STARTED)
                              .add("PAUSED", PAUSED)
                              .add("CONTINUED", STARTED)
                              .add("ABORTED", ABORTED)
                              .add("MUSTREPEAT", MUST_REPEAT)
                              .add("TERMINATED", EXECUTED);

    /** Version id for object serialization */
    static final long serialVersionUID = 3976534496581525077L;

    /**
     * The Observing Run to which this observation block belongs.
     */
    private transient ObservingRun obsRun_;

    /**
     * Information provided by the observer back to the OB author.
     */
    private String obsComments_ = "";

    /**
     * Reference to the associated Observation Description
     */
    private transient ObservationDescription obsDesc_;

    /**
     * Is the OB a calibration standard? If false, the observation is repeated
     * nightly as a standard calibration observation of a star.
     */
    private boolean standard_ = false;

    /**
     * Status of this CalibrationBlock (a single char).
     */
    private String status_ = PARTIALLY_DEFINED;

    public static final String EDITED_BY_USG = "E";

    /**
     * Information provided by the OB author to the observer
     */
    private String userComments_ = "";

    /**
     * Priority given by the user to this block; lower numbers indicate higher
     * priority.
     */
    private int userPriority_ = 1;

    /**
     * Calculated execution time (in seconds) cmavros added made 04/Sept/2001
     * 
     */
    private int executionTime_;

    /**
     * Redirects property change events for linked Targets and ODs to this
     * object, so that (eg) name changes in the Target are detected.
     * <p>
     * XXX This is a bit of a hack but there seems no cleaner way to do it.
     */
    protected transient PropertyChangeListener eventRedirector_ = new EventForwarder(
            this);

    public static int FLAMES_SETUP = 60;

    private int expTime_;

    private PropertyChangeEvent execTimeEvent;

    public static final String[] markLabels = { "(P)artially defined",
            "(D)efined, after phase II", "(-)Rejected, OB verification failed",
            "(+)Accepted OB verified", "(Q)ueued for scheduling",
            "(I)nitiated, passed on to VCS",
            "(S)tarted ,after VCS stated execution",
            "(P)aused ,during execution", "(A)borted",
            "e(X)ecuted ,all exposures taken", "(M)ust repeat",
            "(V)erified, Passed Quality Control", "(!)Alarm, QC not passed",
            "(C)ompleted,passed quality control",
            "(R)eleased,data was distributed", "(K)ancelled" };

    /**
     * Default constructor, mainly used by deserialization code.
     */
    CalibrationBlock() {
    }

    /**
     * MIPS Project: Set the ipversion of all templates in this object. Not, the
     * "conversion" to the new template does not occur until refreshTemplates()
     * is called.
     */
    public void convertIPVersion(float ipv) {
        obsDesc_.convertIPVersion(ipv);
    }

    /**
     * MIPS Project: Reload the templates (converting as required to a new IP
     * version). The firePropChangeEvent indicates if a property change event
     * should be fired as a result of the change (this will trigger the
     * autosave).
     */
    public void refreshTemplates(boolean firePropChangeEvent)
            throws IOException {
        refreshTemplates(firePropChangeEvent, false);
    }

    public void refreshTemplates(boolean firePropChangeEvent,
            boolean fullRefresh) throws IOException {
        obsDesc_.refreshTemplates(firePropChangeEvent, fullRefresh);
        if (firePropChangeEvent) {
            firePropertyChangeEvent();
        }
    }

    /**
     * @return value for ipversion from the OD, if there is one, else -1. This
     *         method is not called getIPVersion() as IPVersion is not a
     *         property of this class. This method is a short-cut for
     *         this.getOd().getIPVersion().
     * 
     * <br>
     * ChangeLog: amchavan, 22-Oct-2001: moved here from ObservationBlock.java,
     *  
     */
    public float fetchIPVersionFromOd() {
        float rc = -1;
        if (getOd() != null) {
            rc = getOd().getIPVersion();
        }
        return rc;
    }

    //protected boolean isNTT() {
    //  if("NTT".equals(obsRun_.getTelescope()))
    //      return true;
    //  return false;
    // }

    /**
     * Set the ipVersion_ property to the value of the <code>VERSION</code>
     * keyword of the Instrument Summary File.
     * 
     * <br>
     * ChangeLog: amchavan, 22-Oct-2001: Creation.
     */
    /*
     * public void setIPVersion() {
     * 
     * ObservationDescription od = getOd();
     * 
     * if( od != null ) {
     * 
     * String instName = od.getInstrument(); Instrument inst =
     * InstrumentList.getInstance().getInstrument( instName ); ipVersion_ =
     * inst.getVersion(); // stdlog_.debug( ">>>
     * CalibrationBlock.setIPVersion(): " + ipVersion_ ); // We do NOT throw a
     * PropertyChangeEvent because // this method is only called just before
     * writing // to the local cache -- therefore, no GUI update // is needed. //
     * amchavan, 22-Oct-2001 // NO NO NO // firePropertyChangeEvent(); // NO NO
     * NO } }
     */

    /**
     * Set the ipversion in the OD, if there is an OD. This method is NOT named
     * setIPVerion() as ipversion is not a property of this object.
     * 
     * This method is a "short cut" for this.getOd().setIPVersion().
     * 
     * @param newValue
     *            the new value for the property
     * 
     * <br>
     * ChangeLog: amchavan, 22-Oct-2001: moved here from ObservationBlock.java
     */
    public void putIPVersionToOd(float version) {
        if (getOd() != null) {
            getOd().setIPVersion(version);
        }
    }

    // @@@@ Hack to aid reflection;
    public String getType() {
        return CALBLK_TYPE;
    }

    public void setType(String s) {
    }

    /**
     * Set new value for obsRun_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setObsRun(ObservingRun newValue) {
        if (newValue == null) {
            throw new IllegalArgumentException("ObservingRun cannot be null");
        } else {
            obsRun_ = newValue;
            firePropertyChangeEvent();
        }
    }

    /**
     * Get value for obsRun_ property
     * 
     * @return the current value of the property
     */
    public ObservingRun getObsRun() {
        return obsRun_;
    }

    /**
     * Set new value for obsComments_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setObsComments(String newValue) {
        obsComments_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for obsComments_ property
     * 
     * @return the current value of the property
     */
    public String getObsComments() {
        if (obsComments_==null)
            return("");
        else
            return obsComments_;
    }

    /**
     * Set new value for obsDesc_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setOd(ObservationDescription newValue) {
        if (obsDesc_ != null) {
            obsDesc_.removePropertyChangeListener(eventRedirector_);
        }
        obsDesc_ = newValue;
        if (obsDesc_ != null) {
            obsDesc_.addPropertyChangeListener(eventRedirector_);
        }
        firePropertyChangeEvent();
    }

    /**
     * Get value for obsDesc_ property
     * 
     * @return the current value of the property
     */
    public ObservationDescription getOd() {
        return obsDesc_;
    }

    /**
     * Set new value for standard_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setStandard(boolean newValue) {
        standard_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for standard_ property
     * 
     * @return the current value of the property
     */
    public boolean getStandard() {
        return standard_;
    }

    /**
     * Set new value for status_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setStatus(String newValue) {
        status_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for status_ property
     * 
     * @return the current value of the property
     */
    public String getStatus() {
        if (status_ == null)
            return("");
        else
            return status_;
    }

    /**
     * Set new value for userComments_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setUserComments(String newValue) {
        userComments_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for userComments_ property
     * 
     * @return the current value of the property
     */
    public String getUserComments() {
        if (userComments_==null)
            return("");
        else
            return userComments_;
    }

    /**
     * Set new value for userPriority_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setUserPriority(int newValue) {
        userPriority_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for userPriority_ property
     * 
     * @return the current value of the property
     */
    public int getUserPriority() {
        return userPriority_;
    }

    /**
     * Return the first template signature in the list of templates
     * 
     * @return TemplateSignature
     */

    public TemplateSignature firstTS() {
        TemplateSignature[] tsfs = getOd().getSignatures();
        if (tsfs != null && tsfs.length > 0) {
            return tsfs[0];
        }
        return null;
    }

    /**
     * Set new value for executionTime_ property
     * 
     * cmavros change made 04/Sept/2001
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setExecutionTime(int newValue) {
        stdlog_.debug(">>>>>>>!!!!!!!!!!!! Calling Execution Time" + newValue);
        executionTime_ = newValue;
        if (execTimeEvent == null)
            execTmeEvent = new PropertyChangeEvent(this, "ExecutionTime", null,
                    null);
        firePropertyChangeEvent(execTmeEvent);
    }

    /**
     * Get value for executionTime_ property
     * 
     * cmavros change made 04/Sept/2001
     * 
     * @return the current value of the if(bo instanceof ObservationBlock)
     *         CalibrationBlock.getCalculateExecutionTime((ObservationBlock)bo);
     *         if(bo instanceof CalibrationBlock)
     *         CalibrationBlock.getCalculateExecutionTime((CalibrationBlock)bo);
     *         property
     */
    public int getExecutionTime() {
        if (executionTime_ < 0 || executionTime_ > Convert.EXECTIME_MAX)
            executionTime_ = Convert.EXECTIME_ERR;
        return executionTime_;
    }

    /**
     * Set new value for executionTime_ property cmavros change made
     * 04/Sept/2001
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setExposureTime(int newValue) {
        stdlog_.debug("!!!!!!!!!!!!!!! Exposure Time " + newValue);
        expTime_ = newValue;
    }

    /**
     * Get value for executionTime_ property
     * 
     * cmavros change made 04/Sept/2001
     * 
     * @return the current value of the property
     */
    public int getExposureTime() {
        if (expTime_ < 0 || expTime_ > Convert.EXECTIME_MAX)
            expTime_ = Convert.EXECTIME_ERR;
        return expTime_;
    }

    public String toOBD(boolean addHeader, boolean display, long id,String desc) {
        String text = null;

        if (addHeader) {
            text = getOBDHeader(desc) + getOBDBody(id);
        } else {
            text = getOBDBody(id);
        }
        text += "\n";
        text += "\n";

        if (getOd() != null) {
            TemplateSignature[] tss = getOd().getSignatures();
            for (int j = 0; j < tss.length; j++) {
                text += toParameterFormat(tss[j], j + 2, display);
            }
        }

        return text;
    }

    public String toOBD(boolean addHeader, long id,String desc) throws IOException {
        return toOBD(addHeader, false, id,desc);
    }

    protected String getOBDTemplateHeader(TemplateSignature ts, int seqno) {
        StringBuffer sb = new StringBuffer();
        String mode = (ts.getMode() == null) ? "" : ts.getMode();
        sb.append("# " + ts.getTemplateType() + " template "
                + ts.getTemplateName());
        sb.append("\n");
        sb.append("TPL.ID    \"" + ts.getTemplateName() + "\"");
        sb.append("\n");
        sb.append("TPL.NAME  \"" + ts.getTemplateDesc() + "\"");
        sb.append("\n");
        sb.append("TPL.MODE \"" + mode + "\"");
        sb.append("\n");
        return sb.toString();
    }

    public String getVIMOSADMParameterValue(Parameter p, boolean display,
            String vimosDateTime) {
        String value = p.getValue();
        if (!value.equals("NODEFAULT")) {
            try {
                PafFile f = new PafFile(new StringReader(value));
                f.setKeywordValue("PAF.NAME", vimosDateTime + "-101-"
                        + getKeywordIndex(p.getParameterName()) + ".adm");
                if (!display)
                    return p.toOBDParameterString(f.toFileString());
                else
                    return p.toDisplayOBDParameterString(f.toFileString());
            } catch (IOException e) {
                stdlog_.debug("This is not a paf file");
            }
        }
        return value;
    }

    private static int getKeywordIndex(String keyword) {
        int index = -1;
        int startIndex = -1;
  
        for (int k = 0; k < keyword.length(); k++) {
            if (Character.isDigit(keyword.charAt(k))) {
                startIndex = k;
            }
        }
        //dIndex = keyword.lastIndexOf('.');
        String idxStr = keyword.substring(startIndex, startIndex + 1);
        try {
            index = new Integer(idxStr).intValue();
        } catch (NumberFormatException e) {
            //stdlog_.error( e );
        }
        return index;
    }

    /**
     * Return the parameter string needed for communication with BOB
     * 
     * @return String to send to BOB for execution.
     */
    public String toParameterFormat(TemplateSignature ts, int seqno,
            boolean display) {
        StringBuffer sb = new StringBuffer();
        sb.append(getOBDTemplateHeader(ts, seqno));
        Parameter[] params = ts.getParamList();
        String vimosDateTimeStr = TextUtils.currentISODateTime();
        for (int j = 0; j < params.length; j++) {
            if (params[j].isFixed()) {
                continue;
            }
            if (params[j].getParameterName().indexOf("INS.ADM") != -1
                    && getOd().getInstrument().equals("VIMOS")) {
                sb.append(getVIMOSADMParameterValue(params[j], display,
                        vimosDateTimeStr));
            } else {
                if (display)
                    sb.append(params[j].toDisplayOBDParameterString());
                else
                    sb.append(params[j].toOBDParameterString());
            }
        }
        sb.append("\n");
        return sb.toString();
    }

    
    /**
     * Set the header in the description.
     * 
     * @return an OBD header
     */
    protected String getOBDHeader(String desc) {

        String text;

        String dateTime = TextUtils.currentISODateTime();

        text = "PAF.HDR.START;                             # Marks start of header"
                + "\nPAF.TYPE           \"OB Description\"; # Type of param file"
                + "\nPAF.ID             \"\""
                + "\nPAF.NAME           \"\""
                + "\nPAF.DESC           \"\""
                + "\nPAF.CRTE.NAME      \""
                + desc
                + "\";  # Creator"
                + "\nPAF.CRTE.DAYTIM    \""
                + dateTime
                + "\";      # Creation date"
                + "\nPAF.LCHG.NAME      \"\";                   # Who did last change"
                + "\nPAF.LCHG.DAYTIM    \""
                + dateTime
                + "\";      # Time of last change"
                + "\nPAF.CHCK.NAME      \"\";                   # Applic checking file"
                + "\nPAF.CHCK.DAYTIM    \"\";                   # Date of last check"
                + "\nPAF.CHCK.CHECKSUM  \"\";                   # Param file checksum"
                + "\nPAF.HDR.END" + "\n" + "\n";

        return text;
    }

    
    

    /**
     * Returns the body of an OBD format representation of this object.
     * 
     * cmavros 17/Sept/01 Add Execution Time
     * 
     * @return an OBD body
     */
    protected String getOBDBody(long id) {

        String obName = getName().replace(' ', '-').replace('\t', '-');

     //   String dateTime = new Date().toString();
        String text = "# Observation Block description follows\n";

        text += "\n";

        text += "\nOBS.ID\t\"" + id + "\"\t# Observation block Id";

        text += "\nOBS.NAME\t\"" + obName + "\"\t# Observation block Name";

        ObservingRun or = getObsRun();
        String orName = or.getName();
        int slashIndex = orName.indexOf('/');
        if (slashIndex >= 0) {
            // JP2PP makes names like "63.H-0420(A)/VM/FORS1"
            orName = orName.substring(0, slashIndex);
        }

        text += "\nOBS.GRP	      \"0\"		# Unused";
        text += "\nOBS.PROG.ID    \"" + orName + "\"";
        text += "\nOBS.PI-COI.ID  \"" + getOwnerId() + "\"";
        text += "\nOBS.PI-COI.NAME  \"" + "UNKNOWN" + "\"";
        /*
         * Commented out - this keyword breaks existing OSs on Paranal. To be
         * reintroduced when OBS.EXECTIME is officially part of the VLT OBS
         * dictionary. amchavan, 12-Dec-2001
         * 
         * cmavros 06May02 0919/P2PP and 1710/OT Reintroduce exectime
         *  
         */
        text += "\nOBS.EXECTIME  \"" + getExecutionTime() + "\"";
        text += "\nOBS.OBSERVER  \"" + "UNKNOWN" + "\"";

        return text;
    }

    /**
     * Private method used by the Java Serialisation library.
     * 
     * @exception IOException
     *                thrown if there are I/O errors from the underlying stream
     */
    private void writeObject(ObjectOutputStream out) throws IOException {
        out.defaultWriteObject();
    }

    /**
     * Private method used by the Java Serialisation library.
     * 
     * @exception IOException
     *                thrown if there are I/O errors from the underlying stream
     * @exception ClassNotFoundException
     *                thrown if a class in the serialised stream does not have a
     *                corresponding .class file known to the system.
     */
    private void readObject(ObjectInputStream in) throws IOException,
            ClassNotFoundException {
        eventRedirector_ = new EventForwarder(this);
        in.defaultReadObject();
    }

    /**
     * Verifies this block and all attached blocks, returning the concatenation
     * of all error messages.
     * 
     * @return the (possibly empty) concatenation of all error messages.
     */
    public String verify() {

        StringBuffer sb = new StringBuffer();

        sb.append(super.verify());
        if (getOd() == null) {
            sb.append("You have not defined an Observation Description.\n");
        } else {
            sb.append(getOd().verify());
        }

        return sb.toString();
    }

    public boolean verifyCheck() {
        String ret = verify();
              
        return ( ret.equals("") );

    }
    
    public String verifyBO() {
        return super.verify();
    }

    /**
     * Returns an array of the resources used by each template. Each resource is
     * itself specified as an array of strings.
     * 
     * @return an array of arrays of strings
     */
    public String[][] getResources() {
        return new String[][] {};
    }

    /**
     * @return The new status this CB gets when the corresponding event is
     *         received from BOB. See Appx. G.2.4 of the SRD and sec. 3.4.2 of
     *         the VCS-OHS ICD.
     * 
     * <br>
     * ChangeLog: A M Chavan, 19-Jan-2000, adapted from the now obsolete method
     * convertLogtoStatus().
     */
    public static String convertBobEventToStatus(String bobNewStatus) {
        // stdlog_.debug(">>> BobEvent->Status: '" + bobNewStatus + "'");
        String status = (String) bobStatusValues.get(bobNewStatus);

        return status != null ? status : "?";
    }

    /**
     * Returns the string used to refer to this type of BusinessObject in
     * messages to the user.
     * 
     * @return user-oriented name
     */
    public static String getDisplayName() {
        return "CalBlock";
    }

    /**
     * Returns the filename suffix for files containing this type of object.
     * 
     * @return suffix for filename when stored in local filesystem
     */
    public static String getSuffix() {
        return "cb";
    }

    /**
     * Returns an array of the immediate child BusinessObjects for this BO.
     * 
     * @return an array of business objects referred to by this BO.
     */
    public BusinessObject[] getBusObjChildren() {
        // A CalBlk can only have 1 attached BusinessObject
        return (getOd() != null) ? new BusinessObject[] { getOd() }
                : new BusinessObject[] {};
    }

    /**
     * Returns a deep copy of this BO, ie recursively copies this BO and all of
     * it's attached BOs.
     * 
     * @return a deep copy of this business object.
     */
    public BusinessObject deepCopy() {
        CalibrationBlock cb = null;
        ObservationDescription od = null;

        cb = (CalibrationBlock) copy();
        if (getOd() != null) {
            od = (ObservationDescription) getOd().deepCopy();
            cb.setOd(od);
        }

        return cb;
    }

    /**
     * Returns a copy of the current object, identical except for it's id and
     * the medium and directory containing it.
     * 
     * @param device
     *            the device on which the new object should reside
     * @param dir
     *            the directory in which the new object should reside
     * @return a clone of the current BusinessObject (or subclass)
     */
    public BusinessObject copy() {
        CalibrationBlock cb = (CalibrationBlock) super.copy();

        cb.eventRedirector_ = new EventForwarder(cb);

        // Show user it's _not_ checked-in.
        cb.setStatus(CalibrationBlock.PARTIALLY_DEFINED);

        return cb;
    }

    public boolean isFlames() {
        if (getOd().getInstrument().equals("FLAMES"))
            return true;
        return false;
    }

    /**
     * Generic hook for "visitor" Design Pattern. See the BusinessVisitor
     * interface for more details.
     * 
     * @param visitor
     *            the class whose visit() method we wish to invoke.
     * 
     * @see BusinessVisitor
     */
    public Object accept(BusinessVisitor visitor, Object arg) {
        return visitor.visit(this, arg);
    }

    //getObd();

    //    Media device =
    //        ObjectManager.getObjectManager().lookup(getId()).getMedia();
    //
    //    long id;
    //    if (device.equals(Media.DBASE))
    //        id = getDbaseId();
    //    else {
    //        id =
    //            (getDbaseId() != 0)
    //                ? Config.getCfg().uniqueToTableId(getDbaseId())
    //                : getId();
    //    }

    //toReportString
    //+ Config.getCfg().uniqueToTableId(getDbaseId())

    //    if (getId() < 0) //Local
    //       buf.append(
    //            getName()
    //                + " (OB Id="
    //                + id
    //                + ") ");
    //    else {
    //        buf.append(getName() + " (OB Id=" + getDbaseId() + ") ");
    //    }

    public String toReport(long dbId) {
        StringBuffer buf = new StringBuffer();
        if (dbId != 0) {
            buf.append(getName() + " (OB Id=" + dbId + ") ");
        } else {
            buf.append(getName());
            buf.append(" ");
        }
        buf.append(getObsRun().getName() + "\n");
        String usrCom = getUserComments();
        if (usrCom != null && usrCom.length() > 0) {
            buf.append("User Com. : ");
            String[] usrStrings = TextUtils.splitString(usrCom, 60);
            for (int i = 0; i < usrStrings.length; i++) {
                if (i != 0)
                    buf.append("            ");
                buf.append(usrStrings[i] + "\n");
            }

        }
        buf.append("\n");
        buf.append("ObsDesc: " + obsDesc_.getName() + "\n");
        TemplateSignature[] tsfs = obsDesc_.getSignatures();
        for (int i = 0; i < tsfs.length; i++) {
            buf.append(tsfs[i].toReportString());
            buf.append("\n");
        }
        buf.append("\n"
                + "====================================================="
                + "\n");
        return buf.toString();
    }

    /**
     * Utility class used to contain status values and the corresponding text to
     * be displayed to the user.
     */
    public static class StatusValue implements Serializable {

        /** Version id for object serialization */
        static final long serialVersionUID = 6564143740396077923L;

        public final String internal;

        public final String displayed;

        public StatusValue(String msg, String name) {
            internal = name;
            displayed = msg;
        }

        public StatusValue(String msg, String name, String disp) {
            internal = name;
            displayed = disp;
        }

    }

    /**
     * Returns the execTimeUpToDate.
     * 
     * @return boolean
     */
    public boolean isExecTimeUpToDate() {
        return execTimeUpToDate;
    }

    /**
     * Sets the execTimeUpToDate.
     * 
     * @param execTimeUpToDate
     *            The execTimeUpToDate to set
     */
    public void setExecTimeUpToDate(boolean execTimeUpToDate) {
        this.execTimeUpToDate = execTimeUpToDate;
        firePropertyChangeEvent();
    }

    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }

    /**
     * Set the instrument comments for a CB/OB
     * 
     * @param string
     */
    public void setInstrumentComments(String string) {
        instrumentComments = string;
        firePropertyChangeEvent();
    }

    /**
     * Get the instrument specific comments for an OB.
     * 
     * @return Get the instrument specific comments for an OB.
     */
    public String getInstrumentComments() {
        if ( instrumentComments == null )
            return("");
        else
            return instrumentComments;
    }

	/**
	 * Compares two <code>CalibrationBlock</code> objects
	 * 
	 * @param other
	 * @return boolean
	 */
	public boolean equals(CalibrationBlock other) {
	    
	    if (getInstrumentComments().compareTo(other.getInstrumentComments()) != 0)
	        return(false);
	    
	    if (getObsComments().compareTo(other.getObsComments()) != 0)
	        return(false);
	    
	    if (getUserComments().compareTo(other.getUserComments()) != 0)
	        return(false);
	    
	    if (getUserPriority() != other.getUserPriority())
	        return(false);
	    
	    if (getStatus().compareTo(other.getStatus()) != 0 ) 
	    	return(false);
	    
	    return( getOd().equals(other.getOd()));

	}
    
}