package org.eso.ohs.dfs;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DecimalFormat;
import java.util.HashMap;

import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.Paramfile;
import org.eso.ohs.core.utilities.ReflectionException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentConstraintConfig;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ParamFileParameter;
/**
 * 
 * 
 * <p>
 * $Id: ConstraintSet.java,v 1.52 2006/06/20 13:57:05 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */

public class ConstraintSet extends BusinessObject {

    /** Version id for object serialization */
    static final long serialVersionUID = -7526074509242804567L;

    /** Maximum airmass: between 1 and 5 */
    private float airmass_ = 5.0f;

    /** Maximum Fractional Lunar Illumination: between 0 and 1 */
    private float fractionalLunarIllumination_ = 1.0f;

    /**
     * Minimum allowed angular distance between the target and the Moon,
     * in degress between 0 and 180
     */
    private int moonAngularDistance_ = 30;

    /**
     * Minimun accepted Seeing or Image Quality: is specified to the R band.
     * Expressed as a number between 0.2 and 2.0 (image size in arcseconds)
     */
    private float seeing_ = (float)Seeing.MIN_SEEING;
    /**
     * Max accepted Seeing or Image Quality: is specified to the R band.
     * Expressed as a number between 0.2 and 2.0 (image size in arcseconds)
     */
    private double  seeingMax_ = Seeing.MAX_SEEING;
    /**
     * Wind angle:
     * Expressed as a number between -180 and 180 degree
     */
    private int windAngle = WindInfo.DEFAULT_WIND_ANGLE;

    /**
     * SkyTransparency can be defined as "Photometric", "Clear",
     * "Variable, thin cirrus", "Variable, thick cirrus"
     */
    private String skyTransparency_ = SkyTransparency.validSkyTransparencyViewArray[0];

    /** Telescope temperature in degrees celsius, between -20 and +60 */
    private int telTemp_ = 0;

    /** Precipitable water vapour can be defined as "Low", "Median" or "Any" */
    private String waterVapour_ = validWaterVapour_[0];

    /** Description of the ConstraintSet for the PI astronomer to fill out */
    private String description_;

    /** Strehl Ratio (float value between 0.0 < sr < 100.0). */
    private float strehlRatio_ = (float) 0.0;
    
    
    private String baseLine;

    /**
     * Legal values for the WaterVapour property
     */
    private static String[] validWaterVapour_ = { "Low", "Median", "Any" };

 
    /**
     * The name of the config keyword searched for in OHSConfig which initializes
     * the strehlRationParamKeyword
     */
    public static final String strehlRatioParamKeyword = OHSKeys.CSStrehlParam;

    /**
     * null constructor
     */
    public ConstraintSet() {
    }

    /**
     * Returns an array of the valid values for the SkyTransparency property.
     * 
     * @return array of strings
     */
    public static String[] getValidSkyTransparency() {
        String[] valid = new String[SkyTransparency.validSkyTransparencyViewArray.length];
        System.arraycopy(SkyTransparency.validSkyTransparencyViewArray, 0, valid, 0, valid.length);

        return valid;
    }

    /**
     * Returns an array of the valid values for the SkyTransparency property.
     * 
     * @return array of strings
     */
    public static String[] getDbSkyTransparency() {
        String[] valid = new String[SkyTransparency.skyTransparencyDbRange.length];
        System.arraycopy(SkyTransparency.skyTransparencyDbRange, 0, valid, 0, valid.length);

        return valid;
    }
    /**
     * Returns the SkyTransparency property DB value.
     * 
     * @return string
     */
    public  String getCurrDbSkyTransparency() {
        String sky = getSkyTransparency();
        String dbSky = SkyTransparency.toDbSkyTransparency(sky); 
        return dbSky;
    }

    /**
     * Returns the SkyTransparency value corresponding to a column value
     * in the database.
     * 
     * @return SkyTransparency value
     */
    public static String fromDbSkyTransparency(String dbValue) {
        for (int i = 0; i < SkyTransparency.skyTransparencyDbRange.length; i++) {
            if (SkyTransparency.skyTransparencyDbRange[i].equals(dbValue))
                return SkyTransparency.validSkyTransparencyViewArray[i];
        }
        return null;
    }  

    /**
     * Returns the SkyTransparency index corresponding to a column value
     * in the database.
     * 
     * @return SkyTransparency value
     */
    public static int fromDbSkyTransparencySort(String dbValue) {
        for (int i = 0; i < SkyTransparency.skyTransparencyDbRange.length; i++) {
            if (SkyTransparency.skyTransparencyDbRange[i].equals(dbValue))
                return i;
        }
        return SkyTransparency.skyTransparencyDbRange.length;
    }
    /**
     * Returns the SkyTransparency index corresponding to the user sky transp.
     * 
     * 
     * @return SkyTransparency value
     */
    public static int fromValidSkyTransparencySort(String dbValue) {
        for (int i = 0; i < SkyTransparency.validSkyTransparencyViewArray.length; i++) {
            if (SkyTransparency.validSkyTransparencyViewArray[i].equals(dbValue))
                return i;
        }
        return SkyTransparency.validSkyTransparencyViewArray.length;
    }

    /**
     * Get value for Airmass property
     *
     * @return the current value of the property
     */
    public float getAirmass() {
        return airmass_;
    }

    /**
     * Get value for FractionalLunarIllumination property
     *
     * @return the current value of the property
     */
    public float getFractionalLunarIllumination() {
        return fractionalLunarIllumination_;

    }

    /**
     * Get value for SkyTransparency property
     *
     * @return the current value of the property
     */
    public String getSkyTransparency() {
        if (skyTransparency_ == null)
            return("");
        else
            return skyTransparency_;
    }

    /**
     * Get value for MoonAngularDistance property
     *
     * @return the current value of the property
     */
    public int getMoonAngularDistance() {
        return moonAngularDistance_;

    }

    /**
     * Get value for Seeing property
     *
     * @return the current value of the property
     */
    public float getSeeing() {
        
        return seeing_;
    } 
    
    /**
     * Get value for Seeing property
    *
    * @return the current value of the property
    */
   public double getMaxSeeing() {
       return seeingMax_;
   }

    /**
     * Get value for TelTemp property
     *
     * @return the current value of the property
     */
    public int getTelTemp() {
        return telTemp_;
    }

    /**
     * Get value for WaterVapour property
     *
     * @return the current value of the property
     */
    public String getWaterVapour() {
        if (waterVapour_==null)
            return("");
        else
            return waterVapour_;
    }

    /**
     * Get value for Description property
     *
     * @return the current value of the property
     */
    public String getDescription() {
        if (description_==null) 
            return("");
        else
            return description_;
    }

    /**
     * @return the Strehl Ratio
     */
    public float getStrehlRatio() {
        return strehlRatio_;
    }

    /**
     * set the Strehl Ratio. Note that 0.0 is accepted although it is a non-sense value.
     * This is because a default value if 0.0 will be read from the database and needs
     * to be handled without exception.
     */
    public void setStrehlRatio(float sr) {
        if (sr >= 100.0) {
            throw new IllegalArgumentException("strehl ratio not in range 0.0 <= sr < 100.0");
        }
        strehlRatio_ = sr;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for Airmass property
     *
     * @param newValue the new value for the property
     */
    public void setAirmass(float airmass) {
        if (airmass < 1 || airmass > 5) {
            throw new IllegalArgumentException("airmass not in range 1 to 5");
        }
        airmass_ = airmass;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for Description property
     *
     * @param newValue the new value for the property
     */
    public void setDescription(String description) {
        description_ = description;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for FractionalLunarIllumination property
     *
     * @param newValue the new value for the property
     */
    public void setFractionalLunarIllumination(float fractIllum) {
        if (fractIllum < 0 || fractIllum > 1) {
            throw new IllegalArgumentException("fractional lunar illumination not in range 0 to 1");
        }

        fractionalLunarIllumination_ = fractIllum;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for MoonAngularDistance property
     *
     * @param newValue the new value for the property
     */
    public void setMoonAngularDistance(int moonAngularDistance) {
        if (moonAngularDistance < 0 || moonAngularDistance > 180) {
            throw new IllegalArgumentException("moon angular distance not in range 0 to 180");
        }
        moonAngularDistance_ = moonAngularDistance;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for Seeing property
     *
     * @param newValue the new value for the property
     */
    public void setSeeing(float seeing) {
        DecimalFormat df = new DecimalFormat("#.#");
        float seeingTmp = Float.parseFloat(df.format(seeing));
        if (seeingTmp < Seeing.MIN_SEEING || seeingTmp > Seeing.MAX_SEEING) {
            throw new IllegalArgumentException("seeing/image quality not in range "+Seeing.MIN_SEEING+" to "+Seeing.MAX_SEEING);
        }
        
        seeing_ = seeingTmp;
        firePropertyChangeEvent();
    }
    
    /**
     * Set new value for Seeing property
     *
     * @param newValue the new value for the property
     */
    public void setMaxSeeing(float seeing) {
        if (seeing < Seeing.MIN_SEEING || seeing > Seeing.MAX_SEEING) {
            throw new IllegalArgumentException("seeing/image quality not in range "+Seeing.MIN_SEEING+" to "+Seeing.MAX_SEEING);
        }
        seeingMax_ = seeing;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for TelTemp property
     *
     * @param newValue the new value for the property
     */
    public void setTelTemp(int telTemp) {
        telTemp_ = telTemp;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for WaterVapour property
     *
     * @param newValue the new value for the property
     */
    public void setWaterVapour(String waterVapour) {
        for (int index = 0; index < validWaterVapour_.length; index++) {
            if (validWaterVapour_[index].equals(waterVapour)) {
                waterVapour_ = waterVapour;
                firePropertyChangeEvent();
                return;
            }
        }

        throw new IllegalArgumentException(
            "water vapour unrecognised: " + waterVapour);
    }

    /**
     * Set new value for SkyTransparency property
     *
     * @param newValue the new value for the property
     */
    public void setSkyTransparency(String skyTransparency) {
        for (int index = 0; index < SkyTransparency.validSkyTransparencyViewArray.length; index++) {
            if (SkyTransparency.validSkyTransparencyViewArray[index].equals(skyTransparency)) {
                skyTransparency_ = skyTransparency;
                firePropertyChangeEvent();
                return;
            }
        }

        throw new IllegalArgumentException(
            "sky transparency not recognised: " + skyTransparency);
    }

    /**
     * Returns the string used to refer to this type of BusinessObject
     * in messages to the user.
     *
     * @return user-oriented name
     */
    public static String getDisplayName() {
        return "ConSet";
    }

    /**
     * Returns the filename suffix for files containing this type of object.
     *
     * @return suffix for filename when stored in local filesystem
     */
    public static String getSuffix() {
        return "cs";
    }

    /**
     * Returns a string describing the property values of this object,
     * used by toString()
     *
     * @return string describing the objects state.
     */
    public String paramString() {
        return super.paramString()
            + ",airmass="
            + airmass_
            + ",lunarIllumin="
            + fractionalLunarIllumination_
            + ",moonDist="
            + moonAngularDistance_
            + ",seeing="
            + seeing_
            + ",skyTransp="
            + skyTransparency_
            + ",telescopeTemp="
            + telTemp_
            + ",strehlRatio="
            + strehlRatio_
            + ",waterVapour="
            + waterVapour_
            + ",description=<"
            + description_
            + ">";
    }

    public String toReportString() {
        StringBuffer buf = new StringBuffer();
        buf.append("ConSet: " + getName() + "\n");
        buf.append("Seeing                        : " + getSeeing() + "\n");
        buf.append(
            "Sky transparency              : " + getSkyTransparency() + "\n");
        buf.append("Air mass                      : " + getAirmass() + "\n");
        buf.append(
            "Strehl Ratio                  : " + getStrehlRatio() + "\n");
        buf.append(
            "Fractional lunar illumination : "
                + getFractionalLunarIllumination()
                + "\n");
        buf.append(
            "Moon angular distance         : "
                + getMoonAngularDistance()
                + "\n");
        if (description_ != null && description_.length() > 0) {
            buf.append("User Com. : ");
            String[] descStrings = TextUtils.splitString(description_, 60);
            for (int i = 0; i < descStrings.length; i++) {
                if (i != 0)
                    buf.append("            ");
                buf.append(descStrings[i] + "\n");
            }
        }
        return buf.toString();
    }
    
    
    public String toReportString(String instrument,float ipVersion) {
        Instrument instObj = InstrumentList.getInstance().getInstrument(instrument,ipVersion);
        InstrumentConstraintConfig [] instConfList = instObj.getCsConfList();
        HashMap<String, String> map = new HashMap<String, String>();
        for(int i=0;i<instConfList.length;i++) {
              map.put(instConfList[i].getName(),instConfList[i].getName());
        }
        StringBuffer buf = new StringBuffer();
        buf.append("ConSet: " + getName() + "\n");
        if(map.containsKey("Seeing"))
        buf.append("Seeing                        : " + getSeeing() + "\n");
        if(map.containsKey("SkyTransparency"))        
        buf.append(
            "Sky transparency              : " + getSkyTransparency() + "\n");
        if(map.containsKey("Airmass"))                    
        buf.append("Air mass                      : " + getAirmass() + "\n");
        if(map.containsKey("StrehlRatio"))                    
        buf.append(
            "Strehl Ratio                  : " + getStrehlRatio() + "\n");
        if(map.containsKey("FractionalLunarIllumination"))                    
            
        buf.append(
            "Fractional lunar illumination : "
                + getFractionalLunarIllumination()
                + "\n");
        if(map.containsKey("MoonAngularDistance"))                            
        buf.append(
            "Moon angular distance         : "
                + getMoonAngularDistance()
                + "\n");
        String baseline=null;        
        if(map.containsKey("Baseline")) {
            baseline = getBaseline();
            if(baseline == null || baseline.length() <= 0)
                baseline="NONE";
            buf.append(
                "Baseline                      : "
                    + baseline 
                    + "\n");
                            
        }
        
        if (description_ != null && description_.length() > 0) {
            buf.append("User Com. : ");
            String[] descStrings = TextUtils.splitString(description_, 60);
            for (int i = 0; i < descStrings.length; i++) {
                if (i != 0)
                    buf.append("            ");
                buf.append(descStrings[i] + "\n");
            }
        }
        return buf.toString();
    }


    /**
     * Returns a string describing errors in this object's state.
     *
     * @return string describing the errors.
     */
    public String verify() {
        StringBuffer buf = new StringBuffer();
        if (airmass_ < 1 || airmass_ > 5) {
            buf.append("airmass not in range 1 to 5\n");
        }
        if (fractionalLunarIllumination_ < 0
            || fractionalLunarIllumination_ > 1) {
            buf.append("fractional lunar illumination not in range 0 to 1\n");
        }
        if (moonAngularDistance_ < 0 || moonAngularDistance_ > 180) {
            buf.append("moon angular distance not in range 0 to 180\n");
        }
        if (seeing_ < Seeing.MIN_SEEING || seeing_ > Seeing.MAX_SEEING) {
            buf.append("minimum seeing/image quality not in range "+Seeing.MIN_SEEING+" to "+Seeing.MAX_SEEING+"\n");
        } 
//        if (seeingMax_ < Seeing.MIN_SEEING || seeingMax_ > Seeing.MAX_SEEING) {
//            buf.append("maximum seeing/image quality not in range "+Seeing.MIN_SEEING+" to "+Seeing.MAX_SEEING+"\n");
//        }
//        if (seeingMax_ < seeing_ ) {
//            buf.append("seeing/image range wrong min: "+seeing_+" max: "+seeingMax_+"\n");
//        }
        return buf.toString();
    }

    public String verifyBO() {
        return verify();
    }


    /**
     * Generic hook for "visitor" Design Pattern.
     * See the BusinessVisitor interface for more details.
     * 
     * @param visitor	the class whose visit() method we wish to invoke.
     * 
     * @see BusinessVisitor
     */
    public Object accept(BusinessVisitor visitor, Object arg) {
        return visitor.visit(this, arg);
    }

    /**
     * Update this object with some values extracted from the input
     * <code>Paramfile</code>.
     *
     * @exception IllegalArgumentException
     */

    public void updateFromParamFileParameter(ParamFileParameter pfp)
        throws IllegalArgumentException {

        if (!pfp.getValue().equals("NODEFAULT")) {
            Paramfile p = pfp.getParamfile();
            updateFromParamfile(p);
        }
    }

    public void updateFromParamfile(Paramfile paramfile)
        throws IllegalArgumentException {
        /*
         * extract Strehl Ratio PAF keyword
         */
        IllegalArgumentException conversionException = null;
        suspendPropertyChangeEvents();

        String kval; // keyword value
        // config contains a definition of which param to extract

        // Strehl Ratio
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSStrehlParam, paramfile)
                .trim();

        kval = kval.trim();

        if (kval.length() > 0) {
            try {
                float sr = Float.valueOf(kval).floatValue();
                setStrehlRatio(sr);
            } catch (NumberFormatException e) {
                conversionException =
                    new IllegalArgumentException(
                        "Invalid Strehl Ratio value: " + kval);
            }
        }
        
        
        // Air Mass
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSAirMassParam, paramfile)
                .trim();

        if (kval.length() > 0) {
            try {
                float fVal = Float.valueOf(kval).floatValue();
                setAirmass(fVal);
            } catch (NumberFormatException e) {
                conversionException =
                    new IllegalArgumentException(
                        "Invalid Air Mass value: " + kval);
            }
            
        }
        
        // FLI
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSFliParam, paramfile)
                .trim();

        if (kval.length() > 0) {
            try {
                float fVal = Float.valueOf(kval).floatValue();
                setFractionalLunarIllumination(fVal);
            } catch (NumberFormatException e) {
                conversionException =
                    new IllegalArgumentException(
                        "Invalid FLI value: " + kval);
            }
            
        }
        
        // Moon Distance
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSMoonParam, paramfile)
                .trim();

        if (kval.length() > 0) {
            try {
                int iVal = Integer.valueOf(kval).intValue();
                setMoonAngularDistance(iVal);
            } catch (NumberFormatException e) {
                conversionException =
                    new IllegalArgumentException(
                        "Invalid Moon Distance value: " + kval);
            }
            
        }               
                
        // Seeing
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSSeeingParam, paramfile)
                .trim();
        if (kval.length() > 0) {
            try {
                float fVal = Float.valueOf(kval).floatValue();
                setSeeing(fVal);
            } catch (NumberFormatException e) {
                conversionException =
                    new IllegalArgumentException(
                        "Invalid Seeing value: " + kval);
            }
            
        }               
        
        // Sky
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSSkyParam, paramfile)
                .trim();
        if (kval.length() > 0) {
            setSkyTransparency(kval); 
        }               
                  
        // Telescope Temp
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSTelTempParam, paramfile)
                .trim();
        if (kval.length() > 0) {
            try {
                int iVal = Integer.valueOf(kval).intValue();
                setTelTemp(iVal);
            } catch (NumberFormatException e) {
                conversionException =
                    new IllegalArgumentException(
                        "Invalid Telescope Tempurature value: " + kval);
            }
            
        }               
            
        // Water Vapour
        kval =
            OHSConfig
                .getInstance()
                .evalPropertyFromParamfile(OHSKeys.CSVapourParam, paramfile)
                .trim();
        if (kval.length() > 0) {
            setWaterVapour(kval);
        }
        
        resumePropertyChangeEvents();
        firePropertyChangeEvent();

        if (conversionException != null) {
            throw conversionException;
        }
    }
    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }
    /**
     * @return String baseline
     */
    public String getBaseline() {
        return baseLine;
    }

    /**
     * @param string
     */
    public void setBaseline(String string) {
        baseLine = string;
        firePropertyChangeEvent();
    }

    
    /**
     * Compares the two <code>ConstraintSet</code> objets.
     * It takes into account only properties contained in the props array
     * If the list is empty, the method returns false
     * 
     * @param other - the other ConstraintSet object
     * @param props - list of proerty names to be compared.
     * @return boolean
     * @throws ReflectionException
     */
    public boolean equals( ConstraintSet other, String [] props) throws ReflectionException{
        
        boolean res = true;
        Class<? extends ConstraintSet> cls = this.getClass();
        String prop;
        Method get;
          
        for (int i = 0; i < props.length; i++) {
            prop = props[i];
             
     
            try {
                get = cls.getMethod("get" + prop, new Class[] {});
                Object objeThis;                
                objeThis = get.invoke(this, new Object[] {});
                Object objeOther;
                objeOther = get.invoke(other, new Object[] {});
                
                if (!objeThis.equals(objeOther)) {
                    res = false;

                } 
                return(res);
                
            } catch (SecurityException e) {                
                throw new ReflectionException(e.toString());
            } catch (NoSuchMethodException e) {
                throw new ReflectionException(e.toString());
            } catch (IllegalArgumentException e) {
                throw new ReflectionException(e.toString());
            } catch (IllegalAccessException e) {
                throw new ReflectionException(e.toString());
            } catch (InvocationTargetException e) {
                throw new ReflectionException(e.toString());
            }
            
        }
        return(res);
        
    }

    /**
     * @param windAngle The windAngle to set.
     */
    public void setWindAngle(int windAngle) {
        this.windAngle = windAngle;
    }

    /**
     * @return Returns the windAngle.
     */
    public int getWindAngle() {
        return windAngle;
    }
}
