package org.eso.ohs.dfs;

import java.beans.PropertyChangeEvent;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;


/**
 * A Queue class is used for manipulating a collection of ObservationBlock
 * objects 
 *
 * @author Giorgio Passera <ESO \ DMD> <gpassera@eso.org>
 */

public class Queue extends BusinessObject {

    public static final int SECONDS_PER_DAY = 86400;
    public final static String ITEMLIST = "itemlist";
    public final static String EXECUTION_SEQUENCE = "executionSequence";
    public final static String TELESCOPE = "telescope";
    public final static String DURATION = "duration";
    public final static String START_TIME = "startTime";
    public final static String DESCRIPTION = "description";
    public final static String NAME = "name";
    public final static String QI_LIST_SIZE = "QIListSize";

    public static long OFFSET = 99000;

    public static long TIMEOFFSET = 16200;

    private String description; //textual description of this queue
    private long startTime; //Time /date for the start of the queue
    private long duration = SECONDS_PER_DAY; //duration of schedule in seconds
    private String telescope; //telescope used for the schedule
    //private Vector collection;  //a collection of QueueItem objects
    //private Vector executionSequence; //a collection of ObservationBlock

    private QueueItem[] queueItems_ = {
    };

    private boolean update = false; //true the queue must be updated 

    private int size = 0;

    /**
     * Default constructor
     */
    public Queue() {
        super();
        description = "";
        telescope = "UT1";
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(
            calendar.get(Calendar.YEAR),
            calendar.get(Calendar.MONTH),
            calendar.get(Calendar.DATE),
            18,
            0,
            0);
        startTime = calendar.getTime().getTime() / 1000;
        duration = SECONDS_PER_DAY;
    }

    /**
     * OverWrite the setName method
     */
    public void setName(String newName) {
        boolean upd = !getName().equals(newName);
        super.setName(newName);
        if (upd) {
            PropertyChangeEvent ev =
                new PropertyChangeEvent(this, Queue.NAME, null, null);
            update = true;
            firePropertyChangeEvent(ev);
        }
    }

    /**
     * Returns the nuber of OBs that belong to the 
     * QueueItem list
     * @return the number of OBs that belong to the QueueItemList
     */
    public int getSize() {
        return size;
    }

    public void setSize(int newSize) {

        size = newSize;

        PropertyChangeEvent ev =
            new PropertyChangeEvent(this, Queue.QI_LIST_SIZE, null, null);
        firePropertyChangeEvent(ev);

    }

    /**
     * Sets a textual description for a queue
     * @param desc: specifies a textual description
     */
    public void setDescription(String desc) {
        if (!description.equals(desc)) {
            description = desc;
            setUpdate(true);
        }
    }

    /**
     *Gets a textual description for the queue
     *@return a textual description for a queue
     */
    public String getDescription() {
        return description;
    }

    /**
     * Sets time/date for the start of a queue
     * @param time: specifies the time/date for the start of a queue
     */
    public void setStartTime(long time) {
        boolean upd = startTime != time;
        startTime = time;
        if (upd) {
            setUpdate(true);
        }
    }

    /**
    **Sets time/date for the start of a queue
    *@return the time/date for the start a queue
    */
    public long getStartTime() {
        return startTime;
    }

    /**
     * Sets a duration af a queue in seconds
     * If durtion is 0, default to 84600
     * @param value specifies the duration
     */
    public void setDuration(long value) {
        boolean upd = duration != value;
        if (value == 0) {
            duration = SECONDS_PER_DAY;  
        } else {
           duration = value;
        }
        if (upd) {
            setUpdate(true);
        }
    }

    /**
     * Gets a duration of a queue 
     * @return the duration of a queue
     */
    public long getDuration() {
        return duration;
    }

    /**
     *Sets the telescope used
     *@param name of the telescope used 
     */
    public void setTelescope(String name) {
        boolean upd = !telescope.equals(name);
        telescope = name;
        if (upd) {
            setUpdate(true);
        }
    }

    /**
     * Gets name of telescope used 
     * @return the name of telescope
     */
    public String getTelescope() {
        return telescope;
    }

    /**
     * Returns the filename suffix for files containing this type of object.
     *
     * @return suffix for filename when stored in local filesystem
     */
    public static String getSuffix() {
        return "queue";
    }

    /**
     * Generic hook for "visitor" Design Pattern.
     * See the BusinessVisitor interface for more details.
     * 
     * @param visitor	the class whose visit() method we wish to invoke.
     * 
     * @see BusinessVisitor
     */
    public Object accept(BusinessVisitor visitor, Object arg) {
        return visitor.visit(this, arg);
    }

    /**
     *Tests if the Queue must be updated
     *@return true if the Queue must be updated
     */
    public boolean isUpdate() {
        return update;
    }

    /**
     * Sets the value of the update parameter
     */
    public void setUpdate(boolean value) {
        setUpdate(value, null);
    }

    public void setUpdateAndFire(boolean update) {
        this.update = update;
        firePropertyChangeEvent();
    }

    public void setUpdate(boolean value, PropertyChangeEvent ev) {
        boolean fire = (update != value);
        if (fire) {
            update = value;
            if (ev == null) {
                firePropertyChangeEvent();
            } else {
                firePropertyChangeEvent(ev);
            }
        }
    }

    public long[] getAllOBIds() {
        Vector<Long> vec = new Vector<Long>();
        for (int i = 0; i < queueItems_.length; i++) {
            vec.addElement(new Long(queueItems_[i].getObId()));
        }
        long[] returnIds = new long[vec.size()];
        for (int j = 0; j < vec.size(); j++) {
            returnIds[j] = vec.elementAt(j).longValue();
        }
        return returnIds;
    }

    public long[] getAllUniqueOBIds() {
        Vector<Long> vec = new Vector<Long>();
        for (int i = 0; i < queueItems_.length; i++) {
            if (queueItems_[i].getType().equals("O"))
                vec.addElement(
                    new Long(queueItems_[i].getObId()));
            else
                vec.addElement(
                    new Long(queueItems_[i].getObId()));
                        
        }
        long[] returnIds = new long[vec.size()];
        for (int j = 0; j < vec.size(); j++) {
            returnIds[j] = vec.elementAt(j).longValue();
        }
        return returnIds;
    }

    public void sortQueueItems(long[] srcOBIds) {
        Vector<QueueItem> dataVector = new Vector<QueueItem>();
        int destIndex = 0;
        for (int i = 0; i < queueItems_.length; i++) {
            boolean found = false;
            for (int j = 0; j < srcOBIds.length; j++) {
                if (queueItems_[i].getObId()
                    == srcOBIds[j]) {
                    found = true;
                    if (i <= destIndex)
                        destIndex = i;
                }
            }
            if (!found)
                dataVector.addElement(queueItems_[i]);
        }
        for (int j = srcOBIds.length - 1; j >= 0; j--) {
            QueueItem qItem =
                getQueueItemFromObId(
                    srcOBIds[j]);
            dataVector.insertElementAt(qItem, destIndex);
        }
        QueueItem[] queueItems = new QueueItem[dataVector.size()];
        dataVector.copyInto(queueItems);
        setQueueList(queueItems);
    }

    public void moveUpQueueItems(long[] srcOBIds, long destOBId) {
        Vector<QueueItem> dataVector = new Vector<QueueItem>();
        int destIndex = -1;
        long destId = destOBId;
        for (int i = 0; i < queueItems_.length; i++) {
            boolean found = false;
            for (int j = 0; j < srcOBIds.length; j++) {
                if (queueItems_[i].getObId()
                    == srcOBIds[j]) {
                    found = true;
                }
            }
            if (!found)
                dataVector.addElement(queueItems_[i]);
            if (destId == queueItems_[i].getObId())
                destIndex = i;
        }
        for (int j = srcOBIds.length - 1; j >= 0; j--) {
            QueueItem qItem =
                getQueueItemFromObId(
                    srcOBIds[j]);
            dataVector.insertElementAt(qItem, destIndex);
        }
        QueueItem[] queueItems = new QueueItem[dataVector.size()];
        dataVector.copyInto(queueItems);
        setQueueList(queueItems);
    }

    public void moveDownQueueItems(long[] srcOBIds, long destOBId) {
        Vector<QueueItem> dataVector = new Vector<QueueItem>();
        int destIndex = -1;
        long destId = destOBId;
        for (int i = 0; i < queueItems_.length; i++) {
            boolean found = false;
            for (int j = 0; j < srcOBIds.length; j++) {
                if (queueItems_[i].getObId()
                    == srcOBIds[j]) {
                    found = true;
                }
            }
            if (!found)
                dataVector.addElement(queueItems_[i]);
            if (destId == queueItems_[i].getObId())
                destIndex = i;
        }
        for (int j = 0; j < srcOBIds.length; j++) {
            QueueItem qItem =
                getQueueItemFromObId(
                    srcOBIds[j]);
            dataVector.insertElementAt(
                qItem,
                destIndex - srcOBIds.length + j + 1);
        }
        QueueItem[] queueItems = new QueueItem[dataVector.size()];
        dataVector.copyInto(queueItems);
        setQueueList(queueItems);
    }

    public QueueItem getQueueItemFromObId(long obId) {
        for (int i = 0; i < queueItems_.length; i++) {
            if (obId == queueItems_[i].getObId())
                return queueItems_[i];
        }
        return null;
    }

    public void setQueueList(QueueItem[] queueItems) {
        queueItems_ = queueItems;
        update = true;
        boolean upd = true;
        if (queueItems_ != null) {
            setSize(queueItems_.length);
        } else {
            setSize(0);
        }
        if (upd) {
            PropertyChangeEvent ev =
                new PropertyChangeEvent(this, Queue.ITEMLIST, null, null);
            firePropertyChangeEvent(ev);
        }

    }

    public QueueItem[] getQueueList() {
        return queueItems_;
    }

    public void setQueueSequenceList(QueueItem[] queueItems) {
        queueItems_ = queueItems;

    }

    public boolean inQueueItems(long obId) {
        for (int i = 0; i < queueItems_.length; i++) {
            if (queueItems_[i].getObId()
                == obId)
                return true;
        }
        return false;
    }

    public void removeQueueItems(long[] obIds) {
        Vector<QueueItem> queueList = new Vector<QueueItem>();
        for (int i = 0; i < queueItems_.length; i++) {
            boolean found = false;
            for (int j = 0; j < obIds.length; j++) {
                if (queueItems_[i].getObId()
                    == obIds[j]) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                queueList.addElement(queueItems_[i]);
            }
        }
        QueueItem[] queueItems = new QueueItem[queueList.size()];
        queueList.copyInto(queueItems);
        setQueueList(queueItems);
    }

    public QueueItem[] appendQueueItems(long[] obIds, QueueItem[] queueItems,String [] types) {
        Vector<QueueItem> queueList = new Vector<QueueItem>();
        for (int i = 0; i < queueItems.length; i++) {
            queueList.addElement(queueItems[i]);
        }
        int seq = 0;
        for (int j = 0; j < obIds.length; j++) {
            boolean found = false;
            for (int k = 0; k < queueItems.length; k++) {
                if (queueItems[k].getObId()
                    == obIds[j]) {
                    found = true;
                    seq = queueItems.length + seq;
                    seq++;
                    break;
                }
            }
            if (!found) {
                QueueItem queueItem =
                    new QueueItem(
                        obIds[j],
                        getId(),
                        seq,
                        types[j]);
                queueList.addElement(queueItem);
            }
        }
        QueueItem[] retQueueItems = new QueueItem[queueList.size()];
        queueList.copyInto(retQueueItems);
        return retQueueItems;
    }

    public void appendQueueItems(long[] obIds,String [] types) {
        setQueueList(appendQueueItems(obIds, queueItems_,types));
    }

    public QueueItem[] getQueueSequenceList() {
        return queueItems_;
    }

    public QueueItem[] getExecList() {
        return queueItems_;
    }

    public void fireNameEvent() {
        PropertyChangeEvent ev =
            new PropertyChangeEvent(this, Queue.NAME, null, null);

        firePropertyChangeEvent(ev);
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < queueItems_.length; i++) {
            sb.append(queueItems_[i].toString());
            sb.append("\n");
        }
        return sb.toString();
    }
    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }

}
