package org.eso.ohs.instruments;

import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ExtFileList;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.exolab.castor.xml.schema.ComplexType;
import org.exolab.castor.xml.schema.ElementDecl;
import org.exolab.castor.xml.schema.Facet;
import org.exolab.castor.xml.schema.Group;
import org.exolab.castor.xml.schema.Schema;
import org.exolab.castor.xml.schema.SimpleType;
import org.exolab.castor.xml.schema.XMLType;
import org.exolab.castor.xml.schema.reader.SchemaReader;
import org.xml.sax.InputSource;

/**
 * The instrument class stores basic instrument details like
 * instrument name and telescope name.
 * It also stores a list of template signature summary information.
 *
 * $Id: Instrument.java,v 1.57 2006/04/27 10:37:43 tcanavan Exp $
 * @version $Revision: 1.57 $
 * @author Tim Canavan
 */

public class Instrument {
    
    
    

    private final static String rcsid =
        "$Id: Instrument.java,v 1.57 2006/04/27 10:37:43 tcanavan Exp $";

    private static final String[] mandatoryInstrumentValues =
        { "PAF.TYPE", "PAF.NAME" };

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(Instrument.class);

    /** List of valid instrument types */
    public static final String[] instrumentTypes =
        { "science", "calib", "test" };

    /** PAF file which contains the data to initialise this class */
    protected PafFile pafFile_;
    
    

    /**
     * The numbet of recoords in the index file that store the summary
     * information relating to the template signature
     */
    private int numRecords;

    /** Instrument file name that contains the date to initialize the class */
    private String instrumentFile_;

    /**
     * Date and time of the .isf file
     */
    private long lastModified_;

    /** Instrument name */
    private String instrumentName_;

    /** Version number for the telescope 1.0.  */
    private String version_;

    /**
     * The version is a floating point number, the form it is typically used.
     * A value of -1 indicates an invalid version.
     */
    private float versionNumber_ = -1;

    /** Telescope name associated with the instrument.  */
    private String telescopeName_;

    /** The directory that contains the .tsf files.  */
    private String tsfDirectory_;

    /** This is the list of modes that an instrument can have  (RILD, etc)  */
    private String[] modes_ = null;

    /** List of all the templateSummaryNames */
    private String[] templateNames_ = null;

    /** List of all templateSummary objects */
    private TemplateSignatureSummary[] templateSummaryList_ = {
    };

    /** List of identifiers and data for the instrument */
    private PafData[] instrumentData_;

    /** File type shuold be Instrument Summary */
    private String fileType_;

    /**
     * List of instrument values which get retrieved when
     * building a template signature. See also Template Signature.
     */
    private Hashtable<String, PafData> instrumentValues_;

    /** Paf Values that are missing from .isf file */
    private String[] pafLinesMissing_;

    /** Boolean to check whether the instrument file is out of date */
    private boolean indexOutOfDate_ = false;
    
    private String labelInstComment;
    
    private boolean instComments = false;
    
    

    /** List of template names that represent .tsf file which had critical errors */
    private String[] tmplErrList_ = {
    };

    /** Vector of mandatory keywords missing */
    private Vector<String> mandatoryKeywordsMissing_ = new Vector<String>();

    /** Vector of mandatory values missing */
    private Vector<String> mandatoryValuesMissing_ = new Vector<String>();

    /**
     * Comiler Error message for display
     * This happens if a .isf or .tsf file has a misrepresented quote
     */
    private String compileErrorMsg_;
    
    /**
     * Array of attributes that define the meta data information
     * for  a constraint set.
     */

    private InstrumentConstraintConfig[] csConfList;
    
    /**
     * Array of attributes that define the reame attributes for 
     * a constraint set. 
     */
    
    private InstrumentConstraintConfig[] readmeConfList;



    ////////////////////////////////////////////////////////////////
    // Constructor(s)                                              /
    ////////////////////////////////////////////////////////////////

    /**
     * This constructor call the PafFile base class to initially parse the Instrument file into Identifier and
     * and data pairs. Then it constructs a list of TemplateSignatureSummary classes associated with the
     * instrument
     *
     * @param instrumentFileName location of instrument file
     * @param tsfDirectory       location of tsf files
     * @param indexFilename      name of index file
     * @exception IOException IO error occurs
     * @exception ClassNotFoundException Class not in index files
     */
    public Instrument(String tsfDirectory)
        throws IOException, IllegalArgumentException {
        this(
            tsfDirectory,
            tsfDirectory + File.separator + InstrumentList.defaultISF);
    }

    public Instrument(String tsfDirectory, String instrumentFile)
        throws IOException, IllegalArgumentException {
        instrumentFile_ = instrumentFile;
        tsfDirectory_ = tsfDirectory;
        init();
    }

    /**
     * Initialises the member variables of the class from the identifier list
     * of name value pairs.
     *
     * @exception IOException Error reading or writing index file
     * @exception IllegalArgumentException Illegal argument in mkdir call
     */
    private void init() throws IOException, IllegalArgumentException {
        lastModified_ = (new File(instrumentFile_)).lastModified();
        pafFile_ = new PafFile(instrumentFile_);
        instrumentValues_ = new Hashtable<String, PafData>();
        compileErrorMsg_ = pafFile_.getCompileErrorMsg();
        if (compileErrorMsg_ != null && compileErrorMsg_.length() > 0)
            return;
        pafLinesMissing_ = pafFile_.getPafLinesMissing();
        if (pafLinesMissing_ != null && pafLinesMissing_.length > 0)
            return;
        instrumentName_ = pafFile_.getPafName();
        fileType_ = pafFile_.getPafType();
        setValuesFromIdentifierList();
        File dir = new File(tsfDirectory_);
        if (dir == null || !dir.isDirectory())
            throw new IllegalArgumentException(
                "No Such Directory " + tsfDirectory_);
        if (!dir.canRead()) {
            throw new IOException(
                "No read access righs to directory " + tsfDirectory_);
        }
        initConstraintMetaData();
        initReadmeMetaData();
        initCommentsMetaFile();
    }
    
    public boolean hasReadmeData() {
        if(readmeConfList != null)
            return true;
        return false;
    }
    
    
    /**
     * Initialise the readme meta information
     * @throws IOException
     */
    
    private void initReadmeMetaData() throws IOException {
        File f = initReadmeMetaFile();
        if(!f.exists()) return;
        readmeConfList = initMetaData(f,"Readme");     
    }
    

    /**
     * Initialise the constraint information.
     * @throws IOException
     */

    private void initConstraintMetaData() throws IOException {
        File f = initConstraintMetaFile();
        csConfList = initMetaData(f,"Constraints");     
    }
    
    /**
     * Initialise attributes 
     * @param f
     * @param rootElementName
     * @param instConfList Array of elements.  
     * @throws IOException
     */

    private InstrumentConstraintConfig[] initMetaData(File f,String rootElementName) throws IOException {
        //leInputStream fStream = new FileInputStream(new File(path + "defaultconstraints.xsd"));
        InstrumentConstraintConfig[] confList;
        SchemaReader reader =
            new SchemaReader(new InputSource(f.getAbsolutePath()));

        Schema s = reader.read();
        ElementDecl rootElement = s.getElementDecl(rootElementName);
        Enumeration simpleTypes = s.getSimpleTypes();
        HashMap<String, SimpleType> map = new HashMap<String, SimpleType>();
        while (simpleTypes.hasMoreElements()) {
            SimpleType sType = (SimpleType) simpleTypes.nextElement();
            map.put(sType.getName(), sType);
        }
        Vector<InstrumentConstraintConfig> vec = new Vector<InstrumentConstraintConfig>();
        String name = rootElement.getName();
        XMLType type = rootElement.getType();
        if (type instanceof ComplexType) {
            ComplexType ct = (ComplexType) type;
            Enumeration en = ct.enumerate();
            while (en.hasMoreElements()) {
                Group grp = (Group) en.nextElement();
                Enumeration erm = grp.enumerate();
                while (erm.hasMoreElements()) {
                    InstrumentConstraintConfig insConfig =
                        new InstrumentConstraintConfig();
                    ElementDecl decl = (ElementDecl) erm.nextElement();
                    XMLType type1 = decl.getType();
                    if (type1 instanceof ComplexType) {
                        ComplexType ctProp = (ComplexType) type1;
                        insConfig.setName(decl.getName());
                        ElementDecl elemProp =
                            ctProp.getElementDecl("TypeInfo");
                        XMLType propType = (XMLType)elemProp.getType();
                        if(propType.getBaseType()!=null) {
                            String baseName = propType.getBaseType().getName();
                            insConfig.getMetaData().setBaseType(baseName);                            
                        }
                        insConfig.getMetaData().setDefaultValue(elemProp.getDefaultValue());
                        String typeName = propType.getName();
                        SimpleType simpleType = map.get(typeName);
                        Enumeration enum0 = simpleType.getFacets();
                        Vector<String> allowedValues = new Vector<String>();
                        while (enum0.hasMoreElements()) {
                            Facet facet = (Facet) enum0.nextElement();
                            setMetaInfo(insConfig,facet,allowedValues);                                                        
                        }
                        String[] setValues =
                            new String[allowedValues.size()];
                        allowedValues.copyInto(setValues);
                        insConfig.getMetaData().setAllowedValues(setValues);

                        setInstConfig(insConfig, ctProp);
                        vec.addElement(insConfig);
                    }
                }

            }
        }
        confList = new InstrumentConstraintConfig[vec.size()];
        vec.copyInto(confList);
        return confList;
    }
    
    
    /**
     * Initialise the constraint file meta information
     * @throws FileNotFoundException
     */
    private void initCommentsMetaFile() throws FileNotFoundException, IOException {
        File f = new File(tsfDirectory_ + File.separator + "comments" + File.separator + "comments.cf");
        if(!f.exists())
            return;
        
        PafFile pfile = new PafFile(f.getAbsolutePath());
        labelInstComment = pfile.getValueForKeyword("INST.LABEL");
        if(labelInstComment != null && labelInstComment.length() > 0)
            instComments = true;
            
    }

    
    
    
    /**
     * Initialise the constraint file meta information
     * @return XSD file handle for constraints.
     * @throws FileNotFoundException
     */
    private File initConstraintMetaFile() throws FileNotFoundException {
        File f = new File(tsfDirectory_ + File.separator + "constraints" + File.separator + "constraints.xsd");
        if (!f.exists())
            f = new File("config" + File.separator + "constraints.xsd");
        if (!f.exists())
            throw new FileNotFoundException("Constraint file unavailable");
        return f;
    }
    
    /**
     * Initialise the readme file meta information
     * @return Readme xsd file
     * @throws FileNotFoundException
     */
    
    private File initReadmeMetaFile() throws FileNotFoundException {
        File f = new File(tsfDirectory_ + File.separator + "readme" + File.separator + "readme.xsd");
        return f;
    }




    /**
     * Set the various elements other than the datavalidation elements ib 
     * @param insConfig
     * @param ctProp
     */
    private void setInstConfig(InstrumentConstraintConfig insConfig, ComplexType ctProp) {
        ElementDecl decl;
        decl = ctProp.getElementDecl("Labels");                                               
        insConfig.setLabel(decl.getFixedValue());
        
        decl = ctProp.getElementDecl("ToolTip");
        if(decl != null)                                               
            insConfig.setToolTip(decl.getFixedValue());
        decl = ctProp.getElementDecl("TypeInfo");
        if(decl != null) {
             insConfig.setSuperType(decl.getType().getName());                                                                                                                                                                                                                                       
        }        
        decl = ctProp.getElementDecl("DBCOLUMN");
        if(decl != null)                                
            insConfig.setDbColumn(decl.getFixedValue());
        decl = ctProp.getElementDecl("HideTelProp");
        if(decl != null) {                               
            String boolStr = decl.getFixedValue();
            if(boolStr.equals("true")) {
                insConfig.setHideTelProperty(true);                            
            }
        }    
        decl = ctProp.getElementDecl("FKCOLUMN");
        if(decl != null)                        
            insConfig.setFkColumn(decl.getFixedValue());                        
        decl = ctProp.getElementDecl("FKTABLE");
        if(decl!=null)
            insConfig.setFkTable(decl.getFixedValue());                        
        decl = ctProp.getElementDecl("PAFKEYWORD");
        if (decl != null)
            insConfig.setPafKeyword(decl.getFixedValue());
        decl = ctProp.getElementDecl("compulsory");
        if (decl != null) {
             String boolStr = decl.getFixedValue();
             if(boolStr.equals("true")) {
                 insConfig.setCompulsory(true);                              
             }
        }    
    
            
        decl = ctProp.getElementDecl("isProperty");
        if (decl != null) {
             String boolStr = decl.getFixedValue();
             if(boolStr.equals("true")) {
                 insConfig.setProperty(true);                              
             }
        }    
    }
    
    
    

    /**
     * Set the meta information for a given attribute
     * @param insConfig
     * @param allowedValues
     * @param facet
     */
    private void setMetaInfo(InstrumentConstraintConfig insConfig,Facet facet,Vector<String> allowedValues) {
        if (facet.getName().equals("minInclusive"))
             insConfig.getMetaData().setMinInclusive(
                new Double(facet.getValue()).doubleValue());
        if (facet.getName().equals("maxInclusive"))
            insConfig.getMetaData().setMaxInclusive(
                new Double(facet.getValue()).doubleValue());
        if (facet.getName().equals("maxLength")) 
            insConfig.getMetaData().setMaxLength(
                new Double(facet.getValue()).intValue());
        if (facet.getName().equals("minLength")) 
            insConfig.getMetaData().setMaxLength(
                new Double(facet.getValue()).intValue());                                
        if (facet.getName().equals("pattern")) {
            insConfig.getMetaData().setPattern(
                facet.getValue());
        }
        if (facet.getName().equals("enumeration")) {
            allowedValues.addElement(facet.getValue());
        }
    }

    /**
     * @return true if this instrument requires refreshing (the ISF has been
     * updated, else false.
     */
    public boolean refreshRequired() {
        long lastMod = (new File(instrumentFile_)).lastModified();
        //stdlog_.debug("Instrument refresh: "+lastModified_+" "+lastMod);
        return lastMod != lastModified_;
    }

    /**
     * @return true if the refresh succeeded, else false. If false is returned,
     * the instrument will be removed from the InstrumentList as it is no longer
     * valid.
     */
    public boolean refreshInstrument() {
        String instrument = getInstrumentName();
        float version = getVersionNumber();
        boolean rc = true;
        try {
            init();
            rc = true;
        } catch (IOException e1) {
            rc = false;
        } catch (IllegalArgumentException e2) {
            rc = false;
        }
        if (!rc) {
            InstrumentList.getInstance().removeInstrument(instrument, version);
        }
        return rc;
    }

    /**
     * Checks to see if the index file is out of data
     * @return True if index file is out of date with .tsf files
     */
    public boolean isIndexOutOfDate() {
        return indexOutOfDate_;
    }

    /**
     * If any invalid templates are in the directory associated with
     * a given instrument return true else false
     * @return boolean value true if invalid templates else false
     */
    public boolean hasInvalidTemplates() {

        for (int i = 0; i < tmplErrList_.length; i++) {
            stdlog_.debug("MIPS error: " + i + " " + tmplErrList_[i]);
        }

        if (tmplErrList_.length <= 0 && templateSummaryList_.length > 0)
            return false;
        return true;
    }

    /**
     * Check to see whether there is an execTime calculator in
     * script format in the execTime driectory off instruments.
     * @return boolean
     */

    public boolean hasExecTimeCalc() {
        File execFile = new File(tsfDirectory_ + File.separator + "exectime");
        if (execFile.exists() && execFile.isDirectory()) {
            stdlog_.debug("Directory  " + execFile);
            ExtFileList filter =
                new ExtFileList(execFile.getAbsolutePath(), ".tcl");
            String[] fileList = filter.getFileList();
            if (fileList.length > 0) {
                stdlog_.debug("Filelist > 0");
                return true;
            }
        }

        return false;
    }

    /**
     * Initialise the index file
     */
    public void initIndexFile() throws IOException {
        File dir = new File(tsfDirectory_);
        String fileName = tsfDirectory_ + File.separator + "summary.idx";
        boolean reindex = false;
        File f = new File(fileName);
        if (f.exists()) {
            if (f.lastModified() < dir.lastModified()) {
                reindex = true;
            } else {
                ExtFileList filter = new ExtFileList(tsfDirectory_, ".tsf");
                File[] fileHandles = filter.getFileHandles();
                for (int i = 0; i < fileHandles.length; i++) {
                    if (fileHandles[i].lastModified() > f.lastModified())
                        reindex = true;
                }
            }
        } else {
            reindex = true;
        }

        if (reindex) {
            if (dir.canWrite()) {
                writeIndexFile(fileName);
            } else {
                indexOutOfDate_ = true;
            }
        }
        readIndexFile(tsfDirectory_ + File.separator + "summary.idx");
    }

    /**
     * Set the values for mandatory keywords missing array
     */
    public void setMandatoryKeywordsMissing() {
        Vector idList = pafFile_.getIdVector();
        if (!idList.contains("TELESCOPE")) {
            mandatoryKeywordsMissing_.addElement("TELESCOPE");
        }
        if (!idList.contains("VERSION")) {
            mandatoryKeywordsMissing_.addElement("VERSION");
        }
    }

    /**
     * Check if the correct file tppe is use in .isf file
     *
     * @return false if fileType valuie is set oncorrectly
     */
    public boolean isValidFileType() {
        if (fileType_ != null && fileType_.length() >= 0) {
            // you are probably wondering why Instrument Summmary is
            // included here this is because the orignal files had this
            // mistake in it and in order to be backword compatiable we
            // had to include it
            if (fileType_.equals("Instrument Summary")
                || fileType_.equals("Instrument Summmary"))
                return true;
        }
        return false;
    }

    /**
     * Set the mandatory values missing array
     */
    public void setManadatoryValues() {
        Vector idList = pafFile_.getIdVector();
        for (int i = 0; i < Instrument.mandatoryInstrumentValues.length; i++) {
            if (!idList.contains(Instrument.mandatoryInstrumentValues[i]))
                mandatoryValuesMissing_.addElement(
                    mandatoryInstrumentValues[i]);
        }
    }

    /**
     * This is a check to see if the instrument file has critical
     * errors in it and can be used
     * @return boolean true critical error else false
     */
    public boolean getCriticalStatus() {
        if (compileErrorMsg_ != null && compileErrorMsg_.length() > 0)
            return true;
        if (pafLinesMissing_ != null && pafLinesMissing_.length > 0)
            return true;
        if (mandatoryValuesMissing_.size() > 0
            || mandatoryKeywordsMissing_.size() > 0
            || !isValidFileType()
            || pafLinesMissing_.length > 0)
            return true;
        return false;
    }

    /**
     * Returns true if mandatory values are missing
     * @return True if mandatory values are missing else false
     */
    public boolean mandatoryValuesMissing() {
        if (mandatoryValuesMissing_.size() > 0)
            return false;
        return true;
    }

    /**
     * Returns true if mandatoryKeywordsMissing
     * @return true if mandatory keywords missing else false
     */
    public boolean mandatoryKeywordsMissing() {
        if (mandatoryKeywordsMissing_.size() > 0)
            return false;
        return true;
    }

    /**
     * Checks to see if an instrument name is value
     * @return True if Invalid else false
     */
    public boolean isValidInstrumentName() {
        if (!instrumentName_.equals("Instrument Summary"))
            return false;
        return true;
    }

    /**
     * returns the pafFile
     */
    public PafFile getPafFile() {
        return pafFile_;
    }

    /**
     * An array of strings that represent the lines that are missing in the
     * instrument file
     * @return An array of PAF strings that are mandatory in the instrument file
     */
    public String[] getPafLinesMissing() {
        return pafFile_.getPafLinesMissing();
    }

    public boolean hasMasks() {
        return false;
    }

    /**
     * Returns the data associated with a component
     * This is a string enclosed in quotes in the
     * instrument file.
     *
     * @param componentName Identifier of a component for the instrument
     * @return  data associated with an Identifier
     */
    public String queryComponent(String componentName) {
        PafData[] identifierList = pafFile_.getIdentifierList();
        for (int i = 0; i < instrumentData_.length; i++) {
            String s = instrumentData_[i].getIdentifier();
            if (s.equals(componentName))
                return instrumentData_[i].getValue();
        }
        return null;
    }

    /**
       * This method gets called from the template signature class
       * so the parameter gets the appropiate range value for
       * validation purposes
       *
       * @param rangeFunction Identifier of a component for the instrument
       * @return data range for parameter
       */
    public String getInstrumentRange(String rangeFunction, String mode) {
        boolean found = false;
        String id = null;
        String rangeFuncMode = null;
        int size = instrumentData_.length;
        // stdlog_.debug("Range Function " + rangeFunction);
        if (mode != null && mode.length() > 0) {
            rangeFuncMode = rangeFunction + " " + mode;
        }
        for (int i = 0; i < size; i++) {
            String value = instrumentData_[i].getValue();
            id = instrumentData_[i].getIdentifier();
            //    stdlog_.debug("ID " + id + "Range Function " + rangeFunction);
            if (id.equals(rangeFunction)) {
                found = true;
                break;
            }
            if (value != null && value.length() > 0) {
                if (value.equals(rangeFunction)) {
                    found = true;
                } else if (
                    rangeFuncMode != null && value.equals(rangeFuncMode)) {
                    found = true;
                }
                if (id.endsWith("ALIAS") == true) {
                    int lastIndex = id.lastIndexOf('.');
                    id = id.substring(0, lastIndex);
                }
                if (found) {
                    break;
                }
            }
        }
        if (found == true) {
            PafData paf = instrumentValues_.get(id);
            if (paf != null) {
                return paf.getValue();
            }
        }
        return null;
    }

    /**
     * Returns a string array of the appropiate modes
     *
     * @return Array of Modes supported by the instrument
     */
    public String[] getModes() {
        return modes_;
    }

    /**
     * Return the instrument name stored in PAF.NAME
     *
     * @return Name of Imstrument
     */
    public String getInstrumentName() {
        return instrumentName_;
    }

    /**
     * @return the full path to the directory containing the TSF files.
     * This is the key means to determine the location of TSF files for
     * an Instrument as it binds the ISF file with the TSF files.
     */
    public String getTSFDirectory() {
        return tsfDirectory_;
    }

    /**
     * Return the CCD Windows paramaters
     * @param mode Mode of operation of instrument
     * @return coordinates of CCD window
     */
    public String getCCDWindow(String mode) {
        String param = null;
        boolean modeSupported = false;
        if (instrumentData_ == null)
            stdlog_.debug("No instrument data");
        for (int i = 0; i < instrumentData_.length; i++) {
            if (instrumentData_[i].getIdentifier().endsWith("CCD.WINDOW")) {
                param = instrumentData_[i].getValue();
            }
            String data = instrumentData_[i].getValue();
            if (data != null) {
                if (data.startsWith("getCCDWINDOW") && data.endsWith(mode))
                    modeSupported = true;
            } else {
                stdlog_.debug(
                    "Error no data for id "
                        + instrumentData_[i].getIdentifier());
            }
        }
        return param;
    }

    /**
     * Returns the telescope name for an instrument which is stored
     * in the TELESCOPE identifier in the identifier list
     *
     * @return Name of telescope
     */
    public String getTelescopeName() {
        return telescopeName_;
    }

    /**
     * Template summary list is summary information from
     * the .tsf files which is stored and read from an index file
     * This function rerurns this as a list
     *
     * @return List of summary information from index file
     */
    public TemplateSignatureSummary[] getTemplateSummaryList() {
        return templateSummaryList_;
    }

    /**
       * Template summary list is summary information from
       * the .tsf files which is stored and read from an index file
       * This function rerurns this as a list
       *
       * @return List of summary information from index file
       */
    public void setTemplateSummaryList(TemplateSignatureSummary[] templateSummaryList) {
        stdlog_.debug(
            "MIPS ("
                + getInstrumentName()
                + ") setTemplateSummaryList: "
                + templateSummaryList.length);
        templateSummaryList_ = templateSummaryList;
    }

    /**
     * For a given template name in the summary list return the associated
     * description.
     *
     * @param templateName Name of template RILDOT01.tsf
     * @return Description to template from TMPL.DESC keyword in .tsf file
     */
    public String getTemplateDesc(String templateName) {
        for (int i = 0; i < templateSummaryList_.length; i++) {
            if (templateName
                .equals(templateSummaryList_[i].getTemplateName())) {
                return templateSummaryList_[i].getTemplateDesc();
            }
        }
        return null;
    }

    /**
     * Template summary list is summary information from
     * the .tsf files which is stored and read from an index file
     * This function rerurns this as a list
     *
     * @param type typw of template signature calib,science
     * @return List of summary information from index file
     */
    public TemplateSignatureSummary[] getTemplateSummaryList(String type) {
        if (type.length() == 0)
            return templateSummaryList_;
        TemplateSignatureSummary templateSignatureSummary = null;
        TemplateSignatureSummary[] templateList =
            new TemplateSignatureSummary[numRecords];
        int j = 0;
        for (int i = 0; i < templateSummaryList_.length; i++) {
            templateSignatureSummary = templateSummaryList_[i];
            if (templateSignatureSummary == null)
                stdlog_.debug("Null value found");
            if (type.equals(templateSignatureSummary.getTemplateType())) {
                templateList[j] = templateSignatureSummary;
                j++;
            }
        }
        TemplateSignatureSummary[] templatesFound =
            new TemplateSignatureSummary[j];
        for (int i = 0; i < j; i++) {
            templatesFound[i] = templateList[i];
        }
        return templatesFound;
    }

    /**
     * Returns all the template names for a particular type.
     *
     * @param type TYPE refers to TPL.TYPE e.g. science,calibrated etc..
     * @return  List of all the summary information from the index file
     */
    public String[] getTemplateNames(String type) {
        int j = 0;
        TemplateSignatureSummary[] templateSummaryList =
            getTemplateSummaryList(type);
        if (templateSummaryList != null) {
            String[] templateNameList = new String[templateSummaryList.length];
            for (int i = 0; i < templateSummaryList.length; i++) {
                templateNameList[i] = templateSummaryList[i].getTemplateName();
            }
            return templateNameList;
        }
        return null;
    }

    /**
     * Returns all the  template names.
     *
     * @return  List of all the template names from the index file
     */
    public String[] getTemplateNames() {
        return templateNames_;
    }


    /**
     * The last modified date for the Templaate Signature
     * @return The last modified date of the .tsf file
     */

    public long getLastModified() {
        return lastModified_;
    }


    /**
     * Set the name of the telescope associated with the
     * instrument  This data is stored
     * in the instrument file with the IDENTIFIER TELESCOPE
     *
     * @param telescope Name of telescope NTT VLT
     */
    private void setTelescopeName(String telescope) {
        telescopeName_ = telescope;
    }

    /**
     * Set the name of the instrument. This data is stored
     * in the  the IDENTIFIER PAF.NAME
     * @param istrumentName Name of Instrrument e.g EMMI
     */
    private void setInstrumentName(String instrumentName) {
        instrumentName_ = instrumentName;
    }

    /**
     * Set the modes that the instrument can use  This data is stored
     * in the  the IDENTIFIER MODES in the .tsf file
     * @param mode String containing the modes for an instrument
     */
    private void setModes(String mode) {
        Vector<String> modes = new Vector<String>();
        StringTokenizer tokenString = new StringTokenizer(mode);
        while (tokenString.hasMoreElements()) {
            String modeValue = new String(tokenString.nextToken());
            modes.addElement(modeValue);
        }
        modes_ = new String[modes.size()];
        modes.copyInto(modes_);
    }

    /**
     * @return Version string of the instrument; i.e. the value of the
     * <code>VERSION</code> keyword in the Instrument Summary File.
     */
    public String getVersion() {
        return version_;
    }

    /**
     * @return the version as a float - or -1 if the version was not valid.
     */
    public float getVersionNumber() {
        return versionNumber_;
    }

    /**
     * Sets version string of the instrument; i.e. the value of the
     * <code>VERSION</code> keyword in the Instrument Summary File.
     * @param version Version of instrument
     */
    private void setVersion(String version) {
        version_ = version;
        if (version_ != null) {
            try {
                Float f = new Float(version);
                versionNumber_ = f.floatValue();
            } catch (NumberFormatException e) {
                versionNumber_ = -1;
            }
        }
    }

    /**
     * @return true if this instrument has a valid version value. That is,
     * a version value that is a non-negative number.
     */
    public boolean hasValidVersion() {
        return versionNumber_ >= 0.0;
    }

    /**
     *  Set the data in the instrument class that can be obtained directly
     *  from the identifier list in the base PafFile class
     */
    protected void setValuesFromIdentifierList() {
        PafData[] identifierList = pafFile_.getIdentifierList();

        Vector<PafData> instrumentData = new Vector<PafData>();
        for (int i = 0; i < identifierList.length; i++) {
            String id = identifierList[i].getIdentifier();
            String value = identifierList[i].getValue();
            if (id.equals("TELESCOPE"))
                setTelescopeName(identifierList[i].getValue());
            else if (id.equals("VERSION"))
                setVersion(identifierList[i].getValue());
            else if (!id.startsWith("PAF")) {
                if (id.equals("MODES"))
                    setModes(identifierList[i].getValue());
                instrumentData.addElement(identifierList[i]);
                if (!id.endsWith("ALIAS"))
                    instrumentValues_.put(id, identifierList[i]);
            }
        }
        instrumentData_ = new PafData[instrumentData.size()];
        instrumentData.copyInto(instrumentData_);
    }

    /**
     * Reads the index file to create the initial list of Template Summaries
     *
     * @param fileName Summary information file
     * @exception IOException IO error occurs
     * @exception ClassNotFoundException Class not in index files
     */
    private void readIndexFile(String fileName) throws IOException {
        try {
            FileInputStream istream = new FileInputStream(fileName);
            ObjectInputStream instream = new ObjectInputStream(istream);
            numRecords = instream.readInt();
            // stdlog_.debug("MIPS ("+getInstrumentName()+") readIndexFile: "+numRecords);
            if (numRecords == 0)
                return;
            templateSummaryList_ = new TemplateSignatureSummary[numRecords];
            templateNames_ = new String[numRecords];

            for (int i = 0; i < numRecords; i++) {
                TemplateSignatureSummary templateSignatureSummary =
                    (TemplateSignatureSummary) instream.readObject();
                templateSummaryList_[i] = templateSignatureSummary;
                templateNames_[i] = templateSignatureSummary.getTemplateName();
            }
        } catch (EOFException e2) {
            stdlog_.debug("Num Records = " + numRecords);
        } catch (IOException e) {
            stdlog_.error(
                "Error opening index file:" + fileName + " " + e.getMessage());
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            stdlog_.debug("Class definition not found in index file");
            e.printStackTrace();
        }
    }

    /**
     * Reads the index file to create the initial list of Template Summaries
     *
     * @param fileName Summary information file
     * @exception IOException IO error occurs
     * @exception ClassNotFoundException Class not in index files
     */
    public void writeIndexFile(String fileName) throws IOException {
        Vector<TemplateSignatureSummary> tmplSummList = new Vector<TemplateSignatureSummary>();
        Vector<String> tmplErrList = new Vector<String>();
        File dir = new File(tsfDirectory_);
        if (!dir.isDirectory()) {
            stdlog_.debug("No Such Directory " + tsfDirectory_);
            return;
        }
        ExtFileList filter = new ExtFileList(tsfDirectory_, ".tsf");
        String[] fileList = filter.getFileList();
        if (fileList == null) {
            stdlog_.debug(" No tsf files in the directory " + tsfDirectory_);
            return;
        }
        try {
            FileOutputStream ostream = new FileOutputStream(fileName);
            ObjectOutputStream summaryStream = new ObjectOutputStream(ostream);
            for (int i = 0; i < fileList.length; i++) {
                String templateName =
                    fileList[i].substring(0, fileList[i].indexOf('.'));
                stdlog_.debug(
                    "Instrument "
                        + dir.getName()
                        + " Template "
                        + templateName);
                TemplateSignatureMetaData tsMet =
                    TemplateSignatureFactory.createTemplateMetaData(
                        this,
                        templateName);
                if (tsMet != null && !tsMet.getCriticalStatus()) {
                    TemplateSignatureSummary templateSignatureSummary =
                        new TemplateSignatureSummary();
                    templateSignatureSummary.setTemplateName(
                        tsMet.getTemplateName());
                    templateSignatureSummary.setTemplateDesc(
                        tsMet.getTemplateDesc());
                    templateSignatureSummary.setTemplateType(
                        tsMet.getTemplateType());
                    templateSignatureSummary.setTemplateMode(tsMet.getMode());
                    tmplSummList.addElement(templateSignatureSummary);
                } else {
                    tmplErrList.addElement(fileList[i]);
                    stdlog_.debug("Critical Error in template " + fileList[i]);
                }
            }
            tmplErrList_ = new String[tmplErrList.size()];
            tmplErrList.copyInto(tmplErrList_);
            //            if(tmplSummList.size() > 0)
            stdlog_.debug(
                "!!!!!!!!!!!!!!! Writing out integer of size "
                    + tmplSummList.size());
            summaryStream.writeInt(tmplSummList.size());
            // else
            //stdlog_.debug("There is a problem " + dir.getName());
            for (int j = 0; j < tmplSummList.size(); j++) {
                TemplateSignatureSummary sum =
                    tmplSummList.elementAt(j);
                summaryStream.writeObject(sum);
            }
            ostream.close();

        } catch (IOException e) {

            e.printStackTrace();
            stdlog_.debug("Invalid output file " + e.getMessage());
        }
    }

    /**
     * This checks all the error variables and produces a string
     * that will be used when checking this file in Engineering Mode
     * @return String for Engineering Mode
     */
    public String toEngString() {
        StringBuffer buffer = new StringBuffer();
        int slashPos = instrumentFile_.lastIndexOf(File.separator) + 1;
        String isfFile = instrumentFile_.substring(slashPos);
        buffer.append("Processing instrument: " + isfFile + "\n");
        if (getCriticalStatus()) {
            buffer.append("A critical error has been found in the .isf file\n");
            if (compileErrorMsg_ != null) {
                buffer.append(compileErrorMsg_ + "\n");
                return buffer.toString();
            }
            if (pafLinesMissing_ != null && pafLinesMissing_.length > 0) {
                for (int i = 0; i < pafLinesMissing_.length; i++) {
                    buffer.append(
                        "Mandatory Paf keyword missing "
                            + pafLinesMissing_[i]
                            + "\n");
                }
                return buffer.toString();
            }
            if (mandatoryKeywordsMissing_.size() > 0) {
                for (int i = 0; i < mandatoryKeywordsMissing_.size(); i++) {
                    buffer.append(
                        "Mandatory keyword missing "
                            + (String) mandatoryKeywordsMissing_.elementAt(i)
                            + "\n");
                }
            }
            if (mandatoryValuesMissing_.size() > 0) {
                for (int i = 0; i < mandatoryValuesMissing_.size(); i++) {
                    buffer.append(
                        "Mandatory value missing for keyword "
                            + mandatoryValuesMissing_.elementAt(i)
                            + "\n");
                }
            }
            if (!isValidFileType()) {
                buffer.append(
                    "Set PAF.TYPE to Instrument Summary\n " + fileType_);
            }
        } else {
            buffer.append("No critical errors in this instrument file");
        }
        return buffer.toString();
    }

    /**
     * Returns a String representation of the class
     *
     * @return String representation of the class
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("Class Instrument\n");
        sb.append("Instrument name=" + instrumentName_ + "\n");
        sb.append("Telescope=" + telescopeName_ + "\n");
        sb.append("Version=" + version_ + "\n");
        sb.append("Instrument filename=" + instrumentFile_ + "\n");
        String[] modes = getModes();
        if (modes.length == 0) {
            sb.append("<ERROR Instrument Class> No modes set for instrument\n");
        } else {
            for (int i = 0; i < modes.length; i++) {
                sb.append("Mode=" + modes[i] + "\n");
            }
        }
        for (int i = 0; i < templateSummaryList_.length; i++) {
            TemplateSignatureSummary templateSummary = templateSummaryList_[i];
            sb.append(templateSummary.toString());
        }
        return sb.toString();
    }
    
    /**
     * Return the instraint constraint config classes for a given 
     * instrument.
     * @return The instrument constraint config classes for a given instrument
     */
    public InstrumentConstraintConfig[] getCsConfList() {
        return csConfList;
    }
    
    /**
     * Return the reamde configuration information.   
     * @return The readme metadata classes for a given instrument.
     */
    public InstrumentConstraintConfig[] getReadmeConfList() {
        return readmeConfList;
    }


    /**
     * Return the constraint properties names for a given instrument.
     * @return Constraint properties for a given instrument
     */
    public String [] getCsProps() {
        String [] csProps = new String[csConfList.length];
        for(int i=0;i<csConfList.length;i++) {
            csProps[i] = new String(csConfList[i].getName()); 
        }
        return csProps;
    }

    /**
     * Set the constraint properties for a given instrument.
     * @param Set the constraint proerties for a given instrument.
     */
    public void setCsConfList(InstrumentConstraintConfig[] configs) {
        csConfList = configs;
    }

    
    /**
     * The label for instrument specific comments
     * @return The instrument specific comments.
     */
    public String getLabelInstComment() {
        return labelInstComment;
    }

    /**
     * Set the instrument specific comments label
     * @param string
     */
    public void setLabelInstComment(String string) {
        labelInstComment = string;
    }

    /**
     * @return this ip has inst specific comments.
     */
    public boolean hasInstComments() {
        return instComments;
    }

    /**
     * @param b
     */
    public void setInstComments(boolean b) {
        instComments = b;
    }

}
