package org.eso.ohs.instruments;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;


/**
 * This class parses a .tsf file and initialised all the data associated with
 * that file including all the error handling variables which are output for
 * the Engineering mode users. A TemplateSignatureMetaData object can be referenced
 * from many TemplateSignature Objects
 * <p>
 * %Id: TemplateSignatureMetaData.java,v 1.25 2001/03/23 14:22:40 tschlich Exp %
 * @author Tim Canavan
 * @version $Revision: 1.57 $
 */



public class TemplateSignatureMetaData implements Serializable {

    /** Version id for object serialization */
    static final long serialVersionUID = -9032502422738435508L;
    static final String id_TemplateSignatureMetaData = "$Id: TemplateSignatureMetaData.java,v 1.57 2006/04/27 10:37:43 tcanavan Exp $";

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(TemplateSignatureMetaData.class);

    /** Any parameter with more than one entry in the template signature file */
    private transient String[] duplicateNameList_ = {};

    /**
     * The Instrument that this TSF is associated - the Instrument object
     * contains all of the ISF data, which is needed t build a TSF object
     */
    private transient Instrument instrument_;

    /**
     * Hashtable of TPL keywords (except TPL.PARAM)
     */
    private Hashtable<String, String> tplKeywords_ = new Hashtable<String, String>();

    /**
     * Name of the template this is usually the name of the file
     * This is stored in PAF.NAME in the .tsf file
     */
    private String templateName_;

    /** Description of what the template actually does */
    private String templateDesc_;

    /** Name of template signature */
    private String tsfName_;

    /**
     * A list of base classes which contain all the data describing the
     * various different types of parameters
     */
    private ParamMetaData [] paramList_;


    /** Mandatory keywords missing from the TemplateSignature file */
    private transient Vector<String> mandatoryKeywordsMissing_ = new Vector<String>();

    /** Mandatory values that should have entered in .tsf file */
    private transient Vector<String> mandatoryValuesMissing_ = new Vector<String>();

    /** Parameters which are in error */
    private transient Hashtable<String, Vector> paramErrorStatus_ = new Hashtable<String, Vector>();

    /** parameters which have a warning against them */
    private transient Hashtable<String, Vector> paramWarningStatus_ = new Hashtable<String, Vector>();

    /** List Errors assocaited with a particular parameter */
    private transient Vector<String> paramErrors_ = new Vector<String>();

    /** List of warnings associated with a particular parameter */
    private transient Vector<String> paramWarnings_ = new Vector<String>();

    /** Value should be TemplateSignature or Template Signature */
    private transient String fileType_;

    /** Paf lines that are missing */
    private transient String [] pafLinesMissing_ = {};

    /** Data that is not associated with any parameters */
    private transient PafData[] unLinkedParamData_ = {};

    /** Keywords that are not part of the ICD */
    private transient PafData[] invalidParamKeywords_ = {};


    /** Data that the .tsf file was last modified on */
    private long lastModified_;

    /** Execution time for a template */
    private int execTime_ ;

    /**
     * Message detailing the line number that the Paf file parser
     * failed to compile
     */
    private transient String compileErrorMsg_;

    /** Name of instrument is valid or not in .tsf file */
    private transient boolean isValidInstrument_ = true;

    /**
     * A keyword value pair list which is used to generate
     * the values for the rest of the data members for this class
     */
    private PafData [] identifierList_ = {};

    public final String SPACES = "      ";


    /**
     * Create TemplateSignatureMeta for a given template.
     * This data will be resued by all templates of this type.
     * @param tsfName template name not including .tsf extension
     * @param instrument  instrument associated with template signatute
     */
    public TemplateSignatureMetaData(String tsfName, Instrument instrument)
        throws IOException {
        this(tsfName, instrument,
             TemplateSignatureConstants.mandatoryTemplateKeywords,
             TemplateSignatureConstants.mandatoryTemplateValues,
             true);
    }


    /**
     * Create TemplateSignatureMeta for a given template.
     * This data will be resued by all templates of this type.
     * @param tsfName template name not including .tsf extension
     * @param instrument  instrument associated with template signatute
     * @param mandatoryKeys array of manadatory keywords to check
     * @param mandatoryVals array of manadatory values to check
     */
    public TemplateSignatureMetaData(String tsfName,
                                     Instrument instrument,
                                     String[] mandatoryKeys,
                                     String[] mandatoryVals,
                                     boolean checkParameterValidity)
        throws IOException {

        instrument_ = instrument;
        tsfName_ = tsfName;
        PafFile paf = new PafFile(tsfName);
        File f = new File(tsfName);
        lastModified_ = f.lastModified();
        if (paf.compileError()) {
            compileErrorMsg_ =  paf.getCompileErrorMsg();
            return;
        }
        if (!paf.isPafFile()) {
            pafLinesMissing_ = paf.getPafLinesMissing();
            return;
        }
        identifierList_ = paf.getIdentifierList();
        templateName_ = paf.getPafName();
        templateDesc_ = paf.getPafDescription();
        fileType_ = paf.getPafType();
        setValuesFromIdentifierList();
        setMandatoryKeywordsMissing(mandatoryKeys);
        setMandatoryValuesMissing(mandatoryVals);
        String in = getInstrumentName();
        if (in == null || !in.equals(instrument.getInstrumentName())) {
            isValidInstrument_ = false;
        }
        if (!isValidInstrument() || !isValidTemplateName())
            return;
        createParameters(checkParameterValidity);
    }


    public static String getStack (Throwable t) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        t.printStackTrace(pw);
        pw.close();
        return (sw.toString());
    }


    /**
     * Template signature file contains a critical error
     * @return true if signature file contains no critical errors otherwise
     *         false
     */
    public boolean getCriticalStatus() {
        Throwable t = new Throwable();
        String s = getStack(t);
        if (compileErrorMsg_ != null && compileErrorMsg_.length() > 0){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 01");

            return true;
        }
        if (pafLinesMissing_.length > 0){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 02");
            return true;
        }
        if (!isValidInstrument()){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 03");
            return true;
        }
        if (mandatoryKeywordsMissing_.size() > 0){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 04");
            return true;
        }
        if (mandatoryValuesMissing_.size() > 0){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 05");
            return true;
        }
        if (paramErrorStatus_.size() > 0){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 06");
            return true;
        }
        if (invalidParamKeywords_.length > 0){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 07");
            return true;
        }
        if (!isValidTemplateType()){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 08");
            return true;
        }
        if (!isValidFileType()){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 09");
            return true;
        }
        if (!isValidTemplateName()){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 10");
            return true;
        }
        if (execTime_ == -1){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 11");
            return true;
        }
        if (duplicateNameList_.length > 0){
            stdlog_.debug("CLASS TemplateSignatureMetaData "
                               + "- getCriticalStatus: 12");
            return true;
        }
        return false;
    }


    /**
     * Template Signature file contains information that generates
     * a warning after parsing
     *
     * @return true if template signature file contains faults that are not
     *         critical but should be corrected otherwise false
     */
    public boolean getWarningStatus() {
        if (paramWarningStatus_.size() > 0 ||
                unLinkedParamData_.length > 0)
            return true;
        return false;
    }


    /**
     * Template name passed to the classes constructor equals tthe value
     * associated with the PAF.NAME keyword.
     *
     * @return true if templae name passed to constructor equals
     * the vale associated with the PAF.NAME keyword otherwise false
     */
    public boolean isValidTemplateName() {
        int index = tsfName_.lastIndexOf(".");
        String tmpName = tsfName_.substring(0,index);
        if (templateName_ == null || !tmpName.endsWith(templateName_))
            return false;
        return true;
    }


    /**
     * Last modified date of the associated .tsf file
     * @return last modified date
     */
    public long getLastModified() {
        return lastModified_;
    }


    /**
     * Add a String TPL Keyword value to the tplKeyword Hashtable.
     */
    public void setTplKeyword(String name, String val) {
        tplKeywords_.put(name, val);
    }


    /**
     * @return the requested value (as a String), or null if not found
     */
    public String getTplKeywordValue(String name) {
        return tplKeywords_.get(name);
    }


    /**
     * Set the execution time for the template signature
     *
     * @param execTime execution time
     */
    public void setExecTime(int execTime) {
        execTime_ = execTime;
    }


    /**
     * Return the execution time
     *
     * @return execution time
     */
    public int getExecTime() {
        return execTime_;
    }


    /**
     * Creates a list of mandatory keywords which are missing
     *
     */
    private void setMandatoryKeywordsMissing(String[] keywords) {

        for (int i=0; keywords != null && i<keywords.length; i++) {
            boolean found = false;
            for (int j=0;j<identifierList_.length;j++) {
                if (identifierList_[j].getIdentifier().equals(
                                                             "TPL."+keywords[i])) {
                    found = true;
                    break;
                }

            }
            if (!found)
                mandatoryKeywordsMissing_.addElement(keywords[i]);
        }
    }


    /**
     * Create a list of mandatory keywords who do not have
     * mandatory keyword  values
     *
     */
    private void setMandatoryValuesMissing(String[] values) {
        for (int i=0; values != null && i< values.length;i++) {
            for (int j=0;j<identifierList_.length;j++) {
                if (identifierList_[j].getIdentifier().equals(
                                                             "TPL."+values[i])) {
                    if (!identifierList_[j].hasValue()) {
                        mandatoryValuesMissing_.addElement(identifierList_[j].getIdentifier());
                        break;
                    }
                }
            }
        }
    }


    /**
     * Valid instrument value
     * @return true if instrument value passed to constructor equals the value
     *         refered to by TPL.INSTRUM
     */
    private boolean isValidInstrument() {
        return isValidInstrument_;
    }


    /**
     * Check that the paramater type is a valid template type.
     * @return true if valid template type
     */
    private boolean isValidTemplateType() {
        boolean found = false;
        String tt = getTemplateType();
        if (tt == null || tt.length() <= 0)
            return false;
        for (int i=0;i<TemplateSignatureConstants.allowedTemplateTypes.length;i++) {
            if (TemplateSignatureConstants.allowedTemplateTypes[i].compareTo(tt)
                    == 0) {
                found = true;
                break;
            }
        }
        return found;
    }


    /**
     * Check that the value store in PAF.TYPE is valid
     * @return true if valid file type otherwise false
     */
    private boolean isValidFileType() {
        if (fileType_ != null && fileType_.length() > 0) {
            if (fileType_.equals("Template")
                    || fileType_.equals("Template Signature"))
                return true;
        }
        return false;
    }


    /**
     * Set the member variables of the template class from
     * the identifier list created in the base PafFile class.
     */
    private void setValuesFromIdentifierList() {
        PafData paf = null;
        String execTimeStr = null;
        for (int i=0;i<identifierList_.length;i++) {
            paf = identifierList_[i];
            if (paf.getIdentifier().equals(
                                           TemplateSignatureConstants.execTimeKeyword)) {
                execTimeStr = paf.getValue();
                if (execTimeStr != null && execTimeStr.length() > 0
                        && !execTimeStr.equals("computed")) {
                    try {
                        setExecTime(new Integer(execTimeStr).intValue());
                    }
                    catch (NumberFormatException e) {
                        setExecTime(-1);
                    }
                }
            } else {
                setTplKeyword(paf.getIdentifier(), paf.getValue());
            }
        }
        if (execTimeStr != null && execTimeStr.equals("computed")) {
            String tt = getTemplateType();
            if (tt != null && tt.equals("acquisition")) {
                execTime_ = -1;
            }
        }
    }


    /**
     * list of base class parameter classes
     * @return parmeter meta list
     */
    public ParamMetaData[] getParamList() {
        return paramList_;
    }


    /**
     * Value in TPL.MODE
     * @return  Value in TPL.MODE RILD etc..
     */
    public String getMode() {
        return getTplKeywordValue(TemplateSignatureConstants.modeKeyword);
    }


    /**
     * Value in TPL.INSTRUM
     * @return  Value in TPL.INSTRUN EMMI,SUSI etc..
     */
    public String getInstrumentName() {
        return getTplKeywordValue(TemplateSignatureConstants.instrumentKeyword);
    }


    /**
     * Value in TPL.TYPE
     * @return  Value in TPL.TYPE calibrated,science etc..
     */
    public String getTemplateType() {
        return getTplKeywordValue(TemplateSignatureConstants.typeKeyword);
    }


    /**
     * Value in TPL.RESOURCES
     * @param value in TPL.RESOURCES
     */
    public String getResources() {
        return getTplKeywordValue(TemplateSignatureConstants.resourceKeyword);
    }


    /**
     * Name of template
     * @return name of template
     */
    public String getTemplateName() {return templateName_;}


    /**
     * Template Description from PAF.DESC
     * @return Descripton of template
     */
    public String getTemplateDesc() {return templateDesc_;}


    /**
     * Version number for .tsf file. Value in VERSION keyword in .tsf file
     * @return Version number of .tsf file
     */
    public String getVersion() {
        return getTplKeywordValue(TemplateSignatureConstants.versionKeyword);
    }




    /**
     * Check to see if there is enough data to construnct a valid
     * paramater class. This means it must have a valid type number,
     * integer etc..
     * and default value if it is not fixed
     * @param parameterName Name of the parameter stored in
     * TPL.PARAM keywords in .tsf file
     *
     * @return boolean false if prameter is valid
     */
    private boolean isValidParam(String paramaterName, Hashtable paramMetaData){

        boolean inError=true;
        String type = paramaterName+"."
            +TemplateSignatureConstants.manadatoryParamKeywords[0];
        String range = paramaterName+"."
            + TemplateSignatureConstants.manadatoryParamKeywords[1];
        String defaultValue = paramaterName+"."
            +TemplateSignatureConstants.manadatoryParamKeywords[2];
        String value = paramaterName+"."+"VALUE";
        int lineNumber
            = ((PafData)paramMetaData.get("TPL.PARAM")).getLineNumber();
        if (paramMetaData.containsKey(value)) {
            PafData paf = (PafData)paramMetaData.get(value);
            if (!paf.hasValue())
                paramWarnings_.addElement("Warning Line "+"("
                                          +paf.getLineNumber()+") : "
                                          +"Please enter value into .VALUE parameter");
            return true;
        }
        if (!paramMetaData.containsKey(type)) {
            paramErrors_.addElement("Error Line "+"("+lineNumber+") : "
                                    +"Please define a type for parameter " + paramaterName);
            inError=false;
        }
        if (!paramMetaData.containsKey(defaultValue)) {
            paramErrors_.addElement("Error Line "+"("+lineNumber+") : "
                                    +"Please define a default value for this parameter "
                                    +paramaterName);
            inError=false;
        }
        if (type != null && !type.equals("boolean") && !type.equals("file")) {
            if (!paramMetaData.containsKey(range)) {
                paramErrors_.addElement("Error Line "+"("+lineNumber+") : "
                                        +"Please define a valid range for this parameter "
                                        +paramaterName);
                inError=false;
            }
        }
        return inError;
    }


    /**
     * Check that the paramater type is in the validParamTypes_ array.
     * in TemplateSignatureConstants
     * @param parameterType Type of parameter.
     * @return boolean True if in TemplateSignatureConstants.validParamTypes
     * else false
     */
    private boolean isValidParamType(String parameterType) {
        boolean found = false;
        for (int i=0;i<TemplateSignatureConstants.validParamTypes.length;i++) {
            if (TemplateSignatureConstants.validParamTypes[i].equals(
                                                                    parameterType)) {
                found = true;
                break;
            }
        }
        return found;
    }


    /**
     * Get appropiate value from instrument file. These are values
     * that are referenced in the .tsf by the value associated with the
     * keyword QUERY-INST for a given parameter in a .tsf file.
     * See Instrument documentation for more details
     * @param value Any range value in the .tsf file which begins with
     *        QUERY-INST
     * @return Value from instument file
     */
    private String getValueFromInstrument(String value) {
        String rangeValue = null;
        StringTokenizer tokenString = new StringTokenizer(value);
        while (tokenString.hasMoreElements()) {
            rangeValue = tokenString.nextToken();
        }
        rangeValue = instrument_.getInstrumentRange(rangeValue,getMode());
        return rangeValue;
    }


    /**
     * Initialise the range value for a parameter in a .tsf file
     * from a keyword in the instrument file
     *
     * @param p  reference to  parameter meta class
     * @param range reference keyword in instrument file which holds range
     * @param paf   reference to pafdata in .isf file
     * @return String Instrument range
     */
    private String initValueFromInst(ParamMetaData p, String value,PafData paf){
        String instRange = getValueFromInstrument(value);
        String typeValue = p.getType();
        if (instRange == null) {
            if (p.isDisplay())
                paramErrors_.addElement("Error Line "+"("
                                        +paf.getLineNumber()+") : "
                                        +"No reference to " + value + " in instrument file");
            else
                paramWarnings_.addElement("Warning Line "+"("
                                          +paf.getLineNumber()+") : "
                                          +"No reference to " + value + " in instrument file");
        }
        else if (instRange.length() <= 0) {
            paramWarnings_.addElement("Warning Line "+"("
                                      +paf.getLineNumber()+") : "
                                      +"No value entered for "+value+" in instrument file");
        }
        return instRange;
    }


    

    /**
     * Create the paramater Meta Data to be stored in the parameter list
     * @param parameterName Name of parameter to create
     * @param paramData A hashtable of parameter metaData
     * @return ParamMetaData All the read only data for every
     * parameter in a .tsf file
     *
     * <br>ChangeLog:  AMC, 04-Jul-2001: fixed 1454
     */
    private ParamMetaData createParameter(String parameterName,
                                          Hashtable paramData) {
        ParamMetaData p = null;
        PafData paf = null;
        String typeValue = "";
        paramWarnings_ = new Vector<String>();
        paramErrors_ = new Vector<String>();
        int lineNumber = ((PafData)paramData.get("TPL.PARAM")).getLineNumber();
        if (isValidParam(parameterName,paramData)==true) {
            String typeId = parameterName + ".TYPE";
            if (paramData.containsKey(typeId)) {
                paf = (PafData)paramData.get(typeId);
                if (paf.hasValue())
                    typeValue = paf.getValue();
                else {
                    paramErrors_.addElement("Error Line "+"("
                                            +lineNumber+"): "
                                            +"No value entered for parameter type");
                    return null;
                }
            }
            //if (typeValue.trim().equals("paramfile"))
            //      typeValue = "file";
            if (isValidParamType(typeValue)) {
                //stdlog_.debug("Parameter Name " + parameterName);
                p = new ParamMetaData(parameterName);
                p.setType(typeValue);
                String paramValueId = parameterName + ".VALUE";
                if (paramData.containsKey(paramValueId)) {
                    paf = (PafData)paramData.get(paramValueId);
                    String value = paf.getValue();
                    if (value.startsWith("ISF")
                            ||value.startsWith("QUERY-INST")) {
                        value = initValueFromInst(p,value,paf);
                    }
                    //if (paf.hasValue()) {
                    p.setFixed(true);
                    p.setValue(value);
                    //}
                }
                String hiddenId = parameterName + ".HIDE";
                //stdlog_.debug("Hodden ID " + hiddenId);
                if (paramData.containsKey(hiddenId)) {
                    paf = (PafData)paramData.get(hiddenId);
                    // AMC, 04-Jul-2001: fixed 1454
                    if (paf.hasValue()) {
                        p.setHiddenKeyword(true);
                        String value  = paf.getValue().toUpperCase();
                        if (value.indexOf("OHS")!=-1) {
                            stdlog_.debug("Setting hidden to true");
                            p.setHidden(true);
                        }
                    }
                }
                String paramRangeId = parameterName + ".RANGE";
                String range = "";
                if (paramData.containsKey(paramRangeId)) {
                    paf = (PafData)paramData.get(paramRangeId);
                    if (paf.hasValue()) {
                        range = paf.getValue();
                        if (range.startsWith("ISF")
                                || range.startsWith("QUERY-INST")) {
                            String instRange = initValueFromInst(p,range,paf);
                            range = instRange;
                        }
                    }
                }
                p.setRange(range);
                if (typeValue.equals("number")
                        || typeValue.equals("numberlist")
                        || typeValue.equals("integer")
                        || typeValue.equals("intlist")) {
                    if (range != null && range.length() > 0) {
                        int index = range.indexOf("...");
                        if (index != -1)
                            paramErrors_.addElement("Error line "+"("
                                                    +paf.getLineNumber() + ") : "
                                                    +"Illegal substring ... in range : " + range);
                    }
                }
                String paramDefaultId = parameterName + ".DEFAULT";
                if (paramData.containsKey(paramDefaultId)) {
                    paf = (PafData)paramData.get(paramDefaultId);
                    String defaultValue="";
                    if (paf.hasValue()) {
                        defaultValue = paf.getValue();
                        if (defaultValue.startsWith("ISF")
                                || defaultValue.startsWith("QUERY-INST")) {
                            String instValue = initValueFromInst(p,
                                                                 defaultValue,
                                                                 paf);
                            defaultValue = instValue;
                        }
                        if (!p.isFixed()) {                           
                            p.setValue(defaultValue);
                        }
                        p.setDefault(defaultValue);
                    } else {

                        if (typeValue.equals("coord")) {
                            // This if statement is for historical reasons to accomodate FOTS...
                            if (parameterName.indexOf("GS1.ALPHA")!=-1
                                    ||
                                    parameterName.indexOf("GS1.DELTA")!=-1) {
                                stdlog_.debug(
                                              "Setting default for Guide star "
                                              + parameterName);
                                p.setDefault("0");
                                p.setValue("0");
                            } else {
                                p.setDefault("");
                                p.setValue("");
                            }

                        } else {
                            if (!p.isHidden()) {
                                p.setDefault("NODEFAULT");
                                p.setValue("NODEFAULT");
                            } else {
                                p.setDefault("");
                                p.setValue("");
                            }
                        }
                    }
                }
                
                String paramMiniHelpId = parameterName + ".MINIHELP";
                if (paramData.containsKey(paramMiniHelpId)) {
                     paf = (PafData)paramData.get(paramMiniHelpId);
                     if (paf.hasValue())
                         p.setMiniHelp(paf.getValue());
                     else {
                         if(p.isDisplay())
                             paramWarnings_.addElement("Warning Line "
                                                      +"("+paf.getLineNumber()  + ") : "
                                                      +"No mini help entered for "
                                                      +p.getParameterName());
                     }
                } else {
                        if(p.isDisplay())
                            paramWarnings_.addElement("Warning Line "
                                                  +"("+lineNumber  + ") : "
                                                  +"No mini help for parameter "
                                                  +p.getParameterName());
                }
                String paramLabelId = parameterName + ".LABEL";
                if (paramData.containsKey(paramLabelId)) {
                    paf = (PafData)paramData.get(paramLabelId);
                    if (paf.hasValue())
                        p.setLabel(paf.getValue());
                    else
                        p.setLabel(parameterName);
                } else {
                    if(p.isDisplay())
                    paramWarnings_.addElement("Warning Line "+"("
                                              +lineNumber  + ") : "
                                              +"No label for parameter "
                                              +p.getParameterName());
                    p.setLabel(parameterName);
                }
                
            } else {
                paramErrors_.addElement("Error Line "+"("+lineNumber+") : "
                                        +"Invalid parameter tpye "+typeValue);

            }
        }
        return p;
    }


    /**
     * Create a list data items that are not linked to ant parameters
     * @param paramNameList list of parameter names associated with
     * this template signature
     */
    private void initunLinkedParamData(Hashtable<String, Hashtable<String,PafData>> paramNameList) {
        Vector<PafData> unLinkedParamData = new Vector<PafData>();
        for (int i=0;i<identifierList_.length;i++) {
            String id = identifierList_[i].getIdentifier();
            if (id.indexOf(".")!=-1) {
                String firstKeyword = id.substring(0,
                                                   id.indexOf("."));
                if (Parameter.validParamKeyword(firstKeyword)) {
                    String paramName = id.substring(0,id.lastIndexOf("."));
                    if (paramName != null
                            && !paramNameList.containsKey(paramName))
                        unLinkedParamData.addElement(identifierList_[i]);
                }
            }
        }
        unLinkedParamData_ = new PafData[unLinkedParamData.size()];
        unLinkedParamData.copyInto(unLinkedParamData_);
    }


    /**
     * Initialise the base class parameter list from the identifier list
     * and all the other engineering mode error reporting Vectors.
     */
    private void createParameters(boolean checkValidParam) {
        int size = identifierList_.length;
        Hashtable<String, Hashtable<String,PafData>> initialParamList = new Hashtable<String, Hashtable<String,PafData>>();
        Vector<String> paramNameList = new Vector<String>();
        Vector<String> duplicateNameList = new Vector<String>();
        Vector<PafData> invalidParamKeywords = new Vector<PafData>();
        for (int i=0;i<size;i++) {
            String id = identifierList_[i].getIdentifier();
            if (id.equals("TPL.PARAM")) {
                String value = identifierList_[i].getValue();
                if (value.indexOf('.')==-1) {
                    invalidParamKeywords.addElement(identifierList_[i]);
                    continue;
                }
                if (checkValidParam && !ParamMetaData.validParamKeyword(
                                                                       value.substring(0,value.indexOf('.')))) {
                    invalidParamKeywords.addElement(identifierList_[i]);
                    continue;
                }
                if (!initialParamList.containsKey(value)) {
                    initialParamList.put(value,
                                         new Hashtable<String,PafData>());
                    paramNameList.addElement(value);
                } else {
                    duplicateNameList.addElement(
                                                 identifierList_[i].getValue());
                }
            }
        }
        setParamData(initialParamList);
        initunLinkedParamData(initialParamList);
        Vector<ParamMetaData> paramList = new Vector<ParamMetaData>();
        for (int i=0;i<paramNameList.size();i++) {
            String paramName = paramNameList.elementAt(i);
            Hashtable paramMeta = initialParamList.get(paramName);
            ParamMetaData p =  createParameter(paramName,paramMeta);
            if (p != null)
                paramList.addElement(p);
            if (paramErrors_.size() > 0)
                paramErrorStatus_.put(paramName,paramErrors_);
            if (paramWarnings_.size() > 0) {                
                paramWarningStatus_.put(paramName,paramWarnings_);
            }
        }
        paramList_ = new ParamMetaData[paramList.size()];
        paramList.copyInto(paramList_);
        duplicateNameList_ = new String[duplicateNameList.size()];
        duplicateNameList.copyInto(duplicateNameList_);
        invalidParamKeywords_ = new PafData[invalidParamKeywords.size()];
        invalidParamKeywords.copyInto(invalidParamKeywords_);
    }


    /**
     * A list of duplicate parameters.
     * @return list of duplicate parameter names
     */
    public String[] getDuplicateNameList() {
        return duplicateNameList_;
    }


    /**
     * Scans the warning hashtable and produces a warning string
     * associated with an individual paramter in the meta data parameter list
     * @param paramName Name of parameter
     */
    private String getWarningString(String paramName) {
        StringBuffer buffer = new StringBuffer();
        if (paramWarningStatus_.containsKey(paramName)) {
            Vector warningList = paramWarningStatus_.get(paramName);
            for (int l=0;l<warningList.size();l++)
                buffer.append((String)warningList.elementAt(l) + "\n");
        }
        return buffer.toString();
    }


    /**
     * Create an string for warning
     *  processing that constains un linkedparam data,
     * @return unlinked param data
     */
    private String getUnLinkedString() {
        StringBuffer buffer = new StringBuffer();
        for (int i=0;i<unLinkedParamData_.length;i++) {
            PafData paf = unLinkedParamData_[i];
            buffer.append("Warning Line "+"("+paf.getLineNumber()+") : "
                          +paf.getIdentifier()
                          +" should have an associated TPL.PARAM\n");
        }
        return buffer.toString();
    }


    /**
     * This method is used for engineering mode to produce a report on
     * the state of the .tsf file after it is parsed.
     *
     * @return list of errors and warnings associated with .tsf file
     */
    public String toEngString() {

        int slashPos = tsfName_.lastIndexOf(File.separator) + 1;
        String tsfFile = tsfName_.substring(slashPos);

        StringBuffer buffer = new StringBuffer();
        buffer.append("Processing template signature file: " + tsfFile + "\n");

        if (!getCriticalStatus()) {
            buffer.append("No critical errors found in this file\n");

        } else {
            buffer.append("A critical error has been found:\n");
            if (compileErrorMsg_ != null) {
                buffer.append(compileErrorMsg_ + "\n");
            }
            if (duplicateNameList_ != null) {
                for (int i=0; i<duplicateNameList_.length; i++) {
                    buffer.append("Duplicate Parameter found " +
                        duplicateNameList_[i] + "\n");
                }
            }
            if (pafLinesMissing_.length > 0) {
                for (int i=0; i<pafLinesMissing_.length; i++) {
                    buffer.append("Mandatory Paf keyword missing " +
                        pafLinesMissing_[i] + "\n");
                }
            }
            if (invalidParamKeywords_.length > 0) {
                for (int i=0; i<invalidParamKeywords_.length; i++) {
                    buffer.append("Incorrect parameter prefix at linenumber " +
                        invalidParamKeywords_[i].getLineNumber() + "\n");
                }
            }
            if (!isValidTemplateName()){
                buffer.append("Set PAF.NAME to the name of the file\n");
            }
            if (mandatoryKeywordsMissing_.size() > 0) {
                for (int i=0; i<mandatoryKeywordsMissing_.size(); i++) {
                    buffer.append("Mandatory keyword missing " +
                        mandatoryKeywordsMissing_.elementAt(i) + "\n");
                }
            }
            if (mandatoryValuesMissing_.size() > 0) {
                for (int i=0; i<mandatoryValuesMissing_.size(); i++) {
                    buffer.append("Mandatory value missing for keyword " +
                        mandatoryValuesMissing_.elementAt(i) + "\n");
                }
            }
            if (!isValidFileType()) {
                buffer.append("Set PAF.TYPE to Template\n");
            }
            String tt = getTemplateType();
            if (!isValidTemplateType()) {
                if (tt == null || tt.length() <=0){
                    buffer.append("TPL.PARAM value not set\n");
                } else {
                    buffer.append("Invalid Template Type in TPL.TYPE "+tt+"\n");
                }
            }
            if (tt != null && tt.equals("acquisition")) {
                if (execTime_ == -1) {
                    buffer.append("Enter an integer value into the " +
                        "TPL.EXECTIME value in the acquisition .tsf file\n");
                    buffer.append("Acquisition template cannot have computed " +
                        "value in TPL.EXECTIME\n");
                }
            }
            if (!isValidInstrument()) {
                if (getInstrumentName() == null){
                    buffer.append("No instrument specified for this template " +
                        "in TPL.INSTRUM\n");
                } else {
                    buffer.append("Template refers to an invalid instrument " +
                        getInstrumentName() + " in TPL.INSTRUM"+ "\n");
                }
            }
            if (paramErrorStatus_.size() > 0) {
                Enumeration<String> keys = paramErrorStatus_.keys();
                while (keys.hasMoreElements()) {
                    String paramName = keys.nextElement();
                    Vector errList = paramErrorStatus_.get(paramName);
                    for (int j=0; j<errList.size(); j++){
                        buffer.append((String)errList.elementAt(j) + "\n");
                    }
                    buffer.append(getWarningString(paramName));
                }
            }
            return buffer.toString();
        }

        /** Warning messages */
        if (getWarningStatus()) {
            buffer.append("Warning(s) have been found in this file\n");
        } else {
            buffer.append("No warnings have been found in this file\n");
        }
        if (paramWarningStatus_.size() > 0) {
            Enumeration<String> keys = paramWarningStatus_.keys();
            while (keys.hasMoreElements()) {
                String paramName = keys.nextElement();
                if (!paramErrorStatus_.containsKey(paramName))
                    buffer.append(getWarningString(paramName));
            }
        }
        buffer.append(getUnLinkedString());
        buffer.append("Finished processing template " + templateName_ + "\n");
        return buffer.toString();
    }


    /**
     * Create a Hashtable of values that contain TPL.
     * but does not contain TLP.PARAM
     * @param identifierList list of keyword,value pairs
     * @return list of keyword,value pairs that contains paf data.
     */
    private void setParamData(Hashtable<String, Hashtable<String,PafData>> paramData)  {
        Enumeration<String> keys = paramData.keys();
        while (keys.hasMoreElements()) {
            String paramName = keys.nextElement();
            Hashtable<String,PafData> paramMetaData = paramData.get(paramName);
            paramMetaData.put("TPL.PARAM",getPafDataForParamName(paramName));
            for (int i=0;i<identifierList_.length;i++) {
                String id = identifierList_[i].getIdentifier();
                if (id.indexOf(paramName)!=-1) {
                    paramMetaData.put(id,identifierList_[i]);
                }
            }
        }
    }


    /**
     * Return the PafData attribute from a given
     *
     */
    private PafData getPafDataForParamName(String paramName) {
        for (int i=0;i<identifierList_.length;i++) {
            if (identifierList_[i].getIdentifier().equals("TPL.PARAM")) {
                if (paramName.equals(identifierList_[i].getValue()))
                    return identifierList_[i];
            }
        }
        return null;
    }


    /**
     * @return the indentifier list - for readonly use!
     */
    public PafData[] getIdentifierList() {
        return identifierList_;
    }
}
