package org.eso.ohs.instruments;

import gnu.regexp.RE;
import gnu.regexp.REException;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;

/**
 * methods for handling IP Zip files
 *
 * @version $Revision: 1.22 $
 * @author Ron J Lobo
 */
public class ZipUtils {

    public static final String cvsID_ =
        "$Id: ZipUtils.java,v 1.22 2006/04/26 09:26:49 tcanavan Exp $";

    private static Logger stdlog_ = Logger.getLogger(ZipUtils.class);

    private static final String sep = File.separator;
    private static final String[] sepList = {"/","\\"};
       private static String baseTmpDir_;  // eg /tmp/
    private File tmpDirL1_;             // eg /tmp/IPacMan0
    private File rootDir_;              // eg /tmp/IPacMan0/SOFI-3.14

    private String zipFile_;    // name of zipFile
    private String dir1_;       // name of the first level directory

    private Instrument instrument_;

    private int numTSFs_;
    private int numWarnings_;
    private int numCritical_;
    private int numInvalidTemplateNames_;

    private String report_;
    private String reportHead_;
    private boolean makeReport_;
    private boolean dirStructureOk_;
    private boolean dir1NameOk_;



    /**
     * constructor
     *
     * @param zipFile   name of zip file to verify
     */
    public ZipUtils(String zipFile) {

        // stdlog_.debug("ZipUtils ctor: " + zipFile);
        zipFile_ = zipFile;

        numWarnings_ = 0;
        numCritical_ = 0;
        numInvalidTemplateNames_ = 0;

        dirStructureOk_ = true;
        dir1NameOk_ = true;

        int p1 = zipFile_.lastIndexOf(sep) + 1;
        int p2 = zipFile_.lastIndexOf("-");
        dir1_ = zipFile_.substring(p1, p2);
    }


    /**
     * verify an Instrument Package
     *
     * @param makeReport if true, writes report_ on all warnings and errors
     *                          if false, quits after 1st error.
     * @return number of warnings found
     */
    public int verifyIP(boolean makeReport) throws ZipUtilsException {

        // stdlog_.debug("ZipUtils.verifyIP: mR=" + makeReport);
        reportHead_ = "\nVerify Instrument Package\n\nFilename: " + zipFile_;
        report_ = "\n\n\n";
        makeReport_ = makeReport;

        ZipFile zf = null;
        try {
            zf = new ZipFile(zipFile_);
        } catch (IOException e) {
            stdlog_.error("Zip file error: " + e.getMessage());
            String mesg = "Error Verfying IP\nInvalid Zip File";
            throw new ZipUtilsException(mesg);
        }
        Enumeration<? extends ZipEntry> enu = zf.entries();

        // Initial basic checks of Zip file
        ZipEntry defaultISF_ze = basicChecks(enu);

        /** Zip file is OK - now examine its contents */

        /** first get instrument and  version from default.isf */
        handleDefaultISF(zf, defaultISF_ze);

        /** now verify all the tsf files */
        enu = zf.entries();
        while (enu.hasMoreElements()) {
            ZipEntry ze = enu.nextElement();
            if (ze.getName().endsWith(".tsf"))
                verifyTSF(zf, ze);
        }

        removeTmpDirs();
        return numWarnings_;
    }


    public String getReport() {

        String summary = "\n\n" +
            "Number of TSF Files:        " + numTSFs_ + "\n" +
            "Files with Warnings:        " + numWarnings_ + "\n" +
            "Files with Critical Errors: " + numCritical_;

        return reportHead_ + summary + report_;
    }


    /**
     * extract the given InputStream to the given directory
     *
     * @param   is      InputStream containing the ZipFile
     * @param   dir     directory to unzip to
     * @param   zipFile filename of ZipFile
     * @param   unzip   write the single zip file or unzip the file
     * @return  void
     */
    public static void extract(
        InputStream is, String dir, String zipFile, boolean unzip)
    throws IOException {

        //long start = (new Date()).getTime();
        //stdlog_.debug("ZipUtils.unZip InputStream to " + dir);

        if (!unzip) {
            // extract only
            String f = dir + sep + zipFile;
            writeFileFromStream(is, f);
            is.close();

        } else {
            // unzip archive
            ZipInputStream zis = new ZipInputStream(is);
            ZipEntry ze;
            while ((ze = zis.getNextEntry()) != null) {
                String fileName = ze.getName();
                //stdlog_.debug(" got next entry: " + fileName);

                if (ze.isDirectory()) {
                    //stdlog_.debug(" found a directory");
                    makeDirs(dir + sep + fileName);
                } else {
                    //stdlog_.debug(" found a file");
                    int pos = fileName.lastIndexOf(sep);
                    String d = (pos == -1) ? "" : fileName.substring(0, pos);
                    makeDirs(dir + sep + d);

                    String f = dir + sep + fileName;
                    //stdlog_.debug(" about to write: " + f);
                    writeFileFromStream(zis, f);
                }
                zis.closeEntry();
            }
            zis.close();
        }
        //long duration = (new Date()).getTime() - start;
        //stdlog_.debug("ZipUtils extract took: "+duration+" ms.");
    }


    /**
     * Remove the summary.idx file from a Instrument Package zip file
     *
     * @param zip    zip file to remove summary.idx from
     * @return the modified file (or the original file if there was no summary.idx)
     */
    public static File removeSummaryIDX(File zip) throws ZipUtilsException {

        File f = null;
        try {
            ZipFile zipFileIn = new ZipFile(zip);
            boolean found = false;
            f = File.createTempFile("ipacman", "zip");
            ZipOutputStream zipFileOut = new ZipOutputStream(
                new BufferedOutputStream(new FileOutputStream(f)));

            Enumeration<? extends ZipEntry> enu = zipFileIn.entries();
            while (enu.hasMoreElements()) {
                ZipEntry ze = enu.nextElement();
                // stdlog_.debug("ze: " + ze.getName());
                // stdlog_.debug("  size  " + ze.getSize() + ", " + ze.getCompressedSize());
                if (ze.getName().endsWith("summary.idx")) {
                    // stdlog_.debug("removing summary.idx");
                    found = true;
                } else if (ze.isDirectory()) {
                    // don't save directories because it can cause
                    // permissions problems on Solaris
                    // stdlog_.debug("removing directory: " + ze.getName());
                } else {
                    zipFileOut.putNextEntry(ze);
                    InputStream inStr = zipFileIn.getInputStream(ze);
                    BufferedInputStream bis = new BufferedInputStream(inStr);
                    final int BUFFER = 1024;
                    byte data[] = new byte[BUFFER];
                    int count;
                    while((count = bis.read(data, 0, BUFFER)) != -1) {
                        // stdlog_.debug("count=" + count);
                        zipFileOut.write(data, 0, count);
                    }
                    bis.close();
                    zipFileOut.closeEntry();
                }
            }
            zipFileOut.close();

            if (found) {
                return f;
            } else {
                f.delete();
                return zip;
            }
        } catch (FileNotFoundException e) {
            throw new ZipUtilsException(e.getMessage());
        } catch (IOException e) {
            // String mesg = "can't create tmp file" + e.getMessage();
            stdlog_.error(e.getMessage());
            if (f != null && f.exists()) {
                f.delete();
            }
            throw new ZipUtilsException(e.getMessage());
        }
    }


    /**
     * used by extract() to write zipFile or zipFileEntry to disk
     */
    private static void writeFileFromStream(InputStream is, String f)
    throws IOException {
        FileOutputStream fos = new FileOutputStream(f);
        BufferedInputStream bis = new BufferedInputStream(is);
        BufferedOutputStream bos = new BufferedOutputStream(fos);
        int count = 0;
        int byteRead;
        while ((byteRead = bis.read()) != -1) {
            // stdlog_.debug(" byteRead=" + byteRead);
            bos.write(byteRead);
            count++;
        }
        //stdlog_.debug(" wrote " + count + " bytes to " + f);
        bos.flush();
        bos.close();
        fos.flush();
        fos.close();
    }


    /**
     * used by extract() create directories
     */
    private static void makeDirs(String newDir) {
        File newDirFile = new File(newDir);
        if (newDirFile.exists()) {
            return;
        }
        newDir = newDirFile.getAbsolutePath();

        //stdlog_.debug("makeDirs: newDir=" + newDir);
        String base = "";
        StringTokenizer st = new StringTokenizer(newDir, sep);
        while (st.hasMoreTokens()) {
            String dirName = base + sep + st.nextToken();
            //stdlog_.debug(" dir=" + dirName);
            File tmpDir = new File(dirName);
            if (!tmpDir.exists()) {
                //stdlog_.debug(" mkdir " + dirName);
                tmpDir.mkdir();
            }
            base = dirName;
        }
    }


    /**
     * bacic checks of the ZipFile
     *
     * @param enu    zip file entries enum
     * @return returns the ZipEntry of the default.isf file
     */
    public ZipEntry basicChecks(Enumeration<? extends ZipEntry> enu) throws ZipUtilsException {
         
        // stdlog_.debug("basicChecks");
        int numDirs = 0;
        int numDefISF = 0;
        int numEntries = 0; // = zf.size(); {JDK1.4}
        int numOthers = 0;
        numTSFs_ = 0;
        ZipEntry defaultISF_ze = null;

        while (enu.hasMoreElements()) {
            ZipEntry ze = enu.nextElement();
            numEntries++;

            String name = ze.getName();
            stdlog_.debug("filename=" + name);
            boolean tsfOrIsf = true;
            
            if (ze.isDirectory()) {
                numDirs++;
                tsfOrIsf = false;
            } else if (name.endsWith(".tsf")) {
                numTSFs_++;
            } else if ((name.endsWith(sepList[0] + "default.isf"))|| (name.endsWith(sepList[1] + "default.isf"))) {
                defaultISF_ze = ze;
                numDefISF++;
                if (dir1NameOk_ ) {
                    if ( !name.equals(dir1_ + sepList[0] + "default.isf") || !name.equals(dir1_ + sepList[0] + "default.isf")) {
                        err("default.isf must be in directory " + dir1_);
                    }
                }                
            } else if (name.endsWith("summary.idx")) {
                err("Instrument Package contains 'summary.idx'\n" +
                    "Please remove it to insert the IP in the database");
            } else {
                numOthers++;
                tsfOrIsf = false;
            }

            if (tsfOrIsf) {
                checkDirStructure(name);
            }
        }

        stdlog_.debug("entries in Zip     = " + numEntries);
        stdlog_.debug("directories in Zip = " + numDirs);
        stdlog_.debug("isf files in Zip   = " + numDefISF);
        stdlog_.debug("tsf files in Zip   = " + numTSFs_);
        stdlog_.debug("other files in Zip = " + numOthers);

        if (numDefISF < 1) {
            makeReport_ = false;                // force processing to quit
            err("default.isf not present in zip file");
        } else if (numDefISF > 1) {
            makeReport_ = false;                // force processing to quit
            err("More than 1 default.isf found in zip file");
        } else if (numTSFs_ == 0) {
            err("No tsf's present in zip file");
        }

        return defaultISF_ze;
    }


    /**
     * verify the default.isf file
     *
     * @param zf    zip file
     * @param ze    zip entry of the default.isf file
     */
    private void handleDefaultISF(ZipFile zf, ZipEntry ze)
    throws ZipUtilsException {

        String filename = ze.getName();
        // stdlog_.debug("verifying " + filename);

        makeTmpDir(filename);
        String ISFfile = writeZipEntryToFile(zf, ze);
        String ISFdir = ISFfile.substring(0, ISFfile.lastIndexOf(sep));

        try {
            instrument_ = new Instrument(ISFdir, ISFfile);
            String instrVerNumStr = instrument_.getVersion();
            float instrVerNum = instrument_.getVersionNumber();
            stdlog_.debug("instrument_=" + instrument_.getInstrumentName());
            stdlog_.debug("versionS=" + instrVerNumStr);
            stdlog_.debug("versionF=" + instrVerNum);

            String verS = dir1_.substring(dir1_.lastIndexOf("-")+1);
            float verF = Float.valueOf(verS).floatValue();
            stdlog_.debug("dir1_=" + dir1_);
            stdlog_.debug("verS=" + verS);
            stdlog_.debug("verF=" + verF);

            if (instrVerNumStr == null) {
                err("Version number missing in default.isf file");
            } else {
                RE regExp = new RE("^(0|[1-9][0-9]{0,3})\\.[0-9]{2}$");
                if (!regExp.isMatch(instrVerNumStr)) {
                    err("Illegal version number in default.isf file (" +
                        instrVerNumStr + ")\n" +
                        "Version number in default.isf should be " + verS);
                }

                if (!verS.equals(instrVerNumStr)) {
                    err("Version number error:\n" +
                        "The version number in the default.isf file (" +
                        instrVerNumStr + ") and\n" +
                        "the version number of the first level directory (" +
                        verS + ")\n" +
                        "should be exactly the same");
                }
            }

        } catch (IOException e) {
            deleteFile(ISFfile);
            // makeReport_ = false;     // force this verification to quit
            err("default.isf error: " + e.getMessage());
        } catch (REException e) {
            stdlog_.error("RegExp error: " + e.getMessage());
            err("An error occurred while checking version number in default.isf");
        }

        deleteFile(ISFfile);
        report_ += instrument_.toEngString() + "\n\n\n";
    }


    /**
     * verify a .tsf file
     *
     * @param zf    zip file
     * @param ze    zip entry of the tsf file
     */
    private void verifyTSF(ZipFile zf, ZipEntry ze) throws ZipUtilsException {

        String tsfName = ze.getName();
        // stdlog_.debug("verifyTSF: " + tsfName);
        String TSFfile = writeZipEntryToFile(zf, ze);

        try {
            TemplateSignatureMetaData tsmd =
                new TemplateSignatureMetaData(TSFfile, instrument_);
            deleteFile(TSFfile);

            report_ += tsmd.toEngString() + "\n";

            if (tsmd.getWarningStatus()) {
                numWarnings_++;
            }
            if (tsmd.getCriticalStatus()) {
                numCritical_++;
                if (!makeReport_)
                    err("Critical status: " + tsfName);
            }
            if (!tsmd.isValidTemplateName()) {
                numInvalidTemplateNames_++;
                if (!makeReport_)
                    err("Invalid template name: " + tsfName);
            }
        } catch (IOException e) {
            err("File IO error: " + e.getMessage());
        }
    }


    /**
     * creates a temporary directory into which zip entries will be written
     *
     * @param filename  contains name of directory to be created
     */
    public void makeTmpDir(String filename) {

        // stdlog_.debug("makTmpDir: " + filename);

        // create tmpDirL1_
        tmpDirL1_ = makeNewTmpDir();
        int slashPos = 0;
        for (int i = 0; i < sepList.length; i++) {
            slashPos = filename.indexOf(sepList[i]);
            
            // found separator used.
             if (slashPos > 0) {
                 break;
             }
            }
            // if separator not found set to def length
            if (slashPos < 0){
                slashPos = 0;
            }
        // create rootDir_
        String newDir = filename.substring(0, slashPos);
        String dirName = tmpDirL1_ + sep + newDir;
        // stdlog_.debug("making directory: " + dirName);
        rootDir_ = new File(dirName);
        rootDir_.mkdir();
    }


    public static File makeNewTmpDir() {
        int count = 0;
        File newDirF;
        do {
            String newDir = baseTmpDir_ + "IPacMan" + count++;
            // stdlog_.debug("checking dir exists: " + newDir);
            newDirF = new File(newDir);
        } while (newDirF.exists());
        // stdlog_.debug("making directory: " + newDirF);
        newDirF.mkdir();
        return newDirF;
    }


    /**
     * Ensure that directory structure is exactly 1 level deep
     * and the first directory is "[Instrument]-[version]/"
     *
     * @param filename  filename to be checked
     * @return returns position of the file separator character
     */
    private int checkDirStructure(String filename) throws ZipUtilsException {
        int slashPos = 0;
        
        // default separator
        String currSep = sep;
        for (int i = 0; i < sepList.length; i++) {
        slashPos = filename.indexOf(sepList[i]);
        
        // found separator used.
         if (slashPos > 0) {
             currSep = sepList[i]; 
             break;
         }
        }
        // if separator not found set to def length
        if (slashPos < 0){
            slashPos = 0;
        }
        if (dirStructureOk_ && (slashPos != filename.lastIndexOf(currSep))) {
            dirStructureOk_ = false;
            err("Illegal directory structure in zip file "+ filename);
        }

        if (dir1NameOk_ && !filename.startsWith(dir1_ + currSep)) {
            stdlog_.error("dir1_=" + dir1_);
            stdlog_.error("file=" + filename);
            dir1NameOk_ = false;
            err("Illegal name for first level directory: " +
                filename.substring(0, slashPos) +
                "\nFirst level directory should be called: " + dir1_ +
                "\nfor all 'tsf' and 'isf' files in this Instrument Package");
        }

        return slashPos;
    }


    /**
     * Writes a ZipEntry to the filesystem
     *
     * @param zf    zipfile in which zipEntry is located
     * @param ze    entry in the zipfile to write
     * @return returns full pathname of file written
     */
    public String writeZipEntryToFile(ZipFile zf, ZipEntry ze)
    throws ZipUtilsException {

        // get the contents of ZipEntry and write to tmpFile
        String tmpFile = tmpDirL1_ + sep + ze.getName();
        // stdlog_.debug("writeZipEntryToFile: " + tmpFile);
        try {
            // File.createTempFile(prefix, "tsf", "");
            BufferedWriter buffW = new BufferedWriter(new FileWriter(tmpFile));
            InputStream is = zf.getInputStream(ze);
            BufferedInputStream bis = new BufferedInputStream(is);
            int aByte, count = 0;
            while ((aByte = bis.read()) != -1) {
                buffW.write(aByte);
                count++;
            }
            bis.close();
            buffW.close();
            stdlog_.debug("wrote " + count + " bytes to " + tmpFile);
        } catch (IOException e) {
            deleteFile(tmpFile);
            err("File IO error: " + e.getMessage());
        }
        return tmpFile;
    }


    /**
     * delete a file
     *
     * @param file_ file to delete
     */
    public void deleteFile(String file_) {
        File f = new File(file_);
        if (f.exists()) {
            // stdlog_.debug("deleting " + file_);
            if (!f.delete()) {
                stdlog_.error("file IO error: can't delete " + file_);
            }
        } else {
            stdlog_.debug("can't delete: file doesn't exist: " + file_);
        }
    }


    /**
     * removes temporary directories created while verifying the zip file
     */
    public void removeTmpDirs() {
        delTree(tmpDirL1_);
    }


    /**
     * Recursively removes the specified directory and all of its subdirectories
     * @param file The file or directory that you wish to delete
     */
    private void delTree(File file) {
        if (file == null) {
            return;
        } else if (file.isDirectory()) {
            stdlog_.debug("deleting directory: " + file.toString());
            File[] dir = file.listFiles();
            for (int i=0; i < dir.length; i++) {
                delTree(dir[i]);
            }
            file.delete();
        } else if (file.exists()) {
            file.delete();
        }
    }


    /**
     * called when an error in the IP is found
     * either clean up files/directories and quit by throwing ZipUtilsException
     * or add to the report_ and allow verification to continue
     *
     * @param message   error message
     */
    private void err(String message) throws ZipUtilsException {

        stdlog_.error(message);
        report_ += message + "\n\n\n";
        if (!makeReport_) {
            removeTmpDirs();
            throw new ZipUtilsException(message);
        }
    }


    /**
     * called from setTmpDir
     *
     * @param message   error message
     */
    private static void err1(String message) throws ZipUtilsException {
        stdlog_.error(message);
        throw new ZipUtilsException(message);
    }


    /**
     * allow the base temp directory to be set
     */
    public static void setTmpDir(String newDir) throws ZipUtilsException {

        if (newDir == null || newDir.trim().equals("")) {
            err1("Folder name was not given");
        }

        newDir = newDir.trim();
        if (!newDir.endsWith(sep)) {
            newDir += sep;
        }

        File f = new File(newDir);
        if (!f.exists()) {
            err1("Temp folder [" + newDir + "] does not exist");
        }

        // try to create a file/dir
        String dirName = newDir + "IPacMan-temp-test-dir";
        stdlog_.debug("creating & deleting directory: " + dirName);
        File testDir = new File(dirName);
        if (testDir.mkdir() && testDir.delete()) {
            stdlog_.debug(dirName + " created & deleted ok");
        } else {
            stdlog_.error("failed to create and delete " + dirName);
            err1("Could not create files in temp folder " + newDir);
        }

        baseTmpDir_ = newDir;
        stdlog_.debug("baseTmpDir_=[" + baseTmpDir_ + "]");
    }
}

