package org.eso.ohs.persistence;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.DefaultGTreeModel;
import org.eso.ohs.core.utilities.DefaultGTreePath;
import org.eso.ohs.core.utilities.Encrypt;
import org.eso.ohs.core.utilities.GTreeNode;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjConfig;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;

/**
 * Application Configuration query singleton.
 * This class is the class to be used for most OHS applications. If this
 * class is not appropriate, utilities/OHSConfig (this classes bas class)
 * should be used.

 * This class provides and easy interface to most of the OHS application
 * config query needs (based on p2pp and ot) as well as provides an
 * interface for adding config data.
 * <br>ChangeLog: cmavros 2002/02/11, 1512/P2PP, set EngModeCloningKey, add log4j
 * <br>ChangeLog: cmavros 20JUNE02 [1652/P2PP] Read proxy settings.
 * <br>ChangeLog: cmavros 04JULY02 [2.4Beta/OT] Add getOptionDir() 
 */

public class AppConfig extends OHSConfig {

    protected static Logger stdlog_ = Logger.getLogger(AppConfig.class);

    /**
     * Constructor.
     */
    public AppConfig() throws IOException {
    }

    /**
     * Set the standard OHS keys.
     * 
     * @param cDir	directory containing the config files
     * @param hDir	home directory
     * @param startAppServer	configure app server start setting
     */
    public void setStandardOHSKeys(String cDir) {
        setStandardOHSKeys(cDir, System.getProperty("user.home"), false);
    }

    public void setStandardOHSKeys(String cDir, boolean startAppServer) {
        setStandardOHSKeys(
            cDir,
            System.getProperty("user.home"),
            startAppServer);
    }

    public void setStandardOHSKeys(String cDir, String hDir) {
        setStandardOHSKeys(cDir, hDir, false);
    }

    public void setStandardOHSKeys(
        String cDir,
        String hDir,
        boolean startAppServer) {
        // Set Configuration defaults.

        setValue(OHSKeys.ConfigDirKey, cDir);
        setValue(OHSKeys.HomeDirKey, hDir);
        setValue(OHSKeys.BaseDirKey, System.getProperty("user.dir"));
        setValue(OHSKeys.StartAppServerKey, startAppServer);

        setValue(
            OHSKeys.InstFolderKey,
            (new File("instruments")).getAbsolutePath());
        setValue(OHSKeys.ExpFolderKey, (new File("impex")).getAbsolutePath());
		setValue(OHSKeys.FCFolderKey, (new File(System.getProperty("user.dir")).getAbsolutePath()));
        setValue(OHSKeys.CacheFolderKey, (new File("cache")).getAbsolutePath());
        
        // dump the contenys of a queue
        setValue(
            OHSKeys.DumpFolderKey,
            (new File(System.getProperty("user.dir")).getAbsolutePath()));
        // Save param files out in p2pp and also save adp/adm data out 
        // in ot.     
        setValue(
            OHSKeys.ParamFolderKey,
            (new File(System.getProperty("user.dir")).getAbsolutePath()));
            
        // This is defaulted to user home if no option exists as well.
        // This is abadly named as it is only used for MaskTracker.
        // where it savs orders and reports out.     
        
            
        setValue(OHSKeys.PafFolderKey,(new File(System.getProperty("user.home")).getAbsolutePath()));
        setValue(
            OHSKeys.AppServerLogKey,
            (new File("logs/appserver.log")).getAbsolutePath());

        // Persistence settings.
        setValue(OHSKeys.MaxNameLengthKey, Config.MAX_NAME_LENGTH);        
        setValue(OHSKeys.MaxTextLengthKey, Config.MAX_TEXT_LENGTH);

        setValue(OHSKeys.EngModeCloningKey, false);
        
	setValue(OHSKeys.FCNotesEnabledKey, false);
        setValue(OHSKeys.ReadmeP2PPEnabledKey, true);
                
        setValue(OHSKeys.FCScaledKey,false);



        // MIPS setting: the period from which Multiple IP data is expected.
        setValue(OHSKeys.IPVersioningStartPeriodKey, "71");
        setValue(OHSKeys.IPVersioningKey, "y");
        setValue(OHSKeys.MaxFindingChartKey, "4");
        setValue(OHSKeys.MaxFindingChartSize, "1000000");
        setValue(OHSKeys.ObsLongKey, "1.228768");
        setValue(OHSKeys.ObsLatKey, "-0.429802");
    }

    /**
     * @return the singleton appconfig, if it is an AppConfig object
     * and has been set. <BR>
     * <strong>Note</strong>: should be depricated and getInstance()
     * should be used.
     */
    public static AppConfig getAppConfig() {
        OHSConfig cfg = getInstance();
        AppConfig rc = null;
        if (cfg instanceof AppConfig) {
            rc = (AppConfig) cfg;
        }
        return rc;
    }

    /**
     * <strong>Note</strong>: should be depricated and getInstance()
     * should be used.
     */
    public static void setAppConfig(AppConfig cfg) {
        setInstance(cfg);
    }

    public void setVersion(String v) {
        if (v != null) {
            setValue(OHSKeys.VersionKey, v);
        }
    }

    public String getVersion() {
        return getValue(OHSKeys.VersionKey);
    }

    public int getPersistenceMaxNameLength() {
        return getInteger(OHSKeys.MaxNameLengthKey);
    }

    public int getPersistenceMaxTextLength() {
        return getInteger(OHSKeys.MaxTextLengthKey);
    }

    public double getLongitude() {
        return Double.valueOf(getValue(OHSKeys.ObsLongKey)).doubleValue();
    }

    public double getLatitude() {
        return Double.valueOf(getValue(OHSKeys.ObsLatKey)).doubleValue();
    }

    // this is never used
//    public void setSuperUserEntitled(boolean entitled) {
//        setValue(OHSKeys.SuperUserKey, entitled);
//    }
//
//    public boolean isSuperUserEntitled() {
//        return getBoolean(OHSKeys.SuperUserKey);
//    }

    public boolean isIPVersioningEnabled() {
        return getBoolean(OHSKeys.IPVersioningKey);
    }

    public boolean isIPVersioningSingleIPMode() {
        return getBoolean(OHSKeys.IPVersioningSingleIPModeEnabled);
    }

    public int getIPVersionStartPeriod() {
        return getInteger(OHSKeys.IPVersioningStartPeriodKey);
    }
    
    
    /**
     * Returns the engineering role name from configuration
     * 
     * @return a String
     */
    public String getEngineeringRoleName() {
    	String engineeringRoleName = getValue(OHSKeys.engineeringRoleName);
    	if (engineeringRoleName == null || engineeringRoleName.length() == 0) {
			throw new IllegalStateException("No value defined for keyword: " + OHSKeys.engineeringRoleName);
		}
    	return(engineeringRoleName);
    }
    
    /**
     * Returns the operations role name from configuration
     * 
     * @return a String 
     */
    public String getOperationsRoleName() {
    	String operationsRoleName = getValue(OHSKeys.operationsRoleName);
    	if (operationsRoleName == null || operationsRoleName.length() == 0) {
			throw new IllegalStateException("No value defined for keyword: " + OHSKeys.operationsRoleName);
		}
    	return(operationsRoleName);    	    	
    	
    }

    
    
    
    public int getMaxFindingCharts() {
        int max = -1;
        try {
            max = getInteger(OHSKeys.MaxFindingChartKey);
        } catch (NumberFormatException e) {
            max = 4;

        }
        return max;
    }

    public int getMaxFindingChartSize() {
        return getInteger(OHSKeys.MaxFindingChartSize);

    }
    


    public String getIPVersionIPCache() {
        return prependBase(OHSKeys.IPVersioningIPCache);
    }

    public boolean getIPVersioningAutodownloadIPEnabled() {
        return getBoolean(OHSKeys.IPVersioningAutodownloadIP);
    }

    public boolean isReadmeEnabled() {
        return getBoolean(OHSKeys.ReadmeP2PPEnabledKey);
    }

    public void setCcsActive(boolean isActive) {
        setValue(OHSKeys.CcsActiveKey, isActive);
    }

    public boolean isCcsActive() {
        return getBoolean(OHSKeys.CcsActiveKey);
    }

    public void setEngModeClone(boolean isActive) {
        setValue(OHSKeys.EngModeCloningKey, isActive);
    }

    public boolean isCloneModeId() {
        return getBoolean(OHSKeys.EngModeCloningKey);
    }

    public boolean shouldUseAppServer() {
        return (getValue(OHSKeys.P2PPServerUrl).length() > 0);
    }

    public String getAppServerUrl() {
        return getValue(OHSKeys.P2PPServerUrl);
    }

    public boolean getCCSEnabled() {
        return getBoolean(OHSKeys.CcsConnEnabledKey);
    }
    
	public boolean getFCNotesEnabled() {
		return getBoolean(OHSKeys.FCNotesEnabledKey);
	}
    
    
    public boolean getUSGNotesEnabled() {
        return getBoolean(OHSKeys.USGEnabledKey);
    }

    
    public boolean isFCScaled() {
        return getBoolean(OHSKeys.FCScaledKey);
    }



    public String getCCSServer() {
        return getValue(OHSKeys.CcsServerNameKey);
    }

    public String getDbUserName() {
        //return getUserName(OHSKeys.DBDirectSubConfig);
        return getUserName("");
    }

    public String getDbPassword() {
        // return getPassword(OHSKeys.DBDirectSubConfig);
        return getPassword("");
    }

    public String getDbServer() {
        //return getServer(OHSKeys.DBDirectSubConfig);
        return getServer("");
    }

    public String getUrl(String key) {
        //return getUrl(OHSKeys.DBDirectSubConfig, key);
        return getUrl("", key);
    }

    public String getDbName(String key) {
        //return getDbName(OHSKeys.DBDirectSubConfig, key); 
        return getDbName("", key);
    }

    public Class[] getDisplayedObjectTypes() {
        return new Class[] { ObservationBlock.class, CalibrationBlock.class };
    }

    public boolean hasConnectInfo() {
        //return hasConnectInfo(OHSKeys.DBDirectSubConfig);
        return hasConnectInfo("");
    }

    public void putObjCfg(ObjConfig oc) {
        Hashtable<Class, ObjConfig> objCfg = getObjCfg();
        if (objCfg != null) {
            objCfg.put(oc.key, oc);
        } else {
            throw new IllegalArgumentException(
                "ObjCfg at " + OHSKeys.BusObjConfigKey + " not found");
        }
    }

    public Hashtable<Class, ObjConfig> getObjCfg() {
        Hashtable<Class, ObjConfig> objCfg = (Hashtable<Class, ObjConfig>) get(OHSKeys.BusObjConfigKey);
        if (objCfg == null) {
            objCfg = new Hashtable<Class, ObjConfig>();
            set(OHSKeys.BusObjConfigKey, objCfg);
        }
        return objCfg;
    }

    public ObjConfig getObjCfg(Class objType) {
        Hashtable<Class, ObjConfig> objCfg = getObjCfg();
        if (objCfg != null) {
            ObjConfig oc = objCfg.get(objType);
            if (oc != null) {
                return oc;
            }
        }
        throw new IllegalArgumentException("getObjCfg: " + objType);
    }

    public String[] folderColumnNames(Class objType) {
        return getObjCfg(objType).columnNames;
    }

    public String[] folderColumnProperties(Class objType) {
        return getObjCfg(objType).propertyNames;
    }

    public boolean isEditableProperty(Class cls, String fieldName) {
        String[] editableFields = getObjCfg(cls).editableProperties;

        if (editableFields != null) {
            for (int j = 0; j < editableFields.length; j++) {
                if (editableFields[j].equals(fieldName)) {
                    return true;
                }
            }
        }

        return false;
    }

    public boolean hasDbCfg() {
        return getDbCfg().exists();
    } 
    public File getDbCfg() {
        String filename =
            new String(OHSApplication.getShortName() + "-db.cf").toLowerCase();
        return new File(getConfigDir() + File.separator + filename);
    }

    public boolean dbCfgCanRead() {
        return getDbCfg().canRead();
    }

    public boolean getStartAppServer() {
        return getBoolean(OHSKeys.StartAppServerKey);
    }

    public String getOptionDir() {
        return prependBase(OHSKeys.OptionFolderKey);
    }

    public String getInstrumDir() {
        return prependBase(OHSKeys.InstFolderKey);
    }

    public String getExpDir() {
        return prependBase(OHSKeys.ExpFolderKey);
    }
    
    // Finding charts can be saved out to this 
    // directory
	public String getFcDir() {
		return prependBase(OHSKeys.FCFolderKey);
	}
    
    public String getSmtpName() {
        return getValue(OHSKeys.MailHostKey);
    }
    
    public String getP2PPSubmit() {
        return getValue(OHSKeys.P2PPSubmitKey);
    }



    public String getCacheDir() {
        return prependBase(OHSKeys.CacheFolderKey);
    }

    public String getDumpDir() {
        return prependBase(OHSKeys.DumpFolderKey);
    }

    public String getPafDirectory() {
        return prependBase(OHSKeys.PafFolderKey);
    }

    public String getAppServerName() {
        return getValue(OHSKeys.AppServerNameKey);
    }

    public String getAppServerPort() {
        return getValue(OHSKeys.AppServerPortKey);
    }

    public String getAppServerHttpProxyHost() {
        return getValue(OHSKeys.HttpProxyHostKey);
    }

    public String getAppServerHttpProxyPort() {
        return getValue(OHSKeys.HttpProxyPortKey);
    }

    public String getAppServerDataPort() {
        return getValue(OHSKeys.AppServerDataPortKey);
    }

    public String getAppServerLog() {
        return prependBase(OHSKeys.AppServerLogKey);
    }

    public String getParameterDir() {
        return prependBase(OHSKeys.ParamFolderKey);
    }

    /**
     * Get a keyword from the configuration
     * @param the name of the keyword
     * @return value matching the keyword
     * This method has been included for backwards compatibility
     * with the previous AppConfig interface.
     */
    public String getPropertyValue(String s) {
        stdlog_.debug("apps.AppConfig  VALUE for: " + s + " :: " + getValue(s));
        stdlog_.debug("apps.AppConfig  TREE:\n" + toString());
        return getValue(s);
    }

// pnunes  - not used anymore    
//    /**
//     * Is the given user id an "Operations Mode" id?
//     * 
//     * @return true if it is an operations mode id, eg user 0
//     */
//    public boolean isOperationsModeId(int uid) {
//        return uid <= 10;
//    }
//
//    /**
//     * Is the given user id an "Engineering Mode" id?
//     * 
//     * @return true if it is an engineering mode id
//     */
//    public boolean isEngineeringModeId(int uid) {
//        return uid > 50000;
//    }

    

    /**
     * This method must only be used for running a unit test.
     * Underlying Problem: When running the unit test DriverAppConfig it compares
     * what it found as a valid cache directory with what AppConfig finds.
     * Unfortunatelly the makefile before cd's to $DATAFLOW_JAVA while this 
     * is using $HOME if it finds a private .p2pp.cf file here.
     * The purpose of setHomeDir is to set any other dir which has no
     * .p2pp.cf file. Then AppConfig will choose the default, which is
     * assumed when running the test. 
     */

    public void setHomeDir(String new_home_dir) {
        setValue(OHSKeys.HomeDirKey, new_home_dir);
    }

    /**
     * Read in the specified PafFile and place all data under the
     * specifieid subCfg. The PafFile is assumed to be the DbDirect
     * configuration parameters, and any encrypted entries (user and pw)
     * will be decypted tp test their validity.
     */
    public void setupDbDirectConfig(String subCfg, File f) throws IOException {
        if (f != null) {
            setValue(subCfg + ".CFGFILE", f.getName());
            PafFile pafFile = new PafFile(f.getAbsolutePath());
            if (pafFile.compileError()) {
                throw new IOException(
                    "File "
                        + f.getName()
                        + " : "
                        + pafFile.getCompileErrorMsg());
            }
            addPafData(pafFile, subCfg);
        }

        // test the validity of the encrypted entries, if they exist.
        String userName = getValue(subCfg + ".ENCRYPTED.DATABASE.USERID");
        if (userName.length() > 0 && decrypt(userName) == null) {
            throw new IOException(
                "Encrypted key ENCRYPTED.DATABASE.USERID "
                    + " in config file "
                    + f.getName()
                    + " is corrupted. ");
        }
        String password = getValue(subCfg + ".ENCRYPTED.DATABASE.USERID");
        if (password.length() > 0 && decrypt(password) == null) {
            throw new IOException(
                "Encrypted key ENCRYPTED.DATABASE.USERID "
                    + " in config file "
                    + f.getName()
                    + " is corrupted. ");
        }
    }

    /**
     * Private decrypt routing.
     * @return null if the decryption fails.
     */
    private static String decrypt(String val) {
        String rc = null;
        try {
            rc = Encrypt.deCrypt(val);
        } catch (IllegalArgumentException e) {
            rc = null;
        }
        return rc;
    }

    /**
     * Get the DbDirect user name from the specified sub config.
     * The entry will be decrypted if it is encrypted.
     * @return the user name, or null if not found or not decryptable.
     */
    protected String getUserName(String subCfg) {
        String userKey = "DATABASE.USERID";
        String rc = getValue(subCfg + ".ENCRYPTED." + userKey);
        if (rc.length() > 0) {
            rc = decrypt(rc);
        } else {
            rc = null;
        }

        // failed getting an encrypted user name, try a plain text one.
        if (rc == null) {
            rc = getValue(subCfg + "." + userKey);
            if (rc.length() == 0) {
                rc = null;
            }
        }
        return rc;
    }

    /**
     * Get the DbDirect password from the specified sub config.
     * The entry will be decrypted if it is encrypted.
     * @return the password, or null if not found or not decryptable.
     */
    protected String getPassword(String subCfg) {
        String pwKey = "DATABASE.PASSWORD";
        String rc = getValue(subCfg + ".ENCRYPTED." + pwKey);
        if (rc.length() > 0) {
            rc = decrypt(rc);
        } else {
            rc = null;
        }

        // failed getting an encrypted user name, try a plain text one.
        if (rc == null) {
            rc = getValue(subCfg + "." + pwKey);
            if (rc.length() == 0) {
                rc = null;
            }
        }
        return rc;
    }

    /**
     * @return the URL for a given database, or null if not found.
     * For example: "OR" may return "jdbc:sybase:Tds:opc63"
     * Note that a search for an excrypted entry will also be made.
     */
    protected String getUrl(String subCfg, String key) {
        String rc = getDbName(key);
        if (rc != null && rc.length() > 0) {
            String server = getServer(subCfg);
            // NOTE: no check for valid server value performed!
            // This may be required...but will break the unit tests...
            rc = "jdbc:sybase:Tds:" + server + "/" + rc;
        } else {
            rc = null;
        }
        return rc;
    }

    /**
     * @return the database for a given table key, or null if not found.
     * For example "OR" may return "PHASE2.DATABASE".
     */
    protected String getDbName(String subCfg, String key) {
        String rc = "";
        if (key != null) {
        key = "DATABASE." + key.toUpperCase();
        rc = getValue(subCfg + "." + key);
        if (rc.length() == 0) {
            rc = decrypt(getValue(subCfg + ".ENCRYPTED." + key));
        }
        if (rc != null && rc.length() > 0) {
            rc = getValue(subCfg + "." + rc);
        }
        }
        return rc;
    }

    /**
     * @return the database server JDBC url.
     */
    protected String getServer(String subCfg) {
        String rc = getValue(subCfg + ".DATABASE.SERVER");
        if (rc.length() == 0) {
            rc = null;
        }
        return rc;
    }

    /**
     * @return true if this DB Direct config has enough info to 
     * connect to the database.
     */
    public boolean hasConnectInfo(String subCfg) {

        if (!shouldUseAppServer()) {
            if (getUserName(subCfg) == null || getPassword(subCfg) == null) {
                return false;
            }
        }

        // The following info is required independant of appserver
        // or direct connection.
        GTreeNode busObj = getNode(subCfg + ".DATABASE");

        // Test if there is an entry for each business object.
        for (Enumeration e = busObj.getChildren(); e.hasMoreElements();) {
            GTreeNode child = (GTreeNode) e.nextElement();
            if (child.getName().equals("SERVER")) {
                // not a business object
                continue;
            }
            if (getUrl(child.getName()) == null) {
                // no database for this object - error
                return false;
            }
        }

        // Must have more than 1 children: the server name (SERVER)
        // and at least one table key (OD, TP, USR, etc).

        if (busObj == null || busObj.getChildCount() < 2) {
            return false;
        }

        return true;
    }

    /**
     * Process all entries in under the OHSKeys.UserOverrideKey
     * node as follows, for all leaf nodes:
     *        - if the path ends with FOLDER, leave as they are.
     *        - if not, move it into the root directory, to add or
     *          overwrite existing configuration settings.
     */
    public void processOverrides() {
        OHSConfig overrideConfig = getSubConfig(OHSKeys.UserOverrideKey);
        if (overrideConfig != null) {
            // perform a depth first enumeration of the chilren, processing the leaves
            processOverrides(
                overrideConfig.getTree().getRoot(),
                (DefaultGTreeModel) overrideConfig.getTree());
        }
    }

    private void processOverrides(
        GTreeNode node,
        DefaultGTreeModel overrideConfig) {
        if (node.isLeaf()) {
            if (node.getName().equals(OHSKeys.FolderKey)) {
                // leave as is, do not override config setting
            } else {
                // copy into main config and overwrite/append to settings there.                
                GTreeNode[] overridePathNodes =
                    overrideConfig.getPathToRoot(node);
                GTreeNode[] pathNodes =
                    new GTreeNode[overridePathNodes.length - 1];
                System.arraycopy(
                    overridePathNodes,
                    1,
                    pathNodes,
                    0,
                    pathNodes.length);
                DefaultGTreePath path = new DefaultGTreePath(pathNodes);
                setValue(tree_, path, node.getValue());
            }
        } else {
            Enumeration children = node.getChildren();
            while (children.hasMoreElements()) {
                processOverrides(
                    (GTreeNode) children.nextElement(),
                    overrideConfig);
            }
        }
    }

    /**
     * @return a Vector of String objects contining the names of all
     * instruments for which VM OBs may be checked into the repository.
     * The OHSKeys.VMCheckInOk key (INSTRUMENTS.VMCHECKINOK) is the base
     * of a numeric series of config keys read from the config set.
     * The set is read numerically and in order and the first key which is
     * missing or empty terminates the sequence. That is, in the following:
     * INSTRUMENTS.VMCHECKINOK.1 "VIMOS"
     * INSTRUMENTS.VMCHECKINOK.2 "FORS1"
     * INSTRUMENTS.VMCHECKINOK.4 "ISAAC"
     * Only a Vector of 2 String objects: "VIMOS" and "FORS1" would be returned.
     * 
     * Returns an empty Vector if no keys were found.
     */
    public Vector<String> getInstrumentsWithVMCheckInOk() {
        Vector<String> rc = new Vector<String>();
        int idx = 1;
        do {
            String key = OHSKeys.VMCheckInOk + "." + idx;
            String val = this.getValue(key);
            if (val.length() > 0) {
                rc.addElement(val);
            } else {
                break;
            }
            idx++;
        } while (true);
        return rc;
    }
    
    /**
     * @return a Vector of String objects contining the names of all
     * instruments for which OBS have sidereal time.
     * Returns an empty Vector if no keys were found.
     */
    public Vector<String> getInstrumentsWithST() {
        Vector<String> rc = new Vector<String>();
        int idx = 1;
        do {
            String key = OHSKeys.STOk + "." + idx;
            String val = this.getValue(key);
            if (val.length() > 0) {
                rc.addElement(val);
            } else {
                break;
            }
            idx++;
        } while (true);
        return rc;
    }
    
    public Vector<String> getAllowNoCheckSum() {
        Vector<String> rc = new Vector<String>();
        int idx = 1;
        do {
            String key = OHSKeys.CheckSumEnabled + "." + idx;
            String val = this.getValue(key);
            if (val.length() > 0) {
                rc.addElement(val);
            } else {
                break;
            }
            idx++;
        } while (true);
        return rc;
    }


    /**
     * @return true if the given instrument is in the list of
     * instruments for which VM OBs may be checked into the repository.
     * See getInstrumentsWithVMCheckInOk() for details.
     */
    public boolean canCheckInVMOBs(String instName) {
        boolean rc = false;
        if (instName == null || instName.length() == 0) {
            return false;
        }
        Vector<String> list = getInstrumentsWithVMCheckInOk();
        for (Enumeration<String> e = list.elements(); e.hasMoreElements();) {
            String inst = e.nextElement();
            if (instName.equalsIgnoreCase(inst)) {
                rc = true;
                break;
            }
        }
        return rc;
    }
    
    /**
     * Returns whethet an instrument has a siderel time or
     * not from site.cf
     * @param instName
     * @return boolean
     */
    
    
    public boolean hasSTTime(String instName) {
        boolean rc = false;
        if (instName == null || instName.length() == 0) {
            return false;
        }
        Vector<String> list = getInstrumentsWithST();
        for (Enumeration<String> e = list.elements(); e.hasMoreElements();) {
            String inst = e.nextElement();
            if (instName.equalsIgnoreCase(inst)) {
                rc = true;
                break;
            }
        }
        return rc;
    }

}
    
