package org.eso.ohs.persistence;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.AccountInfo;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.instruments.TemplateVersionInfo;

/**
 * A StorageManager class to authenticate the user to another
 * (possibly remote) StorageManager.  This class is intended to form
 * a matching pair with ProtectingStorageMgr.  ProtectingStorageMgr
 * requires it's caller to be authenticated and throws an
 * ObjectAuthorisationException if it is not, whereas
 * AuthenticatingStorageMgr catches the exception, authenticates
 * the user and retries the previously aborted call.
 * 
 * <p>
 * The two classes are typically used either as
 * 
 * <pre>
 *    StorageManager stMgr = new AuthenticatingStorageMgr(
 * 		new ProtectingStorageMgr(
 * 			someStorageMgr
 * 		)
 *    );
 * </pre>
 * 
 * or with one at each end of an RMI connection between a client
 * program and the application server.
 * 
 * <p>
 * $Id: AuthenticatingStorageMgr.java,v 1.27.12.5 2007/11/06 14:18:50 pnunes Exp $
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP AppServer.
 *
 * 
 * @see ProtectingStorageMgr
 * @see StorageManager
 */
public class AuthenticatingStorageMgr implements StorageManager {

    /**
     * The StorageManager object to which all operations should be delegated.
    */
    private StorageManager stMgr_;

    public AuthenticatingStorageMgr(StorageManager stMgr) {
        stMgr_ = stMgr;
    }

    public void setAccountId(int userId) throws ObjectIOException {
        stMgr_.setAccountId(userId);
    }

    public int getAccountId() {
        return stMgr_.getAccountId();
    }

    /**
     * Get the root node of the DirectoryNode tree representing
     * the directories found on this medium.  For example, a
     * representation of the root of the local StorableObject
     * storage subtree.
     * 
     * @return a representation of the root node
     * 
     * @exception ObjectIOException
     *			if there was an I/O error while determining the
     * 			directory hierarchy.
     */
    public DirectoryNode getRoot() throws ObjectIOException {
        return stMgr_.getRoot();
    }

    /**
     * Verify that the TemplateSignature files used by the StorageManager
     * are the same version as those specified (and used by the client)
     * 
     * @param tsList	array of version info objects for each template
     *			used in an OB
     * 
     * @return a list of the names of TemplateSignatures that are out of date.
     * 
     * @exception ObjectIOException
     *			if there was an I/O error while checking
     */
    public String[] verifyTemplateSignatures(TemplateVersionInfo[] tsList)
        throws ObjectIOException {
        return stMgr_.verifyTemplateSignatures(tsList);
    }

    /**
     * @return the requested IP as a byte array, or an empty (size 0) array
     * if the IP does not exist on the server.
     * @exception ObjectIOException when there is an I/O exception.
     */
    public byte[] downloadInstrumentPackage(String instrument, float version)
        throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.downloadInstrumentPackage(instrument, version);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    public FindingChart[] populateFindingCharts(long boId)
        throws ObjectIOException {
        return stMgr_.populateFindingCharts(boId);
    }

    /**
     * Checkin the readme file for a given observing run
     * @param Readme file business object
     */

    public void checkInReadme(ObservingRun obsRun) throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.checkInReadme(obsRun);
                return; 
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Checkout the readme file for a given observing run.
     * @param boId Observing run definition. 
     */

    public String checkOutReadme(ObservingRun obsRun)
        throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.checkOutReadme(obsRun);

            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }

    }

    /**
     * Get the type of media maintained by this StorageManager instance.
     * 
     * @return an instance of the Media class.
     */
    public Media getMedia() {
        return stMgr_.getMedia();
    }

    /**
     * Calculates a unique id for a StorableObject.
     *
     * @return Unique id
     * 
     * @exception ObjectIOException
     *			if there was an I/O error writing the object.
     */
    public long getNewId(Class cls, long baseId) throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.getNewId(cls, baseId);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Initialise the data structures for the appropiate media. For 
     * local storage this will be a directory structure.
     * 
     * @exception ObjectIOException
     *			if there was an I/O error writing the object.
     */
    public void initialise() throws ObjectIOException {
        stMgr_.initialise();
    }

    /**
     * Authenticates the end-user.
     * 
     * @param password	password for the current user
     * 
     * @return	true if user was successfully authenticated, false otherwise
     * 
     * @exception ObjectIOException
     *			if there was an I/O error while authenticating
     */
    public boolean loginUser(String password) throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.loginUser(password);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Returns a DirectoryNode representing the path within the
     * storage medium of an object with the given id and type.
     * For media that do not support directories, the root directory
     * may be returned.
     * 
     * @param id	the id of the StorableObject.
     * @param cls	the type of stored StorableObject.
     * 
     * @return Directory in which to find the object.
     * 
     * @exception ObjectIOException
     *			if there was an I/O error writing the object.
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found.
     */
    public DirectoryNode find(long id, Class objType)
        throws ObjectNotFoundException, ObjectIOException {
        for (;;) {
            try {
                return stMgr_.find(id, objType);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Returns an array of objects representing the contents of an
     * object with the given id and type from the chosen directory.
     * If the medium does not support directories, the <code>node</code>
     * parameter is ignored.
     * 
     * <p>
     * The array of objects contains:
     * <ul>
     *     <li> as the first (0th) element, an instance of a
     *          StorableObject subclass; followed by
     *     <li> an alternating sequence of values:
     *          <ul>
     *              <li> at even indexes, a property name, ie a string X
     *                   such that setX() and getX() methods exist.
     *              <li> at odd indexes, either an instance of
     *                   java.lang.Long containing the the id of the
     *                   StorableObject subclass to which the property
     *                   should refer or a reference to the subclass itself.
     *          </ul>
     * </ul>
     * 
     * @param node	directory from which the object should be loaded.
     * @param id	id of the StorableObject.
     * @param objType	type of StorableObject.
     * 
     * @return the array of objects and object references,
     * 			as described above.
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found.
     * @exception ObjectIOException
     * 			if an I/O error occurred while reading the object
     * 
     * @see #write
     * @see #update
     */
    public UnpackedStorableObject read(
        DirectoryNode node,
        long id,
        Class objType)
        throws ObjectNotFoundException, ObjectIOException {
        for (;;) {
            try {
                return stMgr_.read(node, id, objType);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Writes an array of objects with the given id and type to
     * this storage medium, in the chosen directory.  If the medium
     * does not support directories, the <code>node</code> parameter
     * is ignored.
     * 
     * @param node	the directory in which the object should be saved.
     * @param id	the id under which to store the StorableObject.
     * @param objType	the type of StorableObject.
     * @param obj	the array of objects and object references to store,
     *			as described in the section for the read method.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while writing the object
     * 
     * @see #read
     * @see #update
     */
    public void write(
        DirectoryNode node,
        long id,
        Class objType,
        UnpackedStorableObject obj)
        throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.write(node, id, objType, obj);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Write several object to the external media in an atomic operation.
     *      
     * @param moi	Array of object descriptions.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while writing the object
     * 
     * @see #read
     * @see #write
     * @see #update
     */
    public void multiStore(MultiObjSpec[] mos) throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.multiStore(mos);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Updates an array of objects with the given id and type on
     * this storage medium, in the chosen directory.  If the medium
     * does not support directories, the <code>node</code> parameter
     * is ignored.
     * 
     * @param node	the directory in which the object should be saved.
     * @param id	the id under which to store the StorableObject.
     * @param objType	the type of StorableObject.
     * @param obj	the array of objects and object references to store,
     *			as described in the section for the read method.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while updating the object
     * 
     * @see #read
     * @see #write
     */
    public void update(
        DirectoryNode node,
        long id,
        Class objType,
        UnpackedStorableObject obj)
        throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.update(node, id, objType, obj);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Deletes an object with the given directory, id and type from
     * this storage medium.  If the medium does not support
     * directories, the <code>node</code> parameter is ignored.
     * 
     * @param node	directory from which the object should be deleted.
     * @param id	id of the StorableObject.
     * @param objType	type of StorableObject.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while updating the object
     * 
     * @see #read
     * @see #update
     */
    public void delete(DirectoryNode node, long id, Class objType)
        throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.delete(node, id, objType);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Moves an object between directories on this storage medium.
     * 
     * @param id	id of the StorableObject.
     * @param cls	type of StorableObject.
     * @param fromNode	directory from which the object should be moved.
     * @param toNode	directory to which the object should be moved.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while updating the object
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found in fromNode.
     */
    public void move(
        long id,
        Class objType,
        DirectoryNode fromNode,
        DirectoryNode toNode)
        throws ObjectNotFoundException, ObjectIOException {
        for (;;) {
            try {
                stMgr_.move(id, objType, fromNode, toNode);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Create a directory in local storage
     *
     * @param node The directory to be created
     * @param name The name of the directory to be created.
     * 
     * @return True on success false on error
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     */
    public boolean createDirectory(DirectoryNode node, long id)
        throws ObjectIOException {

        for (;;) {
            try {
                return stMgr_.createDirectory(node, id);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Move a directory.
     *
     * @param node	directory to be moved
     * @param toNode	the new parent directory
     * 
     * @return True on success false on error
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found in fromNode.
     */
    public boolean moveDirectory(DirectoryNode fromNode, DirectoryNode toNode)
        throws ObjectIOException, ObjectNotFoundException {

        for (;;) {
            try {
                return stMgr_.moveDirectory(fromNode, toNode);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Delete a directory from local storeage
     *      
     * @param node	Directory to be deleted
     * 
     * @return True on success, false on failure
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     */
    public boolean deleteDirectory(DirectoryNode node)
        throws ObjectIOException {

        for (;;) {
            try {
                return stMgr_.deleteDirectory(node);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Returns a list of Summary objects associated with a directory.
     * 
     * @param node	Directory to be used to build up list of objects
     * 
     * @return Array of Summary[] objects describing the directory contents
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     */
    public Summary[] listObjects(DirectoryNode node) throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.listObjects(node);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Do a EVM verification test for the specified objects in a single atomic operation.
     *
     * @param   usos    Descriptions of each StorableObject to do EVM verification
     *
     * @return boolean if OB passed EVM test
     */
    public boolean verifyObject(MultiObjSpec[] moi) throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.verifyObject(moi);
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Check in the specified objects in a single atomic operation.
     * 
     * @param	usos	Descriptions of each StorableObject to be checked in.
     * 
     * @return		array of error strings.
     *			If length = 0, no errors.
     */
    public void checkIn(MultiObjSpec[] moi) throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.checkIn(moi);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Check out the ObsBlk or CalBlk with the specified id plus
     * any attached objects, ie Target, ObsDescr, etc.
     * 
     * @param	id	id of the ObsBlk or CalBlk to be checked out
     * 
     * @return		CheckOutStatus object that contains a return
     *			status and (if successful) references to the
     *			checked-out objects.
     */
    public void checkOut(IdType[] idTypes)
        throws ObjectNotFoundException, ObjectIOException {
        for (;;) {
            try {
                stMgr_.checkOut(idTypes);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /**
     * Authenticates the user for operations through the delegated
     * StorageManager.  
     * 
     * @param node Directory to be used to build up list of objects
     * @return An array of ObjectFileInfo[] objects 
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     * 			or the user did not authenticate themselves
     */
    protected void authenticateUser(Exception e) throws ObjectIOException {
        // e.printStackTrace();
        Config.getCfg().askForAuthentication(stMgr_.getMedia());
    }

    /* (non-Javadoc)
     * @see org.eso.ohs.persistence.StorageManager#usdSubmit(org.eso.ohs.dfs.ObservingRun)
     */
    public void usdSubmit(ObservingRun obsRun) throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.usdSubmit(obsRun);
                return;
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.eso.ohs.persistence.StorageManager#populateReadmeFile(org.eso.ohs.dfs.ObservingRun)
     */
    public Readme populateReadmeFile(ObservingRun obsRun) throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.populateReadmeFile(obsRun); 
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }

    }

    /* (non-Javadoc)
     * @see org.eso.ohs.persistence.StorageManager#setReadmeFileStatus(java.lang.String, long)
     */
    public void setReadmeFileStatus(String status, long runId) throws ObjectIOException {
        for (;;) {
            try {
                stMgr_.setReadmeFileStatus(status,runId); 
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }

    }

    /* (non-Javadoc)
     * @see org.eso.ohs.persistence.StorageManager#populateReadmeFileReplicate(org.eso.ohs.dfs.ObservingRun)
     */
    public Readme populateReadmeFileReplicate(ObservingRun obsRun) throws ObjectIOException {
        for (;;) {
            try {
                return stMgr_.populateReadmeFileReplicate(obsRun); 
            } catch (ObjectAuthorisationException e) {
                authenticateUser(e);
            }
        }

    }

	

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.StorageManager#getAccountInfo(java.lang.String, java.lang.String)
	 */
	public AccountInfo getAccountInfo(String username, String password)
			throws ObjectIOException {
		for (;;) {
			try {                    
				return stMgr_.getAccountInfo(username,password);	
			} catch (ObjectAuthorisationException e) {
				authenticateUser(e);
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.StorageManager#getAccountInfo(int, java.lang.String)
	 */
	public AccountInfo getAccountInfo(int accountId, String password)
			throws ObjectIOException {
		for (;;) {
			try {                    
				return stMgr_.getAccountInfo(accountId,password);	
			} catch (ObjectAuthorisationException e) {
				authenticateUser(e);
			}
		}
	}
	
	

	

}
