package org.eso.ohs.persistence;

import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.AccountInfo;

/**
 * Clearing house for low-level configuration information.
 * All methods are static, use them directly not via an instance.
 * 
 * <p>
 * This class is a little strange: the methods are static but the data
 * is held in a singleton object.  This gives convenience (less to type
 * when calling) but means that we can also serialize it to disk and
 * inheritance still works.
 * 
 * <p>
 * Since deserialization doesn't call the constructors, the default
 * initialisation is not performed when reading from disk.
 * 
 * <p>
 * $Id: Config.java,v 1.57.4.3 2007/10/11 08:28:23 pnunes Exp $
 */

public abstract class Config {
    protected final static int MAX_OBJECT_TYPES = 100;

    // Persistence
    protected static Config config_ = null;
    
    // current account being used
    private AccountInfo accountInfo;

    // A M Chavan, 17-Jan-2000
    private boolean debugMode_ = false;

    /** Maximum number of chars in a BusinessObject's name */
    public static final int MAX_NAME_LENGTH = 32;

    /** Maximum number of chars in a UserComments field, etc */
    public static final int MAX_TEXT_LENGTH = 240;

    /** Per-objectType config info */
    protected Hashtable<Class, ObjConfigPersistence> objCfgPersistence = new Hashtable<Class, ObjConfigPersistence>();

	

    protected Config() {
        // A M Chavan, 17-Jan-2000; Bob Kemp, 16-Jan-2000
        debugMode_ =
            new Boolean(System.getProperty("debug.mode")).booleanValue();

        initPersistenceObjInfo();
    }
    
	/** Standard log channel */
		private static Logger stdlog_ = Logger.getLogger(Config.class);

    /**
     * A reference to a config file
     * @return Config file reference
     */

    public static Config getCfg() {
        return config_;
    }

    /**
     * Set the configuration file for this class 
     * @param cfg Refernce to config file
     *
     */

    public static void setCfg(Config cfg) {
        config_ = cfg;
    }
    
    public static void reset() {
        config_ = null;
        
    }

    /**
     * Get the ObjConfigPersistence class given a Class Object
     * @param objType Object Type class
     * @return ObjConfigPersistence  class
     */

    private ObjConfigPersistence getObjCfgPersistence(Class objType) {
        ObjConfigPersistence ocp =
            objCfgPersistence.get(objType);

        if (ocp != null) {
            return ocp;
        }

        Enumeration<Class> enum0 = objCfgPersistence.keys();
        // 	while (enum.hasMoreElements()) {
        // 	    System.out.println("Key: " + enum.nextElement());
        // 	}

        throw new IllegalArgumentException("getObjCfg: " + objType);
    }

    /**
     * Set user name and password that the user used to log in with
     * int userId User id login
     */
    public void setAccount(AccountInfo info) {
        this.accountInfo = info;
    }

    /**
     * Get the username.
     * @return User name
     *
     */

    public int getAccountId() {
        return accountInfo.getAccountId();
    }
    

    /**
     * Get the password for the user
     * @return Users password
     *
     */

    public String getPassword() {
        return accountInfo.getPasswd();
    }

    /**
     * Returns a class with the associated a file that has been
     * saved on disk 
     * @param suffix .or .tp .od etc..
     * @return Class associated with a given business object suffix
     */

    public Class getClassForFileSuffix(String suffix) {
        Enumeration<Class> enum0 = objCfgPersistence.keys();
        ObjConfigPersistence ocp;

        while (enum0.hasMoreElements()) {
            ocp =
                objCfgPersistence.get(
                    enum0.nextElement());
            // if(ocp == null)
            // System.out.println("ocp == null");
            // System.out.println("Suffix = " + suffix);
            // System.out.println("OPc = " + ocp.suffix);
            if (ocp.suffix.equals(suffix)) {
                return ocp.key;
            }
        }

        return null;
    }

    public Media getMediaFromId(long id) {
        if (id < 0)
            return Media.LOCAL;
        return Media.DBASE;
    }












    /**
     * This method takes the system wide id for a given object i.e. 
     * and then returns the class associated with this object.
     * A Special version of this method was need to deal with dbase ids 
     * less than 10.
     * @param id system wide id for a given business object
     * @return class associated with a business object
     */

    public Class getClassFromQueueId(long id) {
        Enumeration<Class> enum0 = objCfgPersistence.keys();
        ObjConfigPersistence ocp;
        Class rc = null;
        long tableId = uniqueToTableId(id);
        if (tableId != 0) {             
            if(tableId < 10 && tableId > 0) {
                tableId  = tableId * 100;
            }
            int index = Math.abs((int) (id % tableId));
         
            while (enum0.hasMoreElements()) {
                ocp =
                    objCfgPersistence.get(
                        enum0.nextElement()); 
                if (ocp.index == index) {
                    return ocp.key;
                }
            }
        }
        return null;
    }

    /**
     * This method takes the system wide id for a given object i.e. 
     * and then returns the class associated with this object.
     * @param id system wide id for a given business object
     * @return class associated with a business object
     */

    public Class getClassFromId(long id) {
        Enumeration<Class> enum0 = objCfgPersistence.keys();
        ObjConfigPersistence ocp;
        Class rc = null;
        long tableId = uniqueToTableId(id);
        if (tableId != 0) {

            int index = Math.abs((int) (id % tableId));
   
            while (enum0.hasMoreElements()) {
                ocp =
                    objCfgPersistence.get(
                        enum0.nextElement());
                if (ocp.index == index) {
                    return ocp.key;
                }
            }
        }
        return null;
    }

    /**
     * Convert a database per-table id into a unique id that we can
     * use within the program.
     * 
     * @param id    	per-table id straight from the database.
     * @param objType	type of the object
     * @return		a unique id
     * 
     * @see #uniqueToTableId
     */
    public long tableToUniqueId(long id, Class objType) {
        if (id == 0 || id == -1) {
            return 0;
        }

        return (id > 0) ? transformId(id, objType) : -transformId(-id, objType);
    }

    private long transformId(long id, Class objType) {
        return MAX_OBJECT_TYPES * id + getObjCfgPersistence(objType).index;
    }

    /**
     * Convert a unique id into a per-table id that we can
     * store in the database.
     * 
     * @param id    	a unique id produced by tableToUniqueId()
     * @param objType	type of the object
     * @return		per-table id that can be stored in the database.
     * 
     * @see #tableToUniqueId
     */
    public long uniqueToTableId(long id) {
    	
    	//stdlog_.debug("--------------------> unique id: " + id);
    	long tableId = id == 0 || id == -1 ? 0 : id / MAX_OBJECT_TYPES;
		//stdlog_.debug("--------------------> max object types: " + MAX_OBJECT_TYPES);
		//stdlog_.debug("--------------------> table id: " + tableId);
        return  tableId;
    }
    
    
    /**
     * Are we in "Visitor Mode"?
     * 
     * @return true if we are running in engineering mode.
     */
    public boolean isVisitorModeId(int uid) {
    	if (accountInfo.getAccountId() != uid)
    		throw new IllegalStateException("Wrong user id: " + uid + " Current logged-in user is: " + accountInfo.getAccountId());
    	
        return !isInOperationsMode()
            && !isInEngineeringMode()
            && isCcsActive();
    }

    private boolean isCcsActive() {
		return AppConfig.getAppConfig().isCcsActive();
	}

	/**
     * Are we in "Standard Mode"?
     * In other words, Service Mode or preparation of VisitorMode OBs.
     * 
     * @return true if we are running in standard mode.
     */
    public boolean isStandardModeId(int uid) {
    	if (accountInfo.getAccountId() != uid)
    		throw new IllegalStateException("Wrong user id: " + uid + " Current logged-in user is: " + accountInfo.getAccountId());
        return !isInOperationsMode()
            && !isInEngineeringMode()
            && !isCcsActive();
    }
    
    
    /**
     * <br>ChangeLog: A M Chavan, 17-Jan-2000
     */
    public boolean isDebugMode() {
        return debugMode_;
    }

    /**
     * Are we in "Operations Mode"?
     * 
     * @return true if we are running in operations mode, ie as user 0
     */
    public boolean isInOperationsMode() {
        return accountInfo.getOperations();
    }

    /**
     * Are we in "Engineering Mode"?
     * 
     * @return true if we are running in engineering mode.
     */
    public boolean isInEngineeringMode() {
        return accountInfo.getEngineering();
    }

    /**
     * Am I a "Super User"?
     * 
     * @return true if I'm a super user
     */
    public boolean isSuperUser() {
        return false;
    }

    /**
     * Are we in "Visitor Mode"?
     * 
     * @return true if we are running in visitor mode.
     */
    public boolean isInVisitorMode() {
        return isVisitorModeId(accountInfo.getAccountId());
    }

    /**
     * Is cloning selected?
     * 
     * @return true if we have cloning selected.
     */
    public boolean isInCloneMode() {
        return isCloneModeId();
    }

    /**
     * Are we in "Standard Mode"?
     * In other words, Service Mode or preparation of VisitorMode OBs.
     * 
     * @return true if we are running in engineering mode.
     */
    public boolean isInStandardMode() {
        return isStandardModeId(accountInfo.getAccountId());
    }

    /**
     * The properties that occur in the FoderGridView 
     * @param objType A refernce to a class object
     */

    public abstract String[] folderColumnProperties(Class objType);

    /**
     * Database user name 
     * @return Db user name opc,
     *
     */

    public abstract String getDbUserName();

    /**
     * Database password
     * @return Database password
     *
     */

    public abstract String getDbPassword();

    /**
     * Get JDBC url
     * @param key 
     * @return JDBC URL for a given database
     */
    public abstract String getUrl(String key);

    /**
     * Get database name 
     * @param key
     * @return Database name
     */

    public abstract String getDbName(String key);





    /**
     * Did the user select cloning?
     *
     * @return true if cloning was selected
    */
    public abstract boolean isCloneModeId();


    /**
     * What is the first period for which IP versioning info
     * is expected in the data. 0 indicates no start period, no
     * version info is expected.
     */
    public abstract int getStartPeriodForIPVersioning();

    /**
     * Authentication mechanism
     * @param device Media to save data DB,LOCAL STORAGE
     */

    public abstract void askForAuthentication(Media device)
        throws ObjectIOException;

    public abstract boolean isUserLevel(Class cls);

    public abstract void initPersistenceObjInfo();

    //public abstract void alertUser(Exception exc);

    /**
     * Get the Maximum Length of a Text Field in P2PP
     * @return Maximum Length
     */

    public int getMaxNameLength() {
        return MAX_NAME_LENGTH;
    }

    /**
     * Get the Maximum Length of a Text Area in P2PP
     * @return Maximum Length
     */

    public int getMaxTextLength() {
        return MAX_TEXT_LENGTH;
    }



    /**
     * Get summary of database tables
     * @param objType ObjType to be saved
     * @return A String that refers to a database table for the summary object
     */
    public String summaryDbaseTables(Class objType) {
        return getObjCfgPersistence(objType).dbaseTables;
    }

    /**
       * Get summary of database tables
       * @param objType ObjType to be saved
       * @return An array of string that refer to Dbase columns
       */

    public String[] summaryDbaseColumns(Class objType) {
        return getObjCfgPersistence(objType).dbaseColumns;
    }

    public static class ObjConfigPersistence {
        public Class key; // compulsory field

        /** used by DbaseMgr.listObjects() to construct Summary fields */
        public String dbaseTables;

        /** used by DbaseMgr.listObjects() to construct Summary fields */
        public String[] dbaseColumns;

        public String suffix;

        public int index;

        public ObjConfigPersistence(
            Class key,
            String[] dbaseColumns,
            String dbaseTables,
            String suffix,
            int index) {
            this.key = key;
            this.dbaseTables = dbaseTables;
            this.dbaseColumns = dbaseColumns;
            this.suffix = suffix;
            this.index = index;

        }
    }
    
    /*
     * @return true if the given instrument is in the list of
     * instruments for which VM OBs may be checked into the repository.
     * See AppConfig.getInstrumentsWithVMCheckInOk() for details.
     */
    public boolean isVMOBCheckinAllowed(String inst) {
        return false;
    }

	/**
	 * @return
	 */
	public String getUsername() {
		return accountInfo.getUsername();
	}

	/**
	 * @param passwd
	 */
	public void setPassword(String passwd) {
		accountInfo.setPasswd(passwd);
		
	}
	

}
