package org.eso.ohs.persistence;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;

/**
 * retrieves data from DB hiding the connection operations.
 * The connection info are retrieved from Config class.
 * Config class is always subclassed by an ohs application.
 * In the subclass are stored the connection informations.
 * <p>
 * $Id: DBdataHelper.java,v 1.45.4.1 2007/10/24 11:32:09 pnunes Exp $
 *
 * <br>
 * @author D. Dorigo
 *
 */
public class DBdataHelper {

	/**
	 * Field ZIP_EXTENSION
	 */
	public static final String ZIP_EXTENSION = ".zip";

	/**
	 * Field url
	 */
	private String url = null;
	/**
	 * Field userName
	 */
	private String userName = null;
	/**
	 * Field passwd
	 */
	private String passwd = null;
	/**
	 * Field stdlog_
	 */
	private static Logger stdlog_ = Logger.getLogger(DBdataHelper.class);

	/**
	 * Field BUFFERSIZE
	 */
	private static final int BUFFERSIZE = (1024 * 1024);
	/**
	 * Field singleton
	 */
	private static DBdataHelper singleton = null;

    private static boolean logEnabled_ = false;

    private Level logLevel_ = null;

	/** Does nothing */
	public DBdataHelper() {
	}

	/** 
	 * 
	 * @param url String
	 * @param userName String
	 * @param passwd String
	 */
	public DBdataHelper(String url, String userName, String passwd) {
		initializeConnection(url, userName, passwd);
	}

	/**
	 * Method initializeConnection variables. 
	 * By default they are taken from Config.getCfg()
	 * @param url String
	 * @param userName String
	 * @param passwd String
	 */
	public void initializeConnection(
		String url,
		String userName,
		String passwd) {
		this.url = url;
		this.userName = userName;
		this.passwd = passwd;
	}

	/**
	 * Method getInstance
	 * @return DBdataHelper
	 */
	public static DBdataHelper getInstance() {
		if (singleton == null) {
			singleton = new DBdataHelper();
		}
		return singleton;
	}

	/**
	 * Returns the cached JDBC database Connection object creating
	 * it first, if necessary. It gets the connection info from Config
	 * class.
	 *
	 * @return  JDBC Connection object
	 *
	 * @exception ObjectIOException
	 *		    an error occured during an access to a business object
	 * @exception SQLException
	 *		    an error occurred during the dbase operation
	 */
	public Connection acquireConnection()
		throws ObjectIOException, SQLException {

		if (url == null || userName == null || passwd == null) {
			Config cfg = Config.getCfg();
			if (cfg == null) {
				throw new ObjectIOException("configuration object not initialized");
			}
			url = cfg.getUrl("OR");
			userName = cfg.getDbUserName();
			passwd = cfg.getDbPassword();
            stdlog_.debug("URL: "+ url+" username: "+userName);
           System.out.println("URL: "+ url+" username: "+userName);
		}

		return ConnectionPool.getPool().acquireConnection(
			url,
			userName,
			passwd);
	}

	/**
	 * Returns a File zipped in the DB.
	 * 
	 *
	 * @param filename File
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return The inputstream
	 *
	 * @exception ObjectIOException
	 *			if the object was not found.
	 *
	 */
	public File getBinaryFile(File filename, String selectClause)
		throws ObjectIOException {
		InputStream stream = null;
		Connection con = null;
		Statement stmt = null;
		ObjectIOException exception = null;
		File file = null;
		if ( filename == null) {
			throw new ObjectIOException ("file class is null");
		}
		try {
			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			file = getZippedFile(filename, selectClause, stmt);
			getStdlog().debug("rs closed");
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error(
				"IllegalArgumentException error Sql statement: " + selectClause,
				e);
			exception = new  ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error("IOException filename : " + filename.getPath(), e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);	
			if (exception != null) {
				throw exception;
			}
		}
		return file;
	}

	/**
	 * Returns a File in the DB.
	 * 
	 *
	 * @param filename File
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return The inputstream
	 *
	 * @exception ObjectIOException
	 *			if the object was not found.
	 *
	 */
	public File getFile(File filename, String selectClause)
		throws ObjectIOException {
		InputStream stream = null;
		Connection con = null;
		Statement stmt = null;
		File file = null;
		ObjectIOException exception = null;
		try {
			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			file = getFile(filename, selectClause, stmt);
			getStdlog().debug("rs closed");
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error(
				"IllegalArgumentException error Sql statement: " + selectClause,
				e);
			exception = new  ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error("IOException filename : " + filename.getPath(), e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
		
		return file;
	}

	/**
	 * Method getFile retrieve a  file into the DB as it is
	 * @param filename File
	 * @param selectClause String
	 * @param stmt Statement
	 * @return File
	 * @throws SQLException
	 * @throws ObjectIOException
	 * @throws IOException
	 */
	public File getFile(File filename, String selectClause, Statement stmt)
		throws SQLException, ObjectIOException, IOException {
		InputStream stream;
		int fieldId = 1;
		File file = filename;
		ResultSet rs = execQuery(selectClause, stmt);
		if (!rs.next()) {
			getStdlog().error("Sql statement error: " + selectClause);
			throw new ObjectIOException("No data retreived from select");
		}
		// get data
		stream = rs.getBinaryStream(fieldId);
		FileOutputStream fos;
		byte[] buffer = new byte[BUFFERSIZE];
		int readCount = 0;
		fos = new FileOutputStream(file);
		if (stream != null) {
			while ((readCount = stream.read(buffer)) != -1) {
				fos.write(buffer, 0, readCount);
			}
			stream.close();
		}
		fos.flush();
		fos.close();
		rs.close();
		return file;
	}
	/**
	 * Method getZippedFile retrieve a zipped file into the DB as unzipped
	 * @param filename File
	 * @param selectClause String
	 * @param stmt Statement
	 * @return File
	 * @throws SQLException
	 * @throws ObjectIOException
	 * @throws IOException
	 */
	public File getZippedFile(
		File filename,
		String selectClause,
		Statement stmt)
		throws SQLException, ObjectIOException, IOException {
		InputStream stream;
		int fieldId = 1;
		File file;
		ResultSet rs = execQuery(selectClause, stmt);
		if (!rs.next()) {
			getStdlog().error("Sql statement error: " + selectClause);
			throw new ObjectIOException("No data retreived from select");
		}
		// get data
		stream = rs.getBinaryStream(fieldId);
		file = unzip(filename, stream);
		// this sometimes is needed
		stream.close();
		rs.close();
		return file;
	}
	/**
	 * Returns an input stream from which the binary data could be get.
	 * The input stream must be used and closed before getting other
	 * data from the result set.
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return The inputstream
	 *
	 * @exception ObjectIOException
	 *			if the object was not found.
	 *
	 */
	public InputStream getBinaryStream(String selectClause)
		throws ObjectIOException {

		int fieldId = 1;
		InputStream stream = null;
		Connection con = null;
		Statement stmt = null;
		ResultSet rs = null;
		File file = null;
		ObjectIOException exception = null;
		try {
			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			execQuery(selectClause, stmt);
			if (!rs.next()) {
				getStdlog().error("Sql statement error: " + selectClause);
				throw new ObjectIOException("No data retreived from select");
			}
			// get data
			stream = rs.getBinaryStream(fieldId);
			// this sometimes is needed
			rs.close();
			getStdlog().debug("rs closed");
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error(
				"IllegalArgumentException error Sql statement: " + selectClause,
				e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
		
		return stream;
	}

	/**
	 *Update a binary zipped stream in the database.
	 *
	 * @param file retriever of a list of objects.
	 * @param updateClause String
	 * @exception ObjectIOException
	 *			if the object was not found.
	 */
	public void updateBinaryZippedStream(File file, String updateClause)
		throws ObjectIOException {

		int filesize = (int) file.length();
		byte[] fileData = new byte[filesize];
		ObjectIOException exception = null;
		// read the file into filedata
		try {
			DataInputStream inputStream =
				new DataInputStream(new FileInputStream(file));

			inputStream.readFully(fileData);
			inputStream.close();
		} catch (FileNotFoundException e) {
			getStdlog().error("File not found : " + file.getName(), e);
			throw new  ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error("IO error : " + file.getName(), e);
			throw new  ObjectIOException(e);
		}
	
		String zippedFileName = file.getName();
		File zippedFile = new File(zippedFileName + ZIP_EXTENSION);

		// write filedata to zippedfile
		try {
			FileOutputStream bufferOutput = new FileOutputStream(zippedFile);
			GZIPOutputStream gzos = new GZIPOutputStream(bufferOutput);

			gzos.write(fileData, 0, fileData.length);
			gzos.flush();
			gzos.close();
			bufferOutput.close();

		} catch (FileNotFoundException e) {
			getStdlog().error("File not found : " + file.getName(), e);
			throw new ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error("IO error : " + file.getName(), e);
			throw new ObjectIOException(e);
		}

		// update data  
		updateBinaryStream(zippedFile, updateClause);

		zippedFile.delete();

	}
	/**
	 * Method updateBinaryStream
	 * @param file File
	 * @param updateClause String
	 * @throws ObjectIOException
	 */
	public void updateBinaryStream(File file, String updateClause)
		throws ObjectIOException {

		int filesize = (int) file.length();
		String fileName = file.getName();
		Connection connection = null;
		PreparedStatement prepStatement = null;
		DataInputStream binaryStream = null;
		ObjectIOException exception = null;
		//Read the file
		try {
			binaryStream = new DataInputStream(new FileInputStream(file));
		} catch (FileNotFoundException e) {
			getStdlog().error("FileNotFoundException filename : " + fileName, e);
			throw new ObjectIOException(e);
		}

		getStdlog().debug(updateClause);

		try {
			// initialize the connection
			connection = acquireConnection();
			//statement = con.createStatement();
			// insert binary in the database
			prepStatement = connection.prepareStatement(updateClause);
			prepStatement.setBinaryStream(1, binaryStream, filesize);
			prepStatement.execute();
			connection.commit();
			binaryStream.close();
			//prepStatement.close();
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + updateClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error(
				"IllegalArgumentException error Sql statement: " + updateClause,
				e);
			exception = new  ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error(
				"IOLException error Sql statement: " + updateClause,
				e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(connection, prepStatement);
			if (exception != null) {
				throw exception;
			}
		}
		
	}

	/**
	 *Insert a binary stream in the database.
	 *
	 * @param file retriever of a list of objects.
	 * @param insertClause String
	 * @param connection Connection
	 * @exception ObjectIOException
	 *			if the object was not found.
	 *
	 * @throws SQLException
	 */
	public void insertBinaryZippedFile(
		File file,
		String insertClause,
		Connection connection)
		throws ObjectIOException, SQLException, IOException {
		getStdlog().debug(
			"inserting file: "
				+ file.getAbsolutePath()
				+ "\n Statement: "
				+ insertClause);
		// insert binary in the database
		PreparedStatement prepStatement =
			connection.prepareStatement(insertClause);
		insertBinaryZippedFile(file, prepStatement);
		prepStatement.close();

	}
	/**
	 *Insert a binary stream in the database.
	 *
	 * @param file retriever of a list of objects.
	 * @param prepStatement String
	 * @exception ObjectIOException
	 *			if the object was not found.
	 *
	 * @throws SQLException
	 */
	protected void insertBinaryZippedFile(
		File file,
		PreparedStatement prepStatement)
		throws ObjectIOException, SQLException, IOException {

		File zippedFile = convertToZip(file);
		int filesize = (int) zippedFile.length();
		String fileName = zippedFile.getAbsolutePath();
		DataInputStream binaryStream = null;
		ObjectIOException exception = null;
		//Read the file
		try {
			binaryStream = new DataInputStream(new FileInputStream(fileName));
		} catch (FileNotFoundException e) {
			getStdlog().error("FileNotFoundException filename : " + fileName, e);
			throw new ObjectIOException(e);
		}

		prepStatement.setBinaryStream(1, binaryStream, filesize);
		prepStatement.execute();
		binaryStream.close();
		zippedFile.delete();

	}

	/**
	 *Insert a binary stream in the database.
	 *
	 * @param file retriever of a list of objects.
	 * @param prepStatement String
	 * @exception ObjectIOException
	 *			if the object was not found.
	 *
	 * @throws SQLException
	 */
	protected void insertBinaryFile(File file, PreparedStatement prepStatement)
		throws ObjectIOException, SQLException, IOException {

		int filesize = (int) file.length();
		String fileName = file.getAbsolutePath();
		DataInputStream binaryStream = null;
		ObjectIOException exception = null;
		//Read the file
		try {
			binaryStream = new DataInputStream(new FileInputStream(fileName));
		} catch (FileNotFoundException e) {
			getStdlog().error("FileNotFoundException filename : " + fileName, e);
			throw new ObjectIOException(e);
		}
		if (filesize > 0) {
			prepStatement.setBinaryStream(1, binaryStream, filesize);
			prepStatement.execute();
		}
		binaryStream.close();
	}

	/**
	 *Insert a binary stream in the database.
	 *
	 * @param file retriever of a list of objects.
	 * @param insertClause String
	 * @exception ObjectIOException
	 *			if the object was not found.
	 *
	 */
	public void insertBinaryZippedFile(File file, String insertClause)
		throws ObjectIOException {

		File zippedFile = convertToZip(file);
		insertBinaryFile(zippedFile, insertClause);
		zippedFile.delete();

	}

	/**
	 * Method convertToZip
	 * @param file File
	 * @return File
	 * @throws ObjectIOException
	 */
	private File convertToZip(File file) throws ObjectIOException {
		int filesize = (int) file.length();
		byte[] fileData = new byte[filesize];

		try {
			DataInputStream inputStream =
				new DataInputStream(new FileInputStream(file));

			inputStream.readFully(fileData);
			inputStream.close();
		} catch (FileNotFoundException e) {
			getStdlog().error("File not found : " + file.getName(), e);
			throw new ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error("IO error : " + file.getName(), e);
			throw new ObjectIOException(e);
		}
		String zippedFileName = file.getAbsolutePath();
		File zippedFile = new File(zippedFileName + ZIP_EXTENSION);

		try {
			FileOutputStream bufferOutput = new FileOutputStream(zippedFile);
			GZIPOutputStream gzos = new GZIPOutputStream(bufferOutput);

			gzos.write(fileData, 0, fileData.length);
			gzos.flush();
			gzos.close();
			bufferOutput.close();

		} catch (FileNotFoundException e) {
			getStdlog().error("File not found : " + file.getName(), e);
			throw new ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error("IO error : " + file.getName(), e);
			throw new ObjectIOException(e);
		}
		return zippedFile;
	}

	/**
	 * Method insertBinaryStream
	 * @param file File
	 * @param insertClause String
	 * @throws ObjectIOException
	 */
	public void insertBinaryFile(File file, String insertClause)
		throws ObjectIOException {

		Connection connection = null;
		DataInputStream binaryStream = null;

		getStdlog().debug(insertClause);
		ObjectIOException exception = null;
		try {
			// initialize the connection
			connection = acquireConnection();
			//statement = con.createStatement();
			// insert binary in the database
			insertBinaryFile(file, insertClause, connection);
			connection.commit();

			//prepStatement.close();
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + insertClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error(
				"IllegalArgumentException error Sql statement: " + insertClause,
				e);
			exception = new  ObjectIOException(e);
		} catch (IOException e) {
			getStdlog().error(
				"IOLException error Sql statement: " + insertClause,
				e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(connection);
			if (exception != null) {
				throw exception;
			}
		}
	
	}

	public void insertBinaryFile(
		File file,
		String insertClause,
		Connection connection)
		throws SQLException, ObjectIOException, IOException {
		getStdlog().debug(
			"inserting file: "
				+ file.getAbsolutePath()
				+ "\n Statement: "
				+ insertClause);
		// insert binary in the database
		PreparedStatement prepStatement =
			connection.prepareStatement(insertClause);
		insertBinaryFile(file, prepStatement);
		prepStatement.close();
	}

	// unzip data and writes to file unzipped.
	/**
	 * Method unzip
	 * @param file File
	 * @param is InputStream
	 * @return File
	 * @throws IOException
	 */
	static private File unzip(File file, InputStream is) throws IOException {
		FileOutputStream fos;
		GZIPInputStream zis = new GZIPInputStream(is);
		byte[] buffer = new byte[BUFFERSIZE];
		int readCount = 0;

		getStdlog().debug("unzip start ");
		fos = new FileOutputStream(file);
		while ((readCount = zis.read(buffer)) != -1) {
			fos.write(buffer, 0, readCount);
		}
		fos.flush();
		fos.close();
		zis.close();
		getStdlog().debug("unzip end  " + file.getAbsolutePath());
		return file;
	}

    /**
     * Returns an array of String from the select clause.
     * If data are not found, a 0 length array is returned.
     *
     * @param selectClause  retriever of a list of objects.
     *
     * @return the array of objects
     *
     * @exception ObjectIOException
     *          If sql error.
     */
    public String[] readStringArray(String selectClause)
        throws ObjectIOException {

        String[] usoArray = new String[0];
        Connection con = null;
        Statement stmt = null;

        ObjectIOException exception = null;
        try {
            // initialize the connection
            con = acquireConnection();

            stmt = con.createStatement();
            usoArray = readStringArray(selectClause, stmt);
        } catch (SQLException e) {
            getStdlog().error("Sql statement error: " + selectClause, e);
            exception = new  ObjectIOException(e);
        } catch (IllegalArgumentException e) {
            getStdlog().error(
                "IllegalArgumentException error Sql statement: " + selectClause,
                e);
            exception = new  ObjectIOException(e);
        } finally {
            freeConnection(con, stmt);
            if (exception != null) {
                throw exception;
            }
        }
        return usoArray;
    }

	/**
	 * Returns an array of String[][] from the select clause.
	 * If data are not found, a 0 length array is returned. 
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return the array of objects
	 *
	 * @exception ObjectIOException
	 *			If sql error.
	 */
	public String[][] readDoubleStringArray(String selectClause)
		throws ObjectIOException {

		String[][] usoArray = new String[0][2];
		Connection con = null;
		Statement stmt = null;

		ObjectIOException exception = null;
		try {
			// initialize the connection
			con = acquireConnection();

			stmt = con.createStatement();
			usoArray = readDoubleStringArray(selectClause, stmt);
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error(
				"IllegalArgumentException error Sql statement: " + selectClause,
				e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
		return usoArray;
	}

    /**
     * Method readStringArray
     * @param selectClause String
     * @param stmt Statement
     * @return String[]
     * @throws SQLException
     */
    public String[] readStringArray(String selectClause, Statement stmt)
        throws SQLException {
        String[] usoArray= new String[0];
        ResultSet rs;
        int fieldId = 1;
        Vector<String> result = new Vector<String>();
        // execute query
        rs = execQuery(selectClause, stmt);
        // get data
        while (rs.next()) {
            result.addElement(rs.getString(fieldId));
        }
        rs.close();
        usoArray = new String[result.size()];
        result.copyInto(usoArray);
        return usoArray;
    } 
    /**
     * Method readDoubleStringArray
     * @param selectClause String
     * @param stmt Statement
     * @return String[][] the first is the number of rows, the second is of two elements
     * @throws SQLException
     */
    public String[][] readDoubleStringArray(String selectClause, Statement stmt)
        throws SQLException {
        String[][] usoArray= new String[0][0];
        ResultSet rs;
        int firstFieldId = 1;
        int secondFieldId = 2;
        Vector<String> resultFirst = new Vector<String>();
        Vector<String> resultSecond = new Vector<String>();
         // execute query
        rs = execQuery(selectClause, stmt);
        // get data
        while (rs.next()) {
            resultFirst.addElement(rs.getString(firstFieldId));
            resultSecond.addElement(rs.getString(secondFieldId));
              }
        rs.close();
        usoArray = new String[resultFirst.size()][2];
        for (int i = 0; i < resultFirst.size();i++) {
            usoArray[i][0] = resultFirst.elementAt(i);
            usoArray[i][1] = resultSecond.elementAt(i);
        }
        return usoArray;
    }

	/**
	 * Method execQuery
	 * @param selectClause String
	 * @param stmt Statement
	 * @return ResultSet
	 * @throws SQLException
	 */
	public ResultSet execQuery(String selectClause, Statement stmt)
		throws SQLException {
		ResultSet rs;
	
		getStdlog().debug(selectClause);
		rs = stmt.executeQuery(selectClause);
		return rs;
	}

	// releases the connection.
	/**
	 * Method freeConnection
	 * @param con Connection
	 * @param stmt Statement
	 * @throws ObjectIOException
	 */

	public void freeConnection(Connection con, Statement stmt) {
		getStdlog().debug(" freeConnection");
		if (con != null) {
			try {
				if (stmt != null) {
					stmt.close();
				}
			} catch (SQLException e) {
				getStdlog().error("SQLException ", e);
			} finally {
				try {
					releaseConnection(con);
				} catch (ObjectIOException e1) {
					getStdlog().error("Error while releasing connection: ", e1);
				}
			}
		}
	}
	/**
	 * Method freeConnection
	 * @param con Connection
	 * @param stmt Statement
	 * @throws ObjectIOException
	 */

	private void freeConnection(Connection con) {
		getStdlog().debug(" freeConnection");
		if (con != null) {
			try {
				releaseConnection(con);
			} catch (ObjectIOException e1) {
				getStdlog().error("Error while releasing connection: ", e1);
			}
		}
	}
	/**
	* Method setAutoCommitAndFreeConnection
	* set autocommit to true and release the connection
	* param con Connection
	* * @param con Connection
	@param stmt Statement
	* @throws ObjectIOException
			*/
	public void setAutoCommitAndFreeConnection(Connection con, Statement stmt)
		throws ObjectIOException {
		if (con != null) {
			try {
				con.setAutoCommit(true);
			} catch (SQLException e) {
				getStdlog().error(e);
			}
		}
		freeConnection(con, stmt);
		
		
		
	}

	/**
	 * Method rollbackConnection
	 * @param conn Connection
	 */
	public void rollbackConnection(Connection conn) {

		try {
			if (conn != null) {
				conn.rollback();
			}
		} catch (SQLException e1) {
			getStdlog().error("rollback failed: ", e1);
		}
	}

	/**
	 * Returns an array of Integer objects.
	 * If data are not found, a 0 length array is returned.
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return the array of objects
	 *
	 * @exception ObjectIOException
	 *			sql error.
	 */
	public Long[] readLongObjArray(String selectClause)
		throws ObjectIOException {

		long[] intArray = readLongArray(selectClause);
		Long[] intObjArray = new Long[intArray.length];
		for (int i = 0; i < intArray.length; i++) {
			intObjArray[i] = new Long(intArray[i]);
		}
		return intObjArray;
	}

	/**
	 * Returns an array of Integer objects.
	 * If data are not found, a 0 length array is returned.
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return the array of objects
	 *
	 * @exception ObjectIOException
	 *			sql error.
	 */
	public Integer[] readIntObjArray(String selectClause)
		throws ObjectIOException {

		int[] intArray = readIntArray(selectClause);
		Integer[] intObjArray = new Integer[intArray.length];
		for (int i = 0; i < intArray.length; i++) {
			intObjArray[i] = new Integer(intArray[i]);
		}
		return intObjArray;
	}

	/**
	 * Returns an array of int representing the contents
	 * If data are not found, a 0 length array is returned.
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return the array of objects
	 *
	 * @exception ObjectIOException
	 *			sql exception
	 *
	 */
	public int[] readIntArray(String selectClause) throws ObjectIOException {

		int fieldId = 1;
		Connection con = null;
		Statement stmt = null;
		Vector<Integer> result = new Vector<Integer>();
		ResultSet rs = null;
		ObjectIOException exception = null;
		int[] intArray = new int[0];
		try {
			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			// execute query
			rs = execQuery(selectClause, stmt);
			// get data
			while (rs.next()) {
				result.addElement(new Integer(rs.getInt(fieldId)));
			}
			rs.close();
			intArray = new int[result.size()];
			for (int i = 0; i < result.size(); i++) {
				Integer intObj = result.elementAt(i);
				intArray[i] = intObj.intValue();
			}
		
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error("IllegalArgumentException error", e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
		return intArray;
	}

	/**
		 * Returns an array of int representing the contents
		 * If data are not found, a 0 length array is returned.
		 *
		 * @param selectClause	retriever of a list of objects.
		 *
		 * @return the array of objects
		 *
		 * @exception ObjectIOException
		 *			sql exception
		 *
		 */
	public long[] readLongArray(String selectClause) throws ObjectIOException {

		int fieldId = 1;
		Connection con = null;
		Statement stmt = null;
		Vector<Long> result = new Vector<Long>();
		ResultSet rs = null;
		getStdlog().debug(selectClause);
		ObjectIOException exception = null;
		long[] longArray = new long[0];
		try {
			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();

			// execute query
			rs = execQuery(selectClause, stmt);
			// get data
			while (rs.next()) {
				result.addElement(new Long(rs.getLong(fieldId)));
			}
			rs.close();
			 longArray = new long[result.size()];
			for (int i = 0; i < result.size(); i++) {
				Long intObj = result.elementAt(i);
				longArray[i] = intObj.longValue();
			}
			
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error("IllegalArgumentException error", e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
		return longArray;
	}

	/**
	 * Returns an integer of a select. It should be one row only (e.g count(), max...)
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return integer
	 *
	 * @exception ObjectIOException
	 *			sql exception
	 * @throws ObjectNotFoundException
	 */
	public int readIntValue(String selectClause) throws ObjectIOException {

		Connection con = null;
		Statement stmt = null;
		int result = -1;
		ObjectIOException exception = null;
	
		try {
            con = acquireConnection();
        } catch (Exception e) {
            getStdlog().error("Cannot connect: " + selectClause, e);
        	exception = new  ObjectIOException(e);
        	throw exception;
        }
		try {

			// initialize the connection
		
			stmt = con.createStatement();
			// execute query
			result = readIntValue(selectClause, stmt);
			
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
        	throw exception;
		} catch (IllegalArgumentException e) {
		   
			getStdlog().error("IllegalArgumentException error", e);
			exception = new  ObjectIOException(e);
        	throw exception;
		} finally {
			freeConnection(con, stmt);
		}
		return result;
	}
	/**
	 * Returns an integer of a select. It should be one row only (e.g count(), max...)
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return integer
	 *
	 * @exception ObjectIOException
	 *			sql exception
	 */
	public String readString(String selectClause) throws ObjectIOException {

		Connection con = null;
		Statement stmt = null;
		String result = "";
		ObjectIOException exception = null;
		try {

			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			// execute query
			result = readString(selectClause, stmt);
		
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error("IllegalArgumentException error", e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
		return result;
	}

	/**
	 * Returns an integer of a select. It should be one row only (e.g count(), max...)
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return integer
	 *
	 * @exception ObjectIOException
	 *			sql exception
	 */
	public long readLongValue(String selectClause) throws ObjectIOException {

		Connection con = null;
		Statement stmt = null;
		long result = -1;
		ObjectIOException exception = null;
		try {

			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			// execute query
			result = readLongValue(selectClause, stmt);
	
		} catch (SQLException e) {
			getStdlog().error("Sql statement error: " + selectClause, e);
			exception = new  ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error("IllegalArgumentException error", e);
			exception = new  ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
		return result;
	}

	/**
	 * Method readIntValue
	 * @param selectClause String
	 * @param stmt Statement
	 * @return int
	 * @throws SQLException
	 * @throws ObjectIOException
	 */
	public int readIntValue(String selectClause, Statement stmt)
		throws SQLException, ObjectIOException {
		int fieldId = 1;
		ResultSet rs = execQuery(selectClause, stmt);
		// get data
		int result = -1;
		if (rs.next()) {
			result = rs.getInt(fieldId);
			rs.close();
		} else {
			rs.close();
			throw new ObjectIOException(
				" value not found in select: " + selectClause);
		}

		return result;
	}

	/**
	 * Method readIntValue
	 * @param selectClause String
	 * @param stmt Statement
	 * @return int
	 * @throws SQLException
	 * @throws ObjectIOException
	 */
	public String readString(String selectClause, Statement stmt)
		throws SQLException, ObjectIOException {
		int fieldId = 1;
		ResultSet rs = execQuery(selectClause, stmt);
		// get data
		String result;
		if (rs.next()) {
			result = rs.getString(fieldId);
			rs.close();
		} else {
			rs.close();
			throw new ObjectIOException(
				" value not found in select: " + selectClause);
		}

		return result;
	}

	/**
	 * Method readLongValue
	 * @param selectClause String
	 * @param stmt Statement
	 * @return long
	 * @throws SQLException
	 * @throws ObjectIOException
	 */
	public long readLongValue(String selectClause, Statement stmt)
		throws SQLException, ObjectIOException {
		int fieldId = 1;
		ResultSet rs = execQuery(selectClause, stmt);
		// get data
		long result = -1;
		if (rs.next()) {
			result = rs.getLong(fieldId);
			rs.close();
		} else {
			rs.close();
			throw new ObjectIOException(
				" value not found in select: " + selectClause);
		}

		return result;
	}
	/**
	 * Returns the cached JDBC database Connection object creating
	 * it first, if necessary.
	 *
	 * @param con Connection
	 * @exception ObjectIOException
	 *		    an error occured during an access to a business object
	 */
	public void releaseConnection(Connection con) throws ObjectIOException {
		ConnectionPool.getPool().releaseConnection(con);
	}

	/**
	 * Updates, deletes or inserts data into the DB.
	 *
	 * @param 	updateClause the delete, insert or update statement.
	 *
	 * @exception ObjectIOException
	 *			sql error.
	 */
	public void execUpdate(String updateClause) throws ObjectIOException {

		Connection con = null;
		Statement stmt = null;
		ObjectIOException exception = null;
		try {
			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			// execute query
			execUpdate(updateClause, stmt);
		} catch (SQLException e) {
			getStdlog().error(e);
		exception = new ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			getStdlog().error(e);
		exception =  new ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
			if (exception != null) {
				throw exception;
			}
		}
	}

	/**
	 * Method execUpdate
	 * @param updateClause String
	 * @param stmt Statement
	 * @throws SQLException
	 */
	public void execUpdate(String updateClause, Statement stmt)
		throws SQLException {
		getStdlog().debug(updateClause);
		stmt.executeUpdate(updateClause);
	}

	

    /**
     * @return Returns the stdlog_.
     */
    private static Logger getStdlog() { 
        return stdlog_;      
    }
    

    /**
     * @param stdlog_ The stdlog_ to set.
     */
    public  void enableLog() {
          stdlog_.setLevel(logLevel_);
   
    }

    /**
     * @return Returns the stdlog_.
     */
    public void disableLog() {
        logLevel_ = stdlog_.getLevel();
        stdlog_.setLevel(Level.FATAL);
    }

    /**
     *  void
     * @throws ObjectIOException
     */
    public void testConnection() throws ObjectIOException {
    	Connection con = null;
		
		ObjectIOException exception = null;
		try {
			// initialize the connection
			con = acquireConnection();
			
		} catch (SQLException e) {
			getStdlog().error(e);
		exception = new ObjectIOException(e);
		throw exception;
		} finally {
			freeConnection(con);			
		}
        
    }

}
