package org.eso.ohs.persistence;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;

import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.docview.datatrans.ObjectAdaptor;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.BusinessVisitor;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.EphemerisFile;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentConstraintConfig;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;

public class Export {

    /**
     * Visitor for all the business objects
     */

    private ExportVisitor exportVisitor_;

    /**
     * Current Directory used to export objects to
     */

    private String currentDir_;

    private boolean version2_ = false;
    
    
    private Media device_;
    
    

    /**
     * Constructor for exported class
     */

    public Export() {
         
    }

    /**
     * Export a file to this directory given a business object to export
     *
     * @param currentDir export directory
     * @param obj business object to export
     * @exception Exception Any Exception that is thrown by the visitor class
     */

    public void export(String currentDir, BusinessObject obj, boolean version,Media device)
        throws Exception {
        currentDir_ = currentDir;
        version2_ = version;
        device_ = device;
        if (exportVisitor_ == null) {
            // lazy instantiation -- saves memory.
            exportVisitor_ = new ExportVisitor();
        }

        /*
         * Since we cannot allow exceptions to be easily thrown
         * via the BusinessVisitor interface, we have to return it
         * as a result ad then re-throw it here.  It's not very nice
         * but seems like a price worth paying in order to avoid calls
         * to instanceof.  See BusinessVisitor interface for more info.
         */
        Exception exc = (Exception) obj.accept(exportVisitor_, null);
        if (exc != null) {
            throw exc;
        }
    }

    /**
     * Export business object to impex format 
     * @param f Pafile to export to
     * @param obj Business Object to export
     * @param exclude Properties from the business object which should not be exported
     * @param map Map between the properties and what that property should be named in the impex file   
     *      
     * @exception NoSuchMethodException method does not exist
     * @exception IllegalAccessException unable to access method
     * @exception InvocationTargetException unable to invoke method      
     *
     * <br>ChangeLog: A M Chavan, 06-Mar-01: Fixed 1270: added encoding (escaping) 
     *                                of value string prior to generating 
     *                                export file.
     */

    private void exportInternal(
        PafFile f,
        BusinessObject obj,
        Hashtable exclude,
        Hashtable map)
        throws
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException {
        String keyword;
        Class cls = obj.getClass();
        String[] props = obj.propertyNames();
        int maxLen = 0;
        for (int j = 0; j < props.length; j++) {
            String oldValue = (String) map.get(props[j]);
            if (props[j].length() > maxLen)
                maxLen = props[j].length();
            if (oldValue != null && oldValue.length() > maxLen)
                maxLen = oldValue.length();
        }
        maxLen++;
        for (int i = 0; i < props.length; i++) {
            if (!exclude.containsKey(props[i])
                && !ImpexConstants.BO_EXCLUDE.containsKey(props[i])) {
                Method get = cls.getMethod("get" + props[i], new Class[] {
                });
                Object data = get.invoke(obj, new Object[] {
                });
                ObjectAdaptor oa = AdaptorFactory.getAdaptor(obj, props[i]);
                if (map.containsKey(props[i]))
                    keyword = (String) map.get(props[i]);
                else
                    keyword = props[i];
                String value=null;
                if (data != null)
                    value = new String(data.toString());
                else
                    value = "";
                int padlen = maxLen - keyword.length();
                StringBuffer buf = new StringBuffer();
                for (int k = 0; k < padlen; k++) {
                    buf.append(' ');
                }

                // A M Chavan, 06-Mar-01: added escaping of value
                String escapedValue = TextUtils.escapedString(oa.getValue());
                f.writeLine(
                    keyword + buf.toString() + "\"" + escapedValue + "\"");
            }
        }
    }

    /**
     * Compute the full pathname for a Calibration or an
     * Observation Block to be exported.  
     *
     * @return A full pathname for the exported OB/CB.
     *
     * @param directory Full pathname of the target directory; 
     *                  for instance, <code>/tmp</code>.
     * @param ob        Calibration Or Observation Block to export
     *
     * <br>ChangeLog: A M Chavan, 04-Jan-01 */
    public String exportPathname(String directory, CalibrationBlock ob) {
        String filename = ob.getName().replace(' ', '_');
        String ret = directory + File.separator + filename + ".obx";
        return (ret);
    }

    /**
     * Export target 
     * @param tg Target business object.
     * @exception NoSuchMethodException method does not exist
     * @exception IllegalAccessException unable to access method
     * @exception InvocationTargetException unable to invoke method     
     * @exception IOException I/O Error
     */

    public void exportTG(Target tg)
        throws
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException,
            IOException {
        String filename = tg.getName().replace(' ', '_');
        PafFile f =
            new PafFile(currentDir_ + File.separator + filename + ".tpx", true);
        exportInternal(f, tg, ImpexConstants.TG_EXCLUDE, ImpexConstants.TG_MAP);
        f.closeWriter();
    }

    /**
     * Export observation description
     * @param od Observation description business object  
     * @exception NoSuchMethodException method does not exist
     * @exception IllegalAccessException unable to access method
     * @exception InvocationTargetException unable to invoke method        
     * @exception IOException I/O Error
     */

    public void exportOD(ObservationDescription od)
        throws
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException,
            IOException {
        String filename = od.getName().replace(' ', '_');
        PafFile f =
            new PafFile(currentDir_ + File.separator + filename + ".odx", true);
        exportInternal(f, od, ImpexConstants.OD_EXCLUDE, ImpexConstants.OD_MAP);
        TemplateSignature[] tsfs = od.getSignatures();
        for (int i = 0; i < tsfs.length; i++) {
            exportTS(tsfs[i], f);
        }
        f.closeWriter();
    }

    /**
     * Export Observation Block
     * @param ob Observation block business object
     * @exception NoSuchMethodException method does not exist
     * @exception IllegalAccessException unable to access method
     * @exception InvocationTargetException unable to invoke method
     * @exception IOException I/O Error
     *
     * <br>ChangeLog: A M Chavan, 04-Jan-01: refactored w/ exportPathname()
     */

    public void exportOB(ObservationBlock ob)
        throws
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException,
            IOException, ObjectNotFoundException, ObjectIOException {

        PafFile f = new PafFile(exportPathname(currentDir_, ob), true);

        if (version2_)
            f.writeLine("IMPEX.VERSION \"2.0\"");
        exportInternal(f, ob, ImpexConstants.OB_EXCLUDE, ImpexConstants.OB_MAP);
        
        
        TimeInterval[] timeIntervals = ob.getTimeIntervals();
        if (timeIntervals != null && timeIntervals.length > 0) {
            SimpleDateFormat timeFormat =
                new SimpleDateFormat(Convert.ISO_DATE_FORMAT);
            Date dt;
            StringBuffer buf = new StringBuffer("absolute_times_list ");
            buf.append("\"");
            for (int i = 0; i < timeIntervals.length; i++) {
                buf.append("{");
                dt =
                    new Date(
                        timeIntervals[i].getStartTime()
                            * ImpexConstants.MILLISECS_PER_SEC);
                String minTime = timeFormat.format(dt);
                buf.append(minTime);
                buf.append(" ");
                dt =
                    new Date(
                        timeIntervals[i].getEndTime()
                            * ImpexConstants.MILLISECS_PER_SEC);
                String maxTime = timeFormat.format(dt);
                buf.append(maxTime);
                buf.append(" ");
                buf.append(timeIntervals[i].getPreference());
                buf.append("} ");

            }
            buf.append("\"");
            f.writeLine(buf.toString());
        }
        OHSPersistence persistence = (OHSPersistence) OHSPersistence.getObjectManager();
        FindingChart [] fcs=null; 
        if(device_.equals(Media.DBASE))
             fcs = persistence.populateFindingCharts(Media.DBASE,ob.getId());
        else      
             fcs = ob.getFindingCharts();        
        if (fcs != null && fcs.length > 0) {
            StringBuffer buf = new StringBuffer("finding_chart_list ");
            buf.append("\"");
            for (int i = 0; i < fcs.length; i++) {
                buf.append(fcs[i].getFcName());
                if(i < fcs.length)
                    buf.append(" ");
                File outFile =
                    new File(currentDir_ + File.separator + fcs[i].getFcName());                    
                if(device_.equals(Media.LOCAL)) {                
                     String rootName = persistence.getRootFolderPath(ob.getObsRun());
                     FileIOUtils.writeBinaryFile(fcs[i].getFcDataLocal(rootName),outFile);
                } else {
                     FileIOUtils.writeBinaryFile(fcs[i].getFcData(),outFile);                             
                }
            }
            buf.append("\"");
            f.writeLine(buf.toString());
        }
        
        
        if (version2_) {
            f.writeLine("\n");
            exportInternal(
                f,
                ob.getTarget(),
                ImpexConstants.TG_EXCLUDE,
                ImpexConstants.TG_MAP_VERSION2);

        } 
        if (version2_) {
            f.writeLine("\n");
            String instrument=ob.getOd().getInstrument();
            Instrument instObj = InstrumentList.getInstance().getInstrument(instrument,ob.fetchIPVersionFromOd());
            InstrumentConstraintConfig [] instConfList = instObj.getCsConfList();  
            ConstraintSet cs = ob.getConstraintSet();
            String[] props = cs.propertyNames();
            boolean found = false;
            Hashtable<String,String> excMap = new Hashtable<String,String>();
            for(int i=0;i<props.length;i++) {
                if(props[i].equals("Name"))
                    continue;
                for(int j=0;j<instConfList.length;j++) {                    
                    if(props[i].equals(instConfList[j].getName())) {
                        found=true;
                    }
                }
                if(!found)
                  excMap.put(props[i],props[i]);
                found=false;                             
            }
            exportInternal(
                f,
                ob.getConstraintSet(),
                excMap,
                ImpexConstants.CS_MAP_VERSION2);
            f.writeLine("\n");
        } 
        if (version2_) {
            exportInternal(
                f,
                ob.getOd(),
                ImpexConstants.OD_EXCLUDE,
                ImpexConstants.OD_MAP_VERSION2);
            if (ob.getTemplateSignature() != null) {
                exportTS(ob.getTemplateSignature(), f);
                f.writeLine("\n");
            }
            TemplateSignature[] tsfs = ob.getOd().getSignatures();
            for (int i = 0; i < tsfs.length; i++) {
                exportTS(tsfs[i], f);
            }
            f.writeLine("\n");
        }
        if (!version2_) {
            if (ob.getTemplateSignature() != null)
                exportTS(ob.getTemplateSignature(), f);
        }
        EphemerisFile file = ob.getEphemerisFile();
        if(file != null) {
            String filename = file.getFilename();
            if (filename != null && filename.length() > 0) {
                f.writeLine(
                        "ephemeris_file "
                        + "\""
                        + TextUtils.escapedString(filename)
                        + "\"");
                File outFile =
                    new File(currentDir_ + File.separator + filename);
                FileOutputStream writeStream =
                    new FileOutputStream(outFile);
                writeStream.write(file.getFileString().getBytes());
                writeStream.close();
            }
      
        }
        f.closeWriter();
         
    }

    /**
     * Export calibration block
     * @param cb Refernce to calibration block
     * @exception NoSuchMethodException method does not exist
     * @exception IllegalAccessException unable to access method
     * @exception InvocationTargetException unable to invoke method 
     * @exception IOException I/O Error
     *
     * <br>ChangeLog: A M Chavan, 04-Jan-01: refactored w/ exportPathname()
     */

    public void exportCB(CalibrationBlock cb)
        throws
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException,
            IOException {

        PafFile f = new PafFile(exportPathname(currentDir_, cb), true);

        if (version2_)
            f.writeLine("IMPEX.VERSION \"2.0\"");
        exportInternal(f, cb, ImpexConstants.OB_EXCLUDE, ImpexConstants.OB_MAP);
        if (version2_)
            f.writeLine("\n");
        if (version2_) {
            exportInternal(
                f,
                cb.getOd(),
                ImpexConstants.OD_EXCLUDE,
                ImpexConstants.OD_MAP_VERSION2);
            TemplateSignature[] tsfs = cb.getOd().getSignatures();
            for (int i = 0; i < tsfs.length; i++) {
                exportTS(tsfs[i], f);
            }
            f.writeLine("\n");
        } else if (cb.getOd() != null) {
            exportOD(cb.getOd());
            String odFilename = cb.getOd().getName().replace(' ', '_');
            f.writeLine(
                "observationDescription " + "\"" + odFilename + ".odx" + "\"");
        }
        f.closeWriter();
    }

    /**
     * Export constraint set
     * @param cs Refernce to constraint set
     * @exception NoSuchMethodException method does not exist
     * @exception IllegalAccessException unable to access method
     * @exception InvocationTargetException unable to invoke method 
     * @exception IOException I/O Error
     */

    public void exportCS(ConstraintSet cs)
        throws
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException,
            IOException {
        String filename = cs.getName().replace(' ', '_');
        PafFile f =
            new PafFile(currentDir_ + File.separator + filename + ".csx", true);
        exportInternal(f, cs, ImpexConstants.CS_EXCLUDE, ImpexConstants.CS_MAP);
        f.closeWriter();
    }
    

    /**
     * Export template signature
     * @param ts reference to template signature
     * @param f PafFile reference to export to
     */

    private void exportTS(TemplateSignature ts, PafFile f) throws IOException {
        Parameter[] plist = ts.getDisplayParamList();
        int maxLen = ts.getTemplateName().length();
        for (int i = 0; i < plist.length; i++) {
            if (plist[i].getParameterName().length() > maxLen)
                maxLen = plist[i].getParameterName().length();
        }
        maxLen++;
        f.writeLine("\n");
        if (version2_) {
            if (ts.getTemplateType().equals("acquisition"))
                f.writeLine(
                    "ACQUISITION.TEMPLATE.NAME "
                        + "\""
                        + ts.getTemplateName()
                        + "\"");
            else {
                f.writeLine(
                    "TEMPLATE.NAME " + "\"" + ts.getTemplateName() + "\"");
            }
        } else
            f.writeLine("template " + "\"" + ts.getTemplateName() + "\"");
        for (int i = 0; i < plist.length; i++) {
            String value = plist[i].getValue();
            if (plist[i].getValue() == null)
                value = "";
            int padlen = maxLen - plist[i].getParameterName().length();
            StringBuffer buf = new StringBuffer();
            for (int k = 0; k < padlen; k++) {
                buf.append(' ');
            }
            if (plist[i].getType().equals("paramfile")) {
                String filename = ((ParamFileParameter) plist[i]).getFilename();
                if (filename != null && filename.length() > 0) {
                    f.writeLine(
                        plist[i].getParameterName()
                            + buf.toString()
                            + "\""
                            + TextUtils.escapedString(filename)
                            + "\"");
                    File outFile =
                        new File(currentDir_ + File.separator + filename);
                    FileOutputStream writeStream =
                        new FileOutputStream(outFile);
                    writeStream.write(plist[i].getValue().getBytes());
                    writeStream.close();
                } else {
                    f.writeLine(
                        plist[i].getParameterName()
                            + buf.toString()
                            + "\""
                            + "NODEFAULT"
                            + "\"");
                }
            } else {
                f.writeLine(
                    plist[i].getParameterName()
                        + buf.toString()
                        + "\""
                        + TextUtils.escapedString(value)
                        + "\"");
            }
        }
    }

    /**
     * Visitor class to implement all the different business objects
     */

    private class ExportVisitor implements BusinessVisitor {

        /**
         * Export observation description 
         * @param od Reference to observation description business object
         * @param arg set to null
         */

        public Object visit(ObservationDescription od, Object arg) {
            try {
                exportOD(od);
                return null;
            } catch (Exception e) {
                return e;
            }
        }

        /**
         * Export target 
         * @param tg Reference to target business object
         * @param arg set to null
         */

        public Object visit(Target tg, Object arg) {
            try {
                exportTG(tg);
                return null;
            } catch (Exception e) {
                return e;
            }
        }

        /**
         * Export calibration block
         * @param cb Reference to Calibration Block  business object
         * @param arg set to null
         */

        public Object visit(CalibrationBlock cb, Object arg) {
            try {
                exportCB(cb);
                return null;
            } catch (Exception e) {
                return e;
            }
        }

        /**
          * Export constraint set
          * @param cs Reference to Constraint Set business object
          * @param arg set to null
          */
        public Object visit(ConstraintSet cs, Object arg) {
            try {
                exportCS(cs);
                return null;
            } catch (Exception e) {
                return e;
            }
        }

        /**
         * Export observation block 
         * @param ob Reference to observation block business object
         * @param arg set to null
         */

        public Object visit(ObservationBlock ob, Object arg) {
            try {
                exportOB(ob);
                return null;
            } catch (Exception e) {
                return e;
            }
        }

        public Object visit(ObservingRun run, Object arg) {
            return new IllegalArgumentException("Observing runs not supported");
        }

        public Object visit(Queue dbQ, Object arg) {
            return new IllegalArgumentException("Queues not supported");
        }
    }
}
