package org.eso.ohs.persistence;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.CRC32;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.docview.datatrans.ObjectAdaptor;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.EphemerisFile;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObjectFactory;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.instruments.TemplateSignatureFactory;

/**
 * This class imports impex files. Saves the business objects to disk and then
 * registers them with the object manager <br>
 * ChangeLog: cmavros 05JUNE02 [1766/P2PP] Do not import files with blank name
 * fields.
 * 
 * @author Tim Canavan
 * @version $Revision: 1.81 $
 */

public class Import {

    private static Logger stdlog_ = Logger.getLogger(Import.class);

    /**
     * The currentDir where imported files are located
     */

    private String currentDir_;

    private ImportWarning impWarning_ = new ImportWarning();

    private boolean version2_ = false;

    /**
     * Name Keyword fields that are blank in an imported file.
     */
    private Vector<String> nameMissing_ = new Vector<String>();

    /**
     * Constructor for Import
     */

    public Import() {
    }

    /**
     * Warnings associated with Import
     * 
     * @return true if warnings else false
     */

    public boolean hasWarning() {
        return impWarning_.hasWarning();
    }

    /**
     * Warning String to give details the user at the GUI layer later
     * 
     * @return warning string
     */
    public String toWarningString() {
        return impWarning_.toString();
    }

    public String getNameMissing() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < nameMissing_.size(); i++) {
            buf.append("\n" + nameMissing_.elementAt(i));
        }
        return buf.toString();
    }

    /**
     * Check import files referred to by other files have read rights and exist
     * 
     * @exception IOException
     *                IO error reading export file
     * @exception FileNotFoundException
     *                could not find export file
     */

    public static String checkLinks(String currentDir, String filename)
            throws IOException, FileNotFoundException {
        StringBuffer buf = new StringBuffer();
        // File f = null;

        if (filename.endsWith(".obx")) {
            PafFile pafFile = new PafFile(filename);
            if (pafFile.compileError()) {
                buf.append(pafFile.getCompileErrorMsg());
                // System.out.println("CLASS Import - checkLinks: "+buf);
                return (buf.toString());
            }
        }
        // System.out.println("CLASS Import - checkLinks: returning ->"+buf);
        return buf.toString();
    }

    /**
     * Import Business Object
     * 
     * @param currentDir
     *            directory where import files reside
     * @param filename
     *            name of file being imported
     * @param node
     *            Directory Node where object should be created
     * @param obsRun
     *            Observing run to be set in OB created
     * @exception ObjectIOException
     *                ObjectIOException error reading export file
     * 
     * @exception NoSuchMethodException
     *                method does not exist
     * @exception IllegalAccessException
     *                unable to access method
     * @exception InvocationTargetException
     *                unable to invoke method
     */

    public void overwriteOB(String currentDir, String filename,
            DirectoryNode node, ObservingRun obsRun, ObservationBlock ob)
            throws NoSuchMethodException, IllegalAccessException,
            InvocationTargetException, ObjectIOException, IOException,
            ObjectNotFoundException {
        currentDir_ = currentDir;
        PafFile f = new PafFile(filename);

        ob.setTemplateSignature(null);
        ob.getOd().setSignatures(new TemplateSignature[0]);
        if (filename.endsWith(".obx")) {
            overwriteOBInternal(f, ob, obsRun);
        }
    }

    /**
     * Import Business Object
     * 
     * @param currentDir
     *            directory where import files reside
     * @param filename
     *            name of file being imported
     * @param node
     *            Directory Node where object should be created
     * @param obsRun
     *            Observing run to be set in OB created
     * @exception ObjectIOException
     *                ObjectIOException error reading export file
     * 
     * @exception NoSuchMethodException
     *                method does not exist
     * @exception IllegalAccessException
     *                unable to access method
     * @exception InvocationTargetException
     *                unable to invoke method
     */

    public void overwriteCB(String currentDir, String filename,
            DirectoryNode node, ObservingRun obsRun, CalibrationBlock ob)
            throws IOException, NoSuchMethodException, IllegalAccessException,
            InvocationTargetException, ObjectIOException {
        currentDir_ = currentDir;
        PafFile f = new PafFile(filename);
        if (filename.endsWith(".obx")) {
            overwriteCBInternal(f, ob, obsRun);
        }
    }

    private void overwriteCBInternal(PafFile f, CalibrationBlock destCB,
            ObservingRun obsRun) throws NoSuchMethodException,
            IllegalAccessException, InvocationTargetException,
            ObjectIOException, IOException {
        String odName = null;
        CalibrationBlock cb = importCBInternal(f, obsRun);
        updateRecursive(cb, destCB);
    }

    /**
     * Update the business object recursively
     * 
     * @param fromBO
     *            Business Object to update from
     * @param toBO
     *            Business Object to update to
     * 
     */

    private void updateRecursive(BusinessObject fromBO, BusinessObject toBO)
            throws IllegalAccessException, InvocationTargetException,
            ObjectIOException, NoSuchMethodException {
        Stack<BusinessObject> stk = new Stack<BusinessObject>();
        stk.push(fromBO); // src below destination on stk
        stk.push(toBO);
        BusinessObject destBO, bo;

        while (!stk.isEmpty()) {
            destBO = stk.pop();
            bo = stk.pop();

            BusinessObject[] children = bo.getBusObjChildren();
            BusinessObject[] destChildren = destBO.getBusObjChildren();

            for (int i = 0; i < destChildren.length; i++) {
                stk.push(children[i]);
                stk.push(destChildren[i]);
            }
            bo.copyProperties(bo.simpleProperties(), destBO);
        }

    }

    /**
     * This is an internal Method imports a .obs file
     * 
     * @param f
     *            File which contains exported data
     * @param destOB
     *            Destination of the data read in from exported file
     * @param obsRun
     *            Observing Run where newly imported ob should be linked to
     */

    private void overwriteOBInternal(PafFile f, ObservationBlock destOB,
            ObservingRun obsRun) throws NoSuchMethodException,
            IllegalAccessException, InvocationTargetException,
            ObjectIOException, IOException, ObjectNotFoundException {
        ObservationBlock ob = (ObservationBlock) ObjectFactory
                .createRecursive(ObservationBlock.class);
        importInternal(f, ob, ImpexConstants.OB_MAP, ImpexConstants.OB_EXCLUDE,
                obsRun);
        importOBInternal(f, obsRun, ob);
        updateRecursive(ob, destOB);
    }

    private void importFCs(ObservingRun obsRun, ObservationBlock ob,
            String fcList) throws IOException, ObjectNotFoundException {
        Vector<FindingChart> fcChart = new Vector<FindingChart>();
        StringTokenizer tokenizer = new StringTokenizer(fcList);
        Hashtable fcNameTable = new Hashtable();
        Hashtable fcChecksumTable = new Hashtable();
        int numFC = 0;
        while (tokenizer.hasMoreTokens()) {
            String fileName = tokenizer.nextToken();
            FindingChart fc = new FindingChart();
            if (numFC >= AppConfig.getAppConfig().getMaxFindingCharts()) {
                impWarning_.tooManyFC.addElement(fc.getName());
                continue;
            }
            fc.setFcName(fileName);
            File fcFile = new File(currentDir_ + File.separator + fileName);
            if (!fcFile.exists()) {
                impWarning_.fcNotFound.addElement(fcFile.getName());
                continue;
            }
            if (ob.hasFindingChartName(fc.getFcName())
                    || fcNameTable.containsKey(fc.getFcName())) {
                impWarning_.duplicateNameFC.addElement(fc.getFcName());
                continue;
            }

            if (ob.hasFindingChartChecksum(fc.getCheckSum())
                    || fcChecksumTable.containsKey(new Long(fc.getCheckSum()))) {
                impWarning_.duplicateContentsFC.addElement(fc.getFcName());
                continue;

            }
            byte[] fcData = FileIOUtils
                    .readBinaryFile(fcFile.getAbsolutePath());
            if (fcData.length > AppConfig.getAppConfig()
                    .getMaxFindingChartSize()) {
                impWarning_.fcTooBig.addElement(fc.getFcName());
                continue;
            }
            CRC32 crc32 = new CRC32();
            crc32.update(fcData);
            long checkSum = crc32.getValue();
            fc.setCheckSum(checkSum);
            OHSPersistence persistence = (OHSPersistence) OHSPersistence
                    .getObjectManager();
            String rootName = persistence.getRootFolderPath(obsRun);
            fcChart.addElement(fc);
            fc.setObsRun(obsRun);
            File f = new File(rootName + File.separator + fc.getCheckSum()
                    + "." + fc.getFcType());
            if (!f.exists()) {
                FileIOUtils.writeBinaryFile(fcData, f);
            }
            numFC++;
        }
        FindingChart[] fcArray = new FindingChart[fcChart.size()];
        fcChart.copyInto(fcArray);
        ob.setFindingCharts(fcArray);
    }

    private void importEphFile(ObservationBlock ob, String filename) {
        File file = new File(currentDir_ + File.separator + filename);
        if (file.exists() && file.canRead()) {
            EphemerisFile ephFile = new EphemerisFile();
            ephFile.setFilename(file.getName());
            ephFile.setFileString(TextUtils.convertFileToString(file));
            ob.setEphemerisFile(ephFile);
        } else {
            impWarning_.invalidAttributes_
                    .addElement(" \nFile does not exist or is not readable "
                            + file.getName());

        }
    }

    /**
     * Check intrument package is installed if importing od's
     * 
     * @exception IOException
     *                IO error reading export file
     */

    public static boolean checkInstrument(String currentDir, String fileName,
            ObservingRun obsRun) throws IOException {
        if (!fileName.endsWith(".obx")) {
            return true;
        } else {
            if (fileName.endsWith(".obx")) {
                PafFile f = new PafFile(fileName);
                if (f.compileError()) {
                    // System.out.println("CLASS Import - checkInstrument:
                    // "+f.getCompileErrorMsg());
                    return false;
                }
                String instrument = null;
                instrument = f.readLine("instrument");
                if (instrument != null && instrument.length() > 0
                        && obsRun.getInstCode().equals(instrument))
                    return true;
                else
                    return false;

            }
        }
        return false;
    }

    /**
     * Check for blank Keyword Name fields
     * 
     * @exception IOException
     *                IO error reading export file
     */

    public boolean checkNames(String currentDir, String fileName,
            ObservingRun obsRun) throws IOException {
        String target = null;
        String odName = null;
        String csName = null;
        PafFile f = new PafFile(fileName);
        PafData[] identifierList = f.getIdentifierList();
        String type = "O"; // default type to an OB
        // Check version
        // Check type
        for (int i = 0; i < identifierList.length; i++) {
            if (identifierList[i].getIdentifier().equals("type"))
                type = identifierList[i].getValue();
        }

        if (!type.equalsIgnoreCase("C")) {
            target = f.readLine("TARGET.NAME");
            if (target == null || target.length() == 0) {
                nameMissing_.addElement("TARGET.NAME");
            }
            csName = f.readLine("CONSTRAINT.SET.NAME");
            if (csName == null || csName.length() == 0) {
                nameMissing_.addElement("CONSTRAINT.SET.NAME");
            }
        }
        odName = f.readLine("OBSERVATION.DESCRIPTION.NAME");
        if (odName == null || odName.length() == 0) {
            nameMissing_.addElement("OBSERVATION.DESCRIPTION.NAME");
        }
        if (nameMissing_.size() > 0) {
            return false;
        }
        return true;
    }

    /**
     * Check file is an import file
     * 
     * @exception IOException
     *                IO error reading export file
     */

    public static boolean checkImport(String currentDir, String fileName)
            throws IOException {
        PafFile f = new PafFile(fileName);
        if (f.compileError()) {
            // System.out.println("CLASS Import - checkImport:
            // "+f.getCompileErrorMsg());
            return false;
        }

        PafData[] idlist = f.getIdentifierList();
        boolean valid = false;
        for (int i = 0; i < idlist.length; i++) {
            if (idlist[i].getIdentifier().equals("name"))
                valid = true;
        }
        return valid;
    }

    /**
     * Check .obx file has a valid tpye defined
     * 
     * @param fileName
     *            Name of file to check
     * @exception IOException
     *                IO error reading export file
     */

    public static boolean isValidOBType(String fileName) throws IOException {
        String type = getOBType(fileName);
        if (type != null && type.length() > 0
                && (type.equalsIgnoreCase("C") || type.equalsIgnoreCase("O")))
            return true;
        return false;
    }

    public static String getOBType(String fileName) throws IOException {
        PafFile f = new PafFile(fileName);

        if (f.compileError()) {
            throw new IOException("Could not compile PAF File");
        }

        PafData[] idlist = f.getIdentifierList();
        String type = null;
        for (int i = 0; i < idlist.length; i++) {
            if (idlist[i].getIdentifier().equals("type")) {
                type = idlist[i].getValue();
            }
        }
        return type;

    }

    /**
     * Import Business Object
     * 
     * @param currentDir
     *            directory where import files reside
     * @param filename
     *            name of file being imported
     * @param node
     *            Directory Node where object should be created
     * @param obsRun
     *            Observing run to be set in OB created
     * 
     * @exception IOException
     *                I/O error reading export file
     * @exception ObjectIOException
     *                ObjectIOException error reading export file
     * @exception NoSuchMethodException
     *                method does not exist
     * @exception IllegalAccessException
     *                unable to access method
     * @exception InvocationTargetException
     *                unable to invoke method
     * 
     * @return The imported BusinessObject
     * 
     * <br>
     * ChangeLog: A M Chavan, 06-Mar-01: changed return value.
     */

    public BusinessObject importBO(String currentDir, String filename,
            DirectoryNode node, ObservingRun obsRun)
            throws NoSuchMethodException, IllegalAccessException,
            InvocationTargetException, ObjectIOException, IOException,
            ObjectNotFoundException {

        if (!filename.endsWith(".obx"))
            throw new IOException("Invalid IMPEX filename extension:"
                    + filename);
        currentDir_ = currentDir;

        PafFile f = new PafFile(filename);
        // if(f.compileError())
        // System.out.println("CLASS Import - importBO:
        // "+f.getCompileErrorMsg());

        PafData[] identifierList = f.getIdentifierList();
        String impexType = "O"; // Default is type = "O" -- see IMPEX specs

        for (int i = 0; i < identifierList.length; i++)
            if (identifierList[i].getIdentifier().equals("type"))
                impexType = identifierList[i].getValue();

        if (impexType.equalsIgnoreCase("O"))
            return (importObsBlock(f, node, obsRun));

        return (importCalBlock(f, node, obsRun));
    }

    /**
     * This method imports any PAF file to its associated business object using
     * reflection. <br>
     * ChangeLog: A M Chavan, 06-Mar-01: Fixed 1270: added de-encoding
     * (de-escaping) of value string read from import file; Entirely rewritten.
     * 
     * @param f
     *            reference to exported file
     * @param obj
     *            reference to business object
     * @param map
     *            Mapping between properties and exported strings
     * @param exclude
     *            Properties which should not be exported
     * 
     * @exception NoSuchMethodException
     *                method does not exist
     * @exception IllegalAccessException
     *                unable to access method
     * @exception InvocationTargetException
     *                unable to invoke method
     * 
     */

    private void importInternal(PafFile f, BusinessObject obj, Hashtable map,
            Hashtable exclude, ObservingRun obsRun)
            throws NoSuchMethodException, IllegalAccessException,
            InvocationTargetException {

        String keyword;
        Class cls = obj.getClass();
        String[] props = obj.propertyNames();

        for (int i = 0; i < props.length; i++) {

            // Some object properties are not imported,
            // so we just skip them.
            // ----------------------------------------
            if (exclude.contains(props[i]))
                continue;

            // Map property name to IMPEX syntax
            // ----------------------------------------
            if (map.containsKey(props[i]))
                keyword = (String) map.get(props[i]);
            else
                keyword = props[i];

            // System.out.println ( "CLASS Import - importInternal >>> " + i +
            // ": "
            // + props[i]
            // + " --> "
            // + keyword );

            // Skip missing entries
            // ----------------------------------------
            String value = f.readLine(keyword);
            if (value == null)
                continue;

            int linenumber = f.getLineNumber(keyword);
            Method get = cls.getMethod("get" + props[i], new Class[] {});
            Method set = cls.getMethod("set" + props[i], new Class[] { get
                    .getReturnType() });

            if (!version2_) {

                // Support for V.1.x IMPEX files: newline
                // chars are encoded as "<CR>"
                // ----------------------------------------
                int index = 0;
                while (true) {

                    index = value.indexOf("<CR>", index);
                    if (index == -1)
                        break;

                    StringBuffer buf = new StringBuffer();
                    int endIndex = value.indexOf('>', index);
                    buf.append(value.substring(0, index));
                    buf.append("\n");
                    buf.append(value.substring(endIndex + 1, value.length()));
                    value = buf.toString();
                }
            }

            // Decode escaped chars
            // ----------------------------------------
            // value = TextUtils.revertEscapedString( value );

            try {

                // Convert from IMPEX format to internal
                // representation, truncating where
                // necessary.
                // ----------------------------------------
                ObjectAdaptor oa = AdaptorFactory.getAdaptor(obj, props[i]);

                if (props[i].equals("InstrumentComments")) {
                    Instrument instrument = InstrumentList.getInstance()
                            .getInstrument(obsRun.getInstCode(),
                                    obsRun.getIPVersion());
                    if (!instrument.hasInstComments()) {
                        impWarning_.invalidAttributes_
                                .addElement("Warning instrument comments not available for this run");
                        continue;
                    }
                }

                if (props[i].equals("Name")
                        && value.length() > Config.MAX_NAME_LENGTH) {

                    value = value.substring(0, Config.MAX_NAME_LENGTH);
                    impWarning_.invalidAttributes_
                            .addElement("Warning value will be truncated to "
                                    + Config.MAX_NAME_LENGTH + " in size"
                                    + " at linenumber " + linenumber
                                    + " in file " + f.getShortFilename());
                } else if (value.length() > Config.MAX_TEXT_LENGTH) {

                    value = value.substring(0, Config.MAX_TEXT_LENGTH);
                    impWarning_.invalidAttributes_
                            .addElement("Warning value will be truncated to "
                                    + Config.MAX_TEXT_LENGTH + " in size"
                                    + " at linenumber " + linenumber
                                    + " in file " + f.getShortFilename());
                }
                oa.setValue(value);
                set.invoke(obj, new Object[] { oa.getRawValue() });
            } catch (IllegalArgumentException e) {

                impWarning_.invalidAttributes_
                        .addElement("Warning Invalid value : " + value
                                + " at linenumber " + linenumber + " in file "
                                + f.getShortFilename()
                                + " Default will be used");
            }
        }
    }

    /**
     * Update target value from parameter file in an acquisition template
     */

    // public void updateTargetFromParamFile(TemplateSignature
    // acqTemplate,Target target)
    private void updateFromParamFile(TemplateSignature acqTemplate,
            Target target, ConstraintSet cs) {
        Parameter[] plist = acqTemplate.getDisplayParamList();
        for (int i = 0; i < plist.length; i++) {
            if (plist[i].getType().equals("paramfile")) {
                target
                        .updateFromParamFileParameter((ParamFileParameter) plist[i]);
                if (cs != null) {
                    cs
                            .updateFromParamFileParameter((ParamFileParameter) plist[i]);
                }
            }
        }
    }

    /**
     * Import ob
     * 
     * @param f
     *            PafFile to be imported
     * @param node
     *            Directory Node where object should be created
     * @param obsRun
     *            Observing run to be set in OB created
     * 
     * @exception ObjectIOException
     *                ObjectIOException error reading export file
     * @exception NoSuchMethodException
     *                method does not exist
     * @exception IllegalAccessException
     *                unable to access method
     * @exception InvocationTargetException
     *                unable to invoke method
     * 
     * @return The imported ObservationBlock
     * 
     * <br>
     * ChangeLog: Toni Schlichting 22-May-2001 chk.f. TimeInterval limits
     */

    public ObservationBlock importObsBlock(PafFile f, DirectoryNode node,
            ObservingRun obsRun) throws NoSuchMethodException,
            IllegalAccessException, InvocationTargetException,
            ObjectIOException, IOException, ObjectNotFoundException {


        ObservationBlock ob = (ObservationBlock) ObjectFactory
                .createRecursive(ObservationBlock.class);
        importInternal(f, ob, ImpexConstants.OB_MAP, ImpexConstants.OB_EXCLUDE,
                obsRun);

        // Target
        // ----------------------------------------

        importOBInternal(f, obsRun, ob);

        // System.out.println("CLASS Import - importObsBlock: TimeIntervals end
        // ");
        ob.setObsRun(obsRun);
        ObjectManager.getObjectManager().recursiveRegister(Media.LOCAL, node,
                ob);
        return (ob);
    }

    private void importOBInternal(PafFile f, ObservingRun obsRun,
            ObservationBlock ob) throws IOException, FileNotFoundException,
            NoSuchMethodException, IllegalAccessException,
            InvocationTargetException, ObjectNotFoundException {
        String target;
        String odName;
        String csName;

        target = f.readLine("TARGET.NAME");
        if (target != null && target.length() > 0)
            importInternal(f, ob.getTarget(), ImpexConstants.TG_MAP_VERSION2,
                    ImpexConstants.TG_EXCLUDE, obsRun);

        odName = f.readLine("OBSERVATION.DESCRIPTION.NAME");
        if (odName != null && odName.length() > 0) {
            importInternal(f, ob.getOd(), ImpexConstants.OD_MAP_VERSION2,
                    ImpexConstants.OD_EXCLUDE, obsRun);
        }
        TemplateSignature[] tsfs = importTsfs(f, obsRun.getInstCode(), obsRun
                .getIPVersion());
        if (tsfs != null) {
            Vector<TemplateSignature> tmpl = new Vector<TemplateSignature>();
            for (int i = 0; i < tsfs.length; i++) {
                if (tsfs[i].getTemplateType().equals("acquisition")) {
                    ob.setTemplateSignature(tsfs[i]);
                    // updateTargetFromParamFile(tsfs[i],ob.getTarget());
                    updateFromParamFile(tsfs[i], ob.getTarget(), ob
                            .getConstraintSet());
                } else
                    tmpl.addElement(tsfs[i]);
            }
            TemplateSignature[] odTemplates = new TemplateSignature[tmpl.size()];
            tmpl.copyInto(odTemplates);
            ob.getOd().setSignatures(odTemplates);
        }

        csName = f.readLine("CONSTRAINT.SET.NAME");
        if (csName != null && csName.length() > 0)
            importInternal(f, ob.getConstraintSet(),
                    ImpexConstants.CS_MAP_VERSION2, ImpexConstants.CS_EXCLUDE,
                    obsRun);

        String timeInterval = f.readLine("absolute_times_list");
        if (timeInterval != null && timeInterval.length() > 0) {
            importTimeIntervals(f, ob, timeInterval);
        }
        String fcList = f.readLine("finding_chart_list");
        if (fcList != null && fcList.length() > 0) {
            importFCs(obsRun, ob, fcList);
        }
        String ephfile = f.readLine("ephemeris_file");
        if (ephfile != null && ephfile.length() > 0) {
            importEphFile(ob, ephfile);
        }

    }

    private void importTimeIntervals(PafFile f, ObservationBlock ob,
            String timeInterval) {
        Vector<TimeInterval> tiVec = new Vector<TimeInterval>();
        int index = 0;
        for (int ti_index = 0;; ti_index++) {
            index = timeInterval.indexOf("{", index);
            SimpleDateFormat formatter = new SimpleDateFormat(
                    Convert.ISO_DATE_FORMAT);
            String dateStr = timeInterval.substring(index + 1);
            ParsePosition pos = new ParsePosition(0);

            Date startTime = formatter.parse(dateStr, pos);
            Date endTime = formatter.parse(dateStr, pos);
            if (startTime == null || endTime == null) {
                break;
            }

            TimeInterval ti = new TimeInterval();

            ti.setStartTime(startTime.getTime()
                    / ImpexConstants.MILLISECS_PER_SEC);
            ti.setEndTime(endTime.getTime() / ImpexConstants.MILLISECS_PER_SEC);

            /*
             * chk if startTime or endTime exceed time_t wraparound and reset
             * accordingly
             */
            if (ti.getStartTime() > TimeInterval.MAX_TIME_T) {
                ti.setStartTime(TimeInterval.MAX_TIME_T);
                impWarning_.invalidAttributes_
                        .addElement("Warning time interval exceeds the maximum date/time value of "
                                + "2038-01-19T03:14:07"
                                + "\n reset time interval to allowed maximum"
                                + " in file " + f.getShortFilename());
            }
            if (ti.getEndTime() > TimeInterval.MAX_TIME_T) {
                ti.setEndTime(TimeInterval.MAX_TIME_T);
                impWarning_.invalidAttributes_
                        .addElement("Warning time interval exceeds the maximum date/time value of "
                                + "2038-01-19T03:14:07"
                                + "\n reset time interval to allowed maximum"
                                + " in file " + f.getShortFilename());
            }

            ti.setPreference(ti_index + 1);

            tiVec.addElement(ti);
            index++;
        }

        TimeInterval[] tiArray = new TimeInterval[tiVec.size()];
        tiVec.copyInto(tiArray);

        ob.setTimeIntervals(tiArray);
    }

    /**
     * Import ob
     * 
     * @param f
     *            PafFile to be imported
     * @param node
     *            Directory Node where object should be created
     * @param obsRun
     *            Observing run to be set in OB created
     * 
     * @exception ObjectIOException
     *                ObjectIOException error reading export file
     * @exception NoSuchMethodException
     *                method does not exist
     * @exception IllegalAccessException
     *                unable to access method
     * @exception InvocationTargetException
     *                unable to invoke method
     * 
     * @return The imported CalibrationBlock
     * 
     * <br>
     * ChangeLog: A M Chavan, 06-Mar-01: changed name and return value. <br>
     * ChangeLog: A M Chavan, 12-jun-01: fixed PR 1381.
     */

    public CalibrationBlock importCalBlock(PafFile f, DirectoryNode node,
            ObservingRun obsRun) throws NoSuchMethodException,
            IllegalAccessException, InvocationTargetException,
            ObjectIOException, IOException {
        CalibrationBlock cb = importCBInternal(f, obsRun);
        cb.setObsRun(obsRun);
        ObjectManager.getObjectManager().recursiveRegister(Media.LOCAL, node,
                cb);
        return (cb);
    }

    private CalibrationBlock importCBInternal(PafFile f, ObservingRun obsRun)
            throws NoSuchMethodException, IllegalAccessException,
            InvocationTargetException, IOException, FileNotFoundException {
        String odName;
        CalibrationBlock cb = (CalibrationBlock) ObjectFactory
                .createRecursive(CalibrationBlock.class);
        importInternal(f, cb, ImpexConstants.OB_MAP, ImpexConstants.OB_EXCLUDE,
                obsRun);
        odName = f.readLine("OBSERVATION.DESCRIPTION.NAME");
        if (odName != null && odName.length() > 0)
            importInternal(f, cb.getOd(), ImpexConstants.OD_MAP_VERSION2,
                    ImpexConstants.OD_EXCLUDE, obsRun);
        TemplateSignature[] tsfs = importTsfs(f, obsRun.getInstCode(), obsRun
                .getIPVersion());
        cb.getOd().setSignatures(tsfs);
        return cb;
    }

    /**
     * Update duplicate parameters. This is called if duplicate parameters from
     * within a template are detected in .obx file
     * 
     * @param templateName
     *            Name of template with duplicate parameter
     * @param paramId
     *            Name of parameter with duplicate parameter
     */

    private void updateDuplicateParams(String templateName, String paramId) {
        // System.out.println("CLASS Import - updateDuplicateParams: Updating
        // duplicate parameters");
        Vector<String> param = null;
        if (!impWarning_.paramDuplicateList_.containsKey(templateName)) {
            param = new Vector<String>();
            impWarning_.paramDuplicateList_.put(templateName, param);
        } else
            param = impWarning_.paramDuplicateList_.get(templateName);
        param.addElement(paramId);
    }

    /**
     * Update param warings hashtable
     * 
     * @param templateName
     *            name of template
     * @param paramId
     *            id of parameter
     */

    private void updateWarningParams(String templateName, String paramId) {
        Vector<String> param = null;
        if (!impWarning_.paramMissingList_.containsKey(templateName)) {
            param = new Vector<String>();
            impWarning_.paramMissingList_.put(templateName, param);
        } else
            param = impWarning_.paramMissingList_.get(templateName);
        param.addElement(paramId);

    }

    /**
     * Update param warings hashtable
     * 
     * @param templateName
     *            name of template
     * @param paramId
     *            id of parameter
     */

    private void updateFixedWarningParams(String templateName, String paramId) {
        Vector<String> param = null;
        if (!impWarning_.paramFixedList_.containsKey(templateName)) {
            param = new Vector<String>();
            impWarning_.paramFixedList_.put(templateName, param);
        } else
            param = impWarning_.paramFixedList_.get(templateName);
        param.addElement(paramId);

    }

    /**
     * Check to see if a template file exists
     * 
     * @param tmplfl
     *            File handle for .tsf file
     * @param templateName
     *            Name of template in .obx file
     * @return True if .tsf is available else false
     */

    private boolean checkFile(File tmplfl, String templateName) {
        if (!tmplfl.exists()
                && !impWarning_.templateMissing_.contains(templateName)) {
            impWarning_.templateMissing_.addElement(templateName);
            return false;
        }
        if (!tmplfl.canRead()
                && !impWarning_.templateAccess_.contains(templateName)) {
            impWarning_.templateAccess_.addElement(templateName);
            return false;
        }
        return true;
    }

    /**
     * If user chanhes value in .obx file but are marked as fixed in .tsf
     * 
     * @param id
     *            Check to see if Param id is in .tsf file
     * @param ts
     *            Refernce to Template Signatre
     * @return boolean value
     */

    private boolean fixedParamWarnings(String id, TemplateSignature ts) {
        boolean fixed = false;
        if (!isTemplate(id)) {
            Parameter[] allPList = ts.getParamList();
            String templateName = ts.getTemplateName();
            for (int l = 0; l < allPList.length; l++) {
                if (allPList[l].getParameterName().equals(id)
                        && allPList[l].isFixed()) {
                    updateFixedWarningParams(templateName, id);
                    return true;
                }
            }
            if (!fixed)
                updateWarningParams(templateName, id);
        }
        return fixed;
    }

    /**
     * Backwards compatibility method to check for Template Name values for
     * version 1.0 and 2.0 .obx files
     * 
     * @param templateName
     * @return boolean value
     * 
     */

    private boolean isTemplate(String templateName) {
        if (templateName.equals("template")
                || templateName.equals("TEMPLATE.NAME")
                || templateName.equals("ACQUISITION.TEMPLATE.NAME"))
            return true;
        return false;
    }

    /**
     * Check to see if it is a Parameter we are processing
     * 
     * @param pafData
     *            an array of PafData
     * @param offset
     *            Offset int array
     * @return boolean If it has a valid Param prefix return true else false
     */

    private boolean isValidParamId(PafData[] pafData, int offset) {
        if (!isTemplate(pafData[offset].getIdentifier())) {
            int index = pafData[offset].getIdentifier().indexOf('.');
            if (index != -1) {
                // validPrefix =
                // pafData[offset].getIdentifier().substring(0,index);
                return true;
            }
        }
        return false;
    }

    /**
     * Process a parameter from an exprted file
     * 
     * @param p
     *            Reference to a parameter p
     * @param newValue
     *            New value of the parameter
     * @param linenumber
     *            Linenuber from exported file
     */

    private void processParameter(Parameter p, String newValue, int linenumber,
            PafFile f) {
        String value = TextUtils.revertEscapedString(newValue);
        if (p.verifyValue(value)) {
            if (p.getType().equals("paramfile")) {
                ((ParamFileParameter) p).setFilename(value);
                File paramFile = new File(currentDir_ + File.separator + value);
                if (paramFile.exists() && paramFile.canRead()) {
                    p.setValue(TextUtils.convertFileToString(paramFile));
                } else {
                    ((ParamFileParameter) p).setFilename("NODEFAULT");
                    impWarning_.invalidAttributes_
                            .addElement("Warning value for parameter "
                                    + p.getParameterName()
                                    + " at linenumber "
                                    + linenumber
                                    + " in  "
                                    + f.getShortFilename()
                                    + " is invalid."
                                    + " \nFile does not exist or is not readable "
                                    + value);
                }
            } else
                p.setValue(value);
        } else {
            if (!value.equals(p.getDefaultValue()))
                impWarning_.invalidAttributes_
                        .addElement("Warning value for parameter "
                                + p.getParameterName() + " at linenumber "
                                + linenumber + " in  " + f.getShortFilename()
                                + " is invalid." + " Default will be used");
        }
    }

    /**
     * Import tsfs
     * 
     * @param instrumentName
     *            instrument associated with template
     * 
     * 
     * @exception IOException
     *                IO error reading export file
     * @return array of TemplateSignatures
     */

    private TemplateSignature[] importTsfs(PafFile f, String instrumentName,
            float version) throws IOException {
        Vector<TemplateSignature> tsfVec = new Vector<TemplateSignature>();
        PafData[] pafData = f.getIdentifierList();
        TemplateSignature ts;
        Instrument instrument = InstrumentList.getInstance().getInstrument(
                instrumentName, version);
        for (int i = 0; i < pafData.length; i++) {
            if (isTemplate(pafData[i].getIdentifier())) {
                Vector param = null;
                String templateName = pafData[i].getValue();

                String filename = instrument.getTSFDirectory() + File.separator
                        + templateName + ".tsf";
                stdlog_.debug("Filename " + filename);

                File tmplfl = new File(filename);
                if (!checkFile(tmplfl, templateName))
                    continue;

                ts = TemplateSignatureFactory.getTemplate(instrument,
                        templateName);

                if (ts == null || ts.getMetaData().getCriticalStatus()) {
                    impWarning_.templateMissing_.addElement(templateName);
                    continue;
                }
                Parameter[] plist = ts.getDisplayParamList();
                int j = i + 1;
                Hashtable<String, String> pList = new Hashtable<String, String>();
                Hashtable<String, String> duplicates = new Hashtable<String, String>();
                if (plist != null && plist.length > 0) {
                    while (ImpexConstants.BO_LIST.indexOf(pafData[j]
                            .getIdentifier()) == -1) {
                        boolean valueFound = false;

                        for (int k = 0; k < plist.length; k++) {
                            if (pafData[j].getIdentifier().equals(
                                    plist[k].getParameterName())) {
                                if (!pList
                                        .contains(plist[k].getParameterName()))
                                    pList.put(plist[k].getParameterName(),
                                            plist[k].getParameterName());
                                else
                                    duplicates.put(plist[k].getParameterName(),
                                            plist[k].getParameterName());
                                processParameter(plist[k], pafData[j]
                                        .getValue(),
                                        pafData[j].getLineNumber(), f);
                                valueFound = true;
                                break;
                            }
                        } // end process parameter
                        if (!valueFound) {
                            fixedParamWarnings(pafData[j].getIdentifier(), ts);
                        }
                        if (j < pafData.length - 1) {
                            j++;
                            if (!isValidParamId(pafData, j))
                                break; // break out of while loop
                        } else
                            break; // break out of while loop

                    } // end while BO_LIST
                }
                Enumeration<String> enum0 = duplicates.keys();
                while (enum0.hasMoreElements()) {
                    String paramName = enum0.nextElement();
                    updateDuplicateParams(templateName, paramName);
                }
                tsfVec.addElement(ts);
            } // process template
        }
        TemplateSignature[] tsfs = new TemplateSignature[tsfVec.size()];
        tsfVec.copyInto(tsfs);
        return tsfs;
    }

    private class ImportWarning {

        private Hashtable<String, Vector<String>> paramFixedList_ = new Hashtable<String, Vector<String>>();

        /**
         * Parameters that are not in the current .tsf file but were originally
         * exported
         */

        private Hashtable<String, Vector<String>> paramMissingList_ = new Hashtable<String, Vector<String>>();

        /**
         * Duplicate parameters in .tsf fiel
         */

        private Hashtable<String, Vector<String>> paramDuplicateList_ = new Hashtable<String, Vector<String>>();

        /**
         * Templates that are not in the current instrument directory but were
         * originally exported
         */

        private Vector<String> templateMissing_ = new Vector<String>();

        /**
         * No read access to a template so it cannot be imported
         */

        private Vector<String> templateAccess_ = new Vector<String>();

        /**
         * Attributes of business objects which fail verification
         */

        private Vector<String> invalidAttributes_ = new Vector<String>();

        private Vector<String> fcNotFound = new Vector<String>();

        private Vector<String> fcTooBig = new Vector<String>();

        private Vector<String> tooManyFC = new Vector<String>();

        private Vector<String> duplicateNameFC = new Vector<String>();

        private Vector<String> duplicateContentsFC = new Vector<String>();

        /**
         * Imported file generated warnings
         * 
         * @param return
         *            has warnings
         */

        public boolean hasWarning() {
            if (templateAccess_.size() > 0 || invalidAttributes_.size() > 0
                    || templateMissing_.size() > 0
                    || paramMissingList_.size() > 0
                    || paramDuplicateList_.size() > 0
                    || paramFixedList_.size() > 0 || fcNotFound.size() > 0
                    || fcTooBig.size() > 0 || tooManyFC.size() > 0
                    || duplicateNameFC.size() > 0
                    || duplicateContentsFC.size() > 0)
                return true;
            return false;
        }

        /**
         * Warning string
         * 
         * @return warning string
         * 
         */

        public String toString() {
            StringBuffer buf = new StringBuffer();
            templateReadError(buf);
            templateMissingError(buf);
            fcFoundError(buf);
            fcTooBigError(buf);
            fcTooManyError(buf);
            duplicateFCNameError(buf);
            fcDuplicateContentsError(buf);
            paramMissingError(buf);
            paramFixedError(buf);
            paramDuplicateError(buf);
            invalidAttributeErrors(buf);
            return buf.toString();
        }

        /**
         * @param buf
         */
        private void invalidAttributeErrors(StringBuffer buf) {
            for (int i = 0; i < invalidAttributes_.size(); i++) {
                buf.append((String) invalidAttributes_.elementAt(i));
                buf.append("\n");
            }
        }

        /**
         * @param buf
         */
        private void paramDuplicateError(StringBuffer buf) {
            Enumeration<String> tmplDuplicateKey = paramDuplicateList_.keys();
            while (tmplDuplicateKey.hasMoreElements()) {
                String tmplName = tmplDuplicateKey.nextElement();
                buf
                        .append("Warning the following parameters were duplicated in "
                                + tmplName + "\n");
                buf.append("last value will be used");
                buf.append("\n");
                Vector warning = paramDuplicateList_.get(tmplName);
                for (int i = 0; i < warning.size(); i++) {
                    buf.append((String) warning.elementAt(i) + "\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void paramFixedError(StringBuffer buf) {
            Enumeration<String> tmplFixedKey = paramFixedList_.keys();
            while (tmplFixedKey.hasMoreElements()) {
                String tmplName = tmplFixedKey.nextElement();
                buf
                        .append("Warning the following parameters were not imported for "
                                + tmplName + "\n");
                buf
                        .append("because their values are fixed and cannot be edited:");
                buf.append("\n");
                Vector warning = paramFixedList_.get(tmplName);
                for (int i = 0; i < warning.size(); i++) {
                    buf.append((String) warning.elementAt(i) + "\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void paramMissingError(StringBuffer buf) {
            Enumeration<String> tmplKey = paramMissingList_.keys();
            while (tmplKey.hasMoreElements()) {
                String tmplName = tmplKey.nextElement();
                buf
                        .append("Warning the following parameters were not imported for "
                                + tmplName);
                buf.append("\n");
                Vector warning = paramMissingList_.get(tmplName);
                for (int i = 0; i < warning.size(); i++) {
                    buf.append((String) warning.elementAt(i) + "\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void fcDuplicateContentsError(StringBuffer buf) {
            if (duplicateContentsFC.size() > 0) {
                buf
                        .append("The following FindingCharts could not be imported ");
                buf
                        .append("because associated a finding chart with the same contents already exits\n");
                for (int i = 0; i < duplicateContentsFC.size(); i++) {
                    buf.append(duplicateContentsFC.elementAt(i)
                            + " not found\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void duplicateFCNameError(StringBuffer buf) {
            if (duplicateNameFC.size() > 0) {
                buf
                        .append("The following FindingCharts could not be imported ");
                buf
                        .append("because associated a finding chart with the same name already exits\n");
                for (int i = 0; i < duplicateNameFC.size(); i++) {
                    buf.append(duplicateNameFC.elementAt(i) + " not found\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void fcTooManyError(StringBuffer buf) {
            if (tooManyFC.size() > 0) {
                buf
                        .append("The following FindingCharts could not be imported ");
                buf
                        .append("because the number of finding charts attached would have exceeded the maximum "
                                + AppConfig.getAppConfig()
                                        .getMaxFindingChartSize() + "\n");
                for (int i = 0; i < tooManyFC.size(); i++) {
                    buf.append(tooManyFC.elementAt(i) + " not found\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void fcTooBigError(StringBuffer buf) {
            if (fcTooBig.size() > 0) {
                buf
                        .append("The following FindingCharts could not be imported ");
                buf.append("because their size exceeds the maximum limit of "
                        + AppConfig.getAppConfig().getMaxFindingChartSize()
                        + "\n");
                for (int i = 0; i < fcTooBig.size(); i++) {
                    buf.append(fcTooBig.elementAt(i) + " not found\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void fcFoundError(StringBuffer buf) {
            if (fcNotFound.size() > 0) {
                buf
                        .append("The following FindingCharts could not be imported ");
                buf.append("because associated files no longer exist\n");
                for (int i = 0; i < fcNotFound.size(); i++) {
                    buf.append(fcNotFound.elementAt(i) + " not found\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void templateMissingError(StringBuffer buf) {
            if (templateMissing_.size() > 0) {
                buf.append("The following templates could not be imported ");
                buf.append("because associated tsfs no longer exist\n");
                for (int i = 0; i < templateMissing_.size(); i++) {
                    buf.append(templateMissing_.elementAt(i) + " not found\n");
                }
            }
        }

        /**
         * @param buf
         */
        private void templateReadError(StringBuffer buf) {
            if (templateAccess_.size() > 0) {
                buf.append("The following templates could not be imported ");
                buf
                        .append("because associated tsf do not have read permissions\n");
                for (int i = 0; i < templateAccess_.size(); i++) {
                    buf.append(templateAccess_.elementAt(i)
                            + " no read permissions\n");
                }
            }
        }
    }
}
