package org.eso.ohs.persistence;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eso.ohs.core.utilities.ReflectionException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.EphemerisFile;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentConstraintConfig;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Class OBDifferenceBuilder
 *
 * This class allows building of the difference xml structure, 
 * making use of JDom library, a configuration file and reflection
 * The fields of each object are compared given a configuration read from <code>configFile_</code> 
 * Each method returns a JDom tree node or a vector containing nodes
 *  
 *
 * @version $Revision: 1.6 $
 * @author pnunes, Jan 26, 2005
 */

// $Id: OBDifferenceBuilder.java,v 1.6 2006/04/26 09:36:38 ddorigo Exp $ 
public class OBDifferenceBuilder {
        
    private Document configDoc_;    
    private  String configFile_; 
        
    /**
     * Constructs the object
     */
    public OBDifferenceBuilder(String configFile) {
            
        configFile_ = configFile;
        
        try {
            readConfiguration();

        } catch (JDOMException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }       
    }
    
    private void readConfiguration() throws JDOMException, IOException {
        configDoc_ = new Document();
        SAXBuilder builder = new SAXBuilder();
        File cfgFile = new File(configFile_);
        configDoc_ = builder.build(cfgFile);
    }
    
    /**
     * Computes the differences between two <code>TimeInterval</code> objects and returns a JDom <code>Element</code> object
     * 
     * @param timeInterval1
     * @param timeInterval2
     * @return <code>Element</code> object
     * @throws ReflectionException
     */
    public Element differenceTimeInterval(Element props , String tiType , TimeInterval timeInterval1, TimeInterval timeInterval2  ) throws ReflectionException {
                      
        // root node
        Element timeInterval = new Element(tiType);       
        List childs = props.getChildren();
        Class cls = TimeInterval.class;        
        compareProperties(cls,timeInterval,childs,timeInterval1, timeInterval2,"Equal");              
        return(timeInterval);                
    }
    
    /**
     * Computes the differences between two <code>FindingChart</code> objects and returns a JDom <code>Element</code> object
     * 
     * @param chartOne
     * @param chartTwo
     * @return <code>Element</code> object
     * @throws ReflectionException
     */
    public Element differenceFindingCharts(Element props ,FindingChart chartOne, FindingChart chartTwo) throws ReflectionException {
        
        //  root node
        Element findingChart = new Element("FindingChartProps");

        List childs = props.getChildren();
        Class cls = FindingChart.class;        
        compareProperties(cls,findingChart,childs,chartOne, chartTwo, "Equal");
                        
        return(findingChart);

    }



    /**
     * Computes the differences between two <code>EphemerisFile</code> objects and returns a JDom <code>Element</code> object
     * 
     * @param ef1
     * @param ef2
     * @return <code>Element</code> object
     * @throws ReflectionException
     */
    public Element differenceEphemerisFiles(Element props, EphemerisFile ef1, EphemerisFile ef2) throws ReflectionException {

        // root node
        Element ephemerisFile = new Element("EphemerisFileProps");
        
        List childs = props.getChildren();
        Class cls = EphemerisFile.class;        
        compareProperties(cls,ephemerisFile,childs,ef1, ef2,"Equal");

        return(ephemerisFile);
    }



    /**
     * Computes the differences between two <code>Target</code> objects and returns a JDom <code>Element</code> object
     * 
     * @param targetOne
     * @param targetTwo
     * @return <code>Element</code> object
     * @throws ReflectionException
     */
    public Element differenceTargets(Element props ,Target targetOne, Target targetTwo) throws ReflectionException {
        
        //   root node
        Element target = new Element("TargetProps");                
        
        List childs = props.getChildren();
        Class cls = Target.class;        
        compareProperties(cls,target,childs,targetOne, targetTwo,"Equal");
               
        return(target);

    }



    /**
     * Computes the differences between two <code>Parameter</code> objects and returns a JDom <code>Element</code> object
     * 
     * @param p1
     * @param p2
     * @return <code>Element</code> object
     * @throws ReflectionException
     */
    public Element differenceParameters(Element props,Parameter p1, Parameter p2) throws ReflectionException {
        
        // root node
        Element parameter = new Element("DisplayParameter");
        
        List childs = props.getChildren();
        Class cls = Parameter.class;        
        compareProperties(cls,parameter,childs,p1, p2,"Equal");
                
        return (parameter);
    }


    /**
     * Computes the differences between two <code>TemplateSignature</code> objects and returns a JDom <code>Element</code> object
     * 
     * @param ts1
     * @param ts2
     * @return <code>Element</code> object
     * @throws OBDifferenceConfigurationException
     * @throws ReflectionException
     */
    public Element differenceTemplateSignatures(String tsType,TemplateSignature ts1, TemplateSignature ts2) throws OBDifferenceConfigurationException, ReflectionException {
        
        //  root node
        Element templateSignature = new Element(tsType);
        
        
        
        //    Name
        String name = "Name";
        String label = "Name";
        String value1 = ts1.getTemplateName();
        String value2 =  ts2.getTemplateName();
        
        Element templateName = equalsChangedDiffNode(name,label,value1,value2);
        templateSignature.addContent(templateName);
        
        
                        
        
        // if template signatures are equal
        // iterate on parameterLists for the templateSignatures,
        // assuming different parameter order  in the templates
        // adds each difference element to templateSignature's content
        
        if (  ( ts1.getInstrumentName().compareTo(ts2.getInstrumentName()) == 0 ) && 
               (ts1.getTemplateName().compareTo(ts2.getTemplateName()) == 0 )
        	)  
        
        {
        
            Parameter [] plist1 = ts1.getDisplayParamList();
            Parameter [] plist2 = ts2.getDisplayParamList();
            
        
            // 	iterates the first parameter list, trying to find matches in the second
            // if it doesn't find it, marks it as "deleted"
            for (int i = 0; i < plist1.length; i++) {
                Parameter p1 = plist1[i];
                Parameter p2 = null;
            
                // try to find it in the second parameter list
                for (int j = 0; j < plist2.length; j++) {

                
                    // if parameter is the same 
                    if (  p1.getName().compareTo( plist2[j].getName() ) == 0 )
                        p2 = plist2[j];                                
                }
            
                Element diffParameter;
            
                if ( p2 != null ) {
                    // parameter is in the 2nd template
                    // calls diffParameter method 
                    Element props = findConfig("Parameter");
                    diffParameter = differenceParameters(props,p1,p2);
                }
                else  {
                    // its not in the 2nd template     
                
                
                    diffParameter = new Element("DisplayParameter");
                
                    Element props = findConfig("Parameter");
                    List childs = props.getChildren();
                    Class cls = Parameter.class;        
                    compareProperties(cls,diffParameter,childs,p1, p1,"Deleted");
                    
                    /*
                    String nameP = "ParameterName";
                    String labelP = "Name";
                    String valueP = p1.getLabel();
                    Element diffParameterName = addedDeletedDiffNode(nameP,labelP,valueP,true);
                
                    String nameV = "Value";
                    String labelV = "Value";
                    String valueV = p1.getValue();
                    Element diffParameterValue = addedDeletedDiffNode(nameV,labelV,valueV,true);
                
                    
                    
                    
                    diffParameter.addContent(diffParameterName);
                    diffParameter.addContent(diffParameterValue);

					*/
                }
                
            
                // 	adds it to templateSignature's content
                templateSignature.addContent(diffParameter);
            
            } // for
        
        
            //  iterates the second parameter list, trying to find matches in the first
            // if it doesn't find it, marks it as "added"
            for (int i = 0; i < plist2.length; i++) {
                Parameter p1 = plist2[i];
                Parameter p2 = null;
            
                // 	try to find it in the first parameter list
                for (int j = 0; j < plist1.length; j++) {

                    // if parameter is the same 
                    if (  p1.getName().compareTo( plist1[j].getName() ) == 0 )
                        p2 = plist1[j];                                
                }
            
                Element diffParameter = new Element("DisplayParameter");
            
                if ( p2 == null ) {
                    //  its not in the 1st template, just consider this case                
                    diffParameter = new Element("DisplayParameter");
                
                    
                    Element props = findConfig("Parameter");
                    List childs = props.getChildren();
                    Class cls = Parameter.class;        
                    compareProperties(cls,diffParameter,childs,p1, p1,"Added");
                    
                    /*
                    String nameP = "ParameterName";
                    String labelP = "Name";
                    String valueP = p1.getLabel();
                    Element diffParameterName = addedDeletedDiffNode(nameP,labelP,valueP,false);
                
                    String nameV = "Value";
                    String labelV = "Value";
                    String valueV = p1.getValue();
                    Element diffParameterValue = addedDeletedDiffNode(nameV,labelV,valueV,false);
                
                    diffParameter.addContent(diffParameterName);
                    diffParameter.addContent(diffParameterValue);
                    */
                    
                    
                    
                    
                    //  adds it to templateSignature's content
                    templateSignature.addContent(diffParameter);
                }

            } // for
            
        } // if templates are equal
        
        return templateSignature;
    }
    
    
    
    /**
     * Computes the difference between two <code>ConstraintSet</code> objects,
     * returning a JDom <code>Element</code> object
     *      * 
     * 
     * Two cases: 
     *  -  ipVersions are equal, assumes the same properties and compares them
     *  -  for different ipVersions, adds deleted or added according to each case
     * 
     * @param cs1
     * @param cs2
     * @param propsCS1
     * @param propsCS2
     * @param ipVersionCS1
     * @param ipVersionCS2
     * @return  <code>Element</code> object
     */
    public Element differenceConstraintSet(ConstraintSet cs1, ConstraintSet cs2, String[] propsCS1, String[] propsCS2, float ipVersionCS1  , float ipVersionCS2) throws ReflectionException { 
        
        Class cls = ConstraintSet.class;
        Method get;
        
        //      root node
        Element constraintSetProps = new Element("ConstraintSetProps");
        
        //  property
        Element property;          
        Element label;
        Element value1;           
        Element value2;               
        Element status;
        
        try {
        
            // same ip version - compare same properties
            if ( ipVersionCS1 == ipVersionCS2 ) {
            
                for (int i = 0; i < propsCS1.length; i++) {
                    String propertyName = propsCS1[i];
                
                    get = cls.getMethod("get" + propertyName, new Class[] {});
                    Object obj1;                
                    obj1 = get.invoke(cs1, new Object[] {});
                    
                    Object obj2;
                    obj2 = get.invoke(cs2, new Object[] {});
                                                           
                    
                    String nameS = propertyName;
                    String labelS =  propertyName;
                    String value1S = obj1.toString();
                    String value2S =  obj2.toString() ;
                    
                    property = equalsChangedDiffNode(nameS,labelS,value1S,value2S);
                    constraintSetProps.addContent(property);                                        

                }
         
            } else {
        
                // different ip version - iterate over property arrays 
            
            
                //      find out if there are any deleted ones
                for (int i = 0; i < propsCS1.length; i++) {
                    String propertyName1 = propsCS1[i];
                    String propertyName2 = null;
    								
                    for (int j = 0; j < propsCS2.length; j++) {
    			
                        if ( propertyName1.compareTo(propsCS2[j]) == 0 ) {
                            // 	found it
                            propertyName2 = propsCS2[j];
                            break;
    									
                        }
                    }
    		
    		
                    if ( propertyName2 != null ) {
                        // 	it was found - not deleted
                    
                        get = cls.getMethod("get" + propertyName1, new Class[] {});
                        Object obj1;                
                        obj1 = get.invoke(cs1, new Object[] {});
                    
                        Object obj2;
                        obj2 = get.invoke(cs2, new Object[] {});
                    
                        String nameS = propertyName1;
                        String labelS = propertyName1;
                        String value1S = obj1.toString();
                        String value2S =  obj2.toString() ;
                        
                        property = equalsChangedDiffNode(nameS,labelS,value1S,value2S);                        
                        constraintSetProps.addContent(property);                                        

    	
                    // 	not found - deleted
                    } else {
                    
                        get = cls.getMethod("get" + propertyName1, new Class[] {});
                        Object obj1;                
                        obj1 = get.invoke(cs1, new Object[] {});
        
                        property = new Element(propertyName1);
                    
                        label = new Element("Label");                    
                        label.addContent(propertyName1);
                    
                        value1 = new Element("Value1");
                        value1.addContent( obj1.toString() );
                    
                        value2 = new Element("Value2");
                        value2.addContent( "" ); 
                    
                        status = new Element("Status");                          
                        status.addContent("Deleted");
                    
                        //  add contents of parameterValue
                        property.addContent(label);
                        property.addContent(value1);
                        property.addContent(value2);
                        property.addContent(status);  
                        
                        constraintSetProps.addContent(property);                                        
                      
                    }
    		
                }
            
            
                // find out added ones
            
                for (int i = 0; i < propsCS2.length; i++) {
                    String propertyName2 = propsCS2[i];
                    String propertyName1 = null;
                
                    for (int j = 0; j < propsCS1.length; j++) {
                    
                        // found it
                        if ( propertyName2.compareTo(propsCS1[j]) == 0 ) {
                            propertyName1 = propsCS1[j];
                        	break;
                        }
                    }
                
                    // it wasn't found - its an added one
                    if ( propertyName1 == null ) {
                    
                        get = cls.getMethod("get" + propertyName2, new Class[] {});
                        Object obj2;                
                        obj2 = get.invoke(cs2, new Object[] {});
    
                        
                        property = new Element(propertyName2);
                    
                        label = new Element("Label");                    
                        label.addContent(propertyName2);
                    
                        value1 = new Element("Value1");
                        value1.addContent( "" );
                    
                        value2 = new Element("Value2");
                        value2.addContent( obj2.toString() ); 
                    
                        status = new Element("Status");                          
                        status.addContent("Added");
                    
                        //  add contents of parameterValue
                        property.addContent(label);
                        property.addContent(value1);
                        property.addContent(value2);
                        property.addContent(status);  
                    
                        constraintSetProps.addContent(property);                                                           
                } // if                
            } // for 
            
        } // else
        
        } catch (SecurityException e) {                
            throw new ReflectionException(e.toString());
        } catch (NoSuchMethodException e) {
            throw new ReflectionException(e.toString());
        } catch (IllegalArgumentException e) {
            throw new ReflectionException(e.toString());
        } catch (IllegalAccessException e) {
            throw new ReflectionException(e.toString());
        } catch (InvocationTargetException e) {
            throw new ReflectionException(e.toString());
        }
        
    
        
        return (constraintSetProps);
    }

    
    /**
     * Computes the difference between own properties of two <code>ObservationBlock</code> objects
     * 
     * @param props
     * @param ob1
     * @param ob2
     * @return an <code>Element</code> object
     * @throws ReflectionException
     */

    public Element differenceCBProps(Element props, CalibrationBlock cb1, CalibrationBlock cb2) throws ReflectionException {
        
        Element cbProps = new Element("OBProps");                
        List childs = props.getChildren();
        Class cls = CalibrationBlock.class;        
        compareProperties(cls,cbProps,childs,cb1, cb2,"Equal");
        
        return(cbProps);
        
    }
    
    /**
     * Computes the difference between two <code>ObservatinBlock</code> objects
     * 
     * @param obProps
     * @param props
     * @param ob1
     * @param ob2
     * @return <code>Element</code> object
     * @throws ReflectionException
     */
    public void differenceOBProps(Element obProps, Element props, ObservationBlock ob1, ObservationBlock ob2) throws ReflectionException {        
                    
        List childs = props.getChildren();
        Class cls = ObservationBlock.class;              
        compareProperties(cls,obProps,childs,ob1, ob2, "Equal");                                   
                
    }

    

    /**
     * Computes the differences between two observation blocks
     * For each class, reads the configuration and passes an JDom <code>Element</code> object
     * as a parameter to the respective method. Each method will then iterate this configuration in order 
     * compute the differences
     * 
     * @param ob1
     * @param ob2
     * @return a <code>Document</code> object, representing the difference structure tree
     * @throws ReflectionException
     * @throws OBDifferenceConfigurationException
     */
    public Document differenceObservationBlock(ObservationBlock ob1, ObservationBlock ob2, String [] propsCS1, String [] propsCS2  ) throws ReflectionException, OBDifferenceConfigurationException {
                                       
        Document retDoc = new Document();
        Element obDefinition = new Element("OBDEFINITION");
        retDoc.setRootElement(obDefinition);        
        
        // OB properties
        
        // inherited from CB        
        Element props = findConfig("CalibrationBlock");
        Element cbProps = differenceCBProps(props,ob1,ob2);
                
        // own properties        
        props = findConfig("ObservationBlock");
        differenceOBProps(cbProps,props,ob1,ob2);
        
        obDefinition.addContent(cbProps);
        
        
        // Target props
        props = findConfig("Target");
        Element targetProps = differenceTargets(props,ob1.getTarget(),ob2.getTarget());
        obDefinition.addContent(targetProps);
        
        
        // Finding charts
        Vector<Element> findingCharts = this.differenceFindingChartArrays(ob1.getFindingCharts(),ob2.getFindingCharts());
        for (Iterator<Element> iter = findingCharts.iterator(); iter.hasNext();) {
            Element fChart = iter.next();
            obDefinition.addContent(fChart);            
        }
        
        // Time intervals
        Vector<Element> timeIntervals = differenceTimeIntervalArrays("TimeIntervalProps",ob1.getTimeIntervals(),ob2.getTimeIntervals());
        for (Iterator<Element> iter = timeIntervals.iterator(); iter.hasNext();) {
            Element tInt = iter.next();
            obDefinition.addContent(tInt);            
        }
        
        // STTime intervals
        Vector<Element> sttimeIntervals = differenceTimeIntervalArrays("STTimeIntervalProps",ob1.getSTTimeIntervals(),ob2.getSTTimeIntervals());
        for (Iterator<Element> iter = sttimeIntervals.iterator(); iter.hasNext();) {
            Element stInt = iter.next();
            obDefinition.addContent(stInt);            
        }
        
        // Constraint sets        
        Element constraintSets = differenceConstraintSet(ob1.getConstraintSet(),ob2.getConstraintSet(), propsCS1, propsCS2 , ob1.fetchIPVersionFromOd() , ob2.fetchIPVersionFromOd());
        obDefinition.addContent(constraintSets);
        
        // Ephemeris file
        EphemerisFile ef1 , ef2;
        
        if (ob1.hasEphemerisFile() ) 
            ef1 = ob1.getEphemerisFile();
        else
            ef1 = new EphemerisFile();
        
        if (ob2.hasEphemerisFile() ) 
            ef2 = ob2.getEphemerisFile();
        else
            ef2 = new EphemerisFile();
        
        props = findConfig("EphemerisFile");
        
        Element ephemerisFile = differenceEphemerisFiles(props,ef1,ef2);
        obDefinition.addContent(ephemerisFile);
        
        // Acquisition template
        Element acquisitionTemplate = differenceTemplateSignatures("AcquisitionTemplate",ob1.getTemplateSignature(),ob2.getTemplateSignature());
        obDefinition.addContent(acquisitionTemplate);
        
        // observation description
        
        props = findConfig("ObservationDescription");
        Element observationDescription = differenceObservationDescription(props,ob1.getOd(),ob2.getOd());
        Vector<Element> templateVector = this.differenceTemplateSignatureArrays(ob1.getOd().getSignatures(),ob2.getOd().getSignatures());  
        for (Iterator<Element> iter = templateVector.iterator(); iter.hasNext();) {
            Element templateSig = iter.next();
            observationDescription.addContent(templateSig);
        }
        obDefinition.addContent(observationDescription);

        return retDoc;
    }

    /**
     * Computes the differences between two calibration blocks
     * 
     * @param ob1
     * @param ob2
     * @return a <code>Document</code> object, representing the difference structure tree
     * @throws ReflectionException
     * @throws ReflectionException
     * @throws OBDifferenceConfigurationException
     */
    public Document differenceCalibrationBlock(CalibrationBlock cb1, CalibrationBlock cb2, String [] propsCS1, String [] propsCS2  ) throws ReflectionException, OBDifferenceConfigurationException {
       
        //    get a list of all configuration  elements:
        Element root = configDoc_.getRootElement();
        List allChildren = root.getChildren();
        Element config;
        
        Document retDoc = new Document();
        Element obDefinition = new Element("OBDEFINITION");
        retDoc.setRootElement(obDefinition);
        
        // CB  properties       
        Element props = findConfig("CalibrationBlock");        
        
        Element cbProps = differenceCBProps(props,cb1,cb2);
        obDefinition.addContent(cbProps);
                       
        
        // observation description
        props = findConfig("ObservationDescription");
        Element observationDescription = differenceObservationDescription(props,cb1.getOd(),cb2.getOd());
        Vector<Element> templateVector = differenceTemplateSignatureArrays(cb1.getOd().getSignatures(),cb2.getOd().getSignatures());  
        for (Iterator<Element> iter = templateVector.iterator(); iter.hasNext();) {
            Element templateSig = iter.next();
            observationDescription.addContent(templateSig);
        }
        obDefinition.addContent(observationDescription);               
        
        return retDoc;
    }
    
    /**
     * Computes the differences between two <code>FindingChart</code> arrays
     * 
     * @param fc1
     * @param fc2
     * @return a <code>Vector</code> of <code>Element</code> nodes
     * @throws ReflectionException
     * @throws OBDifferenceConfigurationException
     */
    public Vector<Element> differenceFindingChartArrays( FindingChart [] fc1, FindingChart [] fc2  ) throws ReflectionException, OBDifferenceConfigurationException {
        
        //  get configuration of comparison
        Element props = findConfig("FindingChart");
        
        Vector<Element> findingChartsVec = new Vector<Element>();        
        
        // iterate the 1st array
        for (int i = 0; i < fc1.length; i++) {
            FindingChart fChart1 = fc1[i];
            FindingChart fChart2 = null;
            
            // try to find it in the 2nd array
            for (int j = 0; j < fc2.length; j++) {
                String fcName1 = fChart1.getFcName();
                String fcName2 = fc2[j].getFcName();
                
                // if the names match
                if ( fcName1.compareTo(fcName2) == 0) {
                    fChart2 = fc2[j];
                }                             
            }     
            
            Element findingChart ;
            // found it, check if it changed
            if ( fChart2 != null) {
                findingChart = differenceFindingCharts(props,fChart1,fChart2);
                findingChartsVec.add(findingChart);
            
            // not found
            } else {
                findingChart = new Element("FindingChartProps");
                
                
                List childs = props.getChildren();
                Class cls = FindingChart.class;        
                compareProperties(cls,findingChart,childs,fChart1, fChart1,"Deleted");
                
                /*
                
                String name = "CheckSum";
                String label = "Check sum";
                String value = fChart1.getCheckSum()+ "";
                Element checkSum = this.addedDeletedDiffNode(name,label,value,true);
                
                name = "FcName";
                label = "fcName";
                value = fChart1.getFcName();
                Element fcName = this.addedDeletedDiffNode(name,label,value,true);
                
                findingChart.addContent(checkSum);
                findingChart.addContent(fcName);
                
                */
                
                findingChartsVec.add(findingChart);
                
            }
                        
        }
        
        // iterate the 2nd array
        for (int i = 0; i < fc2.length; i++) {
            FindingChart fChart2 = fc2[i];
            FindingChart fChart1 = null;
            
            // look for missing ones in the first
            for (int j = 0; j < fc1.length; j++) {
                String fcName2 = fChart2.getFcName();
                String fcName1 = fc1[j].getFcName();
                
                                
                // if names match
                if ( fcName1.compareTo(fcName2) == 0) {
                    fChart1 = fc1[j];
                }
                
            }
            
            Element findingChart ;
            
            // if it wasn't found
            if ( fChart1 == null ) {
                findingChart = new Element("FindingChartProps");
                
                
                List childs = props.getChildren();
                Class cls = FindingChart.class;        
                compareProperties(cls,findingChart,childs,fChart2, fChart2,"Added");
                
                /*
                String name = "CheckSum";
                String label = "Check sum";
                String value = fChart2.getCheckSum()+ "";
                Element checkSum = this.addedDeletedDiffNode(name,label,value,false);
                
                name = "FcName";
                label = "fcName";
                value = fChart2.getFcName();
                Element fcName = this.addedDeletedDiffNode(name,label,value,false);
                
                findingChart.addContent(checkSum);
                findingChart.addContent(fcName);
                
                */
                
                
                findingChartsVec.add(findingChart);

            }

        }
        
        return(findingChartsVec);
    } 
    
    /**
     * Computes the differences between two <code>TimeInterval</code> arrays
     * @param name
     * @param ti1
     * @param ti2
     * @return a <code>Vector</code> object, containing <code>Element</code> nodes
     * @throws ReflectionException
     * @throws OBDifferenceConfigurationException
     */
    public Vector<Element> differenceTimeIntervalArrays(String tiType, TimeInterval [] ti1 , TimeInterval [] ti2) throws ReflectionException, OBDifferenceConfigurationException {
        
        // gets configuration for comparison       
        Element props = findConfig("TimeInterval");
                                
        Vector<Element> timeIntervals = new Vector<Element>();
        int minSize;
        
        if (ti1.length <= ti2.length) 
            minSize = ti1.length;
        else
            minSize = ti2.length;
           
        // loop until the size of the smaller array
        // and produce "equal" or "changed" nodes
        for (int i = 0; i < minSize; i++) {
            TimeInterval timeInterval1 = ti1[i];
            TimeInterval timeInterval2 = ti2[i];
            
            Element diffTimeInterval = differenceTimeInterval(props,tiType,timeInterval1,timeInterval2);
            timeIntervals.add(diffTimeInterval);                
        } 
        
        String name, label, value;
        
        // here produce "deleted" nodes
        for (int i = minSize; i < ti1.length; i++) {
            TimeInterval interval = ti1[i];
            
            Element timeInt = new Element(tiType);
            
            
            List childs = props.getChildren();
            Class cls = TimeInterval.class;        
            compareProperties(cls,timeInt,childs,interval, interval,"Deleted");
            
            /*
            name = "StartTime";
            label = "Start time";
            value = interval.getStartTime() + "";
            Element startTime = this.addedDeletedDiffNode(name,label,value,true);
            timeInt.addContent(startTime);
            
            name = "EndTime";
            label = "End time";
            value = interval.getEndTime() + "";
            Element endTime = this.addedDeletedDiffNode(name,label,value,true);
            timeInt.addContent(endTime);
            
            */
            
            timeIntervals.add(timeInt);
            
        }
        
        
        // here produce "added" nodes
        for (int i = minSize; i < ti2.length; i++) {
            TimeInterval interval = ti2[i];
            
            Element timeInt = new Element(tiType);
            
            
            List childs = props.getChildren();
            Class cls = TimeInterval.class;        
            compareProperties(cls,timeInt,childs,interval, interval,"Added");
            
            /*
            name = "StartTime";
            label = "Start time";
            value = interval.getStartTime() + "";
            Element startTime = this.addedDeletedDiffNode(name,label,value,false);
            timeInt.addContent(startTime);
            
            name = "EndTime";
            label = "End time";
            value = interval.getEndTime() + "";
            Element endTime = this.addedDeletedDiffNode(name,label,value,false);
            timeInt.addContent(endTime);
            */
            
            timeIntervals.add(timeInt);
        }
        
        

        return(timeIntervals);
        
    }

    /**
     * Computes the differences between two <code>ObservationBlock</code> objects
     * 
     * @param od1
     * @param od2
     * @return <code>Element</code> object
     * @throws ReflectionException
     */
    public Element differenceObservationDescription(Element props,ObservationDescription od1 , ObservationDescription od2  ) throws ReflectionException {
        //      root node
        Element obsDescription = new Element("ObservationDescription");
       
        List childs = props.getChildren();
        Class cls = ObservationDescription.class;        
        compareProperties(cls,obsDescription,childs,od1, od2,"Equal");
        
        /*
        // Instrument
        String name = "Instrument";
        String label = "Instrument";
        String value1 = od1.getInstrument();
        String value2 = od2.getInstrument();
        
        Element instrument = equalsChangedDiffNode(name,label,value1,value2);
         
        
        // UserComments
        
        name = "UserComments";
        label = "User comments";
        value1 = od1.getUserComments();
        value2 = od2.getUserComments();
        Element userComments = equalsChangedDiffNode(name,label,value1,value2);
        
        
        obsDescription.addContent(instrument);
        obsDescription.addContent(userComments);
        
        */
        
        
        return(obsDescription);
        
        
    }
    
    /**
     * Computes the differences between two <code>TemplateSignature</code> arrays
     * 
     * @param tsArray1
     * @param tsArray2
     * @return a <code>Vector</code> object containing <code>Element</code> objects
     * @throws OBDifferenceConfigurationException
     * @throws ReflectionException
     */
    public Vector<Element> differenceTemplateSignatureArrays( TemplateSignature [] tsArray1 , TemplateSignature [] tsArray2 ) throws OBDifferenceConfigurationException, ReflectionException {
        
        Vector<Element> temSigVec = new Vector<Element>();
        int minSize;
        
        if (tsArray1.length <= tsArray2.length) 
            minSize = tsArray1.length; 
        else
            minSize = tsArray2.length;
        
        
        // compare side-by-side
        for (int i = 0; i < minSize; i++) {
            TemplateSignature ts1 =  tsArray1[i];
            TemplateSignature ts2 =  tsArray2[i];            
            
            Element tempSig = differenceTemplateSignatures("TemplateSignature",ts1,ts2);
            temSigVec.add(tempSig);
        }
        
        // add deleted nodes
        for (int i = minSize; i < tsArray1.length; i++ ) {
            TemplateSignature ts = tsArray1[i];
            
            Element tempSig = new Element("TemplateSignature");
            
            String name = "Name";
            String label = "Name";
            String value = ts.getTemplateName();
            Element templateName = this.addedDeletedDiffNode(name,label,value,true);
            tempSig.addContent(templateName);
            
            
            /**** Doesn't make sense to go to parameter level
            for (int j = 0; j < ts.getDisplayParamList().length; j++) {
                Parameter param = ts.getDisplayParamList()[j];
                
                Element parameter = new Element("DisplayParameter");
                
                name = "ParameterName";
                label = "Name";
                value = param.getParameterName();
                Element paramName = this.addedDeletedDiffNode(name,label,value,true);
                parameter.addContent(paramName);
                
                name = "Value";
                label = "Value";
                value = param.getValue();
                Element paramValue = this.addedDeletedDiffNode(name,label,value,true);
                parameter.addContent(paramValue);
                
                
                temSigVec.add(parameter);
                
            }

            */
            temSigVec.add(tempSig);
        }
        
        //      add added nodes
        for (int i = minSize; i < tsArray2.length; i++ ) {
            TemplateSignature ts = tsArray2[i];
            
            Element tempSig = new Element("TemplateSignature");
            
            String name = "Name";
            String label = "Name";
            String value = ts.getTemplateName();
            Element templateName = this.addedDeletedDiffNode(name,label,value,false);
            tempSig.addContent(templateName);
            
            
            /***** Doesn't make sense to go to parameter level
            for (int j = 0; j < ts.getDisplayParamList().length; j++) {
                Parameter param = ts.getDisplayParamList()[j];
                
                Element parameter = new Element("DisplayParameter");
                
                name = "ParameterName";
                label = "Name";
                value = param.getParameterName();
                Element paramName = this.addedDeletedDiffNode(name,label,value,true);
                parameter.addContent(paramName);
                
                name = "Value";
                label = "Value";
                value = param.getValue();
                Element paramValue = this.addedDeletedDiffNode(name,label,value,true);
                parameter.addContent(paramValue);
                
                
                temSigVec.add(parameter);
                
            }
            
            **/
            
            
            temSigVec.add(tempSig);
        }
        
        
        
        return(temSigVec);
    }
    
    
    /**
     * Computes a difference node
     * 
     * @param name
     * @param label
     * @param value1
     * @param value2
     * @return an <code>Element</code> object containing a difference node
     */
    private Element equalsChangedDiffNode( String name, String label,  String value1 , String value2  ) {
        
        
        // returning node
        Element retNode = new Element(name);            
        Element labelA = new Element("Label");
        labelA.addContent(label);        
        
        Element value1A = new Element("Value1");
        value1A.addContent( value1 );        
        
        Element value2A = new Element("Value2");
        value2A.addContent( value2 );        
        
        Element status = new Element("Status");
        
        if ( value1.compareTo(value2) == 0 )
            status.addContent("Equal"); 
         else
            status.addContent("Changed");
        
        //      add contents of startTime
        retNode.addContent(labelA);
        retNode.addContent(value1A);
        retNode.addContent(value2A);
        retNode.addContent(status);
        
        return(retNode);

    }


    /**
     * returns a added/deleted difference node for parameter p 
     *  
     * @param p
     * @param deleted 
     * @return <code>Element</code> object
     * 
     * if <code>deleted</code> parameter is true, the method returns a node containing all value2="" and status="Deleted" 
     * otherwise it returns a node containing all value1="" and status="Added" 
     * 
     */
    private Element addedDeletedDiffNode( String name, String label,String value, boolean deleted ) {
        
        
        Element element = new Element(name);
        
        Element labelE = new Element("Label");
        labelE.addContent(label);        
        
    
        
        Element value1E = new Element("Value1");
        Element value2E = new Element("Value2");
        Element statusE = new Element("Status");
        
        if ( deleted ) {
        
            value1E.addContent( value );                
            value2E.addContent("");                        
            statusE.addContent("Deleted");
        }
        else {
            value1E.addContent("");                
            value2E.addContent(value);                        
            statusE.addContent("Added");            
        }
        
        //  add contents of element
        element.addContent(labelE);
        element.addContent(value1E);
        element.addContent(value2E);
        element.addContent(statusE);  
        
        return (element);
        
    }
    
    /**
     * Returns an XML string representing the differences between the two observation blocks 
     * 
     * @param ob1
     * @param ob2
     * @return xml String 
     * @throws ReflectionException
     * @throws OBDifferenceConfigurationException
     */
    public String compareObservationBlocksToXML( CalibrationBlock cb1, CalibrationBlock cb2  ) throws ReflectionException, OBDifferenceConfigurationException {
        	
        String xmlOutput;
        Document xmlDoc ;
        
        //   obtain property lists for each constraint set
        Instrument instr1 = InstrumentList.getInstance().getInstrument(cb1.getOd().getInstrument(),cb1.getOd().getIPVersion() );
        Instrument instr2 = InstrumentList.getInstance().getInstrument(cb2.getOd().getInstrument(),cb2.getOd().getIPVersion() );
        
        InstrumentConstraintConfig [] insConfList1 = instr1.getCsConfList();
        InstrumentConstraintConfig [] insConfList2 = instr2.getCsConfList();

						
		// get subset of properties to be considered
		String [] props1 = new String[insConfList1.length];
		String [] props2 = new String[insConfList2.length];
		
		for(int i=0;i<insConfList1.length;i++) {
            String propName = insConfList1[i].getName();            
            props1[i] = propName;            
        }
		
		for(int i=0;i<insConfList2.length;i++) {
            String propName = insConfList2[i].getName();            
            props2[i] = propName;            
        }
    
		// check instances
		if ( cb1 instanceof ObservationBlock ) {
		    ObservationBlock ob1 = (ObservationBlock)cb1;
		    ObservationBlock ob2 = (ObservationBlock)cb2;
		    xmlDoc = differenceObservationBlock(ob1,ob2,props1,props2);
		}
		else {
		    xmlDoc = differenceCalibrationBlock(cb1,cb2,props1,props2);
		}
        
        
        xmlOutput = docToString(xmlDoc);
        
        return (xmlOutput);
        
    
    }
    
    /**
     * Returns a formatted XML String built from a <code>Document</code> object 
     * 
     * @param doc
     * @return String
     */
    private String docToString ( Document doc ) {
        XMLOutputter outputter = new XMLOutputter();
        Format format = Format.getPrettyFormat();
        outputter.setFormat(format);
        return(outputter.outputString(doc));
    }
    
    
    /**
     * Searches for node <code>name</code> in the list, and
     * returns a JDom <code>Element</code> object
     * This method is called by <code>getProps()</code> method
     * 
     * @param name
     * @param elements
     * @return Element
     * @throws OBDifferenceConfigurationException
     */
    protected Element findConfig(String name) throws OBDifferenceConfigurationException {
        // gets configuration for comparison
        Element root = configDoc_.getRootElement();
        List elements = root.getChildren();

        for (Iterator iter = elements.iterator(); iter.hasNext();) {
            Element node = (Element) iter.next();
            
            if (node.getName().equals(name)) {
                return(node);
            }
        }  
                              
        throw new OBDifferenceConfigurationException("Not configuration found for TimeInterval object");
             
    }
    
    /**
     * Compares properties of two objects: <code>bo1</code> and <code>bo2</code>, which are instances of <code>cl</code> class
     * Invokes the getters of an object, given a list of properties
     * 
     * Depending on parameter <code>compType</code>, checks equality, or adds "added" or "deleted" nodes 
     *  
     * @param cl
     * @param node
     * @param props
     * @param bo1
     * @param bo2
     * @param compType
     * @throws ReflectionException
     */
    private void compareProperties( Class cl , Element node , List props , Object bo1 , Object bo2, String compType ) throws ReflectionException {
        try {
            // Iterates properties and compares them
            for (Iterator iter = props.iterator(); iter.hasNext();) {
                Element element = (Element) iter.next();
                
                String propertyName = element.getName();
                String label = element.getAttributeValue("label");
                String value1;
                String value2;
                
                // constructs the getter method for the property
                Method get;                
                get = cl.getMethod("get" + propertyName, new Class[] {});
                            
                Object obj1 , obj2;                
                obj1 = get.invoke(bo1, new Object[] {});                                    
                obj2 = get.invoke(bo2, new Object[] {});	
                value1 = obj1.toString();
                value2 = obj2.toString();
                
                // compare the two objects
                if (compType.equals("Equal")) {
                
                    Element diffNode = equalsChangedDiffNode(propertyName,label,value1,value2);
                    node.addContent(diffNode);
                } else {
                    if (compType.equals("Added")) {
                        Element diffNode = this.addedDeletedDiffNode(propertyName,label,value2,false);
                        node.addContent(diffNode);
                    } else {
                        if (compType.equals("Deleted")) {
                            Element diffNode = this.addedDeletedDiffNode(propertyName,label,value1,true);
                            node.addContent(diffNode);
                        }
                    }
                }                
            }	
            
        }	catch (SecurityException e) {
            throw new ReflectionException(e.toString());
        } catch (NoSuchMethodException e) {
            throw new ReflectionException(e.toString());         
        } catch (IllegalArgumentException e) {
            throw new ReflectionException(e.toString());
        } catch (IllegalAccessException e) {
            throw new ReflectionException(e.toString());
        } catch (InvocationTargetException e) {
            throw new ReflectionException(e.toString());
        }
        
        
    }
    
   
    
    
    
    
}