package org.eso.ohs.persistence;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Properties;
import java.util.TimeZone;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JOptionPane;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.utilities.SystemFonts;
import org.eso.ohs.core.gui.widgets.AuthenticationDialog;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSGlobalVariables;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SystemStatus;
import org.eso.ohs.core.utilities.SystemStatusListener;
import org.eso.ohs.dfs.AccountInfo;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;


/**
 * Information and behavior common to all OHS applications. <BR>
 * <strong>Note </strong>: although this class is not a singleton, only one
 * instance should be created at run time.
 *
 * An OHSApplication can be installed both as a unique jar file, or as a
 * directory tree in an installation directory, with lib, config, bin
 * subdirectories. <BR>
 * For accomodating the multiplatform and user needs, a common procedure for
 * retreiving system properties is put in place. <BR>
 * The system properties are read before the OHS configuration files (.cf). When
 * an OHSApplication is created, a file <b>ohs.properties </b> is searched and
 * loaded in several places and the properties are loaded.
 * <OL>
 * <li>Try to load config/ohs.properities from classloader</li>
 * <li>Try to load config/ohs.properities from current working directory <BR>
 * The properties already loaded in the previous step are overridden</li>
 * <li>Try to load .ohs/ohs.properities from user home directory <BR>
 * The properties already loaded in the previous steps are overridden</li>
 * </OL>
 * The directory .ohs will be created if not found in the user home directory.
 *
 * if an application is started from the command line, one can still define
 * system properties with "-Dprop=value" That value override any other value for
 * property "prop" found in the properties files. In other words, the algorithm
 * does not reset any system property for which a value is found.
 *
 * @version $Revision: 1.6.4.7 $
 *
 * @author A M Chavan, 26-Jun-2001: creation, from OhsApp.java
 */

public class OHSApplication {


    private final static String rcsid = "$Id: OHSApplication.java,v 1.6.4.7 2007/11/06 14:15:54 pnunes Exp $";

    /**
     * Standard log channel.
     */
    protected static Logger stdlog_ = Logger.getLogger(OHSApplication.class);

    protected static OHSApplication instance_ = null;

    /**
     * This application's full name: for instance
     * <code>Phase II Proposal Preparation System</code>
     */
    private static String appFullName_ = "OHS application";

    /**
     * The ohs.properties file name
     */
    private static String ohsPropertyFile_ = "ohs.properties";

    /**
     * The common config dir
     */
    private static String ohsCommonConfigDir_ = "config";

    /**
     * The home directory .ohs config dir
     */
    private static String ohsUserConfigDir_ = ".ohs";

    /** This application's short name: for instance <code>P2PP</code> */
    private static String appShortName_ = "OHSapp";

    /** This application's version string: for instance <code>2.1</code> */
    private static String appVersion_ = "0.0";

    protected final static String ERROR_MSG = "Operation aborted: no authentication"
            + "\n\nPlease check your username and password,"
            + "\nor contact the ESO User Support Group.";

    private static OHSPersistence persistence_;

    /** Initialize font offset size */
    private static int fontOffset_ = 0;

    private final int MAXOFFSET = 5;
    private int fontStyle_ = Font.PLAIN;
    private String fontFace_ = "Monospaced";
    
    
  

    /**
     * Initialize static information about the application and setup the logging
     * environment of log4j. <BR>
     *
     * The pathname of the configuration file is taken from the
     * <code>log4j.configuration</code> system property; see the startup
     * scripts and the log4j documentation for details. <BR>
     *
     * The default priority level is taken from the value of the
     * <code>log.level</code> system property.
     *
     * @param fullname
     *            Something like <code>OPC Panel Tool</code>
     * @param shortname
     *            Something like <code>PanelTool</code>
     * @param version
     *            Something like <code>68.7</code>
     */
    public void init(String fullname, String shortname, String version) {

        appFullName_ = fullname;
        appShortName_ = shortname;
        appVersion_ = version;
        TimeZone tz = TimeZone.getTimeZone("UT");
        TimeZone.setDefault(tz);
        stdlog_.info("TimeZone is: " + TimeZone.getDefault().getID());
        // Log4j setup: define the default priority of the standard
        // log channel based on the log.priority system property.
        // A M Chavan, 18-Dec-2001: added
        //------------------------------------------------------------
        String loglevelVar = System.getProperty("log.level");
        Level logLevel = null;
        if (loglevelVar == null || loglevelVar.length() == 0) {
            logLevel = Level.WARN;
            stdlog_.debug("log.level property not found. current log level = <"
                    + logLevel.toString() + ">");
        } else {
            logLevel = Level.toLevel(loglevelVar);
            stdlog_.debug("log.level property = <" + logLevel.toString() + ">");
        }
        Logger.getRootLogger().setLevel(logLevel);

        // set the local regional settings to US for java 5
        Locale.setDefault(Locale.US);
    }


    public static void addSystemStatusListener(SystemStatusListener lnr) {
        SystemStatus.addSystemStatusListener(lnr);
    }

    public static void removeSystemStatusListener(SystemStatusListener lnr) {
        SystemStatus.removeSystemStatusListener(lnr);
    }

    /**
     * Informs listeners of a change in the system status.
     * No details of which property of the old/new value are given.
     */
    public static void fireSystemStatusEvent(int evtType, Object src) {
        SystemStatus.fireSystemStatusEvent(evtType, src);
    }

    /**
     */
    protected OHSApplication() {
        // Allow static references to this object.
        // instance_ = this;

    }

    /** @return This application's (only) instance */
    public static OHSApplication getApplication() {
        if (instance_ == null) {
            instance_ = new OHSApplication();

        }
        instance_.loadEnv();
        return instance_;
    }

    /**
     *  void
     */
    public  void loadEnv() {
            loadSystemProperties(
                    getOhsCommonConfigDir(), getOhsUserConfigDir(),
                    getOhsPropertyFile());
    }

    /**
     * @param config void
     */
    protected void extractAppConfig(OHSConfig config,String configFile) {
        try {
            if (!config.loadConfigFile(configFile, OHSApplication
                    .getOhsUserConfigDirPath())) {
                stdlog_.error("Config file  not found: "+configFile);
                System.exit(0);
            }
        } catch (Exception e) {
            stdlog_.error("Error in config file ", e);
            System.exit(0);
        }
    }
    /**
     * The system properties are loaded in hierarchical way. First they are
     * searched in the classpath, then in the installation config directory, and
     * then in the private home directory.
     */
    protected void loadSystemProperties(String ohsCommonConfigDir,
            String ohsUserConfigDir, String ohsPropertyFileName) {
        File ohsPropertyFile = new File(ohsCommonConfigDir + File.separator
                + ohsPropertyFileName);
        InputStream propStream = null;
        Properties prop = new Properties();
        stdlog_.info(" loading system properties ");
        loadPropFromClassLoader(ohsPropertyFile, prop);
        ohsPropertyFile = getConfigFile(ohsPropertyFile);
        loadPropFromDir(ohsPropertyFile, prop);

        File ohsDir = new File(getOhsUserConfigDirPath());
        File ohsPrivateFile = getPrivateFile(
                ohsDir.getAbsolutePath(), ohsPropertyFileName);
        if (!ohsDir.exists()) {
            ohsDir.mkdir();
        }
        loadPropFromDir(ohsPrivateFile, prop);

        Enumeration propList = prop.propertyNames();
        while (propList.hasMoreElements()) {

            String key = (String) propList.nextElement();
            if (System.getProperty(key) == null) {
                System.setProperty(key, prop.getProperty(key).trim());
            }
        }
    }

    protected File getPrivateFile(String ohsUserConfigDir,
            String ohsPropertyFile) {
        File ohsPrivateFile = new File(ohsUserConfigDir + File.separator
                + ohsPropertyFile);
        return ohsPrivateFile;
    }

    private void loadPropFromDir(File ohsPropertyFile, Properties prop) {
        InputStream configStream = null;
        ////////////////
        // load properties from config dir
        try {
            configStream = new FileInputStream(ohsPropertyFile);
            prop.load(configStream);
            stdlog_.info("loading property file "
                    + ohsPropertyFile.getAbsolutePath());
        } catch (IOException e) {
            stdlog_.debug("property file " + ohsPropertyFile.getAbsolutePath()
                    + " in current working directory: \n"+ e.getMessage());
        }
    }

    protected File getConfigFile(File ohsPropertyFile) {
        return ohsPropertyFile;
    }

    private void loadPropFromClassLoader(File ohsPropertyFile, Properties prop) {
        InputStream propStream = null;
        ///////////////////////
        // load properties from classloader
        try {
            propStream = loadFileFromResource(ohsPropertyFile);
            prop.load(propStream);
            stdlog_.info("loading property file "
                    + ohsPropertyFile.getAbsolutePath() + " in classloader ");
        } catch (IOException e) {
            stdlog_.debug("property file " + ohsPropertyFile.getAbsolutePath()
                    + " in classloader not found: \n" + e.getMessage());
        }

    }

    protected InputStream loadFileFromResource(File ohsPropertyFile)
            throws IOException {
        InputStream propStream;
        propStream = FileIOUtils.getResourceStream(ohsPropertyFile
                .getAbsolutePath());
        return propStream;
    }

    /** @return This application's standard log channel */
    public static Logger getStdlog() {
        return stdlog_;
    }

    public void setInstance(OHSPersistence persistence) {
        persistence_ = persistence;
    }

    /** @see #appFullName_ */
    public static String getFullName() {
        return appFullName_;
    };

    /** @see #appShortName_ */
    public static String getShortName() {
        return appShortName_;
    };

    /** @see #appVersion_ */
    public static String getVersion() {
        return appVersion_;
    };

    public static void checkDbFileExists() {
        AppConfig appcfg = AppConfig.getAppConfig();
        if (!appcfg.hasDbCfg()) {
            JOptionPane.showMessageDialog(
                    null, appcfg.getDbCfg().getAbsolutePath() + " file does not exist" + "\n"
                            + OHSApplication.getShortName()
                            + " cannot continue", "File does not exist : "
                            + appcfg.getDbCfg().getAbsolutePath(), JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
    }

    /**
     * Added this so we can change the shortName
     *
     * @param shortName
     */

    public static void setShortName(String shortName) {
        appShortName_ = shortName;
    }

    /**
     * @return A combination of short name and version; e.g.
     *         <code>P2PP V.2.2</code>
     */
    public static String getShortDesc() {
        if (getVersion() != null && !appVersion_.equals("0.0")) {
            return getShortName() + " V." + getVersion();
        } else
            return getShortName();
    }

    public void setDefaultGUIResources() {

        UIManager.put("List.background", Color.white);
        UIManager.put("List.selectionBackground", Color.blue.darker());
        UIManager.put("List.focusCellHighlightBorder", BorderFactory
                .createLineBorder(Color.yellow));

        UIManager.put("ComboBox.background", Color.white);
        UIManager.put("TextField.background", Color.white);
        UIManager.put("TextArea.background", Color.white);
        UIManager.put("PasswordField.background", Color.white);
        UIManager.put("Table.background", Color.white);
        UIManager.put("Table.textBackground", Color.white);
        UIManager.put("Table.selectionBackground", Color.blue.darker());
        UIManager.put("Table.selectionForeground", Color.white);
        UIManager.put("Table.focusCellHighlightBorder", BorderFactory
                .createLineBorder(Color.yellow));
        UIManager.put("Tree.background", Color.white);
        UIManager.put("Tree.textBackground", Color.white);
        UIManager.put("Tree.selectionBackground", Color.blue.darker());
        UIManager.put("Tree.selectionForeground", Color.white);
        UIManager.put("Tree.selectionBorderColor", Color.yellow);

        ToolTipManager.sharedInstance().setDismissDelay(5000);
        ToolTipManager.sharedInstance().setInitialDelay(0);

        //Add the font offset read in from site.cf used by P2PP and OT
        Font textEntryfont = new Font(getFontFace(),getFontStyle(),
                (OHSGlobalVariables.DEF_FONT_SIZE + fontOffset_));

        // ***** Set new Fonts for text components.
        UIManager.put("TextField.font", textEntryfont);
        UIManager.put("TextArea.font", textEntryfont);
        UIManager.put("List.font", textEntryfont);
        UIManager.put("ComboBox.font", textEntryfont);
        UIManager.put("PasswordField.font", textEntryfont);
        UIManager.put("Table.font", textEntryfont);
        UIManager.put("Tree.font", textEntryfont);
    }

    /**
     * @return Object
     */
    private String getFontFace() {
        return fontFace_;
    }

    /**
     * @return Object
     */
    private int getFontStyle() {
        return fontStyle_;
    }

    /**
     * Method to read a font offset from the site.cf file This offset allows the
     * user to increase or decrease the font size. If the user does not input a
     * valid value within the offset guidelines the offset is ignored.
     *
     *
     * @see org.eso.ohs.core.gui.utilities.SystemFonts
     *
     */
    public void offsetFontSize(OHSConfig appcfg) {
        fontOffset_ = appcfg.getInteger("FONT.SIZE.OFFSET", 0);
        if (fontOffset_ > 0 && fontOffset_ < MAXOFFSET) {
            //				  Increase fonts
            SystemFonts.increaseUIManagerFontSize(fontOffset_);
        } else if (fontOffset_ < 0 && fontOffset_ > -(MAXOFFSET)) {
            //				  Decrease fonts
            SystemFonts.decreaseUIManagerFontSize((fontOffset_ * -1));
        }
    }

    public void setFontStyle(OHSConfig appcfg) {
        String fontStyleString = appcfg.getValue("FONT.STYLE");
		if (fontStyleString.equals("BOLD")) {
		    fontStyle_ = Font.BOLD;
		} else if (fontStyleString.equals("ITALIC")) {
		    fontStyle_ = Font.ITALIC;
		} else { // Plain
		    fontStyle_ = Font.PLAIN;
		}
    }

    public void setFontFace(OHSConfig appcfg) {
       fontFace_ = appcfg.getValue("FONT.FACE");
    }


    /**
     * @return int the current font size
     */
    public static int getFontSize() {
        return OHSGlobalVariables.DEF_FONT_SIZE+ fontOffset_;
    }

    public void setLookAndFeel() {

        try {
            String laf = UIManager.getSystemLookAndFeelClassName();

            if (System.getProperty("os.name").equals("HP-UX")) {
                laf = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
            }
            if (System.getProperty("os.name").equals("Linux")) {
                laf = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
            }

            UIManager.setLookAndFeel(laf);
        } catch (UnsupportedLookAndFeelException e) {
            stdlog_.error("Exception: " + e);
        } catch (ClassNotFoundException e) {
            stdlog_.error("Exception: " + e);
        } catch (InstantiationException e) {
            stdlog_.error("Exception: " + e);
        } catch (IllegalAccessException e) {
            stdlog_.error("Exception: " + e);
        }

    }

    public static void initialiseTargetObjectClass() throws IOException {

        File objClassFile = new File(AppConfig.getAppConfig().getConfigDir(),
                "objClass.dat");
        Vector<String> classifications = new Vector<String>();
        Vector<String> tooltips = new Vector<String>();
        initTargetClassifications(objClassFile, classifications, tooltips);

        String[] objClassValues = new String[classifications.size()];
        classifications.copyInto(objClassValues);

        Target.setObjectClassValues(objClassValues);

        String[] tooltipValues = new String[tooltips.size()];
        tooltips.copyInto(tooltipValues);

        Target.setObjectClassTooltips(tooltipValues);

    }

	public static void initTargetClassifications(File targetClassFile, Vector<String> classifications, Vector<String> tooltips) throws IOException {
		BufferedReader rdr = null;
        String objClass = null, ttip = null;
        final int start_objClass = 0;
        final int end_objClass = 20;
        String line;

        try {
            rdr = new BufferedReader(new FileReader(targetClassFile));

            for (;;) {
                line = rdr.readLine();

                if (line == null) {
                    break;
                }

                int index = line.indexOf(':');

                if (index < 0) {
                    continue;
                }

                line = line.substring(index + 1);

                if (line.trim().length() == 0) {
                    continue;
                }

                if (line.length() > end_objClass) {
                    int firstQuote = line.indexOf('"');
                    int lastQuote = line.lastIndexOf('"');

                    if (firstQuote < 0 || lastQuote < 0) {
                        continue;
                    }

                    objClass = line.substring(start_objClass, end_objClass);
                    ttip = line.substring(firstQuote + 1, lastQuote);
                } else {
                    continue;
                }

                classifications.addElement(objClass);
                tooltips.addElement(ttip);
            }
        } catch (IOException e) {
            stdlog_.error("", e);
            ;
        }

        if (rdr != null) {
            rdr.close();
        }
	}

    /**
     * Convenience method to ask the user for a password.
     *
     * @exception ObjectIOException
     *                if there was an I/O error or the user did not authenticate
     *                themselves
     */
    public static void askForAuthentication(Media device)
            throws ObjectIOException {
        //OHSPersistence objMgr = OHSPersistence.getInstance();
        int accountId = Config.getCfg().getAccountId();
        String userName = Config.getCfg().getUsername();
        String passwd = Config.getCfg().getPassword();

        persistence_.setAccountId(accountId);

        // a password was typed
        if (passwd != null && !passwd.equals("")) {        	        	
        	
            if (persistence_.authenticateUser(device, passwd)) {
            	AccountInfoManager userMap = new AccountInfoManager(persistence_.getStorageManager(device), AppConfig.getAppConfig().getCacheDir());
            	AccountInfo centralAccountInfo = userMap.getAccountInfoFromCentralDB(accountId, passwd);
            	
            	String centralUsername = centralAccountInfo.getUsername();
            	
            	if (userName.equals(centralUsername))            	
                	return;
                else {
                	userMap.deleteLocalAccountInfo(userName);
                	userMap.writeLocalAccountInfo(centralAccountInfo);
                	JOptionPane
							.showMessageDialog(
									null,
									"Your username has changed since the last time you connected to ESO\n. Please use the username \"" + centralUsername
									+ "\" the next time you login.",
									"Changed username",											
									JOptionPane.WARNING_MESSAGE);
                	Config.getCfg().setAccount(centralAccountInfo);
                	
                }
            }
        }

        // no password was typed - ask for it
        AuthenticationDialog authdlg = new AuthenticationDialog(null,
                "Enter your password", "Please enter your password:", userName);

        authdlg.setVisible(true);

        int choice = authdlg.getUserChoice();

        if (choice == JOptionPane.CLOSED_OPTION
                || choice == JOptionPane.CANCEL_OPTION) {
            throw new ObjectIOException("Operation aborted: no authentication");

        }

        passwd = authdlg.getPassword();

        if (passwd == null || passwd.equals("")) {
            // user didn't enter a password
            throw new ObjectIOException(ERROR_MSG);
        }

        Config.getCfg().setPassword(passwd);

        if (!persistence_.authenticateUser(device, passwd)) {
            throw new ObjectIOException(ERROR_MSG);
        }

    }

    public static boolean ensureAuthenticated() {
        try {
            OHSApplication.askForAuthentication(Media.DBASE);
            return true;
        } catch (ObjectAuthorisationException e) {
            JOptionPane.showMessageDialog(
                null,
                new Object[] {
                    "Your objects in the ESO database could not be displayed",
                    "because you have not successfully authenticated",
                    },
                "Authentication Error",
                JOptionPane.ERROR_MESSAGE);
            return false; // Authentication failed
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(null,e);
            return false; // An I/O error occurred
        }
    }




    public void checkInstrumentDirectory(AppConfig appcfg) {
        File instDir = new File(appcfg.getInstrumDir());        
        checkDirectory(instDir, false, true);
    }

    public void checkDirectory(File dir, boolean writeAccess, boolean create) {
        if (!dir.exists()) {
            if (create) {
                if (!dir.mkdirs()) {
                    JOptionPane.showMessageDialog(
                            null, new Object[]
                            { "Directory could not be created",
                                    "Directory: " + dir.getAbsolutePath(), " ",
                                    "Cannot continue!" },
                            "Required directory could not be created",
                            JOptionPane.ERROR_MESSAGE);
                    System.exit(0);
                } else {
                	stdlog_.info("Required directory was created: "+dir.getAbsolutePath() );
//                    JOptionPane
//                            .showMessageDialog(
//                                    null,
//                                    new Object[]
//                                    {
//                                            "Required directory was created: ",
//                                            "Directory: "
//                                                    + dir.getAbsolutePath(), "" },
//                                    "Directory Created",
//                                    JOptionPane.WARNING_MESSAGE);
                }
            } else {
                JOptionPane.showMessageDialog(
                        null, new Object[]
                        { "Directory does not exist",
                                "Directory: " + dir.getAbsolutePath(), " ",
                                "Cannot continue!" },
                        "Required directory does not exist",
                        JOptionPane.ERROR_MESSAGE);
                System.exit(0);
            }
        }

        if (!dir.canRead()) {
            JOptionPane.showMessageDialog(
                    null, new Object[]
                    { "No read permissions for directory",
                            "Directory: " + dir.getAbsolutePath(), " ",
                            "Cannot continue!" }, "No read permissions",
                    JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
        if (writeAccess) {
            if (!dir.canWrite()) {
                JOptionPane.showMessageDialog(
                        null, new Object[]
                        { "No write permissions for directory",
                                "Directory: " + dir.getAbsolutePath(), " ",
                                "Cannot continue!" }, "No write permissions",
                        JOptionPane.ERROR_MESSAGE);
                System.exit(0);
            }
        }
    }

    public void displayInstrumentErrorInfo() {
        Instrument[] instrument = InstrumentList
                .getInstance()
                .getInstrumentList();
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < instrument.length; i++) {
            if (instrument[i].hasInvalidTemplates()) {
                buf.append("    " + instrument[i].getInstrumentName() + ", v"
                        + instrument[i].getVersion() + ", location: "
                        + instrument[i].getTSFDirectory() + "\n");
                if (i < instrument.length - 1)
                    buf.append(" ");
            }
        }
        if (buf.length() > 0) {
            JOptionPane
                    .showMessageDialog(
                            null,
                            new Object[]
                            {
                                    "The following instrument package(s) \n"
                                            + buf.toString()
                                            + "have invalid template signatures.\n Please use engineering mode to get detailed error report",
                                    " ", "" },
                            "Invalid templates in instrument package(s)",
                            JOptionPane.WARNING_MESSAGE);
            //System.exit(0);
        }
    }

    protected static void setOhsCommonConfigDir(String ohsCommonConfigDir) {
        OHSApplication.ohsCommonConfigDir_ = ohsCommonConfigDir;
    }

    public static String getOhsCommonConfigDir() {
        return ohsCommonConfigDir_;
    }

    protected static void setOhsUserConfigDir(String ohsUserConfigDir) {
        OHSApplication.ohsUserConfigDir_ = ohsUserConfigDir;
    }

    public static String getOhsUserConfigDir() {
        return ohsUserConfigDir_;
    }

    public static String getOhsUserConfigDirPath() {
        String fullPath = System.getProperty("user.home") + File.separator
                + getOhsUserConfigDir();
        return fullPath;
    }

    protected static void setOhsPropertyFile(String ohsPropertyFile) {
        OHSApplication.ohsPropertyFile_ = ohsPropertyFile;
    }

    public static String getOhsPropertyFile() {
        return ohsPropertyFile_;
    }
}