package org.eso.ohs.persistence;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Summary;


/**
 * A StorageManager class that requires it's caller to be authenticated.
 * This class is intended to form a matching pair with
 * AuthenticatingStorageMgr.  ProtectingStorageMgr
 * requires it's caller to be authenticated and throws an
 * ObjectAuthorisationException if it is not, whereas
 * AuthenticatingStorageMgr catches the exception, authenticates
 * the user and retries the previously aborted call.
 * 
 * <p>
 * The two classes are typically used either as
 * 
 * <pre>
 *    StorageManager stMgr = new AuthenticatingStorageMgr(
 * 		new ProtectingStorageMgr(
 * 			someStorageMgr
 * 		)
 *    );
 * </pre>
 * 
 * or with one at each end of an RMI connection between a client
 * program and the application server.
 * 
 * <p>
 * $Id: ProtectingStorageMgr.java,v 1.20.16.1 2007/10/05 11:03:38 pnunes Exp $
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP AppServer.
 * 
 * @see AuthenticatingStorageMgr
 * @see StorageManager
 */
public class ProtectingStorageMgr extends DecoratorStorageManager {

    private boolean authenticated_ = false;


    public ProtectingStorageMgr(StorageManager stMgr) {
        super(stMgr);
    }

    public void setAccountId(int userId)
			    throws ObjectIOException {

	   authenticated_ = false;
	   super.setAccountId(userId);
    }

    /**
     * Calculates a unique id for a StorableObject.
     *
     * @return Unique id
     * 
     * @exception ObjectIOException
     *			if there was an I/O error writing the object.
     */
    public long getNewId(Class mo, long baseId)
			    throws ObjectIOException {
    	ensureAuthenticated();
    	return super.getNewId(mo, baseId);
    }


    public boolean loginUser(String password)
		  throws ObjectIOException {
	    authenticated_ = super.loginUser(password);

	    return authenticated_;
    }

    /**
     * Returns a DirectoryNode representing the path within the
     * storage medium of an object with the given id and type.
     * For media that do not support directories, the root directory
     * may be returned.
     * 
     * @param id	the id of the StorableObject.
     * @param cls	the type of stored StorableObject.
     * 
     * @return Directory in which to find the object.
     * 
     * @exception ObjectIOException
     *			if there was an I/O error writing the object.
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found.
     */
    public DirectoryNode find(long id, Class objType)
			    throws ObjectNotFoundException,
				    ObjectIOException {
	    ensureAuthenticated();
        return super.find(id, objType);
    }


    /**
     * Returns an array of objects representing the contents of an
     * object with the given id and type from the chosen directory.
     * If the medium does not support directories, the <code>node</code>
     * parameter is ignored.
     * 
     * <p>
     * The array of objects contains:
     * <ul>
     *     <li> as the first (0th) element, an instance of a
     *          StorableObject subclass; followed by
     *     <li> an alternating sequence of values:
     *          <ul>
     *              <li> at even indexes, a property name, ie a string X
     *                   such that setX() and getX() methods exist.
     *              <li> at odd indexes, either an instance of
     *                   java.lang.Long containing the the id of the
     *                   StorableObject subclass to which the property
     *                   should refer or a reference to the subclass itself.
     *          </ul>
     * </ul>
     * 
     * @param node	directory from which the object should be loaded.
     * @param id	id of the StorableObject.
     * @param objType	type of StorableObject.
     * 
     * @return the array of objects and object references,
     * 			as described above.
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found.
     * @exception ObjectIOException
     * 			if an I/O error occurred while reading the object
     * 
     * @see #write
     * @see #update
     */
    public UnpackedStorableObject read(DirectoryNode node, long id,
						Class objType)
				throws ObjectNotFoundException,
					ObjectIOException {
     	ensureAuthenticated();
     	return super.read(node, id, objType);
    }

    /**
     * Writes an array of objects with the given id and type to
     * this storage medium, in the chosen directory.  If the medium
     * does not support directories, the <code>node</code> parameter
     * is ignored.
     * 
     * @param node	the directory in which the object should be saved.
     * @param id	the id under which to store the StorableObject.
     * @param objType	the type of StorableObject.
     * @param obj	the array of objects and object references to store,
     *			as described in the section for the read method.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while writing the object
     * 
     * @see #read
     * @see #update
     */
    public void write(DirectoryNode node, long id, Class objType,
				    UnpackedStorableObject obj)
				throws ObjectIOException {
    	ensureAuthenticated();
    	super.write(node, id, objType, obj);
    }

    /**
     * Write several object to the external media in an atomic operation.
     *      
     * @param mos	Array of object descriptions.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while writing the object
     * 
     * @see #read
     * @see #write
     * @see #update
     */
    public void multiStore(MultiObjSpec[] mos)
				throws ObjectIOException {
    	ensureAuthenticated();
    	super.multiStore(mos);
    }

    /**
     * Updates an array of objects with the given id and type on
     * this storage medium, in the chosen directory.  If the medium
     * does not support directories, the <code>node</code> parameter
     * is ignored.
     * 
     * @param node	the directory in which the object should be saved.
     * @param id	the id under which to store the StorableObject.
     * @param objType	the type of StorableObject.
     * @param obj	the array of objects and object references to store,
     *			as described in the section for the read method.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while updating the object
     * 
     * @see #read
     * @see #write
     */
    public void update(DirectoryNode node, long id,
				Class objType, UnpackedStorableObject obj)
				throws ObjectIOException {
    	ensureAuthenticated();
    	super.update(node, id, objType, obj);
    }


    /**
     * Deletes an object with the given directory, id and type from
     * this storage medium.  If the medium does not support
     * directories, the <code>node</code> parameter is ignored.
     * 
     * @param node	directory from which the object should be deleted.
     * @param id	id of the StorableObject.
     * @param objType	type of StorableObject.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while updating the object
     * 
     * @see #read
     * @see #update
     */
    public void delete(DirectoryNode node, long id, Class objType)
				throws ObjectIOException {
    	ensureAuthenticated();
    	super.delete(node, id, objType);
    }


    /**
     * Moves an object between directories on this storage medium.
     * 
     * @param id	id of the StorableObject.
     * @param cls	type of StorableObject.
     * @param fromNode	directory from which the object should be moved.
     * @param toNode	directory to which the object should be moved.
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while updating the object
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found in fromNode.
     */
    public void move(long id, Class objType,
				DirectoryNode fromNode,
				DirectoryNode toNode)
				throws ObjectNotFoundException,
					ObjectIOException {
    	ensureAuthenticated();
    	super.move(id, objType, fromNode, toNode);
    }


    /**
     * Create a directory in local storage
     *
     * @param node The directory to be created
     * @param name The name of the directory to be created.
     * 
     * @return True on success false on error
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     */ 
    public boolean createDirectory(DirectoryNode node, long id)
				throws ObjectIOException {
    	ensureAuthenticated();
    	return super.createDirectory(node, id);
    }


    /**
     * Move a directory.
     *
     * @param node	directory to be moved
     * @param toNode	the new parent directory
     * 
     * @return True on success false on error
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     * 
     * @exception ObjectNotFoundException
     *			if the object was not found in fromNode.
     */ 
    public boolean moveDirectory(DirectoryNode fromNode,
					    DirectoryNode toNode)
				throws ObjectIOException,
					ObjectNotFoundException {
    	ensureAuthenticated();
    	return super.moveDirectory(fromNode, toNode);
    }


    /**
     * Delete a directory from local storeage
     *      
     * @param node	Directory to be deleted
     * 
     * @return True on success, false on failure
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred during the operation
     */
    public boolean deleteDirectory(DirectoryNode node)
				throws ObjectIOException {
    	ensureAuthenticated();
    	return super.deleteDirectory(node);
    }

    /**
     * @return the requested IP as a byte array, or an empty (size 0) array
     * if the IP does not exist on the server.
     * @exception ObjectIOException when there is an I/O exception.
     */
    public byte[] downloadInstrumentPackage(String instrument, float version)
                throws ObjectIOException {
        ensureAuthenticated();
        return super.downloadInstrumentPackage(instrument, version);
    }
    
    /**
     * Returns a list of Summary objects associated with a directory.
     * 
     * @param node Directory to be used to build up list of objects
     * @return An array of Summary objects 
     * 
     * @exception ObjectIOException
     * 			if an I/O error occurred while updating the object
     * 
     */
    public Summary[] listObjects(DirectoryNode node)
			throws ObjectIOException {
    	ensureAuthenticated();
    	return super.listObjects(node);
    }


    /**
     * Check in the specified objects in a single atomic operation.
     * 
     * @param	usos	Descriptions of each StorableObject to be checked in.
     * 
     * @return		array of error strings.
     *			If length = 0, no errors.
     */
    public void checkIn(MultiObjSpec[] moi)
				throws ObjectIOException {
    	ensureAuthenticated();
    	super.checkIn(moi);
    }

    /**
     * Do a EVM verification test for the specified objects in a single atomic operation.
     *
     * @param   usos    Descriptions of each StorableObject to do EVM verification
     *
     * @return boolean if OB passed EVM test
     */
    public boolean verifyObject(MultiObjSpec[] moi)
				throws ObjectIOException {
    	ensureAuthenticated();
    	return super.verifyObject(moi);
    }

    /**
     * Check out the ObsBlk or CalBlk with the specified id plus
     * any attached objects, ie Target, ObsDescr, etc.
     * 
     * @param	id	id of the ObsBlk or CalBlk to be checked out
     * 
     * @return		CheckOutStatus object that contains a return
     *			status and (if successful) references to the
     *			checked-out objects.
     */
    public void checkOut(IdType[] idTypes)
				throws ObjectNotFoundException,
					ObjectIOException {
    	ensureAuthenticated();
    	super.checkOut(idTypes);
    }

    protected void ensureAuthenticated()
			    throws ObjectAuthorisationException {
    	if ( ! authenticated_ ) {
    	    throw new ObjectAuthorisationException(
    		getMedia(),
    		"Authentication failed"
    	    );
    	}
    }
}