package org.eso.ohs.phase2.apps.ipacman;

import java.io.File;
import java.io.IOException;
import java.util.Properties;
import java.util.TimeZone;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.LoginDialog;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.gui.widgets.SplashScreen;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ExtFileList;
import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SystemStatus;
import org.eso.ohs.core.utilities.SystemStatusListener;
import org.eso.ohs.dfs.AccountInfo;
import org.eso.ohs.instruments.ZipUtils;
import org.eso.ohs.instruments.ZipUtilsException;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.StorageManager;
import org.eso.ohs.persistence.dbase.phase1.OpcDbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.OTDbaseMgr;

/**
 * The initially executed class for the IPacMan application
 * <p>
 * @version $Revision: 1.28 $
 * @author Ron J Lobo
 */
public class IPACMAN extends OHSApplication {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(IPACMAN.class);

    /** Ensure OutOfMemoryReport class is loaded and static initialisers run */
    private static OutOfMemoryReport outOfMemDialog_;

    /** CVS version id */
    public static final String cvsID_ =
        "$Id: IPACMAN.java,v 1.28 2007/10/11 11:41:16 pnunes Exp $";

    /** the main application window */
    public static JFrame ipacmanWin_;

    /** Used to record if quit dialog is currently being displayed */
    private static boolean showingQuitDialog_ = false;

    /** 'special' periods */
    public static final int EngObsRunsPeriod = 60;
    public static int firstMipsPeriod;


    /**
     * Constructor
     */
    protected IPACMAN(String[] args) {

        // Initialize this application
        OHSApplication ohsApp = getApplication();
        String versionStr = IPacManVersion.getVersion();
        ohsApp.init("Instrument Package Manager", "IPacMan", versionStr);

        SplashScreen splash = new SplashScreen(
            OHSApplication.getShortDesc() + " loading...", "pacmanSplash.gif");
        splash.setVisible(true);

        TimeZone.setDefault(TimeZone.getTimeZone("UTC"));
        setLookAndFeel();
        setupConnectionPool();

        try {
            readCfgs();
            Config.setCfg(new MyPersistCfg());
            setDefaultGUIResources();

            AppConfig appCnfg = AppConfig.getAppConfig();
            firstMipsPeriod = appCnfg.getIPVersionStartPeriod();

            // IPACMAN.TEMPDIR.FOLDER in site.cf must be a valid directory
            ZipUtils.setTmpDir(appCnfg.getValue(OHSKeys.IPacManTmpFolderKey));

            outOfMemDialog_ = new OutOfMemoryReport();
            splash.dispose();
            
            
            initialiseStorage();
            Persistence objMgr = Persistence.getInstance();
            StorageManager strMgr = objMgr.getStorageManager(Media.PHASE1_DB);
            
            int loginStatus = login(null,strMgr  );

            // end login with control
            if (loginStatus < 0) {
                System.exit(0);
            }
            
            
            
            ipacmanWin_ = DbWin.getDbWin();

        } catch (IOException e) {
            errMsg("Startup Error", "An Error has occurred - exiting");
        } catch (ZipUtilsException e) {
            errMsg("Startup Error", "Error setting the temp folder\n" + e.getMessage()); 
            
        } catch (ObjectIOException e) {
        	 errMsg("Startup Error", "An Error has occurred - exiting");
    		
        } catch (IPacManException e) {
            ErrorMessages.announceIOError(null, e);
            errMsg("Fatal Error", "Fatal error during initialisation");
        } 
    }


    /**
     * 
     * 
     * @return -1 if user aborted the login <br>
     *         0 if the user has logged-in but we should download observing
     *         runs, etc. <br>
     *         +1 if the user has logged-in and nothing more needs to be done.
     * @exception ObjectIOException
     *                Object I/O Exception
     */
    private static int login(JFrame frm, StorageManager dbaseMgr)
			throws ObjectIOException {
		String t = OHSApplication.getShortDesc() + " login";
		LoginDialog loginDialog = new LoginDialog(frm, t);
		AppConfig appcfg = AppConfig.getAppConfig();
		Config cfg = Config.getCfg();

		try {
			for (;;) {
				loginDialog.setVisible(true);
				if (loginDialog.getUserChoice() == JOptionPane.OK_OPTION) {					

					String userName = loginDialog.getStringUserName();
					String password = loginDialog.getPassword();

					AccountInfo info = dbaseMgr.getAccountInfo(userName, password);

					if (info != null ) {
						
						if (info.getOperations()) {
						
							cfg.setAccount(info);
							dbaseMgr.setAccountId(info.getAccountId());

							if (dbaseMgr.loginUser(password)) {
								loginDialog.dispose();							
								return 0;
							}
						} else {
							stdlog_.debug("Account doesn't have the operations role.");
							JOptionPane.showMessageDialog(frm,
									"Account doesn't have the operations role.", "Login Error",
									JOptionPane.WARNING_MESSAGE);
							continue;
						}
						
					} else {
						stdlog_.debug("User or password invalid.");
						JOptionPane.showMessageDialog(frm,
								"Username or password invalid", "Login Error",
								JOptionPane.WARNING_MESSAGE);
						continue;
					}

				} else {
					return(-1);
				}
			}

		} catch (ObjectIOException e) {
			stdlog_.error("", e);
			; // Leave as user support info.
			JOptionPane.showMessageDialog(null, new Object[] {
					"Your username and password could not be verified"
							+ " This is usually because",
					"the server at ESO cannot be contacted, for some reason.",
					" ", "Check your connection to ESO" + " and try again.",
					"If error persists, contact ESO User Support.", " ",
					"Fatal error during initialisation" }, "Fatal Error",
					JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}

		return -1;
	}

    
    
    /**
     * Read the required config files into AppConfig
     * This method is public and static so it may be called from Unit Tests
     */
    private static void readCfgs() throws IOException {

        AppConfig.setAppConfig(new AppConfig());
        AppConfig ac = AppConfig.getAppConfig();
        ac.setStandardOHSKeys("config");
        ExtFileList extList = new ExtFileList(ac.getConfigDir(), ".cf");
        if (extList.exists("ipacman-db.cf")) {
            File dbFile = extList.getFileHandle("ipacman-db.cf");
            ac.setupDbDirectConfig("", dbFile);
        } else {
            errMsg("Initialization Error", "No ipacman-db.cf file was found");
        }

        // String currDir = System.getProperty("user.dir");
        String cfFileStr = "site.cf";
        File cfFile = extList.getFileHandle(cfFileStr);
        if (cfFile != null && cfFile.exists() && cfFile.canRead()) {
            try {
                ac.addParamfile(cfFile.getAbsolutePath());
            } catch (NotASCIIException e) {
                // throw new IOException(e.getMessage());
                int rtn = JOptionPane.showConfirmDialog(null,
                    "Syntax Error while reading: " + cfFile +
                    "\non the following lines:\n\n" + e.getMessage() +
                    "\nIgnoring these syntax errors may cause undesirable results.\n\n" +
                    "Please quit and correct the syntax errors before continuing.\n\n" +
                    "Do you want to quit?",
                    "Syntax Error",
                    JOptionPane.YES_NO_OPTION);
                if (rtn == JOptionPane.YES_OPTION) {
                    System.exit(0);
                }
            }
        } else {
            errMsg("Initialization Error", "No " + cfFileStr + " file was found");
        }
    }


    /**
     * The main() method for this program -- execution starts here
     *
     * @param args array of arguments passed to this program by the OS
     */
    public static void main(String[] args) {
        new IPACMAN(args);
    }


    /**
     * Quit the application gracefully without asking the user
     * Used by askAndQuit() and for OutOfMemoryError shutdowns
     */
    public static void quitWithoutAsking() {
        // Deprecated in 1.2 but without it, exiting is V.slow on HP
        // Runtime.getRuntime().runFinalizersOnExit(false);
        System.exit(0);
    }


    /**
     * Asks the user if they really want to quit the application
     * if so, shut down and exit
     */
    public static void askAndQuit() {
        // if there is already a dialog displayed, do not show another one....
        if (showingQuitDialog_) {
            return;
        }
        // quit dialog not being displayed - safe to display one
        showingQuitDialog_ = true;
        int result = JOptionPane.showConfirmDialog(ipacmanWin_,
            "Would you like to quit now?", "Quit Now?",
            JOptionPane.YES_NO_OPTION);
        if (result == JOptionPane.YES_OPTION) {
            quitWithoutAsking();
        }
        showingQuitDialog_ = false;
    }


    /**
     * Initialise the Objectmanager and StorageManagers
     * These classes are used to read/write to local storage and/or dbase
     */
    protected static void initialiseStorage() throws ObjectIOException {
        Persistence.startup();
        Persistence objMgr = Persistence.getInstance();
        objMgr.setStorageManager(Media.PHASE1_DB, new OpcDbaseMgr());
        objMgr.setStorageManager(Media.PHASE2_DB, new OTDbaseMgr());
    }


    /**
     * Setup the connection pool properties prior to its first use
     * Note: IPACMAN requires a connection pool that allows connection re-use
     */
    private void setupConnectionPool() {
        Properties p = new Properties();
        p.put(ConnectionPool.PoolSize, "1");
        p.put(ConnectionPool.Timeout, "60");
        p.put(ConnectionPool.ReopenAllowed, "T");

        if (!ConnectionPool.setInitProperties(p)) {
            System.err.println("Warning: init properties ignored, " +
                "connection pool already in use");
        }
    }


    protected static boolean hasSubdirs(File dir) {

        if (dir == null || !dir.exists() || !dir.isDirectory()) {
            return false;
        }
        String[] listing = dir.list();
        File file;
        if (listing == null) {
            return false;
        }
        for (int j = 0; j < listing.length; j++) {
            file = new File(dir, listing[j]);
            if (file.isDirectory()) {
                return true;
            }
        }
        return false;
    }


    public static void addSystemStatusListener(SystemStatusListener lnr) {
        SystemStatus.addSystemStatusListener(lnr);
    }


    public static void removeSystemStatusListener(SystemStatusListener lnr) {
        SystemStatus.removeSystemStatusListener(lnr);
    }


    /**
     * Informs listeners of a change in the system status
     * No details of which property of the old/new value are given
     */
    public static void fireSystemStatusEvent(int evtType, Object src) {
        SystemStatus.fireSystemStatusEvent(evtType, src);
    }


    /**
     * convenience method to show an error message dialog and then exit
     */
    private static void errMsg(String title, String message) {
        stdlog_.error("Error: " + message);
        JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
        System.exit(1);
    }
}

