package org.eso.ohs.scripting;

/**
 * The ScriptInterpreterManager is a singleton accessed through the 
 * static getInstance() call. 
 * The object is populated with a set of ScriptInterpreter objects, 
 * which are selected based on language names or file extensions.
 *
 * The class follows the factory model in-terms of the creation/addition of
 * ScriptInterpreters. The addScriptInterpreter() method accepts a class name
 * of a ScriptInterpreter implementation and attempts to initialize the class
 * and create and instance of it.
 *
 * @author Nick Kornweibel
 */

import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

public class ScriptInterpreterManager {

    /** Vector of loaded ScriptInterpreter objects.
     */
    private Vector<ScriptInterpreter> interpreters;

    /** singleton instance of ScriptInterpreterManager.
     */
    private static ScriptInterpreterManager instance;

    /** 
     * Private default constructor to prevent external obj creation.
     */
    private ScriptInterpreterManager() {
        interpreters = new Vector<ScriptInterpreter>();
    }

    /**
     * Get singleton instance of the manager
     * @return script interpreter manager
     */
    public static ScriptInterpreterManager getInstance() {
        if (instance == null) {
            instance = new ScriptInterpreterManager();
        }
        return instance;
    }

    /**
     * Add a ScriptInterpreter to the manager
     * @param ScriptInterpreter script interpreter to be registered
     */

    public void registerScriptInterpreter(ScriptInterpreter si) {
        interpreters.addElement(si);
    }

    public boolean isRegistered(String driverName) {
        boolean rc = false;
        for (Enumeration<ScriptInterpreter> e = interpreters.elements(); e.hasMoreElements();) {
           ScriptInterpreter scrint = e.nextElement();
           if (driverName.equals(scrint.getClass().getName())) {
               rc = true;
               break;
           }
        }
        return rc;
    }

    /**
     * Remove a script interpreter from the manager
     * @param ScriptInterpreter script interpreter to be removed
     */

    public void deregisterScriptInterpreter(ScriptInterpreter si) {
        interpreters.removeElement(si);
    }
  
    /**
     * Select a ScriptInterpreter based on the file extension
     * @param String the file extension e.g. "tcl", "py"
     * @return ScriptInterpreter supporting the file extension or null
     */

    public ScriptInterpreter selectInterpreterByFileExt(String ext) {
        ScriptInterpreter rc = null;
        if (ext != null && ext.length() > 0) {
            for (Enumeration<ScriptInterpreter> e = interpreters.elements();
                        e.hasMoreElements();) {
                ScriptInterpreter scrint = e.nextElement();
                String[] fileExts = scrint.getFileExtensions();
    
                for (int i = 0; i < fileExts.length; i++) {
                    if (ext.equalsIgnoreCase(fileExts[i])) {
                        rc = scrint;
                        break;
                    }
                }
                if (rc != null) {
                    break;
                }
            }
        }
        return rc;
    }

    /**
     * Select a ScriptInterpreter based on the language names
     * @param String the language names e.g. "TCL", "Python"
     * @return ScriptInterpreter supporting the language or null
     */

    public ScriptInterpreter selectInterpreterByLanguage(String lang) {
        ScriptInterpreter rc = null;
        for (Enumeration<ScriptInterpreter> e = interpreters.elements(); e.hasMoreElements();) {
            ScriptInterpreter scrint = e.nextElement();
            String[] langs = scrint.getLanguageNames();

            for (int i = 0; i < langs.length; i++) {
                if (lang.equalsIgnoreCase(langs[i])) {
                    rc = scrint;
                    break;
                }
            }
            if (rc != null) {
                break;
            }
        }
        return rc;
    }
    
    /**
     * Get the full set of language names supported by all script interpreters
     * in the manager.
     * @return array of String names of languages.
     */

    public String[] getLanguageNames() {
        String[] rc = null;
        Vector<String> langVec = new Vector<String>();

        // collect all the language names

        for (Enumeration<ScriptInterpreter> e = interpreters.elements(); e.hasMoreElements();) {
            ScriptInterpreter scrint = e.nextElement();
            String[] langs = scrint.getLanguageNames();

            for (int i = 0; i < langs.length; i++) {
                langVec.addElement(langs[i]);
            }
        }

        // copy them into the rc array

        rc = new String[langVec.size()];
        for (int i = 0; i < rc.length; i++) {
            rc[i] = langVec.elementAt(i);
        }
        return rc;
    }

    /**
     * Get the full set of file extensions supported by all script interpreters
     * in the manager.
     * @return array of String names of file extensions.
     */

    public String[] getFileExtensions() {
        String[] rc = null;
        Vector<String> fileExtVec = new Vector<String>();

        // collect all the file extensions

        for (Enumeration<ScriptInterpreter> e = interpreters.elements(); e.hasMoreElements();) {
            ScriptInterpreter scrint = e.nextElement();
            String[] fExts = scrint.getFileExtensions();

            for (int i = 0; i < fExts.length; i++) {
                fileExtVec.addElement(fExts[i]);
            }
        }

        // copy them into the rc array

        rc = new String[fileExtVec.size()];
        for (int i = 0; i < rc.length; i++) {
            rc[i] = fileExtVec.elementAt(i);
        }
        return rc;
    }

    /**
     * Get an Enumeration of the Interpreters
     * @return Enumeration of ScriptInterpreter objects
     */

    public Enumeration<ScriptInterpreter> getInterpreters() {
        return interpreters.elements();
    }

    /**
     * Add a script interpreter to the ScriptInterpreterManager
     * @param String the class name to be initialized
     * @return boolean true if class was initialized, else false
     */

    public boolean addScriptInterpreter(String className) {
	boolean rc = true;
        try {
            // Class.forName(className, true, null);
            // for 1.0.8
            Class.forName(className);
        } catch (Exception e) {
        }
        rc = isRegistered(className);
        return rc;
    }
    
    /**
     * Main was included to help debug the file path problems on MacOS
     * The problem was finally found to be in the Jacl TCL interpreter.
     * @param args
     */
    public static void main(String args[]) {
             
        ScriptInterpreterManager man = ScriptInterpreterManager.getInstance();
        man.addScriptInterpreter(
            "org.eso.ohs.scripting.JaclScriptInterpreter");
        ScriptInterpreter si = man.selectInterpreterByLanguage("tcl");
        if (si == null) {
            System.out.println("Script Interpreter is NULL");
        } else {
            System.out.println("cmd: "+args[0]);
            File script = new File(args[0]);
            String files[] = script.list();
            for (int i = 0; files != null && i < files.length; i++) {
                System.out.println("File ("+i+") "+files[i]);
            }
            System.out.println("File: "+script.getAbsolutePath());
            StringBuffer scriptBuffer = new StringBuffer();
            try {
                si.executeFile(script);
                //si.executeScript(scriptBuffer.toString());
            } catch (ScriptInterpreterException e) {
                System.out.println("Error in script: "+e.toString());
            }
        }
    }
}
